<?php
class ControllerCatalogAttribute {
	private static $error = array();

	public static function index() {
		load::language('catalog/attribute');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('catalog/attribute');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_attribute::addAttribute(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/attribute', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('catalog/attribute');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_attribute::editAttribute(request::$get['attribute_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/attribute', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('catalog/attribute');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $attribute_id) {
				admin_catalog_attribute::deleteAttribute($attribute_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/attribute', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'ad.name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/attribute', $url)
		);

		$data['add'] = url::link('catalog/attribute/add', $url);
		$data['delete'] = url::link('catalog/attribute/delete', $url);

		$data['attributes'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$attribute_total = admin_catalog_attribute::getTotalAttributes();

		$results = admin_catalog_attribute::getAttributes($filter_data);

		foreach ($results as $result) {
			$data['attributes'][] = array(
				'attribute_id'    => $result['attribute_id'],
				'name'            => $result['name'],
				'attribute_group' => $result['attribute_group'],
				'sort_order'      => $result['sort_order'],
				'edit'            => url::link('catalog/attribute/edit', '&attribute_id=' . $result['attribute_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('catalog/attribute', '&sort=ad.name' . $url);
		$data['sort_attribute_group'] = url::link('catalog/attribute', '&sort=attribute_group' . $url);
		$data['sort_sort_order'] = url::link('catalog/attribute', '&sort=a.sort_order' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $attribute_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('catalog/attribute', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($attribute_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($attribute_total - config('limit_admin'))) ? $attribute_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $attribute_total, ceil($attribute_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/attribute_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['attribute_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = array();
		}

		if (isset(self::$error['attribute_group'])) {
			$data['error_attribute_group'] = self::$error['attribute_group'];
		} else {
			$data['error_attribute_group'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/attribute', $url)
		);

		if (!isset(request::$get['attribute_id'])) {
			$data['action'] = url::link('catalog/attribute/add', $url);
		} else {
			$data['action'] = url::link('catalog/attribute/edit', '&attribute_id=' . request::$get['attribute_id'] . $url);
		}

		$data['cancel'] = url::link('catalog/attribute', $url);

		if (isset(request::$get['attribute_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$attribute_info = admin_catalog_attribute::getAttribute(request::$get['attribute_id']);
		}


		$data['languages'] = admin_localisation_language::getLanguages();

		if (isset(request::$post['attribute_description'])) {
			$data['attribute_description'] = request::$post['attribute_description'];
		} elseif (isset(request::$get['attribute_id'])) {
			$data['attribute_description'] = admin_catalog_attribute::getAttributeDescriptions(request::$get['attribute_id']);
		} else {
			$data['attribute_description'] = array();
		}

		if (isset(request::$post['attribute_group_id'])) {
			$data['attribute_group_id'] = request::$post['attribute_group_id'];
		} elseif (!empty($attribute_info)) {
			$data['attribute_group_id'] = $attribute_info['attribute_group_id'];
		} else {
			$data['attribute_group_id'] = '';
		}


		$data['attribute_groups'] = admin_catalog_attribute_group::getAttributeGroups();

		if (isset(request::$post['sort_order'])) {
			$data['sort_order'] = request::$post['sort_order'];
		} elseif (!empty($attribute_info)) {
			$data['sort_order'] = $attribute_info['sort_order'];
		} else {
			$data['sort_order'] = '';
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/attribute_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'catalog/attribute')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if (!request::$post['attribute_group_id']) {
			self::$error['attribute_group'] = language::get('error_attribute_group');
		}

		foreach (request::$post['attribute_description'] as $language_id => $value) {
			if ((utf8_strlen($value['name']) < 1) || (utf8_strlen($value['name']) > 64)) {
				self::$error['name'][$language_id] = language::get('error_name');
			}
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'catalog/attribute')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $attribute_id) {
			$product_total = admin_catalog_product::getTotalProductsByAttributeId($attribute_id);

			if ($product_total) {
				self::$error['warning'] = sprintf(language::get('error_product'), $product_total);
			}
		}

		return !self::$error;
	}

	public static function autocomplete() {
		$json = array();

		if (isset(request::$get['filter_name'])) {

			$filter_data = array(
				'filter_name' => request::$get['filter_name'],
				'start'       => 0,
				'limit'       => 5
			);

			$results = admin_catalog_attribute::getAttributes($filter_data);

			foreach ($results as $result) {
				$json[] = array(
					'attribute_id'    => $result['attribute_id'],
					'name'            => strip_tags(html_entity_decode($result['name'], ENT_QUOTES, 'UTF-8')),
					'attribute_group' => $result['attribute_group']
				);
			}
		}

		$sort_order = array();

		foreach ($json as $key => $value) {
			$sort_order[$key] = $value['name'];
		}

		array_multisort($sort_order, SORT_ASC, $json);

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
