<?php
class ControllerCatalogAttributeGroup {
	private static $error = array();

	public static function index() {
		load::language('catalog/attribute_group');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('catalog/attribute_group');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_attribute_group::addAttributeGroup(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/attribute_group', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('catalog/attribute_group');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_attribute_group::editAttributeGroup(request::$get['attribute_group_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/attribute_group', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('catalog/attribute_group');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $attribute_group_id) {
				admin_catalog_attribute_group::deleteAttributeGroup($attribute_group_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/attribute_group', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'agd.name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/attribute_group', $url)
		);

		$data['add'] = url::link('catalog/attribute_group/add', $url);
		$data['delete'] = url::link('catalog/attribute_group/delete', $url);

		$data['attribute_groups'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$attribute_group_total = admin_catalog_attribute_group::getTotalAttributeGroups();

		$results = admin_catalog_attribute_group::getAttributeGroups($filter_data);

		foreach ($results as $result) {
			$data['attribute_groups'][] = array(
				'attribute_group_id' => $result['attribute_group_id'],
				'name'               => $result['name'],
				'sort_order'         => $result['sort_order'],
				'edit'               => url::link('catalog/attribute_group/edit', '&attribute_group_id=' . $result['attribute_group_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('catalog/attribute_group', '&sort=agd.name' . $url);
		$data['sort_sort_order'] = url::link('catalog/attribute_group', '&sort=ag.sort_order' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $attribute_group_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('catalog/attribute_group', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($attribute_group_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($attribute_group_total - config('limit_admin'))) ? $attribute_group_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $attribute_group_total, ceil($attribute_group_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/attribute_group_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['attribute_group_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = array();
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/attribute_group', $url)
		);

		if (!isset(request::$get['attribute_group_id'])) {
			$data['action'] = url::link('catalog/attribute_group/add', $url);
		} else {
			$data['action'] = url::link('catalog/attribute_group/edit', '&attribute_group_id=' . request::$get['attribute_group_id'] . $url);
		}

		$data['cancel'] = url::link('catalog/attribute_group', $url);

		if (isset(request::$get['attribute_group_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$attribute_group_info = admin_catalog_attribute_group::getAttributeGroup(request::$get['attribute_group_id']);
		}


		$data['languages'] = admin_localisation_language::getLanguages();

		if (isset(request::$post['attribute_group_description'])) {
			$data['attribute_group_description'] = request::$post['attribute_group_description'];
		} elseif (isset(request::$get['attribute_group_id'])) {
			$data['attribute_group_description'] = admin_catalog_attribute_group::getAttributeGroupDescriptions(request::$get['attribute_group_id']);
		} else {
			$data['attribute_group_description'] = array();
		}

		if (isset(request::$post['sort_order'])) {
			$data['sort_order'] = request::$post['sort_order'];
		} elseif (!empty($attribute_group_info)) {
			$data['sort_order'] = $attribute_group_info['sort_order'];
		} else {
			$data['sort_order'] = '';
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/attribute_group_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'catalog/attribute_group')) {
			self::$error['warning'] = language::get('error_permission');
		}

		foreach (request::$post['attribute_group_description'] as $language_id => $value) {
			if ((utf8_strlen($value['name']) < 1) || (utf8_strlen($value['name']) > 64)) {
				self::$error['name'][$language_id] = language::get('error_name');
			}
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'catalog/attribute_group')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $attribute_group_id) {
			$attribute_total = admin_catalog_attribute::getTotalAttributesByAttributeGroupId($attribute_group_id);

			if ($attribute_total) {
				self::$error['warning'] = sprintf(language::get('error_attribute'), $attribute_total);
			}
		}

		return !self::$error;
	}
}
