<?php
class ControllerCatalogCategory {
	private static $error = array();

	public static function index() {
		load::language('catalog/category');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('catalog/category');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_category::addCategory(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/category', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('catalog/category');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_category::editCategory(request::$get['category_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/category', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('catalog/category');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $category_id) {
				admin_catalog_category::deleteCategory($category_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/category', $url));
		}

		self::getList();
	}

	public static function repair() {
		load::language('catalog/category');

		document::setTitle(language::get('heading_title'));


		if (self::validateRepair()) {
			admin_catalog_category::repairCategories();

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/category', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/category', $url)
		);

		$data['add'] = url::link('catalog/category/add', $url);
		$data['delete'] = url::link('catalog/category/delete', $url);
		$data['repair'] = url::link('catalog/category/repair', $url);

		$data['categories'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$category_total = admin_catalog_category::getTotalCategories();

		$results = admin_catalog_category::getCategories($filter_data);

		foreach ($results as $result) {
			$data['categories'][] = array(
				'category_id' => $result['category_id'],
				'name'        => $result['name'],
				'sort_order'  => $result['sort_order'],
				'edit'        => url::link('catalog/category/edit', '&category_id=' . $result['category_id'] . $url),
				'delete'      => url::link('catalog/category/delete', '&category_id=' . $result['category_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('catalog/category', '&sort=name' . $url);
		$data['sort_sort_order'] = url::link('catalog/category', '&sort=sort_order' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $category_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('catalog/category', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($category_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($category_total - config('limit_admin'))) ? $category_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $category_total, ceil($category_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/category_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['category_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = array();
		}

		if (isset(self::$error['meta_title'])) {
			$data['error_meta_title'] = self::$error['meta_title'];
		} else {
			$data['error_meta_title'] = array();
		}

		if (isset(self::$error['keyword'])) {
			$data['error_keyword'] = self::$error['keyword'];
		} else {
			$data['error_keyword'] = '';
		}

		if (isset(self::$error['parent'])) {
			$data['error_parent'] = self::$error['parent'];
		} else {
			$data['error_parent'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/category', $url)
		);

		if (!isset(request::$get['category_id'])) {
			$data['action'] = url::link('catalog/category/add', $url);
		} else {
			$data['action'] = url::link('catalog/category/edit', '&category_id=' . request::$get['category_id'] . $url);
		}

		$data['cancel'] = url::link('catalog/category', $url);

		if (isset(request::$get['category_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$category_info = admin_catalog_category::getCategory(request::$get['category_id']);
		}



		$data['languages'] = admin_localisation_language::getLanguages();

		if (isset(request::$post['category_description'])) {
			$data['category_description'] = request::$post['category_description'];
		} elseif (isset(request::$get['category_id'])) {
			$data['category_description'] = admin_catalog_category::getCategoryDescriptions(request::$get['category_id']);
		} else {
			$data['category_description'] = array();
		}

		if (isset(request::$post['path'])) {
			$data['path'] = request::$post['path'];
		} elseif (!empty($category_info)) {
			$data['path'] = $category_info['path'];
		} else {
			$data['path'] = '';
		}

		if (isset(request::$post['parent_id'])) {
			$data['parent_id'] = request::$post['parent_id'];
		} elseif (!empty($category_info)) {
			$data['parent_id'] = $category_info['parent_id'];
		} else {
			$data['parent_id'] = 0;
		}


		if (isset(request::$post['category_filter'])) {
			$filters = request::$post['category_filter'];
		} elseif (isset(request::$get['category_id'])) {
			$filters = admin_catalog_category::getCategoryFilters(request::$get['category_id']);
		} else {
			$filters = array();
		}

		$data['category_filters'] = array();

		foreach ($filters as $filter_id) {
			$filter_info = admin_catalog_filter::getFilter($filter_id);

			if ($filter_info) {
				$data['category_filters'][] = array(
					'filter_id' => $filter_info['filter_id'],
					'name'      => $filter_info['group'] . ' &gt; ' . $filter_info['name']
				);
			}
		}


		$data['stores'] = array();

		$data['stores'][] = array(
			'store_id' => 0,
			'name'     => language::get('text_default')
		);

		$stores = admin_setting_store::getStores();

		foreach ($stores as $store) {
			$data['stores'][] = array(
				'store_id' => $store['store_id'],
				'name'     => $store['name']
			);
		}

		if (isset(request::$post['category_store'])) {
			$data['category_store'] = request::$post['category_store'];
		} elseif (isset(request::$get['category_id'])) {
			$data['category_store'] = admin_catalog_category::getCategoryStores(request::$get['category_id']);
		} else {
			$data['category_store'] = array(0);
		}

		if (isset(request::$post['image'])) {
			$data['image'] = request::$post['image'];
		} elseif (!empty($category_info)) {
			$data['image'] = $category_info['image'];
		} else {
			$data['image'] = '';
		}


		if (isset(request::$post['image']) && is_file('image/' . request::$post['image'])) {
			$data['thumb'] = admin_tool_image::resize(request::$post['image'], 100, 100);
		} elseif (!empty($category_info) && is_file('image/' . $category_info['image'])) {
			$data['thumb'] = admin_tool_image::resize($category_info['image'], 100, 100);
		} else {
			$data['thumb'] = admin_tool_image::resize('no_image.png', 100, 100);
		}

		$data['placeholder'] = admin_tool_image::resize('no_image.png', 100, 100);

		if (isset(request::$post['top'])) {
			$data['top'] = request::$post['top'];
		} elseif (!empty($category_info)) {
			$data['top'] = $category_info['top'];
		} else {
			$data['top'] = 0;
		}

		if (isset(request::$post['column'])) {
			$data['column'] = request::$post['column'];
		} elseif (!empty($category_info)) {
			$data['column'] = $category_info['column'];
		} else {
			$data['column'] = 1;
		}

		if (isset(request::$post['sort_order'])) {
			$data['sort_order'] = request::$post['sort_order'];
		} elseif (!empty($category_info)) {
			$data['sort_order'] = $category_info['sort_order'];
		} else {
			$data['sort_order'] = 0;
		}

		if (isset(request::$post['status'])) {
			$data['status'] = request::$post['status'];
		} elseif (!empty($category_info)) {
			$data['status'] = $category_info['status'];
		} else {
			$data['status'] = true;
		}

		if (isset(request::$post['category_seo_url'])) {
			$data['category_seo_url'] = request::$post['category_seo_url'];
		} elseif (isset(request::$get['category_id'])) {
			$data['category_seo_url'] = admin_catalog_category::getCategorySeoUrls(request::$get['category_id']);
		} else {
			$data['category_seo_url'] = array();
		}

		if (isset(request::$post['category_layout'])) {
			$data['category_layout'] = request::$post['category_layout'];
		} elseif (isset(request::$get['category_id'])) {
			$data['category_layout'] = admin_catalog_category::getCategoryLayouts(request::$get['category_id']);
		} else {
			$data['category_layout'] = array();
		}


		$data['layouts'] = admin_design_layout::getLayouts();

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/category_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'catalog/category')) {
			self::$error['warning'] = language::get('error_permission');
		}

		foreach (request::$post['category_description'] as $language_id => $value) {
			if ((utf8_strlen($value['name']) < 1) || (utf8_strlen($value['name']) > 255)) {
				self::$error['name'][$language_id] = language::get('error_name');
			}

			if ((utf8_strlen($value['meta_title']) < 1) || (utf8_strlen($value['meta_title']) > 255)) {
				self::$error['meta_title'][$language_id] = language::get('error_meta_title');
			}
		}

		if (isset(request::$get['category_id']) && request::$post['parent_id']) {
			$results = admin_catalog_category::getCategoryPath(request::$post['parent_id']);

			foreach ($results as $result) {
				if ($result['path_id'] == request::$get['category_id']) {
					self::$error['parent'] = language::get('error_parent');

					break;
				}
			}
		}

		if (request::$post['category_seo_url']) {

			foreach (request::$post['category_seo_url'] as $store_id => $language) {
				foreach ($language as $language_id => $keyword) {
					if (!empty($keyword)) {
						if (count(array_keys($language, $keyword)) > 1) {
							self::$error['keyword'][$store_id][$language_id] = language::get('error_unique');
						}

						$seo_urls = admin_design_seo_url::getSeoUrlsByKeyword($keyword);

						foreach ($seo_urls as $seo_url) {
							if (($seo_url['store_id'] == $store_id) && (!isset(request::$get['category_id']) || ($seo_url['query'] != 'category_id=' . request::$get['category_id']))) {
								self::$error['keyword'][$store_id][$language_id] = language::get('error_keyword');

								break;
							}
						}
					}
				}
			}
		}

		if (self::$error && !isset(self::$error['warning'])) {
			self::$error['warning'] = language::get('error_warning');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'catalog/category')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}

	protected static function validateRepair() {
		if (!user::hasPermission('modify', 'catalog/category')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}

	public static function autocomplete() {
		$json = array();

		if (isset(request::$get['filter_name'])) {

			$filter_data = array(
				'filter_name' => request::$get['filter_name'],
				'sort'        => 'name',
				'order'       => 'ASC',
				'start'       => 0,
				'limit'       => 5
			);

			$results = admin_catalog_category::getCategories($filter_data);

			foreach ($results as $result) {
				$json[] = array(
					'category_id' => $result['category_id'],
					'name'        => strip_tags(html_entity_decode($result['name'], ENT_QUOTES, 'UTF-8'))
				);
			}
		}

		$sort_order = array();

		foreach ($json as $key => $value) {
			$sort_order[$key] = $value['name'];
		}

		array_multisort($sort_order, SORT_ASC, $json);

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
