<?php
class ControllerCatalogDownload {
	private static $error = array();

	public static function index() {
		load::language('catalog/download');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('catalog/download');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_download::addDownload(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/download', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('catalog/download');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_download::editDownload(request::$get['download_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/download', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('catalog/download');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $download_id) {
				admin_catalog_download::deleteDownload($download_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/download', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'dd.name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/download', $url)
		);

		$data['add'] = url::link('catalog/download/add', $url);
		$data['delete'] = url::link('catalog/download/delete', $url);

		$data['downloads'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$download_total = admin_catalog_download::getTotalDownloads();

		$results = admin_catalog_download::getDownloads($filter_data);

		foreach ($results as $result) {
			$data['downloads'][] = array(
				'download_id' => $result['download_id'],
				'name'        => $result['name'],
				'date_added'  => date(language::get('date_format_short'), strtotime($result['date_added'])),
				'edit'        => url::link('catalog/download/edit', '&download_id=' . $result['download_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('catalog/download', '&sort=dd.name' . $url);
		$data['sort_date_added'] = url::link('catalog/download', '&sort=d.date_added' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $download_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('catalog/download', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($download_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($download_total - config('limit_admin'))) ? $download_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $download_total, ceil($download_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/download_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['download_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = array();
		}

		if (isset(self::$error['filename'])) {
			$data['error_filename'] = self::$error['filename'];
		} else {
			$data['error_filename'] = '';
		}

		if (isset(self::$error['mask'])) {
			$data['error_mask'] = self::$error['mask'];
		} else {
			$data['error_mask'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/download', $url)
		);

		if (!isset(request::$get['download_id'])) {
			$data['action'] = url::link('catalog/download/add', $url);
		} else {
			$data['action'] = url::link('catalog/download/edit', '&download_id=' . request::$get['download_id'] . $url);
		}

		$data['cancel'] = url::link('catalog/download', $url);


		$data['languages'] = admin_localisation_language::getLanguages();

		if (isset(request::$get['download_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$download_info = admin_catalog_download::getDownload(request::$get['download_id']);
		}


		if (isset(request::$get['download_id'])) {
			$data['download_id'] = (int)request::$get['download_id'];
		} else {
			$data['download_id'] = 0;
		}

		if (isset(request::$post['download_description'])) {
			$data['download_description'] = request::$post['download_description'];
		} elseif (isset(request::$get['download_id'])) {
			$data['download_description'] = admin_catalog_download::getDownloadDescriptions(request::$get['download_id']);
		} else {
			$data['download_description'] = array();
		}

		if (isset(request::$post['filename'])) {
			$data['filename'] = request::$post['filename'];
		} elseif (!empty($download_info)) {
			$data['filename'] = $download_info['filename'];
		} else {
			$data['filename'] = '';
		}

		if (isset(request::$post['mask'])) {
			$data['mask'] = request::$post['mask'];
		} elseif (!empty($download_info)) {
			$data['mask'] = $download_info['mask'];
		} else {
			$data['mask'] = '';
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/download_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'catalog/download')) {
			self::$error['warning'] = language::get('error_permission');
		}

		foreach (request::$post['download_description'] as $language_id => $value) {
			if ((utf8_strlen($value['name']) < 3) || (utf8_strlen($value['name']) > 64)) {
				self::$error['name'][$language_id] = language::get('error_name');
			}
		}

		if ((utf8_strlen(request::$post['filename']) < 3) || (utf8_strlen(request::$post['filename']) > 128)) {
			self::$error['filename'] = language::get('error_filename');
		}

		if (!is_file(DIR_DOWNLOAD . request::$post['filename'])) {
			self::$error['filename'] = language::get('error_exists');
		}

		if ((utf8_strlen(request::$post['mask']) < 3) || (utf8_strlen(request::$post['mask']) > 128)) {
			self::$error['mask'] = language::get('error_mask');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'catalog/download')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $download_id) {
			$product_total = admin_catalog_product::getTotalProductsByDownloadId($download_id);

			if ($product_total) {
				self::$error['warning'] = sprintf(language::get('error_product'), $product_total);
			}
		}

		return !self::$error;
	}

	public static function upload() {
		load::language('catalog/download');

		$json = array();

		// Check user has permission
		if (!user::hasPermission('modify', 'catalog/download')) {
			$json['error'] = language::get('error_permission');
		}

		if (!$json) {
			if (!empty(request::$files['file']['name']) && is_file(request::$files['file']['tmp_name'])) {
				// Sanitize the filename
				$filename = basename(html_entity_decode(request::$files['file']['name'], ENT_QUOTES, 'UTF-8'));

				// Validate the filename length
				if ((utf8_strlen($filename) < 3) || (utf8_strlen($filename) > 128)) {
					$json['error'] = language::get('error_filename');
				}

				// Allowed file extension types
				$allowed = array();

				$extension_allowed = preg_replace('~\r?\n~', "\n", config('file_ext_allowed'));

				$filetypes = explode("\n", $extension_allowed);

				foreach ($filetypes as $filetype) {
					$allowed[] = trim($filetype);
				}

				if (!in_array(strtolower(substr(strrchr($filename, '.'), 1)), $allowed)) {
					$json['error'] = language::get('error_filetype');
				}

				// Allowed file mime types
				$allowed = array();

				$mime_allowed = preg_replace('~\r?\n~', "\n", config('file_mime_allowed'));

				$filetypes = explode("\n", $mime_allowed);

				foreach ($filetypes as $filetype) {
					$allowed[] = trim($filetype);
				}

				if (!in_array(request::$files['file']['type'], $allowed)) {
					$json['error'] = language::get('error_filetype');
				}

				// Check to see if any PHP files are trying to be uploaded
				$content = file_get_contents(request::$files['file']['tmp_name']);

				if (preg_match('/\<\?php/i', $content)) {
					$json['error'] = language::get('error_filetype');
				}

				// Return any upload error
				if (request::$files['file']['error'] != UPLOAD_ERR_OK) {
					$json['error'] = language::get('error_upload_' . request::$files['file']['error']);
				}
			} else {
				$json['error'] = language::get('error_upload');
			}
		}

		if (!$json) {
			$file = $filename . '.' . token(32);

			move_uploaded_file(request::$files['file']['tmp_name'], DIR_DOWNLOAD . $file);

			$json['filename'] = $file;
			$json['mask'] = $filename;

			$json['success'] = language::get('text_upload');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function autocomplete() {
		$json = array();

		if (isset(request::$get['filter_name'])) {

			$filter_data = array(
				'filter_name' => request::$get['filter_name'],
				'start'       => 0,
				'limit'       => 5
			);

			$results = admin_catalog_download::getDownloads($filter_data);

			foreach ($results as $result) {
				$json[] = array(
					'download_id' => $result['download_id'],
					'name'        => strip_tags(html_entity_decode($result['name'], ENT_QUOTES, 'UTF-8'))
				);
			}
		}

		$sort_order = array();

		foreach ($json as $key => $value) {
			$sort_order[$key] = $value['name'];
		}

		array_multisort($sort_order, SORT_ASC, $json);

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}