<?php
class ControllerCatalogFilter {
	private static $error = array();

	public static function index() {
		load::language('catalog/filter');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('catalog/filter');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_filter::addFilter(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/filter', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('catalog/filter');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_filter::editFilter(request::$get['filter_group_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/filter', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('catalog/filter');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $filter_group_id) {
				admin_catalog_filter::deleteFilter($filter_group_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/filter', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'fgd.name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/filter', $url)
		);

		$data['add'] = url::link('catalog/filter/add', $url);
		$data['delete'] = url::link('catalog/filter/delete', $url);

		$data['filters'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$filter_total = admin_catalog_filter::getTotalFilterGroups();

		$results = admin_catalog_filter::getFilterGroups($filter_data);

		foreach ($results as $result) {
			$data['filters'][] = array(
				'filter_group_id' => $result['filter_group_id'],
				'name'            => $result['name'],
				'sort_order'      => $result['sort_order'],
				'edit'            => url::link('catalog/filter/edit', '&filter_group_id=' . $result['filter_group_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('catalog/filter', '&sort=fgd.name' . $url);
		$data['sort_sort_order'] = url::link('catalog/filter', '&sort=fg.sort_order' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $filter_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('catalog/filter', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($filter_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($filter_total - config('limit_admin'))) ? $filter_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $filter_total, ceil($filter_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/filter_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['filter_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['group'])) {
			$data['error_group'] = self::$error['group'];
		} else {
			$data['error_group'] = array();
		}

		if (isset(self::$error['filter'])) {
			$data['error_filter'] = self::$error['filter'];
		} else {
			$data['error_filter'] = array();
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/filter', $url)
		);

		if (!isset(request::$get['filter_group_id'])) {
			$data['action'] = url::link('catalog/filter/add', $url);
		} else {
			$data['action'] = url::link('catalog/filter/edit', '&filter_group_id=' . request::$get['filter_group_id'] . $url);
		}

		$data['cancel'] = url::link('catalog/filter', $url);

		if (isset(request::$get['filter_group_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$filter_group_info = admin_catalog_filter::getFilterGroup(request::$get['filter_group_id']);
		}



		$data['languages'] = admin_localisation_language::getLanguages();

		if (isset(request::$post['filter_group_description'])) {
			$data['filter_group_description'] = request::$post['filter_group_description'];
		} elseif (isset(request::$get['filter_group_id'])) {
			$data['filter_group_description'] = admin_catalog_filter::getFilterGroupDescriptions(request::$get['filter_group_id']);
		} else {
			$data['filter_group_description'] = array();
		}

		if (isset(request::$post['sort_order'])) {
			$data['sort_order'] = request::$post['sort_order'];
		} elseif (!empty($filter_group_info)) {
			$data['sort_order'] = $filter_group_info['sort_order'];
		} else {
			$data['sort_order'] = '';
		}

		if (isset(request::$post['filter'])) {
			$data['filters'] = request::$post['filter'];
		} elseif (isset(request::$get['filter_group_id'])) {
			$data['filters'] = admin_catalog_filter::getFilterDescriptions(request::$get['filter_group_id']);
		} else {
			$data['filters'] = array();
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/filter_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'catalog/filter')) {
			self::$error['warning'] = language::get('error_permission');
		}

		foreach (request::$post['filter_group_description'] as $language_id => $value) {
			if ((utf8_strlen($value['name']) < 1) || (utf8_strlen($value['name']) > 64)) {
				self::$error['group'][$language_id] = language::get('error_group');
			}
		}

		if (isset(request::$post['filter'])) {
			foreach (request::$post['filter'] as $filter_id => $filter) {
				foreach ($filter['filter_description'] as $language_id => $filter_description) {
					if ((utf8_strlen($filter_description['name']) < 1) || (utf8_strlen($filter_description['name']) > 64)) {
						self::$error['filter'][$filter_id][$language_id] = language::get('error_name');
					}
				}
			}
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'catalog/filter')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}

	public static function autocomplete() {
		$json = array();

		if (isset(request::$get['filter_name'])) {

			$filter_data = array(
				'filter_name' => request::$get['filter_name'],
				'start'       => 0,
				'limit'       => 5
			);

			$filters = admin_catalog_filter::getFilters($filter_data);

			foreach ($filters as $filter) {
				$json[] = array(
					'filter_id' => $filter['filter_id'],
					'name'      => strip_tags(html_entity_decode($filter['group'] . ' &gt; ' . $filter['name'], ENT_QUOTES, 'UTF-8'))
				);
			}
		}

		$sort_order = array();

		foreach ($json as $key => $value) {
			$sort_order[$key] = $value['name'];
		}

		array_multisort($sort_order, SORT_ASC, $json);

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}