<?php
class ControllerCatalogInformation {
	private static $error = array();

	public static function index() {
		load::language('catalog/information');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('catalog/information');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_information::addInformation(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/information', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('catalog/information');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_information::editInformation(request::$get['information_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/information', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('catalog/information');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $information_id) {
				admin_catalog_information::deleteInformation($information_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/information', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'id.title';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/information', $url)
		);

		$data['add'] = url::link('catalog/information/add', $url);
		$data['delete'] = url::link('catalog/information/delete', $url);

		$data['informations'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$information_total = admin_catalog_information::getTotalInformations();

		$results = admin_catalog_information::getInformations($filter_data);

		foreach ($results as $result) {
			$data['informations'][] = array(
				'information_id' => $result['information_id'],
				'title'          => $result['title'],
				'sort_order'     => $result['sort_order'],
				'edit'           => url::link('catalog/information/edit', '&information_id=' . $result['information_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_title'] = url::link('catalog/information', '&sort=id.title' . $url);
		$data['sort_sort_order'] = url::link('catalog/information', '&sort=i.sort_order' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $information_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('catalog/information', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($information_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($information_total - config('limit_admin'))) ? $information_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $information_total, ceil($information_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/information_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['information_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['title'])) {
			$data['error_title'] = self::$error['title'];
		} else {
			$data['error_title'] = array();
		}

		if (isset(self::$error['description'])) {
			$data['error_description'] = self::$error['description'];
		} else {
			$data['error_description'] = array();
		}

		if (isset(self::$error['meta_title'])) {
			$data['error_meta_title'] = self::$error['meta_title'];
		} else {
			$data['error_meta_title'] = array();
		}

		if (isset(self::$error['keyword'])) {
			$data['error_keyword'] = self::$error['keyword'];
		} else {
			$data['error_keyword'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/information', $url)
		);

		if (!isset(request::$get['information_id'])) {
			$data['action'] = url::link('catalog/information/add', $url);
		} else {
			$data['action'] = url::link('catalog/information/edit', '&information_id=' . request::$get['information_id'] . $url);
		}

		$data['cancel'] = url::link('catalog/information', $url);

		if (isset(request::$get['information_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$information_info = admin_catalog_information::getInformation(request::$get['information_id']);
		}



		$data['languages'] = admin_localisation_language::getLanguages();

		if (isset(request::$post['information_description'])) {
			$data['information_description'] = request::$post['information_description'];
		} elseif (isset(request::$get['information_id'])) {
			$data['information_description'] = admin_catalog_information::getInformationDescriptions(request::$get['information_id']);
		} else {
			$data['information_description'] = array();
		}


		$data['stores'] = array();

		$data['stores'][] = array(
			'store_id' => 0,
			'name'     => language::get('text_default')
		);

		$stores = admin_setting_store::getStores();

		foreach ($stores as $store) {
			$data['stores'][] = array(
				'store_id' => $store['store_id'],
				'name'     => $store['name']
			);
		}

		if (isset(request::$post['information_store'])) {
			$data['information_store'] = request::$post['information_store'];
		} elseif (isset(request::$get['information_id'])) {
			$data['information_store'] = admin_catalog_information::getInformationStores(request::$get['information_id']);
		} else {
			$data['information_store'] = array(0);
		}

		if (isset(request::$post['bottom'])) {
			$data['bottom'] = request::$post['bottom'];
		} elseif (!empty($information_info)) {
			$data['bottom'] = $information_info['bottom'];
		} else {
			$data['bottom'] = 0;
		}

		if (isset(request::$post['status'])) {
			$data['status'] = request::$post['status'];
		} elseif (!empty($information_info)) {
			$data['status'] = $information_info['status'];
		} else {
			$data['status'] = true;
		}

		if (isset(request::$post['sort_order'])) {
			$data['sort_order'] = request::$post['sort_order'];
		} elseif (!empty($information_info)) {
			$data['sort_order'] = $information_info['sort_order'];
		} else {
			$data['sort_order'] = '';
		}

		if (isset(request::$post['information_seo_url'])) {
			$data['information_seo_url'] = request::$post['information_seo_url'];
		} elseif (isset(request::$get['information_id'])) {
			$data['information_seo_url'] = admin_catalog_information::getInformationSeoUrls(request::$get['information_id']);
		} else {
			$data['information_seo_url'] = array();
		}

		if (isset(request::$post['information_layout'])) {
			$data['information_layout'] = request::$post['information_layout'];
		} elseif (isset(request::$get['information_id'])) {
			$data['information_layout'] = admin_catalog_information::getInformationLayouts(request::$get['information_id']);
		} else {
			$data['information_layout'] = array();
		}


		$data['layouts'] = admin_design_layout::getLayouts();

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/information_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'catalog/information')) {
			self::$error['warning'] = language::get('error_permission');
		}

		foreach (request::$post['information_description'] as $language_id => $value) {
			if ((utf8_strlen($value['title']) < 1) || (utf8_strlen($value['title']) > 64)) {
				self::$error['title'][$language_id] = language::get('error_title');
			}

			if (utf8_strlen($value['description']) < 3) {
				self::$error['description'][$language_id] = language::get('error_description');
			}

			if ((utf8_strlen($value['meta_title']) < 1) || (utf8_strlen($value['meta_title']) > 255)) {
				self::$error['meta_title'][$language_id] = language::get('error_meta_title');
			}
		}

		if (request::$post['information_seo_url']) {

			foreach (request::$post['information_seo_url'] as $store_id => $language) {
				foreach ($language as $language_id => $keyword) {
					if (!empty($keyword)) {
						if (count(array_keys($language, $keyword)) > 1) {
							self::$error['keyword'][$store_id][$language_id] = language::get('error_unique');
						}

						$seo_urls = admin_design_seo_url::getSeoUrlsByKeyword($keyword);

						foreach ($seo_urls as $seo_url) {
							if (($seo_url['store_id'] == $store_id) && (!isset(request::$get['information_id']) || ($seo_url['query'] != 'information_id=' . request::$get['information_id']))) {
								self::$error['keyword'][$store_id][$language_id] = language::get('error_keyword');
							}
						}
					}
				}
			}
		}

		if (self::$error && !isset(self::$error['warning'])) {
			self::$error['warning'] = language::get('error_warning');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'catalog/information')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $information_id) {
			if (config('account_id') == $information_id) {
				self::$error['warning'] = language::get('error_account');
			}

			if (config('checkout_id') == $information_id) {
				self::$error['warning'] = language::get('error_checkout');
			}

			if (config('affiliate_id') == $information_id) {
				self::$error['warning'] = language::get('error_affiliate');
			}

			if (config('return_id') == $information_id) {
				self::$error['warning'] = language::get('error_return');
			}

			$store_total = admin_setting_store::getTotalStoresByInformationId($information_id);

			if ($store_total) {
				self::$error['warning'] = sprintf(language::get('error_store'), $store_total);
			}
		}

		return !self::$error;
	}
}
