<?php
class ControllerCatalogManufacturer {
	private static $error = array();

	public static function index() {
		load::language('catalog/manufacturer');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('catalog/manufacturer');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_manufacturer::addManufacturer(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/manufacturer', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('catalog/manufacturer');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_manufacturer::editManufacturer(request::$get['manufacturer_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/manufacturer', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('catalog/manufacturer');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $manufacturer_id) {
				admin_catalog_manufacturer::deleteManufacturer($manufacturer_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/manufacturer', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/manufacturer', $url)
		);

		$data['add'] = url::link('catalog/manufacturer/add', $url);
		$data['delete'] = url::link('catalog/manufacturer/delete', $url);

		$data['manufacturers'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$manufacturer_total = admin_catalog_manufacturer::getTotalManufacturers();

		$results = admin_catalog_manufacturer::getManufacturers($filter_data);

		foreach ($results as $result) {
			$data['manufacturers'][] = array(
				'manufacturer_id' => $result['manufacturer_id'],
				'name'            => $result['name'],
				'sort_order'      => $result['sort_order'],
				'edit'            => url::link('catalog/manufacturer/edit', '&manufacturer_id=' . $result['manufacturer_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('catalog/manufacturer', '&sort=name' . $url);
		$data['sort_sort_order'] = url::link('catalog/manufacturer', '&sort=sort_order' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $manufacturer_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('catalog/manufacturer', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($manufacturer_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($manufacturer_total - config('limit_admin'))) ? $manufacturer_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $manufacturer_total, ceil($manufacturer_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/manufacturer_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['manufacturer_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = '';
		}

		if (isset(self::$error['keyword'])) {
			$data['error_keyword'] = self::$error['keyword'];
		} else {
			$data['error_keyword'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/manufacturer', $url)
		);

		if (!isset(request::$get['manufacturer_id'])) {
			$data['action'] = url::link('catalog/manufacturer/add', $url);
		} else {
			$data['action'] = url::link('catalog/manufacturer/edit', '&manufacturer_id=' . request::$get['manufacturer_id'] . $url);
		}

		$data['cancel'] = url::link('catalog/manufacturer', $url);

		if (isset(request::$get['manufacturer_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$manufacturer_info = admin_catalog_manufacturer::getManufacturer(request::$get['manufacturer_id']);
		}


		if (isset(request::$post['name'])) {
			$data['name'] = request::$post['name'];
		} elseif (!empty($manufacturer_info)) {
			$data['name'] = $manufacturer_info['name'];
		} else {
			$data['name'] = '';
		}


		$data['stores'] = array();

		$data['stores'][] = array(
			'store_id' => 0,
			'name'     => language::get('text_default')
		);

		$stores = admin_setting_store::getStores();

		foreach ($stores as $store) {
			$data['stores'][] = array(
				'store_id' => $store['store_id'],
				'name'     => $store['name']
			);
		}

		if (isset(request::$post['manufacturer_store'])) {
			$data['manufacturer_store'] = request::$post['manufacturer_store'];
		} elseif (isset(request::$get['manufacturer_id'])) {
			$data['manufacturer_store'] = admin_catalog_manufacturer::getManufacturerStores(request::$get['manufacturer_id']);
		} else {
			$data['manufacturer_store'] = array(0);
		}

		if (isset(request::$post['image'])) {
			$data['image'] = request::$post['image'];
		} elseif (!empty($manufacturer_info)) {
			$data['image'] = $manufacturer_info['image'];
		} else {
			$data['image'] = '';
		}


		if (isset(request::$post['image']) && is_file('image/' . request::$post['image'])) {
			$data['thumb'] = admin_tool_image::resize(request::$post['image'], 100, 100);
		} elseif (!empty($manufacturer_info) && is_file('image/' . $manufacturer_info['image'])) {
			$data['thumb'] = admin_tool_image::resize($manufacturer_info['image'], 100, 100);
		} else {
			$data['thumb'] = admin_tool_image::resize('no_image.png', 100, 100);
		}

		$data['placeholder'] = admin_tool_image::resize('no_image.png', 100, 100);

		if (isset(request::$post['sort_order'])) {
			$data['sort_order'] = request::$post['sort_order'];
		} elseif (!empty($manufacturer_info)) {
			$data['sort_order'] = $manufacturer_info['sort_order'];
		} else {
			$data['sort_order'] = '';
		}


		$data['languages'] = admin_localisation_language::getLanguages();

		if (isset(request::$post['manufacturer_seo_url'])) {
			$data['manufacturer_seo_url'] = request::$post['manufacturer_seo_url'];
		} elseif (isset(request::$get['manufacturer_id'])) {
			$data['manufacturer_seo_url'] = admin_catalog_manufacturer::getManufacturerSeoUrls(request::$get['manufacturer_id']);
		} else {
			$data['manufacturer_seo_url'] = array();
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/manufacturer_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'catalog/manufacturer')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(request::$post['name']) < 1) || (utf8_strlen(request::$post['name']) > 64)) {
			self::$error['name'] = language::get('error_name');
		}

		if (request::$post['manufacturer_seo_url']) {
			
			foreach (request::$post['manufacturer_seo_url'] as $store_id => $language) {
				foreach ($language as $language_id => $keyword) {
					if (!empty($keyword)) {
						if (count(array_keys($language, $keyword)) > 1) {
							self::$error['keyword'][$store_id][$language_id] = language::get('error_unique');
						}

						$seo_urls = admin_design_seo_url::getSeoUrlsByKeyword($keyword);

						foreach ($seo_urls as $seo_url) {
							if (($seo_url['store_id'] == $store_id) && (!isset(request::$get['manufacturer_id']) || (($seo_url['query'] != 'manufacturer_id=' . request::$get['manufacturer_id'])))) {
								self::$error['keyword'][$store_id][$language_id] = language::get('error_keyword');
							}
						}
					}
				}
			}
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'catalog/manufacturer')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $manufacturer_id) {
			$product_total = admin_catalog_product::getTotalProductsByManufacturerId($manufacturer_id);

			if ($product_total) {
				self::$error['warning'] = sprintf(language::get('error_product'), $product_total);
			}
		}

		return !self::$error;
	}

	public static function autocomplete() {
		$json = array();

		if (isset(request::$get['filter_name'])) {

			$filter_data = array(
				'filter_name' => request::$get['filter_name'],
				'start'       => 0,
				'limit'       => 5
			);

			$results = admin_catalog_manufacturer::getManufacturers($filter_data);

			foreach ($results as $result) {
				$json[] = array(
					'manufacturer_id' => $result['manufacturer_id'],
					'name'            => strip_tags(html_entity_decode($result['name'], ENT_QUOTES, 'UTF-8'))
				);
			}
		}

		$sort_order = array();

		foreach ($json as $key => $value) {
			$sort_order[$key] = $value['name'];
		}

		array_multisort($sort_order, SORT_ASC, $json);

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
