<?php
class ControllerCatalogOption {
	private static $error = array();

	public static function index() {
		load::language('catalog/option');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('catalog/option');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_option::addOption(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/option', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('catalog/option');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_catalog_option::editOption(request::$get['option_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/option', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('catalog/option');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $option_id) {
				admin_catalog_option::deleteOption($option_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('catalog/option', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'od.name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/option', $url)
		);

		$data['add'] = url::link('catalog/option/add', $url);
		$data['delete'] = url::link('catalog/option/delete', $url);

		$data['options'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$option_total = admin_catalog_option::getTotalOptions();

		$results = admin_catalog_option::getOptions($filter_data);

		foreach ($results as $result) {
			$data['options'][] = array(
				'option_id'  => $result['option_id'],
				'name'       => $result['name'],
				'sort_order' => $result['sort_order'],
				'edit'       => url::link('catalog/option/edit', '&option_id=' . $result['option_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('catalog/option', '&sort=od.name' . $url);
		$data['sort_sort_order'] = url::link('catalog/option', '&sort=o.sort_order' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $option_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('catalog/option', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($option_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($option_total - config('limit_admin'))) ? $option_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $option_total, ceil($option_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/option_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['option_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = array();
		}

		if (isset(self::$error['option_value'])) {
			$data['error_option_value'] = self::$error['option_value'];
		} else {
			$data['error_option_value'] = array();
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('catalog/option', $url)
		);

		if (!isset(request::$get['option_id'])) {
			$data['action'] = url::link('catalog/option/add', $url);
		} else {
			$data['action'] = url::link('catalog/option/edit', '&option_id=' . request::$get['option_id'] . $url);
		}

		$data['cancel'] = url::link('catalog/option', $url);

		if (isset(request::$get['option_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$option_info = admin_catalog_option::getOption(request::$get['option_id']);
		}



		$data['languages'] = admin_localisation_language::getLanguages();

		if (isset(request::$post['option_description'])) {
			$data['option_description'] = request::$post['option_description'];
		} elseif (isset(request::$get['option_id'])) {
			$data['option_description'] = admin_catalog_option::getOptionDescriptions(request::$get['option_id']);
		} else {
			$data['option_description'] = array();
		}

		if (isset(request::$post['type'])) {
			$data['type'] = request::$post['type'];
		} elseif (!empty($option_info)) {
			$data['type'] = $option_info['type'];
		} else {
			$data['type'] = '';
		}

		if (isset(request::$post['sort_order'])) {
			$data['sort_order'] = request::$post['sort_order'];
		} elseif (!empty($option_info)) {
			$data['sort_order'] = $option_info['sort_order'];
		} else {
			$data['sort_order'] = '';
		}

		if (isset(request::$post['option_value'])) {
			$option_values = request::$post['option_value'];
		} elseif (isset(request::$get['option_id'])) {
			$option_values = admin_catalog_option::getOptionValueDescriptions(request::$get['option_id']);
		} else {
			$option_values = array();
		}


		$data['option_values'] = array();

		foreach ($option_values as $option_value) {
			if (is_file('image/' . $option_value['image'])) {
				$image = $option_value['image'];
				$thumb = $option_value['image'];
			} else {
				$image = '';
				$thumb = 'no_image.png';
			}

			$data['option_values'][] = array(
				'option_value_id'          => $option_value['option_value_id'],
				'option_value_description' => $option_value['option_value_description'],
				'image'                    => $image,
				'thumb'                    => admin_tool_image::resize($thumb, 100, 100),
				'sort_order'               => $option_value['sort_order']
			);
		}

		$data['placeholder'] = admin_tool_image::resize('no_image.png', 100, 100);

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('catalog/option_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'catalog/option')) {
			self::$error['warning'] = language::get('error_permission');
		}

		foreach (request::$post['option_description'] as $language_id => $value) {
			if ((utf8_strlen($value['name']) < 1) || (utf8_strlen($value['name']) > 128)) {
				self::$error['name'][$language_id] = language::get('error_name');
			}
		}

		if ((request::$post['type'] == 'select' || request::$post['type'] == 'radio' || request::$post['type'] == 'checkbox') && !isset(request::$post['option_value'])) {
			self::$error['warning'] = language::get('error_type');
		}

		if (isset(request::$post['option_value'])) {
			foreach (request::$post['option_value'] as $option_value_id => $option_value) {
				foreach ($option_value['option_value_description'] as $language_id => $option_value_description) {
					if ((utf8_strlen($option_value_description['name']) < 1) || (utf8_strlen($option_value_description['name']) > 128)) {
						self::$error['option_value'][$option_value_id][$language_id] = language::get('error_option_value');
					}
				}
			}
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'catalog/option')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $option_id) {
			$product_total = admin_catalog_product::getTotalProductsByOptionId($option_id);

			if ($product_total) {
				self::$error['warning'] = sprintf(language::get('error_product'), $product_total);
			}
		}

		return !self::$error;
	}

	public static function autocomplete() {
		$json = array();

		if (isset(request::$get['filter_name'])) {
			load::language('catalog/option');



			$filter_data = array(
				'filter_name' => request::$get['filter_name'],
				'start'       => 0,
				'limit'       => 5
			);

			$options = admin_catalog_option::getOptions($filter_data);

			foreach ($options as $option) {
				$option_value_data = array();

				if ($option['type'] == 'select' || $option['type'] == 'radio' || $option['type'] == 'checkbox' || $option['type'] == 'image') {
					$option_values = admin_catalog_option::getOptionValues($option['option_id']);

					foreach ($option_values as $option_value) {
						if (is_file('image/' . $option_value['image'])) {
							$image = admin_tool_image::resize($option_value['image'], 50, 50);
						} else {
							$image = admin_tool_image::resize('no_image.png', 50, 50);
						}

						$option_value_data[] = array(
							'option_value_id' => $option_value['option_value_id'],
							'name'            => strip_tags(html_entity_decode($option_value['name'], ENT_QUOTES, 'UTF-8')),
							'image'           => $image
						);
					}

					$sort_order = array();

					foreach ($option_value_data as $key => $value) {
						$sort_order[$key] = $value['name'];
					}

					array_multisort($sort_order, SORT_ASC, $option_value_data);
				}

				$type = '';

				if ($option['type'] == 'select' || $option['type'] == 'radio' || $option['type'] == 'checkbox') {
					$type = language::get('text_choose');
				}

				if ($option['type'] == 'text' || $option['type'] == 'textarea') {
					$type = language::get('text_input');
				}

				if ($option['type'] == 'file') {
					$type = language::get('text_file');
				}

				if ($option['type'] == 'date' || $option['type'] == 'datetime' || $option['type'] == 'time') {
					$type = language::get('text_date');
				}

				$json[] = array(
					'option_id'    => $option['option_id'],
					'name'         => strip_tags(html_entity_decode($option['name'], ENT_QUOTES, 'UTF-8')),
					'category'     => $type,
					'type'         => $option['type'],
					'option_value' => $option_value_data
				);
			}
		}

		$sort_order = array();

		foreach ($json as $key => $value) {
			$sort_order[$key] = $value['name'];
		}

		array_multisort($sort_order, SORT_ASC, $json);

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}