<?php
class ControllerCommonDashboard {
	public static function index() {
		load::language('common/dashboard');

		document::setTitle(language::get('heading_title'));


		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('common/dashboard')
		);

		// Check install directory exists
		if (is_dir(DIR_CATALOG . '../install')) {
			$data['error_install'] = language::get('error_install');
		} else {
			$data['error_install'] = '';
		}

		// Dashboard Extensions
		$dashboards = array();


		// Get a list of installed modules
		$extensions = admin_setting_extension::getInstalled('dashboard');

		// Add all the modules which have multiple settings for each module
		foreach ($extensions as $code) {
			if (config::get('dashboard_' . $code . '_status') && user::hasPermission('access', 'extension/dashboard/' . $code)) {
				$output = load::controller('extension/dashboard/' . $code . '/dashboard');

				if ($output) {
					$dashboards[] = array(
						'code'       => $code,
						'width'      => config::get('dashboard_' . $code . '_width'),
						'sort_order' => config::get('dashboard_' . $code . '_sort_order'),
						'output'     => $output
					);
				}
			}
		}

		$sort_order = array();

		foreach ($dashboards as $key => $value) {
			$sort_order[$key] = $value['sort_order'];
		}

		array_multisort($sort_order, SORT_ASC, $dashboards);

		// Split the array so the columns width is not more than 12 on each row.
		$width = 0;
		$column = array();
		$data['rows'] = array();

		foreach ($dashboards as $dashboard) {
			$column[] = $dashboard;

			$width = ($width + $dashboard['width']);

			if ($width >= 12) {
				$data['rows'][] = $column;

				$width = 0;
				$column = array();
			}
		}

		if (!empty($column)) {
    			$data['rows'][] = $column;
		}

		if (DIR_STORAGE == 'system/storage/') {
			$data['security'] = load::controller('common/security');
		} else {
			$data['security'] = '';
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		// Run currency update
		if (config('currency_auto')) {

			admin_localisation_currency::refresh();
		}

		response::setOutput(load::view('common/dashboard', $data));
	}
}
