<?php
class ControllerCommonLogin {
	private static $error = array();

	public static function index() {
		load::language('common/login');

		document::setTitle(language::get('heading_title'));

		if (user::isLogged()) {
			response::redirect(url::link('common/dashboard'));
		}

		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			session::$data['user_token'] = token(32);

			if (isset(request::$post['redirect']) && (strpos(request::$post['redirect'], HTTP_SERVER) === 0 || strpos(request::$post['redirect'], HTTPS_SERVER) === 0)) {
				response::redirect(request::$post['redirect']);
			} else {
				response::redirect(url::link('common/dashboard'));
			}
		}

		if (isset(self::$error['error_attempts'])) {
			$data['error_warning'] = self::$error['error_attempts'];
		} elseif (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		$data['action'] = url::link('common/login', '');

		if (isset(request::$post['username'])) {
			$data['username'] = request::$post['username'];
		} else {
			$data['username'] = '';
		}

		if (isset(request::$post['password'])) {
			$data['password'] = request::$post['password'];
		} else {
			$data['password'] = '';
		}

		if (isset(request::$get['route'])) {
			$route = request::$get['route'];

			unset(request::$get['route']);

			$url = '';

			if (request::$get) {
				$url .= http_build_query(request::$get);
			}

			$data['redirect'] = url::link($route, $url);
		} else {
			$data['redirect'] = '';
		}

		if (config('password')) {
			$data['forgotten'] = url::link('common/forgotten', '');
		} else {
			$data['forgotten'] = '';
		}

		$data['header'] = load::controller('common/header');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('common/login', $data));
	}

	protected static function validate() {
		if (!isset(request::$post['username']) || !isset(request::$post['password']) || !request::$post['username'] || !request::$post['password']) {
			self::$error['warning'] = language::get('error_login');
		} else {

			// Check how many login attempts have been made.
			$login_info = admin_user_user::getLoginAttempts(request::$post['username']);

			if ($login_info && ($login_info['total'] >= config('login_attempts')) && strtotime('-1 hour') < strtotime($login_info['date_modified'])) {
				self::$error['error_attempts'] = language::get('error_attempts');
			}
		}

		if (!self::$error) {
			if (!user::login(request::$post['username'], html_entity_decode(request::$post['password'], ENT_QUOTES, 'UTF-8'))) {
				self::$error['warning'] = language::get('error_login');

				admin_user_user::addLoginAttempt(request::$post['username']);

				unset(session::$data['user_token']);
			} else {
				admin_user_user::deleteLoginAttempts(request::$post['username']);
			}
		}

		return !self::$error;
	}
}
