<?php
class ControllerCommonProfile {
	private static $error = array();

	public static function index() {
		load::language('common/profile');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			$user_data = array_merge(request::$post, array(
				'user_group_id' => user::getGroupId(),
				'status'        => 1,
			));

			admin_user_user::editUser(user::getId(), $user_data);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('common/profile'));
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['username'])) {
			$data['error_username'] = self::$error['username'];
		} else {
			$data['error_username'] = '';
		}

		if (isset(self::$error['password'])) {
			$data['error_password'] = self::$error['password'];
		} else {
			$data['error_password'] = '';
		}

		if (isset(self::$error['confirm'])) {
			$data['error_confirm'] = self::$error['confirm'];
		} else {
			$data['error_confirm'] = '';
		}

		if (isset(self::$error['firstname'])) {
			$data['error_firstname'] = self::$error['firstname'];
		} else {
			$data['error_firstname'] = '';
		}

		if (isset(self::$error['lastname'])) {
			$data['error_lastname'] = self::$error['lastname'];
		} else {
			$data['error_lastname'] = '';
		}

		if (isset(self::$error['email'])) {
			$data['error_email'] = self::$error['email'];
		} else {
			$data['error_email'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('common/profile')
		);

		$data['action'] = url::link('common/profile');

		$data['cancel'] = url::link('common/dashboard');

		if (request::$server['REQUEST_METHOD'] != 'POST') {
			$user_info = admin_user_user::getUser(user::getId());
		}

		if (isset(request::$post['username'])) {
			$data['username'] = request::$post['username'];
		} elseif (!empty($user_info)) {
			$data['username'] = $user_info['username'];
		} else {
			$data['username'] = '';
		}

		if (isset(request::$post['password'])) {
			$data['password'] = request::$post['password'];
		} else {
			$data['password'] = '';
		}

		if (isset(request::$post['confirm'])) {
			$data['confirm'] = request::$post['confirm'];
		} else {
			$data['confirm'] = '';
		}

		if (isset(request::$post['firstname'])) {
			$data['firstname'] = request::$post['firstname'];
		} elseif (!empty($user_info)) {
			$data['firstname'] = $user_info['firstname'];
		} else {
			$data['firstname'] = '';
		}

		if (isset(request::$post['lastname'])) {
			$data['lastname'] = request::$post['lastname'];
		} elseif (!empty($user_info)) {
			$data['lastname'] = $user_info['lastname'];
		} else {
			$data['lastname'] = '';
		}

		if (isset(request::$post['email'])) {
			$data['email'] = request::$post['email'];
		} elseif (!empty($user_info)) {
			$data['email'] = $user_info['email'];
		} else {
			$data['email'] = '';
		}

		if (isset(request::$post['image'])) {
			$data['image'] = request::$post['image'];
		} elseif (!empty($user_info)) {
			$data['image'] = $user_info['image'];
		} else {
			$data['image'] = '';
		}


		if (isset(request::$post['image']) && is_file('image/' . request::$post['image'])) {
			$data['thumb'] = admin_tool_image::resize(request::$post['image'], 100, 100);
		} elseif (!empty($user_info) && $user_info['image'] && is_file('image/' . $user_info['image'])) {
			$data['thumb'] = admin_tool_image::resize($user_info['image'], 100, 100);
		} else {
			$data['thumb'] = admin_tool_image::resize('no_image.png', 100, 100);
		}

		$data['placeholder'] = admin_tool_image::resize('no_image.png', 100, 100);

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('common/profile', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'common/profile')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(request::$post['username']) < 3) || (utf8_strlen(request::$post['username']) > 20)) {
			self::$error['username'] = language::get('error_username');
		}

		$user_info = admin_user_user::getUserByUsername(request::$post['username']);

		if ($user_info && (user::getId() != $user_info['user_id'])) {
			self::$error['warning'] = language::get('error_exists_username');
		}

		if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
			self::$error['firstname'] = language::get('error_firstname');
		}

		if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
			self::$error['lastname'] = language::get('error_lastname');
		}

		if ((utf8_strlen(request::$post['email']) > 96) || !filter_var(request::$post['email'], FILTER_VALIDATE_EMAIL)) {
			self::$error['email'] = language::get('error_email');
		}

		$user_info = admin_user_user::getUserByEmail(request::$post['email']);

		if ($user_info && (user::getId() != $user_info['user_id'])) {
			self::$error['warning'] = language::get('error_exists_email');
		}

		if (request::$post['password']) {
			if ((utf8_strlen(html_entity_decode(request::$post['password'], ENT_QUOTES, 'UTF-8')) < 4) || (utf8_strlen(html_entity_decode(request::$post['password'], ENT_QUOTES, 'UTF-8')) > 40)) {
				self::$error['password'] = language::get('error_password');
			}

			if (request::$post['password'] != request::$post['confirm']) {
				self::$error['confirm'] = language::get('error_confirm');
			}
		}

		return !self::$error;
	}
}
