<?php
class ControllerCustomerCustomField {
	private static $error = array();

	public static function index() {
		load::language('customer/custom_field');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('customer/custom_field');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_customer_custom_field::addCustomField(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('customer/custom_field', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('customer/custom_field');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_customer_custom_field::editCustomField(request::$get['custom_field_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('customer/custom_field', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('customer/custom_field');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $custom_field_id) {
				admin_customer_custom_field::deleteCustomField($custom_field_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('customer/custom_field', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'cfd.name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('customer/custom_field', $url)
		);

		$data['add'] = url::link('customer/custom_field/add', $url);
		$data['delete'] = url::link('customer/custom_field/delete', $url);

		$data['custom_fields'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$custom_field_total = admin_customer_custom_field::getTotalCustomFields();

		$results = admin_customer_custom_field::getCustomFields($filter_data);

		foreach ($results as $result) {
			$type = '';

			switch ($result['type']) {
				case 'select':
					$type = language::get('text_select');
					break;
				case 'radio':
					$type = language::get('text_radio');
					break;
				case 'checkbox':
					$type = language::get('text_checkbox');
					break;
				case 'input':
					$type = language::get('text_input');
					break;
				case 'text':
					$type = language::get('text_text');
					break;
				case 'textarea':
					$type = language::get('text_textarea');
					break;
				case 'file':
					$type = language::get('text_file');
					break;
				case 'date':
					$type = language::get('text_date');
					break;
				case 'datetime':
					$type = language::get('text_datetime');
					break;
				case 'time':
					$type = language::get('text_time');
					break;
			}

			$data['custom_fields'][] = array(
				'custom_field_id' => $result['custom_field_id'],
				'name'            => $result['name'],
				'location'        => language::get('text_' . $result['location']),
				'type'            => $type,
				'status'          => $result['status'],
				'sort_order'      => $result['sort_order'],
				'edit'            => url::link('customer/custom_field/edit', '&custom_field_id=' . $result['custom_field_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('customer/custom_field', '&sort=cfd.name' . $url);
		$data['sort_location'] = url::link('customer/custom_field', '&sort=cf.location' . $url);
		$data['sort_type'] = url::link('customer/custom_field', '&sort=cf.type' . $url);
		$data['sort_status'] = url::link('customer/custom_field', '&sort=cf.status' . $url);
		$data['sort_sort_order'] = url::link('customer/custom_field', '&sort=cf.sort_order' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $custom_field_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('customer/custom_field', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($custom_field_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($custom_field_total - config('limit_admin'))) ? $custom_field_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $custom_field_total, ceil($custom_field_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('customer/custom_field_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['custom_field_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = array();
		}

		if (isset(self::$error['custom_field_value'])) {
			$data['error_custom_field_value'] = self::$error['custom_field_value'];
		} else {
			$data['error_custom_field_value'] = array();
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('customer/custom_field', $url)
		);

		if (!isset(request::$get['custom_field_id'])) {
			$data['action'] = url::link('customer/custom_field/add', $url);
		} else {
			$data['action'] = url::link('customer/custom_field/edit', '&custom_field_id=' . request::$get['custom_field_id'] . $url);
		}

		$data['cancel'] = url::link('customer/custom_field', $url);

		if (isset(request::$get['custom_field_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$custom_field_info = admin_customer_custom_field::getCustomField(request::$get['custom_field_id']);
		}



		$data['languages'] = admin_localisation_language::getLanguages();

		if (isset(request::$post['custom_field_description'])) {
			$data['custom_field_description'] = request::$post['custom_field_description'];
		} elseif (isset(request::$get['custom_field_id'])) {
			$data['custom_field_description'] = admin_customer_custom_field::getCustomFieldDescriptions(request::$get['custom_field_id']);
		} else {
			$data['custom_field_description'] = array();
		}

		if (isset(request::$post['location'])) {
			$data['location'] = request::$post['location'];
		} elseif (!empty($custom_field_info)) {
			$data['location'] = $custom_field_info['location'];
		} else {
			$data['location'] = '';
		}

		if (isset(request::$post['type'])) {
			$data['type'] = request::$post['type'];
		} elseif (!empty($custom_field_info)) {
			$data['type'] = $custom_field_info['type'];
		} else {
			$data['type'] = '';
		}

		if (isset(request::$post['value'])) {
			$data['value'] = request::$post['value'];
		} elseif (!empty($custom_field_info)) {
			$data['value'] = $custom_field_info['value'];
		} else {
			$data['value'] = '';
		}

		if (isset(request::$post['validation'])) {
			$data['validation'] = request::$post['validation'];
		} elseif (!empty($custom_field_info)) {
			$data['validation'] = $custom_field_info['validation'];
		} else {
			$data['validation'] = '';
		}

		if (isset(request::$post['status'])) {
			$data['status'] = request::$post['status'];
		} elseif (!empty($custom_field_info)) {
			$data['status'] = $custom_field_info['status'];
		} else {
			$data['status'] = '';
		}

		if (isset(request::$post['sort_order'])) {
			$data['sort_order'] = request::$post['sort_order'];
		} elseif (!empty($custom_field_info)) {
			$data['sort_order'] = $custom_field_info['sort_order'];
		} else {
			$data['sort_order'] = '';
		}

		if (isset(request::$post['custom_field_value'])) {
			$custom_field_values = request::$post['custom_field_value'];
		} elseif (isset(request::$get['custom_field_id'])) {
			$custom_field_values = admin_customer_custom_field::getCustomFieldValueDescriptions(request::$get['custom_field_id']);
		} else {
			$custom_field_values = array();
		}

		$data['custom_field_values'] = array();

		foreach ($custom_field_values as $custom_field_value) {
			$data['custom_field_values'][] = array(
				'custom_field_value_id'          => $custom_field_value['custom_field_value_id'],
				'custom_field_value_description' => $custom_field_value['custom_field_value_description'],
				'sort_order'                     => $custom_field_value['sort_order']
			);
		}

		if (isset(request::$post['custom_field_customer_group'])) {
			$custom_field_customer_groups = request::$post['custom_field_customer_group'];
		} elseif (isset(request::$get['custom_field_id'])) {
			$custom_field_customer_groups = admin_customer_custom_field::getCustomFieldCustomerGroups(request::$get['custom_field_id']);
		} else {
			$custom_field_customer_groups = array();
		}

		$data['custom_field_customer_group'] = array();

		foreach ($custom_field_customer_groups as $custom_field_customer_group) {
			$data['custom_field_customer_group'][] = $custom_field_customer_group['customer_group_id'];
		}

		$data['custom_field_required'] = array();

		foreach ($custom_field_customer_groups as $custom_field_customer_group) {
			if ($custom_field_customer_group['required']) {
				$data['custom_field_required'][] = $custom_field_customer_group['customer_group_id'];
			}
		}


		$data['customer_groups'] = admin_customer_customer_group::getCustomerGroups();

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('customer/custom_field_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'customer/custom_field')) {
			self::$error['warning'] = language::get('error_permission');
		}

		foreach (request::$post['custom_field_description'] as $language_id => $value) {
			if ((utf8_strlen($value['name']) < 1) || (utf8_strlen($value['name']) > 128)) {
				self::$error['name'][$language_id] = language::get('error_name');
			}
		}

		if ((request::$post['type'] == 'select' || request::$post['type'] == 'radio' || request::$post['type'] == 'checkbox')) {
			if (!isset(request::$post['custom_field_value'])) {
				self::$error['warning'] = language::get('error_type');
			}

			if (isset(request::$post['custom_field_value'])) {
				foreach (request::$post['custom_field_value'] as $custom_field_value_id => $custom_field_value) {
					foreach ($custom_field_value['custom_field_value_description'] as $language_id => $custom_field_value_description) {
						if ((utf8_strlen($custom_field_value_description['name']) < 1) || (utf8_strlen($custom_field_value_description['name']) > 128)) {
							self::$error['custom_field_value'][$custom_field_value_id][$language_id] = language::get('error_custom_value');
						}
					}
				}
			}
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'customer/custom_field')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
}