<?php
class ControllerDesignLayout {
	private static $error = array();

	public static function index() {
		load::language('design/layout');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('design/layout');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_design_layout::addLayout(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('design/layout', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('design/layout');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_design_layout::editLayout(request::$get['layout_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('design/layout', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('design/layout');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $layout_id) {
				admin_design_layout::deleteLayout($layout_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('design/layout', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('design/layout', $url)
		);

		$data['add'] = url::link('design/layout/add', $url);
		$data['delete'] = url::link('design/layout/delete', $url);

		$data['layouts'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$layout_total = admin_design_layout::getTotalLayouts();

		$results = admin_design_layout::getLayouts($filter_data);

		foreach ($results as $result) {
			$data['layouts'][] = array(
				'layout_id' => $result['layout_id'],
				'name'      => $result['name'],
				'edit'      => url::link('design/layout/edit', '&layout_id=' . $result['layout_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('design/layout', '&sort=name' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $layout_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('design/layout', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($layout_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($layout_total - config('limit_admin'))) ? $layout_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $layout_total, ceil($layout_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('design/layout_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['layout_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('design/layout', $url)
		);

		if (!isset(request::$get['layout_id'])) {
			$data['action'] = url::link('design/layout/add', $url);
		} else {
			$data['action'] = url::link('design/layout/edit', '&layout_id=' . request::$get['layout_id'] . $url);
		}

		$data['cancel'] = url::link('design/layout', $url);


		if (isset(request::$get['layout_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$layout_info = admin_design_layout::getLayout(request::$get['layout_id']);
		}

		if (isset(request::$post['name'])) {
			$data['name'] = request::$post['name'];
		} elseif (!empty($layout_info)) {
			$data['name'] = $layout_info['name'];
		} else {
			$data['name'] = '';
		}


		$data['stores'] = admin_setting_store::getStores();

		if (isset(request::$post['layout_route'])) {
			$data['layout_routes'] = request::$post['layout_route'];
		} elseif (isset(request::$get['layout_id'])) {
			$data['layout_routes'] = admin_design_layout::getLayoutRoutes(request::$get['layout_id']);
		} else {
			$data['layout_routes'] = array();
		}



		$data['extensions'] = array();

		// Get a list of installed modules
		$extensions = admin_setting_extension::getInstalled('module');

		// Add all the modules which have multiple settings for each module
		foreach ($extensions as $code) {
			load::language('extension/module/' . $code, 'extension');

			$module_data = array();

			$modules = admin_setting_module::getModulesByCode($code);

			foreach ($modules as $module) {
				$module_data[] = array(
					'name' => strip_tags($module['name']),
					'code' => $code . '.' .  $module['module_id']
				);
			}

			if (config::has('module_' . $code . '_status') || $module_data) {
				$data['extensions'][] = array(
					'name'   => strip_tags(language::get('extension')::get('heading_title')),
					'code'   => $code,
					'module' => $module_data
				);
			}
		}

		// Modules layout
		if (isset(request::$post['layout_module'])) {
			$layout_modules = request::$post['layout_module'];
		} elseif (isset(request::$get['layout_id'])) {
			$layout_modules = admin_design_layout::getLayoutModules(request::$get['layout_id']);
		} else {
			$layout_modules = array();
		}

		$data['layout_modules'] = array();

		// Add all the modules which have multiple settings for each module
		foreach ($layout_modules as $layout_module) {
			$part = explode('.', $layout_module['code']);

			if (!isset($part[1])) {
				$data['layout_modules'][] = array(
					'code'       => $layout_module['code'],
					'edit'       => url::link('extension/module/' . $part[0]),
					'position'   => $layout_module['position'],
					'sort_order' => $layout_module['sort_order']
				);
			} else {
				$module_info = admin_setting_module::getModule($part[1]);

				if ($module_info) {
					$data['layout_modules'][] = array(
						'code'       => $layout_module['code'],
						'edit'       => url::link('extension/module/' . $part[0], '&module_id=' . $part[1]),
						'position'   => $layout_module['position'],
						'sort_order' => $layout_module['sort_order']
					);
				}
			}
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('design/layout_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'design/layout')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(request::$post['name']) < 3) || (utf8_strlen(request::$post['name']) > 64)) {
			self::$error['name'] = language::get('error_name');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'design/layout')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $layout_id) {
			if (config('layout_id') == $layout_id) {
				self::$error['warning'] = language::get('error_default');
			}

			$store_total = admin_setting_store::getTotalStoresByLayoutId($layout_id);

			if ($store_total) {
				self::$error['warning'] = sprintf(language::get('error_store'), $store_total);
			}

			$product_total = admin_catalog_product::getTotalProductsByLayoutId($layout_id);

			if ($product_total) {
				self::$error['warning'] = sprintf(language::get('error_product'), $product_total);
			}

			$category_total = admin_catalog_category::getTotalCategoriesByLayoutId($layout_id);

			if ($category_total) {
				self::$error['warning'] = sprintf(language::get('error_category'), $category_total);
			}

			$information_total = admin_catalog_information::getTotalInformationsByLayoutId($layout_id);

			if ($information_total) {
				self::$error['warning'] = sprintf(language::get('error_information'), $information_total);
			}
		}

		return !self::$error;
	}
}
