<?php
class ControllerDesignTheme {
	public static function index() {
		load::language('design/theme');

		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('design/theme')
		);


		$data['stores'] = array();


		$results = admin_setting_store::getStores();

		foreach ($results as $result) {
			$data['stores'][] = array(
				'store_id' => $result['store_id'],
				'name'     => $result['name']
			);
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('design/theme', $data));
	}

	public static function history() {
		load::language('design/theme');

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$data['histories'] = array();


		$history_total = admin_design_theme::getTotalThemes();

		$results = admin_design_theme::getThemes(($page - 1) * 10, 10);

		foreach ($results as $result) {
			$store_info = admin_setting_store::getStore($result['store_id']);

			if ($store_info) {
				$store = $store_info['name'];
			} else {
				$store = '';
			}

			$data['histories'][] = array(
				'store_id'   => $result['store_id'],
				'store'      => ($result['store_id'] ? $store : language::get('text_default')),
				'route'      => $result['route'],
				'theme'      => $result['theme'],
				'date_added' => date(language::get('date_format_short'), strtotime($result['date_added'])),
				'edit'       => url::link('design/theme/template'),
				'delete'     => url::link('design/theme/delete', '&theme_id=' . $result['theme_id'])
			);
		}

		$pagination = new Pagination();
		$pagination::$total = $history_total;
		$pagination::$page = $page;
		$pagination::$limit = 10;
		$pagination::$url = url::link('design/theme/history', '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($history_total) ? (($page - 1) * 10) + 1 : 0, ((($page - 1) * 10) > ($history_total - 10)) ? $history_total : ((($page - 1) * 10) + 10), $history_total, ceil($history_total / 10));

		response::setOutput(load::view('design/theme_history', $data));
	}

	public static function path() {
		load::language('design/theme');

		$json = array();

		if (isset(request::$get['store_id'])) {
			$store_id = request::$get['store_id'];
		} else {
			$store_id = 0;
		}


		$theme = admin_setting_setting::getSettingValue('config_theme', $store_id);

		// This is only here for compatibility with old themes.
		if ($theme == 'default') {
			$theme = admin_setting_setting::getSettingValue('theme_default_directory', $store_id);
		}

		if (isset(request::$get['path'])) {
			$path = request::$get['path'];
		} else {
			$path = '';
		}

		if (substr(str_replace('\\', '/', realpath(DIR_CATALOG . 'view/theme/default/template/' . $path)), 0, strlen(DIR_CATALOG . 'view')) == DIR_CATALOG . 'view') {
			$path_data = array();

			// We grab the files from the default theme directory first as the custom themes drops back to the default theme if selected theme files can not be found.
			$files = glob(rtrim(DIR_CATALOG . 'view/theme/{default,' . $theme . '}/template/' . $path, '/') . '/*', GLOB_BRACE);

			if ($files) {
				foreach($files as $file) {
					if (!in_array(basename($file), $path_data))  {
						if (is_dir($file)) {
							$json['directory'][] = array(
								'name' => basename($file),
								'path' => trim($path . '/' . basename($file), '/')
							);
						}

						if (is_file($file)) {
							$json['file'][] = array(
								'name' => basename($file),
								'path' => trim($path . '/' . basename($file), '/')
							);
						}

						$path_data[] = basename($file);
					}
				}
			}
		}

		if (!empty(request::$get['path'])) {
			$json['back'] = array(
				'name' => language::get('button_back'),
				'path' => urlencode(substr($path, 0, strrpos($path, '/'))),
			);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function template() {
		load::language('design/theme');

		$json = array();

		if (isset(request::$get['store_id'])) {
			$store_id = request::$get['store_id'];
		} else {
			$store_id = 0;
		}


		$theme = admin_setting_setting::getSettingValue('config_theme', $store_id);

		// This is only here for compatibility with old themes.
		if ($theme == 'default') {
			$theme = admin_setting_setting::getSettingValue('theme_default_directory', $store_id);
		}

		if (isset(request::$get['path'])) {
			$path = request::$get['path'];
		} else {
			$path = '';
		}


		$theme_info = admin_design_theme::getTheme($store_id, $theme, $path);

		if ($theme_info) {
			$json['code'] = html_entity_decode($theme_info['code']);
		} elseif (is_file(DIR_CATALOG . 'view/theme/' . $theme . '/template/' . $path) && (substr(str_replace('\\', '/', realpath(DIR_CATALOG . 'view/theme/' . $theme . '/template/' . $path)), 0, strlen(DIR_CATALOG . 'view')) == DIR_CATALOG . 'view')) {
			$json['code'] = file_get_contents(DIR_CATALOG . 'view/theme/' . $theme . '/template/' . $path);
		} elseif (is_file(DIR_CATALOG . 'view/theme/default/template/' . $path) && (substr(str_replace('\\', '/', realpath(DIR_CATALOG . 'view/theme/default/template/' . $path)), 0, strlen(DIR_CATALOG . 'view')) == DIR_CATALOG . 'view')) {
			$json['code'] = file_get_contents(DIR_CATALOG . 'view/theme/default/template/' . $path);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function save() {
		load::language('design/theme');

		$json = array();

		if (isset(request::$get['store_id'])) {
			$store_id = request::$get['store_id'];
		} else {
			$store_id = 0;
		}


		$theme = admin_setting_setting::getSettingValue('config_theme', $store_id);

		// This is only here for compatibility with old themes.
		if ($theme == 'default') {
			$theme = admin_setting_setting::getSettingValue('theme_default_directory', $store_id);
		}

		if (isset(request::$get['path'])) {
			$path = request::$get['path'];
		} else {
			$path = '';
		}

		// Check user has permission
		if (!user::hasPermission('modify', 'design/theme')) {
			$json['error'] = language::get('error_permission');
		}

		if (substr($path, -5) != '.twig') {
			$json['error'] = language::get('error_twig');
		}

		if (!$json) {

			$pos = strpos($path, '.');

			admin_design_theme::editTheme($store_id, $theme, ($pos !== false) ? substr($path, 0, $pos) : $path, request::$post['code']);

			$json['success'] = language::get('text_success');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function reset() {
		load::language('design/theme');

		$json = array();

		if (isset(request::$get['store_id'])) {
			$store_id = request::$get['store_id'];
		} else {
			$store_id = 0;
		}


		$theme = admin_setting_setting::getSettingValue('config_theme', $store_id);

		// This is only here for compatibility with old themes.
		if ($theme == 'default') {
			$theme = admin_setting_setting::getSettingValue('theme_default_directory', $store_id);
		}

		if (isset(request::$get['path'])) {
			$path = request::$get['path'];
		} else {
			$path = '';
		}

		if (is_file(DIR_CATALOG . 'view/theme/' . $theme . '/template/' . $path) && (substr(str_replace('\\', '/', realpath(DIR_CATALOG . 'view/theme/' . $theme . '/template/' . $path)), 0, strlen(DIR_CATALOG . 'view')) == DIR_CATALOG . 'view')) {
			$json['code'] = file_get_contents(DIR_CATALOG . 'view/theme/' . $theme . '/template/' . $path);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function delete() {
		load::language('design/theme');

		$json = array();

		if (isset(request::$get['theme_id'])) {
			$theme_id = request::$get['theme_id'];
		} else {
			$theme_id = 0;
		}

		// Check user has permission
		if (!user::hasPermission('modify', 'design/theme')) {
			$json['error'] = language::get('error_permission');
		}

		if (!$json) {

			admin_design_theme::deleteTheme($theme_id);

			$json['success'] = language::get('text_success');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
