<?php

use \googleshopping\exception\Connection as ConnectionException;
use \googleshopping\Googleshopping;
use \googleshopping\traits\LibraryLoader;
use \googleshopping\traits\StoreLoader;

class ControllerExtensionAdvertiseGoogle {
    use StoreLoader;
    use LibraryLoader;

    private static $error = array();
    private static $store_id = 0;

    public static function init() {
        parent::__construct();

        self::$store_id = isset(request::$get['store_id']) ? (int)request::$get['store_id'] : 0;

        self::loadStore(self::$store_id);

        self::loadLibrary(self::$store_id);
    }

    public static function index() {
        load::language('extension/advertise/google');


        load::config('googleshopping/googleshopping');

        // Fix clashes with third-party extension table names
        admin_extension_advertise_google::renameTables();

        // Even though this should be ran during install, there are known cases of webstores which do not trigger the install method. This is why we run createTables here explicitly.
        admin_extension_advertise_google::createTables();

        // Fix a missing AUTO_INCREMENT
        admin_extension_advertise_google::fixColumns();
        
        // Redirect to the preliminary check-list
        if (!setting::get('advertise_google_checklist_confirmed')) {
            response::redirect(url::link('extension/advertise/google/checklist', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
        }

        try {
            // If we have not connected, navigate to connect screen
            if (!setting::has('advertise_google_access_token')) {
                response::redirect(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } else if (!setting::has('advertise_google_gmc_account_selected')) {
                // In case the merchant has made no decision about which GMC account to use, redirect to the form for connection
                response::redirect(url::link('extension/advertise/google/merchant', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } else if (!googleshopping::isStoreUrlClaimed()) {
                if (empty(session::$data['error'])) {
                    session::$data['error'] = language::get('error_store_url_claim');
                }

                // In case the merchant has made no decision about which GMC account to use, redirect to the form for connection
                response::redirect(url::link('extension/advertise/google/merchant', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } else if (count(googleshopping::getTargets(self::$store_id)) == 0) {
                response::redirect(url::link('extension/advertise/google/campaign', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } else if (!setting::has('advertise_google_gmc_shipping_taxes_configured')) {
                // In case the merchant has not set up shipping and taxes, redirect them to the form for shipping and taxes
                response::redirect(url::link('extension/advertise/google/shipping_taxes', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } else if (count(admin_extension_advertise_google::getMapping(self::$store_id)) == 0) {
                // In case the merchant has not set up mapping, redirect them to the form for mapping
                response::redirect(url::link('extension/advertise/google/mapping', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            }

            // Pull the campaign reports
            googleshopping::getCampaignReports();
        } catch (ConnectionException $e) {
            session::$data['error'] = $e::getMessage();

            response::redirect(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
        } catch (\RuntimeException $e) {
            self::$error['warning'] = $e::getMessage();
        }

        if (request::$server['REQUEST_METHOD'] == 'POST' && self::validateSettings()) {
            self::applyNewSettings(request::$post);

            try {
                // Profilactic target push, as sometimes targets are not initialized properly
                googleshopping::pushTargets();
                googleshopping::pushCampaignStatus();

                session::$data['success'] = language::get('success_index');

                response::redirect(url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } catch (ConnectionException $e) {
                session::$data['error'] = $e::getMessage();

                response::redirect(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } catch (\RuntimeException $e) {
                self::$error['warning'] = $e::getMessage();
            }
        }

        document::setTitle(language::get('heading_title'));

        $data = array();

        $data['text_connected'] = sprintf(language::get('text_connected'), setting::get('advertise_google_gmc_account_id'));

        $data['error'] = '';

        if (isset(session::$data['error'])) {
            $data['error'] = session::$data['error'];
            unset(session::$data['error']);
        } else if (!empty(self::$error['warning'])) {
            $data['error'] = self::$error['warning'];
        }

        $data['error_cron_email']                   = self::getValidationError('cron_email');
        $data['error_cron_acknowledge']             = self::getValidationError('cron_acknowledge');

        $data['success'] = '';

        if (isset(session::$data['success'])) {
            $data['success'] = session::$data['success'];
            unset(session::$data['success']);
        }

        $advertised_count = admin_extension_advertise_google::getAdvertisedCount(self::$store_id);
        $last_cron_executed = (int)setting::get('advertise_google_cron_last_executed');

        $data['warning'] = '';

        if (!setting::get('advertise_google_status') && admin_extension_advertise_google::hasActiveTarget(self::$store_id)) {
            $data['warning'] = language::get('warning_disabled');
        } else if (!admin_extension_advertise_google::hasActiveTarget(self::$store_id)) {
            $data['warning'] = sprintf(language::get('warning_no_active_campaigns'), url::link('extension/advertise/google/campaign', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token'] . '&from_dashboard=true'));
        } else if ($advertised_count == 0) {
            $data['warning'] = sprintf(language::get("warning_no_advertised_products"), language::get("text_video_tutorial_url_advertise"));
        } else if ($last_cron_executed + 24 * 60 * 60 <= time()) {
            $data['warning'] = sprintf(language::get("warning_last_cron_executed"), language::get("text_tutorial_cron"));
        }

        $data['breadcrumbs']   = array();
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_home'),
            'href' => url::link('common/dashboard'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_extensions'),
            'href' => url::link('marketplace/extension', '&type=advertise'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('heading_title'),
            'href' => url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']),
        );

        $reporting_intervals = config::get('advertise_google_reporting_intervals');


        $data['cancel'] = url::link('marketplace/extension', '&type=advertise');
        $data['action'] = url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']);
        $data['shipping_taxes'] = url::link('extension/advertise/google/shipping_taxes', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token'] . '&from_dashboard=true');
        $data['campaign'] = url::link('extension/advertise/google/campaign', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token'] . '&from_dashboard=true');
        $data['mapping'] = url::link('extension/advertise/google/mapping', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token'] . '&from_dashboard=true');
        $data['disconnect'] = url::link('extension/advertise/google/disconnect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']);
        $data['list_ads'] = html_entity_decode(url::link('extension/advertise/google/list_ads', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
        $data['advertise'] = html_entity_decode(url::link('extension/advertise/google/advertise', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
        $data['url_popup'] = html_entity_decode(url::link('extension/advertise/google/popup_product', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
        $data['url_category_autocomplete'] = html_entity_decode(url::link('extension/advertise/google/category_autocomplete', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
        $data['url_debug_log_download'] = html_entity_decode(url::link('extension/advertise/google/debug_log_download', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');

        $data['advertise_google_status']                    = self::getSettingValue('advertise_google_status', 0);
        $data['advertise_google_debug_log']                 = self::getSettingValue('advertise_google_debug_log', 0);
        $data['advertise_google_cron_email_status']         = self::getSettingValue('advertise_google_cron_email_status');
        $data['advertise_google_cron_email']                = self::getSettingValue('advertise_google_cron_email', config('email'));
        $data['advertise_google_cron_token']                = self::getSettingValue('advertise_google_cron_token');
        $data['advertise_google_cron_acknowledge']          = self::getSettingValue('advertise_google_cron_acknowledge', null, true);

        if (isset(request::$post['advertise_google_reporting_interval'])) {
            $data['advertise_google_reporting_interval'] = request::$post['advertise_google_reporting_interval'];
        } else if (setting::has('advertise_google_reporting_interval') && in_array(setting::get('advertise_google_reporting_interval'), $reporting_intervals)) {
            $data['advertise_google_reporting_interval'] = setting::get('advertise_google_reporting_interval');
        } else {
            $data['advertise_google_reporting_interval'] = config::get('advertise_google_reporting_intervals_default');
        }

        $server = googleshopping::getStoreUrl();

        $data['advertise_google_cron_command'] = 'export CUSTOM_SERVER_NAME=' . parse_url($server, PHP_URL_HOST) . '; export CUSTOM_SERVER_PORT=443; export ADVERTISE_GOOGLE_CRON=1; export ADVERTISE_GOOGLE_STORE_ID=' . self::$store_id . '; ' . PHP_BINDIR . '/php -d session.save_path=' . session_save_path() . ' -d memory_limit=256M ' . 'system/library/googleshopping/cron.php > /dev/null 2> /dev/null';
        
        if (!setting::get('advertise_google_cron_token')) {
            $data['advertise_google_cron_token'] = md5(mt_rand());
        }

        $host_and_uri = parse_url($server, PHP_URL_HOST) . parse_url($server, PHP_URL_PATH);

        $data['advertise_google_cron_url'] = 'https://' . rtrim($host_and_uri, '/') . '/index.php?route=extension/advertise/google/cron&cron_token={CRON_TOKEN}';

        $data['reporting_intervals'] = array();

        foreach ($reporting_intervals as $interval) {
            $data['reporting_intervals'][$interval] = language::get('text_reporting_interval_' . $interval);
        }

        $campaign_reports = setting::get('advertise_google_report_campaigns');

        $data['campaigns'] = googleshopping::getTargets(self::$store_id);

        $data['text_report_date_range'] = sprintf(language::get('text_report_date_range'), $campaign_reports['date_range']);
        $data['text_ads_intro'] = sprintf(language::get('text_ads_intro'), $data['shipping_taxes']);
        $data['advertise_google_report_campaigns'] = $campaign_reports['reports'];
        $data['text_panel_heading'] = sprintf(language::get('text_panel_heading'), googleshopping::getStoreName());

        $data['text_selection_all'] = str_replace("'", "\\'", language::get('text_selection_all'));
        $data['text_selection_page'] = str_replace("'", "\\'", language::get('text_selection_page'));

        $data['tab_settings'] = load::view('extension/advertise/google_settings', $data);
        $data['tab_ads']      = load::view('extension/advertise/google_ads', $data);
        $data['tab_reports']  = load::view('extension/advertise/google_reports', $data);

        $data['header']       = load::controller('common/header');
        $data['column_left']  = load::controller('common/column_left');
        $data['footer']       = load::controller('common/footer');

        response::setOutput(load::view('extension/advertise/google', $data));
    }

    public static function debug_log_download() {
        $filename = sprintf(Googleshopping::$DEBUG_LOG_FILENAME, self::$store_id);

        header('Pragma: no-cache');
        header('Expires: 0');
        header('Content-Description: File Transfer');
        header('Content-Type: plain/text');
        header('Content-Disposition: attachment; filename="' . $filename . '"');
        header('Content-Transfer-Encoding: binary');

        $file = DIR_LOGS . $filename;

        if (file_exists($file)) {
            readfile($file);
        }

        exit;
    }

    public static function advertise() {
        load::language('extension/advertise/google');
        
        $json = array(
            'success' => null,
            'redirect' => null,
            'error' => null,
            'warning' => null
        );

        if (self::validatePermission()) {

            $select = array();
            $filter_data = array();

            if (!empty(request::$post['all_pages'])) {
                $filter_data = self::getFilter(request::$post['filter']);
            } else if (isset(request::$post['select']) && is_array(request::$post['select'])) {
                $select = request::$post['select'];
            }

            if (!empty($select) || !empty($filter_data)) {
                $target_ids = !empty(request::$post['target_ids']) ? request::$post['target_ids'] : array();

                if (!empty($select)) {
                    admin_extension_advertise_google::setAdvertisingBySelect($select, $target_ids, self::$store_id);
                } else if (!empty($filter_data)) {
                    admin_extension_advertise_google::setAdvertisingByFilter($filter_data, $target_ids, self::$store_id);
                }

                if (!empty($target_ids)) {
                    $json['success'] = language::get('success_advertise_listed');
                } else {
                    $json['success'] = language::get('success_advertise_unlisted');
                }
            }
        } else {
            $json['error'] = self::$error['warning'];
        }

        // Refresh warnings
        $advertised_count = admin_extension_advertise_google::getAdvertisedCount(self::$store_id);
        $last_cron_executed = (int)setting::get('advertise_google_cron_last_executed');

        if (!setting::get('advertise_google_status') && admin_extension_advertise_google::hasActiveTarget(self::$store_id)) {
            $json['warning'] = language::get('warning_disabled');
        } else if (!admin_extension_advertise_google::hasActiveTarget(self::$store_id)) {
            $json['warning'] = sprintf(language::get('warning_no_active_campaigns'), url::link('extension/advertise/google/campaign', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token'] . '&from_dashboard=true'));
        } else if ($advertised_count == 0) {
            $json['warning'] = sprintf(language::get("warning_no_advertised_products"), language::get("text_video_tutorial_url_advertise"));
        } else if ($last_cron_executed + 24 * 60 * 60 <= time()) {
            $json['warning'] = sprintf(language::get("warning_last_cron_executed"), language::get("text_tutorial_cron"));
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function list_ads() {
        $json = array();


        admin_extension_advertise_google::insertNewProducts(array(), self::$store_id);

        load::language('extension/advertise/google');

        $page = (int)request::$post['page'];

        $filter_data = array(
            'sort' => request::$post['sort'],
            'order' => request::$post['order'],
            'start' => ($page - 1) * config('limit_admin'),
            'limit' => config('limit_admin')
        );

        $filter_data = array_merge($filter_data, self::getFilter(request::$post['filter']));

        $products = googleshopping::getProducts($filter_data, self::$store_id);

        $json['products'] = array_map(array($this, 'product'), $products);

        $product_total = googleshopping::getTotalProducts($filter_data, self::$store_id);

        $pagination = new Pagination();
        $pagination::$total = $product_total;
        $pagination::$page = request::$post['page'];
        $pagination::$limit = config('limit_admin');
        $pagination::$url = '{page}';

        $pages = ceil($product_total / config('limit_admin'));

        $json['showing'] = sprintf(language::get('text_pagination'), ($product_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($product_total - config('limit_admin'))) ? $product_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $product_total, $pages);

        $json['pagination'] = $pagination::render();
        $json['total'] = (int)$product_total;
        $json['pages'] = (int)$pages;

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function merchant() {
        load::language('extension/advertise/google');

        document::setTitle(language::get('heading_merchant'));

        document::addStyle('view/stylesheet/googleshopping/stepper.css');


        if (request::$server['REQUEST_METHOD'] == 'POST' && self::validatePermission()) {
            try {
                $redirect_uri = html_entity_decode(url::link('extension/advertise/google/callback_merchant', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
                $state = md5(microtime(true) . $redirect_uri . microtime(true));

                $auth_url_data = array(
                    'account_type' => request::$post['advertise_google_gmc_account_type'],
                    'redirect_uri' => $redirect_uri . '&state=' . $state
                );

                session::$data['advertise_google'] = $auth_url_data;
                session::$data['advertise_google']['state'] = $state;

                response::redirect(googleshopping::getMerchantAuthUrl($auth_url_data));
            } catch (ConnectionException $e) {
                session::$data['error'] = $e::getMessage();

                response::redirect(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } catch (\RuntimeException $e) {
                self::$error['warning'] = $e::getMessage();
            }
        }

        $data = array();

        $data['error'] = '';

        if (isset(session::$data['error'])) {
            $data['error'] = session::$data['error'];
            unset(session::$data['error']);
        } else if (!empty(self::$error['warning'])) {
            $data['error'] = self::$error['warning'];
        }

        $data['success'] = '';

        if (isset(session::$data['success'])) {
            $data['success'] = session::$data['success'];
            unset(session::$data['success']);
        }

        $data['breadcrumbs']   = array();
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_home'),
            'href' => url::link('common/dashboard'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_extensions'),
            'href' => url::link('marketplace/extension', '&type=advertise'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('heading_title'),
            'href' => url::link('extension/advertise/google/merchant', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']),
        );

        $data['cancel']       = url::link('marketplace/extension', '&type=advertise');
        $data['action']       = url::link('extension/advertise/google/merchant', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']);

        if (isset(request::$post['advertise_google_gmc_account_type'])) {
            $data['advertise_google_gmc_account_type'] = request::$post['advertise_google_gmc_account_type'];
        } else {
            $data['advertise_google_gmc_account_type'] = 'api';
        }

        $data['header']       = load::controller('common/header');
        $data['column_left']  = load::controller('common/column_left');
        $data['footer']       = load::controller('common/footer');

        $data['current_step'] = 2;
        $data['steps'] = load::view('extension/advertise/google_steps', $data);

        response::setOutput(load::view('extension/advertise/google_merchant', $data));
    }

    public static function shipping_taxes() {
        load::language('extension/advertise/google');

        document::setTitle(language::get('heading_shipping_taxes'));

        document::addStyle('view/stylesheet/googleshopping/stepper.css');


        load::config('googleshopping/googleshopping');

        if (request::$server['REQUEST_METHOD'] == 'POST' && self::validateShippingAndTaxes()) {
            try {
                self::applyNewSettings(request::$post);

                googleshopping::pushShippingAndTaxes();

                self::applyNewSettings(array(
                    'advertise_google_gmc_shipping_taxes_configured' => '1'
                ));

                session::$data['success'] = language::get('success_shipping_taxes');

                response::redirect(url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } catch (ConnectionException $e) {
                session::$data['error'] = $e::getMessage();

                response::redirect(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } catch (\RuntimeException $e) {
                self::$error['warning'] = $e::getMessage();
            }
        }

        $available_carriers = array();

        try {
            $available_carriers = googleshopping::getAvailableCarriers();
        } catch (ConnectionException $e) {
            session::$data['error'] = $e::getMessage();

            response::redirect(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
        } catch (\RuntimeException $e) {
            self::$error['warning'] = $e::getMessage();
        }

        $data = array();

        $data['error'] = '';

        if (isset(session::$data['error'])) {
            $data['error'] = session::$data['error'];
            unset(session::$data['error']);
        } else if (!empty(self::$error['warning'])) {
            $data['error'] = self::$error['warning'];
        }

        if (isset(self::$error['min_transit_time'])) {
            $data['error_min_transit_time'] = self::$error['min_transit_time'];
        } else {
            $data['error_min_transit_time'] = '';
        }

        if (isset(self::$error['max_transit_time'])) {
            $data['error_max_transit_time'] = self::$error['max_transit_time'];
        } else {
            $data['error_max_transit_time'] = '';
        }

        if (isset(self::$error['flat_rate'])) {
            $data['error_flat_rate'] = self::$error['flat_rate'];
        } else {
            $data['error_flat_rate'] = '';
        }

        if (isset(self::$error['carrier_postcode'])) {
            $data['error_carrier_postcode'] = self::$error['carrier_postcode'];
        } else {
            $data['error_carrier_postcode'] = '';
        }

        if (isset(self::$error['carrier_price_percentage'])) {
            $data['error_carrier_price_percentage'] = self::$error['carrier_price_percentage'];
        } else {
            $data['error_carrier_price_percentage'] = '';
        }

        if (isset(self::$error['carrier'])) {
            $data['error_carrier'] = self::$error['carrier'];
        } else {
            $data['error_carrier'] = '';
        }

        $data['success'] = '';

        if (isset(session::$data['success'])) {
            $data['success'] = session::$data['success'];
            unset(session::$data['success']);
        }

        $data['from_dashboard'] = isset(request::$get['from_dashboard']);

        $data['breadcrumbs']   = array();
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_home'),
            'href' => url::link('common/dashboard'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_extensions'),
            'href' => url::link('marketplace/extension', '&type=advertise'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('heading_title'),
            'href' => url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']),
        );

        if ($data['from_dashboard']) {
            $data['breadcrumbs'][] = array(
                'text' => language::get('heading_shipping_taxes'),
                'href' => url::link('extension/advertise/google/shipping_taxes', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token'] . '&from_dashboard=true'),
            );
        }

        if ($data['from_dashboard']) {
            $data['cancel']       = url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']);
        } else {
            $data['cancel']       = url::link('marketplace/extension', '&type=advertise');
        }

        $data['action']       = url::link('extension/advertise/google/shipping_taxes', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']);

        if (isset(request::$post['advertise_google_shipping_taxes'])) {
            $data['advertise_google_shipping_taxes'] = request::$post['advertise_google_shipping_taxes'];
        } else if (setting::has('advertise_google_shipping_taxes')) {
            $data['advertise_google_shipping_taxes'] = setting::get('advertise_google_shipping_taxes');
        } else {
            $data['advertise_google_shipping_taxes'] = array(
                'shipping_type' => 'flat',
                'flat_rate' => config::get('shipping_flat_cost'),
                'min_transit_time' => 1,
                'max_transit_time' => 14,
                'carrier_price_percentage' => 5,
                'tax_type' => config('country_id') == 223 ? 'usa' : 'not_usa'
            );
        }

        $data['available_carriers'] = $available_carriers;

        $data['states'] = config::get('advertise_google_tax_usa_states');

        $data['header']       = load::controller('common/header');
        $data['column_left']  = load::controller('common/column_left');
        $data['footer']       = load::controller('common/footer');

        $data['current_step'] = 4;
        $data['steps'] = load::view('extension/advertise/google_steps', $data);

        response::setOutput(load::view('extension/advertise/google_shipping_taxes', $data));
    }

    public static function mapping() {
        load::language('extension/advertise/google');

        document::setTitle(language::get('heading_mapping'));

        document::addStyle('view/stylesheet/googleshopping/stepper.css');


        if (request::$server['REQUEST_METHOD'] == 'POST' && self::validateMapping()) {
            try {
                foreach (request::$post['advertise_google_mapping'] as $google_product_category => $category_id) {
                    admin_extension_advertise_google::setCategoryMapping($google_product_category, self::$store_id, $category_id);
                }

                if (!empty(request::$post['advertise_google_modify_existing'])) {
                    admin_extension_advertise_google::updateGoogleProductCategoryMapping(self::$store_id);
                }

                session::$data['success'] = language::get('success_mapping');

                response::redirect(url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } catch (ConnectionException $e) {
                session::$data['error'] = $e::getMessage();

                response::redirect(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } catch (\RuntimeException $e) {
                self::$error['warning'] = $e::getMessage();
            }
        }

        $data = array();

        $data['error'] = '';

        if (isset(session::$data['error'])) {
            $data['error'] = session::$data['error'];
            unset(session::$data['error']);
        } else if (!empty(self::$error['warning'])) {
            $data['error'] = self::$error['warning'];
        }

        $data['success'] = '';

        if (isset(session::$data['success'])) {
            $data['success'] = session::$data['success'];
            unset(session::$data['success']);
        }

        $data['from_dashboard'] = isset(request::$get['from_dashboard']);

        $data['breadcrumbs']   = array();
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_home'),
            'href' => url::link('common/dashboard'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_extensions'),
            'href' => url::link('marketplace/extension', '&type=advertise'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('heading_title'),
            'href' => url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']),
        );

        if ($data['from_dashboard']) {
            $data['breadcrumbs'][] = array(
                'text' => language::get('heading_shipping_taxes'),
                'href' => url::link('extension/advertise/google/mapping', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token'] . '&from_dashboard=true'),
            );
        }

        load::config('googleshopping/googleshopping');

        $data['mapping'] = array();

        foreach (config::get('advertise_google_google_product_categories') as $google_product_category_id => $google_product_category_name) {
            if ($google_product_category_id == 0) continue;

            $category_id = '';
            $name = '';

            if (null !== $category = admin_extension_advertise_google::getMappedCategory($google_product_category_id, self::$store_id)) {
                $category_id = $category['category_id'];
                $name = $category['name'];
            }

            $map = array(
                'google_product_category' => array(
                    'id' => $google_product_category_id,
                    'name' => $google_product_category_name
                ),
                'oc_category' => array(
                    'category_id' => $category_id,
                    'name' => $name
                )
            );

            $data['mapping'][] = $map;
        }

        $data['mapping_json'] = json_encode($data['mapping']);

        if ($data['from_dashboard']) {
            $data['cancel']       = url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']);
        } else {
            $data['cancel']       = url::link('marketplace/extension', '&type=advertise');
        }

        $data['action']       = url::link('extension/advertise/google/mapping', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']);


        $data['url_mapping_verify'] = html_entity_decode(url::link('extension/advertise/google/mapping_verify', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
        $data['url_category_autocomplete'] = html_entity_decode(url::link('extension/advertise/google/category_autocomplete', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');

        $data['header']       = load::controller('common/header');
        $data['column_left']  = load::controller('common/column_left');
        $data['footer']       = load::controller('common/footer');

        $data['current_step'] = 5;
        $data['steps'] = load::view('extension/advertise/google_steps', $data);

        response::setOutput(load::view('extension/advertise/google_mapping', $data));
    }

    public static function mapping_verify() {
        load::language('extension/advertise/google');

        
        $data = array();

        $json = array(
            'submit_directly' => !admin_extension_advertise_google::isAnyProductCategoryModified(self::$store_id),
            'modal_confirmation' => load::view('extension/advertise/google_mapping_verify', $data)
        );

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function campaign_test() {
        $json = array(
            'status' => false,
            'redirect' => null,
            'error' => null
        );

        if (self::validatePermission()) {
            try {
                $json['status'] = googleshopping::testCampaigns();
            } catch (ConnectionException $e) {
                session::$data['error'] = $e::getMessage();

                $json['redirect'] = html_entity_decode(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
            } catch (\RuntimeException $e) {
                $json['status'] = false;
                $json['error'] = $e::getMessage();
            }

            self::applyNewSettings(array(
                'advertise_google_can_edit_campaigns' => (int)$json['status']
            ));
        } else {
            $json['error'] = self::$error['warning'];
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function campaign() {
        load::language('extension/advertise/google');

        document::setTitle(language::get('heading_campaign'));

        document::addStyle('view/stylesheet/googleshopping/stepper.css');


        if (request::$server['REQUEST_METHOD'] == 'POST' && self::validateCampaign()) {
            self::applyNewSettings(request::$post);

            // If there is no redirect from the push of targets, go back to the extension dashboard
            response::redirect(url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
        }

        $data = array();

        $data['error'] = '';

        if (isset(session::$data['error'])) {
            $data['error'] = session::$data['error'];
            unset(session::$data['error']);
        } else if (!empty(self::$error['warning'])) {
            $data['error'] = self::$error['warning'];
        }

        $data['success'] = '';

        if (isset(session::$data['success'])) {
            $data['success'] = session::$data['success'];
            unset(session::$data['success']);
        }

        $data['warning'] = '';

        if (!setting::get('advertise_google_status') && admin_extension_advertise_google::hasActiveTarget(self::$store_id)) {
            $data['warning'] = language::get('warning_paused_targets');
        }

        $data['from_dashboard'] = isset(request::$get['from_dashboard']);

        $data['breadcrumbs']   = array();
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_home'),
            'href' => url::link('common/dashboard'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_extensions'),
            'href' => url::link('marketplace/extension', '&type=advertise'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('heading_title'),
            'href' => url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']),
        );

        if ($data['from_dashboard']) {
            $data['breadcrumbs'][] = array(
                'text' => language::get('heading_campaign'),
                'href' => url::link('extension/advertise/google/campaign', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token'] . '&from_dashboard=true'),
            );
        }

        if (isset(request::$post['advertise_google_auto_advertise'])) {
            $data['advertise_google_auto_advertise'] = request::$post['advertise_google_auto_advertise'];
        } else if (setting::has('advertise_google_auto_advertise')) {
            $data['advertise_google_auto_advertise'] = setting::get('advertise_google_auto_advertise');
        } else {
            $data['advertise_google_auto_advertise'] = '0';
        }

        if ($data['from_dashboard']) {
            $data['cancel']       = url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']);
        } else {
            $data['cancel']       = url::link('marketplace/extension', '&type=advertise');
        }

        $data['action'] = url::link('extension/advertise/google/campaign', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']);
        $data['target_add'] = html_entity_decode(url::link('extension/advertise/google/target_add', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
        $data['target_edit'] = html_entity_decode(url::link('extension/advertise/google/target_edit', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token'] . '&advertise_google_target_id={target_id}'), ENT_QUOTES, 'UTF-8');
        $data['target_delete'] = html_entity_decode(url::link('extension/advertise/google/target_delete', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token'] . '&advertise_google_target_id={target_id}'), ENT_QUOTES, 'UTF-8');
        $data['target_list'] = html_entity_decode(url::link('extension/advertise/google/target_list', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
        $data['url_campaign_test'] = html_entity_decode(url::link('extension/advertise/google/campaign_test', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
        $data['can_edit_campaigns'] = (bool)setting::get('advertise_google_can_edit_campaigns');
        $data['text_roas_warning'] = sprintf(language::get('warning_roas'), date(language::get('date_format_long'), time() + Googleshopping::$ROAS_WAIT_INTERVAL));

        $data['json_allowed_targets'] = json_encode(admin_extension_advertise_google::getAllowedTargets());

        $targets = googleshopping::getTargets(self::$store_id);

        foreach ($targets as &$target) {
            if (!$target['roas_status']) {
                $target['roas_warning'] = sprintf(language::get('warning_roas'), date(language::get('date_format_long'), $target['roas_available_on']));
            } else {
                $target['roas_warning'] = null;
            }
        }

        $data['targets'] = $targets;
        $data['json_targets'] = json_encode($targets);

        $data['header']       = load::controller('common/header');
        $data['column_left']  = load::controller('common/column_left');
        $data['footer']       = load::controller('common/footer');

        $data['current_step'] = 3;
        $data['steps'] = load::view('extension/advertise/google_steps', $data);

        response::setOutput(load::view('extension/advertise/google_campaign', $data));
    }

    public static function target_add() {
        load::language('extension/advertise/google');

        $json = array(
            'success' => null,
            'redirect' => null,
            'error' => null
        );

        if (self::validatePermission()) {
            if (request::$server['REQUEST_METHOD'] == 'POST' && self::validateTarget()) {

                $target = array(
                    'store_id' => self::$store_id,
                    'campaign_name' => str_replace(',', '&#44;', trim(request::$post['campaign_name'])),
                    'country' => request::$post['country'],
                    'status' => request::$post['status'] == 'active' ? 'active' : 'paused',
                    'budget' => (float)preg_replace('~[^0-9\.]~i', '', request::$post['budget']),
                    'roas' => isset(request::$post['roas']) ? (int)request::$post['roas'] : 0,
                    'feeds' => array_values(request::$post['feed'])
                );

                admin_extension_advertise_google::addTarget($target, self::$store_id);

                try {
                    googleshopping::pushTargets();

                    $json['success'] = language::get('success_target_add');
                } catch (ConnectionException $e) {
                    session::$data['error'] = $e::getMessage();

                    $json['redirect'] = html_entity_decode(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
                } catch (\RuntimeException $e) {
                    $json['error'] = $e::getMessage();
                }
            } else {
                $json['error'] = self::$error['warning'];

                if (isset(self::$error['campaign_name'])) {
                    $json['error_campaign_name'] = self::$error['campaign_name'];
                }

                if (isset(self::$error['country'])) {
                    $json['error_country'] = self::$error['country'];
                }

                if (isset(self::$error['budget'])) {
                    $json['error_budget'] = self::$error['budget'];
                }

                if (isset(self::$error['feed'])) {
                    $json['error_feed'] = self::$error['feed'];
                }
            }
        } else {
            $json['error'] = self::$error['warning'];
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function target_edit() {
        load::language('extension/advertise/google');

        $json = array(
            'success' => null,
            'redirect' => null,
            'error' => null
        );

        if (self::validatePermission()) {
            if (request::$server['REQUEST_METHOD'] == 'POST' && self::validateTarget()) {

                $target = array(
                    'campaign_name' => str_replace(',', '&#44;', trim(request::$post['campaign_name'])),
                    'country' => request::$post['country'],
                    'status' => request::$post['status'] == 'active' ? 'active' : 'paused',
                    'budget' => (float)preg_replace('~[^0-9\.]~i', '', request::$post['budget']),
                    'roas' => isset(request::$post['roas']) ? (int)request::$post['roas'] : 0,
                    'feeds' => array_values(request::$post['feed'])
                );

                googleshopping::editTarget((int)request::$get['advertise_google_target_id'], $target);

                try {
                    googleshopping::pushTargets();

                    $json['success'] = language::get('success_target_edit');
                } catch (ConnectionException $e) {
                    session::$data['error'] = $e::getMessage();

                    $json['redirect'] = html_entity_decode(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
                } catch (\RuntimeException $e) {
                    $json['error'] = $e::getMessage();
                }
            } else {
                $json['error'] = self::$error['warning'];

                if (isset(self::$error['campaign_name'])) {
                    $json['error_campaign_name'] = self::$error['campaign_name'];
                }

                if (isset(self::$error['country'])) {
                    $json['error_country'] = self::$error['country'];
                }

                if (isset(self::$error['budget'])) {
                    $json['error_budget'] = self::$error['budget'];
                }

                if (isset(self::$error['feed'])) {
                    $json['error_feed'] = self::$error['feed'];
                }
            }
        } else {
            $json['error'] = self::$error['warning'];
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function target_delete() {
        load::language('extension/advertise/google');

        $json = array(
            'success' => null,
            'redirect' => null,
            'error' => null
        );

        if (self::validatePermission()) {

            $advertise_google_target_id = (int)request::$get['advertise_google_target_id'];

            $target_info = googleshopping::getTarget($advertise_google_target_id);

            if (!empty($target_info)) {
                try {
                    googleshopping::deleteCampaign($target_info['campaign_name']);

                    googleshopping::deleteTarget($advertise_google_target_id);

                    $json['success'] = language::get('success_target_delete');
                } catch (ConnectionException $e) {
                    session::$data['error'] = $e::getMessage();

                    $json['redirect'] = html_entity_decode(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
                } catch (\RuntimeException $e) {
                    $json['error'] = $e::getMessage();
                }
            }
        } else {
            $json['error'] = self::$error['warning'];
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function target_list() {
        load::language('extension/advertise/google');

        $json = array(
            'targets' => null,
            'error' => null
        );


        $targets = googleshopping::getTargets(self::$store_id);

        foreach ($targets as &$target) {
            if (!$target['roas_status']) {
                $target['roas_warning'] = sprintf(language::get('warning_roas'), date(language::get('date_format_long'), $target['roas_available_on']));
            } else {
                $target['roas_warning'] = null;
            }
        }

        $json['targets'] = $targets;

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function callback_merchant() {
        $state_verified =
            !empty(session::$data['advertise_google']['state']) &&
            !empty(request::$get['state']) && 
            request::$get['state'] == session::$data['advertise_google']['state'];

        $error = isset(request::$get['error']) ? request::$get['error'] : null;
        $merchant_id = isset(request::$get['merchant_id']) ? request::$get['merchant_id'] : null;

        if ($state_verified && is_null($error)) {
            load::language('extension/advertise/google');

            try {
                googleshopping::verifySite();

                $user_info = admin_user_user::getUser(user::getId());

                self::applyNewSettings(array(
                    'advertise_google_gmc_account_selected' => true,
                    'advertise_google_gmc_account_id' => $merchant_id,
                    'advertise_google_gmc_account_accepted_by' => array(
                        'user_id' => $user_info['user_id'],
                        'user_group_id' => $user_info['user_group_id'],
                        'user_group' => $user_info['user_group'],
                        'username' => $user_info['username'],
                        'firstname' => $user_info['firstname'],
                        'lastname' => $user_info['lastname'],
                        'email' => $user_info['email'],
                        'ip' => $user_info['ip']
                    ),
                    'advertise_google_gmc_account_accepted_at' => time(),
                    'advertise_google_conversion_tracker' => googleshopping::getConversionTracker(),
                    'advertise_google_can_edit_campaigns' => '0'
                ));

                if (session::$data['advertise_google']['account_type'] == 'api') {
                    session::$data['success'] = sprintf(language::get('success_merchant_access'), $merchant_id);
                } else {
                    session::$data['success'] = language::get('success_merchant');
                }

                if (count(googleshopping::getTargets(self::$store_id)) > 0) {
                    response::redirect(url::link('extension/advertise/google/campaign', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
                }
            } catch (ConnectionException $e) {
                session::$data['error'] = $e::getMessage();

                unset(session::$data['advertise_google']);

                response::redirect(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } catch (\RuntimeException $e) {
                session::$data['error'] = $e::getMessage();
            }
        } else if (!is_null($error)) {
            session::$data['error'] = $error;

            $setting = admin_setting_setting::getSetting('advertise_google', self::$store_id);

            unset($setting['advertise_google_status']);
            unset($setting['advertise_google_work']);
            unset($setting['advertise_google_gmc_account_selected']);
            unset($setting['advertise_google_gmc_shipping_taxes_configured']);
            unset($setting['advertise_google_can_edit_campaigns']);

            admin_setting_setting::editSetting('advertise_google', $setting, self::$store_id);
        }

        unset(session::$data['advertise_google']);

        response::redirect(url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
    }

    public static function callback_connect() {
        $state_verified =
            !empty(session::$data['advertise_google']['state']) &&
            !empty(request::$get['state']) && 
            request::$get['state'] == session::$data['advertise_google']['state'];

        if ($state_verified) {
            load::language('extension/advertise/google');


            try {
                $access = googleshopping::access(session::$data['advertise_google'], urldecode(request::$get['code']));

                self::applyNewSettings(array(
                    'advertise_google_app_id' => session::$data['advertise_google']['app_id'],
                    'advertise_google_app_secret' => session::$data['advertise_google']['app_secret'],
                    'advertise_google_status' => session::$data['advertise_google']['status'],
                    'advertise_google_cron_token' => session::$data['advertise_google']['cron_token'],
                    'advertise_google_cron_acknowledge' => session::$data['advertise_google']['cron_acknowledge'],
                    'advertise_google_cron_email' => session::$data['advertise_google']['cron_email'],
                    'advertise_google_cron_email_status' => session::$data['advertise_google']['cron_email_status'],
                    'advertise_google_access_token' => $access['access_token'],
                    'advertise_google_refresh_token' => $access['refresh_token']
                ));

                session::$data['success'] = language::get('success_connect');

                if (count(googleshopping::getTargets(self::$store_id)) > 0 && setting::get('advertise_google_gmc_account_selected')) {
                    response::redirect(url::link('extension/advertise/google/campaign', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
                }
            } catch (ConnectionException $e) {
                session::$data['error'] = $e::getMessage();

                response::redirect(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } catch (\RuntimeException $e) {
                session::$data['error'] = $e::getMessage();
            }
        } else if (isset(request::$get['error'])) {
            session::$data['error'] = request::$get['error'];
        }

        unset(session::$data['advertise_google']);

        if (setting::get('advertise_google_gmc_account_selected')) {
            response::redirect(url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
        } else {
            response::redirect(url::link('extension/advertise/google/merchant', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
        }
    }

    public static function connect() {
        load::language('extension/advertise/google');

        document::setTitle(language::get('heading_title'));

        document::addStyle('view/stylesheet/googleshopping/stepper.css');


        if (request::$server['REQUEST_METHOD'] == 'POST' && self::validateSettings() && self::validateConnect()) {
            unset(session::$data['advertise_google']);

            session::$data['advertise_google']['app_id'] = request::$post['advertise_google_app_id'];
            session::$data['advertise_google']['app_secret'] = request::$post['advertise_google_app_secret'];
            session::$data['advertise_google']['status'] = request::$post['advertise_google_status'];
            session::$data['advertise_google']['cron_email_status'] = request::$post['advertise_google_cron_email_status'];
            session::$data['advertise_google']['cron_email'] = request::$post['advertise_google_cron_email'];
            session::$data['advertise_google']['cron_token'] = request::$post['advertise_google_cron_token'];
            session::$data['advertise_google']['cron_acknowledge'] = isset(request::$post['advertise_google_cron_acknowledge']);
            session::$data['advertise_google']['redirect_uri'] = html_entity_decode(url::link('extension/advertise/google/callback_connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']), ENT_QUOTES, 'UTF-8');
            session::$data['advertise_google']['state'] = md5(microtime(true) . json_encode(session::$data['advertise_google']) . microtime(true));

            $url = googleshopping::authorize(session::$data['advertise_google']);

            response::redirect($url);
        }

        $data = array();

        $data['error'] = '';

        if (isset(session::$data['error'])) {
            if (empty(session::$data['success']) && self::getSettingValue('advertise_google_app_id', false) && self::getSettingValue('advertise_google_app_secret', false)) {
                $data['error'] = session::$data['error'];
            }
            unset(session::$data['error']);
        } else if (!empty(self::$error['warning'])) {
            $data['error'] = self::$error['warning'];
        }

        $data['error_cron_email']                   = self::getValidationError('cron_email');
        $data['error_cron_acknowledge']             = self::getValidationError('cron_acknowledge');

        if (isset(self::$error['app_id'])) {
            $data['error_app_id'] = self::$error['app_id'];
        } else {
            $data['error_app_id'] = '';
        }

        if (isset(self::$error['app_secret'])) {
            $data['error_app_secret'] = self::$error['app_secret'];
        } else {
            $data['error_app_secret'] = '';
        }

        $data['success'] = '';

        if (isset(session::$data['success'])) {
            $data['success'] = session::$data['success'];
            unset(session::$data['success']);
        }

        $data['breadcrumbs']   = array();
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_home'),
            'href' => url::link('common/dashboard'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_extensions'),
            'href' => url::link('marketplace/extension', '&type=advertise'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('heading_title'),
            'href' => url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']),
        );

        $data['advertise_google_status']                = self::getSettingValue('advertise_google_status', 1);
        $data['advertise_google_app_id']                = self::getSettingValue('advertise_google_app_id', '');
        $data['advertise_google_app_secret']            = self::getSettingValue('advertise_google_app_secret', '');
        $data['advertise_google_cron_email_status']     = self::getSettingValue('advertise_google_cron_email_status');
        $data['advertise_google_cron_email']            = self::getSettingValue('advertise_google_cron_email', config('email'));
        $data['advertise_google_cron_token']            = self::getSettingValue('advertise_google_cron_token');
        $data['advertise_google_cron_acknowledge']      = self::getSettingValue('advertise_google_cron_acknowledge', null, true);

        $server = googleshopping::getStoreUrl();

        $data['advertise_google_cron_command'] = 'export CUSTOM_SERVER_NAME=' . parse_url($server, PHP_URL_HOST) . '; export CUSTOM_SERVER_PORT=443; export ADVERTISE_GOOGLE_CRON=1; export ADVERTISE_GOOGLE_STORE_ID=' . self::$store_id . '; ' . PHP_BINDIR . '/php -d session.save_path=' . session_save_path() . ' -d memory_limit=256M ' . 'system/library/googleshopping/cron.php > /dev/null 2> /dev/null';
        
        if (!setting::get('advertise_google_cron_token')) {
            $data['advertise_google_cron_token'] = md5(mt_rand());
        }

        $host_and_uri = parse_url($server, PHP_URL_HOST) . dirname(parse_url($server, PHP_URL_PATH));

        $data['advertise_google_cron_url'] = 'https://' . rtrim($host_and_uri, '/') . '/index.php?route=extension/advertise/google/cron&cron_token={CRON_TOKEN}';

        $data['header']       = load::controller('common/header');
        $data['column_left']  = load::controller('common/column_left');
        $data['footer']       = load::controller('common/footer');

        $data['cancel']       = url::link('marketplace/extension', '&type=advertise');
        $data['action']       = url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']);

        $data['text_connect_intro'] = sprintf(language::get('text_connect_intro'), Googleshopping::$API_URL);

        $data['current_step'] = 1;
        $data['steps'] = load::view('extension/advertise/google_steps', $data);

        response::setOutput(load::view('extension/advertise/google_connect', $data));
    }

    public static function disconnect() {
        load::language('extension/advertise/google');

        if (self::validatePermission()) {
            try {

                googleshopping::disconnect();

                foreach (googleshopping::getTargets(self::$store_id) as $target) {
                    googleshopping::deleteTarget($target['target_id']);
                }

                $setting = admin_setting_setting::getSetting('advertise_google', self::$store_id);

                unset($setting['advertise_google_status']);
                unset($setting['advertise_google_work']);
                unset($setting['advertise_google_access_token']);
                unset($setting['advertise_google_refresh_token']);
                unset($setting['advertise_google_gmc_account_selected']);
                unset($setting['advertise_google_gmc_shipping_taxes_configured']);
                unset($setting['advertise_google_can_edit_campaigns']);

                admin_setting_setting::editSetting('advertise_google', $setting, self::$store_id);

                session::$data['success'] = language::get('success_disconnect');
            } catch (ConnectionException $e) {
                session::$data['error'] = $e::getMessage();

                response::redirect(url::link('extension/advertise/google/connect', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
            } catch (\RuntimeException $e) {
                session::$data['error'] = $e::getMessage();
            }
        } else {
            session::$data['error'] = self::$error['warning'];
        }

        response::redirect(url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
    }

    public static function checklist() {
        load::language('extension/advertise/google');

        document::setTitle(language::get('heading_title'));

        if (request::$server['REQUEST_METHOD'] == 'POST' && self::validatePermission()) {

            admin_setting_setting::editSetting('advertise_google', request::$post, self::$store_id);

            response::redirect(url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']));
        }

        $data = array();

        $data['error'] = '';

        if (isset(session::$data['error'])) {
            $data['error'] = session::$data['error'];
            unset(session::$data['error']);
        } else if (!empty(self::$error['warning'])) {
            $data['error'] = self::$error['warning'];
        }

        $data['breadcrumbs']   = array();
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_home'),
            'href' => url::link('common/dashboard'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('text_extensions'),
            'href' => url::link('marketplace/extension', '&type=advertise'),
        );
        $data['breadcrumbs'][] = array(
            'text' => language::get('heading_title'),
            'href' => url::link('extension/advertise/google', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']),
        );

        $data['text_panel_heading'] = sprintf(language::get('text_panel_heading'), googleshopping::getStoreName());

        $data['cancel']       = url::link('marketplace/extension', '&type=advertise');
        $data['action']       = url::link('extension/advertise/google/checklist', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token']);
        
        $data['header']       = load::controller('common/header');
        $data['column_left']  = load::controller('common/column_left');
        $data['footer']       = load::controller('common/footer');

        response::setOutput(load::view('extension/advertise/google_checklist', $data));
    }

    public static function popup_product() {
        $json = array(
            'body' => '',
            'title' => '',
            'success' => false,
            'required_fields' => [],
            'success_message' => ''
        );

        language::load('extension/advertise/google');


        $operand_info = NULL;
        $form_data = NULL;
        $filter_data = NULL;
        $product_ids = array();

        if (request::$post['operand']['type'] == 'single') {
            $product_advertise_google_id = request::$post['operand']['data'];

            $product_info = admin_extension_advertise_google::getProductByProductAdvertiseGoogleId($product_advertise_google_id);

            if ($product_info !== NULL) {
                $json['product_id'] = $product_info['product_id'];
                
                // Required variables:
                $operand_info = array(
                    'title' => sprintf(language::get('text_popup_title_single'), $product_info['name'], $product_info['model'])
                );

                $required_fields = admin_extension_advertise_google::getRequiredFieldsByProductIds(array($product_info['product_id']), self::$store_id);

                if (request::$post['action'] == 'submit') {
                    $form_data = array_merge(request::$post['form'], array(
                        'product_id' => $product_info['product_id']
                    ));
                }

                $options = admin_extension_advertise_google::getProductOptionsByProductIds(array($product_info['product_id']));

                $default_form_data = admin_extension_advertise_google::getProductAdvertiseGoogle($product_advertise_google_id);
            }
        } else if (request::$post['operand']['type'] == 'multiple') {
            if (!empty(request::$post['operand']['data']['all_pages'])) {
                $filter_data = self::getFilter(request::$post['operand']['data']['filter']);

                $total_products = googleshopping::getTotalProducts($filter_data, self::$store_id);

                // Required variables:
                $operand_info = array(
                    'title' => sprintf(language::get('text_popup_title_multiple'), $total_products)
                );

                $required_fields = admin_extension_advertise_google::getRequiredFieldsByFilter($filter_data, self::$store_id);

                if (request::$post['action'] == 'submit') {
                    $form_data = request::$post['form'];
                }

                $options = admin_extension_advertise_google::getProductOptionsByFilter($filter_data);
            } else {
                $product_ids = request::$post['operand']['data']['select'];

                $total_products = count($product_ids);

                // Required variables:
                $operand_info = array(
                    'title' => sprintf(language::get('text_popup_title_multiple'), $total_products)
                );

                $required_fields = admin_extension_advertise_google::getRequiredFieldsByProductIds($product_ids, self::$store_id);

                if (request::$post['action'] == 'submit') {
                    $form_data = request::$post['form'];
                }

                $options = admin_extension_advertise_google::getProductOptionsByProductIds($product_ids);
            }

            $default_form_data = array(
                'google_product_category' => '',
                'condition' => '',
                'adult' => '',
                'multipack' => '',
                'is_bundle' => '',
                'age_group' => '',
                'color' => '',
                'gender' => '',
                'size_type' => '',
                'size_system' => '',
                'size' => ''
            );
        }

        if ($operand_info !== NULL) {
            $json['title'] = $operand_info['title'];
            $json['success_message'] = language::get('success_product');

            load::config('googleshopping/googleshopping');

            $json['required_fields'] = $required_fields;

            if (request::$post['action'] == 'submit' && self::validateProduct($required_fields)) {
                $form_data['store_id'] = (int)self::$store_id;

                if (request::$post['operand']['type'] == 'single') {
                    admin_extension_advertise_google::updateSingleProductFields($form_data);
                } else if (request::$post['operand']['type'] == 'multiple') {
                    if (!empty(request::$post['operand']['data']['all_pages'])) {
                        admin_extension_advertise_google::updateMultipleProductFields($filter_data, $form_data);
                    } else {
                        foreach ($product_ids as $product_id) {
                            $form_data['product_id'] = (int)$product_id;
                            admin_extension_advertise_google::updateSingleProductFields($form_data);
                        }
                    }
                }

                $json['success'] = true;
            }

            $data['error'] = '';

            if (!empty(self::$error['warning'])) {
                $data['error'] = self::$error['warning'];
            }

            if (isset(self::$error['color'])) {
                $data['error_color'] = self::$error['color'];
            } else {
                $data['error_color'] = '';
            }

            if (isset(self::$error['size_system'])) {
                $data['error_size_system'] = self::$error['size_system'];
            } else {
                $data['error_size_system'] = '';
            }

            if (isset(self::$error['size_type'])) {
                $data['error_size_type'] = self::$error['size_type'];
            } else {
                $data['error_size_type'] = '';
            }

            if (isset(self::$error['size'])) {
                $data['error_size'] = self::$error['size'];
            } else {
                $data['error_size'] = '';
            }

            if (isset(self::$error['product_category'])) {
                $data['error_product_category'] = self::$error['product_category'];
            } else {
                $data['error_product_category'] = '';
            }

            if (isset(self::$error['condition'])) {
                $data['error_condition'] = self::$error['condition'];
            } else {
                $data['error_condition'] = '';
            }

            if (isset(self::$error['age_group'])) {
                $data['error_age_group'] = self::$error['age_group'];
            } else {
                $data['error_age_group'] = '';
            }

            if (isset(self::$error['gender'])) {
                $data['error_gender'] = self::$error['gender'];
            } else {
                $data['error_gender'] = '';
            }

            if (isset(self::$error['adult'])) {
                $data['error_adult'] = self::$error['adult'];
            } else {
                $data['error_adult'] = '';
            }

            if (isset(self::$error['multipack'])) {
                $data['error_multipack'] = self::$error['multipack'];
            } else {
                $data['error_multipack'] = '';
            }

            if (isset(self::$error['is_bundle'])) {
                $data['error_is_bundle'] = self::$error['is_bundle'];
            } else {
                $data['error_is_bundle'] = '';
            }

            $data['google_product_categories'] = config::get('advertise_google_google_product_categories');
            $data['conditions'] = array(
                'new' => language::get('text_condition_new'),
                'refurbished' => language::get('text_condition_refurbished'),
                'used' => language::get('text_condition_used')
            );
            $data['age_groups'] = array(
                '' => language::get('text_does_not_apply'),
                'newborn' => language::get('text_age_group_newborn'),
                'infant' => language::get('text_age_group_infant'),
                'toddler' => language::get('text_age_group_toddler'),
                'kids' => language::get('text_age_group_kids'),
                'adult' => language::get('text_age_group_adult')
            );
            $data['genders'] = array(
                'unisex' => language::get('text_gender_unisex'),
                'female' => language::get('text_gender_female'),
                'male' => language::get('text_gender_male')
            );
            $data['size_systems'] = array(
                '' => language::get('text_does_not_apply')
            );
            foreach (config::get('advertise_google_size_systems') as $system) {
                $data['size_systems'][$system] = $system;
            }

            $data['size_types'] = array(
                '' => language::get('text_does_not_apply'),
                'regular' => language::get('text_size_type_regular'),
                'petite' => language::get('text_size_type_petite'),
                'plus' => language::get('text_size_type_plus'),
                'big and tall' => language::get('text_size_type_big_and_tall'),
                'maternity' => language::get('text_size_type_maternity')
            );

            $data['options'] = array(
                '' => language::get('text_does_not_apply')
            );

            foreach ($options as $option) {
                $data['options'][$option['option_id']] = $option['name'];
            }

            $data['required_fields'] = json_encode($required_fields);

            if (request::$post['action'] == 'submit') {
                $form_data = request::$post['form'];
            } else {
                $form_data = $default_form_data;
            }

            $data['google_product_category'] = $form_data['google_product_category'];
            $data['condition'] = $form_data['condition'];
            $data['adult'] = $form_data['adult'];
            $data['multipack'] = $form_data['multipack'];
            $data['is_bundle'] = $form_data['is_bundle'];
            $data['age_group'] = $form_data['age_group'];
            $data['color'] = $form_data['color'];
            $data['gender'] = $form_data['gender'];
            $data['size_type'] = $form_data['size_type'];
            $data['size_system'] = $form_data['size_system'];
            $data['size'] = $form_data['size'];

            $json['body'] = load::view('extension/advertise/google_popup_product', $data);
        } else {
            $json['title'] = language::get('error_popup_not_found_title');
            $json['body'] = language::get('error_popup_not_found_body');
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function popup_issues() {
        $json = array(
            'body' => '',
            'title' => ''
        );

        language::load('extension/advertise/google');


        $product_id = isset(request::$get['product_id']) ? (int)request::$get['product_id'] : 0;

        $product_issues = admin_extension_advertise_google::getProductIssues($product_id, self::$store_id);

        if ($product_issues !== NULL) {
            $json['title'] = sprintf(language::get('text_popup_title_single'), $product_issues['name'], $product_issues['model']);

            $data['product_issues'] = $product_issues['entries'];

            $json['body'] = load::view('extension/advertise/google_popup_issues', $data);
        } else {
            $json['title'] = language::get('error_popup_not_found_title');
            $json['body'] = language::get('error_popup_not_found_body');
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function admin_link(&$route, &$data, &$template) {
        if (!user::hasPermission('access', 'extension/advertise/google')) {
            return;
        }

        foreach ($data['menus'] as &$menu) {
            if ($menu['id'] == 'menu-marketing') {
                $children = array();


                $children[] = array(
                    'name' => config('name'),
                    'children' => array(),
                    'href' => url::link('extension/advertise/google', 'store_id=0&user_token=' . session::$data['user_token'])
                );

                foreach (admin_setting_store::getStores() as $store) {
                    $children[] = array(
                        'name' => $store['name'],
                        'children' => array(),
                        'href' => url::link('extension/advertise/google', 'store_id=' . $store['store_id'] . '&user_token=' . session::$data['user_token'])
                    );
                }

                array_push($menu['children'], array(
                    'name' => 'Google Shopping',
                    'children' => $children,
                    'href' => ''
                ));

                return;
            }
        }
    }

    public static function addProduct(&$route, &$args, &$output) {

        foreach (admin_catalog_product::getProductStores($output) as $store_id) {
            admin_extension_advertise_google::insertNewProducts(array($output), $store_id);
        }
    }

    public static function copyProduct(&$route, &$args, &$output) {

        $final_product_id = admin_extension_advertise_google::getFinalProductId();

        if (!empty($final_product_id)) {
            foreach (admin_catalog_product::getProductStores($final_product_id) as $store_id) {
                admin_extension_advertise_google::insertNewProducts(array($final_product_id), $store_id);
            }
        }
    }

    public static function deleteProduct(&$route, &$args, &$output) {

        admin_extension_advertise_google::deleteProducts(array((int)$args[0]));
    }

    public static function install() {

        admin_extension_advertise_google::createTables();
        admin_extension_advertise_google::createEvents();
    }

    public static function uninstall() {

        admin_extension_advertise_google::dropTables();
        admin_extension_advertise_google::deleteEvents();
    }

    public static function category_autocomplete() {
        $json = array();

        if (isset(request::$get['filter_name'])) {

            $filter_data = array(
                'filter_name' => request::$get['filter_name'],
                'sort'        => 'name',
                'order'       => 'ASC',
                'start'       => 0,
                'limit'       => 5
            );

            $results = admin_extension_advertise_google::getCategories($filter_data, self::$store_id);

            foreach ($results as $result) {
                $json[] = array(
                    'category_id' => $result['category_id'],
                    'name'        => strip_tags(html_entity_decode($result['name'], ENT_QUOTES, 'UTF-8'))
                );
            }
        }

        $sort_order = array();

        foreach ($json as $key => $value) {
            $sort_order[$key] = $value['name'];
        }

        array_multisort($sort_order, SORT_ASC, $json);

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    protected static function getFilter($array) {
        if (!empty($array)) {
            return array(
                'filter_product_name' => $array['product_name'],
                'filter_product_model' => $array['product_model'],
                'filter_category_id' => $array['category_id'],
                'filter_is_modified' => $array['is_modified'],
                'filter_store_id' => self::$store_id
            );
        }

        return array(
            'filter_store_id' => self::$store_id
        );
    }

    protected static function applyNewSettings($new_settings) {

        $old_settings = admin_setting_setting::getSetting('advertise_google', self::$store_id);

        $new_settings = array_merge($old_settings, $new_settings);

        admin_setting_setting::editSetting('advertise_google', $new_settings, self::$store_id);

        foreach ($new_settings as $key => $value) {
            setting::set($key, $value);
        }
    }

    protected static function product(&$row) {        
        load::config('googleshopping/googleshopping');


        if (!empty($row['image']) && file_exists('image/' . $row['image'])) {
            $image = admin_tool_image::resize($row['image'], 50, 50);
        } else {
            $image = admin_tool_image::resize('no_image.png', 50, 50);
        }

        return array(
            'product_advertise_google_id' => (int)$row['product_advertise_google_id'],
            'product_id' => (int)$row['product_id'],
            'image' => $image,
            'name' => htmlentities(html_entity_decode($row['name'], ENT_QUOTES, 'UTF-8'), ENT_QUOTES, 'UTF-8'),
            'model' => $row['model'],
            'impressions' => (int)$row['impressions'],
            'clicks' => (int)$row['clicks'],
            'conversions' => (int)$row['conversions'],
            'cost' => googleshopping::currencyFormat($row['cost']),
            'conversion_value' => googleshopping::currencyFormat($row['conversion_value']),
            'destination_status' => $row['destination_status'],
            'is_modified' => (bool)$row['is_modified'],
            'has_issues' => (bool)$row['has_issues'],
            'url_issues' => html_entity_decode(url::link('extension/advertise/google/popup_issues', 'store_id=' . self::$store_id . '&user_token=' . session::$data['user_token'] . '&product_id=' . $row['product_id']), ENT_QUOTES, 'UTF-8'),
            'campaigns' => admin_extension_advertise_google::getProductCampaigns((int)$row['product_id'], self::$store_id)
        );
    }

    protected static function getSettingValue($key, $default = null, $checkbox = false) {
        if ($checkbox) {
            if (request::$server['REQUEST_METHOD'] == 'POST' && !isset(request::$post[$key])) {
                return $default;
            } else {
                return setting::get($key);
            }
        }

        if (isset(request::$post[$key])) {
            return request::$post[$key]; 
        } else if (setting::has($key)) {
            return setting::get($key);
        } else {
            return $default;
        }
    }

    protected static function getValidationError($key) {
        if (isset(self::$error[$key])) {
            return self::$error[$key];
        } else {
            return '';
        }
    }

    protected static function validateSettings() {
        self::validatePermission();

        if (empty(request::$post['advertise_google_status'])) {
            return true;
        }

        if (!empty(request::$post['advertise_google_cron_email_status'])) {
            if (!filter_var(request::$post['advertise_google_cron_email'], FILTER_VALIDATE_EMAIL)) {
                self::$error['cron_email'] = language::get('error_invalid_email');
            }
        }

        if (empty(request::$post['advertise_google_cron_acknowledge'])) {
            self::$error['cron_acknowledge'] = language::get('error_cron_acknowledge');
        }

        if (self::$error && empty(self::$error['warning'])) {
            self::$error['warning'] = language::get('error_form');
        }

        return !self::$error;
    }

    protected static function validateShippingAndTaxes() {
        self::validatePermission();
        
        if (empty(request::$post['advertise_google_shipping_taxes']['min_transit_time']) || !is_numeric(request::$post['advertise_google_shipping_taxes']['min_transit_time']) || (int)request::$post['advertise_google_shipping_taxes']['min_transit_time'] < 0) {
            self::$error['min_transit_time'] = language::get('error_min_transit_time');
        } else if (empty(request::$post['advertise_google_shipping_taxes']['max_transit_time']) || !is_numeric(request::$post['advertise_google_shipping_taxes']['max_transit_time']) || (int)request::$post['advertise_google_shipping_taxes']['max_transit_time'] < (int)request::$post['advertise_google_shipping_taxes']['min_transit_time']) {
            self::$error['max_transit_time'] = language::get('error_max_transit_time');
        }

        switch (request::$post['advertise_google_shipping_taxes']['shipping_type']) {
            case 'flat' : 
                if (!isset(request::$post['advertise_google_shipping_taxes']['flat_rate']) || !is_numeric(request::$post['advertise_google_shipping_taxes']['flat_rate']) || (float)request::$post['advertise_google_shipping_taxes']['flat_rate'] <= 0) {
                    self::$error['flat_rate'] = language::get('error_flat_rate');
                }
            break;
            case 'carrier' :
                if (empty(request::$post['advertise_google_shipping_taxes']['carrier'])) {
                    self::$error['warning'] = language::get('error_carrier');
                }

                if (empty(request::$post['advertise_google_shipping_taxes']['carrier_postcode'])) {
                    self::$error['carrier_postcode'] = language::get('error_carrier_postcode');
                }

                if (!isset(request::$post['advertise_google_shipping_taxes']['carrier_price_percentage']) || !is_numeric(request::$post['advertise_google_shipping_taxes']['carrier_price_percentage']) || (float)request::$post['advertise_google_shipping_taxes']['carrier_price_percentage'] < 0 || (float)request::$post['advertise_google_shipping_taxes']['carrier_price_percentage'] > 100) {
                    self::$error['carrier_price_percentage'] = language::get('error_carrier_price_percentage');
                }
            break;
        }

        switch (request::$post['advertise_google_shipping_taxes']['tax_type']) {
            case 'usa' :
                if (empty(request::$post['advertise_google_shipping_taxes']['tax'])) {
                    self::$error['warning'] = language::get('error_tax');
                }
            break;
        }

        if (!isset(self::$error['warning']) && self::$error) {
            self::$error['warning'] = language::get('error_warning');
        }

        return !self::$error;
    }

    protected static function validateMapping() {
        self::validatePermission();
        
        if (!isset(self::$error['warning']) && self::$error) {
            self::$error['warning'] = language::get('error_warning');
        }

        return !self::$error;
    }

    protected static function validateProduct($required_fields) {
        if (!user::hasPermission('modify', 'extension/advertise/google')) {
            self::$error['warning'] = language::get('error_permission');
        }

        if (empty(self::$error)) {
            foreach ($required_fields as $key => $requirements) {
                if (empty($requirements['selected_field']) && (!isset(request::$post['form'][$key]) || request::$post['form'][$key] == '')) {
                    self::$error[$key] = language::get('error_field_no_value');
                } else if (!empty($requirements['selected_field'])) {
                    foreach ($requirements['selected_field'] as $dependency => $values) {
                        if (in_array(request::$post['form'][$dependency], $values) && (!isset(request::$post['form'][$key]) || request::$post['form'][$key] == '')) {
                            self::$error[$key] = language::get('error_field_no_value');
                        }
                    }
                }
            }
        }

        if (!isset(self::$error['warning']) && self::$error) {
            self::$error['warning'] = language::get('error_warning');
        }

        return !self::$error;
    }

    protected static function validatePermission() {
        if (!user::hasPermission('modify', 'extension/advertise/google')) {
            self::$error['warning'] = language::get('error_permission');
        }

        return !self::$error;
    }

    protected static function validateCampaign() {
        self::validatePermission();


        $targets = googleshopping::getTargets(self::$store_id);

        if (empty($targets)) {
            self::$error['warning'] = language::get('error_no_targets');
        }

        if (!isset(self::$error['warning']) && self::$error) {
            self::$error['warning'] = language::get('error_warning');
        }

        return !self::$error;
    }

    protected static function validateConnect() {
        self::validatePermission();

        if (!isset(request::$post['advertise_google_app_id']) || trim(request::$post['advertise_google_app_id']) == '') {
            self::$error['app_id'] = language::get('error_empty_app_id');
        } else if (admin_extension_advertise_google::isAppIdUsed(request::$post['advertise_google_app_id'], self::$store_id)) {
            self::$error['app_id'] = language::get('error_used_app_id');
        }

        if (!isset(request::$post['advertise_google_app_secret']) || trim(request::$post['advertise_google_app_secret']) == '') {
            self::$error['app_secret'] = language::get('error_empty_app_secret');
        }

        if (!isset(self::$error['warning']) && self::$error) {
            self::$error['warning'] = language::get('error_warning');
        }

        return !self::$error;
    }

    protected static function validateTarget() {
        self::validatePermission();

        if (!isset(request::$post['budget']) || !is_numeric(request::$post['budget']) || (float)request::$post['budget'] < 5) {
            self::$error['budget'] = language::get('error_budget');
        }

        if (empty(request::$post['feed']) || !is_array(request::$post['feed'])) {
            self::$error['feed'] = language::get('error_empty_feed');
        } else {
            foreach (request::$post['feed'] as $feed) {
                if (empty($feed['language']) || empty($feed['currency'])) {
                    self::$error['feed'] = language::get('error_invalid_feed');
                    break;
                }
            }
        }

        if (empty(request::$post['country'])) {
            self::$error['country'] = language::get('error_empty_country');
        }

        if (empty(request::$post['campaign_name']) || trim(request::$post['campaign_name']) == '') {
            self::$error['campaign_name'] = language::get('error_empty_campaign_name');
        } else {
            $disallowed_names = [];


            foreach (googleshopping::getTargets(self::$store_id) as $existing_target) {
                if (isset(request::$get['advertise_google_target_id']) && $existing_target['target_id'] == request::$get['advertise_google_target_id']) {
                    // Ignore this target as it is currntly being edited
                    continue;
                }

                $disallowed_names[] = strtolower(str_replace('&#44;', ',', trim($existing_target['campaign_name'])));
            }

            if (in_array(trim(strtolower(request::$post['campaign_name'])), $disallowed_names)) {
                self::$error['campaign_name'] = language::get('error_campaign_name_in_use');
            }

            if (strtolower(trim(request::$post['campaign_name'])) == 'total') {
                self::$error['campaign_name'] = language::get('error_campaign_name_total');
            }
        }

        if (!isset(self::$error['warning']) && self::$error) {
            self::$error['warning'] = language::get('error_warning');
        }

        return !self::$error;
    }
}