<?php
class ControllerExtensionDashboardActivity {
	private static $error = array();

	public static function index() {
		load::language('extension/dashboard/activity');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('dashboard_activity', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=dashboard'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = ''; 
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/dashboard/activity')
		);

		$data['action'] = url::link('extension/dashboard/activity');

		$data['cancel'] = url::link('marketplace/extension', '&type=dashboard');

		if (isset(request::$post['dashboard_activity_width'])) {
			$data['dashboard_activity_width'] = request::$post['dashboard_activity_width'];
		} else {
			$data['dashboard_activity_width'] = config::get('dashboard_activity_width');
		}
		
		$data['columns'] = array();
		
		for ($i = 3; $i <= 12; $i++) {
			$data['columns'][] = $i;
		}
		
		if (isset(request::$post['dashboard_activity_status'])) {
			$data['dashboard_activity_status'] = request::$post['dashboard_activity_status'];
		} else {
			$data['dashboard_activity_status'] = config::get('dashboard_activity_status');
		}

		if (isset(request::$post['dashboard_activity_sort_order'])) {
			$data['dashboard_activity_sort_order'] = request::$post['dashboard_activity_sort_order'];
		} else {
			$data['dashboard_activity_sort_order'] = config::get('dashboard_activity_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/dashboard/activity_form', $data));
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/dashboard/activity')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
	
	public static function dashboard() {
		load::language('extension/dashboard/activity');


		$data['activities'] = array();


		$results = admin_extension_dashboard_activity::getActivities();

		foreach ($results as $result) {
			$comment = vsprintf(language::get('text_activity_' . $result['key']), json_decode($result['data'], true));

			$find = array(
				'customer_id=',
				'order_id=',
				'return_id='
			);

			$replace = array(
				url::link('customer/customer/edit', '&customer_id='),
				url::link('sale/order/info', '&order_id='),
				url::link('sale/return/edit', '&return_id=')
			);

			$data['activities'][] = array(
				'comment'    => str_replace($find, $replace, $comment),
				'date_added' => date(language::get('datetime_format'), strtotime($result['date_added']))
			);
		}

		return load::view('extension/dashboard/activity_info', $data);
	}
}