<?php
class ControllerExtensionDashboardChart {
	private static $error = array();

	public static function index() {
		load::language('extension/dashboard/chart');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('dashboard_chart', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=dashboard'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/dashboard/chart')
		);

		$data['action'] = url::link('extension/dashboard/chart');

		$data['cancel'] = url::link('marketplace/extension', '&type=dashboard');

		if (isset(request::$post['dashboard_chart_width'])) {
			$data['dashboard_chart_width'] = request::$post['dashboard_chart_width'];
		} else {
			$data['dashboard_chart_width'] = config::get('dashboard_chart_width');
		}
	
		$data['columns'] = array();
		
		for ($i = 3; $i <= 12; $i++) {
			$data['columns'][] = $i;
		}
				
		if (isset(request::$post['dashboard_chart_status'])) {
			$data['dashboard_chart_status'] = request::$post['dashboard_chart_status'];
		} else {
			$data['dashboard_chart_status'] = config::get('dashboard_chart_status');
		}

		if (isset(request::$post['dashboard_chart_sort_order'])) {
			$data['dashboard_chart_sort_order'] = request::$post['dashboard_chart_sort_order'];
		} else {
			$data['dashboard_chart_sort_order'] = config::get('dashboard_chart_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/dashboard/chart_form', $data));
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/dashboard/chart')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}	
	
	public static function dashboard() {
		load::language('extension/dashboard/chart');


		return load::view('extension/dashboard/chart_info');
	}

	public static function chart() {
		load::language('extension/dashboard/chart');

		$json = array();


		$json['order'] = array();
		$json['customer'] = array();
		$json['xaxis'] = array();

		$json['order']['label'] = language::get('text_order');
		$json['customer']['label'] = language::get('text_customer');
		$json['order']['data'] = array();
		$json['customer']['data'] = array();

		if (isset(request::$get['range'])) {
			$range = request::$get['range'];
		} else {
			$range = 'day';
		}

		switch ($range) {
			default:
			case 'day':
				$results = admin_extension_dashboard_chart::getTotalOrdersByDay();

				foreach ($results as $key => $value) {
					$json['order']['data'][] = array($key, $value['total']);
				}

				$results = admin_extension_dashboard_chart::getTotalCustomersByDay();

				foreach ($results as $key => $value) {
					$json['customer']['data'][] = array($key, $value['total']);
				}

				for ($i = 0; $i < 24; $i++) {
					$json['xaxis'][] = array($i, $i);
				}
				break;
			case 'week':
				$results = admin_extension_dashboard_chart::getTotalOrdersByWeek();

				foreach ($results as $key => $value) {
					$json['order']['data'][] = array($key, $value['total']);
				}

				$results = admin_extension_dashboard_chart::getTotalCustomersByWeek();

				foreach ($results as $key => $value) {
					$json['customer']['data'][] = array($key, $value['total']);
				}

				$date_start = strtotime('-' . date('w') . ' days');

				for ($i = 0; $i < 7; $i++) {
					$date = date('Y-m-d', $date_start + ($i * 86400));

					$json['xaxis'][] = array(date('w', strtotime($date)), date('D', strtotime($date)));
				}
				break;
			case 'month':
				$results = admin_extension_dashboard_chart::getTotalOrdersByMonth();

				foreach ($results as $key => $value) {
					$json['order']['data'][] = array($key, $value['total']);
				}

				$results = admin_extension_dashboard_chart::getTotalCustomersByMonth();

				foreach ($results as $key => $value) {
					$json['customer']['data'][] = array($key, $value['total']);
				}

				for ($i = 1; $i <= date('t'); $i++) {
					$date = date('Y') . '-' . date('m') . '-' . $i;

					$json['xaxis'][] = array(date('j', strtotime($date)), date('d', strtotime($date)));
				}
				break;
			case 'year':
				$results = admin_extension_dashboard_chart::getTotalOrdersByYear();

				foreach ($results as $key => $value) {
					$json['order']['data'][] = array($key, $value['total']);
				}

				$results = admin_extension_dashboard_chart::getTotalCustomersByYear();

				foreach ($results as $key => $value) {
					$json['customer']['data'][] = array($key, $value['total']);
				}

				for ($i = 1; $i <= 12; $i++) {
					$json['xaxis'][] = array($i, date('M', mktime(0, 0, 0, $i)));
				}
				break;
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}