<?php
class ControllerExtensionDashboardCustomer {
	private static $error = array();

	public static function index() {
		load::language('extension/dashboard/customer');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('dashboard_customer', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=dashboard'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/dashboard/customer')
		);

		$data['action'] = url::link('extension/dashboard/customer');

		$data['cancel'] = url::link('marketplace/extension', '&type=dashboard');

		if (isset(request::$post['dashboard_customer_width'])) {
			$data['dashboard_customer_width'] = request::$post['dashboard_customer_width'];
		} else {
			$data['dashboard_customer_width'] = config::get('dashboard_customer_width');
		}

		$data['columns'] = array();
		
		for ($i = 3; $i <= 12; $i++) {
			$data['columns'][] = $i;
		}
				
		if (isset(request::$post['dashboard_customer_status'])) {
			$data['dashboard_customer_status'] = request::$post['dashboard_customer_status'];
		} else {
			$data['dashboard_customer_status'] = config::get('dashboard_customer_status');
		}

		if (isset(request::$post['dashboard_customer_sort_order'])) {
			$data['dashboard_customer_sort_order'] = request::$post['dashboard_customer_sort_order'];
		} else {
			$data['dashboard_customer_sort_order'] = config::get('dashboard_customer_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/dashboard/customer_form', $data));
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/dashboard/customer')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
		
	public static function dashboard() {
		load::language('extension/dashboard/customer');


		// Total Orders

		$today = admin_customer_customer::getTotalCustomers(array('filter_date_added' => date('Y-m-d', strtotime('-1 day'))));

		$yesterday = admin_customer_customer::getTotalCustomers(array('filter_date_added' => date('Y-m-d', strtotime('-2 day'))));

		$difference = $today - $yesterday;

		if ($difference && $today) {
			$data['percentage'] = round(($difference / $today) * 100);
		} else {
			$data['percentage'] = 0;
		}

		$customer_total = admin_customer_customer::getTotalCustomers();

		if ($customer_total > 1000000000000) {
			$data['total'] = round($customer_total / 1000000000000, 1) . 'T';
		} elseif ($customer_total > 1000000000) {
			$data['total'] = round($customer_total / 1000000000, 1) . 'B';
		} elseif ($customer_total > 1000000) {
			$data['total'] = round($customer_total / 1000000, 1) . 'M';
		} elseif ($customer_total > 1000) {
			$data['total'] = round($customer_total / 1000, 1) . 'K';
		} else {
			$data['total'] = $customer_total;
		}

		$data['customer'] = url::link('customer/customer');

		return load::view('extension/dashboard/customer_info', $data);
	}
}
