<?php
class ControllerExtensionDashboardMap {
	private static $error = array();

	public static function index() {
		load::language('extension/dashboard/map');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('dashboard_map', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=dashboard'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/dashboard/map')
		);

		$data['action'] = url::link('extension/dashboard/map');

		$data['cancel'] = url::link('marketplace/extension', '&type=dashboard');

		if (isset(request::$post['dashboard_map_width'])) {
			$data['dashboard_map_width'] = request::$post['dashboard_map_width'];
		} else {
			$data['dashboard_map_width'] = config::get('dashboard_map_width');
		}

		$data['columns'] = array();
		
		for ($i = 3; $i <= 12; $i++) {
			$data['columns'][] = $i;
		}
				
		if (isset(request::$post['dashboard_map_status'])) {
			$data['dashboard_map_status'] = request::$post['dashboard_map_status'];
		} else {
			$data['dashboard_map_status'] = config::get('dashboard_map_status');
		}

		if (isset(request::$post['dashboard_map_sort_order'])) {
			$data['dashboard_map_sort_order'] = request::$post['dashboard_map_sort_order'];
		} else {
			$data['dashboard_map_sort_order'] = config::get('dashboard_map_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/dashboard/map_form', $data));
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/dashboard/map')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
		
	public static function dashboard() {
		load::language('extension/dashboard/map');

		
		return load::view('extension/dashboard/map_info');
	}

	public static function map() {
		$json = array();


		$results = admin_extension_dashboard_map::getTotalOrdersByCountry();

		foreach ($results as $result) {
			$json[strtolower($result['iso_code_2'])] = array(
				'total'  => $result['total'],
				'amount' => currency::format($result['amount'], config('currency'))
			);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
