<?php
class ControllerExtensionDashboardRecent {
	private static $error = array();

	public static function index() {
		load::language('extension/dashboard/recent');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('dashboard_recent', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=dashboard'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/dashboard/recent')
		);

		$data['action'] = url::link('extension/dashboard/recent');

		$data['cancel'] = url::link('marketplace/extension', '&type=dashboard');

		if (isset(request::$post['dashboard_recent_width'])) {
			$data['dashboard_recent_width'] = request::$post['dashboard_recent_width'];
		} else {
			$data['dashboard_recent_width'] = config::get('dashboard_recent_width');
		}

		$data['columns'] = array();
		
		for ($i = 3; $i <= 12; $i++) {
			$data['columns'][] = $i;
		}
				
		if (isset(request::$post['dashboard_recent_status'])) {
			$data['dashboard_recent_status'] = request::$post['dashboard_recent_status'];
		} else {
			$data['dashboard_recent_status'] = config::get('dashboard_recent_status');
		}

		if (isset(request::$post['dashboard_recent_sort_order'])) {
			$data['dashboard_recent_sort_order'] = request::$post['dashboard_recent_sort_order'];
		} else {
			$data['dashboard_recent_sort_order'] = config::get('dashboard_recent_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/dashboard/recent_form', $data));
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/dashboard/recent')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
	
	public static function dashboard() {
		load::language('extension/dashboard/recent');


		// Last 5 Orders
		$data['orders'] = array();

		$filter_data = array(
			'sort'  => 'o.date_added',
			'order' => 'DESC',
			'start' => 0,
			'limit' => 5
		);

		
		$results = admin_sale_order::getOrders($filter_data);

		foreach ($results as $result) {
			$data['orders'][] = array(
				'order_id'   => $result['order_id'],
				'customer'   => $result['customer'],
				'status'     => $result['order_status'],
				'date_added' => date(language::get('date_format_short'), strtotime($result['date_added'])),
				'total'      => currency::format($result['total'], $result['currency_code'], $result['currency_value']),
				'view'       => url::link('sale/order/info', '&order_id=' . $result['order_id']),
			);
		}

		return load::view('extension/dashboard/recent_info', $data);
	}
}
