<?php
class ControllerExtensionDashboardSale {
	private static $error = array();

	public static function index() {
		load::language('extension/dashboard/sale');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('dashboard_sale', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=dashboard'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/dashboard/sale')
		);

		$data['action'] = url::link('extension/dashboard/sale');

		$data['cancel'] = url::link('marketplace/extension', '&type=dashboard');

		if (isset(request::$post['dashboard_sale_width'])) {
			$data['dashboard_sale_width'] = request::$post['dashboard_sale_width'];
		} else {
			$data['dashboard_sale_width'] = config::get('dashboard_sale_width');
		}
	
		$data['columns'] = array();
		
		for ($i = 3; $i <= 12; $i++) {
			$data['columns'][] = $i;
		}
				
		if (isset(request::$post['dashboard_sale_status'])) {
			$data['dashboard_sale_status'] = request::$post['dashboard_sale_status'];
		} else {
			$data['dashboard_sale_status'] = config::get('dashboard_sale_status');
		}

		if (isset(request::$post['dashboard_sale_sort_order'])) {
			$data['dashboard_sale_sort_order'] = request::$post['dashboard_sale_sort_order'];
		} else {
			$data['dashboard_sale_sort_order'] = config::get('dashboard_sale_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/dashboard/sale_form', $data));
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/dashboard/sale')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
	
	public static function dashboard() {
		load::language('extension/dashboard/sale');



		$today = admin_extension_dashboard_sale::getTotalSales(array('filter_date_added' => date('Y-m-d', strtotime('-1 day'))));

		$yesterday = admin_extension_dashboard_sale::getTotalSales(array('filter_date_added' => date('Y-m-d', strtotime('-2 day'))));

		$difference = $today - $yesterday;

		if ($difference && (int)$today) {
			$data['percentage'] = round(($difference / $today) * 100);
		} else {
			$data['percentage'] = 0;
		}

		$sale_total = admin_extension_dashboard_sale::getTotalSales();

		if ($sale_total > 1000000000000) {
			$data['total'] = round($sale_total / 1000000000000, 1) . 'T';
		} elseif ($sale_total > 1000000000) {
			$data['total'] = round($sale_total / 1000000000, 1) . 'B';
		} elseif ($sale_total > 1000000) {
			$data['total'] = round($sale_total / 1000000, 1) . 'M';
		} elseif ($sale_total > 1000) {
			$data['total'] = round($sale_total / 1000, 1) . 'K';
		} else {
			$data['total'] = round($sale_total);
		}

		$data['sale'] = url::link('sale/order');

		return load::view('extension/dashboard/sale_info', $data);
	}
}
