<?php
class ControllerExtensionExtensionModule {
	private static $error = array();

	public static function index() {
		load::language('extension/extension/module');



		self::getList();
	}

	public static function install() {
		load::language('extension/extension/module');



		if (self::validate()) {
			admin_setting_extension::install('module', request::$get['extension']);


			admin_user_user_group::addPermission(user::getGroupId(), 'access', 'extension/module/' . request::$get['extension']);
			admin_user_user_group::addPermission(user::getGroupId(), 'modify', 'extension/module/' . request::$get['extension']);

			// Call install method if it exsits
			load::controller('extension/module/' . request::$get['extension'] . '/install');

			session::$data['success'] = language::get('text_success');
		} else {
			session::$data['error'] = self::$error['warning'];
		}
	
		self::getList();
	}

	public static function uninstall() {
		load::language('extension/extension/module');



		if (self::validate()) {
			admin_setting_extension::uninstall('module', request::$get['extension']);

			admin_setting_module::deleteModulesByCode(request::$get['extension']);

			// Call uninstall method if it exsits
			load::controller('extension/module/' . request::$get['extension'] . '/uninstall');

			session::$data['success'] = language::get('text_success');
		}

		self::getList();
	}
	
	public static function add() {
		load::language('extension/extension/module');



		if (self::validate()) {
			load::language('module' . '/' . request::$get['extension']);
			
			admin_setting_module::addModule(request::$get['extension'], language::get('heading_title'));

			session::$data['success'] = language::get('text_success');
		}

		self::getList();
	}

	public static function delete() {
		load::language('extension/extension/module');



		if (isset(request::$get['module_id']) && self::validate()) {
			admin_setting_module::deleteModule(request::$get['module_id']);

			session::$data['success'] = language::get('text_success');
		}
		
		self::getList();
	}

	protected static function getList() {
		$data['text_layout'] = sprintf(language::get('text_layout'), url::link('design/layout'));

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		$extensions = admin_setting_extension::getInstalled('module');

		foreach ($extensions as $key => $value) {
			if (!is_file(DIR_APPLICATION . 'controller/extension/module/' . $value . '.php') && !is_file(DIR_APPLICATION . 'controller/module/' . $value . '.php')) {
				admin_setting_extension::uninstall('module', $value);

				unset($extensions[$key]);
				
				admin_setting_module::deleteModulesByCode($value);
			}
		}

		$data['extensions'] = array();

		// Create a new language container so we don't pollute the current one
		$language = new Language(config('language'));
		
		// Compatibility code for old extension folders
		$files = glob(DIR_APPLICATION . 'controller/extension/module/*.php');

		if ($files) {
			foreach ($files as $file) {
				$extension = basename($file, '.php');

				load::language('extension/module/' . $extension, 'extension');

				$module_data = array();

				$modules = admin_setting_module::getModulesByCode($extension);

				foreach ($modules as $module) {
					if ($module['setting']) {
						$setting_info = json_decode($module['setting'], true);
					} else {
						$setting_info = array();
					}
					
					$module_data[] = array(
						'module_id' => $module['module_id'],
						'name'      => $module['name'],
						'status'    => (isset($setting_info['status']) && $setting_info['status']) ? language::get('text_enabled') : language::get('text_disabled'),
						'edit'      => url::link('extension/module/' . $extension, '&module_id=' . $module['module_id']),
						'delete'    => url::link('extension/extension/module/delete', '&module_id=' . $module['module_id'])
					);
				}

				$data['extensions'][] = array(
					'name'      => language::get('extension')::get('heading_title'),
					'status'    => config::get('module_' . $extension . '_status') ? language::get('text_enabled') : language::get('text_disabled'),
					'module'    => $module_data,
					'install'   => url::link('extension/extension/module/install', '&extension=' . $extension),
					'uninstall' => url::link('extension/extension/module/uninstall', '&extension=' . $extension),
					'installed' => in_array($extension, $extensions),
					'edit'      => url::link('extension/module/' . $extension)
				);
			}
		}

		$sort_order = array();

		foreach ($data['extensions'] as $key => $value) {
			$sort_order[$key] = $value['name'];
		}

		array_multisort($sort_order, SORT_ASC, $data['extensions']);

		$data['promotion'] = load::controller('extension/extension/promotion');

		response::setOutput(load::view('extension/extension/module', $data));
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/extension/module')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if(!isset(self::$error['warning']) && isset(request::$get['extension']) && ((utf8_strlen(request::$get['extension']) < 3) || (utf8_strlen(request::$get['extension']) > 32))) {
			self::$error['warning'] = language::get('error_code_name');
		}

		return !self::$error;
	}
}
