<?php
class ControllerExtensionExtensionTheme {
	private static $error = array();

	public static function index() {
		load::language('extension/extension/theme');


		self::getList();
	}

	public static function install() {
		load::language('extension/extension/theme');


		if (self::validate()) {
			admin_setting_extension::install('theme', request::$get['extension']);


			admin_user_user_group::addPermission(user::getGroupId(), 'access', 'extension/theme/' . request::$get['extension']);
			admin_user_user_group::addPermission(user::getGroupId(), 'modify', 'extension/theme/' . request::$get['extension']);

			// Call install method if it exsits
			load::controller('extension/theme/' . request::$get['extension'] . '/install');

			session::$data['success'] = language::get('text_success');
		}

		self::getList();
	}

	public static function uninstall() {
		load::language('extension/extension/theme');


		if (self::validate()) {
			admin_setting_extension::uninstall('theme', request::$get['extension']);

			// Call uninstall method if it exsits
			load::controller('extension/theme/' . request::$get['extension'] . '/uninstall');

			session::$data['success'] = language::get('text_success');
		}
		
		self::getList();
	}

	protected static function getList() {
		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		$extensions = admin_setting_extension::getInstalled('theme');

		foreach ($extensions as $key => $value) {
			if (!is_file(DIR_APPLICATION . 'controller/extension/theme/' . $value . '.php') && !is_file(DIR_APPLICATION . 'controller/theme/' . $value . '.php')) {
				admin_setting_extension::uninstall('theme', $value);

				unset($extensions[$key]);
			}
		}


		$stores = admin_setting_store::getStores();

		$data['extensions'] = array();
		
		// Compatibility code for old extension folders
		$files = glob(DIR_APPLICATION . 'controller/extension/theme/*.php');

		if ($files) {
			foreach ($files as $file) {
				$extension = basename($file, '.php');
				
				load::language('extension/theme/' . $extension, 'extension');
					
				$store_data = array();
				
				$store_data[] = array(
					'name'   => config('name'),
					'edit'   => url::link('extension/theme/' . $extension, '&store_id=0'),
					'status' => config::get('theme_' . $extension . '_status') ? language::get('text_enabled') : language::get('text_disabled')
				);
									
				foreach ($stores as $store) {
					$store_data[] = array(
						'name'   => $store['name'],
						'edit'   => url::link('extension/theme/' . $extension, '&store_id=' . $store['store_id']),
						'status' => admin_setting_setting::getSettingValue('theme_' . $extension . '_status', $store['store_id']) ? language::get('text_enabled') : language::get('text_disabled')
					);
				}
				
				$data['extensions'][] = array(
					'name'      => language::get('extension')::get('heading_title'),
					'install'   => url::link('extension/extension/theme/install', '&extension=' . $extension),
					'uninstall' => url::link('extension/extension/theme/uninstall', '&extension=' . $extension),
					'installed' => in_array($extension, $extensions),
					'store'     => $store_data
				);
			}
		}

		$data['promotion'] = load::controller('extension/extension/promotion');

		response::setOutput(load::view('extension/extension/theme', $data));
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/extension/theme')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
}