<?php
class ControllerExtensionFeedGoogleBase {
	private static $error = array();

	public static function index() {
		load::language('extension/feed/google_base');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('feed_google_base', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=feed'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=feed')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/feed/google_base')
		);

		$data['action'] = url::link('extension/feed/google_base');

		$data['cancel'] = url::link('marketplace/extension', '&type=feed');


		$data['data_feed'] = HTTP_CATALOG . 'index.php?route=extension/feed/google_base';

		if (isset(request::$post['feed_google_base_status'])) {
			$data['feed_google_base_status'] = request::$post['feed_google_base_status'];
		} else {
			$data['feed_google_base_status'] = config::get('feed_google_base_status');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/feed/google_base', $data));
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/feed/google_base')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}

	public static function install() {

		admin_extension_feed_google_base::install();
	}

	public static function uninstall() {

		admin_extension_feed_google_base::uninstall();
	}

	public static function import() {
		load::language('extension/feed/google_base');

		$json = array();

		// Check user has permission
		if (!user::hasPermission('modify', 'extension/feed/google_base')) {
			$json['error'] = language::get('error_permission');
		}

		if (!$json) {
			if (!empty(request::$files['file']['name']) && is_file(request::$files['file']['tmp_name'])) {
				// Sanitize the filename
				$filename = basename(html_entity_decode(request::$files['file']['name'], ENT_QUOTES, 'UTF-8'));

				// Allowed file extension types
				if (utf8_strtolower(utf8_substr(strrchr($filename, '.'), 1)) != 'txt') {
					$json['error'] = language::get('error_filetype');
				}

				// Allowed file mime types
				if (request::$files['file']['type'] != 'text/plain') {
					$json['error'] = language::get('error_filetype');
				}

				// Return any upload error
				if (request::$files['file']['error'] != UPLOAD_ERR_OK) {
					$json['error'] = language::get('error_upload_' . request::$files['file']['error']);
				}
			} else {
				$json['error'] = language::get('error_upload');
			}
		}

		if (!$json) {
			$json['success'] = language::get('text_success');


			// Get the contents of the uploaded file
			$content = file_get_contents(request::$files['file']['tmp_name']);

			admin_extension_feed_google_base::import($content);

			unlink(request::$files['file']['tmp_name']);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function category() {
		load::language('extension/feed/google_base');

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$data['google_base_categories'] = array();


		$results = admin_extension_feed_google_base::getCategories(($page - 1) * 10, 10);

		foreach ($results as $result) {
			$data['google_base_categories'][] = array(
				'google_base_category_id' => $result['google_base_category_id'],
				'google_base_category'    => $result['google_base_category'],
				'category_id'             => $result['category_id'],
				'category'                => $result['category']
			);
		}

		$category_total = admin_extension_feed_google_base::getTotalCategories();

		$pagination = new Pagination();
		$pagination::$total = $category_total;
		$pagination::$page = $page;
		$pagination::$limit = 10;
		$pagination::$url = url::link('extension/feed/google_base/category', '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($category_total) ? (($page - 1) * 10) + 1 : 0, ((($page - 1) * 10) > ($category_total - 10)) ? $category_total : ((($page - 1) * 10) + 10), $category_total, ceil($category_total / 10));

		response::setOutput(load::view('extension/feed/google_base_category', $data));
	}

	public static function addCategory() {
		load::language('extension/feed/google_base');

		$json = array();

		if (!user::hasPermission('modify', 'extension/feed/google_base')) {
			$json['error'] = language::get('error_permission');
		} elseif (!empty(request::$post['google_base_category_id']) && !empty(request::$post['category_id'])) {

			admin_extension_feed_google_base::addCategory(request::$post);

			$json['success'] = language::get('text_success');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function removeCategory() {
		load::language('extension/feed/google_base');

		$json = array();

		if (!user::hasPermission('modify', 'extension/feed/google_base')) {
			$json['error'] = language::get('error_permission');
		} else {

			admin_extension_feed_google_base::deleteCategory(request::$post['category_id']);

			$json['success'] = language::get('text_success');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function autocomplete() {
		$json = array();

		if (isset(request::$get['filter_name'])) {

			if (isset(request::$get['filter_name'])) {
				$filter_name = request::$get['filter_name'];
			} else {
				$filter_name = '';
			}

			$filter_data = array(
				'filter_name' => html_entity_decode($filter_name, ENT_QUOTES, 'UTF-8'),
				'start'       => 0,
				'limit'       => 5
			);

			$results = admin_extension_feed_google_base::getGoogleBaseCategories($filter_data);

			foreach ($results as $result) {
				$json[] = array(
					'google_base_category_id' => $result['google_base_category_id'],
					'name'                    => strip_tags(html_entity_decode($result['name'], ENT_QUOTES, 'UTF-8'))
				);
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
