<?php
class ControllerExtensionFraudIp {
	private static $error = array();

	public static function index() {
		load::language('extension/fraud/ip');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('fraud_ip', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=fraud'));
		}


		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=fraud')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/fraud/ip')
		);

		$data['action'] = url::link('extension/fraud/ip');

		$data['cancel'] = url::link('marketplace/extension', '&type=fraud');

		if (isset(request::$post['fraud_ip_order_status_id'])) {
			$data['fraud_ip_order_status_id'] = request::$post['fraud_ip_order_status_id'];
		} else {
			$data['fraud_ip_order_status_id'] = config::get('fraud_ip_order_status_id');
		}


		$data['order_statuses'] = admin_localisation_order_status::getOrderStatuses();

		if (isset(request::$post['fraud_ip_status'])) {
			$data['fraud_ip_status'] = request::$post['fraud_ip_status'];
		} else {
			$data['fraud_ip_status'] = config::get('fraud_ip_status');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/fraud/ip', $data));
	}

	public static function install() {

		admin_extension_fraud_ip::install();
	}

	public static function uninstall() {

		admin_extension_fraud_ip::uninstall();
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/fraud/ip')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}

    public static function ip() {
		load::language('extension/fraud/ip');


		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$data['ips'] = array();

		$results = admin_extension_fraud_ip::getIps(($page - 1) * 10, 10);

		foreach ($results as $result) {
			$data['ips'][] = array(
				'ip'         => $result['ip'],
				'total'      => admin_customer_customer::getTotalCustomersByIp($result['ip']),
				'date_added' => date('d/m/y', strtotime($result['date_added'])),
				'filter_ip'  => url::link('customer/customer', '&filter_ip=' . $result['ip'])
			);
		}

		$ip_total = admin_extension_fraud_ip::getTotalIps();

		$pagination = new Pagination();
		$pagination::$total = $ip_total;
		$pagination::$page = $page;
		$pagination::$limit = 10;
		$pagination::$url = url::link('extension/fraud/ip/ip', '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($ip_total) ? (($page - 1) * 10) + 1 : 0, ((($page - 1) * 10) > ($ip_total - 10)) ? $ip_total : ((($page - 1) * 10) + 10), $ip_total, ceil($ip_total / 10));

		response::setOutput(load::view('extension/fraud/ip_ip', $data));
	}

	public static function addIp() {
		load::language('extension/fraud/ip');

		$json = array();

		if (!user::hasPermission('modify', 'extension/fraud/ip')) {
			$json['error'] = language::get('error_permission');
		} else {

			if (!admin_extension_fraud_ip::getTotalIpsByIp(request::$post['ip'])) {
				admin_extension_fraud_ip::addIp(request::$post['ip']);
			}

			$json['success'] = language::get('text_success');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function removeIp() {
		load::language('extension/fraud/ip');

		$json = array();

		if (!user::hasPermission('modify', 'extension/fraud/ip')) {
			$json['error'] = language::get('error_permission');
		} else {

			admin_extension_fraud_ip::removeIp(request::$post['ip']);

			$json['success'] = language::get('text_success');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
