<?php
class ControllerExtensionPaymentCardinity {
	private static $error = array();

	public static function index() {
		load::language('extension/payment/cardinity');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('payment_cardinity', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=payment'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['key'])) {
			$data['error_key'] = self::$error['key'];
		} else {
			$data['error_key'] = '';
		}

		if (isset(self::$error['secret'])) {
			$data['error_secret'] = self::$error['secret'];
		} else {
			$data['error_secret'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=payment')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/payment/cardinity')
		);

		$data['action'] = url::link('extension/payment/cardinity');

		$data['cancel'] = url::link('marketplace/extension', '&type=payment');

		if (isset(request::$post['payment_cardinity_key'])) {
			$data['payment_cardinity_key'] = request::$post['payment_cardinity_key'];
		} else {
			$data['payment_cardinity_key'] = config::get('payment_cardinity_key');
		}

		if (isset(request::$post['payment_cardinity_secret'])) {
			$data['payment_cardinity_secret'] = request::$post['payment_cardinity_secret'];
		} else {
			$data['payment_cardinity_secret'] = config::get('payment_cardinity_secret');
		}

		if (isset(request::$post['payment_cardinity_debug'])) {
			$data['payment_cardinity_debug'] = request::$post['payment_cardinity_debug'];
		} else {
			$data['payment_cardinity_debug'] = config::get('payment_cardinity_debug');
		}

		if (isset(request::$post['payment_cardinity_total'])) {
			$data['payment_cardinity_total'] = request::$post['payment_cardinity_total'];
		} else {
			$data['payment_cardinity_total'] = config::get('payment_cardinity_total');
		}

		if (isset(request::$post['payment_cardinity_order_status_id'])) {
			$data['payment_cardinity_order_status_id'] = request::$post['payment_cardinity_order_status_id'];
		} else {
			$data['payment_cardinity_order_status_id'] = config::get('payment_cardinity_order_status_id');
		}


		$data['order_statuses'] = admin_localisation_order_status::getOrderStatuses();

		if (isset(request::$post['payment_cardinity_geo_zone_id'])) {
			$data['payment_cardinity_geo_zone_id'] = request::$post['payment_cardinity_geo_zone_id'];
		} else {
			$data['payment_cardinity_geo_zone_id'] = config::get('payment_cardinity_geo_zone_id');
		}


		$data['geo_zones'] = admin_localisation_geo_zone::getGeoZones();

		if (isset(request::$post['payment_cardinity_status'])) {
			$data['payment_cardinity_status'] = request::$post['payment_cardinity_status'];
		} else {
			$data['payment_cardinity_status'] = config::get('payment_cardinity_status');
		}

		if (isset(request::$post['payment_cardinity_sort_order'])) {
			$data['payment_cardinity_sort_order'] = request::$post['payment_cardinity_sort_order'];
		} else {
			$data['payment_cardinity_sort_order'] = config::get('payment_cardinity_sort_order');
		}
		
		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/payment/cardinity', $data));
	}

	public static function order() {
		load::language('extension/payment/cardinity');

		$data['order_id'] = (int)request::$get['order_id'];

		return load::view('extension/payment/cardinity_order', $data);
	}

	public static function getPayment() {
		load::language('extension/payment/cardinity');


		$data['column_refund'] = language::get('column_refund');
		$data['column_date'] = language::get('column_date');
		$data['column_refund_history'] = language::get('column_refund_history');
		$data['column_action'] = language::get('column_action');
		$data['column_status'] = language::get('column_status');
		$data['column_amount'] = language::get('column_amount');
		$data['column_description'] = language::get('column_description');

		$data['button_refund'] = language::get('button_refund');


		$client = admin_extension_payment_cardinity::createClient(array(
			'key'    => config::get('payment_cardinity_key'),
			'secret' => config::get('payment_cardinity_secret')
		));

		$order = admin_extension_payment_cardinity::getOrder(request::$get['order_id']);

		$data['payment'] = false;

		$data['refunds'] = array();

		if ($order && $order['payment_id']) {
			$data['payment'] = true;

			$payment = admin_extension_payment_cardinity::getPayment($client, $order['payment_id']);

			$data['refund_action'] = false;

			$successful_statuses = array(
				'approved'
			);

			if (in_array($payment::getStatus(), $successful_statuses)) {
				$data['refund_action'] = true;
			}

			$max_refund_amount = $payment::getAmount();

			$refunds = admin_extension_payment_cardinity::getRefunds($client, $order['payment_id']);

			if ($refunds) {
				foreach ($refunds as $refund) {
					$successful_refund_statuses = array(
						'approved'
					);

					if (in_array($refund::getStatus(), $successful_refund_statuses)) {
						$max_refund_amount -= $refund::getAmount();
					}

					$data['refunds'][] = array(
						'date_added'  => date(language::get('datetime_format'), strtotime($refund::getCreated())),
						'amount'	  => currency::format($refund::getAmount(), $refund::getCurrency(), '1.00000000', true),
						'status'	  => $refund::getStatus(),
						'description' => $refund::getDescription()
					);
				}
			}

			if (!$max_refund_amount) {
				$data['refund_action'] = false;
			}

			$data['payment_id'] = $payment::getId();
			$data['symbol_left'] = currency::getSymbolLeft($payment::getCurrency());
			$data['symbol_right'] = currency::getSymbolRight($payment::getCurrency());

			$data['max_refund_amount'] = currency::format($max_refund_amount, $payment::getCurrency(), '1.00000000', false);
		}

		response::setOutput(load::view('extension/payment/cardinity_order_ajax', $data));
	}

	public static function refund() {
		load::language('extension/payment/cardinity');


		$json = array();

		$success = $error = '';

		$client = admin_extension_payment_cardinity::createClient(array(
			'key'    => config::get('payment_cardinity_key'),
			'secret' => config::get('payment_cardinity_secret')
		));

		$refund = admin_extension_payment_cardinity::refundPayment($client, request::$post['payment_id'], (float)number_format(request::$post['amount'], 2), request::$post['description']);

		if ($refund) {
			$success = language::get('text_success_action');
		} else {
			$error = language::get('text_error_generic');
		}

		$json['success'] = $success;
		$json['error'] = $error;

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	protected static function validate() {

		$check_credentials = true;

		if (version_compare(phpversion(), '7.3', '<')) {
			self::$error['warning'] = language::get('error_php_version');
		}

		if (!user::hasPermission('modify', 'extension/payment/cardinity')) {
			self::$error['warning'] = language::get('error_permission');

			$check_credentials = false;
		}

		if (!request::$post['payment_cardinity_key']) {
			self::$error['key'] = language::get('error_key');

			$check_credentials = false;
		}

		if (!request::$post['payment_cardinity_secret']) {
			self::$error['secret'] = language::get('error_secret');

			$check_credentials = false;
		}

		if (!class_exists('Cardinity\Client')) {
			self::$error['warning'] = language::get('error_composer');

			$check_credentials = false;
		}

		if ($check_credentials) {
			$client = admin_extension_payment_cardinity::createClient(array(
				'key'    => request::$post['payment_cardinity_key'],
				'secret' => request::$post['payment_cardinity_secret']
			));

			$verify_credentials = admin_extension_payment_cardinity::verifyCredentials($client);

			if (!$verify_credentials) {
				self::$error['warning'] = language::get('error_connection');
			}
		}

		if (self::$error && !isset(self::$error['warning'])) {
			self::$error['warning'] = language::get('error_warning');
		}

		return !self::$error;
	}

	public static function install() {

		admin_extension_payment_cardinity::install();
	}

	public static function uninstall() {

		admin_extension_payment_cardinity::uninstall();
	}
}