<?php
class ControllerExtensionPaymentKlarnaAccount {
	private static $error = array();
	private static $pclasses = array();

	public static function index() {
		load::language('extension/payment/klarna_account');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			$status = false;

			foreach (request::$post['payment_klarna_account_'] as $klarna_account) {
				if ($klarna_account['status']) {
					$status = true;

					break;
				}
			}

			$klarna_data = array(
				'klarna_account_pclasses' => self::$pclasses,
				'klarna_account_status'   => $status
			);

			admin_setting_setting::editSetting('payment_klarna_account', array_merge(request::$post, $klarna_data));

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=payment'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=payment')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/payment/klarna_account')
		);

		$data['action'] = url::link('extension/payment/klarna_account');

		$data['cancel'] = url::link('marketplace/extension', '&type=payment');

		$data['countries'] = array();

		$data['countries'][] = array(
			'name' => language::get('text_germany'),
			'code' => 'DEU'
		);

		$data['countries'][] = array(
			'name' => language::get('text_netherlands'),
			'code' => 'NLD'
		);

		$data['countries'][] = array(
			'name' => language::get('text_denmark'),
			'code' => 'DNK'
		);

		$data['countries'][] = array(
			'name' => language::get('text_sweden'),
			'code' => 'SWE'
		);

		$data['countries'][] = array(
			'name' => language::get('text_norway'),
			'code' => 'NOR'
		);

		$data['countries'][] = array(
			'name' => language::get('text_finland'),
			'code' => 'FIN'
		);

		if (isset(request::$post['payment_klarna_account'])) {
			$data['payment_klarna_account'] = request::$post['payment_klarna_account'];
		} else {
			$data['payment_klarna_account'] = config::get('payment_klarna_account');
		}


		$data['geo_zones'] = admin_localisation_geo_zone::getGeoZones();


		$data['order_statuses'] = admin_localisation_order_status::getOrderStatuses();

		$file = DIR_LOGS . 'klarna_account.log';

		if (file_exists($file)) {
			$data['log'] = file_get_contents($file, FILE_USE_INCLUDE_PATH, null);
		} else {
			$data['log'] = '';
		}

		$data['clear'] = url::link('extension/payment/klarna_account/clear');

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/payment/klarna_account', $data));
	}

	private static function validate() {
		if (!user::hasPermission('modify', 'extension/payment/klarna_account')) {
			self::$error['warning'] = language::get('error_permission');
		}

		$log = new Log('klarna_account.log');

		$country = array(
			'NOR' => array(
				'currency' => 1,
				'country'  => 164,
				'language' => 97,
			),
			'SWE' => array(
				'currency' => 0,
				'country'  => 209,
				'language' => 138,
			),
			'FIN' => array(
				'currency' => 2,
				'country'  => 73,
				'language' => 101,
			),
			'DNK' => array(
				'currency' => 3,
				'country'  => 59,
				'language' => 27,
			),
			'DEU' => array(
				'currency' => 2,
				'country'  => 81,
				'language' => 28,
			),
			'NLD' => array(
				'currency' => 2,
				'country'  => 154,
				'language' => 101,
			),
		);

		foreach (request::$post['klarna_account'] as $key => $klarna_account) {
			if ($klarna_account['status']) {
				$digest = base64_encode(pack("H*", hash('sha256', $klarna_account['merchant']  . ':' . $country[$key]['currency'] . ':' . $klarna_account['secret'])));

				$xml  = '<methodCall>';
				$xml .= '  <methodName>get_pclasses</methodName>';
				$xml .= '  <params>';
				$xml .= '    <param><value><string>4.1</string></value></param>';
				$xml .= '    <param><value><string>API:OPENCART:' . VERSION . '</string></value></param>';
				$xml .= '    <param><value><int>' . (int)$klarna_account['merchant'] . '</int></value></param>';
				$xml .= '    <param><value><int>' . $country[$key]['currency'] . '</int></value></param>';
				$xml .= '    <param><value><string>' . $digest . '</string></value></param>';
				$xml .= '    <param><value><int>' . $country[$key]['country'] . '</int></value></param>';
				$xml .= '    <param><value><int>' . $country[$key]['language'] . '</int></value></param>';
				$xml .= '  </params>';
				$xml .= '</methodCall>';

				if ($klarna_account['server'] == 'live') {
					$url = 'https://payment.klarna.com';
				} else {
					$url = 'https://payment.testdrive.klarna.com';
				}

				$curl = curl_init();

				$header = array();

				$header[] = 'Content-Type: text/xml';
				$header[] = 'Content-Length: ' . strlen($xml);

				curl_setopt($curl, CURLOPT_URL, $url);
				curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
				curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
				curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
				curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
				curl_setopt($curl, CURLOPT_POSTFIELDS, $xml);

				$response = curl_exec($curl);

				if ($response !== false) {
					$xml = new DOMDocument();
					$xml::loadXML($response);

					$xpath = new DOMXPath($xml);

					$nodes = $xpath::query('//methodResponse/params/param/value');

					if ($nodes::$length == 0) {
						self::$error['warning'] = language::get('error_log');

						$error_code = $xpath::query('//methodResponse/fault/value/struct/member/value/int')::item(0)::$nodeValue;
						$error_message = $xpath::query('//methodResponse/fault/value/struct/member/value/string')::item(0)::$nodeValue;

						$log::write(sprintf(language::get('error_pclass'), $key, $error_code, $error_message));

						continue;
					}

					$pclasses = self::parseResponse($nodes::item(0)::$firstChild, $xml);

					while ($pclasses) {
						$pclass = array_slice($pclasses, 0, 10);
						$pclasses = array_slice($pclasses, 10);

						$pclass[3] /= 100;
						$pclass[4] /= 100;
						$pclass[5] /= 100;
						$pclass[6] /= 100;
						$pclass[9] = ($pclass[9] != '-') ? strtotime($pclass[9]) : $pclass[9];

						array_unshift($pclass, $klarna_account['merchant']);

						self::$pclasses[$key][] = array(
							'eid'          => intval($pclass[0]),
							'id'           => intval($pclass[1]),
							'description'  => $pclass[2],
							'months'       => intval($pclass[3]),
							'startfee'     => floatval($pclass[4]),
							'invoicefee'   => floatval($pclass[5]),
							'interestrate' => floatval($pclass[6]),
							'minamount'    => floatval($pclass[7]),
							'country'      => intval($pclass[8]),
							'type'         => intval($pclass[9]),
						);
					}
				} else {
					self::$error['warning'] = language::get('error_log');

					$log::write(sprintf(language::get('error_curl'), curl_errno($curl), curl_error($curl)));
				}

				curl_close($curl);
			}
		}

		return !self::$error;
	}

	private static function parseResponse($node, $document) {
		$child = $node;

		switch ($child::$nodeName) {
			case 'string':
				$value = $child::$nodeValue;
				break;
			case 'boolean':
				$value = (string)$child::$nodeValue;

				if ($value == '0') {
					$value = false;
				} elseif ($value == '1') {
					$value = true;
				} else {
					$value = null;
				}

				break;
			case 'integer':
			case 'int':
			case 'i4':
			case 'i8':
				$value = (int)$child::$nodeValue;
				break;
			case 'array':
				$value = array();

				$xpath = new DOMXPath($document);
				$entries = $xpath::query('.//array/data/value', $child);

				for ($i = 0; $i < $entries::$length; $i++) {
					$value[] = self::parseResponse($entries::item($i)::$firstChild, $document);
				}

				break;
			default:
				$value = null;
		}

		return $value;
	}

	public static function clear() {
		load::language('extension/payment/klarna_account');

		$file = DIR_LOGS . 'klarna_account.log';

		$handle = fopen($file, 'w+');

		fclose($handle);

		session::$data['success'] = language::get('text_success');

		response::redirect(url::link('extension/payment/klarna_account'));
	}
}