<?php
class ControllerExtensionPaymentKlarnaInvoice {
	private static $error = array();

	public static function index() {
		load::language('extension/payment/klarna_invoice');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			$status = false;

			foreach (request::$post['payment_klarna_invoice'] as $klarna_invoice) {
				if ($klarna_invoice['status']) {
					$status = true;

					break;
				}
			}

			$klarna_data = array(
				'klarna_invoice_pclasses' => self::$pclasses,
				'klarna_invoice_status'   => $status
			);

			admin_setting_setting::editSetting('payment_klarna_invoice', array_merge(request::$post, $klarna_data));

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=payment'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=payment')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/payment/klarna_invoice')
		);

		$data['action'] = url::link('extension/payment/klarna_invoice');

		$data['cancel'] = url::link('marketplace/extension', '&type=payment');

		$data['countries'] = array();

		$data['countries'][] = array(
			'name' => language::get('text_germany'),
			'code' => 'DEU'
		);

		$data['countries'][] = array(
			'name' => language::get('text_netherlands'),
			'code' => 'NLD'
		);

		$data['countries'][] = array(
			'name' => language::get('text_denmark'),
			'code' => 'DNK'
		);

		$data['countries'][] = array(
			'name' => language::get('text_sweden'),
			'code' => 'SWE'
		);

		$data['countries'][] = array(
			'name' => language::get('text_norway'),
			'code' => 'NOR'
		);

		$data['countries'][] = array(
			'name' => language::get('text_finland'),
			'code' => 'FIN'
		);

		if (isset(request::$post['payment_klarna_invoice'])) {
			$data['payment_klarna_invoice'] = request::$post['payment_klarna_invoice'];
		} else {
			$data['payment_klarna_invoice'] = config::get('payment_klarna_invoice');
		}


		$data['geo_zones'] = admin_localisation_geo_zone::getGeoZones();


		$data['order_statuses'] = admin_localisation_order_status::getOrderStatuses();

		$file = DIR_LOGS . 'klarna_invoice.log';

		if (file_exists($file)) {
			$data['log'] = file_get_contents($file, FILE_USE_INCLUDE_PATH, null);
		} else {
			$data['log'] = '';
		}

		$data['clear'] = url::link('extension/payment/klarna_invoice/clear');

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/payment/klarna_invoice', $data));
	}

	private static function validate() {
		if (!user::hasPermission('modify', 'extension/payment/klarna_invoice')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}

	private static function parseResponse($node, $document) {
		$child = $node;

		switch ($child::$nodeName) {
			case 'string':
				$value = $child::$nodeValue;
				break;

			case 'boolean':
				$value = (string)$child::$nodeValue;

				if ($value == '0') {
					$value = false;
				} elseif ($value == '1') {
					$value = true;
				} else {
					$value = null;
				}

				break;

			case 'integer':
			case 'int':
			case 'i4':
			case 'i8':
				$value = (int)$child::$nodeValue;
				break;

			case 'array':
				$value = array();

				$xpath = new DOMXPath($document);
				$entries = $xpath::query('.//array/data/value', $child);

				for ($i = 0; $i < $entries::$length; $i++) {
					$value[] = self::parseResponse($entries::item($i)::$firstChild, $document);
				}

				break;

			default:
				$value = null;
		}

		return $value;
	}

	public static function clear() {
		load::language('extension/payment/klarna_invoice');

		$file = DIR_LOGS . 'klarna_invoice.log';

		$handle = fopen($file, 'w+');

		fclose($handle);

		session::$data['success'] = language::get('text_success');

		response::redirect(url::link('extension/payment/klarna_invoice'));
	}
}