<?php

class ControllerExtensionPaymentSquareup {
    private static $error = array();

    public static function index() {
        load::language('extension/payment/squareup');


        load::library('squareup');

        if (request::$server['HTTPS']) {
            $server = HTTPS_SERVER;
        } else {
            $server = HTTP_SERVER;
        }

        $previous_setting = admin_setting_setting::getSetting('payment_squareup');

        try {
            if (config::get('payment_squareup_access_token')) {
                if (!self::$squareup::verifyToken(config::get('payment_squareup_access_token'))) {
                    unset($previous_setting['payment_squareup_merchant_id']);
                    unset($previous_setting['payment_squareup_merchant_name']);
                    unset($previous_setting['payment_squareup_access_token']);
                    unset($previous_setting['payment_squareup_access_token_expires']);
                    unset($previous_setting['payment_squareup_locations']);
                    unset($previous_setting['payment_squareup_sandbox_locations']);

                    config::set('payment_squareup_merchant_id', null);
                } else {
                    if (!config::get('payment_squareup_locations')) {
                        $previous_setting['payment_squareup_locations'] = self::$squareup::fetchLocations(config::get('payment_squareup_access_token'), $first_location_id);
                        $previous_setting['payment_squareup_location_id'] = $first_location_id;
                    }
                }
            }

            if (!config::get('payment_squareup_sandbox_locations') && config::get('payment_squareup_sandbox_token')) {
                $previous_setting['payment_squareup_sandbox_locations'] = self::$squareup::fetchLocations(config::get('payment_squareup_sandbox_token'), $first_location_id);
                $previous_setting['payment_squareup_sandbox_location_id'] = $first_location_id;
            }

            admin_setting_setting::editSetting('payment_squareup', $previous_setting);
        } catch (\Squareup\Exception $e) {
            self::pushAlert(array(
                'type' => 'danger',
                'icon' => 'exclamation-circle',
                'text' => sprintf(language::get('text_location_error'), $e::getMessage())
            ));
        }

        $previous_config = new Config();

        foreach ($previous_setting as $key => $value) {
            $previous_config::set($key, $value);
        }        

        if (request::$server['REQUEST_METHOD'] == 'POST' && self::validate()) {
            admin_setting_setting::editSetting('payment_squareup', array_merge($previous_setting, request::$post));

            session::$data['success'] = language::get('text_success');

            if (isset(request::$get['save_and_auth'])) {
                response::redirect(self::$squareup::authLink(request::$post['payment_squareup_client_id']));
            } else {
                response::redirect(url::link('marketplace/extension', '&type=payment'));
            }
        }

        document::setTitle(language::get('heading_title'));

        $data['error_status']                       = self::getValidationError('status');
        $data['error_display_name']                 = self::getValidationError('display_name');
        $data['error_client_id']                    = self::getValidationError('client_id');
        $data['error_client_secret']                = self::getValidationError('client_secret');
        $data['error_delay_capture']                = self::getValidationError('delay_capture');
        $data['error_sandbox_client_id']            = self::getValidationError('sandbox_client_id');
        $data['error_sandbox_token']                = self::getValidationError('sandbox_token');
        $data['error_location']                     = self::getValidationError('location');
        $data['error_cron_email']                   = self::getValidationError('cron_email');
        $data['error_cron_acknowledge']             = self::getValidationError('cron_acknowledge');

        $data['payment_squareup_status']                    = self::getSettingValue('payment_squareup_status');
        $data['payment_squareup_status_authorized']         = self::getSettingValue('payment_squareup_status_authorized');
        $data['payment_squareup_status_captured']           = self::getSettingValue('payment_squareup_status_captured');
        $data['payment_squareup_status_voided']             = self::getSettingValue('payment_squareup_status_voided');
        $data['payment_squareup_status_failed']             = self::getSettingValue('payment_squareup_status_failed');
        $data['payment_squareup_display_name']              = self::getSettingValue('payment_squareup_display_name');
        $data['payment_squareup_client_id']                 = self::getSettingValue('payment_squareup_client_id');
        $data['payment_squareup_client_secret']             = self::getSettingValue('payment_squareup_client_secret');
        $data['payment_squareup_enable_sandbox']            = self::getSettingValue('payment_squareup_enable_sandbox');
        $data['payment_squareup_debug']                     = self::getSettingValue('payment_squareup_debug');
        $data['payment_squareup_sort_order']                = self::getSettingValue('payment_squareup_sort_order');
        $data['payment_squareup_total']                     = self::getSettingValue('payment_squareup_total');
        $data['payment_squareup_geo_zone_id']               = self::getSettingValue('payment_squareup_geo_zone_id');
        $data['payment_squareup_sandbox_client_id']         = self::getSettingValue('payment_squareup_sandbox_client_id');
        $data['payment_squareup_sandbox_token']             = self::getSettingValue('payment_squareup_sandbox_token');
        $data['payment_squareup_locations']                 = self::getSettingValue('payment_squareup_locations', $previous_config::get('payment_squareup_locations'));
        $data['payment_squareup_location_id']               = self::getSettingValue('payment_squareup_location_id');
        $data['payment_squareup_sandbox_locations']         = self::getSettingValue('payment_squareup_sandbox_locations', $previous_config::get('payment_squareup_sandbox_locations'));
        $data['payment_squareup_sandbox_location_id']       = self::getSettingValue('payment_squareup_sandbox_location_id');
        $data['payment_squareup_delay_capture']             = self::getSettingValue('payment_squareup_delay_capture');
        $data['payment_squareup_recurring_status']          = self::getSettingValue('payment_squareup_recurring_status');
        $data['payment_squareup_cron_email_status']         = self::getSettingValue('payment_squareup_cron_email_status');
        $data['payment_squareup_cron_email']                = self::getSettingValue('payment_squareup_cron_email', config('email'));
        $data['payment_squareup_cron_token']                = self::getSettingValue('payment_squareup_cron_token');
        $data['payment_squareup_cron_acknowledge']          = self::getSettingValue('payment_squareup_cron_acknowledge', null, true);
        $data['payment_squareup_notify_recurring_success']  = self::getSettingValue('payment_squareup_notify_recurring_success');
        $data['payment_squareup_notify_recurring_fail']     = self::getSettingValue('payment_squareup_notify_recurring_fail');
        $data['payment_squareup_merchant_id']               = self::getSettingValue('payment_squareup_merchant_id', $previous_config::get('payment_squareup_merchant_id'));
        $data['payment_squareup_merchant_name']             = self::getSettingValue('payment_squareup_merchant_name', $previous_config::get('payment_squareup_merchant_name'));

        if ($previous_config::get('payment_squareup_access_token') && $previous_config::get('payment_squareup_access_token_expires')) {
            $expiration_time = date_create_from_format('Y-m-d\TH:i:s\Z', $previous_config::get('payment_squareup_access_token_expires'));
            $now = date_create();

            $delta = $expiration_time::getTimestamp() - $now::getTimestamp();
            $expiration_date_formatted = $expiration_time::format('l, F jS, Y h:i:s A, e');

            if ($delta < 0) {
                self::pushAlert(array(
                    'type' => 'danger',
                    'icon' => 'exclamation-circle',
                    'text' => sprintf(language::get('text_token_expired'), url::link('extension/payment/squareup/refresh_token'))
                ));
            } else if ($delta < (5 * 24 * 60 * 60)) { // token is valid, just about to expire
                self::pushAlert(array(
                    'type' => 'warning',
                    'icon' => 'exclamation-circle',
                    'text' => sprintf(language::get('text_token_expiry_warning'), $expiration_date_formatted, url::link('extension/payment/squareup/refresh_token'))
                ));
            }

            $data['access_token_expires_time'] = $expiration_date_formatted;
        } else if ($previous_config::get('payment_squareup_client_id')) {
            self::pushAlert(array(
                'type' => 'danger',
                'icon' => 'exclamation-circle',
                'text' => sprintf(language::get('text_token_revoked'), self::$squareup::authLink($previous_config::get('payment_squareup_client_id')))
            ));

            $data['access_token_expires_time'] = language::get('text_na');
        }

        if ($previous_config::get('payment_squareup_client_id')) {
            $data['payment_squareup_auth_link'] = self::$squareup::authLink($previous_config::get('payment_squareup_client_id'));
        } else {
            $data['payment_squareup_auth_link'] = null;
        }

        $data['payment_squareup_redirect_uri'] = str_replace('&amp;', '&', url::link('extension/payment/squareup/oauth_callback', ''));
        $data['payment_squareup_refresh_link'] = url::link('extension/payment/squareup/refresh_token');

        if (config::get('payment_squareup_enable_sandbox')) {
            self::pushAlert(array(
                'type' => 'warning',
                'icon' => 'exclamation-circle',
                'text' => language::get('text_sandbox_enabled')
            ));
        }

        if (isset(self::$error['warning'])) {
            self::pushAlert(array(
                'type' => 'danger',
                'icon' => 'exclamation-circle',
                'text' => self::$error['warning']
            ));
        }

        // Insert success message from the session
        if (isset(session::$data['success'])) {
            self::pushAlert(array(
                'type' => 'success',
                'icon' => 'exclamation-circle',
                'text' => session::$data['success']
            ));

            unset(session::$data['success']);
        }

        if (request::$server['HTTPS']) {
            // Push the SSL reminder alert
            self::pushAlert(array(
                'type' => 'info',
                'icon' => 'lock',
                'text' => language::get('text_notification_ssl')
            ));
        } else {
            // Push the SSL reminder alert
            self::pushAlert(array(
                'type' => 'danger',
                'icon' => 'exclamation-circle',
                'text' => language::get('error_no_ssl')
            ));
        }

        $tabs = array(
            'tab-transaction',
            'tab-setting',
            'tab-recurring',
            'tab-cron'
        );

        if (isset(request::$get['tab']) && in_array(request::$get['tab'], $tabs)) {
            $data['tab'] = request::$get['tab'];
        } else if (isset(self::$error['cron_email']) || isset(self::$error['cron_acknowledge'])) {
            $data['tab'] = 'tab-cron';
        } else if (self::$error) {
            $data['tab'] = 'tab-setting';
        } else {
            $data['tab'] = $tabs[1];
        }

        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'text' => language::get('text_home'),
            'href' => url::link('common/dashboard')
        );

        $data['breadcrumbs'][] = array(
            'text' => language::get('text_extension'),
            'href' => url::link('marketplace/extension', '&type=payment')
        );

        $data['breadcrumbs'][] = array(
            'text' => language::get('heading_title'),
            'href' => url::link('extension/payment/squareup')
        );

        $data['action'] = html_entity_decode(url::link('extension/payment/squareup'));
        $data['action_save_auth'] = html_entity_decode(url::link('extension/payment/squareup', '&save_and_auth=1'));
        $data['cancel'] = html_entity_decode(url::link('marketplace/extension', '&type=payment'));
        $data['url_list_transactions'] = html_entity_decode(url::link('extension/payment/squareup/transactions', '&page={PAGE}'));

        $data['languages'] = array();
        foreach (admin_localisation_language::getLanguages() as $language) {
            $data['languages'][] = array(
                'language_id' => $language['language_id'],
                'name' => $language['name'] . ($language['code'] == config('language') ? language::get('text_default') : ''),
                'image' => 'language/' . $language['code'] . '/'. $language['code'] . '.png'
            );
        }

        $data['order_statuses'] = admin_localisation_order_status::getOrderStatuses();

        $data['geo_zones'] = admin_localisation_geo_zone::getGeoZones();

        $data['payment_squareup_cron_command'] = PHP_BINDIR . '/php -d session.save_path=' . session_save_path() . ' ' . 'system/library/squareup/cron.php ' . parse_url($server, PHP_URL_HOST) . ' 443 > /dev/null 2> /dev/null';
        
        if (!config::get('payment_squareup_cron_token')) {
            $data['payment_squareup_cron_token'] = md5(mt_rand());
        }

        $data['payment_squareup_cron_url'] = 'https://' . parse_url($server, PHP_URL_HOST) . dirname(parse_url($server, PHP_URL_PATH)) . '/index.php?route=extension/recurring/squareup/recurring&cron_token={CRON_TOKEN}';

        $data['catalog'] = request::$server['HTTPS'] ? HTTPS_CATALOG : HTTP_CATALOG;

        // API login

        $api_info = admin_user_api::getApi(config('api_id'));

        if ($api_info && user::hasPermission('modify', 'sale/order')) {
            $session = new Session(config::get('session_engine'));
            
            $session::start();
                    
            admin_user_api::deleteApiSessionBySessionId($session::getId());
            
            admin_user_api::addApiSession($api_info['api_id'], $session::getId(), request::$server['REMOTE_ADDR']);
            
            $session::$data['api_id'] = $api_info['api_id'];

            $data['api_token'] = $session::getId();
        } else {
            $data['api_token'] = '';
        }

        $data['header'] = load::controller('common/header');
        $data['column_left'] = load::controller('common/column_left');
        $data['footer'] = load::controller('common/footer');

        $data['alerts'] = self::pullAlerts();

        self::clearAlerts();

        response::setOutput(load::view('extension/payment/squareup', $data));
    }

    public static function transaction_info() {
        load::language('extension/payment/squareup');


        load::library('squareup');

        if (isset(request::$get['squareup_transaction_id'])) {
            $squareup_transaction_id = request::$get['squareup_transaction_id'];
        } else {
            $squareup_transaction_id = 0;
        }

        $transaction_info = admin_extension_payment_squareup::getTransaction($squareup_transaction_id);

        if (empty($transaction_info)) {
            response::redirect(url::link('extension/payment/squareup'));
        }

        document::setTitle(sprintf(language::get('heading_title_transaction'), $transaction_info['transaction_id']));

        $data['alerts'] = self::pullAlerts();

        self::clearAlerts();

        $data['text_edit'] = sprintf(language::get('heading_title_transaction'), $transaction_info['transaction_id']);

        $amount = currency::format($transaction_info['transaction_amount'], $transaction_info['transaction_currency']);

        $data['confirm_capture'] = sprintf(language::get('text_confirm_capture'), $amount);
        $data['confirm_void'] = sprintf(language::get('text_confirm_void'), $amount);
        $data['confirm_refund'] = language::get('text_confirm_refund');
        $data['insert_amount'] = sprintf(language::get('text_insert_amount'), $amount, $transaction_info['transaction_currency']);
        $data['text_loading'] = language::get('text_loading_short');
        
        $data['billing_address_company'] = $transaction_info['billing_address_company'];
        $data['billing_address_street'] = $transaction_info['billing_address_street_1'] . ' ' . $transaction_info['billing_address_street_2'];
        $data['billing_address_city'] = $transaction_info['billing_address_city'];
        $data['billing_address_postcode'] = $transaction_info['billing_address_postcode'];
        $data['billing_address_province'] = $transaction_info['billing_address_province'];
        $data['billing_address_country'] = $transaction_info['billing_address_country'];

        $data['transaction_id'] = $transaction_info['transaction_id'];
        $data['merchant'] = $transaction_info['merchant_id'];
        $data['order_id'] = $transaction_info['order_id'];
        $data['type'] = $transaction_info['transaction_type'];
        $data['amount'] = $amount;
        $data['currency'] = $transaction_info['transaction_currency'];
        $data['browser'] = $transaction_info['device_browser'];
        $data['ip'] = $transaction_info['device_ip'];
        $data['date_created'] = date(language::get('datetime_format'), strtotime($transaction_info['created_at']));
        
        $data['cancel'] = url::link('extension/payment/squareup', '&tab=tab-transaction');

        $data['url_order'] = url::link('sale/order/info', '&order_id=' . $transaction_info['order_id']);
        $data['url_void'] = url::link('extension/payment/squareup' . '/void', '&preserve_alert=true&squareup_transaction_id=' . $transaction_info['squareup_transaction_id']);
        $data['url_capture'] = url::link('extension/payment/squareup' . '/capture', '&preserve_alert=true&squareup_transaction_id=' . $transaction_info['squareup_transaction_id']);
        $data['url_refund'] = url::link('extension/payment/squareup' . '/refund', '&preserve_alert=true&squareup_transaction_id=' . $transaction_info['squareup_transaction_id']);
        $data['url_transaction'] = sprintf(
            Squareup::$VIEW_TRANSACTION_URL,
            $transaction_info['transaction_id'],
            $transaction_info['location_id']
        );

        $data['is_authorized'] = in_array($transaction_info['transaction_type'], array('AUTHORIZED'));
        $data['is_captured'] = in_array($transaction_info['transaction_type'], array('CAPTURED'));

        $data['has_refunds'] = (bool)$transaction_info['is_refunded'];

        if ($data['has_refunds']) {
            $refunds = @json_decode($transaction_info['refunds'], true);

            $data['refunds'] = array();

            $data['text_refunds'] = sprintf(language::get('text_refunds'), count($refunds));

            foreach ($refunds as $refund) {
                $amount = currency::format(
                    self::$squareup::standardDenomination(
                        $refund['amount_money']['amount'], 
                        $refund['amount_money']['currency']
                    ), 
                    $refund['amount_money']['currency']
                );

                $fee = currency::format(
                    self::$squareup::standardDenomination(
                        $refund['processing_fee_money']['amount'], 
                        $refund['processing_fee_money']['currency']
                    ), 
                    $refund['processing_fee_money']['currency']
                );

                $data['refunds'][] = array(
                    'date_created' => date(language::get('datetime_format'), strtotime($refund['created_at'])),
                    'reason' => $refund['reason'],
                    'status' => $refund['status'],
                    'amount' => $amount,
                    'fee' => $fee
                );
            }
        }

        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'text' => language::get('text_home'),
            'href' => url::link('common/dashboard')
        );

        $data['breadcrumbs'][] = array(
            'text' => language::get('text_extension'),
            'href' => url::link('marketplace/extension', '&type=payment')
        );

        $data['breadcrumbs'][] = array(
            'text' => language::get('heading_title'),
            'href' => url::link('extension/payment/squareup')
        );

        $data['breadcrumbs'][] = array(
            'text' => sprintf(language::get('heading_title_transaction'), $transaction_info['squareup_transaction_id']),
            'href' => url::link('extension/payment/squareup/transaction_info', '&squareup_transaction_id=' . $squareup_transaction_id)
        );

        $data['catalog'] = request::$server['HTTPS'] ? HTTPS_CATALOG : HTTP_CATALOG;

        // API login

        $api_info = admin_user_api::getApi(config('api_id'));

        if ($api_info && user::hasPermission('modify', 'sale/order')) {
            $session = new Session(config::get('session_engine'));
            
            $session::start();
                    
            admin_user_api::deleteApiSessionBySessionId($session::getId());
            
            admin_user_api::addApiSession($api_info['api_id'], $session::getId(), request::$server['REMOTE_ADDR']);
            
            $session::$data['api_id'] = $api_info['api_id'];

            $data['api_token'] = $session::getId();
        } else {
            $data['api_token'] = '';
        }

        $data['header'] = load::controller('common/header');
        $data['column_left'] = load::controller('common/column_left');
        $data['footer'] = load::controller('common/footer');

        response::setOutput(load::view('extension/payment/squareup_transaction_info', $data));
    }

    public static function transactions() {
        load::language('extension/payment/squareup');


        if (isset(request::$get['page'])) {
            $page = (int)request::$get['page'];
        } else {
            $page = 1;
        }

        $result = array(
            'transactions' => array(),
            'pagination' => ''
        );

        $filter_data = array(
            'start' => ($page - 1) * config('limit_admin'),
            'limit' => config('limit_admin')
        );

        if (isset(request::$get['order_id'])) {
            $filter_data['order_id'] = request::$get['order_id'];
        }

        $transactions_total = admin_extension_payment_squareup::getTotalTransactions($filter_data);
        $transactions = admin_extension_payment_squareup::getTransactions($filter_data);


        foreach ($transactions as $transaction) {
            $amount = currency::format($transaction['transaction_amount'], $transaction['transaction_currency']);

            $order_info = admin_sale_order::getOrder($transaction['order_id']);
            
            $result['transactions'][] = array(
                'squareup_transaction_id' => $transaction['squareup_transaction_id'],
                'transaction_id' => $transaction['transaction_id'],
                'url_order' => url::link('sale/order/info', '&order_id=' . $transaction['order_id']),
                'url_void' => url::link('extension/payment/squareup/void', '&squareup_transaction_id=' . $transaction['squareup_transaction_id']),
                'url_capture' => url::link('extension/payment/squareup/capture', '&squareup_transaction_id=' . $transaction['squareup_transaction_id']),
                'url_refund' => url::link('extension/payment/squareup/refund', '&squareup_transaction_id=' . $transaction['squareup_transaction_id']),
                'confirm_capture' => sprintf(language::get('text_confirm_capture'), $amount),
                'confirm_void' => sprintf(language::get('text_confirm_void'), $amount),
                'confirm_refund' => language::get('text_confirm_refund'),
                'insert_amount' => sprintf(language::get('text_insert_amount'), $amount, $transaction['transaction_currency']),
                'order_id' => $transaction['order_id'],
                'type' => $transaction['transaction_type'],
                'num_refunds' => count(@json_decode($transaction['refunds'], true)),
                'amount' => $amount,
                'customer' => $order_info['firstname'] . ' ' . $order_info['lastname'],
                'ip' => $transaction['device_ip'],
                'date_created' => date(language::get('datetime_format'), strtotime($transaction['created_at'])),
                'url_info' => url::link('extension/payment/squareup/transaction_info', '&squareup_transaction_id=' . $transaction['squareup_transaction_id'])
            );
        }

        $pagination = new Pagination();
        $pagination::$total = $transactions_total;
        $pagination::$page = $page;
        $pagination::$limit = config('limit_admin');
        $pagination::$url = '{page}';

        $result['pagination'] = $pagination::render();

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($result));
    }

    public static function refresh_token() {
        load::language('extension/payment/squareup');

        if (!user::hasPermission('modify', 'extension/payment/squareup')) {
            self::pushAlert(array(
                'type' => 'danger',
                'icon' => 'exclamation-circle',
                'text' => language::get('error_permission')
            ));

            response::redirect(url::link('extension/payment/squareup'));
        }


        load::library('squareup');

        try {
            $response = self::$squareup::refreshToken();

            if (!isset($response['access_token']) || !isset($response['token_type']) || !isset($response['expires_at']) || !isset($response['merchant_id']) ||
                $response['merchant_id'] != config::get('payment_squareup_merchant_id')) {
                self::pushAlert(array(
                    'type' => 'danger',
                    'icon' => 'exclamation-circle',
                    'text' => language::get('error_refresh_access_token') 
                ));
            } else {
                $settings = admin_setting_setting::getSetting('payment_squareup');

                $settings['payment_squareup_access_token'] = $response['access_token']; 
                $settings['payment_squareup_access_token_expires'] = $response['expires_at'];

                admin_setting_setting::editSetting('payment_squareup', $settings); 

                self::pushAlert(array(
                    'type' => 'success',
                    'icon' => 'exclamation-circle',
                    'text' => language::get('text_refresh_access_token_success')
                ));
            }
        } catch (\Squareup\Exception $e) {
            self::pushAlert(array(
                'type' => 'danger',
                'icon' => 'exclamation-circle',
                'text' => sprintf(language::get('error_token'), $e::getMessage())
            ));
        }

        response::redirect(url::link('extension/payment/squareup'));
    }

    public static function oauth_callback() {
        load::language('extension/payment/squareup');

        if (!user::hasPermission('modify', 'extension/payment/squareup')) {
            self::pushAlert(array(
                'type' => 'danger',
                'icon' => 'exclamation-circle',
                'text' => language::get('error_permission')
            ));

            response::redirect(url::link('extension/payment/squareup'));
        }


        load::library('squareup');

        if (isset(request::$get['error']) || isset(request::$get['error_description'])) {
            // auth error
            if (request::$get['error'] == 'access_denied' && request::$get['error_description'] == 'user_denied') {
                // user rejected giving auth permissions to his store
                self::pushAlert(array(
                    'type' => 'warning',
                    'icon' => 'exclamation-circle',
                    'text' => language::get('error_user_rejected_connect_attempt')
                ));
            }

            response::redirect(url::link('extension/payment/squareup'));
        }

        // verify parameters for the redirect from Square (against random url crawling)
        if (!isset(request::$get['state']) || !isset(request::$get['code']) || !isset(request::$get['response_type'])) {
            // missing or wrong info
            self::pushAlert(array(
                'type' => 'danger',
                'icon' => 'exclamation-circle',
                'text' => language::get('error_possible_xss')
            ));

            response::redirect(url::link('extension/payment/squareup'));
        }

        // verify the state (against cross site requests)
        if (!isset(session::$data['payment_squareup_oauth_state']) || session::$data['payment_squareup_oauth_state'] != request::$get['state']) {
            // state mismatch
            self::pushAlert(array(
                'type' => 'danger',
                'icon' => 'exclamation-circle',
                'text' => language::get('error_possible_xss')
            ));

            response::redirect(url::link('extension/payment/squareup'));
        }

        try {
            $token = self::$squareup::exchangeCodeForAccessToken(request::$get['code']);
            
            $previous_setting = admin_setting_setting::getSetting('payment_squareup');

            $previous_setting['payment_squareup_locations'] = self::$squareup::fetchLocations($token['access_token'], $first_location_id);

            if (
                !isset($previous_setting['payment_squareup_location_id']) || 
                (isset($previous_setting['payment_squareup_location_id']) && !in_array(
                    $previous_setting['payment_squareup_location_id'], 
                    array_map(
                        function($location) {
                            return $location['id'];
                        },
                        $previous_setting['payment_squareup_locations']
                    )
                ))
            ) {
                $previous_setting['payment_squareup_location_id'] = $first_location_id;
            }

            if (!config::get('payment_squareup_sandbox_locations') && config::get('payment_squareup_sandbox_token')) {
                $previous_setting['payment_squareup_sandbox_locations'] = self::$squareup::fetchLocations(config::get('payment_squareup_sandbox_token'), $first_location_id);
                $previous_setting['payment_squareup_sandbox_location_id'] = $first_location_id;
            }

            $previous_setting['payment_squareup_merchant_id'] = $token['merchant_id'];
            $previous_setting['payment_squareup_merchant_name'] = ''; // only available in v1 of the API, not populated for now
            $previous_setting['payment_squareup_access_token'] = $token['access_token'];
            $previous_setting['payment_squareup_access_token_expires'] = $token['expires_at'];

            admin_setting_setting::editSetting('payment_squareup', $previous_setting);

            unset(session::$data['payment_squareup_oauth_state']);
            unset(session::$data['payment_squareup_oauth_redirect']);

            self::pushAlert(array(
                'type' => 'success',
                'icon' => 'exclamation-circle',
                'text' => language::get('text_refresh_access_token_success')
            ));
        } catch (\Squareup\Exception $e) {
            self::pushAlert(array(
                'type' => 'danger',
                'icon' => 'exclamation-circle',
                'text' => sprintf(language::get('error_token'), $e::getMessage())
            ));
        }

        response::redirect(url::link('extension/payment/squareup'));
    }

    public static function capture() {
        self::transactionAction(function($transaction_info, &$json) {
            $updated_transaction = self::$squareup::captureTransaction($transaction_info['location_id'], $transaction_info['transaction_id']);

            $status = $updated_transaction['tenders'][0]['card_details']['status'];

            admin_extension_payment_squareup::updateTransaction($transaction_info['squareup_transaction_id'], $status);

            $json['order_history_data'] = array(
                'notify' => 1,
                'order_id' => $transaction_info['order_id'],
                'order_status_id' => admin_extension_payment_squareup::getOrderStatusId($transaction_info['order_id'], $status),
                'comment' => language::get('squareup_status_comment_' . strtolower($status)),
            );

            $json['success'] = language::get('text_success_capture');
        });
    }

    public static function void() {
        self::transactionAction(function($transaction_info, &$json) {
            $updated_transaction = self::$squareup::voidTransaction($transaction_info['location_id'], $transaction_info['transaction_id']);

            $status = $updated_transaction['tenders'][0]['card_details']['status'];

            admin_extension_payment_squareup::updateTransaction($transaction_info['squareup_transaction_id'], $status);

            $json['order_history_data'] = array(
                'notify' => 1,
                'order_id' => $transaction_info['order_id'],
                'order_status_id' => admin_extension_payment_squareup::getOrderStatusId($transaction_info['order_id'], $status),
                'comment' => language::get('squareup_status_comment_' . strtolower($status)),
            );

            $json['success'] = language::get('text_success_void');
        });
    }

    public static function refund() {
        self::transactionAction(function($transaction_info, &$json) {
            if (!empty(request::$post['reason'])) {
                $reason = request::$post['reason'];
            } else {
                $reason = language::get('text_no_reason_provided');
            }

            if (!empty(request::$post['amount'])) {
                $amount = preg_replace('~[^0-9\.\,]~', '', request::$post['amount']);

                if (strpos($amount, ',') !== FALSE && strpos($amount, '.') !== FALSE) {
                    $amount = (float)str_replace(',', '', $amount);
                } else if (strpos($amount, ',') !== FALSE && strpos($amount, '.') === FALSE) {
                    $amount = (float)str_replace(',', '.', $amount);
                } else {
                    $amount = (float)$amount;
                }
            } else {
                $amount = 0;
            }

            $currency = $transaction_info['transaction_currency'];
            $tenders = @json_decode($transaction_info['tenders'], true);

            $updated_transaction = self::$squareup::refundTransaction($transaction_info['location_id'], $transaction_info['transaction_id'], $reason, $amount, $currency, $tenders[0]['id']);

            $status = $updated_transaction['tenders'][0]['card_details']['status'];

            $refunds = array();

            if (!empty($updated_transaction['refunds'])) {
                $refunds = $updated_transaction['refunds'];
            }

            admin_extension_payment_squareup::updateTransaction($transaction_info['squareup_transaction_id'], $status, $refunds);

            $last_refund = array_pop($refunds);

            if ($last_refund) {
                $refunded_amount = currency::format(
                    self::$squareup::standardDenomination(
                        $last_refund['amount_money']['amount'], 
                        $last_refund['amount_money']['currency']
                    ), 
                    $last_refund['amount_money']['currency']
                );

                $comment = sprintf(language::get('text_refunded_amount'), $refunded_amount, $last_refund['status'], $last_refund['reason']);

                $json['order_history_data'] = array(
                    'notify' => 1,
                    'order_id' => $transaction_info['order_id'],
                    'order_status_id' => admin_extension_payment_squareup::getOrderStatusId($transaction_info['order_id']),
                    'comment' => $comment,
                );

                $json['success'] = language::get('text_success_refund');
            } else {
                $json['error'] = language::get('error_no_refund');
            }
        });
    }

    public static function order() {
        load::language('extension/payment/squareup');

        $data['url_list_transactions'] = html_entity_decode(url::link('extension/payment/squareup/transactions', '&order_id=' . request::$get['order_id'] . '&page={PAGE}'));
        $data['order_id'] = (int)request::$get['order_id'];

        $data['catalog'] = request::$server['HTTPS'] ? HTTPS_CATALOG : HTTP_CATALOG;

        // API login

        $api_info = admin_user_api::getApi(config('api_id'));

        if ($api_info && user::hasPermission('modify', 'sale/order')) {
            $session = new Session(config::get('session_engine'));
            
            $session::start();
                    
            admin_user_api::deleteApiSessionBySessionId($session::getId());
            
            admin_user_api::addApiSession($api_info['api_id'], $session::getId(), request::$server['REMOTE_ADDR']);
            
            $session::$data['api_id'] = $api_info['api_id'];

            $data['api_token'] = $session::getId();
        } else {
            $data['api_token'] = '';
        }

        return load::view('extension/payment/squareup_order', $data);
    }

    public static function install() {
        
        admin_extension_payment_squareup::createTables();
    }

    public static function uninstall() {

        admin_extension_payment_squareup::dropTables();
    }

    public static function recurringButtons() {
        if (!user::hasPermission('modify', 'sale/recurring')) {
            return;
        }


        load::language('extension/payment/squareup');

        if (isset(request::$get['order_recurring_id'])) {
            $order_recurring_id = request::$get['order_recurring_id'];
        } else {
            $order_recurring_id = 0;
        }

        $recurring_info = admin_sale_recurring::getRecurring($order_recurring_id);

        $data['button_text'] = language::get('button_cancel_recurring');

        if ($recurring_info['status'] == ModelExtensionPaymentSquareup::$RECURRING_ACTIVE) {
            $data['order_recurring_id'] = $order_recurring_id;
        } else {
            $data['order_recurring_id'] = '';
        }


        $order_info = admin_sale_order::getOrder($recurring_info['order_id']);

        $data['order_id'] = $recurring_info['order_id'];
        $data['store_id'] = $order_info['store_id'];
        $data['order_status_id'] = $order_info['order_status_id'];
        $data['comment'] = language::get('text_order_history_cancel');
        $data['notify'] = 1;

        $data['catalog'] = request::$server['HTTPS'] ? HTTPS_CATALOG : HTTP_CATALOG;

        // API login

        $api_info = admin_user_api::getApi(config('api_id'));

        if ($api_info && user::hasPermission('modify', 'sale/order')) {
            $session = new Session(config::get('session_engine'));
            
            $session::start();
                    
            admin_user_api::deleteApiSessionBySessionId($session::getId());
            
            admin_user_api::addApiSession($api_info['api_id'], $session::getId(), request::$server['REMOTE_ADDR']);
            
            $session::$data['api_id'] = $api_info['api_id'];

            $data['api_token'] = $session::getId();
        } else {
            $data['api_token'] = '';
        }

        $data['cancel'] = html_entity_decode(url::link('extension/payment/squareup/recurringCancel', 'order_recurring_id=' . $order_recurring_id . '&user_token=' . session::$data['user_token']));

        return load::view('extension/payment/squareup_recurring_buttons', $data);
    }

    public static function recurringCancel() {
        load::language('extension/payment/squareup');

        $json = array();
        
        if (!user::hasPermission('modify', 'sale/recurring')) {
            $json['error'] = language::get('error_permission_recurring');
        } else {
            
            if (isset(request::$get['order_recurring_id'])) {
                $order_recurring_id = request::$get['order_recurring_id'];
            } else {
                $order_recurring_id = 0;
            }
            
            $recurring_info = admin_sale_recurring::getRecurring($order_recurring_id);

            if ($recurring_info) {

                admin_extension_payment_squareup::editOrderRecurringStatus($order_recurring_id, ModelExtensionPaymentSquareup::$RECURRING_CANCELLED);

                $json['success'] = language::get('text_canceled_success');
                
            } else {
                $json['error'] = language::get('error_not_found');
            }
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    protected static function validate() {
        if (!user::hasPermission('modify', 'extension/payment/squareup')) {
            self::$error['warning'] = language::get('error_permission');
        }

        if (empty(request::$post['payment_squareup_client_id']) || strlen(request::$post['payment_squareup_client_id']) > 32) {
            self::$error['client_id'] = language::get('error_client_id');
        }

        if (empty(request::$post['payment_squareup_client_secret']) || strlen(request::$post['payment_squareup_client_secret']) > 50) {
            self::$error['client_secret'] = language::get('error_client_secret');
        }

        if (!empty(request::$post['payment_squareup_enable_sandbox'])) {
            if (empty(request::$post['payment_squareup_sandbox_client_id']) || strlen(request::$post['payment_squareup_sandbox_client_id']) > 42) {
                self::$error['sandbox_client_id'] = language::get('error_sandbox_client_id');
            }

            if (empty(request::$post['payment_squareup_sandbox_token']) || strlen(request::$post['payment_squareup_sandbox_token']) > 42) {
                self::$error['sandbox_token'] = language::get('error_sandbox_token');
            }

            if (config::get('payment_squareup_merchant_id') && !config::get('payment_squareup_sandbox_locations')) {
                self::$error['warning'] = language::get('text_no_appropriate_locations_warning');
            }

            if (config::get('payment_squareup_sandbox_locations') && isset(request::$post['payment_squareup_sandbox_location_id']) && !in_array(request::$post['payment_squareup_sandbox_location_id'], array_map(function($location) {
                return $location['id'];
            }, config::get('payment_squareup_sandbox_locations')))) {
                self::$error['location'] = language::get('error_no_location_selected');
            }
        } else {
            if (config::get('payment_squareup_merchant_id') && !config::get('payment_squareup_locations')) {
                self::$error['warning'] = language::get('text_no_appropriate_locations_warning');
            }

            if (config::get('payment_squareup_locations') && isset(request::$post['payment_squareup_location_id']) && !in_array(request::$post['payment_squareup_location_id'], array_map(function($location) {
                return $location['id'];
            }, config::get('payment_squareup_locations')))) {
                self::$error['location'] = language::get('error_no_location_selected');
            }
        }

        if (!empty(request::$post['payment_squareup_cron_email_status'])) {
            if (!filter_var(request::$post['payment_squareup_cron_email'], FILTER_VALIDATE_EMAIL)) {
                self::$error['cron_email'] = language::get('error_invalid_email');
            }
        }

        if (!isset(request::$get['save_and_auth']) && empty(request::$post['payment_squareup_cron_acknowledge'])) {
            self::$error['cron_acknowledge'] = language::get('error_cron_acknowledge');
        }

        if (self::$error && empty(self::$error['warning'])) {
            self::$error['warning'] = language::get('error_form');
        }

        return !self::$error;
    }

    protected static function transactionAction($callback) {
        load::language('extension/payment/squareup');


        load::library('squareup');

        $json = array();

        if (!user::hasPermission('modify', 'extension/payment/squareup')) {
            $json['error'] = language::get('error_permission');
        }

        if (isset(request::$get['squareup_transaction_id'])) {
            $squareup_transaction_id = request::$get['squareup_transaction_id'];
        } else {
            $squareup_transaction_id = 0;
        }

        $transaction_info = admin_extension_payment_squareup::getTransaction($squareup_transaction_id);

        if (empty($transaction_info)) {
            $json['error'] = language::get('error_transaction_missing');
        } else {
            try {
                $callback($transaction_info, $json);
            } catch (\Squareup\Exception $e) {
                $json['error'] = $e::getMessage();
            }
        }

        if (isset(request::$get['preserve_alert'])) {
            if (!empty($json['error'])) {
                self::pushAlert(array(
                    'type' => 'danger',
                    'icon' => 'exclamation-circle',
                    'text' => $json['error']
                ));
            }

            if (!empty($json['success'])) {
                self::pushAlert(array(
                    'type' => 'success',
                    'icon' => 'exclamation-circle',
                    'text' => $json['success']
                ));
            }
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    protected static function pushAlert($alert) {
        session::$data['payment_squareup_alerts'][] = $alert;
    }

    protected static function pullAlerts() {
        if (isset(session::$data['payment_squareup_alerts'])) {
            return session::$data['payment_squareup_alerts'];
        } else {
            return array();
        }
    }

    protected static function clearAlerts() {
        unset(session::$data['payment_squareup_alerts']);
    }

    protected static function getSettingValue($key, $default = null, $checkbox = false) {
        if ($checkbox) {
            if (request::$server['REQUEST_METHOD'] == 'POST' && !isset(request::$post[$key])) {
                return $default;
            } else {
                return config::get($key);
            }
        }

        if (isset(request::$post[$key])) {
            return request::$post[$key]; 
        } else if (config::has($key)) {
            return config::get($key);
        } else {
            return $default;
        }
    }

    protected static function getValidationError($key) {
        if (isset(self::$error[$key])) {
            return self::$error[$key];
        } else {
            return '';
        }
    }
}