<?php
/**
 * @package		OpenCart
 * @author		Meng Wenbin
 * @copyright	Copyright (c) 2010 - 2017, Chengdu Guangda Network Technology Co. Ltd. (https://www.opencart.cn/)
 * @license		https://opensource.org/licenses/GPL-3.0
 * @link		https://www.opencart.cn
 */

class ControllerExtensionPaymentWechatPay {
	private static $error = array();

	public static function index() {
		load::language('extension/payment/wechat_pay');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('payment_wechat_pay', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=payment'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['app_id'])) {
			$data['error_app_id'] = self::$error['app_id'];
		} else {
			$data['error_app_id'] = '';
		}

		if (isset(self::$error['app_secret'])) {
			$data['error_app_secret'] = self::$error['app_secret'];
		} else {
			$data['error_app_secret'] = '';
		}

		if (isset(self::$error['mch_id'])) {
			$data['error_mch_id'] = self::$error['mch_id'];
		} else {
			$data['error_mch_id'] = '';
		}

		if (isset(self::$error['api_secret'])) {
			$data['error_api_secret'] = self::$error['api_secret'];
		} else {
			$data['error_api_secret'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=payment')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/payment/wechat_pay')
		);

		$data['action'] = url::link('extension/payment/wechat_pay');

		$data['cancel'] = url::link('marketplace/extension', '&type=payment');

		if (isset(request::$post['payment_wechat_pay_app_id'])) {
			$data['payment_wechat_pay_app_id'] = request::$post['payment_wechat_pay_app_id'];
		} else {
			$data['payment_wechat_pay_app_id'] = config::get('payment_wechat_pay_app_id');
		}

		if (isset(request::$post['payment_wechat_pay_app_secret'])) {
			$data['payment_wechat_pay_app_secret'] = request::$post['payment_wechat_pay_app_secret'];
		} else {
			$data['payment_wechat_pay_app_secret'] = config::get('payment_wechat_pay_app_secret');
		}

		if (isset(request::$post['payment_wechat_pay_mch_id'])) {
			$data['payment_wechat_pay_mch_id'] = request::$post['payment_wechat_pay_mch_id'];
		} else {
			$data['payment_wechat_pay_mch_id'] = config::get('payment_wechat_pay_mch_id');
		}

		if (isset(request::$post['payment_wechat_pay_api_secret'])) {
			$data['payment_wechat_pay_api_secret'] = request::$post['payment_wechat_pay_api_secret'];
		} else {
			$data['payment_wechat_pay_api_secret'] = config::get('payment_wechat_pay_api_secret');
		}

		if (isset(request::$post['payment_wechat_pay_total'])) {
			$data['payment_wechat_pay_total'] = request::$post['payment_wechat_pay_total'];
		} else {
			$data['payment_wechat_pay_total'] = config::get('payment_wechat_pay_total');
		}

		if (isset(request::$post['payment_wechat_pay_currency'])) {
			$data['payment_wechat_pay_currency'] = request::$post['payment_wechat_pay_currency'];
		} else {
			$data['payment_wechat_pay_currency'] = config::get('payment_wechat_pay_currency');
		}

		if (isset(request::$post['payment_wechat_pay_completed_status_id'])) {
			$data['payment_wechat_pay_completed_status_id'] = request::$post['payment_wechat_pay_completed_status_id'];
		} else {
			$data['payment_wechat_pay_completed_status_id'] = config::get('payment_wechat_pay_completed_status_id');
		}


		$data['order_statuses'] = admin_localisation_order_status::getOrderStatuses();

		if (isset(request::$post['payment_wechat_pay_geo_zone_id'])) {
			$data['payment_wechat_pay_geo_zone_id'] = request::$post['payment_wechat_pay_geo_zone_id'];
		} else {
			$data['payment_wechat_pay_geo_zone_id'] = config::get('payment_wechat_pay_geo_zone_id');
		}


		$data['geo_zones'] = admin_localisation_geo_zone::getGeoZones();

		if (isset(request::$post['payment_wechat_pay_status'])) {
			$data['payment_wechat_pay_status'] = request::$post['payment_wechat_pay_status'];
		} else {
			$data['payment_wechat_pay_status'] = config::get('payment_wechat_pay_status');
		}

		if (isset(request::$post['payment_wechat_pay_sort_order'])) {
			$data['payment_wechat_pay_sort_order'] = request::$post['payment_wechat_pay_sort_order'];
		} else {
			$data['payment_wechat_pay_sort_order'] = config::get('payment_wechat_pay_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/payment/wechat_pay', $data));
	}

	private static function validate() {
		if (!user::hasPermission('modify', 'extension/payment/wechat_pay')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if (!request::$post['payment_wechat_pay_app_id']) {
			self::$error['app_id'] = language::get('error_app_id');
		}

		if (!request::$post['payment_wechat_pay_app_secret']) {
			self::$error['app_secret'] = language::get('error_app_secret');
		}

		if (!request::$post['payment_wechat_pay_mch_id']) {
			self::$error['mch_id'] = language::get('error_mch_id');
		}

		if (!request::$post['payment_wechat_pay_api_secret']) {
			self::$error['api_secret'] = language::get('error_api_secret');
		}

		return !self::$error;
	}
}
