<?php
class ControllerExtensionReportCustomerActivity {
	public static function index() {
		load::language('extension/report/customer_activity');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('report_customer_activity', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=report'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=report')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/report/customer_activity')
		);

		$data['action'] = url::link('extension/report/customer_activity');

		$data['cancel'] = url::link('marketplace/extension', '&type=report');

		if (isset(request::$post['report_customer_activity_status'])) {
			$data['report_customer_activity_status'] = request::$post['report_customer_activity_status'];
		} else {
			$data['report_customer_activity_status'] = config::get('report_customer_activity_status');
		}

		if (isset(request::$post['report_customer_activity_sort_order'])) {
			$data['report_customer_activity_sort_order'] = request::$post['report_customer_activity_sort_order'];
		} else {
			$data['report_customer_activity_sort_order'] = config::get('report_customer_activity_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/report/customer_activity_form', $data));
	}
		
	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/report/customer_activity')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
		
	public static function report() {
		load::language('extension/report/customer_activity');

		if (isset(request::$get['filter_customer'])) {
			$filter_customer = request::$get['filter_customer'];
		} else {
			$filter_customer = '';
		}

		if (isset(request::$get['filter_ip'])) {
			$filter_ip = request::$get['filter_ip'];
		} else {
			$filter_ip = '';
		}

		if (isset(request::$get['filter_date_start'])) {
			$filter_date_start = request::$get['filter_date_start'];
		} else {
			$filter_date_start = '';
		}

		if (isset(request::$get['filter_date_end'])) {
			$filter_date_end = request::$get['filter_date_end'];
		} else {
			$filter_date_end = '';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}


		$data['activities'] = array();

		$filter_data = array(
			'filter_customer'   => $filter_customer,
			'filter_ip'         => $filter_ip,
			'filter_date_start'	=> $filter_date_start,
			'filter_date_end'	=> $filter_date_end,
			'start'             => ($page - 1) * 20,
			'limit'             => 20
		);

		$activity_total = admin_extension_report_customer::getTotalCustomerActivities($filter_data);

		$results = admin_extension_report_customer::getCustomerActivities($filter_data);

		foreach ($results as $result) {
			$comment = vsprintf(language::get('text_activity_' . $result['key']), json_decode($result['data'], true));

			$find = array(
				'customer_id=',
				'order_id='
			);

			$replace = array(
				url::link('customer/customer/edit', '&customer_id='),
				url::link('sale/order/info', '&order_id=')
			);

			$data['activities'][] = array(
				'comment'    => str_replace($find, $replace, $comment),
				'ip'         => $result['ip'],
				'date_added' => date(language::get('datetime_format'), strtotime($result['date_added']))
			);
		}


		$url = '';

		if (isset(request::$get['filter_customer'])) {
			$url .= '&filter_customer=' . urlencode(request::$get['filter_customer']);
		}

		if (isset(request::$get['filter_ip'])) {
			$url .= '&filter_ip=' . request::$get['filter_ip'];
		}

		if (isset(request::$get['filter_date_start'])) {
			$url .= '&filter_date_start=' . request::$get['filter_date_start'];
		}

		if (isset(request::$get['filter_date_end'])) {
			$url .= '&filter_date_end=' . request::$get['filter_date_end'];
		}

		$pagination = new Pagination();
		$pagination::$total = $activity_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('report/report', '&code=customer_activity' . $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($activity_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($activity_total - config('limit_admin'))) ? $activity_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $activity_total, ceil($activity_total / config('limit_admin')));

		$data['filter_customer'] = $filter_customer;
		$data['filter_ip'] = $filter_ip;
		$data['filter_date_start'] = $filter_date_start;
		$data['filter_date_end'] = $filter_date_end;

		return load::view('extension/report/customer_activity_info', $data);
	}
}