<?php
class ControllerExtensionReportCustomerReward {
	public static function index() {
		load::language('extension/report/customer_reward');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('report_customer_reward', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=report'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=report')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/report/customer_reward')
		);

		$data['action'] = url::link('extension/report/customer_reward');

		$data['cancel'] = url::link('marketplace/extension', '&type=report');

		if (isset(request::$post['report_customer_reward_status'])) {
			$data['report_customer_reward_status'] = request::$post['report_customer_reward_status'];
		} else {
			$data['report_customer_reward_status'] = config::get('report_customer_reward_status');
		}

		if (isset(request::$post['report_customer_reward_sort_order'])) {
			$data['report_customer_reward_sort_order'] = request::$post['report_customer_reward_sort_order'];
		} else {
			$data['report_customer_reward_sort_order'] = config::get('report_customer_reward_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/report/customer_reward_form', $data));
	}
	
	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/report/customer_reward')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
		
	public static function report() {
		load::language('extension/report/customer_reward');

		if (isset(request::$get['filter_date_start'])) {
			$filter_date_start = request::$get['filter_date_start'];
		} else {
			$filter_date_start = '';
		}

		if (isset(request::$get['filter_date_end'])) {
			$filter_date_end = request::$get['filter_date_end'];
		} else {
			$filter_date_end = '';
		}

		if (isset(request::$get['filter_customer'])) {
			$filter_customer = request::$get['filter_customer'];
		} else {
			$filter_customer = '';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}


		$data['customers'] = array();

		$filter_data = array(
			'filter_date_start'	=> $filter_date_start,
			'filter_date_end'	=> $filter_date_end,
			'filter_customer'	=> $filter_customer,
			'start'				=> ($page - 1) * config('limit_admin'),
			'limit'				=> config('limit_admin')
		);

		$customer_total = admin_extension_report_customer::getTotalRewardPoints($filter_data);

		$results = admin_extension_report_customer::getRewardPoints($filter_data);

		foreach ($results as $result) {
			$data['customers'][] = array(
				'customer'       => $result['customer'],
				'email'          => $result['email'],
				'customer_group' => $result['customer_group'],
				'status'         => ($result['status'] ? language::get('text_enabled') : language::get('text_disabled')),
				'points'         => $result['points'],
				'orders'         => $result['orders'],
				'total'          => currency::format($result['total'], config('currency')),
				'edit'           => url::link('customer/customer/edit', '&customer_id=' . $result['customer_id'])
			);
		}


		$url = '';

		if (isset(request::$get['filter_date_start'])) {
			$url .= '&filter_date_start=' . request::$get['filter_date_start'];
		}

		if (isset(request::$get['filter_date_end'])) {
			$url .= '&filter_date_end=' . request::$get['filter_date_end'];
		}

		if (isset(request::$get['filter_customer'])) {
			$url .= '&filter_customer=' . urlencode(request::$get['filter_customer']);
		}

		$pagination = new Pagination();
		$pagination::$total = $customer_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('report/report', '&code=customer_reward' . $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($customer_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($customer_total - config('limit_admin'))) ? $customer_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $customer_total, ceil($customer_total / config('limit_admin')));

		$data['filter_date_start'] = $filter_date_start;
		$data['filter_date_end'] = $filter_date_end;
		$data['filter_customer'] = $filter_customer;
	
		return load::view('extension/report/customer_reward_info', $data);
	}
}