<?php
class ControllerExtensionReportCustomerSearch {
	public static function index() {
		load::language('extension/report/customer_search');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('report_customer_search', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=report'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=report')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/report/customer_search')
		);

		$data['action'] = url::link('extension/report/customer_search');

		$data['cancel'] = url::link('marketplace/extension', '&type=report');

		if (isset(request::$post['report_customer_search_status'])) {
			$data['report_customer_search_status'] = request::$post['report_customer_search_status'];
		} else {
			$data['report_customer_search_status'] = config::get('report_customer_search_status');
		}

		if (isset(request::$post['report_customer_search_sort_order'])) {
			$data['report_customer_search_sort_order'] = request::$post['report_customer_search_sort_order'];
		} else {
			$data['report_customer_search_sort_order'] = config::get('report_customer_search_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/report/customer_search_form', $data));
	}
	
	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/report/customer_search')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
	
	public static function report() {
		load::language('extension/report/customer_search');

		if (isset(request::$get['filter_date_start'])) {
			$filter_date_start = request::$get['filter_date_start'];
		} else {
			$filter_date_start = '';
		}

		if (isset(request::$get['filter_date_end'])) {
			$filter_date_end = request::$get['filter_date_end'];
		} else {
			$filter_date_end = '';
		}

		if (isset(request::$get['filter_keyword'])) {
			$filter_keyword = request::$get['filter_keyword'];
		} else {
			$filter_keyword = '';
		}

		if (isset(request::$get['filter_customer'])) {
			$filter_customer = request::$get['filter_customer'];
		} else {
			$filter_customer = '';
		}

		if (isset(request::$get['filter_ip'])) {
			$filter_ip = request::$get['filter_ip'];
		} else {
			$filter_ip = '';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}


		$data['searches'] = array();

		$filter_data = array(
			'filter_date_start' => $filter_date_start,
			'filter_date_end'   => $filter_date_end,
			'filter_keyword'    => $filter_keyword,
			'filter_customer'   => $filter_customer,
			'filter_ip'         => $filter_ip,
			'start'             => ($page - 1) * config('limit_admin'),
			'limit'             => config('limit_admin')
		);

		$search_total = admin_extension_report_customer::getTotalCustomerSearches($filter_data);

		$results = admin_extension_report_customer::getCustomerSearches($filter_data);

		foreach ($results as $result) {
			$category_info = admin_catalog_category::getCategory($result['category_id']);

			if ($category_info) {
				$category = ($category_info['path']) ? $category_info['path'] . ' &gt; ' . $category_info['name'] : $category_info['name'];
			} else {
				$category = '';
			}

			if ($result['customer_id'] > 0) {
				$customer = sprintf(language::get('text_customer'), url::link('customer/customer/edit', '&customer_id=' . $result['customer_id']), $result['customer']);
			} else {
				$customer = language::get('text_guest');
			}

			$data['searches'][] = array(
				'keyword'     => $result['keyword'],
				'products'    => $result['products'],
				'category'    => $category,
				'customer'    => $customer,
				'ip'          => $result['ip'],
				'date_added'  => date(language::get('datetime_format'), strtotime($result['date_added']))
			);
		}


		$url = '';

		if (isset(request::$get['filter_date_start'])) {
			$url .= '&filter_date_start=' . request::$get['filter_date_start'];
		}

		if (isset(request::$get['filter_date_end'])) {
			$url .= '&filter_date_end=' . request::$get['filter_date_end'];
		}

		if (isset(request::$get['filter_keyword'])) {
			$url .= '&filter_keyword=' . urlencode(request::$get['filter_keyword']);
		}

		if (isset(request::$get['filter_customer'])) {
			$url .= '&filter_customer=' . urlencode(request::$get['filter_customer']);
		}

		if (isset(request::$get['filter_ip'])) {
			$url .= '&filter_ip=' . request::$get['filter_ip'];
		}

		$pagination = new Pagination();
		$pagination::$total = $search_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('report/report', '&code=customer_search' . $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($search_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($search_total - config('limit_admin'))) ? $search_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $search_total, ceil($search_total / config('limit_admin')));

		$data['filter_date_start'] = $filter_date_start;
		$data['filter_date_end'] = $filter_date_end;
		$data['filter_keyword'] = $filter_keyword;
		$data['filter_customer'] = $filter_customer;
		$data['filter_ip'] = $filter_ip;

		return load::view('extension/report/customer_search_info', $data);
	}
}
