<?php
class ControllerExtensionReportMarketing {
	public static function index() {
		load::language('extension/report/marketing');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('report_marketing', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=report'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=report')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/report/marketing')
		);

		$data['action'] = url::link('extension/report/marketing');

		$data['cancel'] = url::link('marketplace/extension', '&type=report');

		if (isset(request::$post['report_marketing_status'])) {
			$data['report_marketing_status'] = request::$post['report_marketing_status'];
		} else {
			$data['report_marketing_status'] = config::get('report_marketing_status');
		}

		if (isset(request::$post['report_marketing_sort_order'])) {
			$data['report_marketing_sort_order'] = request::$post['report_marketing_sort_order'];
		} else {
			$data['report_marketing_sort_order'] = config::get('report_marketing_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/report/marketing_form', $data));
	}
	
	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/report/marketing')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
	
	public static function report() {
		load::language('extension/report/marketing');

		if (isset(request::$get['filter_date_start'])) {
			$filter_date_start = request::$get['filter_date_start'];
		} else {
			$filter_date_start = '';
		}

		if (isset(request::$get['filter_date_end'])) {
			$filter_date_end = request::$get['filter_date_end'];
		} else {
			$filter_date_end = '';
		}

		if (isset(request::$get['filter_order_status_id'])) {
			$filter_order_status_id = request::$get['filter_order_status_id'];
		} else {
			$filter_order_status_id = 0;
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}


		$data['marketings'] = array();

		$filter_data = array(
			'filter_date_start'	     => $filter_date_start,
			'filter_date_end'	     => $filter_date_end,
			'filter_order_status_id' => $filter_order_status_id,
			'start'                  => ($page - 1) * config('limit_admin'),
			'limit'                  => config('limit_admin')
		);

		$marketing_total = admin_extension_report_marketing::getTotalMarketing($filter_data);

		$results = admin_extension_report_marketing::getMarketing($filter_data);

		foreach ($results as $result) {
			$data['marketings'][] = array(
				'campaign' => $result['campaign'],
				'code'     => $result['code'],
				'clicks'   => $result['clicks'],
				'orders'   => $result['orders'],
				'total'    => currency::format($result['total'], config('currency')),
				'action'   => url::link('marketing/marketing/edit', '&marketing_id=' . $result['marketing_id'])
			);
		}



		$data['order_statuses'] = admin_localisation_order_status::getOrderStatuses();

		$url = '';

		if (isset(request::$get['filter_date_start'])) {
			$url .= '&filter_date_start=' . request::$get['filter_date_start'];
		}

		if (isset(request::$get['filter_date_end'])) {
			$url .= '&filter_date_end=' . request::$get['filter_date_end'];
		}

		if (isset(request::$get['filter_order_status_id'])) {
			$url .= '&filter_order_status_id=' . request::$get['filter_order_status_id'];
		}

		$pagination = new Pagination();
		$pagination::$total = $marketing_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('report/report', '&code=marketing' . $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($marketing_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($marketing_total - config('limit_admin'))) ? $marketing_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $marketing_total, ceil($marketing_total / config('limit_admin')));

		$data['filter_date_start'] = $filter_date_start;
		$data['filter_date_end'] = $filter_date_end;
		$data['filter_order_status_id'] = $filter_order_status_id;

		return load::view('extension/report/marketing_info', $data);
	}
}