<?php
class ControllerExtensionReportProductViewed {
	public static function index() {
		load::language('extension/report/product_viewed');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('report_product_viewed', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=report'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=report')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/report/product_viewed')
		);

		$data['action'] = url::link('extension/report/product_viewed');

		$data['cancel'] = url::link('marketplace/extension', '&type=report');

		if (isset(request::$post['report_product_viewed_status'])) {
			$data['report_product_viewed_status'] = request::$post['report_product_viewed_status'];
		} else {
			$data['report_product_viewed_status'] = config::get('report_product_viewed_status');
		}

		if (isset(request::$post['report_product_viewed_sort_order'])) {
			$data['report_product_viewed_sort_order'] = request::$post['report_product_viewed_sort_order'];
		} else {
			$data['report_product_viewed_sort_order'] = config::get('report_product_viewed_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/report/product_viewed_form', $data));
	}
	
	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/report/product_viewed')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
		
	public static function report() {
		load::language('extension/report/product_viewed');

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}
		
		$data['reset'] = url::link('extension/report/product_viewed/reset');


		$filter_data = array(
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$data['products'] = array();

		$product_viewed_total = admin_extension_report_product::getTotalProductViews();

		$product_total = admin_extension_report_product::getTotalProductsViewed();

		$results = admin_extension_report_product::getProductsViewed($filter_data);

		foreach ($results as $result) {
			if ($result['viewed']) {
				$percent = round($result['viewed'] / $product_viewed_total * 100, 2);
			} else {
				$percent = 0;
			}

			$data['products'][] = array(
				'name'    => $result['name'],
				'model'   => $result['model'],
				'viewed'  => $result['viewed'],
				'percent' => $percent . '%'
			);
		}
		

		$url = '';

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$pagination = new Pagination();
		$pagination::$total = $product_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('report/report', '&code=product_viewed&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($product_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($product_total - config('limit_admin'))) ? $product_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $product_total, ceil($product_total / config('limit_admin')));
		
		return load::view('extension/report/product_viewed_info', $data);
	}

	public static function reset() {
		load::language('extension/report/product_viewed');

		if (!user::hasPermission('modify', 'extension/report/product_viewed')) {
			session::$data['error'] = language::get('error_permission');
		} else {

			admin_extension_report_product::reset();

			session::$data['success'] = language::get('text_success');
		}

		response::redirect(url::link('report/report', '&code=product_viewed'));
	}
}