<?php
class ControllerExtensionReportSaleCoupon {
	public static function index() {
		load::language('extension/report/sale_coupon');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('report_sale_coupon', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=report'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=report')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/report/sale_coupon')
		);

		$data['action'] = url::link('extension/report/sale_coupon');

		$data['cancel'] = url::link('marketplace/extension', '&type=report');

		if (isset(request::$post['report_sale_coupon_status'])) {
			$data['report_sale_coupon_status'] = request::$post['report_sale_coupon_status'];
		} else {
			$data['report_sale_coupon_status'] = config::get('report_sale_coupon_status');
		}

		if (isset(request::$post['report_sale_coupon_sort_order'])) {
			$data['report_sale_coupon_sort_order'] = request::$post['report_sale_coupon_sort_order'];
		} else {
			$data['report_sale_coupon_sort_order'] = config::get('report_sale_coupon_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/report/sale_coupon_form', $data));
	}
	
	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/report/sale_coupon')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
		
	public static function report() {
		load::language('extension/report/sale_coupon');

		if (isset(request::$get['filter_date_start'])) {
			$filter_date_start = request::$get['filter_date_start'];
		} else {
			$filter_date_start = '';
		}

		if (isset(request::$get['filter_date_end'])) {
			$filter_date_end = request::$get['filter_date_end'];
		} else {
			$filter_date_end = '';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}


		$data['coupons'] = array();

		$filter_data = array(
			'filter_date_start'	=> $filter_date_start,
			'filter_date_end'	=> $filter_date_end,
			'start'             => ($page - 1) * config('limit_admin'),
			'limit'             => config('limit_admin')
		);

		$coupon_total = admin_extension_report_coupon::getTotalCoupons($filter_data);

		$results = admin_extension_report_coupon::getCoupons($filter_data);

		foreach ($results as $result) {
			$data['coupons'][] = array(
				'name'   => $result['name'],
				'code'   => $result['code'],
				'orders' => $result['orders'],
				'total'  => currency::format($result['total'], config('currency')),
				'edit'   => url::link('marketing/coupon/edit', '&coupon_id=' . $result['coupon_id'])
			);
		}


		$url = '';

		if (isset(request::$get['filter_date_start'])) {
			$url .= '&filter_date_start=' . request::$get['filter_date_start'];
		}

		if (isset(request::$get['filter_date_end'])) {
			$url .= '&filter_date_end=' . request::$get['filter_date_end'];
		}

		$pagination = new Pagination();
		$pagination::$total = $coupon_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('report/report', '&code=sale_coupon' . $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($coupon_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($coupon_total - config('limit_admin'))) ? $coupon_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $coupon_total, ceil($coupon_total / config('limit_admin')));

		$data['filter_date_start'] = $filter_date_start;
		$data['filter_date_end'] = $filter_date_end;

		return load::view('extension/report/sale_coupon_info', $data);
	}
}