<?php
class ControllerExtensionReportSaleOrder {
	public static function index() {
		load::language('extension/report/sale_order');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('report_sale_order', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=report'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=report')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/report/sale_order')
		);

		$data['action'] = url::link('extension/report/sale_order');

		$data['cancel'] = url::link('marketplace/extension', '&type=report');

		if (isset(request::$post['report_sale_order_status'])) {
			$data['report_sale_order_status'] = request::$post['report_sale_order_status'];
		} else {
			$data['report_sale_order_status'] = config::get('report_sale_order_status');
		}

		if (isset(request::$post['report_sale_order_sort_order'])) {
			$data['report_sale_order_sort_order'] = request::$post['report_sale_order_sort_order'];
		} else {
			$data['report_sale_order_sort_order'] = config::get('report_sale_order_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/report/sale_order_form', $data));
	}
	
	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/report/sale_order')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
		
	public static function report() {
		load::language('extension/report/sale_order');

		if (isset(request::$get['filter_date_start'])) {
			$filter_date_start = request::$get['filter_date_start'];
		} else {
			$filter_date_start = date('Y-m-d', strtotime(date('Y') . '-' . date('m') . '-01'));
		}

		if (isset(request::$get['filter_date_end'])) {
			$filter_date_end = request::$get['filter_date_end'];
		} else {
			$filter_date_end = date('Y-m-d');
		}

		if (isset(request::$get['filter_group'])) {
			$filter_group = request::$get['filter_group'];
		} else {
			$filter_group = 'week';
		}

		if (isset(request::$get['filter_order_status_id'])) {
			$filter_order_status_id = request::$get['filter_order_status_id'];
		} else {
			$filter_order_status_id = 0;
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}


		$data['orders'] = array();

		$filter_data = array(
			'filter_date_start'	     => $filter_date_start,
			'filter_date_end'	     => $filter_date_end,
			'filter_group'           => $filter_group,
			'filter_order_status_id' => $filter_order_status_id,
			'start'                  => ($page - 1) * config('limit_admin'),
			'limit'                  => config('limit_admin')
		);

		$order_total = admin_extension_report_sale::getTotalOrders($filter_data);

		$results = admin_extension_report_sale::getOrders($filter_data);

		foreach ($results as $result) {
			$data['orders'][] = array(
				'date_start' => date(language::get('date_format_short'), strtotime($result['date_start'])),
				'date_end'   => date(language::get('date_format_short'), strtotime($result['date_end'])),
				'orders'     => $result['orders'],
				'products'   => $result['products'],
				'tax'        => currency::format($result['tax'], config('currency')),
				'total'      => currency::format($result['total'], config('currency'))
			);
		}



		$data['order_statuses'] = admin_localisation_order_status::getOrderStatuses();

		$data['groups'] = array();

		$data['groups'][] = array(
			'text'  => language::get('text_year'),
			'value' => 'year',
		);

		$data['groups'][] = array(
			'text'  => language::get('text_month'),
			'value' => 'month',
		);

		$data['groups'][] = array(
			'text'  => language::get('text_week'),
			'value' => 'week',
		);

		$data['groups'][] = array(
			'text'  => language::get('text_day'),
			'value' => 'day',
		);

		$url = '';

		if (isset(request::$get['filter_date_start'])) {
			$url .= '&filter_date_start=' . request::$get['filter_date_start'];
		}

		if (isset(request::$get['filter_date_end'])) {
			$url .= '&filter_date_end=' . request::$get['filter_date_end'];
		}

		if (isset(request::$get['filter_group'])) {
			$url .= '&filter_group=' . request::$get['filter_group'];
		}

		if (isset(request::$get['filter_order_status_id'])) {
			$url .= '&filter_order_status_id=' . request::$get['filter_order_status_id'];
		}

		$pagination = new Pagination();
		$pagination::$total = $order_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('report/report', '&code=sale_order' . $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($order_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($order_total - config('limit_admin'))) ? $order_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $order_total, ceil($order_total / config('limit_admin')));

		$data['filter_date_start'] = $filter_date_start;
		$data['filter_date_end'] = $filter_date_end;
		$data['filter_group'] = $filter_group;
		$data['filter_order_status_id'] = $filter_order_status_id;

		return load::view('extension/report/sale_order_info', $data);
	}
}