<?php
class ControllerExtensionReportSaleReturn {
	public static function index() {
		load::language('extension/report/sale_return');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			admin_setting_setting::editSetting('report_sale_return', request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('marketplace/extension', '&type=report'));
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_extension'),
			'href' => url::link('marketplace/extension', '&type=report')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('extension/report/sale_return')
		);

		$data['action'] = url::link('extension/report/sale_return');

		$data['cancel'] = url::link('marketplace/extension', '&type=report');

		if (isset(request::$post['report_sale_return_status'])) {
			$data['report_sale_return_status'] = request::$post['report_sale_return_status'];
		} else {
			$data['report_sale_return_status'] = config::get('report_sale_return_status');
		}

		if (isset(request::$post['report_sale_return_sort_order'])) {
			$data['report_sale_return_sort_order'] = request::$post['report_sale_return_sort_order'];
		} else {
			$data['report_sale_return_sort_order'] = config::get('report_sale_return_sort_order');
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('extension/report/sale_return_form', $data));
	}
	
	protected static function validate() {
		if (!user::hasPermission('modify', 'extension/report/sale_return')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
	
	public static function report() {
		load::language('extension/report/sale_return');

		if (isset(request::$get['filter_date_start'])) {
			$filter_date_start = request::$get['filter_date_start'];
		} else {
			$filter_date_start = '';
		}

		if (isset(request::$get['filter_date_end'])) {
			$filter_date_end = request::$get['filter_date_end'];
		} else {
			$filter_date_end = '';
		}

		if (isset(request::$get['filter_group'])) {
			$filter_group = request::$get['filter_group'];
		} else {
			$filter_group = 'week';
		}

		if (isset(request::$get['filter_return_status_id'])) {
			$filter_return_status_id = request::$get['filter_return_status_id'];
		} else {
			$filter_return_status_id = 0;
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}
		

		$data['returns'] = array();

		$filter_data = array(
			'filter_date_start'	      => $filter_date_start,
			'filter_date_end'	      => $filter_date_end,
			'filter_group'            => $filter_group,
			'filter_return_status_id' => $filter_return_status_id,
			'start'                   => ($page - 1) * config('limit_admin'),
			'limit'                   => config('limit_admin')
		);

		$return_total = admin_extension_report_return::getTotalReturns($filter_data);

		$results = admin_extension_report_return::getReturns($filter_data);

		foreach ($results as $result) {
			$data['returns'][] = array(
				'date_start' => date(language::get('date_format_short'), strtotime($result['date_start'])),
				'date_end'   => date(language::get('date_format_short'), strtotime($result['date_end'])),
				'returns'    => $result['returns']
			);
		}



		$data['return_statuses'] = admin_localisation_return_status::getReturnStatuses();

		$data['groups'] = array();

		$data['groups'][] = array(
			'text'  => language::get('text_year'),
			'value' => 'year',
		);

		$data['groups'][] = array(
			'text'  => language::get('text_month'),
			'value' => 'month',
		);

		$data['groups'][] = array(
			'text'  => language::get('text_week'),
			'value' => 'week',
		);

		$data['groups'][] = array(
			'text'  => language::get('text_day'),
			'value' => 'day',
		);

		$url = '';

		if (isset(request::$get['filter_date_start'])) {
			$url .= '&filter_date_start=' . request::$get['filter_date_start'];
		}

		if (isset(request::$get['filter_date_end'])) {
			$url .= '&filter_date_end=' . request::$get['filter_date_end'];
		}

		if (isset(request::$get['filter_group'])) {
			$url .= '&filter_group=' . request::$get['filter_group'];
		}

		if (isset(request::$get['filter_return_status_id'])) {
			$url .= '&filter_return_status_id=' . request::$get['filter_return_status_id'];
		}

		$pagination = new Pagination();
		$pagination::$total = $return_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('report/report', '&code=sale_return' . $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($return_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($return_total - config('limit_admin'))) ? $return_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $return_total, ceil($return_total / config('limit_admin')));

		$data['filter_date_start'] = $filter_date_start;
		$data['filter_date_end'] = $filter_date_end;
		$data['filter_group'] = $filter_group;
		$data['filter_return_status_id'] = $filter_return_status_id;
		
		return load::view('extension/report/sale_return_info', $data);
	}
}