<?php
class ControllerLocalisationCountry {
	private static $error = array();

	public static function index() {
		load::language('localisation/country');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('localisation/country');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_country::addCountry(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/country', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('localisation/country');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_country::editCountry(request::$get['country_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/country', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('localisation/country');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $country_id) {
				admin_localisation_country::deleteCountry($country_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/country', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/country', $url)
		);

		$data['add'] = url::link('localisation/country/add', $url);
		$data['delete'] = url::link('localisation/country/delete', $url);

		$data['countries'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$country_total = admin_localisation_country::getTotalCountries();

		$results = admin_localisation_country::getCountries($filter_data);

		foreach ($results as $result) {
			$data['countries'][] = array(
				'country_id' => $result['country_id'],
				'name'       => $result['name'] . (($result['country_id'] == config('country_id')) ? language::get('text_default') : null),
				'iso_code_2' => $result['iso_code_2'],
				'iso_code_3' => $result['iso_code_3'],
				'edit'       => url::link('localisation/country/edit', '&country_id=' . $result['country_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('localisation/country', '&sort=name' . $url);
		$data['sort_iso_code_2'] = url::link('localisation/country', '&sort=iso_code_2' . $url);
		$data['sort_iso_code_3'] = url::link('localisation/country', '&sort=iso_code_3' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $country_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('localisation/country', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($country_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($country_total - config('limit_admin'))) ? $country_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $country_total, ceil($country_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/country_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['country_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/country', $url)
		);

		if (!isset(request::$get['country_id'])) {
			$data['action'] = url::link('localisation/country/add', $url);
		} else {
			$data['action'] = url::link('localisation/country/edit', '&country_id=' . request::$get['country_id'] . $url);
		}

		$data['cancel'] = url::link('localisation/country', $url);

		if (isset(request::$get['country_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$country_info = admin_localisation_country::getCountry(request::$get['country_id']);
		}

		if (isset(request::$post['name'])) {
			$data['name'] = request::$post['name'];
		} elseif (!empty($country_info)) {
			$data['name'] = $country_info['name'];
		} else {
			$data['name'] = '';
		}

		if (isset(request::$post['iso_code_2'])) {
			$data['iso_code_2'] = request::$post['iso_code_2'];
		} elseif (!empty($country_info)) {
			$data['iso_code_2'] = $country_info['iso_code_2'];
		} else {
			$data['iso_code_2'] = '';
		}

		if (isset(request::$post['iso_code_3'])) {
			$data['iso_code_3'] = request::$post['iso_code_3'];
		} elseif (!empty($country_info)) {
			$data['iso_code_3'] = $country_info['iso_code_3'];
		} else {
			$data['iso_code_3'] = '';
		}

		if (isset(request::$post['address_format'])) {
			$data['address_format'] = request::$post['address_format'];
		} elseif (!empty($country_info)) {
			$data['address_format'] = $country_info['address_format'];
		} else {
			$data['address_format'] = '';
		}

		if (isset(request::$post['postcode_required'])) {
			$data['postcode_required'] = request::$post['postcode_required'];
		} elseif (!empty($country_info)) {
			$data['postcode_required'] = $country_info['postcode_required'];
		} else {
			$data['postcode_required'] = 0;
		}

		if (isset(request::$post['status'])) {
			$data['status'] = request::$post['status'];
		} elseif (!empty($country_info)) {
			$data['status'] = $country_info['status'];
		} else {
			$data['status'] = '1';
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/country_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'localisation/country')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(request::$post['name']) < 1) || (utf8_strlen(request::$post['name']) > 128)) {
			self::$error['name'] = language::get('error_name');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'localisation/country')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $country_id) {
			if (config('country_id') == $country_id) {
				self::$error['warning'] = language::get('error_default');
			}

			$store_total = admin_setting_store::getTotalStoresByCountryId($country_id);

			if ($store_total) {
				self::$error['warning'] = sprintf(language::get('error_store'), $store_total);
			}

			$address_total = admin_customer_customer::getTotalAddressesByCountryId($country_id);

			if ($address_total) {
				self::$error['warning'] = sprintf(language::get('error_address'), $address_total);
			}

			$zone_total = admin_localisation_zone::getTotalZonesByCountryId($country_id);

			if ($zone_total) {
				self::$error['warning'] = sprintf(language::get('error_zone'), $zone_total);
			}

			$zone_to_geo_zone_total = admin_localisation_geo_zone::getTotalZoneToGeoZoneByCountryId($country_id);

			if ($zone_to_geo_zone_total) {
				self::$error['warning'] = sprintf(language::get('error_zone_to_geo_zone'), $zone_to_geo_zone_total);
			}
		}

		return !self::$error;
	}
	
	public static function country() {
		$json = array();


		$country_info = admin_localisation_country::getCountry(request::$get['country_id']);

		if ($country_info) {

			$json = array(
				'country_id'        => $country_info['country_id'],
				'name'              => $country_info['name'],
				'iso_code_2'        => $country_info['iso_code_2'],
				'iso_code_3'        => $country_info['iso_code_3'],
				'address_format'    => $country_info['address_format'],
				'postcode_required' => $country_info['postcode_required'],
				'zone'              => admin_localisation_zone::getZonesByCountryId(request::$get['country_id']),
				'status'            => $country_info['status']
			);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}	
}