<?php
class ControllerLocalisationCurrency {
	private static $error = array();

	public static function index() {
		load::language('localisation/currency');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('localisation/currency');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_currency::addCurrency(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/currency', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('localisation/currency');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_currency::editCurrency(request::$get['currency_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/currency', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('localisation/currency');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $currency_id) {
				admin_localisation_currency::deleteCurrency($currency_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/currency', $url));
		}

		self::getList();
	}

	public static function refresh() {
		load::language('localisation/currency');

		document::setTitle(language::get('heading_title'));


		if (self::validateRefresh()) {
			admin_localisation_currency::refresh(true);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			//response::redirect(url::link('localisation/currency', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'title';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/currency', $url)
		);

		$data['add'] = url::link('localisation/currency/add', $url);
		$data['delete'] = url::link('localisation/currency/delete', $url);
		$data['refresh'] = url::link('localisation/currency/refresh', $url);

		$data['currencies'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$currency_total = admin_localisation_currency::getTotalCurrencies();

		$results = admin_localisation_currency::getCurrencies($filter_data);

		foreach ($results as $result) {
			$data['currencies'][] = array(
				'currency_id'   => $result['currency_id'],
				'title'         => $result['title'] . (($result['code'] == config('currency')) ? language::get('text_default') : null),
				'code'          => $result['code'],
				'value'         => $result['value'],
				'date_modified' => date(language::get('date_format_short'), strtotime($result['date_modified'])),
				'edit'          => url::link('localisation/currency/edit', '&currency_id=' . $result['currency_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_title'] = url::link('localisation/currency', '&sort=title' . $url);
		$data['sort_code'] = url::link('localisation/currency', '&sort=code' . $url);
		$data['sort_value'] = url::link('localisation/currency', '&sort=value' . $url);
		$data['sort_date_modified'] = url::link('localisation/currency', '&sort=date_modified' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $currency_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('localisation/currency', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($currency_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($currency_total - config('limit_admin'))) ? $currency_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $currency_total, ceil($currency_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/currency_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['currency_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['title'])) {
			$data['error_title'] = self::$error['title'];
		} else {
			$data['error_title'] = '';
		}

		if (isset(self::$error['code'])) {
			$data['error_code'] = self::$error['code'];
		} else {
			$data['error_code'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/currency', $url)
		);

		if (!isset(request::$get['currency_id'])) {
			$data['action'] = url::link('localisation/currency/add', $url);
		} else {
			$data['action'] = url::link('localisation/currency/edit', '&currency_id=' . request::$get['currency_id'] . $url);
		}

		$data['cancel'] = url::link('localisation/currency', $url);

		if (isset(request::$get['currency_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$currency_info = admin_localisation_currency::getCurrency(request::$get['currency_id']);
		}

		if (isset(request::$post['title'])) {
			$data['title'] = request::$post['title'];
		} elseif (!empty($currency_info)) {
			$data['title'] = $currency_info['title'];
		} else {
			$data['title'] = '';
		}

		if (isset(request::$post['code'])) {
			$data['code'] = request::$post['code'];
		} elseif (!empty($currency_info)) {
			$data['code'] = $currency_info['code'];
		} else {
			$data['code'] = '';
		}

		if (isset(request::$post['symbol_left'])) {
			$data['symbol_left'] = request::$post['symbol_left'];
		} elseif (!empty($currency_info)) {
			$data['symbol_left'] = $currency_info['symbol_left'];
		} else {
			$data['symbol_left'] = '';
		}

		if (isset(request::$post['symbol_right'])) {
			$data['symbol_right'] = request::$post['symbol_right'];
		} elseif (!empty($currency_info)) {
			$data['symbol_right'] = $currency_info['symbol_right'];
		} else {
			$data['symbol_right'] = '';
		}

		if (isset(request::$post['decimal_place'])) {
			$data['decimal_place'] = request::$post['decimal_place'];
		} elseif (!empty($currency_info)) {
			$data['decimal_place'] = $currency_info['decimal_place'];
		} else {
			$data['decimal_place'] = '';
		}

		if (isset(request::$post['value'])) {
			$data['value'] = request::$post['value'];
		} elseif (!empty($currency_info)) {
			$data['value'] = $currency_info['value'];
		} else {
			$data['value'] = '';
		}

		if (isset(request::$post['status'])) {
			$data['status'] = request::$post['status'];
		} elseif (!empty($currency_info)) {
			$data['status'] = $currency_info['status'];
		} else {
			$data['status'] = '';
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/currency_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'localisation/currency')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(request::$post['title']) < 3) || (utf8_strlen(request::$post['title']) > 32)) {
			self::$error['title'] = language::get('error_title');
		}

		if (utf8_strlen(request::$post['code']) != 3) {
			self::$error['code'] = language::get('error_code');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'localisation/currency')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $currency_id) {
			$currency_info = admin_localisation_currency::getCurrency($currency_id);

			if ($currency_info) {
				if (config('currency') == $currency_info['code']) {
					self::$error['warning'] = language::get('error_default');
				}

				$store_total = admin_setting_store::getTotalStoresByCurrency($currency_info['code']);

				if ($store_total) {
					self::$error['warning'] = sprintf(language::get('error_store'), $store_total);
				}
			}

			$order_total = admin_sale_order::getTotalOrdersByCurrencyId($currency_id);

			if ($order_total) {
				self::$error['warning'] = sprintf(language::get('error_order'), $order_total);
			}
		}

		return !self::$error;
	}

	protected static function validateRefresh() {
		if (!user::hasPermission('modify', 'localisation/currency')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
}