<?php
class ControllerLocalisationGeoZone {
	private static $error = array();

	public static function index() {
		load::language('localisation/geo_zone');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('localisation/geo_zone');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_geo_zone::addGeoZone(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/geo_zone', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('localisation/geo_zone');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_geo_zone::editGeoZone(request::$get['geo_zone_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/geo_zone', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('localisation/geo_zone');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $geo_zone_id) {
				admin_localisation_geo_zone::deleteGeoZone($geo_zone_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/geo_zone', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/geo_zone', $url)
		);

		$data['add'] = url::link('localisation/geo_zone/add', $url);
		$data['delete'] = url::link('localisation/geo_zone/delete', $url);

		$data['geo_zones'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$geo_zone_total = admin_localisation_geo_zone::getTotalGeoZones();

		$results = admin_localisation_geo_zone::getGeoZones($filter_data);

		foreach ($results as $result) {
			$data['geo_zones'][] = array(
				'geo_zone_id' => $result['geo_zone_id'],
				'name'        => $result['name'],
				'description' => $result['description'],
				'edit'        => url::link('localisation/geo_zone/edit', '&geo_zone_id=' . $result['geo_zone_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('localisation/geo_zone', '&sort=name' . $url);
		$data['sort_description'] = url::link('localisation/geo_zone', '&sort=description' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $geo_zone_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('localisation/geo_zone', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($geo_zone_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($geo_zone_total - config('limit_admin'))) ? $geo_zone_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $geo_zone_total, ceil($geo_zone_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/geo_zone_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['geo_zone_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = '';
		}

		if (isset(self::$error['description'])) {
			$data['error_description'] = self::$error['description'];
		} else {
			$data['error_description'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/geo_zone', $url)
		);

		if (!isset(request::$get['geo_zone_id'])) {
			$data['action'] = url::link('localisation/geo_zone/add', $url);
		} else {
			$data['action'] = url::link('localisation/geo_zone/edit', '&geo_zone_id=' . request::$get['geo_zone_id'] . $url);
		}

		$data['cancel'] = url::link('localisation/geo_zone', $url);

		if (isset(request::$get['geo_zone_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$geo_zone_info = admin_localisation_geo_zone::getGeoZone(request::$get['geo_zone_id']);
		}


		if (isset(request::$post['name'])) {
			$data['name'] = request::$post['name'];
		} elseif (!empty($geo_zone_info)) {
			$data['name'] = $geo_zone_info['name'];
		} else {
			$data['name'] = '';
		}

		if (isset(request::$post['description'])) {
			$data['description'] = request::$post['description'];
		} elseif (!empty($geo_zone_info)) {
			$data['description'] = $geo_zone_info['description'];
		} else {
			$data['description'] = '';
		}


		$data['countries'] = admin_localisation_country::getCountries();

		if (isset(request::$post['zone_to_geo_zone'])) {
			$data['zone_to_geo_zones'] = request::$post['zone_to_geo_zone'];
		} elseif (isset(request::$get['geo_zone_id'])) {
			$data['zone_to_geo_zones'] = admin_localisation_geo_zone::getZoneToGeoZones(request::$get['geo_zone_id']);
		} else {
			$data['zone_to_geo_zones'] = array();
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/geo_zone_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'localisation/geo_zone')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(request::$post['name']) < 3) || (utf8_strlen(request::$post['name']) > 32)) {
			self::$error['name'] = language::get('error_name');
		}

		if ((utf8_strlen(request::$post['description']) < 3) || (utf8_strlen(request::$post['description']) > 255)) {
			self::$error['description'] = language::get('error_description');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'localisation/geo_zone')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $geo_zone_id) {
			$tax_rate_total = admin_localisation_tax_rate::getTotalTaxRatesByGeoZoneId($geo_zone_id);

			if ($tax_rate_total) {
				self::$error['warning'] = sprintf(language::get('error_tax_rate'), $tax_rate_total);
			}
		}

		return !self::$error;
	}
}