<?php
class ControllerLocalisationLanguage {
	private static $error = array();

	public static function index() {
		load::language('localisation/language');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('localisation/language');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_language::addLanguage(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/language', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('localisation/language');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_language::editLanguage(request::$get['language_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/language', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('localisation/language');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $language_id) {
				admin_localisation_language::deleteLanguage($language_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/language', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/language', $url)
		);

		$data['add'] = url::link('localisation/language/add', $url);
		$data['delete'] = url::link('localisation/language/delete', $url);

		$data['languages'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$language_total = admin_localisation_language::getTotalLanguages();

		$results = admin_localisation_language::getLanguages($filter_data);

		foreach ($results as $result) {
			$data['languages'][] = array(
				'language_id' => $result['language_id'],
				'name'        => $result['name'] . (($result['code'] == config('language')) ? language::get('text_default') : null),
				'code'        => $result['code'],
				'sort_order'  => $result['sort_order'],
				'edit'        => url::link('localisation/language/edit', '&language_id=' . $result['language_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('localisation/language', '&sort=name' . $url);
		$data['sort_code'] = url::link('localisation/language', '&sort=code' . $url);
		$data['sort_sort_order'] = url::link('localisation/language', '&sort=sort_order' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $language_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('localisation/language', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($language_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($language_total - config('limit_admin'))) ? $language_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $language_total, ceil($language_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;
		
		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/language_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['language_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = '';
		}

		if (isset(self::$error['code'])) {
			$data['error_code'] = self::$error['code'];
		} else {
			$data['error_code'] = '';
		}
		
		if (isset(self::$error['locale'])) {
			$data['error_locale'] = self::$error['locale'];
		} else {
			$data['error_locale'] = '';
		}
		
		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/language', $url)
		);

		if (!isset(request::$get['language_id'])) {
			$data['action'] = url::link('localisation/language/add', $url);
		} else {
			$data['action'] = url::link('localisation/language/edit', '&language_id=' . request::$get['language_id'] . $url);
		}

		$data['cancel'] = url::link('localisation/language', $url);

		if (isset(request::$get['language_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$language_info = admin_localisation_language::getLanguage(request::$get['language_id']);
		}

		if (isset(request::$post['name'])) {
			$data['name'] = request::$post['name'];
		} elseif (!empty($language_info)) {
			$data['name'] = $language_info['name'];
		} else {
			$data['name'] = '';
		}

		if (isset(request::$post['code'])) {
			$data['code'] = request::$post['code'];
		} elseif (!empty($language_info)) {
			$data['code'] = $language_info['code'];
		} else {
			$data['code'] = '';
		}
		
		$data['languages'] = array();
		
		$folders = glob(DIR_LANGUAGE . '*', GLOB_ONLYDIR);

		foreach ($folders as $folder) {
			$data['languages'][] = basename($folder);
		}

		if (isset(request::$post['locale'])) {
			$data['locale'] = request::$post['locale'];
		} elseif (!empty($language_info)) {
			$data['locale'] = $language_info['locale'];
		} else {
			$data['locale'] = '';
		}
		
		if (isset(request::$post['sort_order'])) {
			$data['sort_order'] = request::$post['sort_order'];
		} elseif (!empty($language_info)) {
			$data['sort_order'] = $language_info['sort_order'];
		} else {
			$data['sort_order'] = 1;
		}

		if (isset(request::$post['status'])) {
			$data['status'] = request::$post['status'];
		} elseif (!empty($language_info)) {
			$data['status'] = $language_info['status'];
		} else {
			$data['status'] = true;
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/language_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'localisation/language')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(request::$post['name']) < 3) || (utf8_strlen(request::$post['name']) > 32)) {
			self::$error['name'] = language::get('error_name');
		}

		if (utf8_strlen(request::$post['code']) < 2) {
			self::$error['code'] = language::get('error_code');
		}
		
		if (!request::$post['locale']) {
			self::$error['locale'] = language::get('error_locale');
		}
		
		$language_info = admin_localisation_language::getLanguageByCode(request::$post['code']);

		if (!isset(request::$get['language_id'])) {
			if ($language_info) {
				self::$error['warning'] = language::get('error_exists');
			}
		} else {
			if ($language_info && (request::$get['language_id'] != $language_info['language_id'])) {
				self::$error['warning'] = language::get('error_exists');
			}
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'localisation/language')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $language_id) {
			$language_info = admin_localisation_language::getLanguage($language_id);

			if ($language_info) {
				if (config('language') == $language_info['code']) {
					self::$error['warning'] = language::get('error_default');
				}

				if (config('admin_language') == $language_info['code']) {
					self::$error['warning'] = language::get('error_admin');
				}

				$store_total = admin_setting_store::getTotalStoresByLanguage($language_info['code']);

				if ($store_total) {
					self::$error['warning'] = sprintf(language::get('error_store'), $store_total);
				}
			}

			$order_total = admin_sale_order::getTotalOrdersByLanguageId($language_id);

			if ($order_total) {
				self::$error['warning'] = sprintf(language::get('error_order'), $order_total);
			}
		}

		return !self::$error;
	}
}
