<?php
class ControllerLocalisationReturnStatus {
	private static $error = array();

	public static function index() {
		load::language('localisation/return_status');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('localisation/return_status');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_return_status::addReturnStatus(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/return_status', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('localisation/return_status');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_return_status::editReturnStatus(request::$get['return_status_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/return_status', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('localisation/return_status');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $return_status_id) {
				admin_localisation_return_status::deleteReturnStatus($return_status_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/return_status', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/return_status', $url)
		);

		$data['add'] = url::link('localisation/return_status/add', $url);
		$data['delete'] = url::link('localisation/return_status/delete', $url);

		$data['return_statuses'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$return_status_total = admin_localisation_return_status::getTotalReturnStatuses();

		$results = admin_localisation_return_status::getReturnStatuses($filter_data);

		foreach ($results as $result) {
			$data['return_statuses'][] = array(
				'return_status_id' => $result['return_status_id'],
				'name'             => $result['name'] . (($result['return_status_id'] == config('return_status_id')) ? language::get('text_default') : null),
				'edit'             => url::link('localisation/return_status/edit', '&return_status_id=' . $result['return_status_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('localisation/return_status', '&sort=name' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $return_status_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('localisation/return_status', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($return_status_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($return_status_total - config('limit_admin'))) ? $return_status_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $return_status_total, ceil($return_status_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/return_status_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['return_status_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = array();
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/return_status', $url)
		);

		if (!isset(request::$get['return_status_id'])) {
			$data['action'] = url::link('localisation/return_status/add', $url);
		} else {
			$data['action'] = url::link('localisation/return_status/edit', '&return_status_id=' . request::$get['return_status_id'] . $url);
		}

		$data['cancel'] = url::link('localisation/return_status', $url);


		$data['languages'] = admin_localisation_language::getLanguages();

		if (isset(request::$post['return_status'])) {
			$data['return_status'] = request::$post['return_status'];
		} elseif (isset(request::$get['return_status_id'])) {
			$data['return_status'] = admin_localisation_return_status::getReturnStatusDescriptions(request::$get['return_status_id']);
		} else {
			$data['return_status'] = array();
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/return_status_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'localisation/return_status')) {
			self::$error['warning'] = language::get('error_permission');
		}

		foreach (request::$post['return_status'] as $language_id => $value) {
			if ((utf8_strlen($value['name']) < 3) || (utf8_strlen($value['name']) > 32)) {
				self::$error['name'][$language_id] = language::get('error_name');
			}
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'localisation/return_status')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $return_status_id) {
			if (config('return_status_id') == $return_status_id) {
				self::$error['warning'] = language::get('error_default');
			}

			$return_total = admin_sale_return::getTotalReturnsByReturnStatusId($return_status_id);

			if ($return_total) {
				self::$error['warning'] = sprintf(language::get('error_return'), $return_total);
			}

			$return_total = admin_sale_return::getTotalReturnHistoriesByReturnStatusId($return_status_id);

			if ($return_total) {
				self::$error['warning'] = sprintf(language::get('error_return'), $return_total);
			}
		}

		return !self::$error;
	}
}