<?php
class ControllerLocalisationTaxClass {
	private static $error = array();

	public static function index() {
		load::language('localisation/tax_class');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('localisation/tax_class');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_tax_class::addTaxClass(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/tax_class', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('localisation/tax_class');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_tax_class::editTaxClass(request::$get['tax_class_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/tax_class', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('localisation/tax_class');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $tax_class_id) {
				admin_localisation_tax_class::deleteTaxClass($tax_class_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/tax_class', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'title';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/tax_class', $url)
		);

		$data['add'] = url::link('localisation/tax_class/add', $url);
		$data['delete'] = url::link('localisation/tax_class/delete', $url);

		$data['tax_classes'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$tax_class_total = admin_localisation_tax_class::getTotalTaxClasses();

		$results = admin_localisation_tax_class::getTaxClasses($filter_data);

		foreach ($results as $result) {
			$data['tax_classes'][] = array(
				'tax_class_id' => $result['tax_class_id'],
				'title'        => $result['title'],
				'edit'         => url::link('localisation/tax_class/edit', '&tax_class_id=' . $result['tax_class_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_title'] = url::link('localisation/tax_class', '&sort=title' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $tax_class_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('localisation/tax_class', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($tax_class_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($tax_class_total - config('limit_admin'))) ? $tax_class_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $tax_class_total, ceil($tax_class_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/tax_class_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['tax_class_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['title'])) {
			$data['error_title'] = self::$error['title'];
		} else {
			$data['error_title'] = '';
		}

		if (isset(self::$error['description'])) {
			$data['error_description'] = self::$error['description'];
		} else {
			$data['error_description'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/tax_class', $url)
		);

		if (!isset(request::$get['tax_class_id'])) {
			$data['action'] = url::link('localisation/tax_class/add', $url);
		} else {
			$data['action'] = url::link('localisation/tax_class/edit', '&tax_class_id=' . request::$get['tax_class_id'] . $url);
		}

		$data['cancel'] = url::link('localisation/tax_class', $url);

		if (isset(request::$get['tax_class_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$tax_class_info = admin_localisation_tax_class::getTaxClass(request::$get['tax_class_id']);
		}

		if (isset(request::$post['title'])) {
			$data['title'] = request::$post['title'];
		} elseif (!empty($tax_class_info)) {
			$data['title'] = $tax_class_info['title'];
		} else {
			$data['title'] = '';
		}

		if (isset(request::$post['description'])) {
			$data['description'] = request::$post['description'];
		} elseif (!empty($tax_class_info)) {
			$data['description'] = $tax_class_info['description'];
		} else {
			$data['description'] = '';
		}


		$data['tax_rates'] = admin_localisation_tax_rate::getTaxRates();

		if (isset(request::$post['tax_rule'])) {
			$data['tax_rules'] = request::$post['tax_rule'];
		} elseif (isset(request::$get['tax_class_id'])) {
			$data['tax_rules'] = admin_localisation_tax_class::getTaxRules(request::$get['tax_class_id']);
		} else {
			$data['tax_rules'] = array();
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/tax_class_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'localisation/tax_class')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(request::$post['title']) < 3) || (utf8_strlen(request::$post['title']) > 32)) {
			self::$error['title'] = language::get('error_title');
		}

		if ((utf8_strlen(request::$post['description']) < 3) || (utf8_strlen(request::$post['description']) > 255)) {
			self::$error['description'] = language::get('error_description');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'localisation/tax_class')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $tax_class_id) {
			$product_total = admin_catalog_product::getTotalProductsByTaxClassId($tax_class_id);

			if ($product_total) {
				self::$error['warning'] = sprintf(language::get('error_product'), $product_total);
			}
		}

		return !self::$error;
	}
}
