<?php
class ControllerLocalisationTaxRate {
	private static $error = array();

	public static function index() {
		load::language('localisation/tax_rate');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('localisation/tax_rate');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_tax_rate::addTaxRate(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/tax_rate', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('localisation/tax_rate');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_tax_rate::editTaxRate(request::$get['tax_rate_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/tax_rate', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('localisation/tax_rate');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $tax_rate_id) {
				admin_localisation_tax_rate::deleteTaxRate($tax_rate_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/tax_rate', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'tr.name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/tax_rate', $url)
		);

		$data['add'] = url::link('localisation/tax_rate/add', $url);
		$data['delete'] = url::link('localisation/tax_rate/delete', $url);

		$data['tax_rates'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$tax_rate_total = admin_localisation_tax_rate::getTotalTaxRates();

		$results = admin_localisation_tax_rate::getTaxRates($filter_data);

		foreach ($results as $result) {
			$data['tax_rates'][] = array(
				'tax_rate_id'   => $result['tax_rate_id'],
				'name'          => $result['name'],
				'rate'          => $result['rate'],
				'type'          => ($result['type'] == 'F' ? language::get('text_amount') : language::get('text_percent')),
				'geo_zone'      => $result['geo_zone'],
				'date_added'    => date(language::get('date_format_short'), strtotime($result['date_added'])),
				'date_modified' => date(language::get('date_format_short'), strtotime($result['date_modified'])),
				'edit'          => url::link('localisation/tax_rate/edit', '&tax_rate_id=' . $result['tax_rate_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('localisation/tax_rate', '&sort=tr.name' . $url);
		$data['sort_rate'] = url::link('localisation/tax_rate', '&sort=tr.rate' . $url);
		$data['sort_type'] = url::link('localisation/tax_rate', '&sort=tr.type' . $url);
		$data['sort_geo_zone'] = url::link('localisation/tax_rate', '&sort=gz.name' . $url);
		$data['sort_date_added'] = url::link('localisation/tax_rate', '&sort=tr.date_added' . $url);
		$data['sort_date_modified'] = url::link('localisation/tax_rate', '&sort=tr.date_modified' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $tax_rate_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('localisation/tax_rate', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($tax_rate_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($tax_rate_total - config('limit_admin'))) ? $tax_rate_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $tax_rate_total, ceil($tax_rate_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/tax_rate_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['tax_rate_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = '';
		}

		if (isset(self::$error['rate'])) {
			$data['error_rate'] = self::$error['rate'];
		} else {
			$data['error_rate'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/tax_rate', $url)
		);

		if (!isset(request::$get['tax_rate_id'])) {
			$data['action'] = url::link('localisation/tax_rate/add', $url);
		} else {
			$data['action'] = url::link('localisation/tax_rate/edit', '&tax_rate_id=' . request::$get['tax_rate_id'] . $url);
		}

		$data['cancel'] = url::link('localisation/tax_rate', $url);

		if (isset(request::$get['tax_rate_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$tax_rate_info = admin_localisation_tax_rate::getTaxRate(request::$get['tax_rate_id']);
		}

		if (isset(request::$post['name'])) {
			$data['name'] = request::$post['name'];
		} elseif (!empty($tax_rate_info)) {
			$data['name'] = $tax_rate_info['name'];
		} else {
			$data['name'] = '';
		}

		if (isset(request::$post['rate'])) {
			$data['rate'] = request::$post['rate'];
		} elseif (!empty($tax_rate_info)) {
			$data['rate'] = $tax_rate_info['rate'];
		} else {
			$data['rate'] = '';
		}

		if (isset(request::$post['type'])) {
			$data['type'] = request::$post['type'];
		} elseif (!empty($tax_rate_info)) {
			$data['type'] = $tax_rate_info['type'];
		} else {
			$data['type'] = '';
		}

		if (isset(request::$post['tax_rate_customer_group'])) {
			$data['tax_rate_customer_group'] = request::$post['tax_rate_customer_group'];
		} elseif (isset(request::$get['tax_rate_id'])) {
			$data['tax_rate_customer_group'] = admin_localisation_tax_rate::getTaxRateCustomerGroups(request::$get['tax_rate_id']);
		} else {
			$data['tax_rate_customer_group'] = array(config('customer_group_id'));
		}


		$data['customer_groups'] = admin_customer_customer_group::getCustomerGroups();

		if (isset(request::$post['geo_zone_id'])) {
			$data['geo_zone_id'] = request::$post['geo_zone_id'];
		} elseif (!empty($tax_rate_info)) {
			$data['geo_zone_id'] = $tax_rate_info['geo_zone_id'];
		} else {
			$data['geo_zone_id'] = '';
		}


		$data['geo_zones'] = admin_localisation_geo_zone::getGeoZones();

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/tax_rate_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'localisation/tax_rate')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(request::$post['name']) < 3) || (utf8_strlen(request::$post['name']) > 32)) {
			self::$error['name'] = language::get('error_name');
		}

		if (!request::$post['rate']) {
			self::$error['rate'] = language::get('error_rate');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'localisation/tax_rate')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $tax_rate_id) {
			$tax_rule_total = admin_localisation_tax_class::getTotalTaxRulesByTaxRateId($tax_rate_id);

			if ($tax_rule_total) {
				self::$error['warning'] = sprintf(language::get('error_tax_rule'), $tax_rule_total);
			}
		}

		return !self::$error;
	}
}