<?php
class ControllerLocalisationZone {
	private static $error = array();

	public static function index() {
		load::language('localisation/zone');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('localisation/zone');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_zone::addZone(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/zone', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('localisation/zone');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_localisation_zone::editZone(request::$get['zone_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/zone', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('localisation/zone');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $zone_id) {
				admin_localisation_zone::deleteZone($zone_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('localisation/zone', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'c.name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/zone', $url)
		);

		$data['add'] = url::link('localisation/zone/add', $url);
		$data['delete'] = url::link('localisation/zone/delete', $url);

		$data['zones'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$zone_total = admin_localisation_zone::getTotalZones();

		$results = admin_localisation_zone::getZones($filter_data);

		foreach ($results as $result) {
			$data['zones'][] = array(
				'zone_id' => $result['zone_id'],
				'country' => $result['country'],
				'name'    => $result['name'] . (($result['zone_id'] == config('zone_id')) ? language::get('text_default') : null),
				'code'    => $result['code'],
				'edit'    => url::link('localisation/zone/edit', '&zone_id=' . $result['zone_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_country'] = url::link('localisation/zone', '&sort=c.name' . $url);
		$data['sort_name'] = url::link('localisation/zone', '&sort=z.name' . $url);
		$data['sort_code'] = url::link('localisation/zone', '&sort=z.code' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $zone_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('localisation/zone', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($zone_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($zone_total - config('limit_admin'))) ? $zone_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $zone_total, ceil($zone_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/zone_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['zone_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('localisation/zone', $url)
		);

		if (!isset(request::$get['zone_id'])) {
			$data['action'] = url::link('localisation/zone/add', $url);
		} else {
			$data['action'] = url::link('localisation/zone/edit', '&zone_id=' . request::$get['zone_id'] . $url);
		}

		$data['cancel'] = url::link('localisation/zone', $url);

		if (isset(request::$get['zone_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$zone_info = admin_localisation_zone::getZone(request::$get['zone_id']);
		}

		if (isset(request::$post['status'])) {
			$data['status'] = request::$post['status'];
		} elseif (!empty($zone_info)) {
			$data['status'] = $zone_info['status'];
		} else {
			$data['status'] = '1';
		}

		if (isset(request::$post['name'])) {
			$data['name'] = request::$post['name'];
		} elseif (!empty($zone_info)) {
			$data['name'] = $zone_info['name'];
		} else {
			$data['name'] = '';
		}

		if (isset(request::$post['code'])) {
			$data['code'] = request::$post['code'];
		} elseif (!empty($zone_info)) {
			$data['code'] = $zone_info['code'];
		} else {
			$data['code'] = '';
		}

		if (isset(request::$post['country_id'])) {
			$data['country_id'] = request::$post['country_id'];
		} elseif (!empty($zone_info)) {
			$data['country_id'] = $zone_info['country_id'];
		} else {
			$data['country_id'] = '';
		}


		$data['countries'] = admin_localisation_country::getCountries();

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('localisation/zone_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'localisation/zone')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(request::$post['name']) < 1) || (utf8_strlen(request::$post['name']) > 64)) {
			self::$error['name'] = language::get('error_name');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'localisation/zone')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $zone_id) {
			if (config('zone_id') == $zone_id) {
				self::$error['warning'] = language::get('error_default');
			}

			$store_total = admin_setting_store::getTotalStoresByZoneId($zone_id);

			if ($store_total) {
				self::$error['warning'] = sprintf(language::get('error_store'), $store_total);
			}

			$address_total = admin_customer_customer::getTotalAddressesByZoneId($zone_id);

			if ($address_total) {
				self::$error['warning'] = sprintf(language::get('error_address'), $address_total);
			}

			$zone_to_geo_zone_total = admin_localisation_geo_zone::getTotalZoneToGeoZoneByZoneId($zone_id);

			if ($zone_to_geo_zone_total) {
				self::$error['warning'] = sprintf(language::get('error_zone_to_geo_zone'), $zone_to_geo_zone_total);
			}
		}

		return !self::$error;
	}
}