<?php
class ControllerMarketingContact {
	private static $error = array();

	public static function index() {
		load::language('marketing/contact');

		document::setTitle(language::get('heading_title'));


		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('marketing/contact')
		);

		$data['cancel'] = url::link('common/dashboard');


		$data['stores'] = admin_setting_store::getStores();


		$data['customer_groups'] = admin_customer_customer_group::getCustomerGroups();

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('marketing/contact', $data));
	}

	public static function send() {
		load::language('marketing/contact');

		$json = array();

		if (request::$server['REQUEST_METHOD'] == 'POST') {
			if (!user::hasPermission('modify', 'marketing/contact')) {
				$json['error']['warning'] = language::get('error_permission');
			}

			if (!request::$post['subject']) {
				$json['error']['subject'] = language::get('error_subject');
			}

			if (!request::$post['message']) {
				$json['error']['message'] = language::get('error_message');
			}

			if (!$json) {

				$store_info = admin_setting_store::getStore(request::$post['store_id']);

				if ($store_info) {
					$store_name = $store_info['name'];
				} else {
					$store_name = config('name');
				}

				$setting = admin_setting_setting::getSetting('config', request::$post['store_id']);

				$store_email = isset($setting['config_email']) ? $setting['config_email'] : config('email');

				if (isset(request::$get['page'])) {
					$page = (int)request::$get['page'];
				} else {
					$page = 1;
				}

				$email_total = 0;

				$emails = array();

				switch (request::$post['to']) {
					case 'newsletter':
						$customer_data = array(
							'filter_newsletter' => 1,
							'start'             => ($page - 1) * 10,
							'limit'             => 10
						);

						$email_total = admin_customer_customer::getTotalCustomers($customer_data);

						$results = admin_customer_customer::getCustomers($customer_data);

						foreach ($results as $result) {
							$emails[] = $result['email'];
						}
						break;
					case 'customer_all':
						$customer_data = array(
							'start' => ($page - 1) * 10,
							'limit' => 10
						);

						$email_total = admin_customer_customer::getTotalCustomers($customer_data);

						$results = admin_customer_customer::getCustomers($customer_data);

						foreach ($results as $result) {
							$emails[] = $result['email'];
						}
						break;
					case 'customer_group':
						$customer_data = array(
							'filter_customer_group_id' => request::$post['customer_group_id'],
							'start'                    => ($page - 1) * 10,
							'limit'                    => 10
						);

						$email_total = admin_customer_customer::getTotalCustomers($customer_data);

						$results = admin_customer_customer::getCustomers($customer_data);

						foreach ($results as $result) {
							$emails[$result['customer_id']] = $result['email'];
						}
						break;
					case 'customer':
						if (!empty(request::$post['customer'])) {
							$customers = array_slice(request::$post['customer'], ($page - 1) * 10, 10);

							foreach ($customers as $customer_id) {
								$customer_info = admin_customer_customer::getCustomer($customer_id);

								if ($customer_info) {
									$emails[] = $customer_info['email'];
								}
							}

							$email_total = count($emails);
						}
						break;
					case 'affiliate_all':
						$affiliate_data = array(
							'filter_affiliate' => 1,
							'start'            => ($page - 1) * 10,
							'limit'            => 10
						);

						$email_total = admin_customer_customer::getTotalCustomers($affiliate_data);

						$results = admin_customer_customer::getCustomers($affiliate_data);

						foreach ($results as $result) {
							$emails[] = $result['email'];
						}
						break;
					case 'affiliate':
						if (!empty(request::$post['affiliate'])) {
							$affiliates = array_slice(request::$post['affiliate'], ($page - 1) * 10, 10);

							foreach ($affiliates as $affiliate_id) {
								$affiliate_info = admin_customer_customer::getCustomer($affiliate_id);

								if ($affiliate_info) {
									$emails[] = $affiliate_info['email'];
								}
							}

							$email_total = count(request::$post['affiliate']);
						}
						break;
					case 'product':
						if (isset(request::$post['product'])) {
							$email_total = admin_sale_order::getTotalEmailsByProductsOrdered(request::$post['product']);

							$results = admin_sale_order::getEmailsByProductsOrdered(request::$post['product'], ($page - 1) * 10, 10);

							foreach ($results as $result) {
								$emails[] = $result['email'];
							}
						}
						break;
				}

				if ($emails) {
					$json['success'] = language::get('text_success');

					$start = ($page - 1) * 10;
					$end = $start + 10;

					if ($page == 1 && $email_total < 10) {
						$json['success'] = sprintf(language::get('text_sent'), $email_total, $email_total);
					} elseif ($page == 1 && $email_total > 10) {
						$json['success'] = sprintf(language::get('text_sent'), 10, $email_total);
					} elseif ($page > 1 && $email_total < ($page * 10)) {
						$json['success'] = sprintf(language::get('text_sent'), $email_total, $email_total);
					} else {
						$json['success'] = sprintf(language::get('text_sent'), ($start * $page), $email_total);
					}

					if ($end < $email_total) {
						$json['next'] = str_replace('&amp;', '&', url::link('marketing/contact/send', '&page=' . ($page + 1)));
					} else {
						$json['next'] = '';
					}

					$message  = '<html dir="ltr" lang="' . language::get('code') . '">' . "\n";
					$message .= '  <head>' . "\n";
					$message .= '    <title>' . request::$post['subject'] . '</title>' . "\n";
					$message .= '    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">' . "\n";
					$message .= '  </head>' . "\n";
					$message .= '  <body>' . html_entity_decode(request::$post['message'], ENT_QUOTES, 'UTF-8') . '</body>' . "\n";
					$message .= '</html>' . "\n";

					foreach ($emails as $email) {
						if (filter_var($email, FILTER_VALIDATE_EMAIL)) {
							$mail = new Mail(config('mail_engine'));
							$mail::$parameter = config('mail_parameter');
							$mail::$smtp_hostname = config('mail_smtp_hostname');
							$mail::$smtp_username = config('mail_smtp_username');
							$mail::$smtp_password = html_entity_decode(config('mail_smtp_password'), ENT_QUOTES, 'UTF-8');
							$mail::$smtp_port = config('mail_smtp_port');
							$mail::$smtp_timeout = config('mail_smtp_timeout');

							$mail::setTo($email);
							$mail::setFrom($store_email);
							$mail::setSender(html_entity_decode($store_name, ENT_QUOTES, 'UTF-8'));
							$mail::setSubject(html_entity_decode(request::$post['subject'], ENT_QUOTES, 'UTF-8'));
							$mail::setHtml($message);
							$mail::send();
						}
					}
				} else {
					$json['error']['email'] = language::get('error_email');
				}
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
