<?php
class ControllerMarketplaceEvent {
	private static $error = array();
	
	public static function index() {
		load::language('marketplace/event');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function enable() {
		load::language('marketplace/event');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$get['event_id']) && self::validate()) {
			admin_setting_event::enableEvent(request::$get['event_id']);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('marketplace/event', $url));
		}

		self::getList();
	}

	public static function disable() {
		load::language('marketplace/event');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$get['event_id']) && self::validate()) {
			admin_setting_event::disableEvent(request::$get['event_id']);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('marketplace/event', $url));
		}

		self::getList();
	}
	
	public static function delete() {
		load::language('marketplace/event');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validate()) {
			foreach (request::$post['selected'] as $event_id) {
				admin_setting_event::deleteEvent($event_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('marketplace/event', $url));
		}

		self::getList();
	}	
	
	public static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'code';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('marketplace/event', $url)
		);

		$data['delete'] = url::link('marketplace/event/delete', $url);

		$data['events'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$event_total = admin_setting_event::getTotalEvents();

		$results = admin_setting_event::getEvents($filter_data);

		foreach ($results as $result) {
			$data['events'][] = array(
				'event_id'   => $result['event_id'],
				'code'       => $result['code'],
				'trigger'    => $result['trigger'],
				'action'     => $result['action'],
				'sort_order' => $result['sort_order'],
				'status'     => $result['status'] ? language::get('text_enabled') : language::get('text_disabled'),
				'enable'     => url::link('marketplace/event/enable', '&event_id=' . $result['event_id'] . $url),
				'disable'    => url::link('marketplace/event/disable', '&event_id=' . $result['event_id'] . $url),
				'enabled'    => $result['status']
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_code'] = url::link('marketplace/event', '&sort=code' . $url);
		$data['sort_sort_order'] = url::link('marketplace/event', '&sort=sort_order' . $url);
		$data['sort_status'] = url::link('marketplace/event', '&sort=status' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $event_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('marketplace/event', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($event_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($event_total - config('limit_admin'))) ? $event_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $event_total, ceil($event_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('marketplace/event', $data));
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'marketplace/event')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
}