<?php
class ControllerMarketplaceInstaller {
	public static function index() {
		load::language('marketplace/installer');

		document::setTitle(language::get('heading_title'));
		
		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('marketplace/installer')
		);

		
		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');
		
		response::setOutput(load::view('marketplace/installer', $data));
	}

	public static function history() {
		load::language('marketplace/installer');
		
		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}
					
		$data['histories'] = array();
		
		
		$results = admin_setting_extension::getExtensionInstalls(($page - 1) * 10, 10);
		
		foreach ($results as $result) {
			$data['histories'][] = array(
				'extension_install_id' => $result['extension_install_id'],
				'filename'             => $result['filename'],
				'date_added'           => date(language::get('date_format_short'), strtotime($result['date_added']))
			);
		}
		
		$history_total = admin_setting_extension::getTotalExtensionInstalls();

		$pagination = new Pagination();
		$pagination::$total = $history_total;
		$pagination::$page = $page;
		$pagination::$limit = 10;
		$pagination::$url = url::link('marketplace/installer/history', '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($history_total) ? (($page - 1) * 10) + 1 : 0, ((($page - 1) * 10) > ($history_total - 10)) ? $history_total : ((($page - 1) * 10) + 10), $history_total, ceil($history_total / 10));
				
		response::setOutput(load::view('marketplace/installer_history', $data));
	}	
		
	public static function upload() {
		load::language('marketplace/installer');

		$json = array();

		// Check user has permission
		if (!user::hasPermission('modify', 'marketplace/installer')) {
			$json['error'] = language::get('error_permission');
		}

		// Check if there is a install zip already there
		$files = glob(DIR_UPLOAD . '*.tmp');

		foreach ($files as $file) {
			if (is_file($file) && (filectime($file) < (time() - 5))) {
				unlink($file);
			}
			
			if (is_file($file)) {
				$json['error'] = language::get('error_install');
				
				break;
			}
		}

		// Check for any install directories
		$directories = glob(DIR_UPLOAD . 'tmp-*');
		
		foreach ($directories as $directory) {
			if (is_dir($directory) && (filectime($directory) < (time() - 5))) {
				// Get a list of files ready to upload
				$files = array();
	
				$path = array($directory);
	
				while (count($path) != 0) {
					$next = array_shift($path);
	
					// We have to use scandir static function because glob will not pick up dot files.
					foreach (array_diff(scandir($next), array('.', '..')) as $file) {
						$file = $next . '/' . $file;
	
						if (is_dir($file)) {
							$path[] = $file;
						}
	
						$files[] = $file;
					}
				}
	
				rsort($files);
	
				foreach ($files as $file) {
					if (is_file($file)) {
						unlink($file);
					} elseif (is_dir($file)) {
						rmdir($file);
					}
				}
	
				rmdir($directory);
			}
			
			if (is_dir($directory)) {
				$json['error'] = language::get('error_install');
				
				break;
			}		
		}
		
		if (isset(request::$files['file']['name'])) {
			if (substr(request::$files['file']['name'], -10) != '.ocmod.zip') {
				$json['error'] = language::get('error_filetype');
			}

			if (request::$files['file']['error'] != UPLOAD_ERR_OK) {
				$json['error'] = language::get('error_upload_' . request::$files['file']['error']);
			}
		} else {
			$json['error'] = language::get('error_upload');
		}

		if (!$json) {
			session::$data['install'] = token(10);
			
			$file = DIR_UPLOAD . session::$data['install'] . '.tmp';
			
			move_uploaded_file(request::$files['file']['tmp_name'], $file);

			if (is_file($file)) {
				
				$extension_install_id = admin_setting_extension::addExtensionInstall(request::$files['file']['name']);
				
				$json['text'] = language::get('text_install');

				$json['next'] = str_replace('&amp;', '&', url::link('marketplace/install/install', '&extension_install_id=' . $extension_install_id));		
			} else {
				$json['error'] = language::get('error_file');
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}