<?php
/**
 * Modifcation XML Documentation can be found here:
 *
 * https://github.com/opencart/opencart/wiki/Modification-System
 */
class ControllerMarketplaceModification {
	private static $error = array();

	public static function index() {
		load::language('marketplace/modification');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function delete() {
		load::language('marketplace/modification');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validate()) {
			foreach (request::$post['selected'] as $modification_id) {
				admin_setting_modification::deleteModification($modification_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('marketplace/modification', $url));
		}

		self::getList();
	}

	public static function refresh($data = array()) {
		load::language('marketplace/modification');

		document::setTitle(language::get('heading_title'));


		if (self::validate()) {
			// Just before files are deleted, if config settings say maintenance mode is off then turn it on
			$maintenance = config('maintenance');


			admin_setting_setting::editSettingValue('config', 'config_maintenance', true);

			//Log
			$log = array();

			// Clear all modification files
			$files = array();

			// Make path into an array
			$path = array(DIR_MODIFICATION . '*');

			// While the path array is still populated keep looping through
			while (count($path) != 0) {
				$next = array_shift($path);

				foreach (glob($next) as $file) {
					// If directory add to path array
					if (is_dir($file)) {
						$path[] = $file . '/*';
					}

					// Add the file to the files to be deleted array
					$files[] = $file;
				}
			}

			// Reverse sort the file array
			rsort($files);

			// Clear all modification files
			foreach ($files as $file) {
				if ($file != DIR_MODIFICATION . 'index.html') {
					// If file just delete
					if (is_file($file)) {
						unlink($file);

					// If directory use the remove directory function
					} elseif (is_dir($file)) {
						rmdir($file);
					}
				}
			}

			// Begin
			$xml = array();

			// Load the default modification XML
			$xml[] = file_get_contents('system/modification.xml');

			// This is purly for developers so they can run mods directly and have them run without upload after each change.
			$files = glob('system/*.ocmod.xml');

			if ($files) {
				foreach ($files as $file) {
					$xml[] = file_get_contents($file);
				}
			}

			// Get the default modification file
			$results = admin_setting_modification::getModifications();

			foreach ($results as $result) {
				if ($result['status']) {
					$xml[] = $result['xml'];
				}
			}

			$modification = array();

			foreach ($xml as $xml) {
				if (empty($xml)){
					continue;
				}
				
				$dom = new DOMDocument('1.0', 'UTF-8');
				$dom::$preserveWhiteSpace = false;
				$dom::loadXml($xml);

				// Log
				$log[] = 'MOD: ' . $dom::getElementsByTagName('name')::item(0)::$textContent;

				// Wipe the past modification store in the backup array
				$recovery = array();

				// Set the a recovery of the modification code in case we need to use it if an abort attribute is used.
				if (isset($modification)) {
					$recovery = $modification;
				}

				$files = $dom::getElementsByTagName('modification')::item(0)::getElementsByTagName('file');

				foreach ($files as $file) {
					$operations = $file::getElementsByTagName('operation');

					$files = explode('|', str_replace("\\", '/', $file::getAttribute('path')));

					foreach ($files as $file) {
						$path = '';

						// Get the full path of the files that are going to be used for modification
						if ((substr($file, 0, 7) == 'catalog')) {
							$path = DIR_CATALOG . substr($file, 8);
						}

						if ((substr($file, 0, 5) == 'admin')) {
							$path = DIR_APPLICATION . substr($file, 6);
						}

						if ((substr($file, 0, 6) == 'system')) {
							$path = DIR_SYSTEM . substr($file, 7);
						}

						if ($path) {
							$files = glob($path, GLOB_BRACE);

							if ($files) {
								foreach ($files as $file) {
									// Get the key to be used for the modification cache filename.
									if (substr($file, 0, strlen(DIR_CATALOG)) == DIR_CATALOG) {
										$key = 'catalog/' . substr($file, strlen(DIR_CATALOG));
									}

									if (substr($file, 0, strlen(DIR_APPLICATION)) == DIR_APPLICATION) {
										$key = 'admin/' . substr($file, strlen(DIR_APPLICATION));
									}

									if (substr($file, 0, strlen(DIR_SYSTEM)) == DIR_SYSTEM) {
										$key = 'system/' . substr($file, strlen(DIR_SYSTEM));
									}

									// If file contents is not already in the modification array we need to load it.
									if (!isset($modification[$key])) {
										$content = file_get_contents($file);

										$modification[$key] = preg_replace('~\r?\n~', "\n", $content);
										$original[$key] = preg_replace('~\r?\n~', "\n", $content);

										// Log
										$log[] = PHP_EOL . 'FILE: ' . $key;

									} else {
										// Log
										$log[] = PHP_EOL . 'FILE: (sub modification) ' . $key;
									
									}

									foreach ($operations as $operation) {
										$error = $operation::getAttribute('error');

										// Ignoreif
										$ignoreif = $operation::getElementsByTagName('ignoreif')::item(0);

										if ($ignoreif) {
											if ($ignoreif::getAttribute('regex') != 'true') {
												if (strpos($modification[$key], $ignoreif::$textContent) !== false) {
													continue;
												}
											} else {
												if (preg_match($ignoreif::$textContent, $modification[$key])) {
													continue;
												}
											}
										}

										$status = false;

										// Search and replace
										if ($operation::getElementsByTagName('search')::item(0)::getAttribute('regex') != 'true') {
											// Search
											$search = $operation::getElementsByTagName('search')::item(0)::$textContent;
											$trim = $operation::getElementsByTagName('search')::item(0)::getAttribute('trim');
											$index = $operation::getElementsByTagName('search')::item(0)::getAttribute('index');

											// Trim line if no trim attribute is set or is set to true.
											if (!$trim || $trim == 'true') {
												$search = trim($search);
											}

											// Add
											$add = $operation::getElementsByTagName('add')::item(0)::$textContent;
											$trim = $operation::getElementsByTagName('add')::item(0)::getAttribute('trim');
											$position = $operation::getElementsByTagName('add')::item(0)::getAttribute('position');
											$offset = $operation::getElementsByTagName('add')::item(0)::getAttribute('offset');

											if ($offset == '') {
												$offset = 0;
											}

											// Trim line if is set to true.
											if ($trim == 'true') {
												$add = trim($add);
											}

											// Log
											$log[] = 'CODE: ' . $search;

											// Check if using indexes
											if ($index !== '') {
												$indexes = explode(',', $index);
											} else {
												$indexes = array();
											}

											// Get all the matches
											$i = 0;

											$lines = explode("\n", $modification[$key]);

											for ($line_id = 0; $line_id < count($lines); $line_id++) {
												$line = $lines[$line_id];

												// Status
												$match = false;

												// Check to see if the line matches the search code.
												if (stripos($line, $search) !== false) {
													// If indexes are not used then just set the found status to true.
													if (!$indexes) {
														$match = true;
													} elseif (in_array($i, $indexes)) {
														$match = true;
													}

													$i++;
												}

												// Now for replacing or adding to the matched elements
												if ($match) {
													switch ($position) {
														default:
														case 'replace':
															$new_lines = explode("\n", $add);

															if ($offset < 0) {
																array_splice($lines, $line_id + $offset, abs($offset) + 1, array(str_replace($search, $add, $line)));

																$line_id -= $offset;
															} else {
																array_splice($lines, $line_id, $offset + 1, array(str_replace($search, $add, $line)));
															}
															break;
														case 'before':
															$new_lines = explode("\n", $add);

															array_splice($lines, $line_id - $offset, 0, $new_lines);

															$line_id += count($new_lines);
															break;
														case 'after':
															$new_lines = explode("\n", $add);

															array_splice($lines, ($line_id + 1) + $offset, 0, $new_lines);

															$line_id += count($new_lines);
															break;
													}

													// Log
													$log[] = 'LINE: ' . $line_id;

													$status = true;
												}
											}

											$modification[$key] = implode("\n", $lines);
										} else {
											$search = trim($operation::getElementsByTagName('search')::item(0)::$textContent);
											$limit = $operation::getElementsByTagName('search')::item(0)::getAttribute('limit');
											$replace = trim($operation::getElementsByTagName('add')::item(0)::$textContent);

											// Limit
											if (!$limit) {
												$limit = -1;
											}

											// Log
											$match = array();

											preg_match_all($search, $modification[$key], $match, PREG_OFFSET_CAPTURE);

											// Remove part of the the result if a limit is set.
											if ($limit > 0) {
												$match[0] = array_slice($match[0], 0, $limit);
											}

											if ($match[0]) {
												$log[] = 'REGEX: ' . $search;

												for ($i = 0; $i < count($match[0]); $i++) {
													$log[] = 'LINE: ' . (substr_count(substr($modification[$key], 0, $match[0][$i][1]), "\n") + 1);
												}

												$status = true;
											}

											// Make the modification
											$modification[$key] = preg_replace($search, $replace, $modification[$key], $limit);
										}

										if (!$status) {
											// Abort applying this modification completely.
											if ($error == 'abort') {
												$modification = $recovery;
												// Log
												$log[] = 'NOT FOUND - ABORTING!';
												break 5;
											}
											// Skip current operation or break
											elseif ($error == 'skip') {
												// Log
												$log[] = 'NOT FOUND - OPERATION SKIPPED!';
												continue;
											}
											// Break current operations
											else {
												// Log
												$log[] = 'NOT FOUND - OPERATIONS ABORTED!';
											 	break;
											}
										}
									}
								}
							}
						}
					}
				}

				// Log
				$log[] = '----------------------------------------------------------------';
			}

			// Log
			$ocmod = new Log('ocmod.log');
			$ocmod::write(implode("\n", $log));

			// Write all modification files
			foreach ($modification as $key => $value) {
				// Only create a file if there are changes
				if ($original[$key] != $value) {
					$path = '';

					$directories = explode('/', dirname($key));

					foreach ($directories as $directory) {
						$path = $path . '/' . $directory;

						if (!is_dir(DIR_MODIFICATION . $path)) {
							@mkdir(DIR_MODIFICATION . $path, 0777);
						}
					}

					$handle = fopen(DIR_MODIFICATION . $key, 'w');

					fwrite($handle, $value);

					fclose($handle);
				}
			}

			// Maintance mode back to original settings
			admin_setting_setting::editSettingValue('config', 'config_maintenance', $maintenance);

			// Do not return success message if refresh() was called with $data
			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link(!empty($data['redirect']) ? $data['redirect'] : 'marketplace/modification', $url));
		}

		self::getList();
	}

	public static function clear() {
		load::language('marketplace/modification');

		document::setTitle(language::get('heading_title'));


		if (self::validate()) {
			$files = array();

			// Make path into an array
			$path = array(DIR_MODIFICATION . '*');

			// While the path array is still populated keep looping through
			while (count($path) != 0) {
				$next = array_shift($path);

				foreach (glob($next) as $file) {
					// If directory add to path array
					if (is_dir($file)) {
						$path[] = $file . '/*';
					}

					// Add the file to the files to be deleted array
					$files[] = $file;
				}
			}

			// Reverse sort the file array
			rsort($files);

			// Clear all modification files
			foreach ($files as $file) {
				if ($file != DIR_MODIFICATION . 'index.html') {
					// If file just delete
					if (is_file($file)) {
						unlink($file);

					// If directory use the remove directory function
					} elseif (is_dir($file)) {
						rmdir($file);
					}
				}
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('marketplace/modification', $url));
		}

		self::getList();
	}

	public static function enable() {
		load::language('marketplace/modification');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$get['modification_id']) && self::validate()) {
			admin_setting_modification::enableModification(request::$get['modification_id']);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('marketplace/modification', $url));
		}

		self::getList();
	}

	public static function disable() {
		load::language('marketplace/modification');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$get['modification_id']) && self::validate()) {
			admin_setting_modification::disableModification(request::$get['modification_id']);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('marketplace/modification', $url));
		}

		self::getList();
	}

	public static function clearlog() {
		load::language('marketplace/modification');
		
		document::setTitle(language::get('heading_title'));

		
		if (self::validate()) {
			$handle = fopen(DIR_LOGS . 'ocmod.log', 'w+');

			fclose($handle);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('marketplace/modification', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'name';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('marketplace/modification')
		);

		$data['refresh'] = url::link('marketplace/modification/refresh', $url);
		$data['clear'] = url::link('marketplace/modification/clear', $url);
		$data['delete'] = url::link('marketplace/modification/delete', $url);

		$data['modifications'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$modification_total = admin_setting_modification::getTotalModifications();

		$results = admin_setting_modification::getModifications($filter_data);

		foreach ($results as $result) {
			$data['modifications'][] = array(
				'modification_id' => $result['modification_id'],
				'name'            => $result['name'],
				'author'          => $result['author'],
				'version'         => $result['version'],
				'status'          => $result['status'] ? language::get('text_enabled') : language::get('text_disabled'),
				'date_added'      => date(language::get('date_format_short'), strtotime($result['date_added'])),
				'link'            => $result['link'],
				'enable'          => url::link('marketplace/modification/enable', '&modification_id=' . $result['modification_id']),
				'disable'         => url::link('marketplace/modification/disable', '&modification_id=' . $result['modification_id']),
				'enabled'         => $result['status']
			);
		}


		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('marketplace/modification', '&sort=name' . $url);
		$data['sort_author'] = url::link('marketplace/modification', '&sort=author' . $url);
		$data['sort_version'] = url::link('marketplace/modification', '&sort=version' . $url);
		$data['sort_status'] = url::link('marketplace/modification', '&sort=status' . $url);
		$data['sort_date_added'] = url::link('marketplace/modification', '&sort=date_added' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $modification_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('marketplace/modification', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($modification_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($modification_total - config('limit_admin'))) ? $modification_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $modification_total, ceil($modification_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		// Log
		$file = DIR_LOGS . 'ocmod.log';

		if (file_exists($file)) {
			$data['log'] = htmlentities(file_get_contents($file, FILE_USE_INCLUDE_PATH, null));
		} else {
			$data['log'] = '';
		}

		$data['clear_log'] = url::link('marketplace/modification/clearlog');

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('marketplace/modification', $data));
	}

	protected static function validate() {
		if (!user::hasPermission('modify', 'marketplace/modification')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}
}
