<?php
class ControllerReportStatistics {
	private static $error = array();
	
	public static function index() {
		load::language('report/statistics');

		document::setTitle(language::get('heading_title'));


		self::getList();	
	}
	
	public static function ordersale() {
		load::language('report/statistics');

		document::setTitle(language::get('heading_title'));


		if (self::validate()) {
			
			admin_report_statistics::editValue('order_sale', admin_sale_order::getTotalSales(array('filter_order_status' => implode(',', array_merge(config('complete_status'), config('processing_status'))))));		
		
			session::$data['success'] = language::get('text_success');
			
			response::redirect(url::link('report/statistics'));
		}
		
		self::getList();	
	}
		
	public static function orderprocessing() {
		load::language('report/statistics');

		document::setTitle(language::get('heading_title'));


		if (self::validate()) {
			
			admin_report_statistics::editValue('order_processing', admin_sale_order::getTotalOrders(array('filter_order_status' => implode(',', config('processing_status')))));		
		
			session::$data['success'] = language::get('text_success');
			
			response::redirect(url::link('report/statistics'));
		}
		
		self::getList();	
	}
	
	public static function ordercomplete() {
		load::language('report/statistics');

		document::setTitle(language::get('heading_title'));		
		
		
		if (self::validate()) {
			
			admin_report_statistics::editValue('order_complete', admin_sale_order::getTotalOrders(array('filter_order_status' => implode(',', config('complete_status')))));		
		
			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('report/statistics'));
		}		
		
		self::getList();	
	}
	
	public static function orderother() {
		load::language('report/statistics');

		document::setTitle(language::get('heading_title'));	
		
		
		if (self::validate()) {
				
			$order_status_data = array();
	
			$results = admin_localisation_order_status::getOrderStatuses();
	
			foreach ($results as $result) {
				if (!in_array($result['order_status_id'], array_merge(config('complete_status'), config('processing_status')))) {
					$order_status_data[] = $result['order_status_id'];
				}
			}		
			
			
			admin_report_statistics::editValue('order_other', admin_sale_order::getTotalOrders(array('filter_order_status' => implode(',', $order_status_data))));
		
			session::$data['success'] = language::get('text_success');
			
			response::redirect(url::link('report/statistics'));
		}
		
		self::getList();	
	}

	public static function returns() {
		load::language('report/statistics');

		document::setTitle(language::get('heading_title'));	
				
		
		if (self::validate()) {
			
			admin_report_statistics::editValue('return', admin_sale_return::getTotalReturns(array('filter_return_status_id' => config('return_status_id'))));
		
			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('report/statistics'));		
		}
		
		self::getList();	
	}
	
	public static function customer() {
		load::language('report/statistics');

		document::setTitle(language::get('heading_title'));	
				
		
		if (self::validate()) {	
			
			admin_report_statistics::editValue('customer', admin_customer_customer::getTotalCustomers(array('filter_approved' => 0)));
		
			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('report/statistics'));		
		}
		
		self::getList();	
	}	
		
	public static function affiliate() {
		load::language('report/statistics');

		document::setTitle(language::get('heading_title'));	
				
		
		if (self::validate()) {
	
			admin_report_statistics::editValue('affiliate', admin_customer_customer::getTotalAffiliates(array('filter_approved' => 0)));
			
			session::$data['success'] = language::get('text_success');
			
			response::redirect(url::link('report/statistics'));
		}
		
		self::getList();				
	}

	public static function product() {
		load::language('report/statistics');

		document::setTitle(language::get('heading_title'));	
				
		
		if (self::validate()) {		
			
			admin_report_statistics::editValue('product', admin_catalog_product::getTotalProducts(array('filter_quantity' => 0)));

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('report/statistics'));
		}
		
		self::getList();
	}	
	
	public static function review() {
		load::language('report/statistics');

		document::setTitle(language::get('heading_title'));	
				
		
		if (self::validate()) {	
				
			admin_report_statistics::editValue('review', admin_catalog_review::getTotalReviews(array('filter_status' => 0)));
			
			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('report/statistics'));
		}

		self::getList();
	}
	
	public static function getList() {
		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('report/statistics')
		);

		$data['statistics'] = array();
		
		
		$results = admin_report_statistics::getStatistics();
		
		foreach ($results as $result) {
			$data['statistics'][] = array(
				'name'  => language::get('text_' . $result['code']),
				'value' => $result['value'],
				'href'  => url::link('report/statistics/' . str_replace('_', '', $result['code']))
			);
		}
				
		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}
							
		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('report/statistics', $data));
	}
	
	protected static function validate() {
		if (!user::hasPermission('modify', 'report/statistics')) {
			self::$error['warning'] = language::get('error_permission');
		}
		
		return !self::$error;
	}	
}