<?php
class ControllerSaleVoucher {
	private static $error = array();

	public static function index() {
		load::language('sale/voucher');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('sale/voucher');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_sale_voucher::addVoucher(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('sale/voucher', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('sale/voucher');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_sale_voucher::editVoucher(request::$get['voucher_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('sale/voucher', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('sale/voucher');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $voucher_id) {
				admin_sale_voucher::deleteVoucher($voucher_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('sale/voucher', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'v.date_added';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'DESC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('sale/voucher', $url)
		);

		$data['add'] = url::link('sale/voucher/add', $url);
		$data['delete'] = url::link('sale/voucher/delete', $url);

		$data['vouchers'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$voucher_total = admin_sale_voucher::getTotalVouchers();

		$results = admin_sale_voucher::getVouchers($filter_data);

		foreach ($results as $result) {
			if ($result['order_id']) {	
				$order_href = url::link('sale/order/info', '&order_id=' . $result['order_id'] . $url);
			} else {
				$order_href = '';
			}

			$data['vouchers'][] = array(
				'voucher_id' => $result['voucher_id'],
				'code'       => $result['code'],
				'from'       => $result['from_name'],
				'to'         => $result['to_name'],
				'theme'      => $result['theme'],
				'amount'     => currency::format($result['amount'], config('currency')),
				'status'     => ($result['status'] ? language::get('text_enabled') : language::get('text_disabled')),
				'date_added' => date(language::get('date_format_short'), strtotime($result['date_added'])),
				'edit'       => url::link('sale/voucher/edit', '&voucher_id=' . $result['voucher_id'] . $url),
				'order'      => $order_href
			);
		}


		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_code'] = url::link('sale/voucher', '&sort=v.code' . $url);
		$data['sort_from'] = url::link('sale/voucher', '&sort=v.from_name' . $url);
		$data['sort_to'] = url::link('sale/voucher', '&sort=v.to_name' . $url);
		$data['sort_theme'] = url::link('sale/voucher', '&sort=theme' . $url);
		$data['sort_amount'] = url::link('sale/voucher', '&sort=v.amount' . $url);
		$data['sort_status'] = url::link('sale/voucher', '&sort=v.status' . $url);
		$data['sort_date_added'] = url::link('sale/voucher', '&sort=v.date_added' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $voucher_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('sale/voucher', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($voucher_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($voucher_total - config('limit_admin'))) ? $voucher_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $voucher_total, ceil($voucher_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('sale/voucher_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['voucher_id']) ? language::get('text_add') : language::get('text_edit');

		if (isset(request::$get['voucher_id'])) {
			$data['voucher_id'] = (int)request::$get['voucher_id'];
		} else {
			$data['voucher_id'] = 0;
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['code'])) {
			$data['error_code'] = self::$error['code'];
		} else {
			$data['error_code'] = '';
		}

		if (isset(self::$error['from_name'])) {
			$data['error_from_name'] = self::$error['from_name'];
		} else {
			$data['error_from_name'] = '';
		}

		if (isset(self::$error['from_email'])) {
			$data['error_from_email'] = self::$error['from_email'];
		} else {
			$data['error_from_email'] = '';
		}

		if (isset(self::$error['to_name'])) {
			$data['error_to_name'] = self::$error['to_name'];
		} else {
			$data['error_to_name'] = '';
		}

		if (isset(self::$error['to_email'])) {
			$data['error_to_email'] = self::$error['to_email'];
		} else {
			$data['error_to_email'] = '';
		}

		if (isset(self::$error['amount'])) {
			$data['error_amount'] = self::$error['amount'];
		} else {
			$data['error_amount'] = '';
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('sale/voucher', $url)
		);

		if (!isset(request::$get['voucher_id'])) {
			$data['action'] = url::link('sale/voucher/add', $url);
		} else {
			$data['action'] = url::link('sale/voucher/edit', '&voucher_id=' . request::$get['voucher_id'] . $url);
		}

		$data['cancel'] = url::link('sale/voucher', $url);

		if (isset(request::$get['voucher_id']) && (!request::$server['REQUEST_METHOD'] != 'POST')) {
			$voucher_info = admin_sale_voucher::getVoucher(request::$get['voucher_id']);
		}


		if (isset(request::$post['code'])) {
			$data['code'] = request::$post['code'];
		} elseif (!empty($voucher_info)) {
			$data['code'] = $voucher_info['code'];
		} else {
			$data['code'] = '';
		}

		if (isset(request::$post['from_name'])) {
			$data['from_name'] = request::$post['from_name'];
		} elseif (!empty($voucher_info)) {
			$data['from_name'] = $voucher_info['from_name'];
		} else {
			$data['from_name'] = '';
		}

		if (isset(request::$post['from_email'])) {
			$data['from_email'] = request::$post['from_email'];
		} elseif (!empty($voucher_info)) {
			$data['from_email'] = $voucher_info['from_email'];
		} else {
			$data['from_email'] = '';
		}

		if (isset(request::$post['to_name'])) {
			$data['to_name'] = request::$post['to_name'];
		} elseif (!empty($voucher_info)) {
			$data['to_name'] = $voucher_info['to_name'];
		} else {
			$data['to_name'] = '';
		}

		if (isset(request::$post['to_email'])) {
			$data['to_email'] = request::$post['to_email'];
		} elseif (!empty($voucher_info)) {
			$data['to_email'] = $voucher_info['to_email'];
		} else {
			$data['to_email'] = '';
		}


		$data['voucher_themes'] = admin_sale_voucher_theme::getVoucherThemes();

		if (isset(request::$post['voucher_theme_id'])) {
			$data['voucher_theme_id'] = request::$post['voucher_theme_id'];
		} elseif (!empty($voucher_info)) {
			$data['voucher_theme_id'] = $voucher_info['voucher_theme_id'];
		} else {
			$data['voucher_theme_id'] = '';
		}

		if (isset(request::$post['message'])) {
			$data['message'] = request::$post['message'];
		} elseif (!empty($voucher_info)) {
			$data['message'] = $voucher_info['message'];
		} else {
			$data['message'] = '';
		}

		if (isset(request::$post['amount'])) {
			$data['amount'] = request::$post['amount'];
		} elseif (!empty($voucher_info)) {
			$data['amount'] = $voucher_info['amount'];
		} else {
			$data['amount'] = '';
		}

		if (isset(request::$post['status'])) {
			$data['status'] = request::$post['status'];
		} elseif (!empty($voucher_info)) {
			$data['status'] = $voucher_info['status'];
		} else {
			$data['status'] = true;
		}

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('sale/voucher_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'sale/voucher')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(request::$post['code']) < 3) || (utf8_strlen(request::$post['code']) > 10)) {
			self::$error['code'] = language::get('error_code');
		}

		$voucher_info = admin_sale_voucher::getVoucherByCode(request::$post['code']);

		if ($voucher_info) {
			if (!isset(request::$get['voucher_id'])) {
				self::$error['warning'] = language::get('error_exists');
			} elseif ($voucher_info['voucher_id'] != request::$get['voucher_id'])  {
				self::$error['warning'] = language::get('error_exists');
			}
		}

		if ((utf8_strlen(request::$post['to_name']) < 1) || (utf8_strlen(request::$post['to_name']) > 64)) {
			self::$error['to_name'] = language::get('error_to_name');
		}

		if ((utf8_strlen(request::$post['to_email']) > 96) || !filter_var(request::$post['to_email'], FILTER_VALIDATE_EMAIL)) {
			self::$error['to_email'] = language::get('error_email');
		}

		if ((utf8_strlen(request::$post['from_name']) < 1) || (utf8_strlen(request::$post['from_name']) > 64)) {
			self::$error['from_name'] = language::get('error_from_name');
		}

		if ((utf8_strlen(request::$post['from_email']) > 96) || !filter_var(request::$post['from_email'], FILTER_VALIDATE_EMAIL)) {
			self::$error['from_email'] = language::get('error_email');
		}

		if (request::$post['amount'] < 1) {
			self::$error['amount'] = language::get('error_amount');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'sale/voucher')) {
			self::$error['warning'] = language::get('error_permission');
		}


		foreach (request::$post['selected'] as $voucher_id) {
			$order_voucher_info = admin_sale_order::getOrderVoucherByVoucherId($voucher_id);

			if ($order_voucher_info) {
				self::$error['warning'] = sprintf(language::get('error_order'), url::link('sale/order/info', '&order_id=' . $order_voucher_info['order_id']));

				break;
			}
		}

		return !self::$error;
	}

	public static function history() {
		load::language('sale/voucher');


		$data['text_no_results'] = language::get('text_no_results');

		$data['column_order_id'] = language::get('column_order_id');
		$data['column_customer'] = language::get('column_customer');
		$data['column_amount'] = language::get('column_amount');
		$data['column_date_added'] = language::get('column_date_added');

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$data['histories'] = array();

		$results = admin_sale_voucher::getVoucherHistories(request::$get['voucher_id'], ($page - 1) * 10, 10);

		foreach ($results as $result) {
			$data['histories'][] = array(
				'order_id'   => $result['order_id'],
				'customer'   => $result['customer'],
				'amount'     => currency::format($result['amount'], config('currency')),
				'date_added' => date(language::get('date_format_short'), strtotime($result['date_added']))
			);
		}

		$history_total = admin_sale_voucher::getTotalVoucherHistories(request::$get['voucher_id']);

		$pagination = new Pagination();
		$pagination::$total = $history_total;
		$pagination::$page = $page;
		$pagination::$limit = 10;
		$pagination::$url = url::link('sale/voucher/history', '&voucher_id=' . request::$get['voucher_id'] . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($history_total) ? (($page - 1) * 10) + 1 : 0, ((($page - 1) * 10) > ($history_total - 10)) ? $history_total : ((($page - 1) * 10) + 10), $history_total, ceil($history_total / 10));

		response::setOutput(load::view('sale/voucher_history', $data));
	}

	public static function send() {
		load::language('mail/voucher');

		$json = array();

		if (!user::hasPermission('modify', 'sale/voucher')) {
			$json['error'] = language::get('error_permission');
		}

		if (!$json) {

			$vouchers = array();

			if (isset(request::$post['selected'])) {
				$vouchers = request::$post['selected'];
			} elseif (isset(request::$post['voucher_id'])) {
				$vouchers[] = request::$post['voucher_id'];
			}

			if ($vouchers) {

				foreach ($vouchers as $voucher_id) {
					$voucher_info = admin_sale_voucher::getVoucher($voucher_id);
			
					if ($voucher_info) {
						if ($voucher_info['order_id']) {
							$order_id = $voucher_info['order_id'];
						} else {
							$order_id = 0;
						}
			
						$order_info = admin_sale_order::getOrder($order_id);
			
						// If voucher belongs to an order
						if ($order_info) {
			
							$language = new Language($order_info['language_code']);
							$language::load($order_info['language_code']);
							$language::load('mail/voucher');
			
							// HTML Mail
							$data['title'] = sprintf($language::get('text_subject'), $voucher_info['from_name']);
			
							$data['text_greeting'] = sprintf($language::get('text_greeting'), currency::format($voucher_info['amount'], (!empty($order_info['currency_code']) ? $order_info['currency_code'] : config('currency')), (!empty($order_info['currency_value']) ? $order_info['currency_value'] : currency::getValue(config('currency')))));
							$data['text_from'] = sprintf($language::get('text_from'), $voucher_info['from_name']);
							$data['text_message'] = $language::get('text_message');
							$data['text_redeem'] = sprintf($language::get('text_redeem'), $voucher_info['code']);
							$data['text_footer'] = $language::get('text_footer');
			
							$voucher_theme_info = admin_sale_voucher_theme::getVoucherTheme($voucher_info['voucher_theme_id']);
			
							if ($voucher_theme_info && is_file('image/' . $voucher_theme_info['image'])) {
								$data['image'] = HTTP_CATALOG . 'image/' . $voucher_theme_info['image'];
							} else {
								$data['image'] = '';
							}
			
							$data['store_name'] = $order_info['store_name'];
							$data['store_url'] = $order_info['store_url'];
							$data['message'] = nl2br($voucher_info['message']);
			
							$mail = new Mail(config('mail_engine'));
							$mail::$parameter = config('mail_parameter');
							$mail::$smtp_hostname = config('mail_smtp_hostname');
							$mail::$smtp_username = config('mail_smtp_username');
							$mail::$smtp_password = html_entity_decode(config('mail_smtp_password'), ENT_QUOTES, 'UTF-8');
							$mail::$smtp_port = config('mail_smtp_port');
							$mail::$smtp_timeout = config('mail_smtp_timeout');
			
							$mail::setTo($voucher_info['to_email']);
							$mail::setFrom(config('email'));
							$mail::setSender(html_entity_decode($order_info['store_name'], ENT_QUOTES, 'UTF-8'));
							$mail::setSubject(sprintf($language::get('text_subject'), html_entity_decode($voucher_info['from_name'], ENT_QUOTES, 'UTF-8')));
							$mail::setHtml(load::view('mail/voucher', $data));
							$mail::send();
			
						// If voucher does not belong to an order
						} else {
							language::load('mail/voucher');

							$data['title'] = sprintf(language::get('text_subject'), $voucher_info['from_name']);
			
							$data['text_greeting'] = sprintf(language::get('text_greeting'), currency::format($voucher_info['amount'], config('currency')));
							$data['text_from'] = sprintf(language::get('text_from'), $voucher_info['from_name']);
							$data['text_message'] = language::get('text_message');
							$data['text_redeem'] = sprintf(language::get('text_redeem'), $voucher_info['code']);
							$data['text_footer'] = language::get('text_footer');		
			
							$voucher_theme_info = admin_sale_voucher_theme::getVoucherTheme($voucher_info['voucher_theme_id']);

							if ($voucher_theme_info && is_file('image/' . $voucher_theme_info['image'])) {
								$data['image'] = HTTP_CATALOG . 'image/' . $voucher_theme_info['image'];
							} else {
								$data['image'] = '';
							}

							$data['store_name'] = config('name');
							$data['store_url'] = HTTP_CATALOG;
							$data['message'] = nl2br($voucher_info['message']);
			
							$mail = new Mail(config('mail_engine'));
							$mail::$parameter = config('mail_parameter');
							$mail::$smtp_hostname = config('mail_smtp_hostname');
							$mail::$smtp_username = config('mail_smtp_username');
							$mail::$smtp_password = html_entity_decode(config('mail_smtp_password'), ENT_QUOTES, 'UTF-8');
							$mail::$smtp_port = config('mail_smtp_port');
							$mail::$smtp_timeout = config('mail_smtp_timeout');
			
							$mail::setTo($voucher_info['to_email']);
							$mail::setFrom(config('email'));
							$mail::setSender(html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8'));
							$mail::setSubject(html_entity_decode(sprintf(language::get('text_subject'), $voucher_info['from_name']), ENT_QUOTES, 'UTF-8'));
							$mail::setHtml(load::view('mail/voucher', $data));
							$mail::send();
						}
					}
				}

				$json['success'] = language::get('text_sent');
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}