<?php
class ControllerToolBackup {
	public static function index() {
		load::language('tool/backup');

		document::setTitle(language::get('heading_title'));

		if (isset(session::$data['error'])) {
			$data['error_warning'] = session::$data['error'];

			unset(session::$data['error']);
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('tool/backup')
		);


		$data['export'] = url::link('tool/backup/export');
		

		$data['tables'] = admin_tool_backup::getTables();

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('tool/backup', $data));
	}
	
	public static function import() {
		load::language('tool/backup');
		
		$json = array();
		
		if (!user::hasPermission('modify', 'tool/backup')) {
			$json['error'] = language::get('error_permission');
		}
		
		if (isset(request::$files['import']['tmp_name']) && is_uploaded_file(request::$files['import']['tmp_name'])) {
			$filename = tempnam(DIR_UPLOAD, 'bac');
			
			move_uploaded_file(request::$files['import']['tmp_name'], $filename);
		} elseif (isset(request::$get['import'])) {
			$filename = DIR_UPLOAD . basename(html_entity_decode(request::$get['import'], ENT_QUOTES, 'UTF-8'));
		} else {
			$filename = '';
		}
		
		if (!is_file($filename)) {
			$json['error'] = language::get('error_file');
		}	
		
		if (isset(request::$get['position'])) {
			$position = request::$get['position'];
		} else {
			$position = 0; 	
		}
				
		if (!$json) {
			// We set $i so we can batch execute the queries rather than do them all at once.
			$i = 0;
			$start = false;
			
			$handle = fopen($filename, 'r');

			fseek($handle, $position, SEEK_SET);
			
			while (!feof($handle) && ($i < 100)) {
				$position = ftell($handle);

				$line = fgets($handle, 1000000);
				
				if (substr($line, 0, 14) == 'TRUNCATE TABLE' || substr($line, 0, 11) == 'INSERT INTO') {
					$sql = '';
					
					$start = true;
				}

				if ($i > 0 && (substr($line, 0, 24) == 'TRUNCATE TABLE `oc_user`' || substr($line, 0, 30) == 'TRUNCATE TABLE `oc_user_group`')) {
					fseek($handle, $position, SEEK_SET);

					break;
				}

				if ($start) {
					$sql .= $line;
				}
				
				if ($start && substr($line, -2) == ";\n") {
					db::query(substr($sql, 0, strlen($sql) -2));
					
					$start = false;
				}
					
				$i++;
			}

			$position = ftell($handle);

			$size = filesize($filename);

			$json['total'] = round(($position / $size) * 100);

			if ($position && !feof($handle)) {
				$json['next'] = str_replace('&amp;', '&', url::link('tool/backup/import', '&import=' . $filename . '&position=' . $position));
			
				fclose($handle);
			} else {
				fclose($handle);
				
				unlink($filename);

				$json['success'] = language::get('text_success');

				cache::delete('*');
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function export() {
		load::language('tool/backup');

		if (!isset(request::$post['backup'])) {
			session::$data['error'] = language::get('error_export');

			response::redirect(url::link('tool/backup'));
		} elseif (!user::hasPermission('modify', 'tool/backup')) {
			session::$data['error'] = language::get('error_permission');

			response::redirect(url::link('tool/backup'));
		} else {
			response::addheader('Pragma: public');
			response::addheader('Expires: 0');
			response::addheader('Content-Description: File Transfer');
			response::addheader('Content-Type: application/octet-stream');
			response::addheader('Content-Disposition: attachment; filename="' . DB_DATABASE . '_' . date('Y-m-d_H-i-s', time()) . '_backup.sql"');
			response::addheader('Content-Transfer-Encoding: binary');


			response::setOutput(admin_tool_backup::backup(request::$post['backup']));		
		}
	}	
}
