<?php
class ControllerToolUpload {
	private static $error = array();

	public static function index() {
		load::language('tool/upload');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function delete() {
		load::language('tool/upload');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $upload_id) {
				// Remove file before deleting DB record.
				$upload_info = admin_tool_upload::getUpload($upload_id);

				if ($upload_info && is_file(DIR_UPLOAD . $upload_info['filename'])) {
					unlink(DIR_UPLOAD . $upload_info['filename']);
				}

				admin_tool_upload::deleteUpload($upload_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['filter_name'])) {
				$url .= '&filter_name=' . urlencode(html_entity_decode(request::$get['filter_name'], ENT_QUOTES, 'UTF-8'));
			}

			if (isset(request::$get['filter_date_added'])) {
				$url .= '&filter_date_added=' . request::$get['filter_date_added'];
			}

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('tool/upload', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['filter_name'])) {
			$filter_name = request::$get['filter_name'];
		} else {
			$filter_name = '';
		}

		if (isset(request::$get['filter_date_added'])) {
			$filter_date_added = request::$get['filter_date_added'];
		} else {
			$filter_date_added = '';
		}

		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'date_added';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'DESC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['filter_name'])) {
			$url .= '&filter_name=' . urlencode(html_entity_decode(request::$get['filter_name'], ENT_QUOTES, 'UTF-8'));
		}

		if (isset(request::$get['filter_date_added'])) {
			$url .= '&filter_date_added=' . request::$get['filter_date_added'];
		}

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('tool/upload', $url)
		);

		$data['delete'] = url::link('tool/upload/delete', $url);

		$data['uploads'] = array();

		$filter_data = array(
			'filter_name'	    => $filter_name,
			'filter_date_added'	=> $filter_date_added,
			'sort'              => $sort,
			'order'             => $order,
			'start'             => ($page - 1) * config('limit_admin'),
			'limit'             => config('limit_admin')
		);

		$upload_total = admin_tool_upload::getTotalUploads($filter_data);

		$results = admin_tool_upload::getUploads($filter_data);

		foreach ($results as $result) {
			$data['uploads'][] = array(
				'upload_id'  => $result['upload_id'],
				'name'       => $result['name'],
				'filename'   => $result['filename'],
				'date_added' => date(language::get('date_format_short'), strtotime($result['date_added'])),
				'download'   => url::link('tool/upload/download', '&code=' . $result['code'] . $url)
			);
		}


		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} elseif (isset(session::$data['error'])) {
			$data['error_warning'] = session::$data['error'];

			unset(session::$data['error']);
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if (isset(request::$get['filter_name'])) {
			$url .= '&filter_name=' . urlencode(html_entity_decode(request::$get['filter_name'], ENT_QUOTES, 'UTF-8'));
		}

		if (isset(request::$get['filter_date_added'])) {
			$url .= '&filter_date_added=' . request::$get['filter_date_added'];
		}

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_name'] = url::link('tool/upload', '&sort=name' . $url);
		$data['sort_filename'] = url::link('tool/upload', '&sort=filename' . $url);
		$data['sort_date_added'] = url::link('tool/upload', '&sort=date_added' . $url);

		$url = '';

		if (isset(request::$get['filter_name'])) {
			$url .= '&filter_name=' . urlencode(html_entity_decode(request::$get['filter_name'], ENT_QUOTES, 'UTF-8'));
		}

		if (isset(request::$get['filter_date_added'])) {
			$url .= '&filter_date_added=' . request::$get['filter_date_added'];
		}

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $upload_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('tool/upload', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($upload_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($upload_total - config('limit_admin'))) ? $upload_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $upload_total, ceil($upload_total / config('limit_admin')));

		$data['filter_name'] = $filter_name;
		$data['filter_date_added'] = $filter_date_added;

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('tool/upload', $data));
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'tool/upload')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}

	public static function download() {

		load::language('tool/upload');

		if (isset(request::$get['code'])) {
			$code = request::$get['code'];
		} else {
			$code = 0;
		}

		$url = '';

		if (isset(request::$get['filter_name'])) {
			$url .= '&filter_name=' . urlencode(html_entity_decode(request::$get['filter_name'], ENT_QUOTES, 'UTF-8'));
		}

		if (isset(request::$get['filter_date_added'])) {
			$url .= '&filter_date_added=' . request::$get['filter_date_added'];
		}

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$upload_info = admin_tool_upload::getUploadByCode($code);

		if ($upload_info) {
			$file = DIR_UPLOAD . $upload_info['filename'];
			$mask = basename($upload_info['name']);

			if (file_exists($file) && filesize($file) > 0) {
				response::addheader('Pragma: public');
				response::addheader('Expires: 0');
				response::addheader('Content-Description: File Transfer');
				response::addheader('Content-Type: application/octet-stream');
				response::addheader('Content-Disposition: attachment; filename="' . ($mask ? $mask : basename($file)) . '"');
				response::addheader('Content-Transfer-Encoding: binary');

				response::setOutput(file_get_contents($file, FILE_USE_INCLUDE_PATH, null));
			} else {
				session::$data['error'] = language::get('error_file');

				response::redirect(url::link('tool/upload', $url));
			}
		} else {
			session::$data['error'] = language::get('error_upload');

			response::redirect(url::link('tool/upload', $url));
		}
	}

	public static function upload() {
		load::language('sale/order');

		$json = array();

		// Check user has permission
		if (!user::hasPermission('modify', 'tool/upload')) {
			$json['error'] = language::get('error_permission');
		}

		if (!$json) {
			if (!empty(request::$files['file']['name']) && is_file(request::$files['file']['tmp_name'])) {
				// Sanitize the filename
				$filename = html_entity_decode(request::$files['file']['name'], ENT_QUOTES, 'UTF-8');

				if ((utf8_strlen($filename) < 3) || (utf8_strlen($filename) > 128)) {
					$json['error'] = language::get('error_filename');
				}

				// Allowed file extension types
				$allowed = array();

				$extension_allowed = preg_replace('~\r?\n~', "\n", config('file_ext_allowed'));

				$filetypes = explode("\n", $extension_allowed);

				foreach ($filetypes as $filetype) {
					$allowed[] = trim($filetype);
				}

				if (!in_array(strtolower(substr(strrchr($filename, '.'), 1)), $allowed)) {
					$json['error'] = language::get('error_filetype');
				}

				// Allowed file mime types
				$allowed = array();

				$mime_allowed = preg_replace('~\r?\n~', "\n", config('file_mime_allowed'));

				$filetypes = explode("\n", $mime_allowed);

				foreach ($filetypes as $filetype) {
					$allowed[] = trim($filetype);
				}

				if (!in_array(request::$files['file']['type'], $allowed)) {
					$json['error'] = language::get('error_filetype');
				}

				// Check to see if any PHP files are trying to be uploaded
				$content = file_get_contents(request::$files['file']['tmp_name']);

				if (preg_match('/\<\?php/i', $content)) {
					$json['error'] = language::get('error_filetype');
				}

				// Return any upload error
				if (request::$files['file']['error'] != UPLOAD_ERR_OK) {
					$json['error'] = language::get('error_upload_' . request::$files['file']['error']);
				}
			} else {
				$json['error'] = language::get('error_upload');
			}
		}

		if (!$json) {
			$file = $filename . '.' . token(32);

			move_uploaded_file(request::$files['file']['tmp_name'], DIR_UPLOAD . $file);

			// Hide the uploaded file name so people can not link to it directly.

			$json['code'] = admin_tool_upload::addUpload($filename, $file);

			$json['success'] = language::get('text_upload');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}