<?php
class ControllerUserApi {
	private static $error = array();

	public static function index() {
		load::language('user/api');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		load::language('user/api');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_user_api::addApi(request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('user/api', $url));
		}

		self::getForm();
	}

	public static function edit() {
		load::language('user/api');

		document::setTitle(language::get('heading_title'));


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			admin_user_api::editApi(request::$get['api_id'], request::$post);

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('user/api', $url));
		}

		self::getForm();
	}

	public static function delete() {
		load::language('user/api');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$post['selected']) && self::validateDelete()) {
			foreach (request::$post['selected'] as $api_id) {
				admin_user_api::deleteApi($api_id);
			}

			session::$data['success'] = language::get('text_success');

			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			response::redirect(url::link('user/api', $url));
		}

		self::getList();
	}

	protected static function getList() {
		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'username';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('user/api', $url)
		);

		$data['add'] = url::link('user/api/add', $url);
		$data['delete'] = url::link('user/api/delete', $url);

		$data['apis'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * config('limit_admin'),
			'limit' => config('limit_admin')
		);

		$user_total = admin_user_api::getTotalApis();

		$results = admin_user_api::getApis($filter_data);

		foreach ($results as $result) {
			$data['apis'][] = array(
				'api_id'        => $result['api_id'],
				'username'      => $result['username'],
				'status'        => ($result['status'] ? language::get('text_enabled') : language::get('text_disabled')),
				'date_added'    => date(language::get('date_format_short'), strtotime($result['date_added'])),
				'date_modified' => date(language::get('date_format_short'), strtotime($result['date_modified'])),
				'edit'          => url::link('user/api/edit', '&api_id=' . $result['api_id'] . $url)
			);
		}

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['selected'])) {
			$data['selected'] = (array)request::$post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['sort_username'] = url::link('user/api', '&sort=username' . $url);
		$data['sort_status'] = url::link('user/api', '&sort=status' . $url);
		$data['sort_date_added'] = url::link('user/api', '&sort=date_added' . $url);
		$data['sort_date_modified'] = url::link('user/api', '&sort=date_modified' . $url);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$pagination = new Pagination();
		$pagination::$total = $user_total;
		$pagination::$page = $page;
		$pagination::$limit = config('limit_admin');
		$pagination::$url = url::link('user/api', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($user_total) ? (($page - 1) * config('limit_admin')) + 1 : 0, ((($page - 1) * config('limit_admin')) > ($user_total - config('limit_admin'))) ? $user_total : ((($page - 1) * config('limit_admin')) + config('limit_admin')), $user_total, ceil($user_total / config('limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('user/api_list', $data));
	}

	protected static function getForm() {
		$data['text_form'] = !isset(request::$get['api_id']) ? language::get('text_add') : language::get('text_edit');
		$data['text_ip'] = sprintf(language::get('text_ip'), request::$server['REMOTE_ADDR']);
		

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['username'])) {
			$data['error_username'] = self::$error['username'];
		} else {
			$data['error_username'] = '';
		}

		if (isset(self::$error['key'])) {
			$data['error_key'] = self::$error['key'];
		} else {
			$data['error_key'] = '';
		}
		
		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/dashboard')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('user/api', $url)
		);

		if (!isset(request::$get['api_id'])) {
			$data['action'] = url::link('user/api/add', $url);
		} else {
			$data['action'] = url::link('user/api/edit', '&api_id=' . request::$get['api_id'] . $url);
		}

		$data['cancel'] = url::link('user/api', $url);

		if (isset(request::$get['api_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$api_info = admin_user_api::getApi(request::$get['api_id']);
		}

		if (isset(request::$post['username'])) {
			$data['username'] = request::$post['username'];
		} elseif (!empty($api_info)) {
			$data['username'] = $api_info['username'];
		} else {
			$data['username'] = '';
		}

		if (isset(request::$post['key'])) {
			$data['key'] = request::$post['key'];
		} elseif (!empty($api_info)) {
			$data['key'] = $api_info['key'];
		} else {
			$data['key'] = '';
		}

		if (isset(request::$post['status'])) {
			$data['status'] = request::$post['status'];
		} elseif (!empty($api_info)) {
			$data['status'] = $api_info['status'];
		} else {
			$data['status'] = 0;
		}

		// IP
		if (isset(request::$post['api_ip'])) {
			$data['api_ips'] = request::$post['api_ip'];
		} elseif (isset(request::$get['api_id'])) {
			$data['api_ips'] = admin_user_api::getApiIps(request::$get['api_id']);
		} else {
			$data['api_ips'] = array();
		}
		
		// Session
		$data['api_sessions'] = array();
		
		if (isset(request::$get['api_id'])) {
			$results = admin_user_api::getApiSessions(request::$get['api_id']);
			
			foreach ($results as $result) {
				$data['api_sessions'][] = array(
					'api_session_id' => $result['api_session_id'],
					'session_id'     => $result['session_id'],
					'ip'             => $result['ip'],
					'date_added'     => date(language::get('datetime_format'), strtotime($result['date_added'])),
					'date_modified'  => date(language::get('datetime_format'), strtotime($result['date_modified']))
				);
			}
		}
		
		$data['header'] = load::controller('common/header');
		$data['column_left'] = load::controller('common/column_left');
		$data['footer'] = load::controller('common/footer');

		response::setOutput(load::view('user/api_form', $data));
	}

	protected static function validateForm() {
		if (!user::hasPermission('modify', 'user/user')) {
			self::$error['warning'] = language::get('error_permission');
		}

		if ((utf8_strlen(trim(request::$post['username'])) < 3) || (utf8_strlen(trim(request::$post['username'])) > 64)) {
			self::$error['username'] = language::get('error_username');
		}

		if ((utf8_strlen(request::$post['key']) < 64) || (utf8_strlen(request::$post['key']) > 256)) {
			self::$error['key'] = language::get('error_key');
		}
		
		if (!isset(self::$error['warning']) && !isset(request::$post['api_ip'])) {
			self::$error['warning'] = language::get('error_ip');
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (!user::hasPermission('modify', 'user/api')) {
			self::$error['warning'] = language::get('error_permission');
		}

		return !self::$error;
	}

	public static function deleteSession() {
		load::language('user/api');

		$json = array();

		if (!user::hasPermission('modify', 'user/api')) {
			$json['error'] = language::get('error_permission');
		} else {

			admin_user_api::deleteApiSession(request::$get['api_session_id']);

			$json['success'] = language::get('text_success');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}	
}
