<?php
class admin_extension_payment_cardconnect {
	public static function install() {
		db::query("
			CREATE TABLE IF NOT EXISTS `cardconnect_card` (
			  `cardconnect_card_id` INT(11) NOT NULL AUTO_INCREMENT,
			  `cardconnect_order_id` INT(11) NOT NULL DEFAULT '0',
			  `customer_id` INT(11) NOT NULL DEFAULT '0',
			  `profileid` VARCHAR(16) NOT NULL DEFAULT '',
			  `token` VARCHAR(19) NOT NULL DEFAULT '',
			  `type` VARCHAR(50) NOT NULL DEFAULT '',
			  `account` VARCHAR(4) NOT NULL DEFAULT '',
			  `expiry` VARCHAR(4) NOT NULL DEFAULT '',
			  `date_added` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00',
			  PRIMARY KEY (`cardconnect_card_id`)
			) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci");

		db::query("
			CREATE TABLE IF NOT EXISTS `cardconnect_order` (
			  `cardconnect_order_id` INT(11) NOT NULL AUTO_INCREMENT,
			  `order_id` INT(11) NOT NULL DEFAULT '0',
			  `customer_id` INT(11) NOT NULL DEFAULT '0',
			  `payment_method` VARCHAR(255) NOT NULL DEFAULT '',
			  `retref` VARCHAR(12) NOT NULL DEFAULT '',
			  `authcode` VARCHAR(6) NOT NULL DEFAULT '',
			  `currency_code` VARCHAR(3) NOT NULL DEFAULT '',
			  `total` DECIMAL(10, 2) NOT NULL DEFAULT '0.00',
			  `date_added` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00',
			  PRIMARY KEY (`cardconnect_order_id`)
			) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci");

		db::query("
			CREATE TABLE IF NOT EXISTS `cardconnect_order_transaction` (
			  `cardconnect_order_transaction_id` INT(11) NOT NULL AUTO_INCREMENT,
			  `cardconnect_order_id` INT(11) NOT NULL DEFAULT '0',
			  `type` VARCHAR(50) NOT NULL DEFAULT '',
			  `retref` VARCHAR(12) NOT NULL DEFAULT '',
			  `amount` DECIMAL(10, 2) NOT NULL DEFAULT '0.00',
			  `status` VARCHAR(255) NOT NULL DEFAULT '',
			  `date_modified` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00',
			  `date_added` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00',
			  PRIMARY KEY (`cardconnect_order_transaction_id`)
			) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci");
	}

	public static function uninstall() {
		db::query("DROP TABLE IF EXISTS `cardconnect_card`");
		db::query("DROP TABLE IF EXISTS `cardconnect_order`");
		db::query("DROP TABLE IF EXISTS `cardconnect_order_transaction`");

		self::log('Module uninstalled');
	}

	public static function getOrder($order_id) {
		$query = db::query("SELECT * FROM `cardconnect_order` WHERE `order_id` = " . (int)$order_id . " LIMIT 1");

		if ($query->num_rows) {
			$order = $query->row;

			$order['transactions'] = self::getTransactions($order['cardconnect_order_id']);

			return $order;
		} else {
			return false;
		}
	}

	private static function getTransactions($cardconnect_order_id) {
		$query = db::query("SELECT * FROM `cardconnect_order_transaction` WHERE `cardconnect_order_id` = " . (int)$cardconnect_order_id);

		if ($query->num_rows) {
			return $query->rows;
		} else {
			return array();
		}
	}

	public static function getTotalCaptured($cardconnect_order_id) {
		$query = db::query("SELECT SUM(`amount`) AS `total` FROM `cardconnect_order_transaction` WHERE `cardconnect_order_id` = " . (int)$cardconnect_order_id . " AND (`type` = 'payment' OR `type` = 'refund')");

		return (float)$query->row['total'];
	}

	public static function inquire($order_info, $retref) {
		self::log('Posting inquire to CardConnect');

		self::log('Order ID: ' . $order_info['order_id']);

		$url = 'https://' . config::get('cardconnect_site') . '.cardconnect.com:' . ((config::get('cardconnect_environment') == 'live') ? 8443 : 6443) . '/cardconnect/rest/inquire/' . $retref . '/' . config::get('payment_cardconnect_merchant_id');

		$header = array();

		$header[] = 'Content-type: application/json';
		$header[] = 'Authorization: Basic ' . base64_encode(config::get('cardconnect_api_username') . ':' . config::get('cardconnect_api_password'));

		admin_extension_payment_cardconnect::log('Header: ' . print_r($header, true));

		admin_extension_payment_cardconnect::log('URL: ' . $url);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		$response_data = curl_exec($ch);
		if (curl_errno($ch)) {
			admin_extension_payment_cardconnect::log('cURL error: ' . curl_errno($ch));
		}
		curl_close($ch);

		$response_data = json_decode($response_data, true);

		self::log('Response: ' . print_r($response_data, true));

		return $response_data;
	}

	public static function capture($order_info, $amount) {

		self::log('Posting capture to CardConnect');

		self::log('Order ID: ' . $order_info['order_id']);

		$order = admin_sale_order::getOrder($order_info['order_id']);

		$totals = admin_sale_order::getOrderTotals($order_info['order_id']);

		$shipping_cost = '';

		foreach($totals as $total) {
			if ($total['code'] == 'shipping') {
				$shipping_cost = $total['value'];
			}
		}

		$products = admin_sale_order::getOrderProducts($order_info['order_id']);

		$items = array();

		$i = 1;

		foreach ($products as $product) {
			$items[] = array(
				'lineno'      => $i,
				'material'    => '',
				'description' => $product['name'],
				'upc'         => '',
				'quantity'    => $product['quantity'],
				'uom'         => '',
				'unitcost'    => $product['price'],
				'netamnt'     => $product['total'],
				'taxamnt'     => $product['tax'],
				'discamnt'    => ''
			);

			$i++;
		}

		$data = array(
			'merchid'       => config::get('payment_cardconnect_merchant_id'),
			'retref'        => $order_info['retref'],
			'authcode'      => $order_info['authcode'],
			'ponumber'      => $order_info['order_id'],
			'amount'        => round(floatval($amount), 2, PHP_ROUND_HALF_DOWN),
			'currency'      => $order_info['currency_code'],
			'frtamnt'       => $shipping_cost,
			'dutyamnt'      => '',
			'orderdate'     => '',
			'shiptozip'     => $order['shipping_postcode'],
			'shipfromzip'   => '',
			'shiptocountry' => $order['shipping_iso_code_2'],
			'Items'         => $items
		);

		$data_json = json_encode($data);

		$url = 'https://' . config::get('cardconnect_site') . '.cardconnect.com:' . ((config::get('cardconnect_environment') == 'live') ? 8443 : 6443) . '/cardconnect/rest/capture';

		$header = array();

		$header[] = 'Content-type: application/json';
		$header[] = 'Content-length: ' . strlen($data_json);
		$header[] = 'Authorization: Basic ' . base64_encode(config::get('cardconnect_api_username') . ':' . config::get('cardconnect_api_password'));

		admin_extension_payment_cardconnect::log('Header: ' . print_r($header, true));

		admin_extension_payment_cardconnect::log('Post Data: ' . print_r($data, true));

		admin_extension_payment_cardconnect::log('URL: ' . $url);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
		curl_setopt($ch, CURLOPT_POSTFIELDS, $data_json);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		$response_data = curl_exec($ch);
		if (curl_errno($ch)) {
			admin_extension_payment_cardconnect::log('cURL error: ' . curl_errno($ch));
		}
		curl_close($ch);

		$response_data = json_decode($response_data, true);

		self::log('Response: ' . print_r($response_data, true));

		return $response_data;
	}

	public static function refund($order_info, $amount) {
		self::log('Posting refund to CardConnect');

		self::log('Order ID: ' . $order_info['order_id']);

		$data = array(
			'merchid'   => config::get('payment_cardconnect_merchant_id'),
			'amount'    => round(floatval($amount), 2, PHP_ROUND_HALF_DOWN),
			'currency'  => $order_info['currency_code'],
			'retref'    => $order_info['retref']
		);

		$data_json = json_encode($data);

		$url = 'https://' . config::get('cardconnect_site') . '.cardconnect.com:' . ((config::get('cardconnect_environment') == 'live') ? 8443 : 6443) . '/cardconnect/rest/refund';

		$header = array();

		$header[] = 'Content-type: application/json';
		$header[] = 'Content-length: ' . strlen($data_json);
		$header[] = 'Authorization: Basic ' . base64_encode(config::get('cardconnect_api_username') . ':' . config::get('cardconnect_api_password'));

		admin_extension_payment_cardconnect::log('Header: ' . print_r($header, true));

		admin_extension_payment_cardconnect::log('Post Data: ' . print_r($data, true));

		admin_extension_payment_cardconnect::log('URL: ' . $url);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
		curl_setopt($ch, CURLOPT_POSTFIELDS, $data_json);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		$response_data = curl_exec($ch);
		if (curl_errno($ch)) {
			admin_extension_payment_cardconnect::log('cURL error: ' . curl_errno($ch));
		}
		curl_close($ch);

		$response_data = json_decode($response_data, true);

		self::log('Response: ' . print_r($response_data, true));

		return $response_data;
	}

	public static function void($order_info, $retref) {
		self::log('Posting void to CardConnect');

		self::log('Order ID: ' . $order_info['order_id']);

		$data = array(
			'merchid'   => config::get('payment_cardconnect_merchant_id'),
			'amount'    => 0,
			'currency'  => $order_info['currency_code'],
			'retref'    => $retref
		);

		$data_json = json_encode($data);

		$url = 'https://' . config::get('cardconnect_site') . '.cardconnect.com:' . ((config::get('cardconnect_environment') == 'live') ? 8443 : 6443) . '/cardconnect/rest/void';

		$header = array();

		$header[] = 'Content-type: application/json';
		$header[] = 'Content-length: ' . strlen($data_json);
		$header[] = 'Authorization: Basic ' . base64_encode(config::get('cardconnect_api_username') . ':' . config::get('cardconnect_api_password'));

		admin_extension_payment_cardconnect::log('Header: ' . print_r($header, true));

		admin_extension_payment_cardconnect::log('Post Data: ' . print_r($data, true));

		admin_extension_payment_cardconnect::log('URL: ' . $url);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
		curl_setopt($ch, CURLOPT_POSTFIELDS, $data_json);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		$response_data = curl_exec($ch);
		if (curl_errno($ch)) {
			admin_extension_payment_cardconnect::log('cURL error: ' . curl_errno($ch));
		}
		curl_close($ch);

		$response_data = json_decode($response_data, true);

		self::log('Response: ' . print_r($response_data, true));

		return $response_data;
	}

	public static function updateTransactionStatusByRetref($retref, $status) {
		db::query("UPDATE `cardconnect_order_transaction` SET `status` = " . esc($status) . ", `date_modified` = NOW() WHERE `retref` = " . esc($retref));
	}

	public static function addTransaction($cardconnect_order_id, $type, $retref, $amount, $status) {
		db::query("INSERT INTO `cardconnect_order_transaction` SET `cardconnect_order_id` = " . (int)$cardconnect_order_id . ", `type` = " . esc($type) . ", `retref` = " . esc($retref) . ", `amount` = '" . (float)$amount . "', `status` = " . esc($status) . ", `date_modified` = NOW(), `date_added` = NOW()");
	}

	public static function log($data) {
		if (config::get('cardconnect_logging')) {
			$log = new Log('cardconnect.log');

			$log::write($data);
		}
	}
}