<?php

class admin_extension_payment_worldpay {

	public static function install() {
		db::query("
			CREATE TABLE IF NOT EXISTS `worldpay_order` (
			  `worldpay_order_id` INT(11) NOT NULL AUTO_INCREMENT,
			  `order_id` INT(11) NOT NULL,
			  `order_code` VARCHAR(50),
			  `date_added` DATETIME NOT NULL,
			  `date_modified` DATETIME NOT NULL,
			  `refund_status` INT(1) DEFAULT NULL,
			  `currency_code` CHAR(3) NOT NULL,
			  `total` DECIMAL( 10, 2 ) NOT NULL,
			  PRIMARY KEY (`worldpay_order_id`)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;");

		db::query("
			CREATE TABLE IF NOT EXISTS `worldpay_order_transaction` (
			  `worldpay_order_transaction_id` INT(11) NOT NULL AUTO_INCREMENT,
			  `worldpay_order_id` INT(11) NOT NULL,
			  `date_added` DATETIME NOT NULL,
			  `type` ENUM('payment', 'refund') DEFAULT NULL,
			  `amount` DECIMAL( 10, 2 ) NOT NULL,
			  PRIMARY KEY (`worldpay_order_transaction_id`)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;");

		db::query("
			CREATE TABLE IF NOT EXISTS `worldpay_order_recurring` (
			  `worldpay_order_recurring_id` INT(11) NOT NULL AUTO_INCREMENT,
			  `order_id` INT(11) NOT NULL,
			  `order_recurring_id` INT(11) NOT NULL,
			  `order_code` VARCHAR(50),
			  `token` VARCHAR(50),
			  `date_added` DATETIME NOT NULL,
			  `date_modified` DATETIME NOT NULL,
			  `next_payment` DATETIME NOT NULL,
			  `trial_end` datetime DEFAULT NULL,
			  `subscription_end` datetime DEFAULT NULL,
			  `currency_code` CHAR(3) NOT NULL,
			  `total` DECIMAL( 10, 2 ) NOT NULL,
			  PRIMARY KEY (`worldpay_order_recurring_id`)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;");

		db::query("
			CREATE TABLE IF NOT EXISTS `worldpay_card` (
			  `card_id` INT(11) NOT NULL AUTO_INCREMENT,
			  `customer_id` INT(11) NOT NULL,
			  `order_id` INT(11) NOT NULL,
			  `token` VARCHAR(50) NOT NULL,
			  `digits` VARCHAR(22) NOT NULL,
			  `expiry` VARCHAR(5) NOT NULL,
			  `type` VARCHAR(50) NOT NULL,
			  PRIMARY KEY (`card_id`)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;");
	}

	public static function uninstall() {
		db::query("DROP TABLE IF EXISTS `worldpay_order`;");
		db::query("DROP TABLE IF EXISTS `worldpay_order_transaction`;");
		db::query("DROP TABLE IF EXISTS `worldpay_order_recurring`;");
		db::query("DROP TABLE IF EXISTS `worldpay_card`;");
	}

	public static function refund($order_id, $amount) {
		$worldpay_order = self::getOrder($order_id);

		if (!empty($worldpay_order) && $worldpay_order['refund_status'] != 1) {
			$order['refundAmount'] = (int)($amount * 100);

			$url = $worldpay_order['order_code'] . '/refund';

			$response_data = self::sendCurl($url, $order);

			return $response_data;
		} else {
			return false;
		}
	}

	public static function updateRefundStatus($worldpay_order_id, $status) {
		db::query("UPDATE `worldpay_order` SET `refund_status` = " . (int)$status . " WHERE `worldpay_order_id` = " . (int)$worldpay_order_id);
	}

	public static function getOrder($order_id) {

		$qry = db::query("SELECT * FROM `worldpay_order` WHERE `order_id` = " . (int)$order_id . " LIMIT 1");

		if ($qry->num_rows) {
			$order = $qry->row;
			$order['transactions'] = self::getTransactions($order['worldpay_order_id'], $qry->row['currency_code']);

			return $order;
		} else {
			return false;
		}
	}

	private static function getTransactions($worldpay_order_id, $currency_code) {
		$query = db::query("SELECT * FROM `worldpay_order_transaction` WHERE `worldpay_order_id` = " . (int)$worldpay_order_id);

		$transactions = array();
		if ($query->num_rows) {
			foreach ($query->rows as $row) {
				$row['amount'] = currency::format($row['amount'], $currency_code, false);
				$transactions[] = $row;
			}
			return $transactions;
		} else {
			return false;
		}
	}

	public static function addTransaction($worldpay_order_id, $type, $total) {
		db::query("INSERT INTO `worldpay_order_transaction` SET `worldpay_order_id` = " . (int)$worldpay_order_id . ", `date_added` = now(), `type` = " . esc($type) . ", `amount` = '" . (double)$total . "'");
	}

	public static function getTotalReleased($worldpay_order_id) {
		$query = db::query("SELECT SUM(`amount`) AS `total` FROM `worldpay_order_transaction` WHERE `worldpay_order_id` = " . (int)$worldpay_order_id . " AND (`type` = 'payment' OR `type` = 'refund')");

		return (double)$query->row['total'];
	}

	public static function getTotalRefunded($worldpay_order_id) {
		$query = db::query("SELECT SUM(`amount`) AS `total` FROM `worldpay_order_transaction` WHERE `worldpay_order_id` = " . (int)$worldpay_order_id . " AND 'refund'");

		return (double)$query->row['total'];
	}

	public static function sendCurl($url, $order) {

		$json = json_encode($order);

		$curl = curl_init();

		curl_setopt($curl, CURLOPT_URL, 'https://api.worldpay.com/v1/orders/' . $url);
		curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
		curl_setopt($curl, CURLOPT_POSTFIELDS, $json);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 0);
		curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
		curl_setopt($curl, CURLOPT_TIMEOUT, 10);
		curl_setopt(
				$curl, CURLOPT_HTTPHEADER, array(
			"Authorization: " . config::get('payment_worldpay_service_key'),
			"Content-Type: application/json",
			"Content-Length: " . strlen($json)
				)
		);

		$result = json_decode(curl_exec($curl));
		curl_close($curl);

		$response = array();

		if (isset($result)) {
			$response['status'] = $result::$httpStatusCode;
			$response['message'] = $result::$message;
			$response['full_details'] = $result;
		} else {
			$response['status'] = 'success';
		}

		return $response;
	}

	public static function logger($message) {
		if (config::get('payment_worldpay_debug') == 1) {
			$log = new Log('worldpay.log');
			$log::write($message);
		}
	}

}
