<?php
class admin_localisation_currency {
	public static function addCurrency($data) {
		db::query("INSERT INTO currency SET title = " . esc($data['title']) . ", code = " . esc($data['code']) . ", symbol_left = " . esc($data['symbol_left']) . ", symbol_right = " . esc($data['symbol_right']) . ", decimal_place = " . esc($data['decimal_place']) . ", value = " . esc($data['value']) . ", status = " . (int)$data['status'] . ", date_modified = NOW()");

		$currency_id = db::getLastId();

		if (config('currency_auto')) {
			self::refresh(true);
		}

		cache::delete('currency');
		
		return $currency_id;
	}

	public static function editCurrency($currency_id, $data) {
		db::query("UPDATE currency SET title = " . esc($data['title']) . ", code = " . esc($data['code']) . ", symbol_left = " . esc($data['symbol_left']) . ", symbol_right = " . esc($data['symbol_right']) . ", decimal_place = " . esc($data['decimal_place']) . ", value = " . esc($data['value']) . ", status = " . (int)$data['status'] . ", date_modified = NOW() WHERE currency_id = " . (int)$currency_id);

		cache::delete('currency');
	}

	public static function deleteCurrency($currency_id) {
		db::query("DELETE FROM currency WHERE currency_id = " . (int)$currency_id);

		cache::delete('currency');
	}

	public static function getCurrency($currency_id) {
		$query = db::query("SELECT DISTINCT * FROM currency WHERE currency_id = " . (int)$currency_id);

		return $query->row;
	}

	public static function getCurrencyByCode($currency) {
		$query = db::query("SELECT DISTINCT * FROM currency WHERE code = " . esc($currency));

		return $query->row;
	}

	public static function getCurrencies($data = array()) {
		if ($data) {
			$sql = "SELECT * FROM currency";

			$sort_data = array(
				'title',
				'code',
				'value',
				'date_modified'
			);

			if (isset($data['sort']) && in_array($data['sort'], $sort_data)) {
				$sql .= " ORDER BY " . $data['sort'];
			} else {
				$sql .= " ORDER BY title";
			}

			if (isset($data['order']) && ($data['order'] == 'DESC')) {
				$sql .= " DESC";
			} else {
				$sql .= " ASC";
			}

			if (isset($data['start']) || isset($data['limit'])) {
				if ($data['start'] < 0) {
					$data['start'] = 0;
				}

				if ($data['limit'] < 1) {
					$data['limit'] = 20;
				}

				$sql .= " LIMIT " . (int)$data['start'] . "," . (int)$data['limit'];
			}

			$query = db::query($sql);

			return $query->rows;
		} else {
			$currency_data = cache::get('currency');

			if (!$currency_data) {
				$currency_data = array();

				$query = db::query("SELECT * FROM currency ORDER BY title ASC");

				foreach ($query->rows as $result) {
					$currency_data[$result['code']] = array(
						'currency_id'   => $result['currency_id'],
						'title'         => $result['title'],
						'code'          => $result['code'],
						'symbol_left'   => $result['symbol_left'],
						'symbol_right'  => $result['symbol_right'],
						'decimal_place' => $result['decimal_place'],
						'value'         => $result['value'],
						'status'        => $result['status'],
						'date_modified' => $result['date_modified']
					);
				}

				cache::set('currency', $currency_data);
			}

			return $currency_data;
		}
	}

	public static function refresh($force = false) {
		$currency_data = array();

		if ($force) {
			$query = db::query("SELECT * FROM currency WHERE code != " . esc(config('currency')));
		} else {
			$query = db::query("SELECT * FROM currency WHERE code != " . esc(config('currency')) . " AND date_modified < '" .  db::escape(date('Y-m-d H:i:s', strtotime('-1 day'))) . "'");
		}

		foreach ($query->rows as $result) {
			$currency_data[] = config('currency') . $result['code'] . '=X';
			$currency_data[] = $result['code'] . config('currency') . '=X';
		}

		$curl = curl_init();

		curl_setopt($curl, CURLOPT_URL, 'http://download.finance.yahoo.com/d/quotes.csv?s=' . implode(',', $currency_data) . '&f=sl1&e=.json');
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_HEADER, false);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 30);
		curl_setopt($curl, CURLOPT_TIMEOUT, 30);

		$content = curl_exec($curl);
		
		curl_close($curl);

		$line = explode("\n", trim($content));

		for ($i = 0; $i < count($line); $i = $i + 2) {
			$currency = utf8_substr($line[$i], 4, 3);
			$value = utf8_substr($line[$i], 11, 6);
			
			if ((float)$value < 1 && isset($line[$i + 1])) {
				if ((float)utf8_substr($line[$i + 1], 11, 6) > 0) {
					$value = (1 / (float)utf8_substr($line[$i + 1], 11, 6));
				} else {
					$value = 0;
				}
			}	
						
			if ((float)$value) {
				db::query("UPDATE currency SET value = '" . (float)$value . "', date_modified = '" .  db::escape(date('Y-m-d H:i:s')) . "' WHERE code = " . esc($currency));
			}
		}

		db::query("UPDATE currency SET value = '1.00000', date_modified = '" .  db::escape(date('Y-m-d H:i:s')) . "' WHERE code = " . esc(config('currency')));

		cache::delete('currency');
	}

	public static function getTotalCurrencies() {
		$query = db::query("SELECT COUNT(*) AS total FROM currency");

		return $query->row['total'];
	}
}
