<?php
class ControllerAccountAccount {
	public static function index() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/account', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('account/account');

		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		} 
		
		$data['edit'] = url::link('account/edit', '');
		$data['password'] = url::link('account/password', '');
		$data['address'] = url::link('account/address', '');
		
		$data['credit_cards'] = array();
		
		$files = glob('controller/extension/credit_card/*.php');
		
		foreach ($files as $file) {
			$code = basename($file, '.php');
			
			if (config::get('payment_' . $code . '_status') && config::get('payment_' . $code . '_card')) {
				load::language('extension/credit_card/' . $code, 'extension');

				$data['credit_cards'][] = array(
					'name' => language::get('extension')::get('heading_title'),
					'href' => url::link('extension/credit_card/' . $code, '')
				);
			}
		}
		
		$data['wishlist'] = url::link('account/wishlist');
		$data['order'] = url::link('account/order', '');
		$data['download'] = url::link('account/download', '');
		
		if (config::get('total_reward_status')) {
			$data['reward'] = url::link('account/reward', '');
		} else {
			$data['reward'] = '';
		}		
		
		$data['return'] = url::link('account/return', '');
		$data['transaction'] = url::link('account/transaction', '');
		$data['newsletter'] = url::link('account/newsletter', '');
		$data['recurring'] = url::link('account/recurring', '');
		
		
		$affiliate_info = account_customer::getAffiliate(customer::getId());
		
		if (!$affiliate_info) {	
			$data['affiliate'] = url::link('account/affiliate/add', '');
		} else {
			$data['affiliate'] = url::link('account/affiliate/edit', '');
		}
		
		if ($affiliate_info) {		
			$data['tracking'] = url::link('account/tracking', '');
		} else {
			$data['tracking'] = '';
		}
		
		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');
		
		response::setOutput(load::view('account/account', $data));
	}

	public static function country() {
		$json = array();


		$country_info = localisation_country::getCountry(request::$get['country_id']);

		if ($country_info) {

			$json = array(
				'country_id'        => $country_info['country_id'],
				'name'              => $country_info['name'],
				'iso_code_2'        => $country_info['iso_code_2'],
				'iso_code_3'        => $country_info['iso_code_3'],
				'address_format'    => $country_info['address_format'],
				'postcode_required' => $country_info['postcode_required'],
				'zone'              => localisation_zone::getZonesByCountryId(request::$get['country_id']),
				'status'            => $country_info['status']
			);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
