<?php
class ControllerAccountAddress {
	private static $error = array();

	public static function index() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/address', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('account/address');

		document::setTitle(language::get('heading_title'));


		self::getList();
	}

	public static function add() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/address', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('account/address');

		document::setTitle(language::get('heading_title'));

		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment-with-locales.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.js');
		document::addStyle('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.css');


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			account_address::addAddress(customer::getId(), request::$post);

			session::$data['success'] = language::get('text_add');

			response::redirect(url::link('account/address', ''));
		}

		self::getForm();
	}

	public static function edit() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/address', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('account/address');

		document::setTitle(language::get('heading_title'));

		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment-with-locales.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.js');
		document::addStyle('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.css');


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validateForm()) {
			account_address::editAddress(request::$get['address_id'], request::$post);

			// Default Shipping Address
			if (isset(session::$data['shipping_address']['address_id']) && (request::$get['address_id'] == session::$data['shipping_address']['address_id'])) {
				session::$data['shipping_address'] = account_address::getAddress(request::$get['address_id']);

				unset(session::$data['shipping_method']);
				unset(session::$data['shipping_methods']);
			}

			// Default Payment Address
			if (isset(session::$data['payment_address']['address_id']) && (request::$get['address_id'] == session::$data['payment_address']['address_id'])) {
				session::$data['payment_address'] = account_address::getAddress(request::$get['address_id']);

				unset(session::$data['payment_method']);
				unset(session::$data['payment_methods']);
			}

			session::$data['success'] = language::get('text_edit');

			response::redirect(url::link('account/address', ''));
		}

		self::getForm();
	}

	public static function delete() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/address', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('account/address');

		document::setTitle(language::get('heading_title'));


		if (isset(request::$get['address_id']) && self::validateDelete()) {
			account_address::deleteAddress(request::$get['address_id']);

			// Default Shipping Address
			if (isset(session::$data['shipping_address']['address_id']) && (request::$get['address_id'] == session::$data['shipping_address']['address_id'])) {
				unset(session::$data['shipping_address']);
				unset(session::$data['shipping_method']);
				unset(session::$data['shipping_methods']);
			}

			// Default Payment Address
			if (isset(session::$data['payment_address']['address_id']) && (request::$get['address_id'] == session::$data['payment_address']['address_id'])) {
				unset(session::$data['payment_address']);
				unset(session::$data['payment_method']);
				unset(session::$data['payment_methods']);
			}

			session::$data['success'] = language::get('text_delete');

			response::redirect(url::link('account/address', ''));
		}

		self::getList();
	}

	protected static function getList() {
		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('account/address', '')
		);

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		$data['addresses'] = array();

		$results = account_address::getAddresses();

		foreach ($results as $result) {
			if ($result['address_format']) {
				$format = $result['address_format'];
			} else {
				$format = '{firstname} {lastname}' . "\n" . '{company}' . "\n" . '{address_1}' . "\n" . '{address_2}' . "\n" . '{city} {postcode}' . "\n" . '{zone}' . "\n" . '{country}';
			}

			$find = array(
				'{firstname}',
				'{lastname}',
				'{company}',
				'{address_1}',
				'{address_2}',
				'{city}',
				'{postcode}',
				'{zone}',
				'{zone_code}',
				'{country}'
			);

			$replace = array(
				'firstname' => $result['firstname'],
				'lastname'  => $result['lastname'],
				'company'   => $result['company'],
				'address_1' => $result['address_1'],
				'address_2' => $result['address_2'],
				'city'      => $result['city'],
				'postcode'  => $result['postcode'],
				'zone'      => $result['zone'],
				'zone_code' => $result['zone_code'],
				'country'   => $result['country']
			);

			$data['addresses'][] = array(
				'address_id' => $result['address_id'],
				'address'    => str_replace(array("\r\n", "\r", "\n"), '<br />', preg_replace(array("/\s\s+/", "/\r\r+/", "/\n\n+/"), '<br />', trim(str_replace($find, $replace, $format)))),
				'update'     => url::link('account/address/edit', 'address_id=' . $result['address_id']),
				'delete'     => url::link('account/address/delete', 'address_id=' . $result['address_id'])
			);
		}

		$data['add'] = url::link('account/address/add', '');
		$data['back'] = url::link('account/account', '');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/address_list', $data));
	}

	protected static function getForm() {
		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('account/address', '')
		);

		if (!isset(request::$get['address_id'])) {
			$data['breadcrumbs'][] = array(
				'text' => language::get('text_address_add'),
				'href' => url::link('account/address/add', '')
			);
		} else {
			$data['breadcrumbs'][] = array(
				'text' => language::get('text_address_edit'),
				'href' => url::link('account/address/edit', 'address_id=' . request::$get['address_id'])
			);
		}

		$data['text_address'] = !isset(request::$get['address_id']) ? language::get('text_address_add') : language::get('text_address_edit');

		if (isset(self::$error['firstname'])) {
			$data['error_firstname'] = self::$error['firstname'];
		} else {
			$data['error_firstname'] = '';
		}

		if (isset(self::$error['lastname'])) {
			$data['error_lastname'] = self::$error['lastname'];
		} else {
			$data['error_lastname'] = '';
		}

		if (isset(self::$error['address_1'])) {
			$data['error_address_1'] = self::$error['address_1'];
		} else {
			$data['error_address_1'] = '';
		}

		if (isset(self::$error['city'])) {
			$data['error_city'] = self::$error['city'];
		} else {
			$data['error_city'] = '';
		}

		if (isset(self::$error['postcode'])) {
			$data['error_postcode'] = self::$error['postcode'];
		} else {
			$data['error_postcode'] = '';
		}

		if (isset(self::$error['country'])) {
			$data['error_country'] = self::$error['country'];
		} else {
			$data['error_country'] = '';
		}

		if (isset(self::$error['zone'])) {
			$data['error_zone'] = self::$error['zone'];
		} else {
			$data['error_zone'] = '';
		}

		if (isset(self::$error['custom_field'])) {
			$data['error_custom_field'] = self::$error['custom_field'];
		} else {
			$data['error_custom_field'] = array();
		}

		if (!isset(request::$get['address_id'])) {
			$data['action'] = url::link('account/address/add', '');
		} else {
			$data['action'] = url::link('account/address/edit', 'address_id=' . request::$get['address_id']);
		}

		if (isset(request::$get['address_id']) && (request::$server['REQUEST_METHOD'] != 'POST')) {
			$address_info = account_address::getAddress(request::$get['address_id']);
		}

		if (isset(request::$post['firstname'])) {
			$data['firstname'] = request::$post['firstname'];
		} elseif (!empty($address_info)) {
			$data['firstname'] = $address_info['firstname'];
		} else {
			$data['firstname'] = '';
		}

		if (isset(request::$post['lastname'])) {
			$data['lastname'] = request::$post['lastname'];
		} elseif (!empty($address_info)) {
			$data['lastname'] = $address_info['lastname'];
		} else {
			$data['lastname'] = '';
		}

		if (isset(request::$post['company'])) {
			$data['company'] = request::$post['company'];
		} elseif (!empty($address_info)) {
			$data['company'] = $address_info['company'];
		} else {
			$data['company'] = '';
		}

		if (isset(request::$post['address_1'])) {
			$data['address_1'] = request::$post['address_1'];
		} elseif (!empty($address_info)) {
			$data['address_1'] = $address_info['address_1'];
		} else {
			$data['address_1'] = '';
		}

		if (isset(request::$post['address_2'])) {
			$data['address_2'] = request::$post['address_2'];
		} elseif (!empty($address_info)) {
			$data['address_2'] = $address_info['address_2'];
		} else {
			$data['address_2'] = '';
		}

		if (isset(request::$post['postcode'])) {
			$data['postcode'] = request::$post['postcode'];
		} elseif (!empty($address_info)) {
			$data['postcode'] = $address_info['postcode'];
		} else {
			$data['postcode'] = '';
		}

		if (isset(request::$post['city'])) {
			$data['city'] = request::$post['city'];
		} elseif (!empty($address_info)) {
			$data['city'] = $address_info['city'];
		} else {
			$data['city'] = '';
		}

		if (isset(request::$post['country_id'])) {
			$data['country_id'] = (int)request::$post['country_id'];
		}  elseif (!empty($address_info)) {
			$data['country_id'] = $address_info['country_id'];
		} else {
			$data['country_id'] = config('country_id');
		}

		if (isset(request::$post['zone_id'])) {
			$data['zone_id'] = (int)request::$post['zone_id'];
		}  elseif (!empty($address_info)) {
			$data['zone_id'] = $address_info['zone_id'];
		} else {
			$data['zone_id'] = '';
		}

		if (isset(request::$post['custom_field']['address'])) {
			$data['address_custom_field'] = request::$post['custom_field']['address'];
		} elseif (isset($address_info['custom_field'])) {
			$data['address_custom_field'] = $address_info['custom_field'];
		} else {
			$data['address_custom_field'] = array();
		}


		$data['countries'] = localisation_country::getCountries();

		// Custom fields
		$data['custom_fields'] = array();
		

		$custom_fields = account_custom_field::getCustomFields(config('customer_group_id'));

		foreach ($custom_fields as $custom_field) {
			if ($custom_field['location'] == 'address') {
				if($custom_field['type'] == 'file' && isset($data['address_custom_field'][$custom_field['custom_field_id']])) {
					$code = $data['address_custom_field'][$custom_field['custom_field_id']];

					$upload_result = tool_upload::getUploadByCode($code);

					$data['address_custom_field'][$custom_field['custom_field_id']] = array();
					if($upload_result) {
						$data['address_custom_field'][$custom_field['custom_field_id']]['name'] = $upload_result['name'];
						$data['address_custom_field'][$custom_field['custom_field_id']]['code'] = $upload_result['code'];
					} else {
						$data['address_custom_field'][$custom_field['custom_field_id']]['name'] = "";
						$data['address_custom_field'][$custom_field['custom_field_id']]['code'] = $code;
					}
					$data['custom_fields'][] = $custom_field;
				} else {
					$data['custom_fields'][] = $custom_field;
				}
			}
		}

		if (isset(request::$post['default'])) {
			$data['default'] = request::$post['default'];
		} elseif (isset(request::$get['address_id'])) {
			$data['default'] = customer::getAddressId() == request::$get['address_id'];
		} else {
			$data['default'] = false;
		}

		$data['back'] = url::link('account/address', '');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/address_form', $data));
	}

	protected static function validateForm() {
		if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
			self::$error['firstname'] = language::get('error_firstname');
		}

		if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
			self::$error['lastname'] = language::get('error_lastname');
		}

		if ((utf8_strlen(trim(request::$post['address_1'])) < 3) || (utf8_strlen(trim(request::$post['address_1'])) > 128)) {
			self::$error['address_1'] = language::get('error_address_1');
		}

		if ((utf8_strlen(trim(request::$post['city'])) < 2) || (utf8_strlen(trim(request::$post['city'])) > 128)) {
			self::$error['city'] = language::get('error_city');
		}


		$country_info = localisation_country::getCountry(request::$post['country_id']);

		if ($country_info && $country_info['postcode_required'] && (utf8_strlen(trim(request::$post['postcode'])) < 2 || utf8_strlen(trim(request::$post['postcode'])) > 10)) {
			self::$error['postcode'] = language::get('error_postcode');
		}

		if (request::$post['country_id'] == '' || !is_numeric(request::$post['country_id'])) {
			self::$error['country'] = language::get('error_country');
		}

		if (!isset(request::$post['zone_id']) || request::$post['zone_id'] == '' || !is_numeric(request::$post['zone_id'])) {
			self::$error['zone'] = language::get('error_zone');
		}

		// Custom field validation

		$custom_fields = account_custom_field::getCustomFields(config('customer_group_id'));

		foreach ($custom_fields as $custom_field) {
			if ($custom_field['location'] == 'address') {
				if ($custom_field['required'] && empty(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']])) {
					self::$error['custom_field'][$custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
				} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
					self::$error['custom_field'][$custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
				}
			}
		}

		return !self::$error;
	}

	protected static function validateDelete() {
		if (account_address::getTotalAddresses() == 1) {
			self::$error['warning'] = language::get('error_delete');
		}

		if (customer::getAddressId() == request::$get['address_id']) {
			self::$error['warning'] = language::get('error_default');
		}

		return !self::$error;
	}
}
