<?php
class ControllerAccountDownload {
	public static function index() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/download', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('account/download');

		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_downloads'),
			'href' => url::link('account/download', '')
		);


		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$limit = 10;

		$data['downloads'] = array();

		$download_total = account_download::getTotalDownloads();

		$results = account_download::getDownloads(($page - 1) * $limit, $limit);

		foreach ($results as $result) {
			if (file_exists(DIR_DOWNLOAD . $result['filename'])) {
				$size = filesize(DIR_DOWNLOAD . $result['filename']);

				$i = 0;

				$suffix = array(
					'B',
					'KB',
					'MB',
					'GB',
					'TB',
					'PB',
					'EB',
					'ZB',
					'YB'
				);

				while (($size / 1024) > 1) {
					$size = $size / 1024;
					$i++;
				}

				$data['downloads'][] = array(
					'order_id'   => $result['order_id'],
					'date_added' => date(language::get('date_format_short'), strtotime($result['date_added'])),
					'name'       => $result['name'],
					'size'       => round(substr($size, 0, strpos($size, '.') + 4), 2) . $suffix[$i],
					'href'       => url::link('account/download/download', 'download_id=' . $result['download_id'])
				);
			}
		}

		$pagination = new Pagination();
		$pagination::$total = $download_total;
		$pagination::$page = $page;
		$pagination::$limit = $limit;
		$pagination::$url = url::link('account/download', 'page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($download_total) ? (($page - 1) * $limit) + 1 : 0, ((($page - 1) * $limit) > ($download_total - $limit)) ? $download_total : ((($page - 1) * $limit) + $limit), $download_total, ceil($download_total / $limit));

		$data['continue'] = url::link('account/account', '');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/download', $data));
	}

	public static function download() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/download', '');

			response::redirect(url::link('account/login', ''));
		}


		if (isset(request::$get['download_id'])) {
			$download_id = (int)request::$get['download_id'];
		} else {
			$download_id = 0;
		}

		$download_info = account_download::getDownload($download_id);

		if ($download_info) {
			$file = DIR_DOWNLOAD . $download_info['filename'];
			$mask = basename($download_info['mask']);

			if (!headers_sent()) {
				if (file_exists($file)) {
					header('Content-Type: application/octet-stream');
					header('Content-Disposition: attachment; filename="' . ($mask ? $mask : basename($file)) . '"');
					header('Expires: 0');
					header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
					header('Pragma: public');
					header('Content-Length: ' . filesize($file));

					if (ob_get_level()) {
						ob_end_clean();
					}

					readfile($file, 'rb');

					exit();
				} else {
					exit('Error: Could not find file ' . $file . '!');
				}
			} else {
				exit('Error: Headers already sent out!');
			}
		} else {
			response::redirect(url::link('account/download', ''));
		}
	}
}
