<?php
class ControllerAccountEdit {
	private static $error = array();

	public static function index() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/edit', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('account/edit');

		document::setTitle(language::get('heading_title'));

		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment-with-locales.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.js');
		document::addStyle('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.css');


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			account_customer::editCustomer(customer::getId(), request::$post);

			session::$data['success'] = language::get('text_success');

			response::redirect(url::link('account/account', ''));
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_edit'),
			'href' => url::link('account/edit', '')
		);

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['firstname'])) {
			$data['error_firstname'] = self::$error['firstname'];
		} else {
			$data['error_firstname'] = '';
		}

		if (isset(self::$error['lastname'])) {
			$data['error_lastname'] = self::$error['lastname'];
		} else {
			$data['error_lastname'] = '';
		}

		if (isset(self::$error['email'])) {
			$data['error_email'] = self::$error['email'];
		} else {
			$data['error_email'] = '';
		}

		if (isset(self::$error['telephone'])) {
			$data['error_telephone'] = self::$error['telephone'];
		} else {
			$data['error_telephone'] = '';
		}

		if (isset(self::$error['custom_field'])) {
			$data['error_custom_field'] = self::$error['custom_field'];
		} else {
			$data['error_custom_field'] = array();
		}

		$data['action'] = url::link('account/edit', '');

		if (request::$server['REQUEST_METHOD'] != 'POST') {
			$customer_info = account_customer::getCustomer(customer::getId());
		}

		if (isset(request::$post['firstname'])) {
			$data['firstname'] = request::$post['firstname'];
		} elseif (!empty($customer_info)) {
			$data['firstname'] = $customer_info['firstname'];
		} else {
			$data['firstname'] = '';
		}

		if (isset(request::$post['lastname'])) {
			$data['lastname'] = request::$post['lastname'];
		} elseif (!empty($customer_info)) {
			$data['lastname'] = $customer_info['lastname'];
		} else {
			$data['lastname'] = '';
		}

		if (isset(request::$post['email'])) {
			$data['email'] = request::$post['email'];
		} elseif (!empty($customer_info)) {
			$data['email'] = $customer_info['email'];
		} else {
			$data['email'] = '';
		}

		if (isset(request::$post['telephone'])) {
			$data['telephone'] = request::$post['telephone'];
		} elseif (!empty($customer_info)) {
			$data['telephone'] = $customer_info['telephone'];
		} else {
			$data['telephone'] = '';
		}

		if (isset(request::$post['custom_field']['account'])) {
			$data['account_custom_field'] = request::$post['custom_field']['account'];
		} elseif (isset($customer_info)) {
			$data['account_custom_field'] = json_decode($customer_info['custom_field'], true);
		} else {
			$data['account_custom_field'] = array();
		}

		// Custom Fields
		$data['custom_fields'] = array();


		$custom_fields = account_custom_field::getCustomFields(config('customer_group_id'));

		foreach ($custom_fields as $custom_field) {
			if ($custom_field['location'] == 'account') {
				if($custom_field['type'] == 'file' && isset($data['account_custom_field'][$custom_field['custom_field_id']])) {
					$code = $data['account_custom_field'][$custom_field['custom_field_id']];

					$data['account_custom_field'][$custom_field['custom_field_id']] = array();

					$upload_result = tool_upload::getUploadByCode($code);
					
					if($upload_result) {
						$data['account_custom_field'][$custom_field['custom_field_id']]['name'] = $upload_result['name'];
						$data['account_custom_field'][$custom_field['custom_field_id']]['code'] = $upload_result['code'];
					} else {
						$data['account_custom_field'][$custom_field['custom_field_id']]['name'] = "";
						$data['account_custom_field'][$custom_field['custom_field_id']]['code'] = $code;
					}
					$data['custom_fields'][] = $custom_field;
				} else {
					$data['custom_fields'][] = $custom_field;
				}
			}
		}

		$data['back'] = url::link('account/account', '');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/edit', $data));
	}

	protected static function validate() {
		if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
			self::$error['firstname'] = language::get('error_firstname');
		}

		if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
			self::$error['lastname'] = language::get('error_lastname');
		}

		if ((utf8_strlen(request::$post['email']) > 96) || !filter_var(request::$post['email'], FILTER_VALIDATE_EMAIL)) {
			self::$error['email'] = language::get('error_email');
		}

		if ((customer::getEmail() != request::$post['email']) && account_customer::getTotalCustomersByEmail(request::$post['email'])) {
			self::$error['warning'] = language::get('error_exists');
		}

		if ((utf8_strlen(request::$post['telephone']) < 3) || (utf8_strlen(request::$post['telephone']) > 32)) {
			self::$error['telephone'] = language::get('error_telephone');
		}

		// Custom field validation

		$custom_fields = account_custom_field::getCustomFields(config('customer_group_id'));

		foreach ($custom_fields as $custom_field) {
			if ($custom_field['location'] == 'account') {
				if ($custom_field['required'] && empty(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']])) {
					self::$error['custom_field'][$custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
				} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
					self::$error['custom_field'][$custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
				}
			}
		}

		return !self::$error;
	}
}