<?php
class ControllerAccountLogin {
	private static $error = array();

	public static function index() {

		// Login override for admin users
		if (!empty(request::$get['token'])) {
			customer::logout();
			cart::clear();

			unset(session::$data['order_id']);
			unset(session::$data['payment_address']);
			unset(session::$data['payment_method']);
			unset(session::$data['payment_methods']);
			unset(session::$data['shipping_address']);
			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
			unset(session::$data['comment']);
			unset(session::$data['coupon']);
			unset(session::$data['reward']);
			unset(session::$data['voucher']);
			unset(session::$data['vouchers']);

			$customer_info = account_customer::getCustomerByToken(request::$get['token']);

			if ($customer_info && customer::login($customer_info['email'], '', true)) {
				// Default Addresses

				if (config('tax_customer') == 'payment') {
					session::$data['payment_address'] = account_address::getAddress(customer::getAddressId());
				}

				if (config('tax_customer') == 'shipping') {
					session::$data['shipping_address'] = account_address::getAddress(customer::getAddressId());
				}

				response::redirect(url::link('account/account', ''));
			}
		}

		if (customer::isLogged()) {
			response::redirect(url::link('account/account', ''));
		}

		load::language('account/login');

		document::setTitle(language::get('heading_title'));

		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			// Unset guest
			unset(session::$data['guest']);

			// Default Shipping Address

			if (config('tax_customer') == 'payment') {
				session::$data['payment_address'] = account_address::getAddress(customer::getAddressId());
			}

			if (config('tax_customer') == 'shipping') {
				session::$data['shipping_address'] = account_address::getAddress(customer::getAddressId());
			}

			// Wishlist
			if (isset(session::$data['wishlist']) && is_array(session::$data['wishlist'])) {

				foreach (session::$data['wishlist'] as $key => $product_id) {
					account_wishlist::addWishlist($product_id);

					unset(session::$data['wishlist'][$key]);
				}
			}

			// Added strpos check to pass McAfee PCI compliance test (http://forum.opencart.com/viewtopic.php?f=10&t=12043&p=151494#p151295)
			if (isset(request::$post['redirect']) && request::$post['redirect'] != url::link('account/logout', '') && (strpos(request::$post['redirect'], config('url')) !== false || strpos(request::$post['redirect'], config('ssl')) !== false)) {
				response::redirect(str_replace('&amp;', '&', request::$post['redirect']));
			} else {
				response::redirect(url::link('account/account', ''));
			}
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_login'),
			'href' => url::link('account/login', '')
		);

		if (isset(session::$data['error'])) {
			$data['error_warning'] = session::$data['error'];

			unset(session::$data['error']);
		} elseif (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['action'] = url::link('account/login', '');
		$data['register'] = url::link('account/register', '');
		$data['forgotten'] = url::link('account/forgotten', '');

		// Added strpos check to pass McAfee PCI compliance test (http://forum.opencart.com/viewtopic.php?f=10&t=12043&p=151494#p151295)
		if (isset(request::$post['redirect']) && (strpos(request::$post['redirect'], config('url')) !== false || strpos(request::$post['redirect'], config('ssl')) !== false)) {
			$data['redirect'] = request::$post['redirect'];
		} elseif (isset(session::$data['redirect'])) {
			$data['redirect'] = session::$data['redirect'];

			unset(session::$data['redirect']);
		} else {
			$data['redirect'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(request::$post['email'])) {
			$data['email'] = request::$post['email'];
		} else {
			$data['email'] = '';
		}

		if (isset(request::$post['password'])) {
			$data['password'] = request::$post['password'];
		} else {
			$data['password'] = '';
		}

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/login', $data));
	}

	protected static function validate() {
		// Check how many login attempts have been made.
		$login_info = account_customer::getLoginAttempts(request::$post['email']);

		if ($login_info && ($login_info['total'] >= config('login_attempts')) && strtotime('-1 hour') < strtotime($login_info['date_modified'])) {
			self::$error['warning'] = language::get('error_attempts');
		}

		// Check if customer has been approved.
		$customer_info = account_customer::getCustomerByEmail(request::$post['email']);

		if ($customer_info && !$customer_info['status']) {
			self::$error['warning'] = language::get('error_approved');
		}

		if (!self::$error) {
			if (!customer::login(request::$post['email'], request::$post['password'])) {
				self::$error['warning'] = language::get('error_login');

				account_customer::addLoginAttempt(request::$post['email']);
			} else {
				account_customer::deleteLoginAttempts(request::$post['email']);
			}
		}

		return !self::$error;
	}
}
