<?php
class ControllerAccountOrder {
	public static function index() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/order', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('account/order');

		document::setTitle(language::get('heading_title'));
		
		$url = '';

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}
		
		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);
		
		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('account/order', $url)
		);

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$limit = 10;

		$data['orders'] = array();


		$order_total = account_order::getTotalOrders();

		$results = account_order::getOrders(($page - 1) * $limit, $limit);

		foreach ($results as $result) {
			$product_total = account_order::getTotalOrderProductsByOrderId($result['order_id']);
			$voucher_total = account_order::getTotalOrderVouchersByOrderId($result['order_id']);

			$data['orders'][] = array(
				'order_id'   => $result['order_id'],
				'name'       => $result['firstname'] . ' ' . $result['lastname'],
				'status'     => $result['status'],
				'date_added' => date(language::get('date_format_short'), strtotime($result['date_added'])),
				'products'   => ($product_total + $voucher_total),
				'total'      => currency::format($result['total'], $result['currency_code'], $result['currency_value']),
				'view'       => url::link('account/order/info', 'order_id=' . $result['order_id']),
			);
		}

		$pagination = new Pagination();
		$pagination::$total = $order_total;
		$pagination::$page = $page;
		$pagination::$limit = $limit;
		$pagination::$url = url::link('account/order', 'page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($order_total) ? (($page - 1) * $limit) + 1 : 0, ((($page - 1) * $limit) > ($order_total - $limit)) ? $order_total : ((($page - 1) * $limit) + $limit), $order_total, ceil($order_total / $limit));

		$data['continue'] = url::link('account/account', '');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/order_list', $data));
	}

	public static function info() {
		load::language('account/order');

		if (isset(request::$get['order_id'])) {
			$order_id = request::$get['order_id'];
		} else {
			$order_id = 0;
		}

		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/order/info', 'order_id=' . $order_id);

			response::redirect(url::link('account/login', ''));
		}


		$order_info = account_order::getOrder($order_id);

		if ($order_info) {
			document::setTitle(language::get('text_order'));

			$url = '';

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			$data['breadcrumbs'] = array();

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_home'),
				'href' => url::link('common/home')
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_account'),
				'href' => url::link('account/account', '')
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('heading_title'),
				'href' => url::link('account/order', $url)
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_order'),
				'href' => url::link('account/order/info', 'order_id=' . request::$get['order_id'] . $url)
			);

			if (isset(session::$data['error'])) {
				$data['error_warning'] = session::$data['error'];

				unset(session::$data['error']);
			} else {
				$data['error_warning'] = '';
			}

			if (isset(session::$data['success'])) {
				$data['success'] = session::$data['success'];

				unset(session::$data['success']);
			} else {
				$data['success'] = '';
			}

			if ($order_info['invoice_no']) {
				$data['invoice_no'] = $order_info['invoice_prefix'] . $order_info['invoice_no'];
			} else {
				$data['invoice_no'] = '';
			}

			$data['order_id'] = (int)request::$get['order_id'];
			$data['date_added'] = date(language::get('date_format_short'), strtotime($order_info['date_added']));

			if ($order_info['payment_address_format']) {
				$format = $order_info['payment_address_format'];
			} else {
				$format = '{firstname} {lastname}' . "\n" . '{company}' . "\n" . '{address_1}' . "\n" . '{address_2}' . "\n" . '{city} {postcode}' . "\n" . '{zone}' . "\n" . '{country}';
			}

			$find = array(
				'{firstname}',
				'{lastname}',
				'{company}',
				'{address_1}',
				'{address_2}',
				'{city}',
				'{postcode}',
				'{zone}',
				'{zone_code}',
				'{country}'
			);

			$replace = array(
				'firstname' => $order_info['payment_firstname'],
				'lastname'  => $order_info['payment_lastname'],
				'company'   => $order_info['payment_company'],
				'address_1' => $order_info['payment_address_1'],
				'address_2' => $order_info['payment_address_2'],
				'city'      => $order_info['payment_city'],
				'postcode'  => $order_info['payment_postcode'],
				'zone'      => $order_info['payment_zone'],
				'zone_code' => $order_info['payment_zone_code'],
				'country'   => $order_info['payment_country']
			);

			$data['payment_address'] = str_replace(array("\r\n", "\r", "\n"), '<br />', preg_replace(array("/\s\s+/", "/\r\r+/", "/\n\n+/"), '<br />', trim(str_replace($find, $replace, $format))));

			$data['payment_method'] = $order_info['payment_method'];

			if ($order_info['shipping_address_format']) {
				$format = $order_info['shipping_address_format'];
			} else {
				$format = '{firstname} {lastname}' . "\n" . '{company}' . "\n" . '{address_1}' . "\n" . '{address_2}' . "\n" . '{city} {postcode}' . "\n" . '{zone}' . "\n" . '{country}';
			}

			$find = array(
				'{firstname}',
				'{lastname}',
				'{company}',
				'{address_1}',
				'{address_2}',
				'{city}',
				'{postcode}',
				'{zone}',
				'{zone_code}',
				'{country}'
			);

			$replace = array(
				'firstname' => $order_info['shipping_firstname'],
				'lastname'  => $order_info['shipping_lastname'],
				'company'   => $order_info['shipping_company'],
				'address_1' => $order_info['shipping_address_1'],
				'address_2' => $order_info['shipping_address_2'],
				'city'      => $order_info['shipping_city'],
				'postcode'  => $order_info['shipping_postcode'],
				'zone'      => $order_info['shipping_zone'],
				'zone_code' => $order_info['shipping_zone_code'],
				'country'   => $order_info['shipping_country']
			);

			$data['shipping_address'] = str_replace(array("\r\n", "\r", "\n"), '<br />', preg_replace(array("/\s\s+/", "/\r\r+/", "/\n\n+/"), '<br />', trim(str_replace($find, $replace, $format))));

			$data['shipping_method'] = $order_info['shipping_method'];


			// Products
			$data['products'] = array();

			$products = account_order::getOrderProducts(request::$get['order_id']);

			foreach ($products as $product) {
				$option_data = array();

				$options = account_order::getOrderOptions(request::$get['order_id'], $product['order_product_id']);

				foreach ($options as $option) {
					if ($option['type'] != 'file') {
						$value = $option['value'];
					} else {
						$upload_info = tool_upload::getUploadByCode($option['value']);

						if ($upload_info) {
							$value = $upload_info['name'];
						} else {
							$value = '';
						}
					}

					$option_data[] = array(
						'name'  => $option['name'],
						'value' => (utf8_strlen($value) > 20 ? utf8_substr($value, 0, 20) . '..' : $value)
					);
				}

				$product_info = catalog_product::getProduct($product['product_id']);

				if ($product_info) {
					$reorder = url::link('account/order/reorder', 'order_id=' . $order_id . '&order_product_id=' . $product['order_product_id']);
				} else {
					$reorder = '';
				}

				$data['products'][] = array(
					'name'     => $product['name'],
					'model'    => $product['model'],
					'option'   => $option_data,
					'quantity' => $product['quantity'],
					'price'    => currency::format($product['price'] + (config('tax') ? $product['tax'] : 0), $order_info['currency_code'], $order_info['currency_value']),
					'total'    => currency::format($product['total'] + (config('tax') ? ($product['tax'] * $product['quantity']) : 0), $order_info['currency_code'], $order_info['currency_value']),
					'reorder'  => $reorder,
					'return'   => url::link('account/return/add', 'order_id=' . $order_info['order_id'] . '&product_id=' . $product['product_id'])
				);
			}

			// Voucher
			$data['vouchers'] = array();

			$vouchers = account_order::getOrderVouchers(request::$get['order_id']);

			foreach ($vouchers as $voucher) {
				$data['vouchers'][] = array(
					'description' => $voucher['description'],
					'amount'      => currency::format($voucher['amount'], $order_info['currency_code'], $order_info['currency_value'])
				);
			}

			// Totals
			$data['totals'] = array();

			$totals = account_order::getOrderTotals(request::$get['order_id']);

			foreach ($totals as $total) {
				$data['totals'][] = array(
					'title' => $total['title'],
					'text'  => currency::format($total['value'], $order_info['currency_code'], $order_info['currency_value']),
				);
			}

			$data['comment'] = nl2br($order_info['comment']);

			// History
			$data['histories'] = array();

			$results = account_order::getOrderHistories(request::$get['order_id']);

			foreach ($results as $result) {
				$data['histories'][] = array(
					'date_added' => date(language::get('date_format_short'), strtotime($result['date_added'])),
					'status'     => $result['status'],
					'comment'    => $result['notify'] ? nl2br($result['comment']) : ''
				);
			}

			$data['continue'] = url::link('account/order', '');

			$data['column_left'] = load::controller('common/column_left');
			$data['column_right'] = load::controller('common/column_right');
			$data['content_top'] = load::controller('common/content_top');
			$data['content_bottom'] = load::controller('common/content_bottom');
			$data['footer'] = load::controller('common/footer');
			$data['header'] = load::controller('common/header');

			response::setOutput(load::view('account/order_info', $data));
		} else {
			return 'error/not_found';
		}
	}

	public static function reorder() {
		load::language('account/order');

		if (isset(request::$get['order_id'])) {
			$order_id = request::$get['order_id'];
		} else {
			$order_id = 0;
		}


		$order_info = account_order::getOrder($order_id);

		if ($order_info) {
			if (isset(request::$get['order_product_id'])) {
				$order_product_id = request::$get['order_product_id'];
			} else {
				$order_product_id = 0;
			}

			$order_product_info = account_order::getOrderProduct($order_id, $order_product_id);

			if ($order_product_info) {

				$product_info = catalog_product::getProduct($order_product_info['product_id']);

				if ($product_info) {
					$option_data = array();

					$order_options = account_order::getOrderOptions($order_product_info['order_id'], $order_product_id);

					foreach ($order_options as $order_option) {
						if ($order_option['type'] == 'select' || $order_option['type'] == 'radio' || $order_option['type'] == 'image') {
							$option_data[$order_option['product_option_id']] = $order_option['product_option_value_id'];
						} elseif ($order_option['type'] == 'checkbox') {
							$option_data[$order_option['product_option_id']][] = $order_option['product_option_value_id'];
						} elseif ($order_option['type'] == 'text' || $order_option['type'] == 'textarea' || $order_option['type'] == 'date' || $order_option['type'] == 'datetime' || $order_option['type'] == 'time') {
							$option_data[$order_option['product_option_id']] = $order_option['value'];
						} elseif ($order_option['type'] == 'file') {
							$option_data[$order_option['product_option_id']] = $order_option['value'];
						}
					}

					cart::add($order_product_info['product_id'], $order_product_info['quantity'], $option_data);

					session::$data['success'] = sprintf(language::get('text_success'), url::link('product/product', 'product_id=' . $product_info['product_id']), $product_info['name'], url::link('checkout/cart'));

					unset(session::$data['shipping_method']);
					unset(session::$data['shipping_methods']);
					unset(session::$data['payment_method']);
					unset(session::$data['payment_methods']);
				} else {
					session::$data['error'] = sprintf(language::get('error_reorder'), $order_product_info['name']);
				}
			}
		}

		response::redirect(url::link('account/order/info', 'order_id=' . $order_id));
	}
}
