<?php
class ControllerAccountRecurring {
	public static function index() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/recurring', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('account/recurring');

		document::setTitle(language::get('heading_title'));

		$url = '';

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('account/recurring', $url)
		);

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$limit = 10;

		$data['recurrings'] = array();


		$recurring_total = account_recurring::getTotalOrderRecurrings();

		$results = account_recurring::getOrderRecurrings(($page - 1) * $limit, $limit);

		foreach ($results as $result) {
			if ($result['status']) {
				$status = language::get('text_status_' . $result['status']);
			} else {
				$status = '';
			}

			$data['recurrings'][] = array(
				'order_recurring_id' => $result['order_recurring_id'],
				'product'            => $result['product_name'],
				'status'             => $status,
				'date_added'         => date(language::get('date_format_short'), strtotime($result['date_added'])),
				'view'               => url::link('account/recurring/info', 'order_recurring_id=' . $result['order_recurring_id']),
			);
		}

		$pagination = new Pagination();
		$pagination::$total = $recurring_total;
		$pagination::$page = $page;
		$pagination::$limit = $limit;
		$pagination::$url = url::link('account/recurring', 'page={page}');

		$data['pagination'] = $pagination::render();

		$data['continue'] = url::link('account/account', '');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/recurring_list', $data));
	}

	public static function info() {
		load::language('account/recurring');

		if (isset(request::$get['order_recurring_id'])) {
			$order_recurring_id = (int)request::$get['order_recurring_id'];
		} else {
			$order_recurring_id = 0;
		}

		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/recurring/info', 'order_recurring_id=' . $order_recurring_id);

			response::redirect(url::link('account/login', ''));
		}


		$recurring_info = account_recurring::getOrderRecurring($order_recurring_id);

		if ($recurring_info) {
			document::setTitle(language::get('text_recurring'));

			$url = '';

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			$data['breadcrumbs'] = array();

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_home'),
				'href' => url::link('common/home'),
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_account'),
				'href' => url::link('account/account', ''),
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('heading_title'),
				'href' => url::link('account/recurring', $url),
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_recurring'),
				'href' => url::link('account/recurring/info', 'order_recurring_id=' . request::$get['order_recurring_id'] . $url),
			);

			$data['order_recurring_id'] = (int)request::$get['order_recurring_id'];
			$data['date_added'] = date(language::get('date_format_short'), strtotime($recurring_info['date_added']));

			if ($recurring_info['status']) {
				$data['status'] = language::get('text_status_' . $recurring_info['status']);
			} else {
				$data['status'] = '';
			}

			$data['payment_method'] = $recurring_info['payment_method'];

			$data['order_id'] = $recurring_info['order_id'];
			$data['product_name'] = $recurring_info['product_name'];
			$data['product_quantity'] = $recurring_info['product_quantity'];
			$data['recurring_description'] = $recurring_info['recurring_description'];
			$data['reference'] = $recurring_info['reference'];

			// Transactions
			$data['transactions'] = array();

			$results = account_recurring::getOrderRecurringTransactions(request::$get['order_recurring_id']);

			foreach ($results as $result) {
				$data['transactions'][] = array(
					'date_added' => date(language::get('date_format_short'), strtotime($result['date_added'])),
					'type'       => $result['type'],
					'amount'     => currency::format($result['amount'], $recurring_info['currency_code'])
				);
			}

			$data['order'] = url::link('account/order/info', 'order_id=' . $recurring_info['order_id']);
			$data['product'] = url::link('product/product', 'product_id=' . $recurring_info['product_id']);

			$data['recurring'] = load::controller('extension/recurring/' . $recurring_info['payment_code']);

			$data['column_left'] = load::controller('common/column_left');
			$data['column_right'] = load::controller('common/column_right');
			$data['content_top'] = load::controller('common/content_top');
			$data['content_bottom'] = load::controller('common/content_bottom');
			$data['footer'] = load::controller('common/footer');
			$data['header'] = load::controller('common/header');

			response::setOutput(load::view('account/recurring_info', $data));
		} else {
			document::setTitle(language::get('text_recurring'));

			$data['breadcrumbs'] = array();

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_home'),
				'href' => url::link('common/home')
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_account'),
				'href' => url::link('account/account', '')
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('heading_title'),
				'href' => url::link('account/recurring', '')
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_recurring'),
				'href' => url::link('account/recurring/info', 'order_recurring_id=' . $order_recurring_id)
			);

			$data['continue'] = url::link('account/recurring', '');

			$data['column_left'] = load::controller('common/column_left');
			$data['column_right'] = load::controller('common/column_right');
			$data['content_top'] = load::controller('common/content_top');
			$data['content_bottom'] = load::controller('common/content_bottom');
			$data['footer'] = load::controller('common/footer');
			$data['header'] = load::controller('common/header');

			response::setOutput(load::view('error/not_found', $data));
		}
	}
}
