<?php
class ControllerAccountRegister {
	private static $error = array();

	public static function index() {
		if (customer::isLogged()) {
			response::redirect(url::link('account/account', ''));
		}

		load::language('account/register');

		document::setTitle(language::get('heading_title'));

		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment-with-locales.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.js');
		document::addStyle('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.css');


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			$customer_id = account_customer::addCustomer(request::$post);

			// Clear any previous login attempts for unregistered accounts.
			account_customer::deleteLoginAttempts(request::$post['email']);

			customer::login(request::$post['email'], request::$post['password']);

			unset(session::$data['guest']);

			response::redirect(url::link('account/success'));
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_register'),
			'href' => url::link('account/register', '')
		);
		$data['text_account_already'] = sprintf(language::get('text_account_already'), url::link('account/login', ''));

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['firstname'])) {
			$data['error_firstname'] = self::$error['firstname'];
		} else {
			$data['error_firstname'] = '';
		}

		if (isset(self::$error['lastname'])) {
			$data['error_lastname'] = self::$error['lastname'];
		} else {
			$data['error_lastname'] = '';
		}

		if (isset(self::$error['email'])) {
			$data['error_email'] = self::$error['email'];
		} else {
			$data['error_email'] = '';
		}

		if (isset(self::$error['telephone'])) {
			$data['error_telephone'] = self::$error['telephone'];
		} else {
			$data['error_telephone'] = '';
		}

		if (isset(self::$error['custom_field'])) {
			$data['error_custom_field'] = self::$error['custom_field'];
		} else {
			$data['error_custom_field'] = array();
		}

		if (isset(self::$error['password'])) {
			$data['error_password'] = self::$error['password'];
		} else {
			$data['error_password'] = '';
		}

		if (isset(self::$error['confirm'])) {
			$data['error_confirm'] = self::$error['confirm'];
		} else {
			$data['error_confirm'] = '';
		}

		$data['action'] = url::link('account/register', '');

		$data['customer_groups'] = array();

		if (is_array(config('customer_group_display'))) {

			$customer_groups = account_customer_group::getCustomerGroups();

			foreach ($customer_groups as $customer_group) {
				if (in_array($customer_group['customer_group_id'], config('customer_group_display'))) {
					$data['customer_groups'][] = $customer_group;
				}
			}
		}

		if (isset(request::$post['customer_group_id'])) {
			$data['customer_group_id'] = request::$post['customer_group_id'];
		} else {
			$data['customer_group_id'] = config('customer_group_id');
		}

		if (isset(request::$post['firstname'])) {
			$data['firstname'] = request::$post['firstname'];
		} else {
			$data['firstname'] = '';
		}

		if (isset(request::$post['lastname'])) {
			$data['lastname'] = request::$post['lastname'];
		} else {
			$data['lastname'] = '';
		}

		if (isset(request::$post['email'])) {
			$data['email'] = request::$post['email'];
		} else {
			$data['email'] = '';
		}

		if (isset(request::$post['telephone'])) {
			$data['telephone'] = request::$post['telephone'];
		} else {
			$data['telephone'] = '';
		}

		// Custom Fields
		$data['custom_fields'] = array();
		
		
		$custom_fields = account_custom_field::getCustomFields();
		
		foreach ($custom_fields as $custom_field) {
			if ($custom_field['location'] == 'account') {
				$data['custom_fields'][] = $custom_field;
			}
		}
		
		if (isset(request::$post['custom_field']['account'])) {
			$data['register_custom_field'] = request::$post['custom_field']['account'];
		} else {
			$data['register_custom_field'] = array();
		}

		if (isset(request::$post['password'])) {
			$data['password'] = request::$post['password'];
		} else {
			$data['password'] = '';
		}

		if (isset(request::$post['confirm'])) {
			$data['confirm'] = request::$post['confirm'];
		} else {
			$data['confirm'] = '';
		}

		if (isset(request::$post['newsletter'])) {
			$data['newsletter'] = request::$post['newsletter'];
		} else {
			$data['newsletter'] = '';
		}

		// Captcha
		if (config::get('captcha_' . config('captcha') . '_status') && in_array('register', (array)config('captcha_page'))) {
			$data['captcha'] = load::controller('extension/captcha/' . config('captcha'), self::$error);
		} else {
			$data['captcha'] = '';
		}

		if (config('account_id')) {

			$information_info = catalog_information::getInformation(config('account_id'));

			if ($information_info) {
				$data['text_agree'] = sprintf(language::get('text_agree'), url::link('information/information/agree', 'information_id=' . config('account_id')), $information_info['title']);
			} else {
				$data['text_agree'] = '';
			}
		} else {
			$data['text_agree'] = '';
		}

		if (isset(request::$post['agree'])) {
			$data['agree'] = request::$post['agree'];
		} else {
			$data['agree'] = false;
		}

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/register', $data));
	}

	private static function validate() {
		if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
			self::$error['firstname'] = language::get('error_firstname');
		}

		if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
			self::$error['lastname'] = language::get('error_lastname');
		}

		if ((utf8_strlen(request::$post['email']) > 96) || !filter_var(request::$post['email'], FILTER_VALIDATE_EMAIL)) {
			self::$error['email'] = language::get('error_email');
		}

		if (account_customer::getTotalCustomersByEmail(request::$post['email'])) {
			self::$error['warning'] = language::get('error_exists');
		}

		if ((utf8_strlen(request::$post['telephone']) < 3) || (utf8_strlen(request::$post['telephone']) > 32)) {
			self::$error['telephone'] = language::get('error_telephone');
		}

		// Customer Group
		if (isset(request::$post['customer_group_id']) && is_array(config('customer_group_display')) && in_array(request::$post['customer_group_id'], config('customer_group_display'))) {
			$customer_group_id = request::$post['customer_group_id'];
		} else {
			$customer_group_id = config('customer_group_id');
		}

		// Custom field validation

		$custom_fields = account_custom_field::getCustomFields($customer_group_id);

		foreach ($custom_fields as $custom_field) {
			if ($custom_field['location'] == 'account') {
				if ($custom_field['required'] && empty(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']])) {
					self::$error['custom_field'][$custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
				} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
					self::$error['custom_field'][$custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
				}
			}
		}

		if ((utf8_strlen(html_entity_decode(request::$post['password'], ENT_QUOTES, 'UTF-8')) < 4) || (utf8_strlen(html_entity_decode(request::$post['password'], ENT_QUOTES, 'UTF-8')) > 40)) {
			self::$error['password'] = language::get('error_password');
		}

		if (request::$post['confirm'] != request::$post['password']) {
			self::$error['confirm'] = language::get('error_confirm');
		}

		// Captcha
		if (config::get('captcha_' . config('captcha') . '_status') && in_array('register', (array)config('captcha_page'))) {
			$captcha = load::controller('extension/captcha/' . config('captcha') . '/validate');

			if ($captcha) {
				self::$error['captcha'] = $captcha;
			}
		}

		// Agree to terms
		if (config('account_id')) {

			$information_info = catalog_information::getInformation(config('account_id'));

			if ($information_info && !isset(request::$post['agree'])) {
				self::$error['warning'] = sprintf(language::get('error_agree'), $information_info['title']);
			}
		}
		
		return !self::$error;
	}

	public static function customfield() {
		$json = array();


		// Customer Group
		if (isset(request::$get['customer_group_id']) && is_array(config('customer_group_display')) && in_array(request::$get['customer_group_id'], config('customer_group_display'))) {
			$customer_group_id = request::$get['customer_group_id'];
		} else {
			$customer_group_id = config('customer_group_id');
		}

		$custom_fields = account_custom_field::getCustomFields($customer_group_id);

		foreach ($custom_fields as $custom_field) {
			$json[] = array(
				'custom_field_id' => $custom_field['custom_field_id'],
				'required'        => $custom_field['required']
			);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}