<?php
class ControllerAccountReturn {
	private static $error = array();

	public static function index() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/return', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('account/return');

		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		$url = '';

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('account/return', $url)
		);


		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		$limit = 10;

		$data['returns'] = array();

		$return_total = account_return::getTotalReturns();

		$results = account_return::getReturns(($page - 1) * $limit, $limit);

		foreach ($results as $result) {
			$data['returns'][] = array(
				'return_id'  => $result['return_id'],
				'order_id'   => $result['order_id'],
				'name'       => $result['firstname'] . ' ' . $result['lastname'],
				'status'     => $result['status'],
				'date_added' => date(language::get('date_format_short'), strtotime($result['date_added'])),
				'href'       => url::link('account/return/info', 'return_id=' . $result['return_id'] . $url)
			);
		}

		$pagination = new Pagination();
		$pagination::$total = $return_total;
		$pagination::$page = $page;
		$pagination::$limit = $limit;
		$pagination::$url = url::link('account/return', 'page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($return_total) ? (($page - 1) * $limit) + 1 : 0, ((($page - 1) * $limit) > ($return_total - $limit)) ? $return_total : ((($page - 1) * $limit) + $limit), $return_total, ceil($return_total / $limit));

		$data['continue'] = url::link('account/account', '');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/return_list', $data));
	}

	public static function info() {
		load::language('account/return');

		if (isset(request::$get['return_id'])) {
			$return_id = (int)request::$get['return_id'];
		} else {
			$return_id = 0;
		}

		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/return/info', 'return_id=' . $return_id);

			response::redirect(url::link('account/login', ''));
		}


		$return_info = account_return::getReturn($return_id);

		if ($return_info) {
			document::setTitle(language::get('text_return'));

			$data['breadcrumbs'] = array();

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_home'),
				'href' => url::link('common/home', '')
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_account'),
				'href' => url::link('account/account', '')
			);

			$url = '';

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			$data['breadcrumbs'][] = array(
				'text' => language::get('heading_title'),
				'href' => url::link('account/return', $url)
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_return'),
				'href' => url::link('account/return/info', 'return_id=' . request::$get['return_id'] . $url)
			);

			$data['return_id'] = $return_info['return_id'];
			$data['order_id'] = $return_info['order_id'];
			$data['date_ordered'] = date(language::get('date_format_short'), strtotime($return_info['date_ordered']));
			$data['date_added'] = date(language::get('date_format_short'), strtotime($return_info['date_added']));
			$data['firstname'] = $return_info['firstname'];
			$data['lastname'] = $return_info['lastname'];
			$data['email'] = $return_info['email'];
			$data['telephone'] = $return_info['telephone'];
			$data['product'] = $return_info['product'];
			$data['model'] = $return_info['model'];
			$data['quantity'] = $return_info['quantity'];
			$data['reason'] = $return_info['reason'];
			$data['opened'] = $return_info['opened'] ? language::get('text_yes') : language::get('text_no');
			$data['comment'] = nl2br($return_info['comment']);
			$data['action'] = $return_info['action'];

			$data['histories'] = array();

			$results = account_return::getReturnHistories(request::$get['return_id']);

			foreach ($results as $result) {
				$data['histories'][] = array(
					'date_added' => date(language::get('date_format_short'), strtotime($result['date_added'])),
					'status'     => $result['status'],
					'comment'    => nl2br($result['comment'])
				);
			}

			$data['continue'] = url::link('account/return', $url);

			$data['column_left'] = load::controller('common/column_left');
			$data['column_right'] = load::controller('common/column_right');
			$data['content_top'] = load::controller('common/content_top');
			$data['content_bottom'] = load::controller('common/content_bottom');
			$data['footer'] = load::controller('common/footer');
			$data['header'] = load::controller('common/header');

			response::setOutput(load::view('account/return_info', $data));
		} else {
			document::setTitle(language::get('text_return'));

			$data['breadcrumbs'] = array();

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_home'),
				'href' => url::link('common/home')
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_account'),
				'href' => url::link('account/account', '')
			);

			$data['breadcrumbs'][] = array(
				'text' => language::get('heading_title'),
				'href' => url::link('account/return', '')
			);

			$url = '';

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_return'),
				'href' => url::link('account/return/info', 'return_id=' . $return_id . $url)
			);

			$data['continue'] = url::link('account/return', '');

			$data['column_left'] = load::controller('common/column_left');
			$data['column_right'] = load::controller('common/column_right');
			$data['content_top'] = load::controller('common/content_top');
			$data['content_bottom'] = load::controller('common/content_bottom');
			$data['footer'] = load::controller('common/footer');
			$data['header'] = load::controller('common/header');

			response::setOutput(load::view('error/not_found', $data));
		}
	}

	public static function add() {
		load::language('account/return');


		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			account_return::addReturn(request::$post);

			response::redirect(url::link('account/return/success', ''));
		}

		document::setTitle(language::get('heading_title'));
		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment-with-locales.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.js');
		document::addStyle('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.css');

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('account/return/add', '')
		);

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['order_id'])) {
			$data['error_order_id'] = self::$error['order_id'];
		} else {
			$data['error_order_id'] = '';
		}

		if (isset(self::$error['firstname'])) {
			$data['error_firstname'] = self::$error['firstname'];
		} else {
			$data['error_firstname'] = '';
		}

		if (isset(self::$error['lastname'])) {
			$data['error_lastname'] = self::$error['lastname'];
		} else {
			$data['error_lastname'] = '';
		}

		if (isset(self::$error['email'])) {
			$data['error_email'] = self::$error['email'];
		} else {
			$data['error_email'] = '';
		}

		if (isset(self::$error['telephone'])) {
			$data['error_telephone'] = self::$error['telephone'];
		} else {
			$data['error_telephone'] = '';
		}

		if (isset(self::$error['product'])) {
			$data['error_product'] = self::$error['product'];
		} else {
			$data['error_product'] = '';
		}

		if (isset(self::$error['model'])) {
			$data['error_model'] = self::$error['model'];
		} else {
			$data['error_model'] = '';
		}

		if (isset(self::$error['reason'])) {
			$data['error_reason'] = self::$error['reason'];
		} else {
			$data['error_reason'] = '';
		}

		$data['action'] = url::link('account/return/add', '');


		if (isset(request::$get['order_id'])) {
			$order_info = account_order::getOrder(request::$get['order_id']);
		}


		if (isset(request::$get['product_id'])) {
			$product_info = catalog_product::getProduct(request::$get['product_id']);
		}

		if (isset(request::$post['order_id'])) {
			$data['order_id'] = request::$post['order_id'];
		} elseif (!empty($order_info)) {
			$data['order_id'] = $order_info['order_id'];
		} else {
			$data['order_id'] = '';
		}

		if (isset(request::$post['product_id'])) {
			$data['product_id'] = request::$post['product_id'];
		} elseif (!empty($product_info)) {
			$data['product_id'] = $product_info['product_id'];
		} else {
			$data['product_id'] = '';
		}

		if (isset(request::$post['date_ordered'])) {
			$data['date_ordered'] = request::$post['date_ordered'];
		} elseif (!empty($order_info)) {
			$data['date_ordered'] = date('Y-m-d', strtotime($order_info['date_added']));
		} else {
			$data['date_ordered'] = '';
		}

		if (isset(request::$post['firstname'])) {
			$data['firstname'] = request::$post['firstname'];
		} elseif (!empty($order_info)) {
			$data['firstname'] = $order_info['firstname'];
		} else {
			$data['firstname'] = customer::getFirstName();
		}

		if (isset(request::$post['lastname'])) {
			$data['lastname'] = request::$post['lastname'];
		} elseif (!empty($order_info)) {
			$data['lastname'] = $order_info['lastname'];
		} else {
			$data['lastname'] = customer::getLastName();
		}

		if (isset(request::$post['email'])) {
			$data['email'] = request::$post['email'];
		} elseif (!empty($order_info)) {
			$data['email'] = $order_info['email'];
		} else {
			$data['email'] = customer::getEmail();
		}

		if (isset(request::$post['telephone'])) {
			$data['telephone'] = request::$post['telephone'];
		} elseif (!empty($order_info)) {
			$data['telephone'] = $order_info['telephone'];
		} else {
			$data['telephone'] = customer::getTelephone();
		}

		if (isset(request::$post['product'])) {
			$data['product'] = request::$post['product'];
		} elseif (!empty($product_info)) {
			$data['product'] = $product_info['name'];
		} else {
			$data['product'] = '';
		}

		if (isset(request::$post['model'])) {
			$data['model'] = request::$post['model'];
		} elseif (!empty($product_info)) {
			$data['model'] = $product_info['model'];
		} else {
			$data['model'] = '';
		}

		if (isset(request::$post['quantity'])) {
			$data['quantity'] = request::$post['quantity'];
		} else {
			$data['quantity'] = 1;
		}

		if (isset(request::$post['opened'])) {
			$data['opened'] = request::$post['opened'];
		} else {
			$data['opened'] = false;
		}

		if (isset(request::$post['return_reason_id'])) {
			$data['return_reason_id'] = request::$post['return_reason_id'];
		} else {
			$data['return_reason_id'] = '';
		}


		$data['return_reasons'] = localisation_return_reason::getReturnReasons();

		if (isset(request::$post['comment'])) {
			$data['comment'] = request::$post['comment'];
		} else {
			$data['comment'] = '';
		}

		// Captcha
		if (config::get('captcha_' . config('captcha') . '_status') && in_array('return', (array)config('captcha_page'))) {
			$data['captcha'] = load::controller('extension/captcha/' . config('captcha'), self::$error);
		} else {
			$data['captcha'] = '';
		}

		if (config('return_id')) {

			$information_info = catalog_information::getInformation(config('return_id'));

			if ($information_info) {
				$data['text_agree'] = sprintf(language::get('text_agree'), url::link('information/information/agree', 'information_id=' . config('return_id')), $information_info['title']);
			} else {
				$data['text_agree'] = '';
			}
		} else {
			$data['text_agree'] = '';
		}

		if (isset(request::$post['agree'])) {
			$data['agree'] = request::$post['agree'];
		} else {
			$data['agree'] = false;
		}

		$data['back'] = url::link('account/account', '');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/return_form', $data));
	}

	protected static function validate() {
		if (!request::$post['order_id']) {
			self::$error['order_id'] = language::get('error_order_id');
		}

		if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
			self::$error['firstname'] = language::get('error_firstname');
		}

		if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
			self::$error['lastname'] = language::get('error_lastname');
		}

		if ((utf8_strlen(request::$post['email']) > 96) || !filter_var(request::$post['email'], FILTER_VALIDATE_EMAIL)) {
			self::$error['email'] = language::get('error_email');
		}

		if ((utf8_strlen(request::$post['telephone']) < 3) || (utf8_strlen(request::$post['telephone']) > 32)) {
			self::$error['telephone'] = language::get('error_telephone');
		}

		if ((utf8_strlen(request::$post['product']) < 1) || (utf8_strlen(request::$post['product']) > 255)) {
			self::$error['product'] = language::get('error_product');
		}

		if ((utf8_strlen(request::$post['model']) < 1) || (utf8_strlen(request::$post['model']) > 64)) {
			self::$error['model'] = language::get('error_model');
		}

		if (empty(request::$post['return_reason_id'])) {
			self::$error['reason'] = language::get('error_reason');
		}

		if (config::get('captcha_' . config('captcha') . '_status') && in_array('return', (array)config('captcha_page'))) {
			$captcha = load::controller('extension/captcha/' . config('captcha') . '/validate');

			if ($captcha) {
				self::$error['captcha'] = $captcha;
			}
		}

		if (config('return_id')) {

			$information_info = catalog_information::getInformation(config('return_id'));

			if ($information_info && !isset(request::$post['agree'])) {
				self::$error['warning'] = sprintf(language::get('error_agree'), $information_info['title']);
			}
		}

		return !self::$error;
	}

	public static function success() {
		load::language('account/return');

		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('account/return', '')
		);

		$data['continue'] = url::link('common/home');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('common/success', $data));
	}
}
