<?php
class ControllerAccountVoucher {
	private static $error = array();

	public static function index() {
		load::language('account/voucher');

		document::setTitle(language::get('heading_title'));

		if (!isset(session::$data['vouchers'])) {
			session::$data['vouchers'] = array();
		}

		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			session::$data['vouchers'][mt_rand()] = array(
				'description'      => sprintf(language::get('text_for'), currency::format(request::$post['amount'], session::$data['currency'], 1.0), request::$post['to_name']),
				'to_name'          => request::$post['to_name'],
				'to_email'         => request::$post['to_email'],
				'from_name'        => request::$post['from_name'],
				'from_email'       => request::$post['from_email'],
				'voucher_theme_id' => request::$post['voucher_theme_id'],
				'message'          => request::$post['message'],
				'amount'           => currency::convert(request::$post['amount'], session::$data['currency'], config('currency'))
			);

			response::redirect(url::link('account/voucher/success'));
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_voucher'),
			'href' => url::link('account/voucher', '')
		);

		$data['help_amount'] = sprintf(language::get('help_amount'), currency::format(config('voucher_min'), session::$data['currency']), currency::format(config('voucher_max'), session::$data['currency']));

		if (isset(self::$error['warning'])) {
			$data['error_warning'] = self::$error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset(self::$error['to_name'])) {
			$data['error_to_name'] = self::$error['to_name'];
		} else {
			$data['error_to_name'] = '';
		}

		if (isset(self::$error['to_email'])) {
			$data['error_to_email'] = self::$error['to_email'];
		} else {
			$data['error_to_email'] = '';
		}

		if (isset(self::$error['from_name'])) {
			$data['error_from_name'] = self::$error['from_name'];
		} else {
			$data['error_from_name'] = '';
		}

		if (isset(self::$error['from_email'])) {
			$data['error_from_email'] = self::$error['from_email'];
		} else {
			$data['error_from_email'] = '';
		}

		if (isset(self::$error['theme'])) {
			$data['error_theme'] = self::$error['theme'];
		} else {
			$data['error_theme'] = '';
		}

		if (isset(self::$error['amount'])) {
			$data['error_amount'] = self::$error['amount'];
		} else {
			$data['error_amount'] = '';
		}

		$data['action'] = url::link('account/voucher', '');

		if (isset(request::$post['to_name'])) {
			$data['to_name'] = request::$post['to_name'];
		} else {
			$data['to_name'] = '';
		}

		if (isset(request::$post['to_email'])) {
			$data['to_email'] = request::$post['to_email'];
		} else {
			$data['to_email'] = '';
		}

		if (isset(request::$post['from_name'])) {
			$data['from_name'] = request::$post['from_name'];
		} elseif (customer::isLogged()) {
			$data['from_name'] = customer::getFirstName() . ' '  . customer::getLastName();
		} else {
			$data['from_name'] = '';
		}

		if (isset(request::$post['from_email'])) {
			$data['from_email'] = request::$post['from_email'];
		} elseif (customer::isLogged()) {
			$data['from_email'] = customer::getEmail();
		} else {
			$data['from_email'] = '';
		}


		$data['voucher_themes'] = extension_total_voucher_theme::getVoucherThemes();

		if (isset(request::$post['voucher_theme_id'])) {
			$data['voucher_theme_id'] = request::$post['voucher_theme_id'];
		} else {
			$data['voucher_theme_id'] = '';
		}

		if (isset(request::$post['message'])) {
			$data['message'] = request::$post['message'];
		} else {
			$data['message'] = '';
		}

		if (isset(request::$post['amount'])) {
			$data['amount'] = request::$post['amount'];
		} else {
			$data['amount'] = currency::format(config('voucher_min'), config('currency'), false, false);
		}

		if (isset(request::$post['agree'])) {
			$data['agree'] = request::$post['agree'];
		} else {
			$data['agree'] = false;
		}

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/voucher', $data));
	}

	public static function success() {
		load::language('account/voucher');

		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('account/voucher')
		);

		$data['continue'] = url::link('checkout/cart');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('common/success', $data));
	}

	protected static function validate() {
		if ((utf8_strlen(request::$post['to_name']) < 1) || (utf8_strlen(request::$post['to_name']) > 64)) {
			self::$error['to_name'] = language::get('error_to_name');
		}

		if ((utf8_strlen(request::$post['to_email']) > 96) || !filter_var(request::$post['to_email'], FILTER_VALIDATE_EMAIL)) {
			self::$error['to_email'] = language::get('error_email');
		}

		if ((utf8_strlen(request::$post['from_name']) < 1) || (utf8_strlen(request::$post['from_name']) > 64)) {
			self::$error['from_name'] = language::get('error_from_name');
		}

		if ((utf8_strlen(request::$post['from_email']) > 96) || !filter_var(request::$post['from_email'], FILTER_VALIDATE_EMAIL)) {
			self::$error['from_email'] = language::get('error_email');
		}

		if (!isset(request::$post['voucher_theme_id'])) {
			self::$error['theme'] = language::get('error_theme');
		}

		if ((!isset(request::$post['amount'])) || (!is_numeric(request::$post['amount'])) || (currency::convert(request::$post['amount'], session::$data['currency'], config('currency')) < config('voucher_min')) || (currency::convert(request::$post['amount'], session::$data['currency'], config('currency')) > config('voucher_max'))) {
			self::$error['amount'] = sprintf(language::get('error_amount'), currency::format(config('voucher_min'), session::$data['currency']), currency::format(config('voucher_max'), session::$data['currency']));
		}

		if (!isset(request::$post['agree'])) {
			self::$error['warning'] = language::get('error_agree');
		}

		return !self::$error;
	}
}
