<?php
class ControllerAccountWishList {
	public static function index() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/wishlist', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('account/wishlist');




		if (isset(request::$get['remove'])) {
			// Remove Wishlist
			account_wishlist::deleteWishlist(request::$get['remove']);

			session::$data['success'] = language::get('text_remove');

			response::redirect(url::link('account/wishlist'));
		}

		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('account/wishlist')
		);

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		$data['products'] = array();

		$results = account_wishlist::getWishlist();

		foreach ($results as $result) {
			$product_info = catalog_product::getProduct($result['product_id']);

			if ($product_info) {
				if ($product_info['image']) {
					$image = tool_image::resize($product_info['image'], config::get('theme_' . config('theme') . '_image_wishlist_width'), config::get('theme_' . config('theme') . '_image_wishlist_height'));
				} else {
					$image = false;
				}

				if ($product_info['quantity'] <= 0) {
					$stock = $product_info['stock_status'];
				} elseif (config('stock_display')) {
					$stock = $product_info['quantity'];
				} else {
					$stock = language::get('text_instock');
				}

				if (customer::isLogged() || !config('customer_price')) {
					$price = currency::format(tax::calculate($product_info['price'], $product_info['tax_class_id'], config('tax')), session::$data['currency']);
				} else {
					$price = false;
				}

				if ((float)$product_info['special']) {
					$special = currency::format(tax::calculate($product_info['special'], $product_info['tax_class_id'], config('tax')), session::$data['currency']);
				} else {
					$special = false;
				}

				$data['products'][] = array(
					'product_id' => $product_info['product_id'],
					'thumb'      => $image,
					'name'       => $product_info['name'],
					'model'      => $product_info['model'],
					'stock'      => $stock,
					'price'      => $price,
					'special'    => $special,
					'href'       => url::link('product/product', 'product_id=' . $product_info['product_id']),
					'remove'     => url::link('account/wishlist', 'remove=' . $product_info['product_id'])
				);
			} else {
				account_wishlist::deleteWishlist($result['product_id']);
			}
		}

		$data['continue'] = url::link('account/account', '');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('account/wishlist', $data));
	}

	public static function add() {
		load::language('account/wishlist');

		$json = array();

		if (isset(request::$post['product_id'])) {
			$product_id = request::$post['product_id'];
		} else {
			$product_id = 0;
		}


		$product_info = catalog_product::getProduct($product_id);

		if ($product_info) {
			if (customer::isLogged()) {
				// Edit customers cart

				account_wishlist::addWishlist(request::$post['product_id']);

				$json['success'] = sprintf(language::get('text_success'), url::link('product/product', 'product_id=' . (int)request::$post['product_id']), $product_info['name'], url::link('account/wishlist'));

				$json['total'] = sprintf(language::get('text_wishlist'), account_wishlist::getTotalWishlist());
			} else {
				if (!isset(session::$data['wishlist'])) {
					session::$data['wishlist'] = array();
				}

				session::$data['wishlist'][] = request::$post['product_id'];

				session::$data['wishlist'] = array_unique(session::$data['wishlist']);

				$json['success'] = sprintf(language::get('text_login'), url::link('account/login', ''), url::link('account/register', '', true), url::link('product/product', 'product_id=' . (int)request::$post['product_id']), $product_info['name'], url::link('account/wishlist'));

				$json['total'] = sprintf(language::get('text_wishlist'), (isset(session::$data['wishlist']) ? count(session::$data['wishlist']) : 0));
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
