<?php
class ControllerApiCart {
	public static function add() {
		load::language('api/cart');

		$json = array();
			
		if (!isset(session::$data['api_id'])) {
			$json['error']['warning'] = language::get('error_permission');
		} else {
			if (isset(request::$post['product'])) {
				cart::clear();

				foreach (request::$post['product'] as $product) {
					if (isset($product['option'])) {
						$option = $product['option'];
					} else {
						$option = array();
					}

					cart::add($product['product_id'], $product['quantity'], $option);
				}

				$json['success'] = language::get('text_success');

				unset(session::$data['shipping_method']);
				unset(session::$data['shipping_methods']);
				unset(session::$data['payment_method']);
				unset(session::$data['payment_methods']);
			} elseif (isset(request::$post['product_id'])) {

				$product_info = catalog_product::getProduct(request::$post['product_id']);

				if ($product_info) {
					if (isset(request::$post['quantity'])) {
						$quantity = request::$post['quantity'];
					} else {
						$quantity = 1;
					}

					if (isset(request::$post['option'])) {
						$option = array_filter(request::$post['option']);
					} else {
						$option = array();
					}

					$product_options = catalog_product::getProductOptions(request::$post['product_id']);

					foreach ($product_options as $product_option) {
						if ($product_option['required'] && empty($option[$product_option['product_option_id']])) {
							$json['error']['option'][$product_option['product_option_id']] = sprintf(language::get('error_required'), $product_option['name']);
						}
					}

					if (!isset($json['error']['option'])) {
						cart::add(request::$post['product_id'], $quantity, $option);

						$json['success'] = language::get('text_success');

						unset(session::$data['shipping_method']);
						unset(session::$data['shipping_methods']);
						unset(session::$data['payment_method']);
						unset(session::$data['payment_methods']);
					}
				} else {
					$json['error']['store'] = language::get('error_store');
				}
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function edit() {
		load::language('api/cart');

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} else {
			cart::update(request::$post['key'], request::$post['quantity']);

			$json['success'] = language::get('text_success');

			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
			unset(session::$data['payment_method']);
			unset(session::$data['payment_methods']);
			unset(session::$data['reward']);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function remove() {
		load::language('api/cart');

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} else {
			// Remove
			if (isset(request::$post['key'])) {
				cart::remove(request::$post['key']);

				unset(session::$data['vouchers'][request::$post['key']]);

				$json['success'] = language::get('text_success');

				unset(session::$data['shipping_method']);
				unset(session::$data['shipping_methods']);
				unset(session::$data['payment_method']);
				unset(session::$data['payment_methods']);
				unset(session::$data['reward']);
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function products() {
		load::language('api/cart');

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error']['warning'] = language::get('error_permission');
		} else {
			// Stock
			if (!cart::hasStock() && (!config('stock_checkout') || config('stock_warning'))) {
				$json['error']['stock'] = language::get('error_stock');
			}

			// Products
			$json['products'] = array();

			$products = cart::getProducts();

			foreach ($products as $product) {
				$product_total = 0;

				foreach ($products as $product_2) {
					if ($product_2['product_id'] == $product['product_id']) {
						$product_total += $product_2['quantity'];
					}
				}

				if ($product['minimum'] > $product_total) {
					$json['error']['minimum'][] = sprintf(language::get('error_minimum'), $product['name'], $product['minimum']);
				}

				$option_data = array();

				foreach ($product['option'] as $option) {
					$option_data[] = array(
						'product_option_id'       => $option['product_option_id'],
						'product_option_value_id' => $option['product_option_value_id'],
						'name'                    => $option['name'],
						'value'                   => $option['value'],
						'type'                    => $option['type']
					);
				}

				$json['products'][] = array(
					'cart_id'    => $product['cart_id'],
					'product_id' => $product['product_id'],
					'name'       => $product['name'],
					'model'      => $product['model'],
					'option'     => $option_data,
					'quantity'   => $product['quantity'],
					'stock'      => $product['stock'] ? true : !(!config('stock_checkout') || config('stock_warning')),
					'shipping'   => $product['shipping'],
					'price'      => currency::format(tax::calculate($product['price'], $product['tax_class_id'], config('tax')), session::$data['currency']),
					'total'      => currency::format(tax::calculate($product['price'], $product['tax_class_id'], config('tax')) * $product['quantity'], session::$data['currency']),
					'reward'     => $product['reward']
				);
			}

			// Voucher
			$json['vouchers'] = array();

			if (!empty(session::$data['vouchers'])) {
				foreach (session::$data['vouchers'] as $key => $voucher) {
					$json['vouchers'][] = array(
						'code'             => $voucher['code'],
						'description'      => $voucher['description'],
						'from_name'        => $voucher['from_name'],
						'from_email'       => $voucher['from_email'],
						'to_name'          => $voucher['to_name'],
						'to_email'         => $voucher['to_email'],
						'voucher_theme_id' => $voucher['voucher_theme_id'],
						'message'          => $voucher['message'],
						'price'            => currency::format($voucher['amount'], session::$data['currency']),			
						'amount'           => $voucher['amount']
					);
				}
			}

			// Totals

			$totals = array();
			$taxes = cart::getTaxes();
			$total = 0;

			// Because __call can not keep var references so we put them into an array. 
			$total_data = array(
				'totals' => &$totals,
				'taxes'  => &$taxes,
				'total'  => &$total
			);
			
			$sort_order = array();

			$results = setting_extension::getExtensions('total');

			foreach ($results as $key => $value) {
				$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
			}

			array_multisort($sort_order, SORT_ASC, $results);

			foreach ($results as $result) {
				if (config::get('total_' . $result['code'] . '_status')) {
					
					// We have to put the totals in an array so that they pass by reference.
					call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
				}
			}

			$sort_order = array();

			foreach ($totals as $key => $value) {
				$sort_order[$key] = $value['sort_order'];
			}

			array_multisort($sort_order, SORT_ASC, $totals);

			$json['totals'] = array();

			foreach ($totals as $total) {
				$json['totals'][] = array(
					'title' => $total['title'],
					'text'  => currency::format($total['value'], session::$data['currency'])
				);
			}
		}
		
		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
