<?php
class ControllerApiCustomer {
	public static function index() {
		load::language('api/customer');

		// Delete past customer in case there is an error
		unset(session::$data['customer']);

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error']['warning'] = language::get('error_permission');
		} else {
			// Add keys for missing post vars
			$keys = array(
				'customer_id',
				'customer_group_id',
				'firstname',
				'lastname',
				'email',
				'telephone',
			);

			foreach ($keys as $key) {
				if (!isset(request::$post[$key])) {
					request::$post[$key] = '';
				}
			}

			// Customer
			if (request::$post['customer_id']) {

				$customer_info = account_customer::getCustomer(request::$post['customer_id']);

				if (!$customer_info || !customer::login($customer_info['email'], '', true)) {
					$json['error']['warning'] = language::get('error_customer');
				}
			}

			if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
				$json['error']['firstname'] = language::get('error_firstname');
			}

			if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
				$json['error']['lastname'] = language::get('error_lastname');
			}

			if ((utf8_strlen(request::$post['email']) > 96) || (!filter_var(request::$post['email'], FILTER_VALIDATE_EMAIL))) {
				$json['error']['email'] = language::get('error_email');
			}

			if ((utf8_strlen(request::$post['telephone']) < 3) || (utf8_strlen(request::$post['telephone']) > 32)) {
				$json['error']['telephone'] = language::get('error_telephone');
			}

			// Customer Group
			if (is_array(config('customer_group_display')) && in_array(request::$post['customer_group_id'], config('customer_group_display'))) {
				$customer_group_id = request::$post['customer_group_id'];
			} else {
				$customer_group_id = config('customer_group_id');
			}

			// Custom field validation

			$custom_fields = account_custom_field::getCustomFields($customer_group_id);

			foreach ($custom_fields as $custom_field) {
				if ($custom_field['location'] == 'account') { 
					if ($custom_field['required'] && empty(request::$post['custom_field'][$custom_field['custom_field_id']])) {
						$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
					} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var(request::$post['custom_field'][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
						$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
					}
				}
			}

			if (!$json) {
				session::$data['customer'] = array(
					'customer_id'       => request::$post['customer_id'],
					'customer_group_id' => $customer_group_id,
					'firstname'         => request::$post['firstname'],
					'lastname'          => request::$post['lastname'],
					'email'             => request::$post['email'],
					'telephone'         => request::$post['telephone'],
					'custom_field'      => isset(request::$post['custom_field']) ? request::$post['custom_field'] : array()
				);

				$json['success'] = language::get('text_success');
			}
		}
		
		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
