<?php
class ControllerApiOrder {
	public static function add() {
		load::language('api/order');

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} else {
			// Customer
			if (!isset(session::$data['customer'])) {
				$json['error'] = language::get('error_customer');
			}

			// Payment Address
			if (!isset(session::$data['payment_address'])) {
				$json['error'] = language::get('error_payment_address');
			}

			// Payment Method
			if (!$json && !empty(request::$post['payment_method'])) {
				if (empty(session::$data['payment_methods'])) {
					$json['error'] = language::get('error_no_payment');
				} elseif (!isset(session::$data['payment_methods'][request::$post['payment_method']])) {
					$json['error'] = language::get('error_payment_method');
				}

				if (!$json) {
					session::$data['payment_method'] = session::$data['payment_methods'][request::$post['payment_method']];
				}
			}

			if (!isset(session::$data['payment_method'])) {
				$json['error'] = language::get('error_payment_method');
			}

			// Shipping
			if (cart::hasShipping()) {
				// Shipping Address
				if (!isset(session::$data['shipping_address'])) {
					$json['error'] = language::get('error_shipping_address');
				}

				// Shipping Method
				if (!$json && !empty(request::$post['shipping_method'])) {
					if (empty(session::$data['shipping_methods'])) {
						$json['error'] = language::get('error_no_shipping');
					} else {
						$shipping = explode('.', request::$post['shipping_method']);

						if (!isset($shipping[0]) || !isset($shipping[1]) || !isset(session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]])) {
							$json['error'] = language::get('error_shipping_method');
						}
					}

					if (!$json) {
						session::$data['shipping_method'] = session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]];
					}
				}

				// Shipping Method
				if (!isset(session::$data['shipping_method'])) {
					$json['error'] = language::get('error_shipping_method');
				}
			} else {
				unset(session::$data['shipping_address']);
				unset(session::$data['shipping_method']);
				unset(session::$data['shipping_methods']);
			}

			// Cart
			if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
				$json['error'] = language::get('error_stock');
			}

			// Validate minimum quantity requirements.
			$products = cart::getProducts();

			foreach ($products as $product) {
				$product_total = 0;

				foreach ($products as $product_2) {
					if ($product_2['product_id'] == $product['product_id']) {
						$product_total += $product_2['quantity'];
					}
				}

				if ($product['minimum'] > $product_total) {
					$json['error'] = sprintf(language::get('error_minimum'), $product['name'], $product['minimum']);

					break;
				}
			}

			if (!$json) {
				$json['success'] = language::get('text_success');
				
				$order_data = array();

				// Store Details
				$order_data['invoice_prefix'] = config('invoice_prefix');
				$order_data['store_id'] = config('store_id');
				$order_data['store_name'] = config('name');
				$order_data['store_url'] = config('url');

				// Customer Details
				$order_data['customer_id'] = session::$data['customer']['customer_id'];
				$order_data['customer_group_id'] = session::$data['customer']['customer_group_id'];
				$order_data['firstname'] = session::$data['customer']['firstname'];
				$order_data['lastname'] = session::$data['customer']['lastname'];
				$order_data['email'] = session::$data['customer']['email'];
				$order_data['telephone'] = session::$data['customer']['telephone'];
				$order_data['custom_field'] = session::$data['customer']['custom_field'];

				// Payment Details
				$order_data['payment_firstname'] = session::$data['payment_address']['firstname'];
				$order_data['payment_lastname'] = session::$data['payment_address']['lastname'];
				$order_data['payment_company'] = session::$data['payment_address']['company'];
				$order_data['payment_address_1'] = session::$data['payment_address']['address_1'];
				$order_data['payment_address_2'] = session::$data['payment_address']['address_2'];
				$order_data['payment_city'] = session::$data['payment_address']['city'];
				$order_data['payment_postcode'] = session::$data['payment_address']['postcode'];
				$order_data['payment_zone'] = session::$data['payment_address']['zone'];
				$order_data['payment_zone_id'] = session::$data['payment_address']['zone_id'];
				$order_data['payment_country'] = session::$data['payment_address']['country'];
				$order_data['payment_country_id'] = session::$data['payment_address']['country_id'];
				$order_data['payment_address_format'] = session::$data['payment_address']['address_format'];
				$order_data['payment_custom_field'] = (isset(session::$data['payment_address']['custom_field']) ? session::$data['payment_address']['custom_field'] : array());

				if (isset(session::$data['payment_method']['title'])) {
					$order_data['payment_method'] = session::$data['payment_method']['title'];
				} else {
					$order_data['payment_method'] = '';
				}

				if (isset(session::$data['payment_method']['code'])) {
					$order_data['payment_code'] = session::$data['payment_method']['code'];
				} else {
					$order_data['payment_code'] = '';
				}

				// Shipping Details
				if (cart::hasShipping()) {
					$order_data['shipping_firstname'] = session::$data['shipping_address']['firstname'];
					$order_data['shipping_lastname'] = session::$data['shipping_address']['lastname'];
					$order_data['shipping_company'] = session::$data['shipping_address']['company'];
					$order_data['shipping_address_1'] = session::$data['shipping_address']['address_1'];
					$order_data['shipping_address_2'] = session::$data['shipping_address']['address_2'];
					$order_data['shipping_city'] = session::$data['shipping_address']['city'];
					$order_data['shipping_postcode'] = session::$data['shipping_address']['postcode'];
					$order_data['shipping_zone'] = session::$data['shipping_address']['zone'];
					$order_data['shipping_zone_id'] = session::$data['shipping_address']['zone_id'];
					$order_data['shipping_country'] = session::$data['shipping_address']['country'];
					$order_data['shipping_country_id'] = session::$data['shipping_address']['country_id'];
					$order_data['shipping_address_format'] = session::$data['shipping_address']['address_format'];
					$order_data['shipping_custom_field'] = (isset(session::$data['shipping_address']['custom_field']) ? session::$data['shipping_address']['custom_field'] : array());

					if (isset(session::$data['shipping_method']['title'])) {
						$order_data['shipping_method'] = session::$data['shipping_method']['title'];
					} else {
						$order_data['shipping_method'] = '';
					}

					if (isset(session::$data['shipping_method']['code'])) {
						$order_data['shipping_code'] = session::$data['shipping_method']['code'];
					} else {
						$order_data['shipping_code'] = '';
					}
				} else {
					$order_data['shipping_firstname'] = '';
					$order_data['shipping_lastname'] = '';
					$order_data['shipping_company'] = '';
					$order_data['shipping_address_1'] = '';
					$order_data['shipping_address_2'] = '';
					$order_data['shipping_city'] = '';
					$order_data['shipping_postcode'] = '';
					$order_data['shipping_zone'] = '';
					$order_data['shipping_zone_id'] = '';
					$order_data['shipping_country'] = '';
					$order_data['shipping_country_id'] = '';
					$order_data['shipping_address_format'] = '';
					$order_data['shipping_custom_field'] = array();
					$order_data['shipping_method'] = '';
					$order_data['shipping_code'] = '';
				}

				// Products
				$order_data['products'] = array();

				foreach (cart::getProducts() as $product) {
					$option_data = array();

					foreach ($product['option'] as $option) {
						$option_data[] = array(
							'product_option_id'       => $option['product_option_id'],
							'product_option_value_id' => $option['product_option_value_id'],
							'option_id'               => $option['option_id'],
							'option_value_id'         => $option['option_value_id'],
							'name'                    => $option['name'],
							'value'                   => $option['value'],
							'type'                    => $option['type']
						);
					}

					$order_data['products'][] = array(
						'product_id' => $product['product_id'],
						'name'       => $product['name'],
						'model'      => $product['model'],
						'option'     => $option_data,
						'download'   => $product['download'],
						'quantity'   => $product['quantity'],
						'subtract'   => $product['subtract'],
						'price'      => $product['price'],
						'total'      => $product['total'],
						'tax'        => tax::getTax($product['price'], $product['tax_class_id']),
						'reward'     => $product['reward']
					);
				}

				// Gift Voucher
				$order_data['vouchers'] = array();

				if (!empty(session::$data['vouchers'])) {
					foreach (session::$data['vouchers'] as $voucher) {
						$order_data['vouchers'][] = array(
							'description'      => $voucher['description'],
							'code'             => token(10),
							'to_name'          => $voucher['to_name'],
							'to_email'         => $voucher['to_email'],
							'from_name'        => $voucher['from_name'],
							'from_email'       => $voucher['from_email'],
							'voucher_theme_id' => $voucher['voucher_theme_id'],
							'message'          => $voucher['message'],
							'amount'           => $voucher['amount']
						);
					}
				}

				// Order Totals

				$totals = array();
				$taxes = cart::getTaxes();
				$total = 0;

				// Because __call can not keep var references so we put them into an array.
				$total_data = array(
					'totals' => &$totals,
					'taxes'  => &$taxes,
					'total'  => &$total
				);
			
				$sort_order = array();

				$results = setting_extension::getExtensions('total');

				foreach ($results as $key => $value) {
					$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
				}

				array_multisort($sort_order, SORT_ASC, $results);

				foreach ($results as $result) {
					if (config::get('total_' . $result['code'] . '_status')) {
						
						// We have to put the totals in an array so that they pass by reference.
						call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
					}
				}

				$sort_order = array();

				foreach ($total_data['totals'] as $key => $value) {
					$sort_order[$key] = $value['sort_order'];
				}

				array_multisort($sort_order, SORT_ASC, $total_data['totals']);

				$order_data = array_merge($order_data, $total_data);

				if (isset(request::$post['comment'])) {
					$order_data['comment'] = request::$post['comment'];
				} else {
					$order_data['comment'] = '';
				}

				if (isset(request::$post['affiliate_id'])) {
					$subtotal = cart::getSubTotal();

					// Affiliate

					$affiliate_info = account_customer::getAffiliate(request::$post['affiliate_id']);

					if ($affiliate_info) {
						$order_data['affiliate_id'] = $affiliate_info['customer_id'];
						$order_data['commission'] = ($subtotal / 100) * $affiliate_info['commission'];
					} else {
						$order_data['affiliate_id'] = 0;
						$order_data['commission'] = 0;
					}

					// Marketing
					$order_data['marketing_id'] = 0;
					$order_data['tracking'] = '';
				} else {
					$order_data['affiliate_id'] = 0;
					$order_data['commission'] = 0;
					$order_data['marketing_id'] = 0;
					$order_data['tracking'] = '';
				}

				$order_data['language_id'] = config('language_id');
				$order_data['currency_id'] = currency::getId(session::$data['currency']);
				$order_data['currency_code'] = session::$data['currency'];
				$order_data['currency_value'] = currency::getValue(session::$data['currency']);
				$order_data['ip'] = request::$server['REMOTE_ADDR'];

				if (!empty(request::$server['HTTP_X_FORWARDED_FOR'])) {
					$order_data['forwarded_ip'] = request::$server['HTTP_X_FORWARDED_FOR'];
				} elseif (!empty(request::$server['HTTP_CLIENT_IP'])) {
					$order_data['forwarded_ip'] = request::$server['HTTP_CLIENT_IP'];
				} else {
					$order_data['forwarded_ip'] = '';
				}

				if (isset(request::$server['HTTP_USER_AGENT'])) {
					$order_data['user_agent'] = request::$server['HTTP_USER_AGENT'];
				} else {
					$order_data['user_agent'] = '';
				}

				if (isset(request::$server['HTTP_ACCEPT_LANGUAGE'])) {
					$order_data['accept_language'] = request::$server['HTTP_ACCEPT_LANGUAGE'];
				} else {
					$order_data['accept_language'] = '';
				}


				$json['order_id'] = checkout_order::addOrder($order_data);

				// Set the order history
				if (isset(request::$post['order_status_id'])) {
					$order_status_id = request::$post['order_status_id'];
				} else {
					$order_status_id = config('order_status_id');
				}

				checkout_order::addOrderHistory($json['order_id'], $order_status_id);
				
				// clear cart since the order has already been successfully stored.
				cart::clear();
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function edit() {
		load::language('api/order');

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} else {

			if (isset(request::$get['order_id'])) {
				$order_id = request::$get['order_id'];
			} else {
				$order_id = 0;
			}

			$order_info = checkout_order::getOrder($order_id);

			if ($order_info) {
				// Customer
				if (!isset(session::$data['customer'])) {
					$json['error'] = language::get('error_customer');
				}

				// Payment Address
				if (!isset(session::$data['payment_address'])) {
					$json['error'] = language::get('error_payment_address');
				}

				// Payment Method
				if (!$json && !empty(request::$post['payment_method'])) {
					if (empty(session::$data['payment_methods'])) {
						$json['error'] = language::get('error_no_payment');
					} elseif (!isset(session::$data['payment_methods'][request::$post['payment_method']])) {
						$json['error'] = language::get('error_payment_method');
					}

					if (!$json) {
						session::$data['payment_method'] = session::$data['payment_methods'][request::$post['payment_method']];
					}
				}

				if (!isset(session::$data['payment_method'])) {
					$json['error'] = language::get('error_payment_method');
				}

				// Shipping
				if (cart::hasShipping()) {
					// Shipping Address
					if (!isset(session::$data['shipping_address'])) {
						$json['error'] = language::get('error_shipping_address');
					}

					// Shipping Method
					if (!$json && !empty(request::$post['shipping_method'])) {
						if (empty(session::$data['shipping_methods'])) {
							$json['error'] = language::get('error_no_shipping');
						} else {
							$shipping = explode('.', request::$post['shipping_method']);

							if (!isset($shipping[0]) || !isset($shipping[1]) || !isset(session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]])) {
								$json['error'] = language::get('error_shipping_method');
							}
						}

						if (!$json) {
							session::$data['shipping_method'] = session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]];
						}
					}

					if (!isset(session::$data['shipping_method'])) {
						$json['error'] = language::get('error_shipping_method');
					}
				} else {
					unset(session::$data['shipping_address']);
					unset(session::$data['shipping_method']);
					unset(session::$data['shipping_methods']);
				}

				// Cart
				if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
					$json['error'] = language::get('error_stock');
				}

				// Validate minimum quantity requirements.
				$products = cart::getProducts();

				foreach ($products as $product) {
					$product_total = 0;

					foreach ($products as $product_2) {
						if ($product_2['product_id'] == $product['product_id']) {
							$product_total += $product_2['quantity'];
						}
					}

					if ($product['minimum'] > $product_total) {
						$json['error'] = sprintf(language::get('error_minimum'), $product['name'], $product['minimum']);

						break;
					}
				}

				if (!$json) {
					$json['success'] = language::get('text_success');
					
					$order_data = array();

					// Store Details
					$order_data['invoice_prefix'] = config('invoice_prefix');
					$order_data['store_id'] = config('store_id');
					$order_data['store_name'] = config('name');
					$order_data['store_url'] = config('url');

					// Customer Details
					$order_data['customer_id'] = session::$data['customer']['customer_id'];
					$order_data['customer_group_id'] = session::$data['customer']['customer_group_id'];
					$order_data['firstname'] = session::$data['customer']['firstname'];
					$order_data['lastname'] = session::$data['customer']['lastname'];
					$order_data['email'] = session::$data['customer']['email'];
					$order_data['telephone'] = session::$data['customer']['telephone'];
					$order_data['custom_field'] = session::$data['customer']['custom_field'];

					// Payment Details
					$order_data['payment_firstname'] = session::$data['payment_address']['firstname'];
					$order_data['payment_lastname'] = session::$data['payment_address']['lastname'];
					$order_data['payment_company'] = session::$data['payment_address']['company'];
					$order_data['payment_address_1'] = session::$data['payment_address']['address_1'];
					$order_data['payment_address_2'] = session::$data['payment_address']['address_2'];
					$order_data['payment_city'] = session::$data['payment_address']['city'];
					$order_data['payment_postcode'] = session::$data['payment_address']['postcode'];
					$order_data['payment_zone'] = session::$data['payment_address']['zone'];
					$order_data['payment_zone_id'] = session::$data['payment_address']['zone_id'];
					$order_data['payment_country'] = session::$data['payment_address']['country'];
					$order_data['payment_country_id'] = session::$data['payment_address']['country_id'];
					$order_data['payment_address_format'] = session::$data['payment_address']['address_format'];
					$order_data['payment_custom_field'] = session::$data['payment_address']['custom_field'];

					if (isset(session::$data['payment_method']['title'])) {
						$order_data['payment_method'] = session::$data['payment_method']['title'];
					} else {
						$order_data['payment_method'] = '';
					}

					if (isset(session::$data['payment_method']['code'])) {
						$order_data['payment_code'] = session::$data['payment_method']['code'];
					} else {
						$order_data['payment_code'] = '';
					}

					// Shipping Details
					if (cart::hasShipping()) {
						$order_data['shipping_firstname'] = session::$data['shipping_address']['firstname'];
						$order_data['shipping_lastname'] = session::$data['shipping_address']['lastname'];
						$order_data['shipping_company'] = session::$data['shipping_address']['company'];
						$order_data['shipping_address_1'] = session::$data['shipping_address']['address_1'];
						$order_data['shipping_address_2'] = session::$data['shipping_address']['address_2'];
						$order_data['shipping_city'] = session::$data['shipping_address']['city'];
						$order_data['shipping_postcode'] = session::$data['shipping_address']['postcode'];
						$order_data['shipping_zone'] = session::$data['shipping_address']['zone'];
						$order_data['shipping_zone_id'] = session::$data['shipping_address']['zone_id'];
						$order_data['shipping_country'] = session::$data['shipping_address']['country'];
						$order_data['shipping_country_id'] = session::$data['shipping_address']['country_id'];
						$order_data['shipping_address_format'] = session::$data['shipping_address']['address_format'];
						$order_data['shipping_custom_field'] = session::$data['shipping_address']['custom_field'];

						if (isset(session::$data['shipping_method']['title'])) {
							$order_data['shipping_method'] = session::$data['shipping_method']['title'];
						} else {
							$order_data['shipping_method'] = '';
						}

						if (isset(session::$data['shipping_method']['code'])) {
							$order_data['shipping_code'] = session::$data['shipping_method']['code'];
						} else {
							$order_data['shipping_code'] = '';
						}
					} else {
						$order_data['shipping_firstname'] = '';
						$order_data['shipping_lastname'] = '';
						$order_data['shipping_company'] = '';
						$order_data['shipping_address_1'] = '';
						$order_data['shipping_address_2'] = '';
						$order_data['shipping_city'] = '';
						$order_data['shipping_postcode'] = '';
						$order_data['shipping_zone'] = '';
						$order_data['shipping_zone_id'] = '';
						$order_data['shipping_country'] = '';
						$order_data['shipping_country_id'] = '';
						$order_data['shipping_address_format'] = '';
						$order_data['shipping_custom_field'] = array();
						$order_data['shipping_method'] = '';
						$order_data['shipping_code'] = '';
					}

					// Products
					$order_data['products'] = array();

					foreach (cart::getProducts() as $product) {
						$option_data = array();

						foreach ($product['option'] as $option) {
							$option_data[] = array(
								'product_option_id'       => $option['product_option_id'],
								'product_option_value_id' => $option['product_option_value_id'],
								'option_id'               => $option['option_id'],
								'option_value_id'         => $option['option_value_id'],
								'name'                    => $option['name'],
								'value'                   => $option['value'],
								'type'                    => $option['type']
							);
						}

						$order_data['products'][] = array(
							'product_id' => $product['product_id'],
							'name'       => $product['name'],
							'model'      => $product['model'],
							'option'     => $option_data,
							'download'   => $product['download'],
							'quantity'   => $product['quantity'],
							'subtract'   => $product['subtract'],
							'price'      => $product['price'],
							'total'      => $product['total'],
							'tax'        => tax::getTax($product['price'], $product['tax_class_id']),
							'reward'     => $product['reward']
						);
					}

					// Gift Voucher
					$order_data['vouchers'] = array();

					if (!empty(session::$data['vouchers'])) {
						foreach (session::$data['vouchers'] as $voucher) {
							$order_data['vouchers'][] = array(
								'description'      => $voucher['description'],
								'code'             => token(10),
								'to_name'          => $voucher['to_name'],
								'to_email'         => $voucher['to_email'],
								'from_name'        => $voucher['from_name'],
								'from_email'       => $voucher['from_email'],
								'voucher_theme_id' => $voucher['voucher_theme_id'],
								'message'          => $voucher['message'],
								'amount'           => $voucher['amount']
							);
						}
					}

					// Order Totals

					$totals = array();
					$taxes = cart::getTaxes();
					$total = 0;
					
					// Because __call can not keep var references so we put them into an array. 
					$total_data = array(
						'totals' => &$totals,
						'taxes'  => &$taxes,
						'total'  => &$total
					);
			
					$sort_order = array();

					$results = setting_extension::getExtensions('total');

					foreach ($results as $key => $value) {
						$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
					}

					array_multisort($sort_order, SORT_ASC, $results);

					foreach ($results as $result) {
						if (config::get('total_' . $result['code'] . '_status')) {
							
							// We have to put the totals in an array so that they pass by reference.
							call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
						}
					}

					$sort_order = array();

					foreach ($total_data['totals'] as $key => $value) {
						$sort_order[$key] = $value['sort_order'];
					}

					array_multisort($sort_order, SORT_ASC, $total_data['totals']);

					$order_data = array_merge($order_data, $total_data);

					if (isset(request::$post['comment'])) {
						$order_data['comment'] = request::$post['comment'];
					} else {
						$order_data['comment'] = '';
					}

					if (isset(request::$post['affiliate_id'])) {
						$subtotal = cart::getSubTotal();

						// Affiliate

						$affiliate_info = account_customer::getAffiliate(request::$post['affiliate_id']);

						if ($affiliate_info) {
							$order_data['affiliate_id'] = $affiliate_info['customer_id'];
							$order_data['commission'] = ($subtotal / 100) * $affiliate_info['commission'];
						} else {
							$order_data['affiliate_id'] = 0;
							$order_data['commission'] = 0;
						}
					} else {
						$order_data['affiliate_id'] = 0;
						$order_data['commission'] = 0;
					}

					checkout_order::editOrder($order_id, $order_data);

					// Set the order history
					if (isset(request::$post['order_status_id'])) {
						$order_status_id = request::$post['order_status_id'];
					} else {
						$order_status_id = config('order_status_id');
					}
					
					checkout_order::addOrderHistory($order_id, $order_status_id);

					// When order editing is completed, delete added order status for Void the order first.
					if ($order_status_id) {
						db::query("DELETE FROM `order_history` WHERE order_id = " . (int)$order_id . " AND order_status_id = '0'");
					}
				}
			} else {
				$json['error'] = language::get('error_not_found');
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function delete() {
		load::language('api/order');

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} else {

			if (isset(request::$get['order_id'])) {
				$order_id = request::$get['order_id'];
			} else {
				$order_id = 0;
			}

			$order_info = checkout_order::getOrder($order_id);

			if ($order_info) {
				checkout_order::deleteOrder($order_id);

				$json['success'] = language::get('text_success');
			} else {
				$json['error'] = language::get('error_not_found');
			}
		}
		
		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function info() {
		load::language('api/order');

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} else {

			if (isset(request::$get['order_id'])) {
				$order_id = request::$get['order_id'];
			} else {
				$order_id = 0;
			}

			$order_info = checkout_order::getOrder($order_id);

			if ($order_info) {
				$json['order'] = $order_info;

				$json['success'] = language::get('text_success');
			} else {
				$json['error'] = language::get('error_not_found');
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function history() {
		load::language('api/order');

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} else {
			// Add keys for missing post vars
			$keys = array(
				'order_status_id',
				'notify',
				'override',
				'comment'
			);

			foreach ($keys as $key) {
				if (!isset(request::$post[$key])) {
					request::$post[$key] = '';
				}
			}


			if (isset(request::$get['order_id'])) {
				$order_id = request::$get['order_id'];
			} else {
				$order_id = 0;
			}

			$order_info = checkout_order::getOrder($order_id);

			if ($order_info) {
				checkout_order::addOrderHistory($order_id, request::$post['order_status_id'], request::$post['comment'], request::$post['notify'], request::$post['override']);

				$json['success'] = language::get('text_success');
			} else {
				$json['error'] = language::get('error_not_found');
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
