<?php
class ControllerApiPayment {
	public static function address() {
		load::language('api/payment');

		// Delete old payment address, payment methods and method so not to cause any issues if there is an error
		unset(session::$data['payment_address']);
		unset(session::$data['payment_methods']);
		unset(session::$data['payment_method']);

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error']['warning'] = language::get('error_permission');
		} else {
			// Add keys for missing post vars
			$keys = array(
				'firstname',
				'lastname',
				'company',
				'address_1',
				'address_2',
				'postcode',
				'city',
				'zone_id',
				'country_id'
			);

			foreach ($keys as $key) {
				if (!isset(request::$post[$key])) {
					request::$post[$key] = '';
				}
			}

			if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
				$json['error']['firstname'] = language::get('error_firstname');
			}

			if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
				$json['error']['lastname'] = language::get('error_lastname');
			}

			if ((utf8_strlen(trim(request::$post['address_1'])) < 3) || (utf8_strlen(trim(request::$post['address_1'])) > 128)) {
				$json['error']['address_1'] = language::get('error_address_1');
			}

			if ((utf8_strlen(request::$post['city']) < 2) || (utf8_strlen(request::$post['city']) > 32)) {
				$json['error']['city'] = language::get('error_city');
			}


			$country_info = localisation_country::getCountry(request::$post['country_id']);

			if ($country_info && $country_info['postcode_required'] && (utf8_strlen(trim(request::$post['postcode'])) < 2 || utf8_strlen(trim(request::$post['postcode'])) > 10)) {
				$json['error']['postcode'] = language::get('error_postcode');
			}

			if (request::$post['country_id'] == '') {
				$json['error']['country'] = language::get('error_country');
			}

			if (!isset(request::$post['zone_id']) || request::$post['zone_id'] == '') {
				$json['error']['zone'] = language::get('error_zone');
			}

			// Custom field validation

			$custom_fields = account_custom_field::getCustomFields(config('customer_group_id'));

			foreach ($custom_fields as $custom_field) {
				if ($custom_field['location'] == 'address') {
					if ($custom_field['required'] && empty(request::$post['custom_field'][$custom_field['custom_field_id']])) {
						$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
					} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var(request::$post['custom_field'][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
						$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
					}
				}
			}

			if (!$json) {

				$country_info = localisation_country::getCountry(request::$post['country_id']);

				if ($country_info) {
					$country = $country_info['name'];
					$iso_code_2 = $country_info['iso_code_2'];
					$iso_code_3 = $country_info['iso_code_3'];
					$address_format = $country_info['address_format'];
				} else {
					$country = '';
					$iso_code_2 = '';
					$iso_code_3 = '';
					$address_format = '';
				}


				$zone_info = localisation_zone::getZone(request::$post['zone_id']);

				if ($zone_info) {
					$zone = $zone_info['name'];
					$zone_code = $zone_info['code'];
				} else {
					$zone = '';
					$zone_code = '';
				}

				session::$data['payment_address'] = array(
					'firstname'      => request::$post['firstname'],
					'lastname'       => request::$post['lastname'],
					'company'        => request::$post['company'],
					'address_1'      => request::$post['address_1'],
					'address_2'      => request::$post['address_2'],
					'postcode'       => request::$post['postcode'],
					'city'           => request::$post['city'],
					'zone_id'        => request::$post['zone_id'],
					'zone'           => $zone,
					'zone_code'      => $zone_code,
					'country_id'     => request::$post['country_id'],
					'country'        => $country,
					'iso_code_2'     => $iso_code_2,
					'iso_code_3'     => $iso_code_3,
					'address_format' => $address_format,
					'custom_field'   => isset(request::$post['custom_field']) ? request::$post['custom_field'] : array()
				);

				$json['success'] = language::get('text_address');
				
				unset(session::$data['payment_method']);
				unset(session::$data['payment_methods']);
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function methods() {
		load::language('api/payment');
		
		// Delete past shipping methods and method just in case there is an error
		unset(session::$data['payment_methods']);
		unset(session::$data['payment_method']);

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} else {
			// Payment Address
			if (!isset(session::$data['payment_address'])) {
				$json['error'] = language::get('error_address');
			}
			
			if (!$json) {
				// Totals
				$totals = array();
				$taxes = cart::getTaxes();
				$total = 0;

				// Because __call can not keep var references so we put them into an array. 
				$total_data = array(
					'totals' => &$totals,
					'taxes'  => &$taxes,
					'total'  => &$total
				);


				$sort_order = array();

				$results = setting_extension::getExtensions('total');

				foreach ($results as $key => $value) {
					$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
				}

				array_multisort($sort_order, SORT_ASC, $results);

				foreach ($results as $result) {
					if (config::get('total_' . $result['code'] . '_status')) {
						
						// We have to put the totals in an array so that they pass by reference.
						call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
					}
				}

				// Payment Methods
				$json['payment_methods'] = array();


				$results = setting_extension::getExtensions('payment');

				$recurring = cart::hasRecurringProducts();

				foreach ($results as $result) {
					if (config::get('payment_' . $result['code'] . '_status')) {

						$method = call_model_method('extension_payment_' . $result['code'], 'getMethod', session::$data['payment_address'], $total);

						if ($method) {
							if ($recurring) {
								if (model_method_exists('extension_payment_' . $result['code'], 'recurringPayments') && call_model_method('extension_payment_' . $result['code'], 'recurringPayments', )) {
									$json['payment_methods'][$result['code']] = $method;
								}
							} else {
								$json['payment_methods'][$result['code']] = $method;
							}
						}
					}
				}

				$sort_order = array();

				foreach ($json['payment_methods'] as $key => $value) {
					$sort_order[$key] = $value['sort_order'];
				}

				array_multisort($sort_order, SORT_ASC, $json['payment_methods']);

				if ($json['payment_methods']) {
					session::$data['payment_methods'] = $json['payment_methods'];
				} else {
					$json['error'] = language::get('error_no_payment');
				}
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function method() {
		load::language('api/payment');

		// Delete old payment method so not to cause any issues if there is an error
		unset(session::$data['payment_method']);

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} else {
			// Payment Address
			if (!isset(session::$data['payment_address'])) {
				$json['error'] = language::get('error_address');
			}

			// Payment Method
			if (empty(session::$data['payment_methods'])) {
				$json['error'] = language::get('error_no_payment');
			} elseif (!isset(request::$post['payment_method'])) {
				$json['error'] = language::get('error_method');
			} elseif (!isset(session::$data['payment_methods'][request::$post['payment_method']])) {
				$json['error'] = language::get('error_method');
			}

			if (!$json) {
				session::$data['payment_method'] = session::$data['payment_methods'][request::$post['payment_method']];

				$json['success'] = language::get('text_method');
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
