<?php
class ControllerApiShipping {
	public static function address() {
		load::language('api/shipping');

		// Delete old shipping address, shipping methods and method so not to cause any issues if there is an error
		unset(session::$data['shipping_address']);
		unset(session::$data['shipping_methods']);
		unset(session::$data['shipping_method']);

		$json = array();

		if (cart::hasShipping()) {
			if (!isset(session::$data['api_id'])) {
				$json['error']['warning'] = language::get('error_permission');
			} else {
				// Add keys for missing post vars
				$keys = array(
					'firstname',
					'lastname',
					'company',
					'address_1',
					'address_2',
					'postcode',
					'city',
					'zone_id',
					'country_id'
				);

				foreach ($keys as $key) {
					if (!isset(request::$post[$key])) {
						request::$post[$key] = '';
					}
				}

				if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
					$json['error']['firstname'] = language::get('error_firstname');
				}

				if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
					$json['error']['lastname'] = language::get('error_lastname');
				}

				if ((utf8_strlen(trim(request::$post['address_1'])) < 3) || (utf8_strlen(trim(request::$post['address_1'])) > 128)) {
					$json['error']['address_1'] = language::get('error_address_1');
				}

				if ((utf8_strlen(request::$post['city']) < 2) || (utf8_strlen(request::$post['city']) > 128)) {
					$json['error']['city'] = language::get('error_city');
				}


				$country_info = localisation_country::getCountry(request::$post['country_id']);

				if ($country_info && $country_info['postcode_required'] && (utf8_strlen(trim(request::$post['postcode'])) < 2 || utf8_strlen(trim(request::$post['postcode'])) > 10)) {
					$json['error']['postcode'] = language::get('error_postcode');
				}

				if (request::$post['country_id'] == '') {
					$json['error']['country'] = language::get('error_country');
				}

				if (!isset(request::$post['zone_id']) || request::$post['zone_id'] == '') {
					$json['error']['zone'] = language::get('error_zone');
				}

				// Custom field validation

				$custom_fields = account_custom_field::getCustomFields(config('customer_group_id'));

				foreach ($custom_fields as $custom_field) {
					if ($custom_field['location'] == 'address') {
						if ($custom_field['required'] && empty(request::$post['custom_field'][$custom_field['custom_field_id']])) {
							$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
						} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var(request::$post['custom_field'][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
							$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
						}
					}
				}

				if (!$json) {

					$country_info = localisation_country::getCountry(request::$post['country_id']);

					if ($country_info) {
						$country = $country_info['name'];
						$iso_code_2 = $country_info['iso_code_2'];
						$iso_code_3 = $country_info['iso_code_3'];
						$address_format = $country_info['address_format'];
					} else {
						$country = '';
						$iso_code_2 = '';
						$iso_code_3 = '';
						$address_format = '';
					}


					$zone_info = localisation_zone::getZone(request::$post['zone_id']);

					if ($zone_info) {
						$zone = $zone_info['name'];
						$zone_code = $zone_info['code'];
					} else {
						$zone = '';
						$zone_code = '';
					}

					session::$data['shipping_address'] = array(
						'firstname'      => request::$post['firstname'],
						'lastname'       => request::$post['lastname'],
						'company'        => request::$post['company'],
						'address_1'      => request::$post['address_1'],
						'address_2'      => request::$post['address_2'],
						'postcode'       => request::$post['postcode'],
						'city'           => request::$post['city'],
						'zone_id'        => request::$post['zone_id'],
						'zone'           => $zone,
						'zone_code'      => $zone_code,
						'country_id'     => request::$post['country_id'],
						'country'        => $country,
						'iso_code_2'     => $iso_code_2,
						'iso_code_3'     => $iso_code_3,
						'address_format' => $address_format,
						'custom_field'   => isset(request::$post['custom_field']) ? request::$post['custom_field'] : array()
					);

					$json['success'] = language::get('text_address');

					unset(session::$data['shipping_method']);
					unset(session::$data['shipping_methods']);
				}
			}
		}
		
		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function methods() {
		load::language('api/shipping');

		// Delete past shipping methods and method just in case there is an error
		unset(session::$data['shipping_methods']);
		unset(session::$data['shipping_method']);

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} elseif (cart::hasShipping()) {
			if (!isset(session::$data['shipping_address'])) {
				$json['error'] = language::get('error_address');
			}

			if (!$json) {
				// Shipping Methods
				$json['shipping_methods'] = array();


				$results = setting_extension::getExtensions('shipping');

				foreach ($results as $result) {
					if (config::get('shipping_' . $result['code'] . '_status')) {

						$quote = call_model_method('extension_shipping_' . $result['code'], 'getQuote', session::$data['shipping_address']);

						if ($quote) {
							$json['shipping_methods'][$result['code']] = array(
								'title'      => $quote['title'],
								'quote'      => $quote['quote'],
								'sort_order' => $quote['sort_order'],
								'error'      => $quote['error']
							);
						}
					}
				}

				$sort_order = array();

				foreach ($json['shipping_methods'] as $key => $value) {
					$sort_order[$key] = $value['sort_order'];
				}

				array_multisort($sort_order, SORT_ASC, $json['shipping_methods']);

				if ($json['shipping_methods']) {
					session::$data['shipping_methods'] = $json['shipping_methods'];
				} else {
					$json['error'] = language::get('error_no_shipping');
				}
			}
		} else {
			$json['shipping_methods'] = array();
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function method() {
		load::language('api/shipping');

		// Delete old shipping method so not to cause any issues if there is an error
		unset(session::$data['shipping_method']);

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} else {
			if (cart::hasShipping()) {
				// Shipping Address
				if (!isset(session::$data['shipping_address'])) {
					$json['error'] = language::get('error_address');
				}

				// Shipping Method
				if (empty(session::$data['shipping_methods'])) {
					$json['error'] = language::get('error_no_shipping');
				} elseif (!isset(request::$post['shipping_method'])) {
					$json['error'] = language::get('error_method');
				} else {
					$shipping = explode('.', request::$post['shipping_method']);

					if (!isset($shipping[0]) || !isset($shipping[1]) || !isset(session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]])) {
						$json['error'] = language::get('error_method');
					}
				}

				if (!$json) {
					session::$data['shipping_method'] = session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]];

					$json['success'] = language::get('text_method');
				}
			} else {
				unset(session::$data['shipping_address']);
				unset(session::$data['shipping_method']);
				unset(session::$data['shipping_methods']);
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
