<?php
class ControllerApiVoucher {
	public static function index() {
		load::language('api/voucher');

		// Delete past voucher in case there is an error
		unset(session::$data['voucher']);

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error'] = language::get('error_permission');
		} else {

			if (isset(request::$post['voucher'])) {
				$voucher = request::$post['voucher'];
			} else {
				$voucher = '';
			}

			$voucher_info = extension_total_voucher::getVoucher($voucher);

			if ($voucher_info) {
				session::$data['voucher'] = request::$post['voucher'];

				$json['success'] = language::get('text_success');
			} else {
				$json['error'] = language::get('error_voucher');
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function add() {
		load::language('api/voucher');

		$json = array();

		if (!isset(session::$data['api_id'])) {
			$json['error']['warning'] = language::get('error_permission');
		} else {
			// Add keys for missing post vars
			$keys = array(
				'from_name',
				'from_email',
				'to_name',
				'to_email',
				'voucher_theme_id',
				'message',
				'amount'
			);

			foreach ($keys as $key) {
				if (!isset(request::$post[$key])) {
					request::$post[$key] = '';
				}
			}

			if (isset(request::$post['voucher'])) {
				session::$data['vouchers'] = array();

				foreach (request::$post['voucher'] as $voucher) {
					if (isset($voucher['code']) && isset($voucher['to_name']) && isset($voucher['to_email']) && isset($voucher['from_name']) && isset($voucher['from_email']) && isset($voucher['voucher_theme_id']) && isset($voucher['message']) && isset($voucher['amount'])) {
						session::$data['vouchers'][$voucher['code']] = array(
							'code'             => $voucher['code'],
							'description'      => sprintf(language::get('text_for'), currency::format(currency::convert($voucher['amount'], session::$data['currency'], config('currency')), session::$data['currency']), $voucher['to_name']),
							'to_name'          => $voucher['to_name'],
							'to_email'         => $voucher['to_email'],
							'from_name'        => $voucher['from_name'],
							'from_email'       => $voucher['from_email'],
							'voucher_theme_id' => $voucher['voucher_theme_id'],
							'message'          => $voucher['message'],
							'amount'           => currency::convert($voucher['amount'], session::$data['currency'], config('currency'))
						);
					}
				}

				$json['success'] = language::get('text_cart');

				unset(session::$data['shipping_method']);
				unset(session::$data['shipping_methods']);
				unset(session::$data['payment_method']);
				unset(session::$data['payment_methods']);
			} else {
				// Add a new voucher if set
				if ((utf8_strlen(request::$post['from_name']) < 1) || (utf8_strlen(request::$post['from_name']) > 64)) {
					$json['error']['from_name'] = language::get('error_from_name');
				}

				if ((utf8_strlen(request::$post['from_email']) > 96) || !filter_var(request::$post['from_email'], FILTER_VALIDATE_EMAIL)) {
					$json['error']['from_email'] = language::get('error_email');
				}

				if ((utf8_strlen(request::$post['to_name']) < 1) || (utf8_strlen(request::$post['to_name']) > 64)) {
					$json['error']['to_name'] = language::get('error_to_name');
				}

				if ((utf8_strlen(request::$post['to_email']) > 96) || !filter_var(request::$post['to_email'], FILTER_VALIDATE_EMAIL)) {
					$json['error']['to_email'] = language::get('error_email');
				}

				if ((request::$post['amount'] < config('voucher_min')) || (request::$post['amount'] > config('voucher_max'))) {
					$json['error']['amount'] = sprintf(language::get('error_amount'), currency::format(config('voucher_min'), session::$data['currency']), currency::format(config('voucher_max'), session::$data['currency']));
				}

				if (!$json) {
					$code = mt_rand();

					session::$data['vouchers'][$code] = array(
						'code'             => $code,
						'description'      => sprintf(language::get('text_for'), currency::format(currency::convert(request::$post['amount'], session::$data['currency'], config('currency')), session::$data['currency']), request::$post['to_name']),
						'to_name'          => request::$post['to_name'],
						'to_email'         => request::$post['to_email'],
						'from_name'        => request::$post['from_name'],
						'from_email'       => request::$post['from_email'],
						'voucher_theme_id' => request::$post['voucher_theme_id'],
						'message'          => request::$post['message'],
						'amount'           => currency::convert(request::$post['amount'], session::$data['currency'], config('currency'))
					);

					$json['success'] = language::get('text_cart');

					unset(session::$data['shipping_method']);
					unset(session::$data['shipping_methods']);
					unset(session::$data['payment_method']);
					unset(session::$data['payment_methods']);
				}
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
