<?php
class ControllerCheckoutCart {
	public static function index() {
		load::language('checkout/cart');

		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'href' => url::link('common/home'),
			'text' => language::get('text_home')
		);

		$data['breadcrumbs'][] = array(
			'href' => url::link('checkout/cart'),
			'text' => language::get('heading_title')
		);

		if (cart::hasProducts() || !empty(session::$data['vouchers'])) {
			if (!cart::hasStock() && (!config('stock_checkout') || config('stock_warning'))) {
				$data['error_warning'] = language::get('error_stock');
			} elseif (isset(session::$data['error'])) {
				$data['error_warning'] = session::$data['error'];

				unset(session::$data['error']);
			} else {
				$data['error_warning'] = '';
			}

			if (config('customer_price') && !customer::isLogged()) {
				$data['attention'] = sprintf(language::get('text_login'), url::link('account/login'), url::link('account/register'));
			} else {
				$data['attention'] = '';
			}

			if (isset(session::$data['success'])) {
				$data['success'] = session::$data['success'];

				unset(session::$data['success']);
			} else {
				$data['success'] = '';
			}

			$data['action'] = url::link('checkout/cart/edit', '');

			if (config('cart_weight')) {
				$data['weight'] = weight::format(cart::getWeight(), config('weight_class_id'), language::get('decimal_point'), language::get('thousand_point'));
			} else {
				$data['weight'] = '';
			}


			$data['products'] = array();

			$products = cart::getProducts();

			foreach ($products as $product) {
				$product_total = 0;

				foreach ($products as $product_2) {
					if ($product_2['product_id'] == $product['product_id']) {
						$product_total += $product_2['quantity'];
					}
				}

				if ($product['minimum'] > $product_total) {
					$data['error_warning'] = sprintf(language::get('error_minimum'), $product['name'], $product['minimum']);
				}

				if ($product['image']) {
					$image = tool_image::resize($product['image'], config::get('theme_' . config('theme') . '_image_cart_width'), config::get('theme_' . config('theme') . '_image_cart_height'));
				} else {
					$image = '';
				}

				$option_data = array();

				foreach ($product['option'] as $option) {
					if ($option['type'] != 'file') {
						$value = $option['value'];
					} else {
						$upload_info = tool_upload::getUploadByCode($option['value']);

						if ($upload_info) {
							$value = $upload_info['name'];
						} else {
							$value = '';
						}
					}

					$option_data[] = array(
						'name'  => $option['name'],
						'value' => (utf8_strlen($value) > 20 ? utf8_substr($value, 0, 20) . '..' : $value)
					);
				}

				// Display prices
				if (customer::isLogged() || !config('customer_price')) {
					$unit_price = tax::calculate($product['price'], $product['tax_class_id'], config('tax'));
					
					$price = currency::format($unit_price, session::$data['currency']);
					$total = currency::format($unit_price * $product['quantity'], session::$data['currency']);
				} else {
					$price = false;
					$total = false;
				}

				$recurring = '';

				if ($product['recurring']) {
					$frequencies = array(
						'day'        => language::get('text_day'),
						'week'       => language::get('text_week'),
						'semi_month' => language::get('text_semi_month'),
						'month'      => language::get('text_month'),
						'year'       => language::get('text_year')
					);

					if ($product['recurring']['trial']) {
						$recurring = sprintf(language::get('text_trial_description'), currency::format(tax::calculate($product['recurring']['trial_price'] * $product['quantity'], $product['tax_class_id'], config('tax')), session::$data['currency']), $product['recurring']['trial_cycle'], $frequencies[$product['recurring']['trial_frequency']], $product['recurring']['trial_duration']) . ' ';
					}

					if ($product['recurring']['duration']) {
						$recurring .= sprintf(language::get('text_payment_description'), currency::format(tax::calculate($product['recurring']['price'] * $product['quantity'], $product['tax_class_id'], config('tax')), session::$data['currency']), $product['recurring']['cycle'], $frequencies[$product['recurring']['frequency']], $product['recurring']['duration']);
					} else {
						$recurring .= sprintf(language::get('text_payment_cancel'), currency::format(tax::calculate($product['recurring']['price'] * $product['quantity'], $product['tax_class_id'], config('tax')), session::$data['currency']), $product['recurring']['cycle'], $frequencies[$product['recurring']['frequency']], $product['recurring']['duration']);
					}
				}

				$data['products'][] = array(
					'cart_id'   => $product['cart_id'],
					'thumb'     => $image,
					'name'      => $product['name'],
					'model'     => $product['model'],
					'option'    => $option_data,
					'recurring' => $recurring,
					'quantity'  => $product['quantity'],
					'stock'     => $product['stock'] ? true : !(!config('stock_checkout') || config('stock_warning')),
					'reward'    => ($product['reward'] ? sprintf(language::get('text_points'), $product['reward']) : ''),
					'price'     => $price,
					'total'     => $total,
					'href'      => url::link('product/product', 'product_id=' . $product['product_id'])
				);
			}

			// Gift Voucher
			$data['vouchers'] = array();

			if (!empty(session::$data['vouchers'])) {
				foreach (session::$data['vouchers'] as $key => $voucher) {
					$data['vouchers'][] = array(
						'key'         => $key,
						'description' => $voucher['description'],
						'amount'      => currency::format($voucher['amount'], session::$data['currency']),
						'remove'      => url::link('checkout/cart', 'remove=' . $key)
					);
				}
			}

			// Totals

			$totals = array();
			$taxes = cart::getTaxes();
			$total = 0;
			
			// Because __call can not keep var references so we put them into an array. 			
			$total_data = array(
				'totals' => &$totals,
				'taxes'  => &$taxes,
				'total'  => &$total
			);
			
			// Display prices
			if (customer::isLogged() || !config('customer_price')) {
				$sort_order = array();

				$results = setting_extension::getExtensions('total');

				foreach ($results as $key => $value) {
					$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
				}

				array_multisort($sort_order, SORT_ASC, $results);

				foreach ($results as $result) {
					if (config::get('total_' . $result['code'] . '_status')) {
						
						// We have to put the totals in an array so that they pass by reference.
						call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
					}
				}

				$sort_order = array();

				foreach ($totals as $key => $value) {
					$sort_order[$key] = $value['sort_order'];
				}

				array_multisort($sort_order, SORT_ASC, $totals);
			}

			$data['totals'] = array();

			foreach ($totals as $total) {
				$data['totals'][] = array(
					'title' => $total['title'],
					'text'  => currency::format($total['value'], session::$data['currency'])
				);
			}

			$data['continue'] = url::link('common/home');

			$data['checkout'] = url::link('checkout/checkout', '');


			$data['modules'] = array();
			
			$files = glob('/controller/extension/total/*.php');

			if ($files) {
				foreach ($files as $file) {
					$result = load::controller('extension/total/' . basename($file, '.php'));
					
					if ($result) {
						$data['modules'][] = $result;
					}
				}
			}

			$data['column_left'] = load::controller('common/column_left');
			$data['column_right'] = load::controller('common/column_right');
			$data['content_top'] = load::controller('common/content_top');
			$data['content_bottom'] = load::controller('common/content_bottom');
			$data['footer'] = load::controller('common/footer');
			$data['header'] = load::controller('common/header');

			response::setOutput(load::view('checkout/cart', $data));
		} else {
			$data['text_error'] = language::get('text_empty');
			
			$data['continue'] = url::link('common/home');

			unset(session::$data['success']);

			$data['column_left'] = load::controller('common/column_left');
			$data['column_right'] = load::controller('common/column_right');
			$data['content_top'] = load::controller('common/content_top');
			$data['content_bottom'] = load::controller('common/content_bottom');
			$data['footer'] = load::controller('common/footer');
			$data['header'] = load::controller('common/header');

			response::setOutput(load::view('error/not_found', $data));
		}
	}

	public static function add() {
		load::language('checkout/cart');

		$json = array();

		if (isset(request::$post['product_id'])) {
			$product_id = (int)request::$post['product_id'];
		} else {
			$product_id = 0;
		}


		$product_info = catalog_product::getProduct($product_id);

		if ($product_info) {
			if (isset(request::$post['quantity'])) {
				$quantity = (int)request::$post['quantity'];
			} else {
				$quantity = 1;
			}

			if (isset(request::$post['option'])) {
				$option = array_filter(request::$post['option']);
			} else {
				$option = array();
			}

			$product_options = catalog_product::getProductOptions(request::$post['product_id']);

			foreach ($product_options as $product_option) {
				if ($product_option['required'] && empty($option[$product_option['product_option_id']])) {
					$json['error']['option'][$product_option['product_option_id']] = sprintf(language::get('error_required'), $product_option['name']);
				}
			}

			if (isset(request::$post['recurring_id'])) {
				$recurring_id = request::$post['recurring_id'];
			} else {
				$recurring_id = 0;
			}

			$recurrings = catalog_product::getProfiles($product_info['product_id']);

			if ($recurrings) {
				$recurring_ids = array();

				foreach ($recurrings as $recurring) {
					$recurring_ids[] = $recurring['recurring_id'];
				}

				if (!in_array($recurring_id, $recurring_ids)) {
					$json['error']['recurring'] = language::get('error_recurring_required');
				}
			}

			if (!$json) {
				cart::add(request::$post['product_id'], $quantity, $option, $recurring_id);

				$json['success'] = sprintf(language::get('text_success'), url::link('product/product', 'product_id=' . request::$post['product_id']), $product_info['name'], url::link('checkout/cart'));

				// Unset all shipping and payment methods
				unset(session::$data['shipping_method']);
				unset(session::$data['shipping_methods']);
				unset(session::$data['payment_method']);
				unset(session::$data['payment_methods']);

				// Totals

				$totals = array();
				$taxes = cart::getTaxes();
				$total = 0;
		
				// Because __call can not keep var references so we put them into an array. 			
				$total_data = array(
					'totals' => &$totals,
					'taxes'  => &$taxes,
					'total'  => &$total
				);

				// Display prices
				if (customer::isLogged() || !config('customer_price')) {
					$sort_order = array();

					$results = setting_extension::getExtensions('total');

					foreach ($results as $key => $value) {
						$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
					}

					array_multisort($sort_order, SORT_ASC, $results);

					foreach ($results as $result) {
						if (config::get('total_' . $result['code'] . '_status')) {

							// We have to put the totals in an array so that they pass by reference.
							call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
						}
					}

					$sort_order = array();

					foreach ($totals as $key => $value) {
						$sort_order[$key] = $value['sort_order'];
					}

					array_multisort($sort_order, SORT_ASC, $totals);
				}

				$json['total'] = sprintf(language::get('text_items'), cart::countProducts() + (isset(session::$data['vouchers']) ? count(session::$data['vouchers']) : 0), currency::format($total, session::$data['currency']));
			} else {
				$json['redirect'] = str_replace('&amp;', '&', url::link('product/product', 'product_id=' . request::$post['product_id']));
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function edit() {
		load::language('checkout/cart');

		$json = array();

		// Update
		if (!empty(request::$post['quantity'])) {
			foreach (request::$post['quantity'] as $key => $value) {
				cart::update($key, $value);
			}

			session::$data['success'] = language::get('text_remove');

			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
			unset(session::$data['payment_method']);
			unset(session::$data['payment_methods']);
			unset(session::$data['reward']);

			response::redirect(url::link('checkout/cart'));
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function remove() {
		load::language('checkout/cart');

		$json = array();

		// Remove
		if (isset(request::$post['key'])) {
			cart::remove(request::$post['key']);

			unset(session::$data['vouchers'][request::$post['key']]);

			$json['success'] = language::get('text_remove');

			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
			unset(session::$data['payment_method']);
			unset(session::$data['payment_methods']);
			unset(session::$data['reward']);

			// Totals

			$totals = array();
			$taxes = cart::getTaxes();
			$total = 0;

			// Because __call can not keep var references so we put them into an array. 			
			$total_data = array(
				'totals' => &$totals,
				'taxes'  => &$taxes,
				'total'  => &$total
			);

			// Display prices
			if (customer::isLogged() || !config('customer_price')) {
				$sort_order = array();

				$results = setting_extension::getExtensions('total');

				foreach ($results as $key => $value) {
					$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
				}

				array_multisort($sort_order, SORT_ASC, $results);

				foreach ($results as $result) {
					if (config::get('total_' . $result['code'] . '_status')) {

						// We have to put the totals in an array so that they pass by reference.
						call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
					}
				}

				$sort_order = array();

				foreach ($totals as $key => $value) {
					$sort_order[$key] = $value['sort_order'];
				}

				array_multisort($sort_order, SORT_ASC, $totals);
			}

			$json['total'] = sprintf(language::get('text_items'), cart::countProducts() + (isset(session::$data['vouchers']) ? count(session::$data['vouchers']) : 0), currency::format($total, session::$data['currency']));
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
