<?php
class ControllerCheckoutCheckout {
	public static function index() {
		// Validate cart has products and has stock.
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			response::redirect(url::link('checkout/cart'));
		}

		// Validate minimum quantity requirements.
		$products = cart::getProducts();

		foreach ($products as $product) {
			$product_total = 0;

			foreach ($products as $product_2) {
				if ($product_2['product_id'] == $product['product_id']) {
					$product_total += $product_2['quantity'];
				}
			}

			if ($product['minimum'] > $product_total) {
				response::redirect(url::link('checkout/cart'));
			}
		}

		load::language('checkout/checkout');

		document::setTitle(language::get('heading_title'));

		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/moment/moment-with-locales.min.js');
		document::addScript('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.js');
		document::addStyle('catalog/view/javascript/jquery/datetimepicker/bootstrap-datetimepicker.min.css');

		// Required by klarna
		if (config::get('payment_klarna_account') || config::get('payment_klarna_invoice')) {
			document::addScript('http://cdn.klarna.com/public/kitt/toc/v1.0/js/klarna.terms.min.js');
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_cart'),
			'href' => url::link('checkout/cart')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('checkout/checkout', '')
		);

		$data['text_checkout_option'] = sprintf(language::get('text_checkout_option'), 1);
		$data['text_checkout_account'] = sprintf(language::get('text_checkout_account'), 2);
		$data['text_checkout_payment_address'] = sprintf(language::get('text_checkout_payment_address'), 2);
		$data['text_checkout_shipping_address'] = sprintf(language::get('text_checkout_shipping_address'), 3);
		$data['text_checkout_shipping_method'] = sprintf(language::get('text_checkout_shipping_method'), 4);
		
		if (cart::hasShipping()) {
			$data['text_checkout_payment_method'] = sprintf(language::get('text_checkout_payment_method'), 5);
			$data['text_checkout_confirm'] = sprintf(language::get('text_checkout_confirm'), 6);
		} else {
			$data['text_checkout_payment_method'] = sprintf(language::get('text_checkout_payment_method'), 3);
			$data['text_checkout_confirm'] = sprintf(language::get('text_checkout_confirm'), 4);	
		}

		if (isset(session::$data['error'])) {
			$data['error_warning'] = session::$data['error'];
			unset(session::$data['error']);
		} else {
			$data['error_warning'] = '';
		}

		$data['logged'] = customer::isLogged();

		if (isset(session::$data['account'])) {
			$data['account'] = session::$data['account'];
		} else {
			$data['account'] = '';
		}

		$data['shipping_required'] = cart::hasShipping();

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('checkout/checkout', $data));
	}

	public static function country() {
		$json = array();


		$country_info = localisation_country::getCountry(request::$get['country_id']);

		if ($country_info) {

			$json = array(
				'country_id'        => $country_info['country_id'],
				'name'              => $country_info['name'],
				'iso_code_2'        => $country_info['iso_code_2'],
				'iso_code_3'        => $country_info['iso_code_3'],
				'address_format'    => $country_info['address_format'],
				'postcode_required' => $country_info['postcode_required'],
				'zone'              => localisation_zone::getZonesByCountryId(request::$get['country_id']),
				'status'            => $country_info['status']
			);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function customfield() {
		$json = array();


		// Customer Group
		if (isset(request::$get['customer_group_id']) && is_array(config('customer_group_display')) && in_array(request::$get['customer_group_id'], config('customer_group_display'))) {
			$customer_group_id = request::$get['customer_group_id'];
		} else {
			$customer_group_id = config('customer_group_id');
		}

		$custom_fields = account_custom_field::getCustomFields($customer_group_id);

		foreach ($custom_fields as $custom_field) {
			$json[] = array(
				'custom_field_id' => $custom_field['custom_field_id'],
				'required'        => $custom_field['required']
			);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}