<?php
class ControllerCheckoutConfirm {
	public static function index() {
		$redirect = '';

		if (cart::hasShipping()) {
			// Validate if shipping address has been set.
			if (!isset(session::$data['shipping_address'])) {
				$redirect = url::link('checkout/checkout', '');
			}

			// Validate if shipping method has been set.
			if (!isset(session::$data['shipping_method'])) {
				$redirect = url::link('checkout/checkout', '');
			}
		} else {
			unset(session::$data['shipping_address']);
			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
		}

		// Validate if payment address has been set.
		if (!isset(session::$data['payment_address'])) {
			$redirect = url::link('checkout/checkout', '');
		}

		// Validate if payment method has been set.
		if (!isset(session::$data['payment_method'])) {
			$redirect = url::link('checkout/checkout', '');
		}

		// Validate cart has products and has stock.
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			$redirect = url::link('checkout/cart');
		}

		// Validate minimum quantity requirements.
		$products = cart::getProducts();

		foreach ($products as $product) {
			$product_total = 0;

			foreach ($products as $product_2) {
				if ($product_2['product_id'] == $product['product_id']) {
					$product_total += $product_2['quantity'];
				}
			}

			if ($product['minimum'] > $product_total) {
				$redirect = url::link('checkout/cart');

				break;
			}
		}

		if (!$redirect) {
			$order_data = array();

			$totals = array();
			$taxes = cart::getTaxes();
			$total = 0;

			// Because __call can not keep var references so we put them into an array.
			$total_data = array(
				'totals' => &$totals,
				'taxes'  => &$taxes,
				'total'  => &$total
			);


			$sort_order = array();

			$results = setting_extension::getExtensions('total');

			foreach ($results as $key => $value) {
				$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
			}

			array_multisort($sort_order, SORT_ASC, $results);

			foreach ($results as $result) {
				if (config::get('total_' . $result['code'] . '_status')) {

					// We have to put the totals in an array so that they pass by reference.
					call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
				}
			}

			$sort_order = array();

			foreach ($totals as $key => $value) {
				$sort_order[$key] = $value['sort_order'];
			}

			array_multisort($sort_order, SORT_ASC, $totals);

			$order_data['totals'] = $totals;

			load::language('checkout/checkout');

			$order_data['invoice_prefix'] = config('invoice_prefix');
			$order_data['store_id'] = config('store_id');
			$order_data['store_name'] = config('name');

			if ($order_data['store_id']) {
				$order_data['store_url'] = config('url');
			} else {
				if (request::$server['HTTPS']) {
					$order_data['store_url'] = HTTPS_SERVER;
				} else {
					$order_data['store_url'] = HTTP_SERVER;
				}
			}
			

			if (customer::isLogged()) {
				$customer_info = account_customer::getCustomer(customer::getId());

				$order_data['customer_id'] = customer::getId();
				$order_data['customer_group_id'] = $customer_info['customer_group_id'];
				$order_data['firstname'] = $customer_info['firstname'];
				$order_data['lastname'] = $customer_info['lastname'];
				$order_data['email'] = $customer_info['email'];
				$order_data['telephone'] = $customer_info['telephone'];
				$order_data['custom_field'] = json_decode($customer_info['custom_field'], true);
			} elseif (isset(session::$data['guest'])) {
				$order_data['customer_id'] = 0;
				$order_data['customer_group_id'] = session::$data['guest']['customer_group_id'];
				$order_data['firstname'] = session::$data['guest']['firstname'];
				$order_data['lastname'] = session::$data['guest']['lastname'];
				$order_data['email'] = session::$data['guest']['email'];
				$order_data['telephone'] = session::$data['guest']['telephone'];
				$order_data['custom_field'] = session::$data['guest']['custom_field'];
			}

			$order_data['payment_firstname'] = session::$data['payment_address']['firstname'];
			$order_data['payment_lastname'] = session::$data['payment_address']['lastname'];
			$order_data['payment_company'] = session::$data['payment_address']['company'];
			$order_data['payment_address_1'] = session::$data['payment_address']['address_1'];
			$order_data['payment_address_2'] = session::$data['payment_address']['address_2'];
			$order_data['payment_city'] = session::$data['payment_address']['city'];
			$order_data['payment_postcode'] = session::$data['payment_address']['postcode'];
			$order_data['payment_zone'] = session::$data['payment_address']['zone'];
			$order_data['payment_zone_id'] = session::$data['payment_address']['zone_id'];
			$order_data['payment_country'] = session::$data['payment_address']['country'];
			$order_data['payment_country_id'] = session::$data['payment_address']['country_id'];
			$order_data['payment_address_format'] = session::$data['payment_address']['address_format'];
			$order_data['payment_custom_field'] = (isset(session::$data['payment_address']['custom_field']) ? session::$data['payment_address']['custom_field'] : array());

			if (isset(session::$data['payment_method']['title'])) {
				$order_data['payment_method'] = session::$data['payment_method']['title'];
			} else {
				$order_data['payment_method'] = '';
			}

			if (isset(session::$data['payment_method']['code'])) {
				$order_data['payment_code'] = session::$data['payment_method']['code'];
			} else {
				$order_data['payment_code'] = '';
			}

			if (cart::hasShipping()) {
				$order_data['shipping_firstname'] = session::$data['shipping_address']['firstname'];
				$order_data['shipping_lastname'] = session::$data['shipping_address']['lastname'];
				$order_data['shipping_company'] = session::$data['shipping_address']['company'];
				$order_data['shipping_address_1'] = session::$data['shipping_address']['address_1'];
				$order_data['shipping_address_2'] = session::$data['shipping_address']['address_2'];
				$order_data['shipping_city'] = session::$data['shipping_address']['city'];
				$order_data['shipping_postcode'] = session::$data['shipping_address']['postcode'];
				$order_data['shipping_zone'] = session::$data['shipping_address']['zone'];
				$order_data['shipping_zone_id'] = session::$data['shipping_address']['zone_id'];
				$order_data['shipping_country'] = session::$data['shipping_address']['country'];
				$order_data['shipping_country_id'] = session::$data['shipping_address']['country_id'];
				$order_data['shipping_address_format'] = session::$data['shipping_address']['address_format'];
				$order_data['shipping_custom_field'] = (isset(session::$data['shipping_address']['custom_field']) ? session::$data['shipping_address']['custom_field'] : array());

				if (isset(session::$data['shipping_method']['title'])) {
					$order_data['shipping_method'] = session::$data['shipping_method']['title'];
				} else {
					$order_data['shipping_method'] = '';
				}

				if (isset(session::$data['shipping_method']['code'])) {
					$order_data['shipping_code'] = session::$data['shipping_method']['code'];
				} else {
					$order_data['shipping_code'] = '';
				}
			} else {
				$order_data['shipping_firstname'] = '';
				$order_data['shipping_lastname'] = '';
				$order_data['shipping_company'] = '';
				$order_data['shipping_address_1'] = '';
				$order_data['shipping_address_2'] = '';
				$order_data['shipping_city'] = '';
				$order_data['shipping_postcode'] = '';
				$order_data['shipping_zone'] = '';
				$order_data['shipping_zone_id'] = '';
				$order_data['shipping_country'] = '';
				$order_data['shipping_country_id'] = '';
				$order_data['shipping_address_format'] = '';
				$order_data['shipping_custom_field'] = array();
				$order_data['shipping_method'] = '';
				$order_data['shipping_code'] = '';
			}

			$order_data['products'] = array();

			foreach (cart::getProducts() as $product) {
				$option_data = array();

				foreach ($product['option'] as $option) {
					$option_data[] = array(
						'product_option_id'       => $option['product_option_id'],
						'product_option_value_id' => $option['product_option_value_id'],
						'option_id'               => $option['option_id'],
						'option_value_id'         => $option['option_value_id'],
						'name'                    => $option['name'],
						'value'                   => $option['value'],
						'type'                    => $option['type']
					);
				}

				$order_data['products'][] = array(
					'product_id' => $product['product_id'],
					'name'       => $product['name'],
					'model'      => $product['model'],
					'option'     => $option_data,
					'download'   => $product['download'],
					'quantity'   => $product['quantity'],
					'subtract'   => $product['subtract'],
					'price'      => $product['price'],
					'total'      => $product['total'],
					'tax'        => tax::getTax($product['price'], $product['tax_class_id']),
					'reward'     => $product['reward']
				);
			}

			// Gift Voucher
			$order_data['vouchers'] = array();

			if (!empty(session::$data['vouchers'])) {
				foreach (session::$data['vouchers'] as $voucher) {
					$order_data['vouchers'][] = array(
						'description'      => $voucher['description'],
						'code'             => token(10),
						'to_name'          => $voucher['to_name'],
						'to_email'         => $voucher['to_email'],
						'from_name'        => $voucher['from_name'],
						'from_email'       => $voucher['from_email'],
						'voucher_theme_id' => $voucher['voucher_theme_id'],
						'message'          => $voucher['message'],
						'amount'           => $voucher['amount']
					);
				}
			}

			$order_data['comment'] = session::$data['comment'];
			$order_data['total'] = $total_data['total'];

			if (isset(request::$cookie['tracking'])) {
				$order_data['tracking'] = request::$cookie['tracking'];

				$subtotal = cart::getSubTotal();

				// Affiliate
				$affiliate_info = account_customer::getAffiliateByTracking(request::$cookie['tracking']);

				if ($affiliate_info) {
					$order_data['affiliate_id'] = $affiliate_info['customer_id'];
					$order_data['commission'] = ($subtotal / 100) * $affiliate_info['commission'];
				} else {
					$order_data['affiliate_id'] = 0;
					$order_data['commission'] = 0;
				}

				// Marketing

				$marketing_info = checkout_marketing::getMarketingByCode(request::$cookie['tracking']);

				if ($marketing_info) {
					$order_data['marketing_id'] = $marketing_info['marketing_id'];
				} else {
					$order_data['marketing_id'] = 0;
				}
			} else {
				$order_data['affiliate_id'] = 0;
				$order_data['commission'] = 0;
				$order_data['marketing_id'] = 0;
				$order_data['tracking'] = '';
			}

			$order_data['language_id'] = config('language_id');
			$order_data['currency_id'] = currency::getId(session::$data['currency']);
			$order_data['currency_code'] = session::$data['currency'];
			$order_data['currency_value'] = currency::getValue(session::$data['currency']);
			$order_data['ip'] = request::$server['REMOTE_ADDR'];

			if (!empty(request::$server['HTTP_X_FORWARDED_FOR'])) {
				$order_data['forwarded_ip'] = request::$server['HTTP_X_FORWARDED_FOR'];
			} elseif (!empty(request::$server['HTTP_CLIENT_IP'])) {
				$order_data['forwarded_ip'] = request::$server['HTTP_CLIENT_IP'];
			} else {
				$order_data['forwarded_ip'] = '';
			}

			if (isset(request::$server['HTTP_USER_AGENT'])) {
				$order_data['user_agent'] = request::$server['HTTP_USER_AGENT'];
			} else {
				$order_data['user_agent'] = '';
			}

			if (isset(request::$server['HTTP_ACCEPT_LANGUAGE'])) {
				$order_data['accept_language'] = request::$server['HTTP_ACCEPT_LANGUAGE'];
			} else {
				$order_data['accept_language'] = '';
			}


			session::$data['order_id'] = checkout_order::addOrder($order_data);


			$data['products'] = array();

			foreach (cart::getProducts() as $product) {
				$option_data = array();

				foreach ($product['option'] as $option) {
					if ($option['type'] != 'file') {
						$value = $option['value'];
					} else {
						$upload_info = tool_upload::getUploadByCode($option['value']);

						if ($upload_info) {
							$value = $upload_info['name'];
						} else {
							$value = '';
						}
					}

					$option_data[] = array(
						'name'  => $option['name'],
						'value' => (utf8_strlen($value) > 20 ? utf8_substr($value, 0, 20) . '..' : $value)
					);
				}

				$recurring = '';

				if ($product['recurring']) {
					$frequencies = array(
						'day'        => language::get('text_day'),
						'week'       => language::get('text_week'),
						'semi_month' => language::get('text_semi_month'),
						'month'      => language::get('text_month'),
						'year'       => language::get('text_year'),
					);

					if ($product['recurring']['trial']) {
						$recurring = sprintf(language::get('text_trial_description'), currency::format(tax::calculate($product['recurring']['trial_price'] * $product['quantity'], $product['tax_class_id'], config('tax')), session::$data['currency']), $product['recurring']['trial_cycle'], $frequencies[$product['recurring']['trial_frequency']], $product['recurring']['trial_duration']) . ' ';
					}

					if ($product['recurring']['duration']) {
						$recurring .= sprintf(language::get('text_payment_description'), currency::format(tax::calculate($product['recurring']['price'] * $product['quantity'], $product['tax_class_id'], config('tax')), session::$data['currency']), $product['recurring']['cycle'], $frequencies[$product['recurring']['frequency']], $product['recurring']['duration']);
					} else {
						$recurring .= sprintf(language::get('text_payment_cancel'), currency::format(tax::calculate($product['recurring']['price'] * $product['quantity'], $product['tax_class_id'], config('tax')), session::$data['currency']), $product['recurring']['cycle'], $frequencies[$product['recurring']['frequency']], $product['recurring']['duration']);
					}
				}

				$data['products'][] = array(
					'cart_id'    => $product['cart_id'],
					'product_id' => $product['product_id'],
					'name'       => $product['name'],
					'model'      => $product['model'],
					'option'     => $option_data,
					'recurring'  => $recurring,
					'quantity'   => $product['quantity'],
					'subtract'   => $product['subtract'],
					'price'      => currency::format(tax::calculate($product['price'], $product['tax_class_id'], config('tax')), session::$data['currency']),
					'total'      => currency::format(tax::calculate($product['price'], $product['tax_class_id'], config('tax')) * $product['quantity'], session::$data['currency']),
					'href'       => url::link('product/product', 'product_id=' . $product['product_id'])
				);
			}

			// Gift Voucher
			$data['vouchers'] = array();

			if (!empty(session::$data['vouchers'])) {
				foreach (session::$data['vouchers'] as $voucher) {
					$data['vouchers'][] = array(
						'description' => $voucher['description'],
						'amount'      => currency::format($voucher['amount'], session::$data['currency'])
					);
				}
			}

			$data['totals'] = array();

			foreach ($order_data['totals'] as $total) {
				$data['totals'][] = array(
					'title' => $total['title'],
					'text'  => currency::format($total['value'], session::$data['currency'])
				);
			}

			$data['payment'] = load::controller('extension/payment/' . session::$data['payment_method']['code']);
		} else {
			$data['redirect'] = $redirect;
		}

		response::setOutput(load::view('checkout/confirm', $data));
	}
}
