<?php
class ControllerCheckoutGuest {
	public static function index() {
		load::language('checkout/checkout');

		$data['customer_groups'] = array();

		if (is_array(config('customer_group_display'))) {

			$customer_groups = account_customer_group::getCustomerGroups();

			foreach ($customer_groups as $customer_group) {
				if (in_array($customer_group['customer_group_id'], config('customer_group_display'))) {
					$data['customer_groups'][] = $customer_group;
				}
			}
		}

		if (isset(session::$data['guest']['customer_group_id'])) {
			$data['customer_group_id'] = session::$data['guest']['customer_group_id'];
		} else {
			$data['customer_group_id'] = config('customer_group_id');
		}

		if (isset(session::$data['guest']['firstname'])) {
			$data['firstname'] = session::$data['guest']['firstname'];
		} else {
			$data['firstname'] = '';
		}

		if (isset(session::$data['guest']['lastname'])) {
			$data['lastname'] = session::$data['guest']['lastname'];
		} else {
			$data['lastname'] = '';
		}

		if (isset(session::$data['guest']['email'])) {
			$data['email'] = session::$data['guest']['email'];
		} else {
			$data['email'] = '';
		}

		if (isset(session::$data['guest']['telephone'])) {
			$data['telephone'] = session::$data['guest']['telephone'];
		} else {
			$data['telephone'] = '';
		}

		if (isset(session::$data['payment_address']['company'])) {
			$data['company'] = session::$data['payment_address']['company'];
		} else {
			$data['company'] = '';
		}

		if (isset(session::$data['payment_address']['address_1'])) {
			$data['address_1'] = session::$data['payment_address']['address_1'];
		} else {
			$data['address_1'] = '';
		}

		if (isset(session::$data['payment_address']['address_2'])) {
			$data['address_2'] = session::$data['payment_address']['address_2'];
		} else {
			$data['address_2'] = '';
		}

		if (isset(session::$data['payment_address']['postcode'])) {
			$data['postcode'] = session::$data['payment_address']['postcode'];
		} elseif (isset(session::$data['shipping_address']['postcode'])) {
			$data['postcode'] = session::$data['shipping_address']['postcode'];
		} else {
			$data['postcode'] = '';
		}

		if (isset(session::$data['payment_address']['city'])) {
			$data['city'] = session::$data['payment_address']['city'];
		} else {
			$data['city'] = '';
		}

		if (isset(session::$data['payment_address']['country_id'])) {
			$data['country_id'] = session::$data['payment_address']['country_id'];
		} elseif (isset(session::$data['shipping_address']['country_id'])) {
			$data['country_id'] = session::$data['shipping_address']['country_id'];
		} else {
			$data['country_id'] = config('country_id');
		}

		if (isset(session::$data['payment_address']['zone_id'])) {
			$data['zone_id'] = session::$data['payment_address']['zone_id'];
		} elseif (isset(session::$data['shipping_address']['zone_id'])) {
			$data['zone_id'] = session::$data['shipping_address']['zone_id'];
		} else {
			$data['zone_id'] = '';
		}


		$data['countries'] = localisation_country::getCountries();

		// Custom Fields

		$data['custom_fields'] = account_custom_field::getCustomFields();

		if (isset(session::$data['guest']['custom_field'])) {
			if (isset(session::$data['guest']['custom_field'])) {
				$guest_custom_field = session::$data['guest']['custom_field'];
			} else {
				$guest_custom_field = array();
			}

			if (isset(session::$data['payment_address']['custom_field'])) {
				$address_custom_field = session::$data['payment_address']['custom_field'];
			} else {
				$address_custom_field = array();
			}

			$data['guest_custom_field'] = $guest_custom_field + $address_custom_field;
		} else {
			$data['guest_custom_field'] = array();
		}

		$data['shipping_required'] = cart::hasShipping();

		if (isset(session::$data['guest']['shipping_address'])) {
			$data['shipping_address'] = session::$data['guest']['shipping_address'];
		} else {
			$data['shipping_address'] = true;
		}

		// Captcha
		if (config::get('captcha_' . config('captcha') . '_status') && in_array('guest', (array)config('captcha_page'))) {
			$data['captcha'] = load::controller('extension/captcha/' . config('captcha'));
		} else {
			$data['captcha'] = '';
		}
		
		response::setOutput(load::view('checkout/guest', $data));
	}

	public static function save() {
		load::language('checkout/checkout');

		$json = array();

		// Validate if customer is logged in.
		if (customer::isLogged()) {
			$json['redirect'] = url::link('checkout/checkout', '');
		}

		// Validate cart has products and has stock.
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			$json['redirect'] = url::link('checkout/cart');
		}

		// Check if guest checkout is available.
		if (!config('checkout_guest') || config('customer_price') || cart::hasDownload()) {
			$json['redirect'] = url::link('checkout/checkout', '');
		}

		if (!$json) {
			if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
				$json['error']['firstname'] = language::get('error_firstname');
			}

			if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
				$json['error']['lastname'] = language::get('error_lastname');
			}

			if ((utf8_strlen(request::$post['email']) > 96) || !filter_var(request::$post['email'], FILTER_VALIDATE_EMAIL)) {
				$json['error']['email'] = language::get('error_email');
			}

			if ((utf8_strlen(request::$post['telephone']) < 3) || (utf8_strlen(request::$post['telephone']) > 32)) {

				$json['error']['telephone'] = language::get('error_telephone');
			}

			if ((utf8_strlen(trim(request::$post['address_1'])) < 3) || (utf8_strlen(trim(request::$post['address_1'])) > 128)) {
				$json['error']['address_1'] = language::get('error_address_1');
			}

			if ((utf8_strlen(trim(request::$post['city'])) < 2) || (utf8_strlen(trim(request::$post['city'])) > 128)) {
				$json['error']['city'] = language::get('error_city');
			}


			$country_info = localisation_country::getCountry(request::$post['country_id']);

			if ($country_info && $country_info['postcode_required'] && (utf8_strlen(trim(request::$post['postcode'])) < 2 || utf8_strlen(trim(request::$post['postcode'])) > 10)) {
				$json['error']['postcode'] = language::get('error_postcode');
			}

			if (request::$post['country_id'] == '') {
				$json['error']['country'] = language::get('error_country');
			}

			if (!isset(request::$post['zone_id']) || request::$post['zone_id'] == '' || !is_numeric(request::$post['zone_id'])) {
				$json['error']['zone'] = language::get('error_zone');
			}

			// Customer Group
			if (isset(request::$post['customer_group_id']) && is_array(config('customer_group_display')) && in_array(request::$post['customer_group_id'], config('customer_group_display'))) {
				$customer_group_id = request::$post['customer_group_id'];
			} else {
				$customer_group_id = config('customer_group_id');
			}

			// Custom field validation

			$custom_fields = account_custom_field::getCustomFields($customer_group_id);

			foreach ($custom_fields as $custom_field) {
				if($custom_field['location'] == 'affiliate') {
					continue;
				}
				
				if ($custom_field['required'] && empty(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']])) {
					$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
				} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
                    $json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
                }
			}

			// Captcha
			if (config::get('captcha_' . config('captcha') . '_status') && in_array('guest', (array)config('captcha_page'))) {
				$captcha = load::controller('extension/captcha/' . config('captcha') . '/validate');

				if ($captcha) {
					$json['error']['captcha'] = $captcha;
				}
			}
		}

		if (!$json) {
			session::$data['account'] = 'guest';

			session::$data['guest']['customer_group_id'] = $customer_group_id;
			session::$data['guest']['firstname'] = request::$post['firstname'];
			session::$data['guest']['lastname'] = request::$post['lastname'];
			session::$data['guest']['email'] = request::$post['email'];
			session::$data['guest']['telephone'] = request::$post['telephone'];

			if (isset(request::$post['custom_field']['account'])) {
				session::$data['guest']['custom_field'] = request::$post['custom_field']['account'];
			} else {
				session::$data['guest']['custom_field'] = array();
			}

			session::$data['payment_address']['firstname'] = request::$post['firstname'];
			session::$data['payment_address']['lastname'] = request::$post['lastname'];
			session::$data['payment_address']['company'] = request::$post['company'];
			session::$data['payment_address']['address_1'] = request::$post['address_1'];
			session::$data['payment_address']['address_2'] = request::$post['address_2'];
			session::$data['payment_address']['postcode'] = request::$post['postcode'];
			session::$data['payment_address']['city'] = request::$post['city'];
			session::$data['payment_address']['country_id'] = request::$post['country_id'];
			session::$data['payment_address']['zone_id'] = request::$post['zone_id'];


			$country_info = localisation_country::getCountry(request::$post['country_id']);

			if ($country_info) {
				session::$data['payment_address']['country'] = $country_info['name'];
				session::$data['payment_address']['iso_code_2'] = $country_info['iso_code_2'];
				session::$data['payment_address']['iso_code_3'] = $country_info['iso_code_3'];
				session::$data['payment_address']['address_format'] = $country_info['address_format'];
			} else {
				session::$data['payment_address']['country'] = '';
				session::$data['payment_address']['iso_code_2'] = '';
				session::$data['payment_address']['iso_code_3'] = '';
				session::$data['payment_address']['address_format'] = '';
			}

			if (isset(request::$post['custom_field']['address'])) {
				session::$data['payment_address']['custom_field'] = request::$post['custom_field']['address'];
			} else {
				session::$data['payment_address']['custom_field'] = array();
			}


			$zone_info = localisation_zone::getZone(request::$post['zone_id']);

			if ($zone_info) {
				session::$data['payment_address']['zone'] = $zone_info['name'];
				session::$data['payment_address']['zone_code'] = $zone_info['code'];
			} else {
				session::$data['payment_address']['zone'] = '';
				session::$data['payment_address']['zone_code'] = '';
			}

			if (!empty(request::$post['shipping_address'])) {
				session::$data['guest']['shipping_address'] = request::$post['shipping_address'];
			} else {
				session::$data['guest']['shipping_address'] = false;
			}

			if (session::$data['guest']['shipping_address']) {
				session::$data['shipping_address']['firstname'] = request::$post['firstname'];
				session::$data['shipping_address']['lastname'] = request::$post['lastname'];
				session::$data['shipping_address']['company'] = request::$post['company'];
				session::$data['shipping_address']['address_1'] = request::$post['address_1'];
				session::$data['shipping_address']['address_2'] = request::$post['address_2'];
				session::$data['shipping_address']['postcode'] = request::$post['postcode'];
				session::$data['shipping_address']['city'] = request::$post['city'];
				session::$data['shipping_address']['country_id'] = request::$post['country_id'];
				session::$data['shipping_address']['zone_id'] = request::$post['zone_id'];

				if ($country_info) {
					session::$data['shipping_address']['country'] = $country_info['name'];
					session::$data['shipping_address']['iso_code_2'] = $country_info['iso_code_2'];
					session::$data['shipping_address']['iso_code_3'] = $country_info['iso_code_3'];
					session::$data['shipping_address']['address_format'] = $country_info['address_format'];
				} else {
					session::$data['shipping_address']['country'] = '';
					session::$data['shipping_address']['iso_code_2'] = '';
					session::$data['shipping_address']['iso_code_3'] = '';
					session::$data['shipping_address']['address_format'] = '';
				}

				if ($zone_info) {
					session::$data['shipping_address']['zone'] = $zone_info['name'];
					session::$data['shipping_address']['zone_code'] = $zone_info['code'];
				} else {
					session::$data['shipping_address']['zone'] = '';
					session::$data['shipping_address']['zone_code'] = '';
				}

				if (isset(request::$post['custom_field']['address'])) {
					session::$data['shipping_address']['custom_field'] = request::$post['custom_field']['address'];
				} else {
					session::$data['shipping_address']['custom_field'] = array();
				}
			}

			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
			unset(session::$data['payment_method']);
			unset(session::$data['payment_methods']);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
