<?php
class ControllerCheckoutGuestShipping {
	public static function index() {
		load::language('checkout/checkout');

		if (isset(session::$data['shipping_address']['firstname'])) {
			$data['firstname'] = session::$data['shipping_address']['firstname'];
		} else {
			$data['firstname'] = '';
		}

		if (isset(session::$data['shipping_address']['lastname'])) {
			$data['lastname'] = session::$data['shipping_address']['lastname'];
		} else {
			$data['lastname'] = '';
		}

		if (isset(session::$data['shipping_address']['company'])) {
			$data['company'] = session::$data['shipping_address']['company'];
		} else {
			$data['company'] = '';
		}

		if (isset(session::$data['shipping_address']['address_1'])) {
			$data['address_1'] = session::$data['shipping_address']['address_1'];
		} else {
			$data['address_1'] = '';
		}

		if (isset(session::$data['shipping_address']['address_2'])) {
			$data['address_2'] = session::$data['shipping_address']['address_2'];
		} else {
			$data['address_2'] = '';
		}

		if (isset(session::$data['shipping_address']['postcode'])) {
			$data['postcode'] = session::$data['shipping_address']['postcode'];
		} else {
			$data['postcode'] = '';
		}

		if (isset(session::$data['shipping_address']['city'])) {
			$data['city'] = session::$data['shipping_address']['city'];
		} else {
			$data['city'] = '';
		}

		if (isset(session::$data['shipping_address']['country_id'])) {
			$data['country_id'] = session::$data['shipping_address']['country_id'];
		} else {
			$data['country_id'] = config('country_id');
		}

		if (isset(session::$data['shipping_address']['zone_id'])) {
			$data['zone_id'] = session::$data['shipping_address']['zone_id'];
		} else {
			$data['zone_id'] = '';
		}


		$data['countries'] = localisation_country::getCountries();

		// Custom Fields
		$data['custom_fields'] = array();


		$custom_fields = account_custom_field::getCustomFields(session::$data['guest']['customer_group_id']);

		foreach ($custom_fields as $custom_field) {
			if ($custom_field['location'] == 'address') {
				$data['custom_fields'][] = $custom_field;
			}
		}

		if (isset(session::$data['shipping_address']['custom_field'])) {
			$data['address_custom_field'] = session::$data['shipping_address']['custom_field'];
		} else {
			$data['address_custom_field'] = array();
		}

		response::setOutput(load::view('checkout/guest_shipping', $data));
	}

	public static function save() {
		load::language('checkout/checkout');

		$json = array();

		// Validate if customer is logged in.
		if (customer::isLogged()) {
			$json['redirect'] = url::link('checkout/checkout', '');
		}

		// Validate cart has products and has stock.
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			$json['redirect'] = url::link('checkout/cart');
		}

		// Check if guest checkout is available.
		if (!config('checkout_guest') || config('customer_price') || cart::hasDownload()) {
			$json['redirect'] = url::link('checkout/checkout', '');
		}

		if (!$json) {
			if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
				$json['error']['firstname'] = language::get('error_firstname');
			}

			if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
				$json['error']['lastname'] = language::get('error_lastname');
			}

			if ((utf8_strlen(trim(request::$post['address_1'])) < 3) || (utf8_strlen(trim(request::$post['address_1'])) > 128)) {
				$json['error']['address_1'] = language::get('error_address_1');
			}

			if ((utf8_strlen(trim(request::$post['city'])) < 2) || (utf8_strlen(trim(request::$post['city'])) > 128)) {
				$json['error']['city'] = language::get('error_city');
			}


			$country_info = localisation_country::getCountry(request::$post['country_id']);

			if ($country_info && $country_info['postcode_required'] && (utf8_strlen(trim(request::$post['postcode'])) < 2 || utf8_strlen(trim(request::$post['postcode'])) > 10)) {
				$json['error']['postcode'] = language::get('error_postcode');
			}

			if (request::$post['country_id'] == '') {
				$json['error']['country'] = language::get('error_country');
			}

			if (!isset(request::$post['zone_id']) || request::$post['zone_id'] == '' || !is_numeric(request::$post['zone_id'])) {
				$json['error']['zone'] = language::get('error_zone');
			}

			// Custom field validation

			$custom_fields = account_custom_field::getCustomFields(session::$data['guest']['customer_group_id']);

			foreach ($custom_fields as $custom_field) {
				if ($custom_field['location'] == 'address') { 
					if ($custom_field['required'] && empty(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']])) {
						$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
					} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
						$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
					}
				}
			}
		}

		if (!$json) {
			session::$data['shipping_address']['firstname'] = request::$post['firstname'];
			session::$data['shipping_address']['lastname'] = request::$post['lastname'];
			session::$data['shipping_address']['company'] = request::$post['company'];
			session::$data['shipping_address']['address_1'] = request::$post['address_1'];
			session::$data['shipping_address']['address_2'] = request::$post['address_2'];
			session::$data['shipping_address']['postcode'] = request::$post['postcode'];
			session::$data['shipping_address']['city'] = request::$post['city'];
			session::$data['shipping_address']['country_id'] = request::$post['country_id'];
			session::$data['shipping_address']['zone_id'] = request::$post['zone_id'];


			$country_info = localisation_country::getCountry(request::$post['country_id']);

			if ($country_info) {
				session::$data['shipping_address']['country'] = $country_info['name'];
				session::$data['shipping_address']['iso_code_2'] = $country_info['iso_code_2'];
				session::$data['shipping_address']['iso_code_3'] = $country_info['iso_code_3'];
				session::$data['shipping_address']['address_format'] = $country_info['address_format'];
			} else {
				session::$data['shipping_address']['country'] = '';
				session::$data['shipping_address']['iso_code_2'] = '';
				session::$data['shipping_address']['iso_code_3'] = '';
				session::$data['shipping_address']['address_format'] = '';
			}


			$zone_info = localisation_zone::getZone(request::$post['zone_id']);

			if ($zone_info) {
				session::$data['shipping_address']['zone'] = $zone_info['name'];
				session::$data['shipping_address']['zone_code'] = $zone_info['code'];
			} else {
				session::$data['shipping_address']['zone'] = '';
				session::$data['shipping_address']['zone_code'] = '';
			}

			if (isset(request::$post['custom_field'])) {
				session::$data['shipping_address']['custom_field'] = request::$post['custom_field']['address'];
			} else {
				session::$data['shipping_address']['custom_field'] = array();
			}

			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}