<?php
class ControllerCheckoutLogin {
	public static function index() {
		load::language('checkout/checkout');

		$data['checkout_guest'] = (config('checkout_guest') && !config('customer_price') && !cart::hasDownload());

		if (isset(session::$data['account'])) {
			$data['account'] = session::$data['account'];
		} else {
			$data['account'] = 'register';
		}

		$data['forgotten'] = url::link('account/forgotten', '');

		response::setOutput(load::view('checkout/login', $data));
	}

	public static function save() {
		load::language('checkout/checkout');

		$json = array();

		if (customer::isLogged()) {
			$json['redirect'] = url::link('checkout/checkout', '');
		}

		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			$json['redirect'] = url::link('checkout/cart');
		}

		if (!$json) {

			// Check how many login attempts have been made.
			$login_info = account_customer::getLoginAttempts(request::$post['email']);

			if ($login_info && ($login_info['total'] >= config('login_attempts')) && strtotime('-1 hour') < strtotime($login_info['date_modified'])) {
				$json['error']['warning'] = language::get('error_attempts');
			}

			// Check if customer has been approved.
			$customer_info = account_customer::getCustomerByEmail(request::$post['email']);

			if ($customer_info && !$customer_info['status']) {
				$json['error']['warning'] = language::get('error_approved');
			}

			if (!isset($json['error'])) {
				if (!customer::login(request::$post['email'], request::$post['password'])) {
					$json['error']['warning'] = language::get('error_login');

					account_customer::addLoginAttempt(request::$post['email']);
				} else {
					account_customer::deleteLoginAttempts(request::$post['email']);
				}
			}
		}

		if (!$json) {
			// Unset guest
			unset(session::$data['guest']);

			// Default Shipping Address

			if (config('tax_customer') == 'payment') {
				session::$data['payment_address'] = account_address::getAddress(customer::getAddressId());
			}

			if (config('tax_customer') == 'shipping') {
				session::$data['shipping_address'] = account_address::getAddress(customer::getAddressId());
			}

			// Wishlist
			if (isset(session::$data['wishlist']) && is_array(session::$data['wishlist'])) {

				foreach (session::$data['wishlist'] as $key => $product_id) {
					account_wishlist::addWishlist($product_id);

					unset(session::$data['wishlist'][$key]);
				}
			}

			$json['redirect'] = url::link('checkout/checkout', '');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
