<?php
class ControllerCheckoutPaymentAddress {
	public static function index() {
		load::language('checkout/checkout');

		if (isset(session::$data['payment_address']['address_id'])) {
			$data['address_id'] = session::$data['payment_address']['address_id'];
		} else {
			$data['address_id'] = customer::getAddressId();
		}


		$data['addresses'] = account_address::getAddresses();

		if (isset(session::$data['payment_address']['country_id'])) {
			$data['country_id'] = session::$data['payment_address']['country_id'];
		} else {
			$data['country_id'] = config('country_id');
		}

		if (isset(session::$data['payment_address']['zone_id'])) {
			$data['zone_id'] = session::$data['payment_address']['zone_id'];
		} else {
			$data['zone_id'] = '';
		}


		$data['countries'] = localisation_country::getCountries();

		// Custom Fields
		$data['custom_fields'] = array();
		

		$custom_fields = account_custom_field::getCustomFields(config('customer_group_id'));

		foreach ($custom_fields as $custom_field) {
			if ($custom_field['location'] == 'address') {
				$data['custom_fields'][] = $custom_field;
			}
		}

		if (isset(session::$data['payment_address']['custom_field'])) {
			$data['payment_address_custom_field'] = session::$data['payment_address']['custom_field'];
		} else {
			$data['payment_address_custom_field'] = array();
		}

		response::setOutput(load::view('checkout/payment_address', $data));
	}

	public static function save() {
		load::language('checkout/checkout');

		$json = array();

		// Validate if customer is logged in.
		if (!customer::isLogged()) {
			$json['redirect'] = url::link('checkout/checkout', '');
		}

		// Validate cart has products and has stock.
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			$json['redirect'] = url::link('checkout/cart');
		}

		// Validate minimum quantity requirements.
		$products = cart::getProducts();

		foreach ($products as $product) {
			$product_total = 0;

			foreach ($products as $product_2) {
				if ($product_2['product_id'] == $product['product_id']) {
					$product_total += $product_2['quantity'];
				}
			}

			if ($product['minimum'] > $product_total) {
				$json['redirect'] = url::link('checkout/cart');

				break;
			}
		}

		if (!$json) {
							
			if (isset(request::$post['payment_address']) && request::$post['payment_address'] == 'existing') {
				if (empty(request::$post['address_id'])) {
					$json['error']['warning'] = language::get('error_address');
				} elseif (!in_array(request::$post['address_id'], array_keys(account_address::getAddresses()))) {
					$json['error']['warning'] = language::get('error_address');
				}

				if (!$json) {
					session::$data['payment_address'] = account_address::getAddress(request::$post['address_id']);

					unset(session::$data['payment_method']);
					unset(session::$data['payment_methods']);
				}
			} else {
				if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
					$json['error']['firstname'] = language::get('error_firstname');
				}

				if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
					$json['error']['lastname'] = language::get('error_lastname');
				}

				if ((utf8_strlen(trim(request::$post['address_1'])) < 3) || (utf8_strlen(trim(request::$post['address_1'])) > 128)) {
					$json['error']['address_1'] = language::get('error_address_1');
				}

				if ((utf8_strlen(request::$post['city']) < 2) || (utf8_strlen(request::$post['city']) > 32)) {
					$json['error']['city'] = language::get('error_city');
				}


				$country_info = localisation_country::getCountry(request::$post['country_id']);

				if ($country_info && $country_info['postcode_required'] && (utf8_strlen(trim(request::$post['postcode'])) < 2 || utf8_strlen(trim(request::$post['postcode'])) > 10)) {
					$json['error']['postcode'] = language::get('error_postcode');
				}

				if (request::$post['country_id'] == '') {
					$json['error']['country'] = language::get('error_country');
				}

				if (!isset(request::$post['zone_id']) || request::$post['zone_id'] == '' || !is_numeric(request::$post['zone_id'])) {
					$json['error']['zone'] = language::get('error_zone');
				}

				// Custom field validation

				$custom_fields = account_custom_field::getCustomFields(config('customer_group_id'));

				foreach ($custom_fields as $custom_field) {
					if ($custom_field['location'] == 'address') {
						if ($custom_field['required'] && empty(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']])) {
							$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
						} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
							$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
						}
					}
				}

				if (!$json) {
					$address_id = account_address::addAddress(customer::getId(), request::$post);

					session::$data['payment_address'] = account_address::getAddress($address_id);

					// If no default address ID set we use the last address
					if (!customer::getAddressId()) {
						
						account_customer::editAddressId(customer::getId(), $address_id);
					}

					unset(session::$data['payment_method']);
					unset(session::$data['payment_methods']);
				}
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}