<?php
class ControllerCheckoutPaymentMethod {
	public static function index() {
		load::language('checkout/checkout');

		if (isset(session::$data['payment_address'])) {
			// Totals
			$totals = array();
			$taxes = cart::getTaxes();
			$total = 0;

			// Because __call can not keep var references so we put them into an array.
			$total_data = array(
				'totals' => &$totals,
				'taxes'  => &$taxes,
				'total'  => &$total
			);
			

			$sort_order = array();

			$results = setting_extension::getExtensions('total');

			foreach ($results as $key => $value) {
				$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
			}

			array_multisort($sort_order, SORT_ASC, $results);

			foreach ($results as $result) {
				if (config::get('total_' . $result['code'] . '_status')) {
					
					// We have to put the totals in an array so that they pass by reference.
					call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
				}
			}

			// Payment Methods
			$method_data = array();


			$results = setting_extension::getExtensions('payment');

			$recurring = cart::hasRecurringProducts();

			foreach ($results as $result) {
				if (config::get('payment_' . $result['code'] . '_status')) {

					$method = call_model_method('extension_payment_' . $result['code'], 'getMethod', session::$data['payment_address'], $total);

					if ($method) {
						if ($recurring) {
							if (model_method_exists('extension_payment_' . $result['code'], 'recurringPayments') && call_model_method('extension_payment_' . $result['code'], 'recurringPayments', )) {
								$method_data[$result['code']] = $method;
							}
						} else {
							$method_data[$result['code']] = $method;
						}
					}
				}
			}

			$sort_order = array();

			foreach ($method_data as $key => $value) {
				$sort_order[$key] = $value['sort_order'];
			}

			array_multisort($sort_order, SORT_ASC, $method_data);

			session::$data['payment_methods'] = $method_data;
		}

		if (empty(session::$data['payment_methods'])) {
			$data['error_warning'] = sprintf(language::get('error_no_payment'), url::link('information/contact'));
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['payment_methods'])) {
			$data['payment_methods'] = session::$data['payment_methods'];
		} else {
			$data['payment_methods'] = array();
		}

		if (isset(session::$data['payment_method']['code'])) {
			$data['code'] = session::$data['payment_method']['code'];
		} else {
			$data['code'] = '';
		}

		if (isset(session::$data['comment'])) {
			$data['comment'] = session::$data['comment'];
		} else {
			$data['comment'] = '';
		}

		$data['scripts'] = document::getScripts();

		if (config('checkout_id')) {

			$information_info = catalog_information::getInformation(config('checkout_id'));

			if ($information_info) {
				$data['text_agree'] = sprintf(language::get('text_agree'), url::link('information/information/agree', 'information_id=' . config('checkout_id')), $information_info['title']);
			} else {
				$data['text_agree'] = '';
			}
		} else {
			$data['text_agree'] = '';
		}

		if (isset(session::$data['agree'])) {
			$data['agree'] = session::$data['agree'];
		} else {
			$data['agree'] = '';
		}

		response::setOutput(load::view('checkout/payment_method', $data));
	}

	public static function save() {
		load::language('checkout/checkout');

		$json = array();

		// Validate if payment address has been set.
		if (!isset(session::$data['payment_address'])) {
			$json['redirect'] = url::link('checkout/checkout', '');
		}

		// Validate cart has products and has stock.
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			$json['redirect'] = url::link('checkout/cart');
		}

		// Validate minimum quantity requirements.
		$products = cart::getProducts();

		foreach ($products as $product) {
			$product_total = 0;

			foreach ($products as $product_2) {
				if ($product_2['product_id'] == $product['product_id']) {
					$product_total += $product_2['quantity'];
				}
			}

			if ($product['minimum'] > $product_total) {
				$json['redirect'] = url::link('checkout/cart');

				break;
			}
		}

		if (!isset(request::$post['payment_method'])) {
			$json['error']['warning'] = language::get('error_payment');
		} elseif (!isset(session::$data['payment_methods'][request::$post['payment_method']])) {
			$json['error']['warning'] = language::get('error_payment');
		}

		if (config('checkout_id')) {

			$information_info = catalog_information::getInformation(config('checkout_id'));

			if ($information_info && !isset(request::$post['agree'])) {
				$json['error']['warning'] = sprintf(language::get('error_agree'), $information_info['title']);
			}
		}

		if (!$json) {
			session::$data['payment_method'] = session::$data['payment_methods'][request::$post['payment_method']];

			session::$data['comment'] = strip_tags(request::$post['comment']);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
