<?php
class ControllerCheckoutRegister {
	public static function index() {
		load::language('checkout/checkout');
		
		$data['entry_newsletter'] = sprintf(language::get('entry_newsletter'), config('name'));

		$data['customer_groups'] = array();

		if (is_array(config('customer_group_display'))) {

			$customer_groups = account_customer_group::getCustomerGroups();

			foreach ($customer_groups  as $customer_group) {
				if (in_array($customer_group['customer_group_id'], config('customer_group_display'))) {
					$data['customer_groups'][] = $customer_group;
				}
			}
		}

		$data['customer_group_id'] = config('customer_group_id');

		if (isset(session::$data['shipping_address']['postcode'])) {
			$data['postcode'] = session::$data['shipping_address']['postcode'];
		} else {
			$data['postcode'] = '';
		}

		if (isset(session::$data['shipping_address']['country_id'])) {
			$data['country_id'] = session::$data['shipping_address']['country_id'];
		} else {
			$data['country_id'] = config('country_id');
		}

		if (isset(session::$data['shipping_address']['zone_id'])) {
			$data['zone_id'] = session::$data['shipping_address']['zone_id'];
		} else {
			$data['zone_id'] = '';
		}


		$data['countries'] = localisation_country::getCountries();

		// Custom Fields

		$data['custom_fields'] = account_custom_field::getCustomFields();

		// Captcha
		if (config::get('captcha_' . config('captcha') . '_status') && in_array('register', (array)config('captcha_page'))) {
			$data['captcha'] = load::controller('extension/captcha/' . config('captcha'));
		} else {
			$data['captcha'] = '';
		}

		if (config('account_id')) {

			$information_info = catalog_information::getInformation(config('account_id'));

			if ($information_info) {
				$data['text_agree'] = sprintf(language::get('text_agree'), url::link('information/information/agree', 'information_id=' . config('account_id')), $information_info['title']);
			} else {
				$data['text_agree'] = '';
			}
		} else {
			$data['text_agree'] = '';
		}

		$data['shipping_required'] = cart::hasShipping();
		
		response::setOutput(load::view('checkout/register', $data));
	}

	public static function save() {
		load::language('checkout/checkout');

		$json = array();

		// Validate if customer is already logged out.
		if (customer::isLogged()) {
			$json['redirect'] = url::link('checkout/checkout', '');
		}

		// Validate cart has products and has stock.
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			$json['redirect'] = url::link('checkout/cart');
		}

		// Validate minimum quantity requirements.
		$products = cart::getProducts();

		foreach ($products as $product) {
			$product_total = 0;

			foreach ($products as $product_2) {
				if ($product_2['product_id'] == $product['product_id']) {
					$product_total += $product_2['quantity'];
				}
			}

			if ($product['minimum'] > $product_total) {
				$json['redirect'] = url::link('checkout/cart');

				break;
			}
		}

		if (!$json) {

			if ((utf8_strlen(trim(request::$post['firstname'])) < 1) || (utf8_strlen(trim(request::$post['firstname'])) > 32)) {
				$json['error']['firstname'] = language::get('error_firstname');
			}

			if ((utf8_strlen(trim(request::$post['lastname'])) < 1) || (utf8_strlen(trim(request::$post['lastname'])) > 32)) {
				$json['error']['lastname'] = language::get('error_lastname');
			}

			if ((utf8_strlen(request::$post['email']) > 96) || !filter_var(request::$post['email'], FILTER_VALIDATE_EMAIL)) {
				$json['error']['email'] = language::get('error_email');
			}

			if (account_customer::getTotalCustomersByEmail(request::$post['email'])) {
				$json['error']['warning'] = language::get('error_exists');
			}

			if ((utf8_strlen(request::$post['telephone']) < 3) || (utf8_strlen(request::$post['telephone']) > 32)) {
				$json['error']['telephone'] = language::get('error_telephone');
			}

			if ((utf8_strlen(trim(request::$post['address_1'])) < 3) || (utf8_strlen(trim(request::$post['address_1'])) > 128)) {
				$json['error']['address_1'] = language::get('error_address_1');
			}

			if ((utf8_strlen(trim(request::$post['city'])) < 2) || (utf8_strlen(trim(request::$post['city'])) > 128)) {
				$json['error']['city'] = language::get('error_city');
			}


			$country_info = localisation_country::getCountry(request::$post['country_id']);

			if ($country_info && $country_info['postcode_required'] && (utf8_strlen(trim(request::$post['postcode'])) < 2 || utf8_strlen(trim(request::$post['postcode'])) > 10)) {
				$json['error']['postcode'] = language::get('error_postcode');
			}

			if (request::$post['country_id'] == '') {
				$json['error']['country'] = language::get('error_country');
			}

			if (!isset(request::$post['zone_id']) || request::$post['zone_id'] == '' || !is_numeric(request::$post['zone_id'])) {
				$json['error']['zone'] = language::get('error_zone');
			}

			if ((utf8_strlen(html_entity_decode(request::$post['password'], ENT_QUOTES, 'UTF-8')) < 4) || (utf8_strlen(html_entity_decode(request::$post['password'], ENT_QUOTES, 'UTF-8')) > 40)) {
				$json['error']['password'] = language::get('error_password');
			}

			if (request::$post['confirm'] != request::$post['password']) {
				$json['error']['confirm'] = language::get('error_confirm');
			}

			if (config('account_id')) {

				$information_info = catalog_information::getInformation(config('account_id'));

				if ($information_info && !isset(request::$post['agree'])) {
					$json['error']['warning'] = sprintf(language::get('error_agree'), $information_info['title']);
				}
			}

			// Customer Group
			if (isset(request::$post['customer_group_id']) && is_array(config('customer_group_display')) && in_array(request::$post['customer_group_id'], config('customer_group_display'))) {
				$customer_group_id = request::$post['customer_group_id'];
			} else {
				$customer_group_id = config('customer_group_id');
			}

			// Custom field validation

			$custom_fields = account_custom_field::getCustomFields($customer_group_id);

			foreach ($custom_fields as $custom_field) {
				if($custom_field['location'] == 'affiliate') {
					continue;
				}

				if ($custom_field['required'] && empty(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']])) {
					$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
				} elseif (($custom_field['type'] == 'text') && !empty($custom_field['validation']) && !filter_var(request::$post['custom_field'][$custom_field['location']][$custom_field['custom_field_id']], FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => $custom_field['validation'])))) {
					$json['error']['custom_field' . $custom_field['custom_field_id']] = sprintf(language::get('error_custom_field'), $custom_field['name']);
				}
			}

			// Captcha
			if (config::get('captcha_' . config('captcha') . '_status') && in_array('register', (array)config('captcha_page'))) {
				$captcha = load::controller('extension/captcha/' . config('captcha') . '/validate');

				if ($captcha) {
					$json['error']['captcha'] = $captcha;
				}
			}
		}

		if (!$json) {
			$customer_id = account_customer::addCustomer(request::$post);

			// Default Payment Address
				
			$address_id = account_address::addAddress($customer_id, request::$post);
			
			// Set the address as default
			account_customer::editAddressId($customer_id, $address_id);
			
			// Clear any previous login attempts for unregistered accounts.
			account_customer::deleteLoginAttempts(request::$post['email']);

			session::$data['account'] = 'register';


			$customer_group_info = account_customer_group::getCustomerGroup($customer_group_id);

			if ($customer_group_info && !$customer_group_info['approval']) {
				customer::login(request::$post['email'], request::$post['password']);

				session::$data['payment_address'] = account_address::getAddress(customer::getAddressId());

				if (!empty(request::$post['shipping_address'])) {
					session::$data['shipping_address'] = account_address::getAddress(customer::getAddressId());
				}
			} else {
				$json['redirect'] = url::link('account/success');
			}

			unset(session::$data['guest']);
			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
			unset(session::$data['payment_method']);
			unset(session::$data['payment_methods']);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
