<?php
class ControllerCheckoutShippingMethod {
	public static function index() {
		load::language('checkout/checkout');

		if (isset(session::$data['shipping_address'])) {
			// Shipping Methods
			$method_data = array();


			$results = setting_extension::getExtensions('shipping');

			foreach ($results as $result) {
				if (config::get('shipping_' . $result['code'] . '_status')) {

					$quote = call_model_method('extension_shipping_' . $result['code'], 'getQuote', session::$data['shipping_address']);

					if ($quote) {
						$method_data[$result['code']] = array(
							'title'      => $quote['title'],
							'quote'      => $quote['quote'],
							'sort_order' => $quote['sort_order'],
							'error'      => $quote['error']
						);
					}
				}
			}

			$sort_order = array();

			foreach ($method_data as $key => $value) {
				$sort_order[$key] = $value['sort_order'];
			}

			array_multisort($sort_order, SORT_ASC, $method_data);

			session::$data['shipping_methods'] = $method_data;
		}

		if (empty(session::$data['shipping_methods'])) {
			$data['error_warning'] = sprintf(language::get('error_no_shipping'), url::link('information/contact'));
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['shipping_methods'])) {
			$data['shipping_methods'] = session::$data['shipping_methods'];
		} else {
			$data['shipping_methods'] = array();
		}

		if (isset(session::$data['shipping_method']['code'])) {
			$data['code'] = session::$data['shipping_method']['code'];
		} else {
			$data['code'] = '';
		}

		if (isset(session::$data['comment'])) {
			$data['comment'] = session::$data['comment'];
		} else {
			$data['comment'] = '';
		}
		
		response::setOutput(load::view('checkout/shipping_method', $data));
	}

	public static function save() {
		load::language('checkout/checkout');

		$json = array();

		// Validate if shipping is required. If not the customer should not have reached this page.
		if (!cart::hasShipping()) {
			$json['redirect'] = url::link('checkout/checkout', '');
		}

		// Validate if shipping address has been set.
		if (!isset(session::$data['shipping_address'])) {
			$json['redirect'] = url::link('checkout/checkout', '');
		}

		// Validate cart has products and has stock.
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			$json['redirect'] = url::link('checkout/cart');
		}

		// Validate minimum quantity requirements.
		$products = cart::getProducts();

		foreach ($products as $product) {
			$product_total = 0;

			foreach ($products as $product_2) {
				if ($product_2['product_id'] == $product['product_id']) {
					$product_total += $product_2['quantity'];
				}
			}

			if ($product['minimum'] > $product_total) {
				$json['redirect'] = url::link('checkout/cart');

				break;
			}
		}

		if (!isset(request::$post['shipping_method'])) {
			$json['error']['warning'] = language::get('error_shipping');
		} else {
			$shipping = explode('.', request::$post['shipping_method']);

			if (!isset($shipping[0]) || !isset($shipping[1]) || !isset(session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]])) {
				$json['error']['warning'] = language::get('error_shipping');
			}
		}

		if (!$json) {
			session::$data['shipping_method'] = session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]];

			session::$data['comment'] = strip_tags(request::$post['comment']);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}