<?php
class ControllerCommonCart {
	public static function index() {
		load::language('common/cart');

		// Totals

		$totals = array();
		$taxes = cart::getTaxes();
		$total = 0;

		// Because __call can not keep var references so we put them into an array.
		$total_data = array(
			'totals' => &$totals,
			'taxes'  => &$taxes,
			'total'  => &$total
		);
			
		// Display prices
		if (customer::isLogged() || !config('customer_price')) {
			$sort_order = array();

			$results = setting_extension::getExtensions('total');

			foreach ($results as $key => $value) {
				$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
			}

			array_multisort($sort_order, SORT_ASC, $results);

			foreach ($results as $result) {
				if (config::get('total_' . $result['code'] . '_status')) {

					// We have to put the totals in an array so that they pass by reference.
					call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
				}
			}

			$sort_order = array();

			foreach ($totals as $key => $value) {
				$sort_order[$key] = $value['sort_order'];
			}

			array_multisort($sort_order, SORT_ASC, $totals);
		}

		$data['text_items'] = sprintf(language::get('text_items'), cart::countProducts() + (isset(session::$data['vouchers']) ? count(session::$data['vouchers']) : 0), currency::format($total, session::$data['currency']));


		$data['products'] = array();

		foreach (cart::getProducts() as $product) {
			if ($product['image']) {
				$image = tool_image::resize($product['image'], config::get('theme_' . config('theme') . '_image_cart_width'), config::get('theme_' . config('theme') . '_image_cart_height'));
			} else {
				$image = tool_image::resize('placeholder.png', config::get('theme_' . config('theme') . '_image_cart_width'), config::get('theme_' . config('theme') . '_image_cart_height'));
			}

			$option_data = array();

			foreach ($product['option'] as $option) {
				if ($option['type'] != 'file') {
					$value = $option['value'];
				} else {
					$upload_info = tool_upload::getUploadByCode($option['value']);

					if ($upload_info) {
						$value = $upload_info['name'];
					} else {
						$value = '';
					}
				}

				$option_data[] = array(
					'name'  => $option['name'],
					'value' => (utf8_strlen($value) > 20 ? utf8_substr($value, 0, 20) . '..' : $value),
					'type'  => $option['type']
				);
			}

			// Display prices
			if (customer::isLogged() || !config('customer_price')) {
				$unit_price = tax::calculate($product['price'], $product['tax_class_id'], config('tax'));
				
				$price = currency::format($unit_price, session::$data['currency']);
				$total = currency::format($unit_price * $product['quantity'], session::$data['currency']);
			} else {
				$price = false;
				$total = false;
			}

			$data['products'][] = array(
				'cart_id'   => $product['cart_id'],
				'thumb'     => $image,
				'name'      => $product['name'],
				'model'     => $product['model'],
				'option'    => $option_data,
				'recurring' => ($product['recurring'] ? $product['recurring']['name'] : ''),
				'quantity'  => $product['quantity'],
				'price'     => $price,
				'total'     => $total,
				'href'      => url::link('product/product', 'product_id=' . $product['product_id'])
			);
		}

		// Gift Voucher
		$data['vouchers'] = array();

		if (!empty(session::$data['vouchers'])) {
			foreach (session::$data['vouchers'] as $key => $voucher) {
				$data['vouchers'][] = array(
					'key'         => $key,
					'description' => $voucher['description'],
					'amount'      => currency::format($voucher['amount'], session::$data['currency'])
				);
			}
		}

		$data['totals'] = array();

		foreach ($totals as $total) {
			$data['totals'][] = array(
				'title' => $total['title'],
				'text'  => currency::format($total['value'], session::$data['currency']),
			);
		}

		$data['cart'] = url::link('checkout/cart');
		$data['checkout'] = url::link('checkout/checkout', '');

		return load::view('common/cart', $data);
	}

	public static function info() {
		response::setOutput(self::index());
	}
}