<?php

use \googleshopping\traits\StoreLoader;
use \googleshopping\traits\LibraryLoader;

class ControllerExtensionAdvertiseGoogle {
    use StoreLoader;
    use LibraryLoader;

    private static $store_id = 0;

    public static function init() {
        parent::__construct();

        if (getenv("ADVERTISE_GOOGLE_STORE_ID")) {
            self::$store_id = (int)getenv("ADVERTISE_GOOGLE_STORE_ID");
        } else {
            self::$store_id = config('store_id');
        }

        self::loadStore(self::$store_id);
    }

    public static function google_global_site_tag(&$route, &$data, &$output) {
        // In case the extension is disabled, do nothing
        if (!setting::get('advertise_google_status')) {
            return;
        }

        // If there is no tracker, do nothing
        if (!setting::has('advertise_google_conversion_tracker')) {
            return;
        }

        $tracker = setting::get('advertise_google_conversion_tracker');

        // Insert the tags before the closing <head> tag
        $output = str_replace('</head>', $tracker['google_global_site_tag'] . '</head>', $output);
    }

    public static function before_checkout_success(&$route, &$data) {
        // In case the extension is disabled, do nothing
        if (!setting::get('advertise_google_status')) {
            return;
        }

        // If there is no tracker, do nothing
        if (!setting::has('advertise_google_conversion_tracker')) {
            return;
        }

        // In case there is no order, do nothing
        if (!isset(session::$data['order_id'])) {
            return;
        }

        if (!class_exists('googleshopping')) {
            self::loadLibrary(self::$store_id);
        }


        $order_info = checkout_order::getOrder(session::$data['order_id']);

        $tracker = setting::get('advertise_google_conversion_tracker');
        $currency = $order_info['currency_code'];
        
        $total = googleshopping::convertAndFormat($order_info['total'], $currency);

        $search = array(
            '{VALUE}',
            '{CURRENCY}'
        );

        $replace = array(
            $total,
            $currency
        );

        $snippet = str_replace($search, $replace, $tracker['google_event_snippet']);

        // Store the snippet to display it in the order success view
        $tax = 0;
        $shipping = 0;
        $coupon = extension_advertise_google::getCoupon($order_info['order_id']);

        foreach (checkout_order::getOrderTotals($order_info['order_id']) as $order_total) {
            if ($order_total['code'] == 'shipping') {
                $shipping += googleshopping::convertAndFormat($order_total['value'], $currency);
            }

            if ($order_total['code'] == 'tax') {
                $tax += googleshopping::convertAndFormat($order_total['value'], $currency);
            }
        }

        $order_products = checkout_order::getOrderProducts($order_info['order_id']);

        foreach ($order_products as &$order_product) {
            $order_product['option'] = checkout_order::getOrderOptions($order_info['order_id'], $order_product['order_product_id']);
        }

        $purchase_data = array(
            'transaction_id' => $order_info['order_id'],
            'value' => $total,
            'currency' => $currency,
            'tax' => $tax,
            'shipping' => $shipping,
            'items' => extension_advertise_google::getRemarketingItems($order_products, $order_info['store_id']),
            'ecomm_prodid' => extension_advertise_google::getRemarketingProductIds($order_products, $order_info['store_id'])
        );

        if ($coupon !== null) {
            $purchase_data['coupon'] = $coupon;
        }

        googleshopping::setEventSnippet($snippet);
        googleshopping::setPurchaseData($purchase_data);
    }

    public static function google_dynamic_remarketing_purchase(&$route, &$data, &$output) {
        // In case the extension is disabled, do nothing
        if (!setting::get('advertise_google_status')) {
            return;
        }

        // If the library has not been loaded, or if there is no snippet, do nothing
        if (!class_exists('googleshopping') || googleshopping::getEventSnippet() === null || googleshopping::getPurchaseData() === null) {
            return;
        }

        $data['send_to'] = googleshopping::getEventSnippetSendTo();

        $purchase_data = googleshopping::getPurchaseData();

        $data['transaction_id'] = $purchase_data['transaction_id'];
        $data['value'] = $purchase_data['value'];
        $data['currency'] = $purchase_data['currency'];
        $data['tax'] = $purchase_data['tax'];
        $data['shipping'] = $purchase_data['shipping'];
        $data['items'] = json_encode($purchase_data['items']);
        $data['ecomm_prodid'] = json_encode($purchase_data['ecomm_prodid']);
        $data['ecomm_totalvalue'] = $purchase_data['value'];

        $purchase_snippet = load::view('extension/advertise/google_dynamic_remarketing_purchase', $data);

        // Insert the snippet after the output
        $output = str_replace('</body>', googleshopping::getEventSnippet() . $purchase_snippet . '</body>', $output);
    }

    public static function google_dynamic_remarketing_home(&$route, &$data, &$output) {
        // In case the extension is disabled, do nothing
        if (!setting::get('advertise_google_status')) {
            return;
        }

        // If we are not on the home page, do nothing
        if (isset(request::$get['route']) && request::$get['route'] != config::get('action_default')) {
            return;
        }

        if (!class_exists('googleshopping')) {
            self::loadLibrary(self::$store_id);
        }

        if (null === googleshopping::getEventSnippetSendTo()) {
            return;
        }

        $data = array();
        $data['send_to'] = googleshopping::getEventSnippetSendTo();

        $snippet = load::view('extension/advertise/google_dynamic_remarketing_home', $data);

        // Insert the snippet after the output
        $output = str_replace('</body>', $snippet . '</body>', $output);
    }

    public static function google_dynamic_remarketing_searchresults(&$route, &$data, &$output) {
        // In case the extension is disabled, do nothing
        if (!setting::get('advertise_google_status')) {
            return;
        }

        // If we are not on the search page, do nothing
        if (!isset(request::$get['route']) || request::$get['route'] != 'product/search' || !isset(request::$get['search'])) {
            return;
        }

        if (!class_exists('googleshopping')) {
            self::loadLibrary(self::$store_id);
        }

        if (null === googleshopping::getEventSnippetSendTo()) {
            return;
        }

        $data = array();
        $data['send_to'] = googleshopping::getEventSnippetSendTo();
        $data['search_term'] = request::$get['search'];

        $snippet = load::view('extension/advertise/google_dynamic_remarketing_searchresults', $data);

        // Insert the snippet after the output
        $output = str_replace('</body>', $snippet . '</body>', $output);
    }

    public static function google_dynamic_remarketing_category(&$route, &$data, &$output) {
        // In case the extension is disabled, do nothing
        if (!setting::get('advertise_google_status')) {
            return;
        }

        // If we are not on the search page, do nothing
        if (!isset(request::$get['route']) || request::$get['route'] != 'product/category') {
            return;
        }

        if (!class_exists('googleshopping')) {
            self::loadLibrary(self::$store_id);
        }

        if (null === googleshopping::getEventSnippetSendTo()) {
            return;
        }

        if (isset(request::$get['path'])) {
            $parts = explode('_', request::$get['path']);
            $category_id = (int)end($parts);
        } else if (isset(request::$get['category_id'])) {
            $category_id = (int)request::$get['category_id'];
        } else {
            $category_id = 0;
        }


        $data = array();
        $data['send_to'] = googleshopping::getEventSnippetSendTo();
        $data['description'] = str_replace('"', '\\"', extension_advertise_google::getHumanReadableOpenCartCategory($category_id));

        $snippet = load::view('extension/advertise/google_dynamic_remarketing_category', $data);

        // Insert the snippet after the output
        $output = str_replace('</body>', $snippet . '</body>', $output);
    }

    public static function google_dynamic_remarketing_product(&$route, &$data, &$output) {
        // In case the extension is disabled, do nothing
        if (!setting::get('advertise_google_status')) {
            return;
        }

        // If we do not know the viewed product, do nothing
        if (!isset(request::$get['product_id']) || !isset(request::$get['route']) || request::$get['route'] != 'product/product') {
            return;
        }


        $product_info = catalog_product::getProduct((int)request::$get['product_id']);

        // If product does not exist, do nothing
        if (!$product_info) {
            return;
        }

        if (!class_exists('googleshopping')) {
            self::loadLibrary(self::$store_id);
        }

        if (null === googleshopping::getEventSnippetSendTo()) {
            return;
        }


        $category_name = extension_advertise_google::getHumanReadableCategory($product_info['product_id'], self::$store_id);

        $option_map = extension_advertise_google::getSizeAndColorOptionMap($product_info['product_id'], self::$store_id);

        $data = array();
        $data['send_to'] = googleshopping::getEventSnippetSendTo();
        $data['option_map'] = json_encode($option_map);
        $data['brand'] = $product_info['manufacturer'];
        $data['name'] = $product_info['name'];
        $data['category'] = str_replace('"', '\\"', $category_name);

        $snippet = load::view('extension/advertise/google_dynamic_remarketing_product', $data);

        // Insert the snippet after the output
        $output = str_replace('</body>', $snippet . '</body>', $output);
    }

    public static function google_dynamic_remarketing_cart(&$route, &$data, &$output) {
        // In case the extension is disabled, do nothing
        if (!setting::get('advertise_google_status')) {
            return;
        }

        // If we are not on the cart page, do nothing
        if (!isset(request::$get['route']) || request::$get['route'] != 'checkout/cart') {
            return;
        }

        if (!class_exists('googleshopping')) {
            self::loadLibrary(self::$store_id);
        }

        if (null === googleshopping::getEventSnippetSendTo()) {
            return;
        }


        $data = array();
        $data['send_to'] = googleshopping::getEventSnippetSendTo();
        $data['ecomm_totalvalue'] = cart::getTotal();
        $data['ecomm_prodid'] = json_encode(extension_advertise_google::getRemarketingProductIds(cart::getProducts(), self::$store_id));
        $data['items'] = json_encode(extension_advertise_google::getRemarketingItems(cart::getProducts(), self::$store_id));

        $snippet = load::view('extension/advertise/google_dynamic_remarketing_cart', $data);

        // Insert the snippet after the output
        $output = str_replace('</body>', $snippet . '</body>', $output);
    }

    public static function cron($cron_id = null, $code = null, $cycle = null, $date_added = null, $date_modified = null) {
        self::loadLibrary(self::$store_id);

        if (!self::validateCRON()) {
            // In case this is not a CRON task
            return;
        }

        load::language('extension/advertise/google');

        // Reset taxes to use the store address and zone
        tax::setShippingAddress(config('country_id'), config('zone_id'));
        tax::setPaymentAddress(config('country_id'), config('zone_id'));
        tax::setStoreAddress(config('country_id'), config('zone_id'));

        googleshopping::cron();
    }

    protected static function validateCRON() {
        if (!setting::get('advertise_google_status')) {
            // In case the extension is disabled, do nothing
            return false;
        }

        if (!setting::get('advertise_google_gmc_account_selected')) {
            return false;
        }

        if (!setting::get('advertise_google_gmc_shipping_taxes_configured')) {
            return false;
        }

        try {
            if (count(googleshopping::getTargets(self::$store_id)) === 0) {
                return false;
            }
        } catch (\RuntimeException $e) {
            return false;
        }

        if (isset(request::$get['cron_token']) && request::$get['cron_token'] == config::get('advertise_google_cron_token')) {
            return true;
        }

        if (defined('ADVERTISE_GOOGLE_ROUTE')) {
            return true;
        }

        return false;
    }
}