<?php
class ControllerExtensionCreditCardSagepayServer {
	public static function index() {
		if (!customer::isLogged()) {
			session::$data['redirect'] = url::link('account/account', '');

			response::redirect(url::link('account/login', ''));
		}

		load::language('extension/credit_card/sagepay_server');


		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_account'),
			'href' => url::link('account/account', '')
		);


		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];
			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(session::$data['error_warning'])) {
			$data['error_warning'] = session::$data['error_warning'];
			unset(session::$data['error_warning']);
		} else {
			$data['error_warning'] = '';
		}

		if (config::get('payment_sagepay_server_card')) {
			$data['cards'] = extension_payment_sagepay_server::getCards(customer::getId());
			$data['delete'] = url::link('extension/credit_card/sagepay_server/delete', 'card_id=');

			if (isset(request::$get['page'])) {
				$page = (int)request::$get['page'];
			} else {
				$page = 1;
			}

			$cards_total = count($data['cards']);

			$pagination = new Pagination();
			$pagination::$total = $cards_total;
			$pagination::$page = $page;
			$pagination::$limit = 10;
			$pagination::$url = url::link('extension/credit_card/sagepay_server', 'page={page}');

			$data['pagination'] = $pagination::render();

			$data['results'] = sprintf(language::get('text_pagination'), ($cards_total) ? (($page - 1) * 10) + 1 : 0, ((($page - 1) * 10) > ($cards_total - 10)) ? $cards_total : ((($page - 1) * 10) + 10), $cards_total, ceil($cards_total / 10));
		} else {
			$data['cards'] = false;
			$data['pagination'] = false;
			$data['results'] = false;
		}
		
		$data['add'] = url::link('extension/credit_card/sagepay_server/add', '');
		$data['back'] = url::link('account/account', '');
		
		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('extension/credit_card/sagepay_server_list', $data));
	}

	public static function delete() {
		load::language('extension/credit_card/sagepay_server');


		$card = extension_payment_sagepay_server::getCard(request::$get['card_id'], '');

		if (!empty($card['token'])) {
			if (config::get('payment_sagepay_server_test') == 'live') {
				$url = 'https://live.sagepay.com/gateway/service/removetoken.vsp';
			} else {
				$url = 'https://test.sagepay.com/gateway/service/removetoken.vsp';
			}
			
			$payment_data['VPSProtocol'] = '3.00';
			$payment_data['Vendor'] = config::get('payment_sagepay_server_vendor');
			$payment_data['TxType'] = 'REMOVETOKEN';
			$payment_data['Token'] = $card['token'];

			$response_data = extension_payment_sagepay_server::sendCurl($url, $payment_data);
			
			if ($response_data['Status'] == 'OK') {
				extension_payment_sagepay_server::deleteCard(request::$get['card_id']);
				session::$data['success'] = language::get('text_success_card');
			} else {
				session::$data['error_warning'] = language::get('text_fail_card');
			}
		} else {
			session::$data['error_warning'] = language::get('text_fail_card');
		}
		response::redirect(url::link('extension/credit_card/sagepay_server', ''));
	}

	public static function addCard() {
		load::language('extension/payment/sagepay_server');
		

		$payment_data = array();

		if (config::get('payment_sagepay_server_test') == 'live') {
			$url = 'https://live.sagepay.com/gateway/service/token.vsp';
		} else {
			$url = 'https://test.sagepay.com/gateway/service/token.vsp';
		}
		$payment_data['VPSProtocol'] = '3.00';

		$payment_data['ReferrerID'] = 'E511AF91-E4A0-42DE-80B0-09C981A3FB61';
		$payment_data['TxType'] = 'TOKEN';
		$payment_data['Vendor'] = config::get('payment_sagepay_server_vendor');
		$payment_data['VendorTxCode'] = 'server_card_' . date("YmdHis") . mt_rand(1, 999);
		$payment_data['NotificationURL'] = url::link('extension/credit_card/sagepay_server/callback', '');
		$payment_data['Currency'] = session::$data['currency'];

		$response_data = extension_payment_sagepay_server::sendCurl($url, $payment_data);

		extension_payment_sagepay_server::logger('Response', $response_data);

		if ($response_data['Status'] == 'OK') {
			$json['redirect'] = $response_data['NextURL'];
			$json['Status'] = $response_data['Status'];
			$json['StatusDetail'] = $response_data['StatusDetail'];
			
			$order_info['order_id'] = -1;
			$order_info['VPSTxId'] = substr($response_data['VPSTxId'], 1, -1);
			$order_info['SecurityKey'] = $response_data['SecurityKey'];
			$order_info['VendorTxCode'] = $payment_data['VendorTxCode'];
			$order_info['currency_code'] = session::$data['currency'];
			$order_info['total'] = '';
			extension_payment_sagepay_server::addOrder($order_info);
		} else {
			$json['error'] = $response_data['StatusDetail'];
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function callback() {

		extension_payment_sagepay_server::logger('Callback data', request::$post);

		$success_page = url::link('extension/credit_card/sagepay_server/success', '');
		$error_page = url::link('extension/credit_card/sagepay_server/failure', '');
		$end_ln = chr(13) . chr(10);

		if (isset(request::$post['VendorTxCode'])) {
			$vendor_tx_code = request::$post['VendorTxCode'];
		} else {
			$vendor_tx_code = '';
		}

		if (isset(request::$post['Status'])) {
			$str_status = request::$post['Status'];
		} else {
			$str_status = '';
		}

		if (isset(request::$post['VPSSignature'])) {
			$str_vps_signature = request::$post['VPSSignature'];
		} else {
			$str_vps_signature = '';
		}

		if (isset(request::$post['VPSTxId'])) {
			$str_vps_tx_id = request::$post['VPSTxId'];
		} else {
			$str_vps_tx_id = '';
		}

		if (isset(request::$post['ExpiryDate'])) {
			$str_expiry_date = request::$post['ExpiryDate'];
		} else {
			$str_expiry_date = '';
		}

		if (isset(request::$post['Token'])) {
			$str_token = request::$post['Token'];
		} else {
			$str_token = '';
		}

		$transaction_info = extension_payment_sagepay_server::getOrder('', $str_vps_tx_id);

		if (isset($transaction_info['SecurityKey'])) {
			$str_security_key = $transaction_info['SecurityKey'];
		} else {
			$str_security_key = '';
		}
		extension_payment_sagepay_server::logger('$transaction_info', $transaction_info);
		extension_payment_sagepay_server::logger('$str_vps_tx_id', $str_vps_tx_id);
		extension_payment_sagepay_server::logger('$vendor_tx_code', $vendor_tx_code);
		extension_payment_sagepay_server::logger('$str_status', $str_status);
		extension_payment_sagepay_server::logger('payment_sagepay_server_vendor', config::get('payment_sagepay_server_vendor'));
		extension_payment_sagepay_server::logger('$str_token', $str_token);
		extension_payment_sagepay_server::logger('$str_security_key', $str_security_key);

		$str_message = $str_vps_tx_id . $vendor_tx_code . $str_status . strtolower(config::get('payment_sagepay_server_vendor')) . $str_token . $str_security_key;

		$str_my_signature = strtoupper(md5($str_message));

		/** We can now compare our MD5 Hash signature with that from Sage Pay Server * */
		if ($str_my_signature != $str_vps_signature) {

			echo "Status=INVALID" . $end_ln;
			echo "StatusDetail= Cannot match the MD5 Hash. Order might be tampered with." . $end_ln;
			echo "RedirectURL=" . $error_page . $end_ln;
			extension_payment_sagepay_server::logger('StatusDetail', 'Cannot match the MD5 Hash. Order might be tampered with.');
			exit;
		}

		if ($str_status != "OK") {
			echo "Status=INVALID" . $end_ln;
			echo "StatusDetail= Either status invalid or order info was not found.";
			echo "RedirectURL=" . $error_page . $end_ln;

			extension_payment_sagepay_server::logger('StatusDetail', 'Either status invalid or order info was not found.');

			exit;
		}

		$card_data['customer_id'] = $transaction_info['customer_id'];
		$card_data['Token'] = request::$post['Token'];
		$card_data['Last4Digits'] = request::$post['Last4Digits'];
		$card_data['ExpiryDate'] = substr_replace(request::$post['ExpiryDate'], '/', 2, 0);
		$card_data['CardType'] = request::$post['CardType'];
		extension_payment_sagepay_server::addCard($card_data);

		echo "Status=OK" . $end_ln;
		echo "RedirectURL=" . $success_page . $end_ln;
	}

	public static function success() {
		extension_payment_sagepay_server::logger('Success', '');
		session::$data['success'] = 'Success';
		response::redirect(url::link('extension/credit_card/sagepay_server', ''));
	}

	public static function failure() {
		extension_payment_sagepay_server::logger('Failure', '');
		session::$data['error_warning'] = 'Failure';
		response::redirect(url::link('extension/credit_card/sagepay_server', ''));
	}
}