<?php

class ControllerExtensionCreditCardSquareup {
    public static function index() {
        if (!customer::isLogged()) {
            session::$data['redirect'] = url::link('account/account', '');

            response::redirect(url::link('account/login', ''));
        }

        load::language('extension/credit_card/squareup');


        document::setTitle(language::get('heading_title'));

        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'text' => language::get('text_home'),
            'href' => url::link('common/home')
        );

        $data['breadcrumbs'][] = array(
            'text' => language::get('text_account'),
            'href' => url::link('account/account', '')
        );

        $data['breadcrumbs'][] = array(
            'text' => language::get('heading_title'),
            'href' => url::link('extension/credit_card/squareup', '')
        );

        if (isset(session::$data['success'])) {
            $data['success'] = session::$data['success'];

            unset(session::$data['success']);
        } else {
            $data['success'] = '';
        } 

        if (isset(session::$data['error'])) {
            $data['error'] = session::$data['error'];

            unset(session::$data['error']);
        } else {
            $data['error'] = '';
        } 

        $data['back'] = url::link('account/account', '');

        $data['cards'] = array();

        foreach (extension_credit_card_squareup::getCards(customer::getId(), config::get('payment_squareup_enable_sandbox')) as $card) {
            $data['cards'][] = array(
                'text' => sprintf(language::get('text_card_ends_in'), $card['brand'], $card['ends_in']),
                'delete' => url::link('extension/credit_card/squareup/forget', 'squareup_token_id=' . $card['squareup_token_id'])
            );
        }

        $data['column_left'] = load::controller('common/column_left');
        $data['column_right'] = load::controller('common/column_right');
        $data['content_top'] = load::controller('common/content_top');
        $data['content_bottom'] = load::controller('common/content_bottom');
        $data['footer'] = load::controller('common/footer');
        $data['header'] = load::controller('common/header');
        
        response::setOutput(load::view('extension/credit_card/squareup', $data));
    }

    public static function forget() {
        if (!customer::isLogged()) {
            session::$data['redirect'] = url::link('account/account', '');

            response::redirect(url::link('account/login', ''));
        }

        load::language('extension/credit_card/squareup');


        load::library('squareup');

        $squareup_token_id = !empty(request::$get['squareup_token_id']) ?
            request::$get['squareup_token_id'] : 0;

        if (extension_credit_card_squareup::verifyCardCustomer($squareup_token_id, customer::getId())) {
            $card_info = extension_credit_card_squareup::getCard($squareup_token_id);

            $customer_info = extension_credit_card_squareup::getCustomer(customer::getId(), $card_info['sandbox']);
            
            try {
                self::$squareup::deleteCard($customer_info['square_customer_id'], $card_info['token']);
                
                extension_credit_card_squareup::deleteCard($squareup_token_id);
                
                session::$data['success'] = language::get('text_success_card_delete');
            } catch (\Squareup\Exception $e) {
                session::$data['error'] = $e::getMessage();
            }
        }

        response::redirect(url::link('extension/credit_card/squareup', ''));
    }
}