<?php

class ControllerExtensionModuleAmazonLogin {
    public static function index() {
        if (config::get('payment_amazon_login_pay_status') && config::get('module_amazon_login_status') && !customer::isLogged() && !empty(request::$server['HTTPS'])) {


            // capital L in Amazon cookie name is required, do not alter for coding standards
            if (isset(request::$cookie['amazon_Login_state_cache'])) {
                setcookie('amazon_Login_state_cache', null, -1, '/');
            }

            $amazon_payment_js = extension_payment_amazon_login_pay::getWidgetJs();

            document::addScript($amazon_payment_js);

            $data['client_id'] = trim(config::get('payment_amazon_login_pay_client_id'));
            $data['merchant_id'] = config::get('payment_amazon_login_pay_merchant_id');
            $data['return_url'] = html_entity_decode(url::link('extension/module/amazon_login/login', ''), ENT_COMPAT, "UTF-8");
            $data['button_id'] = 'AmazonLoginButton';

            if (config::get('payment_amazon_login_pay_test') == 'sandbox') {
                $data['sandbox'] = isset(session::$data['user_id']); // Require an active admin panel session to show debug messages
            }

            if (config::get('module_amazon_login_button_type')) {
                $data['button_type'] = config::get('module_amazon_login_button_type');
            } else {
                $data['button_type'] = 'lwa';
            }

            if (config::get('module_amazon_login_button_colour')) {
                $data['button_colour'] = config::get('module_amazon_login_button_colour');
            } else {
                $data['button_colour'] = 'gold';
            }

            if (config::get('module_amazon_login_button_size')) {
                $data['button_size'] = config::get('module_amazon_login_button_size');
            } else {
                $data['button_size'] = 'medium';
            }

            if(!empty(session::$data['language'])) {
              $session_lang = session::$data['language'];
              $session_lang_code = current(explode('-', $session_lang));
              $language_region_mapping = array(
                'EUR' => array('de-De', 'es-ES','fr-FR', 'it-IT', 'en-GB'),
                'GBP' => array('de-De', 'es-ES','fr-FR', 'it-IT', 'en-GB'),
                'USD' =>array('en-US')
              );

              if(config::get('payment_amazon_login_pay_payment_region')) {
                $merchant_location = config::get('payment_amazon_login_pay_payment_region');
                $available_codes = $language_region_mapping[$merchant_location];
                $data['language'] = (config::get('payment_amazon_login_pay_language')) ? config::get('payment_amazon_login_pay_language') : 'en-US';
                foreach ($available_codes as $l_code) {
                  $l_code_short = current(explode('-', $l_code));
                  if($session_lang_code == $l_code_short) {
                    $data['language'] = $l_code;
                  }
                }
              }
            } else {
              $data['language'] = (config::get('payment_amazon_login_pay_language')) ? config::get('payment_amazon_login_pay_language') : 'en-US';
            }

            return load::view('extension/module/amazon_login', $data);
        }
    }

    public static function login() {
        load::language('extension/payment/amazon_login_pay');
        load::language('account/login');


        $from_amazon_pay = isset(request::$get['from_amazon_pay']);

        unset(session::$data['apalwa']);

        try {
            if (isset(request::$get['access_token'])) {
                $access_token = request::$get['access_token'];

                session::$data['apalwa']['login']['access_token'] = $access_token;

                extension_module_amazon_login::verifyAccessToken($access_token);

                $amazon_profile = extension_module_amazon_login::fetchProfile($access_token);
            } else {
                extension_module_amazon_login::debugLog("EXCEPTION", language::get('error_login'));

                throw new \RuntimeException(language::get('error_login'));
            }

            // No issues found, and the Amazon profile has been fetched
            if ($from_amazon_pay) {
                unset(session::$data['guest']);
                unset(session::$data['account']);

                if (config::get('payment_amazon_login_pay_checkout') == 'guest') {
                    session::$data['account'] = 'guest';

                    session::$data['guest']['customer_group_id'] = config('customer_group_id');
                    session::$data['guest']['firstname'] = (string)$amazon_profile::$first_name;
                    session::$data['guest']['lastname'] = (string)$amazon_profile::$last_name;
                    session::$data['guest']['email'] = (string)$amazon_profile::$email;
                    session::$data['guest']['telephone'] = '0000000';

                    session::$data['apalwa']['pay']['profile'] = session::$data['guest'];
                } else {
                    // The payment button must log in a customer
                    session::$data['account'] = 'register';

                    $profile = extension_module_amazon_login::loginProfile($amazon_profile);

                    session::$data['apalwa']['pay']['profile'] = $profile;

                    // If a customer is already logged in, we must overwrite their data with amazon data
                    // Therefore, at the end of the day, we will have a 
                }

                response::redirect(url::link('extension/payment/amazon_login_pay/address', ''));
            } else {
                extension_module_amazon_login::loginProfile($amazon_profile);

                response::redirect(url::link('account/account', ''));
            }
        } catch (\RuntimeException $e) {
            session::$data['apalwa']['login']['error'] = $e::getMessage();

            response::redirect(url::link('extension/module/amazon_login/error', ''));
        }
    }

    public static function error() {
        load::language('extension/payment/amazon_login_pay');

        $data = array();

        $continue = url::link('common/home', '');

        if (isset(session::$data['apalwa']['login']['error'])) {
            $data['error'] = session::$data['apalwa']['login']['error'];
            $data['continue'] = $continue;
            $data['heading_title'] = language::get('error_login');

            document::setTitle(language::get('error_login'));

            unset(session::$data['apalwa']);

            $data['breadcrumbs'] = array();

            $data['breadcrumbs'][] = array(
                'href' => url::link('common/home', ''),
                'text' => language::get('text_home')
            );

            $data['breadcrumbs'][] = array(
                'href' => null,
                'current' => true,
                'text' => language::get('error_login')
            );

            $data['content_main'] = load::view('extension/module/amazon_login_error', $data);
            $data['column_left'] = load::controller('common/column_left');
            $data['column_right'] = load::controller('common/column_right');
            $data['content_top'] = load::controller('common/content_top');
            $data['content_bottom'] = load::controller('common/content_bottom');
            $data['footer'] = load::controller('common/footer');
            $data['header'] = load::controller('common/header');

            response::setOutput(load::view('extension/payment/amazon_login_pay_generic', $data));
        } else {
            response::redirect($continue);
        }
    }

    public static function logout() {
        unset(session::$data['apalwa']);

        // capital L in Amazon cookie name is required, do not alter for coding standards
        if (isset(request::$cookie['amazon_Login_state_cache'])) {
            //@todo - rework this by triggering the JavaScript logout
            setcookie('amazon_Login_state_cache', null, -1, '/');
        }
    }
}
