<?php

class ControllerExtensionPaymentAmazonLoginPay {
    private static $version = "3.2.1";
    public static function session_expired() {
        load::language('extension/payment/amazon_login_pay');


        extension_payment_amazon_login_pay::cartRedirect(language::get('error_session_expired'));
    }

    public static function address() {
        load::language('extension/payment/amazon_login_pay');


        document::setTitle(language::get('heading_address'));

        // Verify cart
        extension_payment_amazon_login_pay::verifyCart();

        // Verify login
        extension_payment_amazon_login_pay::verifyLogin();

        // Verify cart total
        //extension_payment_amazon_login_pay::verifyTotal();

        // Cancel an existing order reference
        unset(session::$data['order_id']);
        
        if (!empty(session::$data['apalwa']['pay']['order_reference_id']) && !extension_payment_amazon_login_pay::isOrderInState(session::$data['apalwa']['pay']['order_reference_id'], array('Canceled', 'Closed', 'Draft'))) {
            extension_payment_amazon_login_pay::cancelOrder(session::$data['apalwa']['pay']['order_reference_id'], "Shipment widget has been requested, cancelling this order reference.");

            unset(session::$data['apalwa']['pay']['order_reference_id']);
        }

        $data['text_cart'] = language::get('text_cart');

        $data['shipping_methods'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/shipping_methods', ''), ENT_COMPAT, "UTF-8");
        $data['shipping'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/shipping', ''), ENT_COMPAT, "UTF-8");
        $data['cart'] = html_entity_decode(url::link('checkout/cart'), ENT_COMPAT, "UTF-8");
        $data['session_expired'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/session_expired'), ENT_COMPAT, "UTF-8");

        $data['client_id'] = config::get('payment_amazon_login_pay_client_id');
        $data['merchant_id'] = config::get('payment_amazon_login_pay_merchant_id');

        if (config::get('payment_amazon_login_pay_test') == 'sandbox') {
            $data['sandbox'] = isset(session::$data['user_id']); // Require an active admin panel session to show debug messages
        }

        $amazon_payment_js = extension_payment_amazon_login_pay::getWidgetJs();
        document::addScript($amazon_payment_js);

        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'href' => url::link('common/home', ''),
            'text' => language::get('text_home')
        );

        $data['breadcrumbs'][] = array(
            'href' => url::link('checkout/cart'),
            'text' => language::get('breadcrumb_cart')
        );

        $data['breadcrumbs'][] = array(
            'href' => url::link('extension/payment/amazon_login_pay/address'),
            'current' => true,
            'text' => language::get('breadcrumb_shipping')
        );

        $data['breadcrumbs'][] = array(
            'href' => null,
            'text' => language::get('breadcrumb_payment')
        );

        $data['breadcrumbs'][] = array(
            'href' => null,
            'text' => language::get('breadcrumb_summary')
        );

        $data['content_main'] = load::view('extension/payment/amazon_login_pay_address', $data);
        $data['column_left'] = load::controller('common/column_left');
        $data['column_right'] = load::controller('common/column_right');
        $data['content_top'] = load::controller('common/content_top');
        $data['content_bottom'] = load::controller('common/content_bottom');
        $data['footer'] = load::controller('common/footer');
        $data['header'] = load::controller('common/header');

        response::setOutput(load::view('extension/payment/amazon_login_pay_generic', $data));
    }

    public static function shipping_methods() {
        load::language('extension/payment/amazon_login_pay');

        $json = array();

        try {

            if (!isset(request::$get['AmazonOrderReferenceId'])) {
                throw extension_payment_amazon_login_pay::loggedException(language::get('error_shipping_methods'), language::get('error_shipping_methods'));
            }

            $order_reference_id = request::$get['AmazonOrderReferenceId'];

            session::$data['apalwa']['pay']['order_reference_id'] = $order_reference_id;

            $address = extension_payment_amazon_login_pay::getAddress($order_reference_id);

            $quotes = array();

            $results = setting_extension::getExtensions('shipping');

            foreach ($results as $result) {
                if (isset($result['code'])) {
                    $code = $result['code'];
                } else {
                    $code = $result['key'];
                }

                if (config::get('shipping_' . $code . '_status')) {

                    $quote = call_model_method('extension_shipping_' . $code, 'getQuote', $address);

                    if ($quote && empty($quote['error'])) {
                        $quotes[$code] = array(
                            'title' => $quote['title'],
                            'quote' => $quote['quote'],
                            'sort_order' => $quote['sort_order'],
                            'error' => $quote['error']
                        );
                    }
                }
            }

            if (empty($quotes)) {
                throw new \RuntimeException(language::get('error_no_shipping_methods'));
            }

            $sort_order = array();

            foreach ($quotes as $key => $value) {
                $sort_order[$key] = $value['sort_order'];
            }

            array_multisort($sort_order, SORT_ASC, $quotes);

            session::$data['apalwa']['pay']['shipping_methods'] = $quotes;
            session::$data['apalwa']['pay']['address'] = $address;

            $json['quotes'] = $quotes;

            if (!empty(session::$data['apalwa']['pay']['shipping_method']['code'])) {
                $json['selected'] = session::$data['apalwa']['pay']['shipping_method']['code'];
            } else {
                $json['selected'] = '';
            }
        } catch (\RuntimeException $e) {
            $json['error'] = $e::getMessage();
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function shipping() {
        load::language('extension/payment/amazon_login_pay');


        $json = array(
            'redirect' => null,
            'error' => null
        );

        try {
            if (!isset(request::$post['shipping_method'])) {
                throw extension_payment_amazon_login_pay::loggedException("No shipping method provided.", language::get('error_process_order'));
            }

            $shipping_method = explode('.', request::$post['shipping_method']);

            if (!isset($shipping_method[0]) || !isset($shipping_method[1]) || !isset(session::$data['apalwa']['pay']['shipping_methods'][$shipping_method[0]]['quote'][$shipping_method[1]])) {

                throw extension_payment_amazon_login_pay::loggedException("Used shipping method is not allowed.", language::get('error_process_order'));
            }

            session::$data['apalwa']['pay']['shipping_method'] = session::$data['apalwa']['pay']['shipping_methods'][$shipping_method[0]]['quote'][$shipping_method[1]];
            session::$data['shipping_method'] = session::$data['apalwa']['pay']['shipping_method'];
            session::$data['payment_address'] = session::$data['apalwa']['pay']['address'];
            session::$data['shipping_address'] = session::$data['apalwa']['pay']['address'];
            session::$data['shipping_country_id'] = session::$data['apalwa']['pay']['address']['country_id'];
            session::$data['shipping_zone_id'] = session::$data['apalwa']['pay']['address']['zone_id'];

            extension_module_amazon_login::persistAddress(session::$data['apalwa']['pay']['address']);

            $json['redirect'] = url::link('extension/payment/amazon_login_pay/payment', '');
        } catch (\RuntimeException $e) {
            $json['error'] = $e::getMessage();
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function payment() {
        load::language('extension/payment/amazon_login_pay');


        document::setTitle(language::get('heading_payment'));

        // Verify cart
        extension_payment_amazon_login_pay::verifyCart();

        // Verify login
        extension_payment_amazon_login_pay::verifyLogin();

        // Verify total
        extension_payment_amazon_login_pay::verifyTotal();

        // Verify shipping
        extension_payment_amazon_login_pay::verifyShipping();

        $data['confirm'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/confirm', ''), ENT_COMPAT, "UTF-8");
        $data['back'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/address', ''), ENT_COMPAT, "UTF-8");
        $data['session_expired'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/session_expired'), ENT_COMPAT, "UTF-8");

        $data['merchant_id'] = config::get('payment_amazon_login_pay_merchant_id');
        $data['client_id'] = config::get('payment_amazon_login_pay_client_id');

        $data['order_reference_id'] = !empty(session::$data['apalwa']['pay']['order_reference_id']) ? session::$data['apalwa']['pay']['order_reference_id'] : null;

        //detect the buyer multi-currency
        $amazon_supported_currencies = array('AUD', 'GBP','DKK', 'EUR', 'HKD', 'JPY', 'NZD','NOK', 'ZAR', 'SEK', 'CHF', 'USD');

        $data['enabled_buyers_multi_currency'] = false;
        $data['buyer_currency'] = false;

        if (config::get('payment_amazon_login_pay_buyer_multi_currency') && config::get('payment_amazon_login_pay_payment_region') != 'USD') {
            $session_currency = !empty(session::$data['currency']) ? session::$data['currency'] : config('currency');

            if (in_array($session_currency, $amazon_supported_currencies)) {
                $data['buyer_currency'] = $session_currency;
                session::$data['apalwa']['pay']['buyer_currency'] = $session_currency;
                $data['enabled_buyers_multi_currency'] = true;
            }
        }

        if (config::get('payment_amazon_login_pay_test') == 'sandbox') {
            $data['sandbox'] = isset(session::$data['user_id']); // Require an active admin panel session to show debug messages
        }

        $data['error'] = '';
        if (isset(session::$data['apalwa']['error'])) {
            $data['error'] = session::$data['apalwa']['error'];
            unset(session::$data['apalwa']['error']);
        }

        $amazon_payment_js = extension_payment_amazon_login_pay::getWidgetJs();
        document::addScript($amazon_payment_js);

        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'href' => url::link('common/home', ''),
            'text' => language::get('text_home')
        );

        $data['breadcrumbs'][] = array(
            'href' => url::link('checkout/cart'),
            'text' => language::get('breadcrumb_cart')
        );

        $data['breadcrumbs'][] = array(
            'href' => url::link('extension/payment/amazon_login_pay/address'),
            'text' => language::get('breadcrumb_shipping')
        );

        $data['breadcrumbs'][] = array(
            'href' => url::link('extension/payment/amazon_login_pay/payment'),
            'current' => true,
            'text' => language::get('breadcrumb_payment')
        );

        $data['breadcrumbs'][] = array(
            'href' => null,
            'text' => language::get('breadcrumb_summary')
        );

        $data['content_main'] = load::view('extension/payment/amazon_login_pay_payment', $data);
        $data['column_left'] = load::controller('common/column_left');
        $data['column_right'] = load::controller('common/column_right');
        $data['content_top'] = load::controller('common/content_top');
        $data['content_bottom'] = load::controller('common/content_bottom');
        $data['footer'] = load::controller('common/footer');
        $data['header'] = load::controller('common/header');

        response::setOutput(load::view('extension/payment/amazon_login_pay_generic', $data));
    }

    public static function persist_comment() {
        if (isset(request::$post['comment'])) {
            session::$data['comment'] = strip_tags(request::$post['comment']);

            session::$data['apalwa']['pay']['order']['comment'] = session::$data['comment'];
        }
    }

    public static function coupon_discard() {

        // Verify reference
        extension_payment_amazon_login_pay::verifyReference();

        if (extension_payment_amazon_login_pay::isOrderInState(session::$data['apalwa']['pay']['order_reference_id'], array('Draft'))) {
            unset(session::$data['coupon']);
        }

        response::redirect(url::link('extension/payment/amazon_login_pay/confirm', ''));
    }

    public static function standard_checkout() {
        load::language('extension/payment/amazon_login_pay');


        // Verify cart
        extension_payment_amazon_login_pay::verifyCart();

        // Verify login
        extension_payment_amazon_login_pay::verifyLogin();

        // Cancel an existing order reference
        if (!empty(session::$data['apalwa']['pay']['order_reference_id']) && extension_payment_amazon_login_pay::isOrderInState(session::$data['apalwa']['pay']['order_reference_id'], array('Open'))) {
            extension_payment_amazon_login_pay::cancelOrder(session::$data['apalwa']['pay']['order_reference_id'], "Shipment widget has been requested, cancelling this order reference.");
        }

        // Unset all payment data
        unset(session::$data['apalwa']['pay']);
        unset(session::$data['order_id']);

        // Redirect to the cart
        response::redirect(url::link('checkout/cart', ''));
    }

    public static function confirm() {
        load::language('extension/payment/amazon_login_pay');
        load::language('checkout/checkout');
        

        document::setTitle(language::get('heading_confirm'));

        // Verify cart
        extension_payment_amazon_login_pay::verifyCart();

        // Verify login
        extension_payment_amazon_login_pay::verifyLogin();

        // Verify cart total
        // Not needed, as we will display an error message later on...

        // Verify reference
        extension_payment_amazon_login_pay::verifyReference();

        // Verify shipping
        extension_payment_amazon_login_pay::verifyShipping();

        $data['merchant_id'] = config::get('payment_amazon_login_pay_merchant_id');
        $data['client_id'] = config::get('payment_amazon_login_pay_client_id');
        
        if (config::get('payment_amazon_login_pay_test') == 'sandbox') {
            $data['sandbox'] = isset(session::$data['user_id']); // Require an active admin panel session to show debug messages
        }

        $amazon_payment_js = extension_payment_amazon_login_pay::getWidgetJs();
        document::addScript($amazon_payment_js);

        try {
            $order = extension_payment_amazon_login_pay::makeOrder();

            session::$data['apalwa']['pay']['order'] = $order;

            $data['order_reference_id'] = session::$data['apalwa']['pay']['order_reference_id'];
            $data['order'] = $order;
        } catch (\RuntimeException $e) {
            extension_payment_amazon_login_pay::cartRedirect($e::getMessage());
        }

        $data['success'] = '';

        if (!empty(session::$data['success'])) {
            $data['success'] = session::$data['success'];
            unset(session::$data['success']);
        }

        if (isset(session::$data['coupon'])) {
            $data['coupon'] = session::$data['coupon'];
        } else {
            $data['coupon'] = '';
        }

        if (isset(session::$data['comment'])) {
            $data['comment'] = session::$data['comment'];
        } else {
            $data['comment'] = '';
        }

        $data['is_order_total_positive'] = extension_payment_amazon_login_pay::isTotalPositive();
        $data['standard_checkout'] = url::link('extension/payment/amazon_login_pay/standard_checkout', '');

        $zero_total = currency::format(0, session::$data['currency']);
        $data['error_order_total_zero'] = sprintf(language::get('error_order_total_zero'), $zero_total);

        $data['process'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/process', ''), ENT_COMPAT, "UTF-8");
        $data['process_us'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/process_us', ''), ENT_COMPAT, "UTF-8");
        $data['address'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/address', ''), ENT_COMPAT, "UTF-8");
        $data['back'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/payment', ''), ENT_COMPAT, "UTF-8");
        $data['session_expired'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/session_expired'), ENT_COMPAT, "UTF-8");
        $data['coupon_discard'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/coupon_discard', ''), ENT_COMPAT, "UTF-8");
        $data['coupon_apply'] = html_entity_decode(url::link('extension/total/coupon/coupon', ''), ENT_COMPAT, "UTF-8");
        $data['persist_comment'] = html_entity_decode(url::link('extension/payment/amazon_login_pay/persist_comment', ''), ENT_COMPAT, "UTF-8");
        $data['is_coupon_change_allowed'] = extension_payment_amazon_login_pay::isOrderInState(session::$data['apalwa']['pay']['order_reference_id'], array('Draft'));
        $data['error_unexpected_network_error'] = language::get('error_unexpected_network_error');
        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'href' => url::link('common/home', ''),
            'text' => language::get('text_home')
        );

        $data['breadcrumbs'][] = array(
            'href' => url::link('checkout/cart'),
            'text' => language::get('breadcrumb_cart')
        );

        $data['breadcrumbs'][] = array(
            'href' => url::link('extension/payment/amazon_login_pay/address'),
            'text' => language::get('breadcrumb_shipping')
        );

        $data['breadcrumbs'][] = array(
            'href' => url::link('extension/payment/amazon_login_pay/payment'),
            'text' => language::get('breadcrumb_payment')
        );

        $data['breadcrumbs'][] = array(
            'href' => url::link('extension/payment/amazon_login_pay/confirm'),
            'current' => true,
            'text' => language::get('breadcrumb_summary')
        );

        //enable mfa only for UK and Europe regions
        $data['psd_enabled'] = "false";
        if(config::get('payment_amazon_login_pay_payment_region') != 'USD') {
            $data['psd_enabled'] = "true";
        }
        //detect the buyer multi-currency
        $amazon_supported_currencies = array('AUD', 'GBP','DKK', 'EUR', 'HKD', 'JPY', 'NZD','NOK', 'ZAR', 'SEK', 'CHF', 'USD');

        $data['enabled_buyers_multi_currency'] = false;
        $data['buyer_currency'] = false;

        if (config::get('payment_amazon_login_pay_buyer_multi_currency') && config::get('payment_amazon_login_pay_payment_region') != 'USD') {
            $session_currency = !empty(session::$data['currency']) ? session::$data['currency'] : config('currency');

            if (in_array($session_currency,$amazon_supported_currencies)) {
                $data['buyer_currency'] = $session_currency;
                session::$data['apalwa']['pay']['buyer_currency'] = $session_currency;
                $data['enabled_buyers_multi_currency'] = true;
            }
        }

        if (!$data['buyer_currency']) {
            $location_currency = config::get('payment_amazon_login_pay_payment_region');
            $rate = round(currency::getValue($location_currency) / currency::getValue($order['currency_code']), 8);
            $amount = currency::format(currency::convert($order['total'], config('currency'), $location_currency), $location_currency, 1, true);

            $data['is_amount_converted'] = $order['currency_code'] != $location_currency;
            $data['text_amount_converted'] = sprintf(language::get('text_amount_converted'), $location_currency, $rate, $amount);
        }

        $data['content_main'] = load::view('extension/payment/amazon_login_pay_confirm', $data);
        $data['column_left'] = load::controller('common/column_left');
        $data['column_right'] = load::controller('common/column_right');
        $data['content_top'] = load::controller('common/content_top');
        $data['content_bottom'] = load::controller('common/content_bottom');
        $data['footer'] = load::controller('common/footer');
        $data['header'] = load::controller('common/header');

        response::setOutput(load::view('extension/payment/amazon_login_pay_generic', $data));
    }
    //handle the SuccessUrl response
    public static function mfa_success() {
        // Verify cart
        extension_payment_amazon_login_pay::verifyCart();
        // Verify login
        extension_payment_amazon_login_pay::verifyLogin();
        // Verify reference
        extension_payment_amazon_login_pay::verifyReference();
        // Verify shipping
        extension_payment_amazon_login_pay::verifyShipping();
        // Verify order
        extension_payment_amazon_login_pay::verifyOrder();
        load::language('extension/payment/amazon_login_pay');

        if (isset(request::$get['AuthenticationStatus']) && request::$get['AuthenticationStatus'] == 'Success') {
            self::authorize();
        } else {
            extension_payment_amazon_login_pay::cartRedirect(language::get('error_invaild_request'));
        }
    }
    //handle the FailureUrl response
    public static function mfa_failure() {
        load::language('extension/payment/amazon_login_pay');

        if(isset(request::$get['AuthenticationStatus'])) {
            $mfa_authorization_status = request::$get['AuthenticationStatus'];

            if($mfa_authorization_status == 'Failure') {
                $text_failed_mfa = language::get('error_failure_mfa');
                extension_payment_amazon_login_pay::cartRedirect($text_failed_mfa);
            } elseif ($mfa_authorization_status == 'Abandoned') {
                session::$data['apalwa']['error'] = language::get('error_abandoned_mfa');
                response::redirect(url::link('extension/payment/amazon_login_pay/payment', ''));
            }
        } else {
            $text_invaild_request = language::get('error_invaild_request');
            extension_payment_amazon_login_pay::cartRedirect($text_invaild_request);
        }
    }


    private static function authorize() {
          load::language('extension/payment/amazon_login_pay');
          load::language('checkout/checkout');


          // Verify cart
          extension_payment_amazon_login_pay::verifyCart();

          // Verify reference
          extension_payment_amazon_login_pay::verifyReference();

          // Verify shipping
          extension_payment_amazon_login_pay::verifyShipping();

          // Verify login
          extension_payment_amazon_login_pay::verifyLogin();

          // Verify order
          extension_payment_amazon_login_pay::verifyOrder();
          try {
            $order_reference_id = session::$data['apalwa']['pay']['order_reference_id'];
            if (empty(session::$data['order_id'])) {
                // Up to this point, everything is fine in the session. Save the order and submit it to Amazon.
                $order_id = checkout_order::addOrder(session::$data['apalwa']['pay']['order']);

                session::$data['order_id'] = $order_id;

                $currency_code = isset(session::$data['apalwa']['pay']['buyer_currency']) ? session::$data['apalwa']['pay']['buyer_currency'] : config::get('payment_amazon_login_pay_payment_region');

                $text_version = sprintf(language::get('text_created_by'), self::$version);

                extension_payment_amazon_login_pay::submitOrderDetails($order_reference_id, $order_id, $currency_code, $text_version);

            } else {
                $order_id = session::$data['order_id'];
            }
            $amazon_order = extension_payment_amazon_login_pay::fetchOrder($order_reference_id);

            // The order has been opened for authorization. Store it in the database
            $amazon_login_pay_order_id = extension_payment_amazon_login_pay::findOrAddOrder($amazon_order);

            // Authorize the order
            $authorization = extension_payment_amazon_login_pay::authorizeOrder($amazon_order);

            // Log the authorization
            extension_payment_amazon_login_pay::addAuthorization($amazon_login_pay_order_id, $authorization);

            if ($authorization::$AuthorizationStatus::$State == 'Declined') {
                $reason_code = (string)$authorization::$AuthorizationStatus::$ReasonCode;

                switch ($reason_code) {
                    case 'InvalidPaymentMethod' :
                        session::$data['apalwa']['error'] = language::get('error_decline_invalid_payment_method');

                        response::redirect(url::link('extension/payment/amazon_login_pay/payment', ''));
                    break;
                    default : 
                        if (extension_payment_amazon_login_pay::isOrderInState($order_reference_id, array('Open'))) {
                            extension_payment_amazon_login_pay::cancelOrder($order_reference_id, "Authorization has failed with the state: " . $authorization::$AuthorizationStatus::$State);
                        }

                        $cart_error_messages = array(
                            'TransactionTimedOut' => language::get('error_decline_transaction_timed_out'),
                            'AmazonRejected' => language::get('error_decline_amazon_rejected'),
                            'ProcessingFailure' => language::get('error_decline_processing_failure')
                        );

                        if (in_array($reason_code, array_keys($cart_error_messages))) {
                            //@todo - do the logout with amazon.Login.logout(); instead
                            unset(session::$data['apalwa']);

                            // capital L in Amazon cookie name is required, do not alter for coding standards
                            if (isset(request::$cookie['amazon_Login_state_cache'])) {
                                //@todo - rework this by triggering the JavaScript logout
                                setcookie('amazon_Login_state_cache', null, -1, '/');
                            }

                            throw new \RuntimeException($cart_error_messages[$reason_code]);
                        } else {
                            // This should never occur, but just in case...
                            throw extension_payment_amazon_login_pay::loggedException("Authorization has failed with code: " . $reason_code, language::get('error_process_order'));
                        }
                    break;
                }
            }

            // Amend the billing address based on the Authorize response
            if (!empty($authorization::$AuthorizationBillingAddress)) {
                extension_payment_amazon_login_pay::updatePaymentAddress($order_id, $authorization::$AuthorizationBillingAddress);
            }

            // Clean the session and redirect to the success page
            unset(session::$data['apalwa']['pay']);

            checkout_order::addOrderHistory($order_id, config::get('payment_amazon_login_pay_pending_status'), '', config::get('payment_amazon_login_pay_mode') != 'payment');

            // In case a payment has been completed, and the order is not closed, close it.
            if (isset($authorization::$CapturedAmount::$Amount) && (float)$authorization::$CapturedAmount::$Amount && extension_payment_amazon_login_pay::isOrderInState($order_reference_id, array('Open', 'Suspended'))) {
                extension_payment_amazon_login_pay::closeOrder($order_reference_id, "A capture has been performed. Closing the order.");
            }

            response::redirect(url::link('checkout/success', ''));
        } catch (\RuntimeException $e) {
            extension_payment_amazon_login_pay::cartRedirect($e::getMessage());
        }
    }

    public static function process_us() {
        load::language('extension/payment/amazon_login_pay');
        load::language('checkout/checkout');


        // Verify cart
        extension_payment_amazon_login_pay::verifyCart();

        // Verify login
        extension_payment_amazon_login_pay::verifyLogin();

        // Verify cart total
        // Not needed, as we will display an error message later on...

        // Verify reference
        extension_payment_amazon_login_pay::verifyReference();

        // Verify shipping
        extension_payment_amazon_login_pay::verifyShipping();

        // Verify order
        extension_payment_amazon_login_pay::verifyOrder();

        try {
            $order_reference_id = session::$data['apalwa']['pay']['order_reference_id'];

            if (empty(session::$data['order_id'])) {
                // Up to this point, everything is fine in the session. Save the order and submit it to Amazon.
                $order_id = checkout_order::addOrder(session::$data['apalwa']['pay']['order']);

                session::$data['order_id'] = $order_id;

                $currency_code =  config::get('payment_amazon_login_pay_payment_region');

                $text_version = sprintf(language::get('text_created_by'), self::$version);

                extension_payment_amazon_login_pay::submitOrderDetails($order_reference_id, $order_id, $currency_code, $text_version);
            } else {
                $order_id = session::$data['order_id'];
            }

            // Check constraints
            $constraints = extension_payment_amazon_login_pay::fetchOrder($order_reference_id)::$Constraints;

            if (!empty($constraints::$Constraint)) {
                // We do not expect to fall under the other kinds of constraints. For more information, see: https://pay.amazon.com/us/developer/documentation/apireference/201752890
                $payment_page_errors = array(
                    'PaymentPlanNotSet' => language::get('error_constraint_payment_plan_not_set'),
                    'PaymentMethodNotAllowed' => language::get('error_constraint_payment_method_not_allowed'),
                    'AmountNotSet' => language::get('error_constraint_amount_not_set')
                );

                $constraint_id = (string)$constraints::$Constraint::$ConstraintID;

                if (in_array($constraint_id, array_keys($payment_page_errors))) {
                    session::$data['apalwa']['error'] = $payment_page_errors[$constraint_id];

                    response::redirect(url::link('extension/payment/amazon_login_pay/payment', ''));
                } else {
                    throw new \RuntimeException($constraints::$Constraint::$Description);
                }
            }

            // Open the order for authorization
            extension_payment_amazon_login_pay::confirmOrder($order_reference_id);

            $amazon_order = extension_payment_amazon_login_pay::fetchOrder($order_reference_id);

            // The order has been opened for authorization. Store it in the database
            $amazon_login_pay_order_id = extension_payment_amazon_login_pay::findOrAddOrder($amazon_order);

            // Authorize the order
            $authorization = extension_payment_amazon_login_pay::authorizeOrder($amazon_order);

            // Log the authorization
            extension_payment_amazon_login_pay::addAuthorization($amazon_login_pay_order_id, $authorization);

            if ($authorization::$AuthorizationStatus::$State == 'Declined') {
                $reason_code = (string)$authorization::$AuthorizationStatus::$ReasonCode;

                switch ($reason_code) {
                    case 'InvalidPaymentMethod' :
                        session::$data['apalwa']['error'] = language::get('error_decline_invalid_payment_method');

                        response::redirect(url::link('extension/payment/amazon_login_pay/payment', ''));
                    break;
                    default :
                        if (extension_payment_amazon_login_pay::isOrderInState($order_reference_id, array('Open'))) {
                            extension_payment_amazon_login_pay::cancelOrder($order_reference_id, "Authorization has failed with the state: " . $authorization::$AuthorizationStatus::$State);
                        }

                        $cart_error_messages = array(
                            'TransactionTimedOut' => language::get('error_decline_transaction_timed_out'),
                            'AmazonRejected' => language::get('error_decline_amazon_rejected'),
                            'ProcessingFailure' => language::get('error_decline_processing_failure')
                        );

                        if (in_array($reason_code, array_keys($cart_error_messages))) {
                            //@todo - do the logout with amazon.Login.logout(); instead
                            unset(session::$data['apalwa']);

                            // capital L in Amazon cookie name is required, do not alter for coding standards
                            if (isset(request::$cookie['amazon_Login_state_cache'])) {
                                //@todo - rework this by triggering the JavaScript logout
                                setcookie('amazon_Login_state_cache', null, -1, '/');
                            }

                            throw new \RuntimeException($cart_error_messages[$reason_code]);
                        } else {
                            // This should never occur, but just in case...
                            throw extension_payment_amazon_login_pay::loggedException("Authorization has failed with code: " . $reason_code, language::get('error_process_order'));
                        }
                    break;
                }
            }

            // Amend the billing address based on the Authorize response
            if (!empty($authorization::$AuthorizationBillingAddress)) {
                extension_payment_amazon_login_pay::updatePaymentAddress($order_id, $authorization::$AuthorizationBillingAddress);
            }

            // Clean the session and redirect to the success page
            unset(session::$data['apalwa']['pay']);

            try {
                checkout_order::addOrderHistory($order_id, config::get('payment_amazon_login_pay_pending_status'), '', config::get('payment_amazon_login_pay_mode') != 'payment');
            } catch (\Exception $e) {
                if (config::get('error_log')) {
                    log::write($e::getMessage());
                }
            }

            // In case a payment has been completed, and the order is not closed, close it.
            if (isset($authorization::$CapturedAmount::$Amount) && (float)$authorization::$CapturedAmount::$Amount && extension_payment_amazon_login_pay::isOrderInState($order_reference_id, array('Open', 'Suspended'))) {
                extension_payment_amazon_login_pay::closeOrder($order_reference_id, "A capture has been performed. Closing the order.");
            }

            // Log any errors triggered by addOrderHistory, but without displaying them
            set_error_handler(array(self::$model_extension_payment_amazon_login_pay, 'logHandler'));

            response::redirect(url::link('checkout/success', ''));
        } catch (\RuntimeException $e) {
            extension_payment_amazon_login_pay::cartRedirect($e::getMessage());
        }
    }
    public static function process() {
        load::language('extension/payment/amazon_login_pay');
        load::language('checkout/checkout');


        // Verify cart
        extension_payment_amazon_login_pay::verifyCart();

        // Verify login
        extension_payment_amazon_login_pay::verifyLogin();

        // Verify reference
        extension_payment_amazon_login_pay::verifyReference();

        // Verify shipping
        extension_payment_amazon_login_pay::verifyShipping();

        // Verify order
        extension_payment_amazon_login_pay::verifyOrder();

        $json = array();

        try {
            $order_reference_id = session::$data['apalwa']['pay']['order_reference_id'];

            if (empty(session::$data['order_id'])) {
                // Up to this point, everything is fine in the session. Save the order and submit it to Amazon.
                $order_id = checkout_order::addOrder(session::$data['apalwa']['pay']['order']);

                session::$data['order_id'] = $order_id;

                if(isset(session::$data['apalwa']['pay']['buyer_currency'])) {
                    $currency_code = session::$data['apalwa']['pay']['buyer_currency'];
                } else {
                    $currency_code =  config::get('payment_amazon_login_pay_payment_region');
                }

                $text_version = sprintf(language::get('text_created_by'), self::$version);

                extension_payment_amazon_login_pay::submitOrderDetails($order_reference_id, $order_id, $currency_code, $text_version);
            } else {
                $order_id = session::$data['order_id'];
            }

            // Check constraints
            $constraints = extension_payment_amazon_login_pay::fetchOrder($order_reference_id)::$Constraints;

            if (!empty($constraints::$Constraint)) {
                // We do not expect to fall under the other kinds of constraints. For more information, see: https://pay.amazon.com/us/developer/documentation/apireference/201752890
                $payment_page_errors = array(
                    'PaymentPlanNotSet' => language::get('error_constraint_payment_plan_not_set'),
                    'PaymentMethodNotAllowed' => language::get('error_constraint_payment_method_not_allowed'),
                    'AmountNotSet' => language::get('error_constraint_amount_not_set')
                );

                $constraint_id = (string)$constraints::$Constraint::$ConstraintID;

                if (in_array($constraint_id, array_keys($payment_page_errors))) {
                    session::$data['apalwa']['error'] = $payment_page_errors[$constraint_id];

                    $json['redirect'] = url::link('extension/payment/amazon_login_pay/payment', '');
                } else {
                    throw new \RuntimeException($constraints::$Constraint::$Description);
                }
            } else {
                // Confirm the order reference
                extension_payment_amazon_login_pay::confirmOrder($order_reference_id);
            }
        } catch (\RuntimeException $e) {
            $json['redirect'] = extension_payment_amazon_login_pay::cartRedirect($e::getMessage(), true);
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function ipn() {
        sleep(5);


        try {
            if (!isset(request::$get['token'])) {
                throw new \RuntimeException('GET variable "token" is missing.');
            }

            if (trim(request::$get['token']) == '') {
                throw new \RuntimeException('GET variable "token" set, but is empty.');
            }

            if (!config::get('payment_amazon_login_pay_ipn_token')) {
                throw new \RuntimeException('CONFIG variable "payment_amazon_login_pay_ipn_token" is empty.');
            }

            if (!hash_equals(trim(config::get('payment_amazon_login_pay_ipn_token')), trim(request::$get['token']))) {
                throw new \RuntimeException('Token values are different.');
            }

            // Everything is fine. Process the IPN
            $body = file_get_contents('php://input');

            extension_payment_amazon_login_pay::debugLog('IPN BODY', $body);

            if ($body) {
                $xml = extension_payment_amazon_login_pay::parseIpnBody($body);

                switch ($xml::getName()) {
                    case 'AuthorizationNotification':
                        if(extension_payment_amazon_login_pay::authorizationIpn($xml)) {
                            $oc_order_status_id = config::get('payment_amazon_login_pay_capture_oc_status');
                            $amazon_capture_id = (string)$xml::$AuthorizationDetails::$IdList::$Id;
                            $amazon_captured_amount = (float)$xml::$AuthorizationDetails::$CapturedAmount::$Amount;
                            $amazon_authorization_id = (string)$xml::$AuthorizationDetails::$AmazonAuthorizationId;
                            $exploded_authorization_id = explode("-", $amazon_authorization_id);
                            array_pop($exploded_authorization_id);
                            $amazon_order_reference_id = implode("-", $exploded_authorization_id);
                            $order_id = extension_payment_amazon_login_pay::findOCOrderId($amazon_order_reference_id);
                            $amazon_login_pay_order_id = extension_payment_amazon_login_pay::findAOrderId($amazon_order_reference_id);
                            $transaction = array(
                                'amazon_login_pay_order_id' => $amazon_login_pay_order_id,
                                'amazon_authorization_id' => $amazon_authorization_id,
                                'amazon_capture_id' => $amazon_capture_id,
                                'amazon_refund_id' => '',
                                'date_added' => date('Y-m-d H:i:s', strtotime((string)$xml::$AuthorizationDetails::$CreationTimestamp)),
                                'type' => 'capture',
                                'status' => 'Completed',
                                'amount' => $amazon_captured_amount
                            );
                            $transaction_exists = (!empty($amazon_capture_id)) ? extension_payment_amazon_login_pay::findCapture($amazon_capture_id) : false;

                            if(!isset($transaction_exists) || !$transaction_exists) {
                                extension_payment_amazon_login_pay::addTransaction($transaction);
                            }
                            $order_reference_details = extension_payment_amazon_login_pay::fetchOrder($amazon_order_reference_id);
                            $order_reason_code = (string) $order_reference_details::$OrderReferenceStatus::$ReasonCode;
                            $order_state = (string) $order_reference_details::$OrderReferenceStatus::$State;
                            $order_total = (float) $order_reference_details::$OrderTotal::$Amount;
                            $total_captured = extension_payment_amazon_login_pay::getTotalCaptured($amazon_login_pay_order_id);
                            //chnage the order status only if the order is closed with the response code maxamountcharged or if the order is fully captured
                            if(($order_state =="Closed" && $order_reason_code =="MaxAmountCharged") || $order_reason_code == "SellerClosed" || $amazon_captured_amount >= $order_total || $total_captured >= $order_total) {
                                //update the order history
                                checkout_order::addOrderHistory($order_id, $oc_order_status_id, "", true);
                            }
                        }
                        break;
                    case 'CaptureNotification':
                        extension_payment_amazon_login_pay::captureIpn($xml);
                        break;
                    case 'RefundNotification':
                        extension_payment_amazon_login_pay::refundIpn($xml);
                        break;
                }
            }
        } catch (\RuntimeException $e) {
            extension_payment_amazon_login_pay::debugLog('IPN ERROR', $e::getMessage());
        }

        response::addHeader('HTTP/1.1 200 OK');
        response::addHeader('Content-Type: application/json');
    }

    public static function capture(&$route, &$args, &$output) {
        load::language('extension/payment/amazon_login_pay');

        $order_id = $args[0];

        $order_info = checkout_order::getOrder($order_id);

        if ($order_info['order_status_id'] == config::get('payment_amazon_login_pay_capture_status')) {
            try {
                $amazon_login_pay_order = extension_payment_amazon_login_pay::getOrderByOrderId($order_id);

                $capture_response = extension_payment_amazon_login_pay::captureOrder($amazon_login_pay_order['amazon_authorization_id'], $amazon_login_pay_order['total'], $amazon_login_pay_order['currency_code']);

                if (isset($capture_response::$CaptureStatus::$State) && in_array($capture_response::$CaptureStatus::$State, array('Completed', 'Pending'))) {
                    $order_reference_id = $amazon_login_pay_order['amazon_order_reference_id'];

                    if (extension_payment_amazon_login_pay::isOrderInState($order_reference_id, array('Open', 'Suspended'))) {
                        extension_payment_amazon_login_pay::closeOrder($order_reference_id, "Captured amount: " . (string)$capture_response::$CaptureAmount::$Amount . " " . (string)$capture_response::$CaptureAmount::$CurrencyCode);
                    }

                    $transaction = array(
                        'amazon_login_pay_order_id' => $amazon_login_pay_order['amazon_login_pay_order_id'],
                        'amazon_authorization_id' => $amazon_login_pay_order['amazon_authorization_id'],
                        'amazon_capture_id' => $capture_response::$AmazonCaptureId,
                        'amazon_refund_id' => '',
                        'date_added' => date('Y-m-d H:i:s', strtotime((string)$capture_response::$CreationTimestamp)),
                        'type' => 'capture',
                        'status' => (string)$capture_response::$CaptureStatus::$State,
                        'amount' => (float)$capture_response::$CaptureAmount::$Amount
                    );

                    extension_payment_amazon_login_pay::addTransaction($transaction);

                    extension_payment_amazon_login_pay::updateStatus($amazon_login_pay_order['amazon_authorization_id'], 'authorization', 'Closed');

                    extension_payment_amazon_login_pay::updateCapturedStatus($amazon_login_pay_order['amazon_login_pay_order_id'], 1);
                }
            } catch (\RuntimeException $e) {
                // Do nothing, as the exception is logged in case of debug logging.
            }
        }
    }
}
