<?php
class ControllerExtensionPaymentAuthorizeNetSim {
	public static function index() {
		load::language('extension/payment/authorizenet_sim');

		$data['button_confirm'] = language::get('button_confirm');


		$order_info = checkout_order::getOrder(session::$data['order_id']);

		$data['x_login'] = config::get('payment_authorizenet_sim_merchant');
		$data['x_fp_sequence'] = session::$data['order_id'];
		$data['x_fp_timestamp'] = time();
		$data['x_amount'] = currency::format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);
		$data['x_fp_hash'] = null; // calculated later, once all fields are populated
		$data['x_show_form'] = 'PAYMENT_FORM';
		$data['x_test_request'] = config::get('payment_authorizenet_sim_mode');
		$data['x_type'] = 'AUTH_CAPTURE';
		$data['x_currency_code'] = session::$data['currency'];
		$data['x_invoice_num'] = session::$data['order_id'];
		$data['x_description'] = html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8');
		$data['x_first_name'] = $order_info['payment_firstname'];
		$data['x_last_name'] = $order_info['payment_lastname'];
		$data['x_company'] = $order_info['payment_company'];
		$data['x_address'] = $order_info['payment_address_1'] . ' ' . $order_info['payment_address_2'];
		$data['x_city'] = $order_info['payment_city'];
		$data['x_state'] = $order_info['payment_zone'];
		$data['x_zip'] = $order_info['payment_postcode'];
		$data['x_country'] = $order_info['payment_country'];
		$data['x_phone'] = $order_info['telephone'];
		$data['x_ship_to_first_name'] = $order_info['shipping_firstname'];
		$data['x_ship_to_last_name'] = $order_info['shipping_lastname'];
		$data['x_ship_to_company'] = $order_info['shipping_company'];
		$data['x_ship_to_address'] = $order_info['shipping_address_1'] . ' ' . $order_info['shipping_address_2'];
		$data['x_ship_to_city'] = $order_info['shipping_city'];
		$data['x_ship_to_state'] = $order_info['shipping_zone'];
		$data['x_ship_to_zip'] = $order_info['shipping_postcode'];
		$data['x_ship_to_country'] = $order_info['shipping_country'];
		$data['x_customer_ip'] = request::$server['REMOTE_ADDR'];
		$data['x_email'] = $order_info['email'];
		$data['x_relay_response'] = 'true';

		$to_hash = $data['x_login'] . '^' . $data['x_fp_sequence'] . '^' . $data['x_fp_timestamp'] . '^' . $data['x_amount'] . '^' . $data['x_currency_code'];
		$data['x_fp_hash'] = self::generateHash($to_hash, config::get('payment_authorizenet_sim_key'));

		return load::view('extension/payment/authorizenet_sim', $data);
	}

	public static function callback() {
		if (isset(request::$post['x_SHA2_Hash']) && (request::$post['x_SHA2_Hash'] == self::generateResponseHash(request::$post, config::get('payment_authorizenet_sim_hash')))) {

			$order_info = checkout_order::getOrder(request::$post['x_invoice_num']);

			if ($order_info && request::$post['x_response_code'] == '1') {
				$message = '';

				if (isset(request::$post['x_response_reason_text'])) {
					$message .= 'Response Text: ' . request::$post['x_response_reason_text'] . "\n";
				}

				if (isset(request::$post['exact_issname'])) {
					$message .= 'Issuer: ' . request::$post['exact_issname'] . "\n";
				}

				if (isset(request::$post['exact_issconf'])) {
					$message .= 'Confirmation Number: ' . request::$post['exact_issconf'];
				}

				if (isset(request::$post['exact_ctr'])) {
					$message .= 'Receipt: ' . request::$post['exact_ctr'];
				}

				checkout_order::addOrderHistory(request::$post['x_invoice_num'], config::get('payment_authorizenet_sim_order_status_id'), $message, true);

				response::redirect(url::link('checkout/success'));
			} else {
				response::redirect(url::link('checkout/failure'));
			}
		} else {
			response::redirect(url::link('checkout/failure'));
		}
	}

	private static function generateHash($to_hash, $key) {
		if ($to_hash != null && $key != null) {
			$sig = hash_hmac('sha512', $to_hash, hex2bin($key));

			return strtoupper($sig);
		} else {
			return false;
		}
	}

	private static function generateResponseHash($post_fields, $signature_key) {
		/**
		 * The following array must not be reordered or elements removed, the hash requires ALL, even if empty/not set
		 */
		$verify_hash_fields = [
			'x_trans_id',
			'x_test_request',
			'x_response_code',
			'x_auth_code',
			'x_cvv2_resp_code',
			'x_cavv_response',
			'x_avs_code',
			'x_method',
			'x_account_number',
			'x_amount',
			'x_company',
			'x_first_name',
			'x_last_name',
			'x_address',
			'x_city',
			'x_state',
			'x_zip',
			'x_country',
			'x_phone',
			'x_fax',
			'x_email',
			'x_ship_to_company',
			'x_ship_to_first_name',
			'x_ship_to_last_name',
			'x_ship_to_address',
			'x_ship_to_city',
			'x_ship_to_state',
			'x_ship_to_zip',
			'x_ship_to_country',
			'x_invoice_num',
		];

		$to_hash = '^';

		foreach ($verify_hash_fields as $hash_field) {
			$to_hash .= (isset($post_fields[$hash_field]) ? $post_fields[$hash_field] : '') . '^';
		}

		return self::generateHash($to_hash, $signature_key);
	}
}