<?php
class ControllerExtensionPaymentCardConnect {
	public static function index() {
		load::language('extension/payment/cardconnect');


		$data['card_types'] = extension_payment_cardconnect::getCardTypes();

		$data['months'] = extension_payment_cardconnect::getMonths();

		$data['years'] = extension_payment_cardconnect::getYears();

		if (customer::isLogged() && config::get('cardconnect_store_cards')) {
			$data['store_cards'] = true;

			$data['cards'] = extension_payment_cardconnect::getCards(customer::getId());
		} else {
			$data['store_cards'] = false;

			$data['cards'] = array();
		}

		$data['echeck'] = config::get('cardconnect_echeck');

		$data['action'] = url::link('extension/payment/cardconnect/send', '');

		return load::view('extension/payment/cardconnect', $data);
	}

	public static function send()	{
		load::language('extension/payment/cardconnect');


		extension_payment_cardconnect::log('Posting order to CardConnect');

		$json = array();

		$json['error'] = '';

		if (config::get('cardconnect_status')) {
			if (request::$server['REQUEST_METHOD'] == 'POST') {
				$error = self::validate();

				if (!$error) {

					$order_info = checkout_order::getOrder(session::$data['order_id']);

					if ($order_info) {
						extension_payment_cardconnect::log('Order ID: ' . $order_info['order_id']);

						$accttype = $account = $expiry = $cvv2 = $profile = $capture = $bankaba = '';

						$existing_card = false;

						if (!isset(request::$post['method']) || request::$post['method'] == 'card') {
							extension_payment_cardconnect::log('Method is card');

							if (request::$post['card_new'] && isset(request::$post['card_save']) && config::get('cardconnect_store_cards') && customer::isLogged()) {
								$profile = 'Y';
							} else if (!request::$post['card_new'] && customer::isLogged()) {
								$existing_card = extension_payment_cardconnect::getCard(request::$post['card_choice'], customer::getId());

								$profile = $existing_card['profileid'];
							}

							if ($existing_card) {
								$accttype = $existing_card['type'];

								$account = $existing_card['token'];

								$expiry = $existing_card['expiry'];

								$cvv2 = '';
							} else {
								$accttype = request::$post['card_type'];

								$account = request::$post['card_number'];

								$expiry = request::$post['card_expiry_month'] . request::$post['card_expiry_year'];

								$cvv2 = request::$post['card_cvv2'];
							}
						} else {
							extension_payment_cardconnect::log('Method is Echeck');

							$account = request::$post['account_number'];

							$bankaba = request::$post['routing_number'];
						}

						if (config::get('cardconnect_transaction') == 'payment') {
							$capture = 'Y';

							$type = 'payment';

							$status = 'New';

							$order_status_id = config::get('cardconnect_order_status_id_processing');
						} else {
							$capture = 'N';

							$type = 'auth';

							$status = 'New';

							$order_status_id = config::get('cardconnect_order_status_id_pending');
						}

						$data = array(
							'merchid'    => config::get('payment_cardconnect_merchant_id'),
							'accttype'   => $accttype,
							'account'    => $account,
							'expiry'     => $expiry,
							'cvv2'       => $cvv2,
							'amount'     => round(floatval($order_info['total']), 2, PHP_ROUND_HALF_DOWN),
							'currency'   => $order_info['currency_code'],
							'orderid'    => $order_info['order_id'],
							'name'       => $order_info['payment_firstname'] . ' ' . $order_info['payment_lastname'],
							'address'    => $order_info['payment_address_1'],
							'city'       => $order_info['payment_city'],
							'region'     => $order_info['payment_zone'],
							'country'    => $order_info['payment_iso_code_2'],
							'postal'     => $order_info['payment_postcode'],
							'email'      => $order_info['email'],
							'phone'      => $order_info['telephone'],
							'ecomind'    => 'E',
							'tokenize'   => 'Y',
							'profile'    => $profile,
							'capture'    => $capture,
							'bankaba'    => $bankaba,
							'userfields' => array('secret_token' => config::get('cardconnect_token')),
							'frontendid' => '26'
						);

						$data_json = json_encode($data);

						$url = 'https://' . config::get('cardconnect_site') . '.cardconnect.com:' . ((config::get('cardconnect_environment') == 'live') ? 8443 : 6443) . '/cardconnect/rest/auth';

						$header = array();

						$header[] = 'Content-type: application/json';
						$header[] = 'Content-length: ' . strlen($data_json);
						$header[] = 'Authorization: Basic ' . base64_encode(config::get('cardconnect_api_username') . ':' . config::get('cardconnect_api_password'));

						extension_payment_cardconnect::log('Header: ' . print_r($header, true));

						extension_payment_cardconnect::log('Post Data: ' . print_r($data, true));

						extension_payment_cardconnect::log('URL: ' . $url);

						$ch = curl_init();
						curl_setopt($ch, CURLOPT_URL, $url);
						curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
						curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
						curl_setopt($ch, CURLOPT_POSTFIELDS, $data_json);
						curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
						curl_setopt($ch, CURLOPT_TIMEOUT, 30);
						curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
						$response_data = curl_exec($ch);
						if (curl_errno($ch)) {
							extension_payment_cardconnect::log('cURL error: ' . curl_errno($ch));
						}
						curl_close($ch);

						$response_data = json_decode($response_data, true);

						extension_payment_cardconnect::log('Response: ' . print_r($response_data, true));

					 	if (isset($response_data['respstat']) && $response_data['respstat'] == 'A') {

							// if a cheque
							if ($bankaba) {
								$payment_method = 'echeck';

								$type = 'payment';
							} else {
								$payment_method = 'card';
							}

							checkout_order::addOrderHistory($order_info['order_id'], $order_status_id);

							$order_info = array_merge($order_info, $response_data);

							$cardconnect_order_id = extension_payment_cardconnect::addOrder($order_info, $payment_method);

							extension_payment_cardconnect::addTransaction($cardconnect_order_id, $type, $status, $order_info);

							if (isset($response_data['profileid']) && config::get('cardconnect_store_cards') && customer::isLogged()) {
								extension_payment_cardconnect::log('Saving card');

								extension_payment_cardconnect::addCard($cardconnect_order_id, customer::getId(), $response_data['profileid'], $response_data['token'], request::$post['card_type'], $response_data['account'], request::$post['card_expiry_month'] . request::$post['card_expiry_year']);
							}

							extension_payment_cardconnect::log('Success');

							$json['success'] = url::link('checkout/success', '');
						} else {
							extension_payment_cardconnect::log($response_data['resptext']);

							$json['error']['warning'] = $response_data['resptext'];
						}
					} else {
						extension_payment_cardconnect::log('No matching order');

						$json['error']['warning'] = language::get('error_no_order');
					}
				} else {
					extension_payment_cardconnect::log('Failed validation');

					$json['error'] = $error;
				}
			} else {
				extension_payment_cardconnect::log('No $_POST data');

				$json['error']['warning'] = language::get('error_no_post_data');
			}
		} else {
			extension_payment_cardconnect::log('Module not enabled');

			$json['error']['warning'] = language::get('error_not_enabled');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function delete() {
		load::language('extension/payment/cardconnect');


		extension_payment_cardconnect::log('Deleting card');

		$json = array();

		if (config::get('cardconnect_status')) {
			if (customer::isLogged()) {
				if (isset(request::$post['card_choice'])) {
					if (request::$post['card_choice']) {
						$card = extension_payment_cardconnect::getCard(request::$post['card_choice'], customer::getId());

						if ($card) {
							extension_payment_cardconnect::deleteCard(request::$post['card_choice'], customer::getId());
						} else {
							extension_payment_cardconnect::log('No such card');

							$json['error'] = language::get('error_no_card');
						}
					} else {
						extension_payment_cardconnect::log('No card selected');

						$json['error'] = language::get('error_select_card');
					}
				} else {
					extension_payment_cardconnect::log('Data missing');

					$json['error'] = language::get('error_data_missing');
				}
			} else {
				extension_payment_cardconnect::log('Not logged in');

				$json['error'] = language::get('error_not_logged_in');
			}
		} else {
			extension_payment_cardconnect::log('Module not enabled');

			$json['error']['warning'] = language::get('error_not_enabled');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function cron() {

		extension_payment_cardconnect::log('Running cron');

		if (config::get('cardconnect_status')) {
			if (isset(request::$get['token']) && hash_equals(config::get('cardconnect_token'), request::$get['token'])) {
				$date = date('md', strtotime('yesterday'));

				$responses = extension_payment_cardconnect::getSettlementStatuses(config::get('payment_cardconnect_merchant_id'), $date);

				foreach($responses as $response) {
					foreach($response['txns'] as $transaction) {
						extension_payment_cardconnect::updateTransactionStatusByRetref($transaction['retref'], $transaction['setlstat']);
					}
				}

				extension_payment_cardconnect::updateCronRunTime();
			} else {
				extension_payment_cardconnect::log('Token does not match.');
			}
		} else {
			extension_payment_cardconnect::log('Module not enabled');
		}
	}

	private static function validate() {
		load::language('extension/payment/cardconnect');


		$error = array();

		if (!isset(request::$post['method']) || request::$post['method'] == 'card') {
			if (request::$post['card_new']) {
				if (!isset(request::$post['card_number']) || utf8_strlen(request::$post['card_number']) < 1 || utf8_strlen(request::$post['card_number']) > 19) {
					$error['card_number'] = language::get('error_card_number');
				}

				if (!isset(request::$post['card_cvv2']) || utf8_strlen(request::$post['card_cvv2']) < 1 || utf8_strlen(request::$post['card_cvv2']) > 4) {
					$error['card_cvv2'] = language::get('error_card_cvv2');
				}
			} else {
				if (isset(request::$post['card_choice']) && request::$post['card_choice']) {
					$card = extension_payment_cardconnect::getCard(request::$post['card_choice'], customer::getId());

					if (!$card) {
						$error['card_choice'] = language::get('error_no_card');
					}
				} else {
					$error['card_choice'] = language::get('error_select_card');
				}
			}
		} else {
			if (config::get('cardconnect_echeck')) {
				if (!isset(request::$post['account_number']) || utf8_strlen(request::$post['account_number']) < 1 || utf8_strlen(request::$post['account_number']) > 19) {
					$error['account_number'] = language::get('error_account_number');
				}

				if (!isset(request::$post['routing_number']) || utf8_strlen(request::$post['routing_number']) < 1 || utf8_strlen(request::$post['routing_number']) > 9) {
					$error['routing_number'] = language::get('error_routing_number');
				}
			} else {
				$error['method'] = language::get('error_no_echeck');
			}
		}

		return $error;
	}
}