<?php
class ControllerExtensionPaymentCardinity {
	public static function index() {
		load::language('extension/payment/cardinity');

		$data['months'] = array();

		for ($i = 1; $i <= 12; $i++) {
			$data['months'][] = array(
				'text'  => sprintf('%02d', $i),
				'value' => sprintf('%02d', $i)
			);
		}

		$today = getdate();

		$data['years'] = array();

		for ($i = $today['year']; $i < $today['year'] + 11; $i++) {
			$data['years'][] = array(
				'text'  => sprintf('%02d', $i % 100),
				'value' => sprintf('%04d', $i)
			);
		}

		return load::view('extension/payment/cardinity', $data);
	}

	public static function send() {

		load::language('extension/payment/cardinity');

		$json = array();

		$json['error'] = $json['success'] = $json['3ds'] = '';

		$payment = false;

		$error = self::validate();

		if (!$error) {
			$order_info = checkout_order::getOrder(session::$data['order_id']);

			if (strlen($order_info['order_id']) < 2) {
				$order_id = '0' . $order_info['order_id'];
			} else {
				$order_id = $order_info['order_id'];
			}

			if (!empty($order_info['payment_iso_code_2'])) {
				$order_country = $order_info['payment_iso_code_2'];
			} else {
				$order_country = $order_info['shipping_iso_code_2'];
			}

			$payment_data = array(
				'amount'			 => (float)currency::format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false),
				'currency'			 => $order_info['currency_code'],
				'order_id'			 => $order_id,
				'country'            => $order_country,
				'payment_method'     => 'card',
				'payment_instrument' => array(
					'pan'		=> preg_replace('!\s+!', '', request::$post['pan']),
					'exp_year'	=> (int)request::$post['exp_year'],
					'exp_month' => (int)request::$post['exp_month'],
					'cvc'		=> request::$post['cvc'],
					'holder'	=> request::$post['holder']
				),
			);

			try {
				$payment = extension_payment_cardinity::createPayment(config::get('payment_cardinity_key'), config::get('payment_cardinity_secret'), $payment_data);
			} catch (Cardinity\Exception\Declined $exception) {
				self::failedOrder(language::get('error_payment_declined'), language::get('error_payment_declined'));

				$json['redirect'] = url::link('checkout/checkout', '');
			} catch (Exception $exception) {
				self::failedOrder();

				$json['redirect'] = url::link('checkout/checkout', '');
			}

			$successful_order_statuses = array(
				'approved',
				'pending'
			);

			if ($payment) {
				if (!in_array($payment::getStatus(), $successful_order_statuses)) {
					self::failedOrder($payment::getStatus());

					$json['redirect'] = url::link('checkout/checkout', '');
				} else {
					extension_payment_cardinity::addOrder(array(
						'order_id'   => session::$data['order_id'],
						'payment_id' => $payment::getId()
					));

					if ($payment::getStatus() == 'pending') {
						//3ds
						$authorization_information = $payment::getAuthorizationInformation();

						$encryption_data = array(
							'order_id' => session::$data['order_id'],
							'secret'   => config::get('payment_cardinity_secret')
						);

						$hash = encryption::encrypt(config('encryption'), json_encode($encryption_data));

						$json['3ds'] = array(
							'url'     => $authorization_information::getUrl(),
							'PaReq'   => $authorization_information::getData(),
							'TermUrl' => url::link('extension/payment/cardinity/threeDSecureCallback', ''),
							'hash'    => $hash
						);
					} elseif ($payment::getStatus() == 'approved') {
						self::finalizeOrder($payment);

						$json['redirect'] = url::link('checkout/success', '');
					}
				}
			}
		} else {
			$json['error'] = $error;
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function threeDSecureForm() {

		load::language('extension/payment/cardinity');

		$success = false;
		$redirect = false;

		$encryption_data = array(
			'order_id' => session::$data['order_id'],
			'secret'   => config::get('payment_cardinity_secret')
		);

		$hash = encryption::encrypt(config('encryption'), json_encode($encryption_data));

		if (hash_equals($hash, request::$post['hash'])) {
			$success = true;

			$data['url'] = request::$post['url'];
			$data['PaReq'] = request::$post['PaReq'];
			$data['TermUrl'] = request::$post['TermUrl'];
			$data['MD'] = $hash;
		} else {
			self::failedOrder(language::get('error_invalid_hash'));

			$redirect = url::link('checkout/checkout', '');
		}

		$data['success'] = $success;
		$data['redirect'] = $redirect;

		response::setOutput(load::view('extension/payment/cardinity_3ds', $data));
	}

	public static function threeDSecureCallback() {

		load::language('extension/payment/cardinity');

		$success = false;

		$error = '';

		$encryption_data = array(
			'order_id' => session::$data['order_id'],
			'secret'   => config::get('payment_cardinity_secret')
		);

		$hash = encryption::encrypt(config('encryption'), json_encode($encryption_data));

		if (hash_equals($hash, request::$post['MD'])) {
			$order = extension_payment_cardinity::getOrder($encryption_data['order_id']);

			if ($order && $order['payment_id']) {
				$payment = extension_payment_cardinity::finalizePayment(config::get('payment_cardinity_key'), config::get('payment_cardinity_secret'), $order['payment_id'], request::$post['PaRes']);

				if ($payment && $payment::getStatus() == 'approved') {
					$success = true;
				} else {
					$error = language::get('error_finalizing_payment');
				}
			} else {
				$error = language::get('error_unknown_order_id');
			}
		} else {
			$error = language::get('error_invalid_hash');
		}

		if ($success) {
			self::finalizeOrder($payment);

			response::redirect(url::link('checkout/success', ''));
		} else {
			self::failedOrder($error);

			response::redirect(url::link('checkout/checkout', ''));
		}
	}

	private static function finalizeOrder($payment) {

		load::language('extension/payment/cardinity');

		checkout_order::addOrderHistory(session::$data['order_id'], config::get('payment_cardinity_order_status_id'));

		extension_payment_cardinity::log(language::get('text_payment_success'));
		extension_payment_cardinity::log($payment);
	}

	private static function failedOrder($log = null, $alert = null) {
		load::language('extension/payment/cardinity');

		extension_payment_cardinity::log(language::get('text_payment_failed'));

		if ($log) {
			extension_payment_cardinity::log($log);
		}

		if ($alert) {
			session::$data['error'] = $alert;
		} else {
			session::$data['error'] = language::get('error_process_order');
		}
	}

	private static function validate() {

		$error = array();

		if (!session::$data['order_id']) {
			$error['warning'] = language::get('error_process_order');
		}

		if (!$error) {
			$order_info = checkout_order::getOrder(session::$data['order_id']);

			if (!$order_info) {
				$error['warning'] = language::get('error_process_order');
			}
		}

		if (!in_array($order_info['currency_code'], extension_payment_cardinity::getSupportedCurrencies())) {
			$error['warning'] = language::get('error_invalid_currency');
		}

		if (!isset(request::$post['holder']) || utf8_strlen(request::$post['holder']) < 1 || utf8_strlen(request::$post['holder']) > 32) {
			$error['holder'] = true;
		}

		if (!isset(request::$post['pan']) || utf8_strlen(request::$post['pan']) < 1 || utf8_strlen(request::$post['pan']) > 19) {
			$error['pan'] = true;
		}

		if (!isset(request::$post['pan']) || !is_numeric(preg_replace('!\s+!', '', request::$post['pan']))) {
			$error['pan'] = true;
		}

		if (!isset(request::$post['exp_month']) || !isset(request::$post['exp_year'])) {
			$error['expiry_date'] = true;
		} else {
			$expiry = new DateTime();
			$expiry::setDate(request::$post['exp_year'], request::$post['exp_month'], '1');
			$expiry::modify('+1 month');
			$expiry::modify('-1 day');

			$now = new DateTime();

			if ($expiry < $now) {
				$error['expiry_date'] = true;
			}
		}

		if (!isset(request::$post['cvc']) || utf8_strlen(request::$post['cvc']) < 1 || utf8_strlen(request::$post['cvc']) > 4) {
			$error['cvc'] = true;
		}

		return $error;
	}
}