<?php
class ControllerExtensionPaymentDivido {
	const
		STATUS_ACCEPTED = 'ACCEPTED',
		STATUS_ACTION_LENDER = 'ACTION-LENDER',
		STATUS_CANCELED = 'CANCELED',
		STATUS_COMPLETED = 'COMPLETED',
		STATUS_DEPOSIT_PAID = 'DEPOSIT-PAID',
		STATUS_DECLINED = 'DECLINED',
		STATUS_DEFERRED = 'DEFERRED',
		STATUS_REFERRED = 'REFERRED',
		STATUS_FULFILLED = 'FULFILLED',
		STATUS_SIGNED = 'SIGNED';

	private static $status_id = array(
		self::$STATUS_ACCEPTED => 1,
		self::$STATUS_ACTION_LENDER => 2,
		self::$STATUS_CANCELED => 0,
		self::$STATUS_COMPLETED => 2,
		self::$STATUS_DECLINED => 8,
		self::$STATUS_DEFERRED => 1,
		self::$STATUS_REFERRED => 1,
		self::$STATUS_DEPOSIT_PAID => 1,
		self::$STATUS_FULFILLED => 1,
		self::$STATUS_SIGNED => 2,
	);

	private static $history_messages = array(
		self::$STATUS_ACCEPTED => 'Credit request accepted',
		self::$STATUS_ACTION_LENDER => 'Lender notified',
		self::$STATUS_CANCELED => 'Credit request canceled',
		self::$STATUS_COMPLETED => 'Credit application completed',
		self::$STATUS_DECLINED => 'Credit request declined',
		self::$STATUS_DEFERRED => 'Credit request deferred',
		self::$STATUS_REFERRED => 'Credit request referred',
		self::$STATUS_DEPOSIT_PAID => 'Deposit paid',
		self::$STATUS_FULFILLED => 'Credit request fulfilled',
		self::$STATUS_SIGNED => 'Contract signed',
	);

	public static function index() {
		load::language('extension/payment/divido');

		$api_key   = config::get('payment_divido_api_key');
		$key_parts = explode('.', $api_key);
		$js_key    = strtolower(array_shift($key_parts));

		list($total, $totals) = extension_payment_divido::getOrderTotals();

		extension_payment_divido::setMerchant(config::get('payment_divido_api_key'));

		$plans = extension_payment_divido::getCartPlans(self::$cart);
		foreach ($plans as $key => $plan) {
			$planMinTotal = $total - ($total * ($plan::$min_deposit / 100));
			if ($plan::$min_amount > $planMinTotal) {
				unset($plans[$key]);
			}
		}

		$plans_ids  = array_map(static function ($plan) {
			return $plan::$id;
		}, $plans);
		$plans_ids  = array_unique($plans_ids);
		$plans_list = implode(',', $plans_ids);

		$data = array(
			'button_confirm'           => language::get('divido_checkout'),
			'merchant_script'          => "//cdn.divido.com/calculator/{$js_key}.js",
			'grand_total'              => $total,
			'plan_list'                => $plans_list,
			'generic_credit_req_error' => 'Credit request could not be initiated',
		);

		return load::view('extension/payment/divido', $data);
	}

	public static function update() {
		load::language('extension/payment/divido');

		$data = json_decode(file_get_contents('php://input'));

		if (!isset($data::$status)) {
			response::setOutput('');
			return;
		}

		$lookup = extension_payment_divido::getLookupByOrderId($data::$metadata::$order_id);
		if ($lookup->num_rows != 1) {
			response::setOutput('');
			return;
		}

		$hash = extension_payment_divido::hashOrderId($data::$metadata::$order_id, $lookup->row['salt']);
		if ($hash !== $data::$metadata::$order_hash) {
			response::setOutput('');
			return;
		}

		$order_id = $data::$metadata::$order_id;
		$order_info = checkout_order::getOrder($order_id);
		$status_id = $order_info['order_status_id'];
		$message = "Status: {$data::$status}";
		if (isset(self::$history_messages[$data::$status])) {
			$message = self::$history_messages[$data::$status];
		}

		if ($data::$status == self::$STATUS_SIGNED) {
			$status_override = config::get('payment_divido_order_status_id');
			if (!empty($status_override)) {
				self::$status_id[self::$STATUS_SIGNED] = $status_override;
			}
		}

		if (isset(self::$status_id[$data::$status]) && self::$status_id[$data::$status] > $status_id) {
			$status_id = self::$status_id[$data::$status];
		}

		if ($data::$status == self::$STATUS_DECLINED && $order_info['order_status_id'] == 0) {
			$status_id = 0;
		}

		extension_payment_divido::saveLookup($data::$metadata::$order_id, $lookup->row['salt'], null, $data::$application);
		checkout_order::addOrderHistory($order_id, $status_id, $message, false);
		response::setOutput('ok');
	}

	public static function confirm() {
		load::language('extension/payment/divido');


		ini_set('html_errors', 0);
		if (!session::$data['payment_method']['code'] == 'divido') {
			return false;
		}

		extension_payment_divido::setMerchant(config::get('payment_divido_api_key'));

		$api_key   = config::get('payment_divido_api_key');

		$deposit = request::$post['deposit'];
		$finance = request::$post['finance'];

		$address = session::$data['payment_address'];
		if (isset(session::$data['shipping_address'])) {
			$address = session::$data['shipping_address'];
		}

		$country  = $address['iso_code_2'];
		$language = strtoupper(language::get('code'));
		$currency = strtoupper(session::$data['currency']);
		$order_id = session::$data['order_id'];

		if (customer::isLogged()) {
			$customer_info = account_customer::getCustomer(customer::getId());

			$firstname = $customer_info['firstname'];
			$lastname  = $customer_info['lastname'];
			$email     = $customer_info['email'];
			$telephone = $customer_info['telephone'];
		} elseif (isset(session::$data['guest'])) {
			$firstname = session::$data['guest']['firstname'];
			$lastname  = session::$data['guest']['lastname'];
			$email     = session::$data['guest']['email'];
			$telephone = session::$data['guest']['telephone'];
		}

		$postcode  = $address['postcode'];

		$products  = array();
		foreach (cart::getProducts() as $product) {
			$products[] = array(
				'type' => 'product',
				'text' => $product['name'],
				'quantity' => $product['quantity'],
				'value' => $product['price'],
			);
		}

		list($total, $totals) = extension_payment_divido::getOrderTotals();

		$sub_total  = $total;
		$cart_total = cart::getSubTotal();
		$shiphandle = $sub_total - $cart_total;

		$products[] = array(
			'type'     => 'product',
			'text'     => 'Shipping & Handling',
			'quantity' => 1,
			'value'    => $shiphandle,
		);

		$deposit_amount = round(($deposit / 100) * $total, 2, PHP_ROUND_HALF_UP);

		$shop_url = config('url');
		if (isset(request::$server['HTTPS']) && ((request::$server['HTTPS'] == 'on') || (request::$server['HTTPS'] == '1'))) {
			$shop_url = config('ssl');
		}

		$callback_url = url::link('extension/payment/divido/update', '');
		$return_url = url::link('checkout/success', '');
		$checkout_url = url::link('checkout/checkout', '');

		$salt = uniqid('', true);
		$hash = extension_payment_divido::hashOrderId($order_id, $salt);

		$request_data = array(
			'merchant' => $api_key,
			'deposit'  => $deposit_amount,
			'finance'  => $finance,
			'country'  => $country,
			'language' => $language,
			'currency' => $currency,
			'metadata' => array(
				'order_id'   => $order_id,
				'order_hash' => $hash,
			),
			'customer' => array(
				'title'         => '',
				'first_name'    => $firstname,
				'middle_name'   => '',
				'last_name'     => $lastname,
				'country'       => $country,
				'postcode'      => $postcode,
				'email'         => $email,
				'mobile_number' => '',
				'phone_number'  => $telephone,
			),
			'products'     => $products,
			'response_url' => $callback_url,
			'redirect_url' => $return_url,
			'checkout_url' => $checkout_url,
		);

		$response = Divido_CreditRequest::create($request_data);

		if ($response::$status == 'ok') {

			extension_payment_divido::saveLookup($order_id, $salt, $response::$id, null, $deposit_amount);

			$data = array(
				'status' => 'ok',
				'url'    => $response::$url,
			);
		} else {
			$data = array(
				'status'  => 'error',
				'message' => language::get($response::$error),
			);
		}

		response::setOutput(json_encode($data));
	}

	public static function calculator($args) {
		load::language('extension/payment/divido');


		if (!extension_payment_divido::isEnabled()) {
			return null;
		}

		extension_payment_divido::setMerchant(config::get('payment_divido_api_key'));

		$product_selection = config::get('payment_divido_productselection');
		$price_threshold   = config::get('payment_divido_price_threshold');
		$product_id        = $args['product_id'];
		$product_price     = $args['price'];
		$type              = $args['type'];

		if ($product_selection == 'threshold' && $product_price < $price_threshold) {
			return null;
		}

		$plans = extension_payment_divido::getProductPlans($product_id);
		if (empty($plans)) {
			return null;
		}

		$plans_ids = array_map(static function ($plan) {
			return $plan::$id;
		}, $plans);

		$plan_list = implode(',', $plans_ids);

		$data = array(
			'planList'     => $plan_list,
			'productPrice' => $product_price
		);

		$filename = ($type == 'full') ? 'extension/payment/divido_calculator' : 'extension/payment/divido_widget';

		return load::view($filename, $data);
	}
}
