<?php
class ControllerExtensionPaymentFirstdataRemote {
	public static function index() {
		load::language('extension/payment/firstdata_remote');

		if (config::get('firstdata_remote_card_storage') == 1 && customer::isLogged()) {
			$data['card_storage'] = 1;
			$data['stored_cards'] = extension_payment_firstdata_remote::getStoredCards();
		} else {
			$data['card_storage'] = 0;
			$data['stored_cards'] = array();
		}

		$data['accepted_cards'] = config::get('firstdata_remote_cards_accepted');

		$data['months'] = array();

		for ($i = 1; $i <= 12; $i++) {
			$data['months'][] = array(
				'text'  => sprintf('%02d', $i),
				'value' => sprintf('%02d', $i)
			);
		}

		$today = getdate();

		$data['year_expire'] = array();

		for ($i = $today['year']; $i < $today['year'] + 11; $i++) {
			$data['year_expire'][] = array(
				'text'  => sprintf('%02d', $i % 100),
				'value' => sprintf('%02d', $i % 100)
			);
		}

		return load::view('extension/payment/firstdata_remote', $data);
	}

	public static function send() {
		load::language('extension/payment/firstdata_remote');

		$address_codes = array(
			'PPX' => language::get('text_address_ppx'),
			'YYY' => language::get('text_address_yyy'),
			'YNA' => language::get('text_address_yna'),
			'NYZ' => language::get('text_address_nyz'),
			'NNN' => language::get('text_address_nnn'),
			'YPX' => language::get('text_address_ypx'),
			'PYX' => language::get('text_address_pyx'),
			'XXU' => language::get('text_address_xxu')
		);

		$cvv_codes = array(
			'M' => language::get('text_card_code_m'),
			'N' => language::get('text_card_code_n'),
			'P' => language::get('text_card_code_p'),
			'S' => language::get('text_card_code_s'),
			'U' => language::get('text_card_code_u'),
			'X' => language::get('text_card_code_x'),
			'NONE' => language::get('text_card_code_blank')
		);

		if (!isset(request::$post['cc_choice']) || (isset(request::$post['cc_choice']) && request::$post['cc_choice'] == 'new')) {
			if (request::$post['cc_number'] == '') {
				$json['error'] = language::get('error_card_number');
			}

			if (request::$post['cc_name'] == '') {
				$json['error'] = language::get('error_card_name');
			}
		}

		if (strlen(request::$post['cc_cvv2']) != 3 && strlen(request::$post['cc_cvv2']) != 4) {
			$json['error'] = language::get('error_card_cvv');
		}

		if (empty($json['error'])) {
			$order_id = session::$data['order_id'];
			$order_info = checkout_order::getOrder($order_id);

			$capture_result = extension_payment_firstdata_remote::capturePayment(request::$post, $order_id);

			$message = '';

			if (isset($capture_result['transaction_result']) && strtoupper($capture_result['transaction_result']) == 'APPROVED') {
				$json['success'] = url::link('checkout/success');

				$message .= language::get('text_result') . $capture_result['transaction_result'] . '<br />';
				$message .= language::get('text_avs') . $address_codes[$capture_result['avs']] . ' (' . $capture_result['avs'] . ')<br />';

				if (!empty($capture_result['ccv'])) {
					$message .= language::get('text_card_code_verify') . $cvv_codes[$capture_result['ccv']] . ' (' . $capture_result['ccv'] . ')<br />';
				}

				$message .= language::get('text_approval_code') . $capture_result['approval_code'] . '<br />';
				$message .= language::get('text_reference_number') . $capture_result['reference_number'] . '<br />';
				$message .= language::get('text_card_brand') . $capture_result['brand'] . '<br />';
				$message .= language::get('text_card_number_ref') . $capture_result['card_number_ref'] . '<br />';
				$message .= language::get('text_response_code') . $capture_result['response_code'] . '<br />';

				$fd_order_id = extension_payment_firstdata_remote::addOrder($order_info, $capture_result);

				if (config::get('firstdata_remote_auto_settle') == 1) {
					extension_payment_firstdata_remote::addTransaction($fd_order_id, 'payment', $order_info);

					checkout_order::addOrderHistory($order_id, config::get('firstdata_remote_order_status_success_settled_id'), $message, false);
				} else {
					extension_payment_firstdata_remote::addTransaction($fd_order_id, 'auth');

					checkout_order::addOrderHistory($order_id, config::get('firstdata_remote_order_status_success_unsettled_id'), $message, false);
				}
			} else {
				if (isset($capture_result['error']) && !empty($capture_result['error'])) {
					$json['error'] = $capture_result['error'];
				} else {
					$json['error'] = language::get('error_failed');
				}

				if (isset($capture_result['fault']) && !empty($capture_result['fault'])) {
					$message .= language::get('text_fault') . $capture_result['fault'] . '<br />';
				}

				$message .= language::get('text_result') . $capture_result['transaction_result'] . '<br />';
				$message .= language::get('text_error') . $capture_result['error'] . '<br />';
				$message .= language::get('text_card_brand') . $capture_result['brand'] . '<br />';
				$message .= language::get('text_card_number_ref') . $capture_result['card_number_ref'] . '<br />';

				extension_payment_firstdata_remote::addHistory($order_id, config::get('firstdata_remote_order_status_decline_id'), $message);
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}