<?php
class ControllerExtensionPaymentG2APay {
	public static function index() {
		load::language('extension/payment/g2apay');

		$data['action'] = url::link('extension/payment/g2apay/checkout', '');

		return load::view('extension/payment/g2apay', $data);
	}

	public static function checkout() {

		if(!isset(session::$data['order_id'])) {
			return false;
		}

		$order_info = checkout_order::getOrder(session::$data['order_id']);

		$order_data = array();


		$totals = array();
		$taxes = cart::getTaxes();
		$total = 0;

		// Because __call can not keep var references so we put them into an array.
		$total_data = array(
			'totals' => &$totals,
			'taxes'  => &$taxes,
			'total'  => &$total
		);

		$i = 0;

		$results = setting_extension::getExtensions('total');

		foreach ($results as $result) {
			if (config::get('total_' . $result['code'] . '_status')) {

				// We have to put the totals in an array so that they pass by reference.
				call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);

				if (isset($order_data['totals'][$i])) {
					if (strstr(strtolower($order_data['totals'][$i]['code']), 'total') === false) {
						$item = new stdClass();
						$item::$sku = $order_data['totals'][$i]['code'];
						$item::$name = $order_data['totals'][$i]['title'];
						$item::$amount = number_format($order_data['totals'][$i]['value'], 2);
						$item::$qty = 1;
						$item::$id = $order_data['totals'][$i]['code'];
						$item::$price = $order_data['totals'][$i]['value'];
						$item::$url = url::link('common/home', '');
						$items[] = $item;
					}

					$i++;
				}
			}
		}

		$ordered_products = account_order::getOrderProducts(session::$data['order_id']);

		foreach ($ordered_products as $product) {
			$item = new stdClass();
			$item::$sku = $product['product_id'];
			$item::$name = $product['name'];
			$item::$amount = $product['price'] * $product['quantity'];
			$item::$qty = $product['quantity'];
			$item::$id = $product['product_id'];
			$item::$price = $product['price'];
			$item::$url = url::link('product/product', 'product_id=' . $product['product_id']);
			$items[] = $item;
		}

		if (config::get('payment_g2apay_environment') == 1) {
			$url = 'https://checkout.pay.g2a.com/index/createQuote';
		} else {
			$url = 'https://checkout.test.pay.g2a.com/index/createQuote';
		}

		$order_total = number_format($order_info['total'], 2);

		$string = session::$data['order_id'] . $order_total . $order_info['currency_code'] . html_entity_decode(config::get('payment_g2apay_secret'));

		$fields = array(
			'api_hash' => config::get('payment_g2apay_api_hash'),
			'hash' => hash('sha256', $string),
			'order_id' => session::$data['order_id'],
			'amount' => $order_total,
			'currency' => $order_info['currency_code'],
			'email' => $order_info['email'],
			'url_failure' => url::link('checkout/failure'),
			'url_ok' => url::link('extension/payment/g2apay/success'),
			'items' => json_encode($items)
		);

		$response_data = extension_payment_g2apay::sendCurl($url, $fields);

		extension_payment_g2apay::logger($order_total);
		extension_payment_g2apay::logger($items);
		extension_payment_g2apay::logger($fields);

		if ($response_data === false) {
			response::redirect(url::link('checkout/failure', ''));
		}

		if (strtolower($response_data::$status) != 'ok') {
			response::redirect(url::link('checkout/failure', ''));
		}

		extension_payment_g2apay::addG2aOrder($order_info);

		if (config::get('payment_g2apay_environment') == 1) {
			response::redirect('https://checkout.pay.g2a.com/index/gateway?token=' . $response_data::$token);
		} else {
			response::redirect('https://checkout.test.pay.g2a.com/index/gateway?token=' . $response_data::$token);
		}
	}

	public static function success() {
		$order_id = session::$data['order_id'];

		if (isset(request::$post['transaction_id'])) {
			$g2apay_transaction_id = request::$post['transaction_id'];
		} elseif (isset(request::$get['transaction_id'])) {
			$g2apay_transaction_id = request::$get['transaction_id'];
		} else {
			$g2apay_transaction_id = '';
		}


		$order_info = checkout_order::getOrder($order_id);

		if ($order_info) {

			$g2apay_order_info = extension_payment_g2apay::getG2aOrder($order_id);

			extension_payment_g2apay::updateOrder($g2apay_order_info['g2apay_order_id'], $g2apay_transaction_id, 'payment', $order_info);

			checkout_order::addOrderHistory($order_id, config::get('payment_g2apay_order_status_id'));
		}

		response::redirect(url::link('checkout/success'));
	}

	public static function ipn() {
		extension_payment_g2apay::logger('ipn');

		if (isset(request::$get['token']) && hash_equals(config::get('payment_g2apay_secret_token'), request::$get['token'])) {
			extension_payment_g2apay::logger('token success');

			if (isset(request::$post['userOrderId'])) {
				$g2apay_order = extension_payment_g2apay::getG2aOrder(request::$post['userOrderId']);

				$string = $g2apay_order['g2apay_transaction_id'] . $g2apay_order['order_id'] . round($g2apay_order['total'], 2) . html_entity_decode(config::get('payment_g2apay_secret'));
				$hash = hash('sha256', $string);
				if($hash != request::$post['hash']){
					extension_payment_g2apay::logger('Hashes do not match, possible tampering!');
					return;
				}

				switch (request::$post['status']) {
					case 'complete':
						$order_status_id = config::get('payment_g2apay_complete_status_id');
						break;
					case 'rejected':
						$order_status_id = config::get('payment_g2apay_rejected_status_id');
						break;
					case 'canceled':
						$order_status_id = config::get('payment_g2apay_cancelled_status_id');
						break;
					case 'partial_refunded':
						$order_status_id = config::get('payment_g2apay_partially_refunded_status_id');
						break;
					case 'refunded':
						$order_status_id = config::get('payment_g2apay_refunded_status_id');
						break;
				}

				checkout_order::addOrderHistory(request::$post['userOrderId'], $order_status_id);
			}
		}
	}
}
