<?php
class ControllerExtensionPaymentLaybuy {
	public static function index() {
		load::language('extension/payment/laybuy');



		if(!isset(session::$data['order_id'])) {
			return false;
		}

		$data['action'] = url::link('extension/payment/laybuy/postToLaybuy', '');

		$order_info = checkout_order::getOrder(session::$data['order_id']);

		$data['order_info'] = $order_info;

		$data['total'] = $order_info['total'];

		$data['currency_symbol_left'] = currency::getSymbolLeft(session::$data['currency']);

		$data['currency_symbol_right'] = currency::getSymbolRight(session::$data['currency']);

		$data['initial_payments'] = extension_payment_laybuy::getInitialPayments();

		$data['months'] = extension_payment_laybuy::getMonths();

		return load::view('extension/payment/laybuy', $data);
	}

	public static function postToLaybuy()	{

		extension_payment_laybuy::log('Posting to Laybuy');

		if (request::$server['REQUEST_METHOD'] == 'POST') {

			$order_info = checkout_order::getOrder(session::$data['order_id']);

			if ($order_info) {
				extension_payment_laybuy::log('Order ID: ' . $order_info['order_id']);

				$data = array();

				$data['VERSION']      = '0.2';
				$data['MEMBER']       = config::get('payment_laybuys_membership_id');
				$data['RETURNURL']    = url::link('extension/payment/laybuy/callback', '');
				$data['CANCELURL']    = url::link('extension/payment/laybuy/cancel', '');
				$data['AMOUNT']       = round(floatval($order_info['total']), 2, PHP_ROUND_HALF_DOWN);
				$data['CURRENCY']     = $order_info['currency_code'];
				$data['INIT']         = (int)request::$post['INIT'];
				$data['MONTHS']       = (int)request::$post['MONTHS'];
				$data['MIND']         = ((int)config::get('payment_laybuy_min_deposit')) ? (int)config::get('payment_laybuy_min_deposit') : 20;
				$data['MAXD']         = ((int)config::get('payment_laybuy_max_deposit')) ? (int)config::get('payment_laybuy_max_deposit') : 50;
				$data['CUSTOM']       = $order_info['order_id'] . ':' . md5(config::get('payment_laybuy_token'));
				$data['EMAIL']        = $order_info['email'];

				$data_string = '';

				foreach ($data as $param => $value) {
					$data_string .= $param . '=' . $value . '&';
				}

				$data_string = rtrim($data_string, '&');

				extension_payment_laybuy::log('Data String: ' . $data_string);

				extension_payment_laybuy::log('Gateway URL: ' . config::get('payment_laybuy_gateway_url'));

				$ch = curl_init();
				curl_setopt($ch, CURLOPT_URL, config::get('payment_laybuy_gateway_url'));
				curl_setopt($ch, CURLOPT_POST, true);
				curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
				curl_setopt($ch, CURLOPT_HEADER, false);
				curl_setopt($ch, CURLOPT_TIMEOUT, 30);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
				$result = curl_exec($ch);
				if (curl_errno($ch)) {
					extension_payment_laybuy::log('cURL error: ' . curl_errno($ch));
				}
				curl_close($ch);

				$result = json_decode($result, true);

				extension_payment_laybuy::log('Response: ' . print_r($result, true));

			 	if (isset($result['ACK']) && isset($result['TOKEN']) && $result['ACK'] == 'SUCCESS') {
					extension_payment_laybuy::log('Success response. Redirecting to PayPal.');

					response::redirect(config::get('payment_laybuy_gateway_url') . '?TOKEN=' . $result['TOKEN']);
				} else {
					extension_payment_laybuy::log('Failure response. Redirecting to checkout/failure.');

					response::redirect(url::link('checkout/failure', ''));
				}
			} else {
				extension_payment_laybuy::log('No matching order. Redirecting to checkout/failure.');

				response::redirect(url::link('checkout/failure', ''));
			}
		} else {
			extension_payment_laybuy::log('No $_POST data. Redirecting to checkout/failure.');

			response::redirect(url::link('checkout/failure', ''));
		}
	}

	public static function callback() {

		extension_payment_laybuy::log('Receiving callback');

		if (request::$server['REQUEST_METHOD'] == 'POST' && isset(request::$post['RESULT']) && request::$post['RESULT'] == 'SUCCESS') {

			$custom = request::$post['CUSTOM'];

			$custom = explode(':', $custom);

			$order_id = $custom[0];

			$token = $custom[1];

			extension_payment_laybuy::log('Received Token: ' . $token);

			extension_payment_laybuy::log('Actual Token: ' . md5(config::get('payment_laybuy_token')));

			if (hash_equals(md5(config::get('payment_laybuy_token')), $token)) {
				extension_payment_laybuy::log('Order ID: ' . $order_id);

				$order_info = checkout_order::getOrder($order_id);

				if ($order_info) {
					checkout_order::addOrderHistory($order_id, config::get('payment_laybuy_order_status_id_pending'));

					$transaction_report = extension_payment_laybuy::prepareTransactionReport(request::$post);

					extension_payment_laybuy::addTransaction($transaction_report, 1);

					extension_payment_laybuy::log('Success. Redirecting to checkout/success.');

					response::redirect(url::link('checkout/success', ''));
				} else {
					extension_payment_laybuy::log('No matching order. Redirecting to checkout/failure.');

					response::redirect(url::link('checkout/failure', ''));
				}
			} else {
				extension_payment_laybuy::log('Token does not match. Redirecting to checkout/failure.');

				response::redirect(url::link('checkout/failure', ''));
			}
		} elseif (request::$server['REQUEST_METHOD'] == 'POST' && isset(request::$post['RESULT']) && request::$post['RESULT'] == 'FAILURE') {
			extension_payment_laybuy::log('Failure Response: ' . request::$post);

			extension_payment_laybuy::log('Redirecting to checkout/failure.');

			response::redirect(url::link('checkout/failure', ''));
		} else {
			extension_payment_laybuy::log('Either no $_POST data or unknown response. Redirecting to checkout/failure.');

			response::redirect(url::link('checkout/failure', ''));
		}
	}

	public static function cancel() {

		extension_payment_laybuy::log('Transaction canceled by user. Redirecting to checkout/checkout.');

		response::redirect(url::link('checkout/checkout', ''));
	}

	public static function reviseCallback() {

		load::language('extension/payment/laybuy');

		extension_payment_laybuy::log('Receiving callback');

		if (request::$server['REQUEST_METHOD'] == 'POST') {
			if (isset(request::$post['RESULT']) && request::$post['RESULT'] == 'SUCCESS') {

				$custom = request::$post['CUSTOM'];

				$custom = explode(':', $custom);

				$order_id = $custom[0];

				$token = $custom[1];

				extension_payment_laybuy::log('Received Token: ' . $token);

				extension_payment_laybuy::log('Actual Token: ' . md5(config::get('payment_laybuy_token')));

				if (hash_equals(md5(config::get('payment_laybuy_token')), $token)) {
					extension_payment_laybuy::log('Order ID: ' . $order_id);

					$order_info = checkout_order::getOrder($order_id);

					if ($order_info) {
						$response = request::$post;

						extension_payment_laybuy::log('Response: ' . print_r($response, true));

						$revised_transaction_id = $response['MERCHANTS_REF_NO'];

						$revised_transaction = extension_payment_laybuy::getRevisedTransaction($revised_transaction_id);

						extension_payment_laybuy::log('Revised transaction: ' . print_r($revised_transaction, true));

						$status = 1;

						$current_date = date('Y-m-d h:i:s');

						if (!isset($response['DOWNPAYMENT']) && !$revised_transaction['payment_type']) {
							extension_payment_laybuy::log('Buy-Now');

							$response['DOWNPAYMENT'] = 100;
							$response['MONTHS'] = 0;
							$response['DOWNPAYMENT_AMOUNT'] = $response['AMOUNT'];
							$response['PAYMENT_AMOUNTS'] = 0;
							$response['FIRST_PAYMENT_DUE'] = $current_date;
							$response['LAST_PAYMENT_DUE'] = $current_date;
							$response['PAYPAL_PROFILE_ID'] = '';

							$status = 5;

							checkout_order::addOrderHistory($order_id, config::get('payment_laybuy_order_status_id_processing'), language::get('text_comment'));
						} else {
							extension_payment_laybuy::log('Lay-Buy');
						}

						session::$data['order_id'] = $order_id;

						$transaction_report = extension_payment_laybuy::prepareTransactionReport($response);

						$transaction_report['order_id'] = $order_id;

						extension_payment_laybuy::addTransaction($transaction_report, $status);

						$old_transaction = extension_payment_laybuy::getTransaction($revised_transaction['laybuy_transaction_id']);

						$report_content = json_decode($old_transaction['report'], true);

						foreach ($report_content as &$array) {
							$array['status'] = str_replace('Pending', 'Canceled', $array['status']);
						}

						$report_content = json_encode($report_content);

						if ($old_transaction['paypal_profile_id']) {
							extension_payment_laybuy::log('Canceling transaction');

							$data_string = 'mid=' . config::get('payment_laybuys_membership_id') . '&' . 'paypal_profile_id=' . $old_transaction['paypal_profile_id'];

							extension_payment_laybuy::log('Data String: ' . $data_string);

							$ch = curl_init();
							$url = 'https://lay-buys.com/vtmob/deal5cancel.php';
							curl_setopt($ch, CURLOPT_URL, $url);
							curl_setopt($ch, CURLOPT_POST, true);
							curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
							curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
							curl_setopt($ch, CURLOPT_HEADER, false);
							curl_setopt($ch, CURLOPT_TIMEOUT, 30);
							curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
							$result = curl_exec($ch);
							if (curl_errno($ch)) {
								extension_payment_laybuy::log('cURL error: ' . curl_errno($ch));
							}
							curl_close($ch);

							extension_payment_laybuy::log('Response: ' . $result);

							if ($result == 'success') {
								extension_payment_laybuy::log('Success');
							} else {
								extension_payment_laybuy::log('Failure');
							}
						} else {
							extension_payment_laybuy::log('Transaction has no paypal_profile_id');
						}

						extension_payment_laybuy::updateTransaction($old_transaction['laybuy_transaction_id'], '51', $report_content, $old_transaction['transaction']);

						extension_payment_laybuy::deleteRevisedTransaction($revised_transaction['laybuy_revise_request_id']);

						response::redirect(url::link('checkout/success', ''));
					} else {
						extension_payment_laybuy::log('No matching order. Redirecting to checkout/failure.');

						response::redirect(url::link('checkout/failure', ''));
					}
				} else {
					extension_payment_laybuy::log('Token does not match. Redirecting to checkout/failure.');

					response::redirect(url::link('checkout/failure', ''));
				}
			} else {
				extension_payment_laybuy::log('No success response');

				response::redirect(url::link('checkout/failure', ''));
			}
		} else {
			extension_payment_laybuy::log('No $_POST data');

			response::redirect(url::link('checkout/failure', ''));
		}
	}

	public static function reviseCancel() {

		extension_payment_laybuy::log('Revise canceled. Redirecting to checkout/checkout.');

		response::redirect(url::link('checkout/checkout', ''));
	}

	public static function deleteOrder($route = '', $output = '', $order_id = 0, $order_status_id = 0) {

		if (isset(session::$data['api_id'])) {
			extension_payment_laybuy::log('Deleting order #' . $order_id);

			extension_payment_laybuy::deleteTransactionByOrderId($order_id);
		} else {
			extension_payment_laybuy::log('No API ID in session');
		}
	}

	public static function cron() {

		load::language('extension/payment/laybuy');

		extension_payment_laybuy::log('Running cron');

		if (isset(request::$get['token']) && hash_equals(config::get('payment_laybuy_token'), request::$get['token'])) {
			$paypal_profile_id_array = extension_payment_laybuy::getPayPalProfileIds();

			if ($paypal_profile_id_array) {
				$paypal_profile_ids = '';

				foreach ($paypal_profile_id_array as $profile_id) {
					$paypal_profile_ids .= $profile_id['paypal_profile_id'] . ',';
				}

				$paypal_profile_ids = rtrim($paypal_profile_ids, ',');

				$data_string = 'mid=' . config::get('payment_laybuys_membership_id') . '&' . 'profileIds=' . $paypal_profile_ids;

				extension_payment_laybuy::log('Data String: ' . $data_string);

				extension_payment_laybuy::log('API URL: ' . config::get('payment_laybuy_api_url'));

				$ch = curl_init();
				curl_setopt($ch, CURLOPT_URL, config::get('payment_laybuy_api_url'));
				curl_setopt($ch, CURLOPT_POST, true);
				curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
				curl_setopt($ch, CURLOPT_HEADER, false);
				curl_setopt($ch, CURLOPT_TIMEOUT, 30);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
				$result = curl_exec($ch);
				if (curl_errno($ch)) {
					extension_payment_laybuy::log('cURL error: ' . curl_errno($ch));
				}
				curl_close($ch);

				$results = json_decode($result, true);

				extension_payment_laybuy::log('Response: ' . print_r($results, true));

				if ($results) {

					foreach ($results as $laybuy_ref_id => $reports) {
						$status = $reports['status'];

						$report = $reports['report'];

						$transaction = array();

						$transaction = extension_payment_laybuy::getTransactionByLayBuyRefId($laybuy_ref_id);

						$order_id = $transaction['order_id'];

						$paypal_profile_id = $transaction['paypal_profile_id'];

						$months = $transaction['months'];

						$report_content = array();

						$pending_flag = false;

						$next_payment_status = language::get('text_status_1');

						foreach ($report as $month => $payment) {
							$payment['paymentDate'] = date('Y-m-d h:i:s', strtotime(str_replace('/', '-', $payment['paymentDate'])));
							$date = date(language::get('date_format_short'), strtotime($payment['paymentDate']));
							$next_payment_date = $payment['paymentDate'];

							if ($payment['type'] == 'd') {
								$report_content[] = array(
									'instalment'	=> 0,
									'amount'		=> currency::format($payment['amount'], $transaction['currency']),
									'date'			=> $date,
									'pp_trans_id'	=> $payment['txnID'],
									'status'		=> $payment['paymentStatus']
								);
							} elseif ($payment['type'] == 'p') {
								$pending_flag = true;

								$report_content[] = array(
									'instalment'	=> $month,
									'amount'		=> currency::format($payment['amount'], $transaction['currency']),
									'date'			=> $date,
									'pp_trans_id'	=> $payment['txnID'],
									'status'		=> $payment['paymentStatus']
								);

								$next_payment_status = $payment['paymentStatus'];
							}
						}

						if ($pending_flag) {
							$start_index = $month + 1;
						} else {
							$start_index = $month + 2;
						}

						if ($month < $months) {
							for ($month = 1; $month <= $months; $month++) {
								$next_payment_date = date("Y-m-d h:i:s", strtotime($next_payment_date . " +1 month"));
								$date = date(language::get('date_format_short'), strtotime($next_payment_date));

								$report_content[] = array(
									'instalment'	=> $month,
									'amount'		=> currency::format($transaction['payment_amounts'], $transaction['currency']),
									'date'			=> $date,
									'pp_trans_id'	=> '',
									'status'		=> $next_payment_status
								);
							}
						}

						$report_content = json_encode($report_content);

						switch ($status) {
							case -1: // Cancel
								extension_payment_laybuy::log('Transaction #' . $transaction['laybuy_transaction_id'] . ' canceled');
								checkout_order::addOrderHistory($order_id, config::get('payment_laybuy_order_status_id_canceled'), language::get('text_comment'), false, false);
								extension_payment_laybuy::updateTransaction($transaction['laybuy_transaction_id'], '7', $report_content, $start_index);
								break;
							case 0: // Pending
								extension_payment_laybuy::log('Transaction #' . $transaction['laybuy_transaction_id'] . ' still pending');
								extension_payment_laybuy::updateTransaction($transaction['laybuy_transaction_id'], $transaction['status'], $report_content, $start_index);
								break;
							case 1: // Paid
								extension_payment_laybuy::log('Transaction #' . $transaction['laybuy_transaction_id'] . ' paid');
								checkout_order::addOrderHistory($order_id, config::get('payment_laybuy_order_status_id_processing'), language::get('text_comment'), false, false);
								extension_payment_laybuy::updateTransaction($transaction['laybuy_transaction_id'], '5', $report_content, $start_index);
								break;
						}
					}
				}
			} else {
				extension_payment_laybuy::log('No PayPal Profile IDs to update');
			}

			extension_payment_laybuy::updateCronRunTime();
		} else {
			extension_payment_laybuy::log('Token does not match.');
		}
	}
}