<?php
// Nochex via form will work for both simple "Seller" account and "Merchant" account holders
// Nochex via APC maybe only avaiable to "Merchant" account holders only - site docs a bit vague on this point
class ControllerExtensionPaymentNochex {
	public static function index() {
		load::language('extension/payment/nochex');


		if(!isset(session::$data['order_id'])) {
			return false;
		}

		$order_info = checkout_order::getOrder(session::$data['order_id']);

		$data['action'] = 'https://secure.nochex.com/';

		// Nochex minimum requirements
		// The merchant ID is usually your Nochex registered email address but can be altered for "Merchant" accounts see below
		if (config::get('payment_nochex_email') != config::get('payment_nochex_merchant')) {
			// This MUST be changed on your Nochex account!!!!
			$data['merchant_id'] = config::get('payment_nochex_merchant');
		} else {
			$data['merchant_id'] = config::get('payment_nochex_email');
		}

		$data['amount'] = currency::format($order_info['total'], 'GBP', false, false);
		$data['order_id'] = session::$data['order_id'];
		$data['description'] = config('name');

		$data['billing_fullname'] = $order_info['payment_firstname'] . ' ' . $order_info['payment_lastname'];

		if ($order_info['payment_address_2']) {
			$data['billing_address']  = $order_info['payment_address_1'] . "\r\n" . $order_info['payment_address_2'] . "\r\n" . $order_info['payment_city'] . "\r\n" . $order_info['payment_zone'] . "\r\n";
		} else {
			$data['billing_address']  = $order_info['payment_address_1'] . "\r\n" . $order_info['payment_city'] . "\r\n" . $order_info['payment_zone'] . "\r\n";
		}

		$data['billing_postcode'] = $order_info['payment_postcode'];

		if (cart::hasShipping()) {
			$data['delivery_fullname'] = $order_info['shipping_firstname'] . ' ' . $order_info['shipping_lastname'];

			if ($order_info['shipping_address_2']) {
				$data['delivery_address'] = $order_info['shipping_address_1'] . "\r\n" . $order_info['shipping_address_2'] . "\r\n" . $order_info['shipping_city'] . "\r\n" . $order_info['shipping_zone'] . "\r\n";
			} else {
				$data['delivery_address'] = $order_info['shipping_address_1'] . "\r\n" . $order_info['shipping_city'] . "\r\n" . $order_info['shipping_zone'] . "\r\n";
			}

			$data['delivery_postcode'] = $order_info['shipping_postcode'];
		} else {
			$data['delivery_fullname'] = $order_info['payment_firstname'] . ' ' . $order_info['payment_lastname'];

			if ($order_info['payment_address_2']) {
				$data['delivery_address'] = $order_info['payment_address_1'] . "\r\n" . $order_info['payment_address_2'] . "\r\n" . $order_info['payment_city'] . "\r\n" . $order_info['payment_zone'] . "\r\n";
			} else {
				$data['delivery_address'] = $order_info['shipping_address_1'] . "\r\n" . $order_info['payment_city'] . "\r\n" . $order_info['payment_zone'] . "\r\n";
			}

			$data['delivery_postcode'] = $order_info['payment_postcode'];
		}

		$data['email_address'] = $order_info['email'];
		$data['customer_phone_number']= $order_info['telephone'];
		$data['test'] = config::get('payment_nochex_test');
		$data['success_url'] = url::link('checkout/success', '');
		$data['cancel_url'] = url::link('checkout/payment', '');
		$data['declined_url'] = url::link('extension/payment/nochex/callback', 'method=decline');
		$data['callback_url'] = url::link('extension/payment/nochex/callback', 'order=' . session::$data['order_id']);

		return load::view('extension/payment/nochex', $data);
	}

	public static function callback() {
		load::language('extension/payment/nochex');

		if (isset(request::$get['method']) && request::$get['method'] == 'decline') {
			session::$data['error'] = language::get('error_declined');

			response::redirect(url::link('checkout/cart'));
		}

		if (isset(request::$post['order_id'])) {
			$order_id = request::$post['order_id'];
		} else {
			$order_id = 0;
		}


		$order_info = checkout_order::getOrder($order_id);

		if (!$order_info) {
			session::$data['error'] = language::get('error_no_order');

			response::redirect(url::link('checkout/cart'));
		}

		// Fraud Verification Step.
		$request = '';

		foreach (request::$post as $key => $value) {
			$request .= '&' . $key . '=' . urlencode(stripslashes($value));
		}

		$curl = curl_init('https://www.nochex.com/nochex.dll/apc/apc');

		curl_setopt($curl, CURLOPT_POST, true);
		curl_setopt($curl, CURLOPT_POSTFIELDS, trim($request, '&'));
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_HEADER, false);
		curl_setopt($curl, CURLOPT_TIMEOUT, 30);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

		$response = curl_exec($curl);

		curl_close($curl);

		if (strcmp($response, 'AUTHORISED') == 0) {
			checkout_order::addOrderHistory($order_id, config::get('payment_nochex_order_status_id'));
		} else {
			checkout_order::addOrderHistory($order_id, config('order_status_id'), 'Auto-Verification step failed. Manually check the transaction.');
		}

		// Since it returned, the customer should see success.
		// It's up to the store owner to manually verify payment.
		response::redirect(url::link('checkout/success', ''));
	}
}