<?php
class ControllerExtensionPaymentPaypoint {
	public static function index() {
		$data['button_confirm'] = language::get('button_confirm');


		if(!isset(session::$data['order_id'])) {
			return false;
		}

		$order_info = checkout_order::getOrder(session::$data['order_id']);

		$data['merchant'] = config::get('payment_paypoint_merchant');
		$data['trans_id'] = session::$data['order_id'];
		$data['amount'] = currency::format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);

		if (config::get('payment_paypoint_password')) {
			$data['digest'] = md5(session::$data['order_id'] . currency::format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false) . config::get('payment_paypoint_password'));
		} else {
			$data['digest'] = '';
		}

		$data['bill_name'] = $order_info['payment_firstname'] . ' ' . $order_info['payment_lastname'];
		$data['bill_addr_1'] = $order_info['payment_address_1'];
		$data['bill_addr_2'] = $order_info['payment_address_2'];
		$data['bill_city'] = $order_info['payment_city'];
		$data['bill_state'] = $order_info['payment_zone'];
		$data['bill_post_code'] = $order_info['payment_postcode'];
		$data['bill_country'] = $order_info['payment_country'];
		$data['bill_tel'] = $order_info['telephone'];
		$data['bill_email'] = $order_info['email'];

		if (cart::hasShipping()) {
			$data['ship_name'] = $order_info['shipping_firstname'] . ' ' . $order_info['shipping_lastname'];
			$data['ship_addr_1'] = $order_info['shipping_address_1'];
			$data['ship_addr_2'] = $order_info['shipping_address_2'];
			$data['ship_city'] = $order_info['shipping_city'];
			$data['ship_state'] = $order_info['shipping_zone'];
			$data['ship_post_code'] = $order_info['shipping_postcode'];
			$data['ship_country'] = $order_info['shipping_country'];
		} else {
			$data['ship_name'] = '';
			$data['ship_addr_1'] = '';
			$data['ship_addr_2'] = '';
			$data['ship_city'] = '';
			$data['ship_state'] = '';
			$data['ship_post_code'] = '';
			$data['ship_country'] = '';
		}

		$data['currency'] = session::$data['currency'];
		$data['callback'] = url::link('extension/payment/paypoint/callback', '');

		switch (config::get('payment_paypoint_test')) {
			case 'live':
				$status = 'live';
				break;
			case 'successful':
			default:
				$status = 'true';
				break;
			case 'fail':
				$status = 'false';
				break;
		}

		$data['options'] = 'test_status=' . $status . ',dups=false,cb_post=false';

		return load::view('extension/payment/paypoint', $data);
	}

	public static function callback() {
		if (isset(request::$get['trans_id'])) {
			$order_id = request::$get['trans_id'];
		} else {
			$order_id = 0;
		}


		$order_info = checkout_order::getOrder($order_id);

		// Validate the request is from PayPoint
		if (config::get('payment_paypoint_password')) {
			if (!empty(request::$get['hash'])) {
				$status = (request::$get['hash'] == md5(str_replace('hash=' . request::$get['hash'], '', htmlspecialchars_decode(request::$server['REQUEST_URI'], ENT_COMPAT)) . config::get('payment_paypoint_password')));
			} else {
				$status = false;
			}
		} else {
			$status = true;
		}

		if ($order_info) {
			load::language('extension/payment/paypoint');

			$data['title'] = sprintf(language::get('heading_title'), config('name'));

			if (!request::$server['HTTPS']) {
				$data['base'] = HTTP_SERVER;
			} else {
				$data['base'] = HTTPS_SERVER;
			}

			$data['language'] = language::get('code');
			$data['direction'] = language::get('direction');

			$data['heading_title'] = sprintf(language::get('heading_title'), config('name'));

			$data['text_success_wait'] = sprintf(language::get('text_success_wait'), url::link('checkout/success'));
			$data['text_failure_wait'] = sprintf(language::get('text_failure_wait'), url::link('checkout/cart'));

			if (isset(request::$get['code']) && request::$get['code'] == 'A' && $status) {
				$message = '';

				if (isset(request::$get['code'])) {
					$message .= 'code: ' . request::$get['code'] . "\n";
				}

				if (isset(request::$get['auth_code'])) {
					$message .= 'auth_code: ' . request::$get['auth_code'] . "\n";
				}

				if (isset(request::$get['ip'])) {
					$message .= 'ip: ' . request::$get['ip'] . "\n";
				}

				if (isset(request::$get['cv2avs'])) {
					$message .= 'cv2avs: ' . request::$get['cv2avs'] . "\n";
				}

				if (isset(request::$get['valid'])) {
					$message .= 'valid: ' . request::$get['valid'] . "\n";
				}


				checkout_order::addOrderHistory($order_id, config::get('payment_paypoint_order_status_id'), $message, false);

				$data['continue'] = url::link('checkout/success');

				$data['column_left'] = load::controller('common/column_left');
				$data['column_right'] = load::controller('common/column_right');
				$data['content_top'] = load::controller('common/content_top');
				$data['content_bottom'] = load::controller('common/content_bottom');
				$data['footer'] = load::controller('common/footer');
				$data['header'] = load::controller('common/header');

				response::setOutput(load::view('extension/payment/paypoint_success', $data));
			} else {
				$data['continue'] = url::link('checkout/cart');

				$data['column_left'] = load::controller('common/column_left');
				$data['column_right'] = load::controller('common/column_right');
				$data['content_top'] = load::controller('common/content_top');
				$data['content_bottom'] = load::controller('common/content_bottom');
				$data['footer'] = load::controller('common/footer');
				$data['header'] = load::controller('common/header');

				response::setOutput(load::view('extension/payment/paypoint_failure', $data));
			}
		}
	}
}