<?php
class ControllerExtensionPaymentPPBraintree {
	private static $customer_id_prefix = 'braintree_oc_';
	private static $gateway = null;

	public static function index() {
		self::initialise();

		load::language('extension/payment/pp_braintree');

		$data['payment_url'] = url::link('extension/payment/pp_braintree/payment', '');
		$data['vaulted_url'] = url::link('extension/payment/pp_braintree/vaulted', '');

		$data['payment_pp_braintree_3ds_status'] = config::get('payment_pp_braintree_3ds_status');
		$data['payment_pp_braintree_vault_cvv_3ds'] = config::get('payment_pp_braintree_vault_cvv_3ds');
		$data['payment_pp_braintree_paypal_option'] = config::get('payment_pp_braintree_paypal_option');
		$data['payment_pp_braintree_vault_cvv'] = config::get('payment_pp_braintree_vault_cvv');
		$data['payment_pp_braintree_settlement_immediate'] = config::get('payment_pp_braintree_settlement_immediate');
		$data['payment_pp_braintree_paypal_button_colour'] = config::get('payment_pp_braintree_paypal_button_colour');
		$data['payment_pp_braintree_paypal_button_size'] = config::get('payment_pp_braintree_paypal_button_size');
		$data['payment_pp_braintree_paypal_button_shape'] = config::get('payment_pp_braintree_paypal_button_shape');

		if (!isset(session::$data['order_id'])) {
			return false;
		}


		$order_info = checkout_order::getOrder(session::$data['order_id']);

		$create_token = array();
		$merchant_id = config::get('payment_pp_braintree_merchant_id');

		if (self::$gateway == '') {
			$merchant_accounts = config::get('payment_pp_braintree_account');

			foreach ($merchant_accounts as $merchant_account_currency => $merchant_account) {
				if (($merchant_account_currency == $order_info['currency_code']) && !empty($merchant_account['merchant_account_id'])) {
					$create_token['merchantAccountId'] = $merchant_account['merchant_account_id'];

					$merchant_id = $merchant_account['merchant_account_id'];

					break;
				}
			}
		}

		$data['merchant_id'] = $merchant_id;

		if (customer::isLogged() && (config::get('payment_pp_braintree_card_vault') || config::get('payment_pp_braintree_paypal_vault'))) {
			$data['payment_pp_braintree_card_vault'] = config::get('payment_pp_braintree_card_vault');
			$data['payment_pp_braintree_paypal_vault'] = config::get('payment_pp_braintree_paypal_vault');
			$data['payment_pp_braintree_card_check_vault'] = config::get('payment_pp_braintree_card_check_vault');
			$data['payment_pp_braintree_paypal_check_vault'] = config::get('payment_pp_braintree_paypal_check_vault');
			$vaulted_customer_info = extension_payment_pp_braintree::getCustomer(self::$gateway, self::$customer_id_prefix . customer::getId(), false);
		} else {
			$data['payment_pp_braintree_card_vault'] = 0;
			$data['payment_pp_braintree_paypal_vault'] = 0;
			$data['payment_pp_braintree_card_check_vault'] = 0;
			$data['payment_pp_braintree_paypal_check_vault'] = 0;
			$vaulted_customer_info = false;
		}

		$data['client_token'] = extension_payment_pp_braintree::generateToken(self::$gateway, $create_token);
		$data['total'] = currency::format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);

		$data['currency_code'] = $order_info['currency_code'];

		// disable paypal option if currency is not in supported array
		if (!in_array($order_info['currency_code'], array('USD', 'EUR', 'GBP', 'CAD', 'AUD', 'DKK', 'NOK', 'PLN', 'SEK', 'CHF', 'TRY'))) {
			$data['payment_pp_braintree_paypal_option'] = false;
		}

		// pass shipping info to paypal if set
		if ($data['payment_pp_braintree_paypal_option'] && cart::hasShipping()) {
			$data['customer_shipping_address'] = array(
				'name'			=> addslashes($order_info['shipping_firstname']) . ' ' . addslashes($order_info['shipping_lastname']),
				'line_1'		=> addslashes($order_info['shipping_address_1']),
				'line_2'		=> addslashes($order_info['shipping_address_2']),
				'city'			=> addslashes($order_info['shipping_city']),
				'state'			=> addslashes($order_info['shipping_zone_code']),
				'post_code'		=> addslashes($order_info['shipping_postcode']),
				'country_code' 	=> addslashes($order_info['shipping_iso_code_2']),
				'phone'			=> addslashes($order_info['telephone']),
			);
		}

		$vaulted_payment_methods = array('cards', 'paypal');
		$vaulted_payment_count = 0;

		if ($vaulted_customer_info) {
			$vaulted_card_count = 0;
			$vaulted_paypal_count = 0;

			if ($vaulted_customer_info::$creditCards && config::get('payment_pp_braintree_card_vault') == 1) {
				$vaulted_card_count = count($vaulted_customer_info::$creditCards);

				foreach ($vaulted_customer_info::$creditCards as $credit_card) {
					$vaulted_payment_methods['cards'][] = array(
						'image'	  => $credit_card::$imageUrl,
						'name'	  => sprintf(language::get('text_vaulted_payment_method_name'), $credit_card::$cardType, $credit_card::$last4, $credit_card::$expirationDate),
						'token'	  => $credit_card::$token,
						'expired' => $credit_card::$expired,
						'default' => $credit_card::$default
					);
				}
			}

			if ($vaulted_customer_info::$paypalAccounts && config::get('payment_pp_braintree_paypal_vault') == 1) {
				$vaulted_paypal_count = count($vaulted_customer_info::$paypalAccounts);

				foreach ($vaulted_customer_info::$paypalAccounts as $paypal_account) {
					$vaulted_payment_methods['paypal'][] = array(
						'image'	  => $paypal_account::$imageUrl,
						'name'	  => $paypal_account::$email,
						'token'	  => $paypal_account::$token,
						'default' => $paypal_account::$default
					);
				}
			}

			$vaulted_payment_count = $vaulted_card_count + $vaulted_paypal_count;
		}

		$data['vaulted_payment_methods'] = $vaulted_payment_methods;
		$data['vaulted_payment_count'] = $vaulted_payment_count;

		$data['form_styles'] = json_encode("{
		  'input': { 'font-size': '12px', 'font-family': 'Source Sans Pro, sans-serif', 'color': '#7A8494' },
		  'input.invalid': { 'color': 'red' },
		  'input.valid': { 'color': 'green' }
	  	}");

		if (customer::isLogged()) {
			$data['guest'] = false;
		} else {
			$data['guest'] = true;
		}

		return load::view('extension/payment/pp_braintree', $data);
	}

	public static function payment() {
		//set_time_limit(120);

		self::initialise();

		load::language('extension/payment/pp_braintree');


		extension_payment_pp_braintree::log('Starting payment');
		extension_payment_pp_braintree::log(request::$post);

		$success = true;

		if (!isset(session::$data['order_id'])) {
			extension_payment_pp_braintree::log('Session data: order_id not found');

			$success = false;
		}

		if (isset(request::$post['device_data'])) {
			$device_data = request::$post['device_data'];
		} else {
			extension_payment_pp_braintree::log('Post data: device_data not found');

			$device_data = '';

			$success = false;
		}

		if (isset(request::$post['payment_method_token'])) {
			$payment_method_token = request::$post['payment_method_token'];
		} else {
			extension_payment_pp_braintree::log('Post data: payment_method_token not found');
			$payment_method_token = '';
		}

		if (isset(request::$post['payment_method_nonce'])) {
			$payment_method_nonce = request::$post['payment_method_nonce'];
		} else {
			extension_payment_pp_braintree::log('Post data: payment_method_nonce not found');
			$payment_method_nonce = '';
		}

		if ($payment_method_nonce == '' && $payment_method_token == '') {
			$success = false;
		}

		//Start creating transaction array
		if ($success) {
			$order_info = checkout_order::getOrder(session::$data['order_id']);

			$create_sale = array(
				'amount'			 => currency::format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false),
				'channel'			 => 'OpenCart_Cart_vzero',
				'orderId'			 => $order_info['order_id'],
				'deviceData'		 => $device_data,
				'customer'           => array(
					'firstName' => $order_info['firstname'],
					'lastName'  => $order_info['lastname'],
					'phone'		=> $order_info['telephone'],
					'email'		=> $order_info['email']
				),
				'billing'			=> array(
					'firstName'			=> $order_info['payment_firstname'],
					'lastName'			=> $order_info['payment_lastname'],
					'company'			=> $order_info['payment_company'],
					'streetAddress'		=> $order_info['payment_address_1'],
					'extendedAddress'	=> $order_info['payment_address_2'],
					'locality'			=> $order_info['payment_city'],
					'countryCodeAlpha2' => $order_info['payment_iso_code_2'],
					'postalCode'		=> $order_info['payment_postcode'],
					'region'			=> $order_info['payment_zone_code']
				),
				'options' => array('three_d_secure' => array('required' => false))
			);

			//Add shipping details
			if (cart::hasShipping()) {
				$create_sale['shipping'] = array(
					'firstName'			=> $order_info['shipping_firstname'],
					'lastName'			=> $order_info['shipping_lastname'],
					'company'			=> $order_info['shipping_company'],
					'streetAddress'		=> $order_info['shipping_address_1'],
					'extendedAddress'	=> $order_info['shipping_address_2'],
					'locality'			=> $order_info['shipping_city'],
					'countryCodeAlpha2' => $order_info['shipping_iso_code_2'],
					'postalCode'		=> $order_info['shipping_postcode'],
					'region'			=> $order_info['shipping_zone_code']
				);
			}

			if (customer::isLogged() && (config::get('payment_pp_braintree_card_vault') || config::get('payment_pp_braintree_paypal_vault'))) {
				$customer_id = self::$customer_id_prefix . customer::getId();

				$vaulted_customer_info = extension_payment_pp_braintree::getCustomer(self::$gateway, $customer_id, false);

				if ($vaulted_customer_info) {
					$create_sale['customerId'] = $customer_id;
				} else {
					$create_sale['customer']['id'] = $customer_id;
				}

				if (isset(request::$post['vault_method']) && request::$post['vault_method'] == '1') {
					$create_sale['options']['storeInVaultOnSuccess'] = true;
				}
			}

			if ($payment_method_token != '') {
				$create_sale['paymentMethodToken'] = $payment_method_token;

				// unset the billing info for a vaulted payment
				$create_sale['billing'] = array();
			}

			if ($payment_method_nonce != '') {
				$create_sale['paymentMethodNonce'] = $payment_method_nonce;
			}

			if (self::$gateway == '') {
				$merchant_accounts = config::get('payment_pp_braintree_account');

				foreach ($merchant_accounts as $merchant_account_currency => $merchant_account) {
					if (($merchant_account_currency == $order_info['currency_code']) && !empty($merchant_account['merchant_account_id'])) {
						$create_sale['merchantAccountId'] = $merchant_account['merchant_account_id'];
					}
				}
			}

			if (config::get('payment_pp_braintree_settlement_immediate') == 1) {
				$create_sale['options']['submitForSettlement'] = true;
			} else {
				$create_sale['options']['submitForSettlement'] = false;
			}
		}

		// If the $payment_method_token is not empty it indicates the vaulted payment used CVV or was set to none
		if ($success && ((config::get('payment_pp_braintree_3ds_status') == 1 && $payment_method_token == '') || (config::get('payment_pp_braintree_vault_cvv_3ds') == '3ds' && $payment_method_token != ''))) {
			$nonce_info = extension_payment_pp_braintree::getPaymentMethodNonce(self::$gateway, $payment_method_nonce);

			extension_payment_pp_braintree::log($nonce_info);

			if ($nonce_info::$type == 'CreditCard' && config::get('payment_pp_braintree_3ds_status') == 1) {
				$create_sale['options']['three_d_secure'] = array(
					'required' => true
				);

				$three_ds_info = array();

				if (isset($nonce_info::$threeDSecureInfo) && !empty($nonce_info::$threeDSecureInfo)) {
					$three_ds_info = $nonce_info::$threeDSecureInfo;
				}

				if (!empty($three_ds_info)) {
					$success = false;

					switch ($three_ds_info::$status) {
						case 'unsupported_card':
							if ($nonce_info::$details['cardType'] == 'American Express') {
								$success = true;
							} else {
								$success = config::get('payment_pp_braintree_3ds_unsupported_card');
							}
							break;
						case 'lookup_error':
							$success = config::get('payment_pp_braintree_3ds_lookup_error');
							break;
						case 'lookup_enrolled':
							$success = config::get('payment_pp_braintree_3ds_lookup_enrolled');
							break;
						case 'lookup_not_enrolled':
							$success = config::get('payment_pp_braintree_3ds_lookup_not_enrolled');
							break;
						case 'authenticate_successful_issuer_not_participating':
							$success = config::get('payment_pp_braintree_3ds_not_participating');
							break;
						case 'authentication_unavailable':
							$success = config::get('payment_pp_braintree_3ds_unavailable');
							break;
						case 'authenticate_signature_verification_failed':
							$success = config::get('payment_pp_braintree_3ds_signature_failed');
							break;
						case 'authenticate_successful':
							$success = config::get('payment_pp_braintree_3ds_successful');
							break;
						case 'authenticate_attempt_successful':
							$success = config::get('payment_pp_braintree_3ds_attempt_successful');
							break;
						case 'authenticate_failed':
							$success = config::get('payment_pp_braintree_3ds_failed');
							break;
						case 'authenticate_unable_to_authenticate':
							$success = config::get('payment_pp_braintree_3ds_unable_to_auth');
							break;
						case 'authenticate_error':
							$success = config::get('payment_pp_braintree_3ds_error');
							break;
					}
				} else {
					extension_payment_pp_braintree::log('Liability shift failed, nonce was not 3D Secured');

					$success = false;
				}
			}
		}
		extension_payment_pp_braintree::log("Success:" . (int)$success);

		//Create transaction
		if ($success) {
			$transaction = extension_payment_pp_braintree::addTransaction(self::$gateway, $create_sale);

			$order_status_id = 0;
			switch ($transaction::$transaction::$status) {
				case 'authorization_expired':
					$order_status_id = config::get('payment_pp_braintree_authorization_expired_id');
					break;
				case 'authorized':
					$order_status_id = config::get('payment_pp_braintree_authorized_id');
					break;
				case 'authorizing':
					$order_status_id = config::get('payment_pp_braintree_authorizing_id');
					break;
				case 'settlement_pending':
					$order_status_id = config::get('payment_pp_braintree_settlement_pending_id');
					break;
				case 'failed':
					$order_status_id = config::get('payment_pp_braintree_failed_id');
					break;
				case 'gateway_rejected':
					$order_status_id = config::get('payment_pp_braintree_gateway_rejected_id');
					break;
				case 'processor_declined':
					$order_status_id = config::get('payment_pp_braintree_processor_declined_id');
					break;
				case 'settled':
					$order_status_id = config::get('payment_pp_braintree_settled_id');
					break;
				case 'settling':
					$order_status_id = config::get('payment_pp_braintree_settling_id');
					break;
				case 'submitted_for_settlement':
					$order_status_id = config::get('payment_pp_braintree_submitted_for_settlement_id');
					break;
				case 'voided':
					$order_status_id = config::get('payment_pp_braintree_voided_id');
					break;
			}

			checkout_order::addOrderHistory(session::$data['order_id'], $order_status_id);

			if ($transaction::$success) {
				extension_payment_pp_braintree::log('Transaction success, details below');
				extension_payment_pp_braintree::log($transaction);

				response::redirect(url::link('checkout/success', ''));
			} else {
				extension_payment_pp_braintree::log('Transaction failed, details below');
				extension_payment_pp_braintree::log($transaction);

				session::$data['error'] = language::get('error_process_order');
				response::redirect(url::link('checkout/checkout', ''));
			}
		}

		//If this is reached, transaction has failed
		extension_payment_pp_braintree::log('Transaction reached end of method without being handled, failure');

		if (isset(session::$data['order_id'])) {
			checkout_order::addOrderHistory(session::$data['order_id'], config::get('payment_pp_braintree_failed_id'));
		}

		response::redirect(url::link('checkout/failure', ''));
	}

	public static function nonce() {
		self::initialise();

		load::language('extension/payment/pp_braintree');


		extension_payment_pp_braintree::log('Starting vaulted');
		extension_payment_pp_braintree::log(request::$post);

		$json = array();

		$json['payment_method'] = '';

		$success = true;

		if (!isset(request::$post['vaulted_payment_token'])) {
			$success = false;
		}

		if ($success) {
			$payment_method = extension_payment_pp_braintree::createPaymentMethodNonce(self::$gateway, request::$post['vaulted_payment_token']);

			if ($payment_method && $payment_method::$success) {
				$json['payment_method'] = array(
					'type'  => $payment_method::$paymentMethodNonce::$type,
					'nonce' => $payment_method::$paymentMethodNonce::$nonce
				);
			} else {
				$success = false;
			}
		}

		$json['success'] = $success;

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function remove() {
		self::initialise();

		load::language('extension/payment/pp_braintree');


		$json = array();

		$json['success'] = false;

		if (isset(request::$post['vaulted_payment_method'])) {
			$vaulted_payment_method = request::$post['vaulted_payment_method'];
		} else {
			$vaulted_payment_method = '';
		}

		$delete_payment_method = extension_payment_pp_braintree::deletePaymentMethod(self::$gateway, $vaulted_payment_method);

		if ($delete_payment_method) {
			$json['success'] = language::get('text_method_removed');
		} else {
			$json['error'] = language::get('text_method_not_removed');
		}

		$vaulted_customer_info = extension_payment_pp_braintree::getCustomer(self::$gateway, self::$customer_id_prefix . customer::getId());

		$vaulted_card_count = 0;
		$vaulted_paypal_count = 0;

		if ($vaulted_customer_info::$creditCards && config::get('payment_pp_braintree_card_vault') == 1) {
			$vaulted_card_count = count($vaulted_customer_info::$creditCards);
		}

		if ($vaulted_customer_info::$paypalAccounts && config::get('payment_pp_braintree_paypal_vault') == 1) {
			$vaulted_paypal_count = count($vaulted_customer_info::$paypalAccounts);
		}

		$json['vaulted_payment_count'] = $vaulted_card_count + $vaulted_paypal_count;

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function expressSetup() {
		// check checkout can continue due to stock checks or vouchers
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			$json = array();
			$json['error'] = true;
			$json['url'] = url::link('checkout/cart');

			response::addHeader('Content-Type: application/json');
			response::setOutput(json_encode($json));
		}

		// if user not logged in check that the guest checkout is allowed
		if (!customer::isLogged() && (!config('checkout_guest') || config('customer_price') || cart::hasDownload() || cart::hasRecurringProducts())) {
			$json = array();
			$json['error'] = true;
			$json['url'] = url::link('checkout/checkout');

			response::addHeader('Content-Type: application/json');
			response::setOutput(json_encode($json));
		} else {
			unset(session::$data['guest']);
		}

		unset(session::$data['shipping_method']);
		unset(session::$data['shipping_methods']);
		unset(session::$data['payment_method']);
		unset(session::$data['payment_methods']);

		if (!customer::isLogged()) {
			session::$data['paypal_braintree']['guest'] = true;

			session::$data['guest']['customer_group_id'] = config('customer_group_id');
			session::$data['guest']['firstname'] = request::$post['details']['firstName'];
			session::$data['guest']['lastname'] = request::$post['details']['lastName'];
			session::$data['guest']['email'] = request::$post['details']['email'];

			if (isset(request::$post['details']['phone'])) {
				session::$data['guest']['telephone'] = request::$post['details']['phone'];
			} else {
				session::$data['guest']['telephone'] = '';
			}

			session::$data['guest']['payment']['company'] = '';

			session::$data['guest']['payment']['firstname'] = request::$post['details']['firstName'];
			session::$data['guest']['payment']['lastname'] = request::$post['details']['lastName'];

			session::$data['guest']['payment']['company_id'] = '';
			session::$data['guest']['payment']['tax_id'] = '';

			if (cart::hasShipping()) {
				$shipping_name = explode(' ', request::$post['details']['shippingAddress']['recipientName']);
				$shipping_first_name = $shipping_name[0];
				unset($shipping_name[0]);
				$shipping_last_name = implode(' ', $shipping_name);

				session::$data['guest']['payment']['address_1'] = request::$post['details']['shippingAddress']['line1'];
				if (isset(request::$post['details']['shippingAddress']['line2'])) {
					session::$data['guest']['payment']['address_2'] = request::$post['details']['shippingAddress']['line2'];
				} else {
					session::$data['guest']['payment']['address_2'] = '';
				}

				session::$data['guest']['payment']['postcode'] = request::$post['details']['shippingAddress']['postalCode'];
				session::$data['guest']['payment']['city'] = request::$post['details']['shippingAddress']['city'];

				session::$data['guest']['shipping']['firstname'] = $shipping_first_name;
				session::$data['guest']['shipping']['lastname'] = $shipping_last_name;
				session::$data['guest']['shipping']['company'] = '';
				session::$data['guest']['shipping']['address_1'] = request::$post['details']['shippingAddress']['line1'];

				if (isset(request::$post['details']['shippingAddress']['line2'])) {
					session::$data['guest']['shipping']['address_2'] =request::$post['details']['shippingAddress']['line2'];
				} else {
					session::$data['guest']['shipping']['address_2'] = '';
				}

				session::$data['guest']['shipping']['postcode'] = request::$post['details']['shippingAddress']['postalCode'];
				session::$data['guest']['shipping']['city'] = request::$post['details']['shippingAddress']['city'];

				session::$data['shipping_postcode'] = request::$post['details']['shippingAddress']['postalCode'];

				$country_info = db::query("SELECT * FROM `country` WHERE `iso_code_2` = " . esc(request::$post['details']['shippingAddress']['countryCode']) . " AND `status` = '1' LIMIT 1")->row;

				if ($country_info) {
					session::$data['guest']['shipping']['country_id'] = $country_info['country_id'];
					session::$data['guest']['shipping']['country'] = $country_info['name'];
					session::$data['guest']['shipping']['iso_code_2'] = $country_info['iso_code_2'];
					session::$data['guest']['shipping']['iso_code_3'] = $country_info['iso_code_3'];
					session::$data['guest']['shipping']['address_format'] = $country_info['address_format'];
					session::$data['guest']['payment']['country_id'] = $country_info['country_id'];
					session::$data['guest']['payment']['country'] = $country_info['name'];
					session::$data['guest']['payment']['iso_code_2'] = $country_info['iso_code_2'];
					session::$data['guest']['payment']['iso_code_3'] = $country_info['iso_code_3'];
					session::$data['guest']['payment']['address_format'] = $country_info['address_format'];
					session::$data['shipping_country_id'] = $country_info['country_id'];

					if (isset(request::$post['details']['shippingAddress']['state'])) {
						$returned_shipping_zone = request::$post['details']['shippingAddress']['state'];
					} else {
						$returned_shipping_zone = '';
					}

					$zone_info = db::query("SELECT * FROM `zone` WHERE (`name` = " . esc($returned_shipping_zone) . " OR `code` = " . esc($returned_shipping_zone) . ") AND `status` = '1' AND `country_id` = " . (int)$country_info['country_id'] . " LIMIT 1")->row;
				} else {
					session::$data['guest']['shipping']['country_id'] = '';
					session::$data['guest']['shipping']['country'] = '';
					session::$data['guest']['shipping']['iso_code_2'] = '';
					session::$data['guest']['shipping']['iso_code_3'] = '';
					session::$data['guest']['shipping']['address_format'] = '';
					session::$data['guest']['payment']['country_id'] = '';
					session::$data['guest']['payment']['country'] = '';
					session::$data['guest']['payment']['iso_code_2'] = '';
					session::$data['guest']['payment']['iso_code_3'] = '';
					session::$data['guest']['payment']['address_format'] = '';
					session::$data['shipping_country_id'] = '';

					$zone_info = array();
				}

				if ($zone_info) {
					session::$data['guest']['shipping']['zone'] = $zone_info['name'];
					session::$data['guest']['shipping']['zone_code'] = $zone_info['code'];
					session::$data['guest']['shipping']['zone_id'] = $zone_info['zone_id'];
					session::$data['guest']['payment']['zone'] = $zone_info['name'];
					session::$data['guest']['payment']['zone_code'] = $zone_info['code'];
					session::$data['guest']['payment']['zone_id'] = $zone_info['zone_id'];
					session::$data['shipping_zone_id'] = $zone_info['zone_id'];
				} else {
					session::$data['guest']['shipping']['zone'] = '';
					session::$data['guest']['shipping']['zone_code'] = '';
					session::$data['guest']['shipping']['zone_id'] = '';
					session::$data['guest']['payment']['zone'] = '';
					session::$data['guest']['payment']['zone_code'] = '';
					session::$data['guest']['payment']['zone_id'] = '';
					session::$data['shipping_zone_id'] = '';
				}

				session::$data['guest']['shipping_address'] = true;
			} else {
				session::$data['guest']['payment']['address_1'] = '';
				session::$data['guest']['payment']['address_2'] = '';
				session::$data['guest']['payment']['postcode'] = '';
				session::$data['guest']['payment']['city'] = '';
				session::$data['guest']['payment']['country_id'] = '';
				session::$data['guest']['payment']['country'] = '';
				session::$data['guest']['payment']['iso_code_2'] = '';
				session::$data['guest']['payment']['iso_code_3'] = '';
				session::$data['guest']['payment']['address_format'] = '';
				session::$data['guest']['payment']['zone'] = '';
				session::$data['guest']['payment']['zone_code'] = '';
				session::$data['guest']['payment']['zone_id'] = '';
				session::$data['guest']['shipping_address'] = false;
			}

			session::$data['account'] = 'guest';

			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
			unset(session::$data['payment_method']);
			unset(session::$data['payment_methods']);
		} else {
			session::$data['paypal_braintree']['guest'] = false;

			unset(session::$data['guest']);
			/**
			 * if the user is logged in, add the address to the account and set the ID.
			 */

			if (cart::hasShipping()) {

				$addresses = account_address::getAddresses();

				/**
				 * Compare all of the user addresses and see if there is a match
				 */
				$match = false;
				foreach($addresses as $address) {
					if (trim(strtolower($address['address_1'])) == trim(strtolower(request::$post['details']['shippingAddress']['line1'])) && trim(strtolower($address['postcode'])) == trim(strtolower(request::$post['details']['shippingAddress']['postalCode']))) {
						$match = true;

						session::$data['payment_address_id'] = $address['address_id'];
						session::$data['payment_country_id'] = $address['country_id'];
						session::$data['payment_zone_id'] = $address['zone_id'];

						session::$data['shipping_address_id'] = $address['address_id'];
						session::$data['shipping_country_id'] = $address['country_id'];
						session::$data['shipping_zone_id'] = $address['zone_id'];
						session::$data['shipping_postcode'] = $address['postcode'];

						break;
					}
				}

				/**
				 * If there is no address match add the address and set the info.
				 */
				if ($match == false) {
					$shipping_name = explode(' ', trim(request::$post['details']['shippingAddress']['recipientName']));
					$shipping_first_name = $shipping_name[0];
					unset($shipping_name[0]);
					$shipping_last_name = implode(' ', $shipping_name);

					$country_info = db::query("SELECT * FROM `country` WHERE `iso_code_2` = " . esc(request::$post['details']['shippingAddress']['countryCode']) . " AND `status` = '1' LIMIT 1")->row;
					$zone_info = db::query("SELECT * FROM `zone` WHERE (`name` = " . esc(request::$post['details']['shippingAddress']['state']) . " OR `code` = " . esc(request::$post['details']['shippingAddress']['state']) . ") AND `status` = '1' AND `country_id` = " . (int)$country_info['country_id'])->row;

					$address_data = array(
						'firstname'  => $shipping_first_name,
						'lastname'   => $shipping_last_name,
						'company'    => '',
						'company_id' => '',
						'tax_id'     => '',
						'address_1'  => request::$post['details']['shippingAddress']['line1'],
						'address_2'  => (isset(request::$post['details']['shippingAddress']['line2']) ? request::$post['details']['shippingAddress']['line2'] : ''),
						'postcode'   => request::$post['details']['shippingAddress']['postalCode'],
						'city'       => request::$post['details']['shippingAddress']['city'],
						'zone_id'    => (isset($zone_info['zone_id']) ? $zone_info['zone_id'] : 0),
						'country_id' => (isset($country_info['country_id']) ? $country_info['country_id'] : 0)
					);

					$address_id = account_address::addAddress(customer::getId(), $address_data);

					session::$data['payment_address_id'] = $address_id;
					session::$data['payment_country_id'] = $address_data['country_id'];
					session::$data['payment_zone_id'] = $address_data['zone_id'];

					session::$data['shipping_address_id'] = $address_id;
					session::$data['shipping_country_id'] = $address_data['country_id'];
					session::$data['shipping_zone_id'] = $address_data['zone_id'];
					session::$data['shipping_postcode'] = $address_data['postcode'];
				}
			} else {
				session::$data['payment_address_id'] = '';
				session::$data['payment_country_id'] = '';
				session::$data['payment_zone_id'] = '';
			}
		}

		session::$data['paypal_braintree'] = request::$post;

		$json = array(
			'error' => false,
			'url' => ''
		);

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function expressConfirm() {
		if (!isset(session::$data['paypal_braintree']) || !isset(session::$data['paypal_braintree']['nonce'])) {
			response::redirect(url::link('checkout/cart', ''));
		}

		load::language('extension/payment/pp_braintree');
		load::language('checkout/cart');


		// Coupon
		if (isset(request::$post['coupon']) && self::validateCoupon()) {
			session::$data['coupon'] = request::$post['coupon'];

			session::$data['success'] = language::get('text_coupon');

			response::redirect(url::link('extension/payment/pp_braintree/expressConfirm', ''));
		}

		// Voucher
		if (isset(request::$post['voucher']) && self::validateVoucher()) {
			session::$data['voucher'] = request::$post['voucher'];

			session::$data['success'] = language::get('text_voucher');

			response::redirect(url::link('extension/payment/pp_braintree/expressConfirm', ''));
		}

		// Reward
		if (isset(request::$post['reward']) && self::validateReward()) {
			session::$data['reward'] = abs(request::$post['reward']);

			session::$data['success'] = language::get('text_reward');

			response::redirect(url::link('extension/payment/pp_braintree/expressConfirm', ''));
		}

		document::setTitle(language::get('text_express_title'));

		$data['heading_title'] = language::get('text_express_title');

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'href' => url::link('common/home'),
			'text' => language::get('text_home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_cart'),
			'href' => url::link('checkout/cart')
		);

		$data['breadcrumbs'][] = array(
			'href' => url::link('extension/payment/pp_braintree/expressConfirm'),
			'text' => language::get('text_express_title')
		);

		$points_total = 0;

		foreach (cart::getProducts() as $product) {
			if ($product['points']) {
				$points_total += $product['points'];
			}
		}

		$data['column_name'] = language::get('column_name');
		$data['column_model'] = language::get('column_model');
		$data['column_quantity'] = language::get('column_quantity');
		$data['column_price'] = language::get('column_price');
		$data['column_total'] = language::get('column_total');

		$data['button_shipping'] = language::get('button_express_shipping');
		$data['button_confirm'] = language::get('button_express_confirm');

		if (isset(request::$post['next'])) {
			$data['next'] = request::$post['next'];
		} else {
			$data['next'] = '';
		}

		$data['action'] = url::link('extension/payment/pp_braintree/expressConfirm', '');


		$products = cart::getProducts();

		if (empty($products)) {
			response::redirect(url::link('checkout/cart', ''));
		}

		foreach ($products as $product) {
			$product_total = 0;

			foreach ($products as $product_2) {
				if ($product_2['product_id'] == $product['product_id']) {
					$product_total += $product_2['quantity'];
				}
			}

			if ($product['minimum'] > $product_total) {
				$data['error_warning'] = sprintf(language::get('error_minimum'), $product['name'], $product['minimum']);
			}

			if ($product['image']) {
				$image = tool_image::resize($product['image'], config::get('theme_' . config('theme') . '_image_cart_width'), config::get('theme_' . config('theme') . '_image_cart_height'));
			} else {
				$image = '';
			}

			$option_data = array();

			foreach ($product['option'] as $option) {
				if ($option['type'] != 'file') {
					$value = $option['value'];
				} else {
					$upload_info = tool_upload::getUploadByCode($option['value']);

					if ($upload_info) {
						$value = $upload_info['name'];
					} else {
						$value = '';
					}
				}

				$option_data[] = array(
					'name'  => $option['name'],
					'value' => (utf8_strlen($value) > 20 ? utf8_substr($value, 0, 20) . '..' : $value)
				);
			}

			// Display prices
			if (customer::isLogged() || !config('customer_price')) {
				$unit_price = tax::calculate($product['price'], $product['tax_class_id'], config('tax'));

				$price = currency::format($unit_price, session::$data['currency']);
				$total = currency::format($unit_price * $product['quantity'], session::$data['currency']);
			} else {
				$price = false;
				$total = false;
			}

			$data['products'][] = array(
				'cart_id'               => $product['cart_id'],
				'thumb'                 => $image,
				'name'                  => $product['name'],
				'model'                 => $product['model'],
				'option'                => $option_data,
				'quantity'              => $product['quantity'],
				'stock'                 => $product['stock'] ? true : !(!config('stock_checkout') || config('stock_warning')),
				'reward'                => ($product['reward'] ? sprintf(language::get('text_points'), $product['reward']) : ''),
				'price'                 => $price,
				'total'                 => $total,
				'href'                  => url::link('product/product', 'product_id=' . $product['product_id']),
				'remove'                => url::link('checkout/cart', 'remove=' . $product['cart_id']),
			);
		}

		$data['vouchers'] = array();

		if (cart::hasShipping()) {
			$data['has_shipping'] = true;
			/**
			 * Shipping services
			 */
			if (customer::isLogged()) {
				$shipping_address = account_address::getAddress(session::$data['shipping_address_id']);
			} elseif (isset(session::$data['guest'])) {
				$shipping_address = session::$data['guest']['shipping'];
			}

			if (!empty($shipping_address)) {
				// Shipping Methods
				$quote_data = array();


				$results = setting_extension::getExtensions('shipping');

				if (!empty($results)) {
					foreach ($results as $result) {
						if (config::get('shipping_' . $result['code'] . '_status')) {

							$quote = call_model_method('extension_shipping_' . $result['code'], 'getQuote', $shipping_address);

							if ($quote) {
								$quote_data[$result['code']] = array(
									'title'      => $quote['title'],
									'quote'      => $quote['quote'],
									'sort_order' => $quote['sort_order'],
									'error'      => $quote['error']
								);
							}
						}
					}

					if (!empty($quote_data)) {
						$sort_order = array();

						foreach ($quote_data as $key => $value) {
							$sort_order[$key] = $value['sort_order'];
						}

						array_multisort($sort_order, SORT_ASC, $quote_data);

						session::$data['shipping_methods'] = $quote_data;
						$data['shipping_methods'] = $quote_data;

						if (!isset(session::$data['shipping_method'])) {
							//default the shipping to the very first option.
							$key1 = key($quote_data);
							$key2 = key($quote_data[$key1]['quote']);
							session::$data['shipping_method'] = $quote_data[$key1]['quote'][$key2];
						}

						$data['code'] = session::$data['shipping_method']['code'];
						$data['action_shipping'] = url::link('extension/payment/pp_braintree/shipping', '');
					} else {
						unset(session::$data['shipping_methods']);
						unset(session::$data['shipping_method']);
						$data['error_no_shipping'] = language::get('error_no_shipping');
					}
				} else {
					unset(session::$data['shipping_methods']);
					unset(session::$data['shipping_method']);
					$data['error_no_shipping'] = language::get('error_no_shipping');
				}
			}
		} else {
			$data['has_shipping'] = false;
		}

		// Totals

		$totals = array();
		$taxes = cart::getTaxes();
		$total = 0;

		// Because __call can not keep var references so we put them into an array.
		$total_data = array(
			'totals' => &$totals,
			'taxes'  => &$taxes,
			'total'  => &$total
		);

		// Display prices
		if (customer::isLogged() || !config('customer_price')) {
			$sort_order = array();

			$results = setting_extension::getExtensions('total');

			foreach ($results as $key => $value) {
				$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
			}

			array_multisort($sort_order, SORT_ASC, $results);

			foreach ($results as $result) {
				if (config::get('total_' . $result['code'] . '_status')) {

					// We have to put the totals in an array so that they pass by reference.
					call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
				}
			}

			$sort_order = array();

			foreach ($totals as $key => $value) {
				$sort_order[$key] = $value['sort_order'];
			}

			array_multisort($sort_order, SORT_ASC, $totals);
		}

		$data['totals'] = array();

		foreach ($totals as $total) {
			$data['totals'][] = array(
				'title' => $total['title'],
				'text'  => currency::format($total['value'], session::$data['currency']),
			);
		}

		/**
		 * Payment methods
		 */
		if (customer::isLogged() && isset(session::$data['payment_address_id'])) {
			$payment_address = account_address::getAddress(session::$data['payment_address_id']);
		} elseif (isset(session::$data['guest'])) {
			$payment_address = session::$data['guest']['payment'];
		}

		$method_data = array();


		$results = setting_extension::getExtensions('payment');

		extension_payment_pp_braintree::log("Payment methods returned based on new data");
		extension_payment_pp_braintree::log($results);

		foreach ($results as $result) {
			if (config::get('payment_' . $result['code'] . '_status')) {

				$method = call_model_method('extension_payment_' . $result['code'], 'getMethod', $payment_address, $total);

				if ($method) {
					$method_data[$result['code']] = $method;
				}
			}
		}

		$sort_order = array();

		foreach ($method_data as $key => $value) {
			$sort_order[$key] = $value['sort_order'];
		}

		array_multisort($sort_order, SORT_ASC, $method_data);

		extension_payment_pp_braintree::log("Payment methods again - sorted");
		extension_payment_pp_braintree::log($method_data);

		if (!isset($method_data['pp_braintree'])) {
			extension_payment_pp_braintree::log("Braintree module was no longer an option. Check configured zones or minimum order amount based on user address info");
			session::$data['error_warning'] = language::get('error_unavailable');
			response::redirect(url::link('checkout/checkout', ''));
		}

		session::$data['payment_methods'] = $method_data;
		session::$data['payment_method'] = $method_data['pp_braintree'];

		$data['action_confirm'] = url::link('extension/payment/pp_braintree/expressComplete', '');

		if (isset(session::$data['error_warning'])) {
			$data['error_warning'] = session::$data['error_warning'];
			unset(session::$data['error_warning']);
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];
			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(session::$data['attention'])) {
			$data['attention'] = session::$data['attention'];
			unset(session::$data['attention']);
		} else {
			$data['attention'] = '';
		}

		$data['coupon'] = load::controller('extension/total/coupon');
		$data['voucher'] = load::controller('extension/total/voucher');
		$data['reward'] = load::controller('extension/total/reward');
		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('extension/payment/pp_braintree_confirm', $data));
	}

	public static function expressComplete() {
		if (!isset(session::$data['paypal_braintree']) || !isset(session::$data['paypal_braintree']['nonce'])) {
			response::redirect(url::link('checkout/cart', ''));
		}

		load::language('extension/payment/pp_braintree');
		$redirect = '';

		if (cart::hasShipping()) {
			// Validate if shipping address has been set.

			if (customer::isLogged() && isset(session::$data['shipping_address_id'])) {
				$shipping_address = account_address::getAddress(session::$data['shipping_address_id']);
			} elseif (isset(session::$data['guest'])) {
				$shipping_address = session::$data['guest']['shipping'];
			}

			if (empty($shipping_address)) {
				$redirect = url::link('checkout/checkout', '');
			}

			// Validate if shipping method has been set.
			if (!isset(session::$data['shipping_method'])) {
				$redirect = url::link('checkout/checkout', '');
			}
		} else {
			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
		}

		// Validate if payment address has been set.

		if (customer::isLogged() && isset(session::$data['payment_address_id'])) {
			$payment_address = account_address::getAddress(session::$data['payment_address_id']);
		} elseif (isset(session::$data['guest'])) {
			$payment_address = session::$data['guest']['payment'];
		}

		// Validate if payment method has been set.
		if (!isset(session::$data['payment_method'])) {
			$redirect = url::link('checkout/checkout', '');
		}

		// Validate cart has products and has stock.
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			$redirect = url::link('checkout/cart');
		}

		// Validate minimum quantity requirements.
		$products = cart::getProducts();

		foreach ($products as $product) {
			$product_total = 0;

			foreach ($products as $product_2) {
				if ($product_2['product_id'] == $product['product_id']) {
					$product_total += $product_2['quantity'];
				}
			}

			if ($product['minimum'] > $product_total) {
				$redirect = url::link('checkout/cart');

				break;
			}
		}

		if ($redirect == '') {
			$totals = array();
			$taxes = cart::getTaxes();
			$total = 0;

			// Because __call can not keep var references so we put them into an array.
			$total_data = array(
				'totals' => &$totals,
				'taxes'  => &$taxes,
				'total'  => &$total
			);


			$sort_order = array();

			$results = setting_extension::getExtensions('total');

			foreach ($results as $key => $value) {
				$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
			}

			array_multisort($sort_order, SORT_ASC, $results);

			foreach ($results as $result) {
				if (config::get('total_' . $result['code'] . '_status')) {

					// We have to put the totals in an array so that they pass by reference.
					call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
				}
			}

			$sort_order = array();

			foreach ($totals as $key => $value) {
				$sort_order[$key] = $value['sort_order'];
			}

			array_multisort($sort_order, SORT_ASC, $totals);

			load::language('checkout/checkout');

			$data = array();

			$data['invoice_prefix'] = config('invoice_prefix');
			$data['store_id'] = config('store_id');
			$data['store_name'] = config('name');

			if ($data['store_id']) {
				$data['store_url'] = config('url');
			} else {
				$data['store_url'] = HTTP_SERVER;
			}

			if (customer::isLogged() && isset(session::$data['payment_address_id'])) {
				$data['customer_id'] = customer::getId();
				$data['customer_group_id'] = config('customer_group_id');
				$data['firstname'] = customer::getFirstName();
				$data['lastname'] = customer::getLastName();
				$data['email'] = customer::getEmail();
				$data['telephone'] = customer::getTelephone();


				$payment_address = account_address::getAddress(session::$data['payment_address_id']);
			} elseif (isset(session::$data['guest'])) {
				$data['customer_id'] = 0;
				$data['customer_group_id'] = session::$data['guest']['customer_group_id'];
				$data['firstname'] = session::$data['guest']['firstname'];
				$data['lastname'] = session::$data['guest']['lastname'];
				$data['email'] = session::$data['guest']['email'];
				$data['telephone'] = session::$data['guest']['telephone'];

				$payment_address = session::$data['guest']['payment'];
			}

			$data['payment_firstname'] = isset($payment_address['firstname']) ? $payment_address['firstname'] : '';
			$data['payment_lastname'] = isset($payment_address['lastname']) ? $payment_address['lastname'] : '';
			$data['payment_company'] = isset($payment_address['company']) ? $payment_address['company'] : '';
			$data['payment_company_id'] = isset($payment_address['company_id']) ? $payment_address['company_id'] : '';
			$data['payment_tax_id'] = isset($payment_address['tax_id']) ? $payment_address['tax_id'] : '';
			$data['payment_address_1'] = isset($payment_address['address_1']) ? $payment_address['address_1'] : '';
			$data['payment_address_2'] = isset($payment_address['address_2']) ? $payment_address['address_2'] : '';
			$data['payment_city'] = isset($payment_address['city']) ? $payment_address['city'] : '';
			$data['payment_postcode'] = isset($payment_address['postcode']) ? $payment_address['postcode'] : '';
			$data['payment_zone'] = isset($payment_address['zone']) ? $payment_address['zone'] : '';
			$data['payment_zone_id'] = isset($payment_address['zone_id']) ? $payment_address['zone_id'] : '';
			$data['payment_country'] = isset($payment_address['country']) ? $payment_address['country'] : '';
			$data['payment_country_id'] = isset($payment_address['country_id']) ? $payment_address['country_id'] : '';
			$data['payment_address_format'] = isset($payment_address['address_format']) ? $payment_address['address_format'] : '';

			$data['payment_method'] = '';
			if (isset(session::$data['payment_method']['title'])) {
				$data['payment_method'] = session::$data['payment_method']['title'];
			}

			$data['payment_code'] = '';
			if (isset(session::$data['payment_method']['code'])) {
				$data['payment_code'] = session::$data['payment_method']['code'];
			}

			if (cart::hasShipping()) {
				if (customer::isLogged()) {

					$shipping_address = account_address::getAddress(session::$data['shipping_address_id']);
				} elseif (isset(session::$data['guest'])) {
					$shipping_address = session::$data['guest']['shipping'];
				}

				$data['shipping_firstname'] = $shipping_address['firstname'];
				$data['shipping_lastname'] = $shipping_address['lastname'];
				$data['shipping_company'] = $shipping_address['company'];
				$data['shipping_address_1'] = $shipping_address['address_1'];
				$data['shipping_address_2'] = $shipping_address['address_2'];
				$data['shipping_city'] = $shipping_address['city'];
				$data['shipping_postcode'] = $shipping_address['postcode'];
				$data['shipping_zone'] = $shipping_address['zone'];
				$data['shipping_zone_id'] = $shipping_address['zone_id'];
				$data['shipping_country'] = $shipping_address['country'];
				$data['shipping_country_id'] = $shipping_address['country_id'];
				$data['shipping_address_format'] = $shipping_address['address_format'];

				$data['shipping_method'] = '';
				if (isset(session::$data['shipping_method']['title'])) {
					$data['shipping_method'] = session::$data['shipping_method']['title'];
				}

				$data['shipping_code'] = '';
				if (isset(session::$data['shipping_method']['code'])) {
					$data['shipping_code'] = session::$data['shipping_method']['code'];
				}
			} else {
				$data['shipping_firstname'] = '';
				$data['shipping_lastname'] = '';
				$data['shipping_company'] = '';
				$data['shipping_address_1'] = '';
				$data['shipping_address_2'] = '';
				$data['shipping_city'] = '';
				$data['shipping_postcode'] = '';
				$data['shipping_zone'] = '';
				$data['shipping_zone_id'] = '';
				$data['shipping_country'] = '';
				$data['shipping_country_id'] = '';
				$data['shipping_address_format'] = '';
				$data['shipping_method'] = '';
				$data['shipping_code'] = '';
			}

			$product_data = array();

			foreach (cart::getProducts() as $product) {
				$option_data = array();

				foreach ($product['option'] as $option) {
					$option_data[] = array(
						'product_option_id'       => $option['product_option_id'],
						'product_option_value_id' => $option['product_option_value_id'],
						'option_id'               => $option['option_id'],
						'option_value_id'         => $option['option_value_id'],
						'name'                    => $option['name'],
						'value'                   => $option['value'],
						'type'                    => $option['type']
					);
				}

				$product_data[] = array(
					'product_id' => $product['product_id'],
					'name'       => $product['name'],
					'model'      => $product['model'],
					'option'     => $option_data,
					'download'   => $product['download'],
					'quantity'   => $product['quantity'],
					'subtract'   => $product['subtract'],
					'price'      => $product['price'],
					'total'      => $product['total'],
					'tax'        => tax::getTax($product['price'], $product['tax_class_id']),
					'reward'     => $product['reward']
				);
			}

			// Gift Voucher
			$voucher_data = array();

			if (!empty(session::$data['vouchers'])) {
				foreach (session::$data['vouchers'] as $voucher) {
					$voucher_data[] = array(
						'description'      => $voucher['description'],
						'code'             => token(10),
						'to_name'          => $voucher['to_name'],
						'to_email'         => $voucher['to_email'],
						'from_name'        => $voucher['from_name'],
						'from_email'       => $voucher['from_email'],
						'voucher_theme_id' => $voucher['voucher_theme_id'],
						'message'          => $voucher['message'],
						'amount'           => $voucher['amount']
					);
				}
			}

			$data['products'] = $product_data;
			$data['vouchers'] = $voucher_data;
			$data['totals'] = $totals;
			$data['total'] = $total;
			$data['comment'] = '';

			if (isset(request::$cookie['tracking'])) {
				$data['tracking'] = request::$cookie['tracking'];

				$subtotal = cart::getSubTotal();

				// Affiliate

				$affiliate_info = affiliate_affiliate::getAffiliateByCode(request::$cookie['tracking']);

				if ($affiliate_info) {
					$data['affiliate_id'] = $affiliate_info['affiliate_id'];
					$data['commission'] = ($subtotal / 100) * $affiliate_info['commission'];
				} else {
					$data['affiliate_id'] = 0;
					$data['commission'] = 0;
				}

				// Marketing

				$marketing_info = checkout_marketing::getMarketingByCode(request::$cookie['tracking']);

				if ($marketing_info) {
					$data['marketing_id'] = $marketing_info['marketing_id'];
				} else {
					$data['marketing_id'] = 0;
				}
			} else {
				$data['affiliate_id'] = 0;
				$data['commission'] = 0;
				$data['marketing_id'] = 0;
				$data['tracking'] = '';
			}

			$data['language_id'] = config('language_id');
			$data['currency_id'] = currency::getId(session::$data['currency']);
			$data['currency_code'] = session::$data['currency'];
			$data['currency_value'] = currency::getValue(session::$data['currency']);
			$data['ip'] = request::$server['REMOTE_ADDR'];

			if (!empty(request::$server['HTTP_X_FORWARDED_FOR'])) {
				$data['forwarded_ip'] = request::$server['HTTP_X_FORWARDED_FOR'];
			} elseif (!empty(request::$server['HTTP_CLIENT_IP'])) {
				$data['forwarded_ip'] = request::$server['HTTP_CLIENT_IP'];
			} else {
				$data['forwarded_ip'] = '';
			}

			if (isset(request::$server['HTTP_USER_AGENT'])) {
				$data['user_agent'] = request::$server['HTTP_USER_AGENT'];
			} else {
				$data['user_agent'] = '';
			}

			if (isset(request::$server['HTTP_ACCEPT_LANGUAGE'])) {
				$data['accept_language'] = request::$server['HTTP_ACCEPT_LANGUAGE'];
			} else {
				$data['accept_language'] = '';
			}


			$order_id = checkout_order::addOrder($data);
			session::$data['order_id'] = $order_id;


			self::initialise();

			$create_sale = [
				"amount" => currency::format($data['total'], $data['currency_code'], $data['currency_value'], false),
				"paymentMethodNonce" => session::$data['paypal_braintree']['nonce'],
				"orderId" => $order_id,
				'channel' => 'OpenCart_Cart_vzero',
			];

			$transaction = extension_payment_pp_braintree::addTransaction(self::$gateway, $create_sale);

			//handle order status

			$order_status_id = 0;
			switch ($transaction::$transaction::$status) {
				case 'authorization_expired':
					$order_status_id = config::get('payment_pp_braintree_authorization_expired_id');
					break;
				case 'authorized':
					$order_status_id = config::get('payment_pp_braintree_authorized_id');
					break;
				case 'authorizing':
					$order_status_id = config::get('payment_pp_braintree_authorizing_id');
					break;
				case 'settlement_pending':
					$order_status_id = config::get('payment_pp_braintree_settlement_pending_id');
					break;
				case 'failed':
					$order_status_id = config::get('payment_pp_braintree_failed_id');
					break;
				case 'gateway_rejected':
					$order_status_id = config::get('payment_pp_braintree_gateway_rejected_id');
					break;
				case 'processor_declined':
					$order_status_id = config::get('payment_pp_braintree_processor_declined_id');
					break;
				case 'settled':
					$order_status_id = config::get('payment_pp_braintree_settled_id');
					break;
				case 'settling':
					$order_status_id = config::get('payment_pp_braintree_settling_id');
					break;
				case 'submitted_for_settlement':
					$order_status_id = config::get('payment_pp_braintree_submitted_for_settlement_id');
					break;
				case 'voided':
					$order_status_id = config::get('payment_pp_braintree_voided_id');
					break;
			}

			checkout_order::addOrderHistory(session::$data['order_id'], $order_status_id);

			if ($transaction::$success) {
				extension_payment_pp_braintree::log('Transaction success, details below');
				extension_payment_pp_braintree::log($transaction);

				response::redirect(url::link('checkout/success', ''));
			} else {
				extension_payment_pp_braintree::log('Transaction failed, details below');
				extension_payment_pp_braintree::log($transaction);

				session::$data['error'] = language::get('error_process_order');
				response::redirect(url::link('checkout/checkout', ''));
			}
		} else {
			response::redirect($redirect);
		}
	}

	private static function initialise() {

		if (config::get('payment_pp_braintree_access_token') != '') {
			self::$gateway = extension_payment_pp_braintree::setGateway(config::get('payment_pp_braintree_access_token'));
		} else {
			extension_payment_pp_braintree::setCredentials();
		}
	}

	public static function shipping() {
		self::shippingValidate(request::$post['shipping_method']);

		response::redirect(url::link('extension/payment/pp_braintree/expressConfirm'));
	}

	protected static function shippingValidate($code) {
		load::language('checkout/cart');
		load::language('extension/payment/pp_braintree');

		if (empty($code)) {
			session::$data['error_warning'] = language::get('error_shipping');
			return false;
		} else {
			$shipping = explode('.', $code);

			if (!isset($shipping[0]) || !isset($shipping[1]) || !isset(session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]])) {
				session::$data['error_warning'] = language::get('error_shipping');
				return false;
			} else {
				session::$data['shipping_method'] = session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]];
				session::$data['success'] = language::get('text_shipping_updated');
				return true;
			}
		}
	}

	protected static function validateCoupon() {

		$coupon_info = extension_total_coupon::getCoupon(request::$post['coupon']);

		if ($coupon_info) {
			return true;
		} else {
			session::$data['error_warning'] = language::get('error_coupon');
			return false;
		}
	}

	protected static function validateVoucher() {

		$voucher_info = extension_total_voucher::getVoucher(request::$post['voucher']);

		if ($voucher_info) {
			return true;
		} else {
			session::$data['error_warning'] = language::get('error_voucher');
			return false;
		}
	}

	protected static function validateReward() {
		$points = customer::getRewardPoints();

		$points_total = 0;

		foreach (cart::getProducts() as $product) {
			if ($product['points']) {
				$points_total += $product['points'];
			}
		}

		$error = '';

		if (empty(request::$post['reward'])) {
			$error = language::get('error_reward');
		}

		if (request::$post['reward'] > $points) {
			$error = sprintf(language::get('error_points'), request::$post['reward']);
		}

		if (request::$post['reward'] > $points_total) {
			$error = sprintf(language::get('error_maximum'), $points_total);
		}

		if (!$error) {
			return true;
		} else {
			session::$data['error_warning'] = $error;
			return false;
		}
	}
}
