<?php
class ControllerExtensionPaymentPPExpress {
	public static function index() {
		load::language('extension/payment/pp_express');
		
		$data['payment_pp_express_incontext_disable'] = config::get('payment_pp_express_incontext_disable');

		if (config::get('payment_pp_express_test') == 1) {
			$data['username'] = config::get('payment_pp_express_sandbox_username');
		} else {
			$data['username'] = config::get('payment_pp_express_username');
		}

		$data['continue'] = url::link('extension/payment/pp_express/checkout', '');

		unset(session::$data['paypal']);

		return load::view('extension/payment/pp_express', $data);
	}

	public static function express() {

		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			response::redirect(url::link('checkout/cart'));
		}

		if (customer::isLogged()) {
			/**
			 * If the customer is already logged in
			 */
			session::$data['paypal']['guest'] = false;

			unset(session::$data['guest']);
		} else {
			if (config('checkout_guest') && !config('customer_price') && !cart::hasDownload() && !cart::hasRecurringProducts()) {
				/**
				 * If the guest checkout is allowed (config ok, no login for price and doesn't have downloads)
				 */
				session::$data['paypal']['guest'] = true;
			} else {
				/**
				 * If guest checkout disabled or login is required before price or order has downloads
				 *
				 * Send them to the normal checkout flow.
				 */
				unset(session::$data['guest']);

				response::redirect(url::link('checkout/checkout', ''));
			}
		}

		unset(session::$data['shipping_method']);
		unset(session::$data['shipping_methods']);
		unset(session::$data['payment_method']);
		unset(session::$data['payment_methods']);


		if (cart::hasShipping()) {
			$shipping = 2;
		} else {
			$shipping = 1;
		}

		$max_amount = cart::getTotal() * 1.5;
		$max_amount = currency::format($max_amount, session::$data['currency'], '', false);

		$data = array(
			'METHOD'             => 'SetExpressCheckout',
			'MAXAMT'             => $max_amount,
			'RETURNURL'          => url::link('extension/payment/pp_express/expressReturn', ''),
			'CANCELURL'          => url::link('checkout/cart', ''),
			'REQCONFIRMSHIPPING' => 0,
			'NOSHIPPING'         => $shipping,
			'ALLOWNOTE'          => config::get('payment_pp_express_allow_note'),
			'LOCALECODE'         => 'EN',
			'LANDINGPAGE'        => 'Login',
			'HDRIMG'             => tool_image::resize(config::get('payment_pp_express_logo'), 750, 90),
			'PAYFLOWCOLOR'       => config::get('payment_pp_express_colour'),
			'CHANNELTYPE'        => 'Merchant'
		);

		if (isset(session::$data['pp_login']['seamless']['access_token']) && (isset(session::$data['pp_login']['seamless']['customer_id']) && session::$data['pp_login']['seamless']['customer_id'] == customer::getId()) && config::get('module_pp_login_seamless')) {
			$data['IDENTITYACCESSTOKEN'] = session::$data['pp_login']['seamless']['access_token'];
		}

		$data = array_merge($data, extension_payment_pp_express::paymentRequestInfo());

		$result = extension_payment_pp_express::call($data);

		/**
		 * If a failed PayPal setup happens, handle it.
		 */
		if (!isset($result['TOKEN'])) {
			session::$data['error'] = $result['L_LONGMESSAGE0'];
			/**
			 * Unable to add error message to user as the session errors/success are not
			 * used on the cart or checkout pages - need to be added?
			 * If PayPal debug log is off then still log error to normal error log.
			 */

			log::write('Unable to create PayPal call: ' . json_encode($result));

			response::redirect(url::link('checkout/checkout', ''));
		}

		session::$data['paypal']['token'] = $result['TOKEN'];

		if (config::get('payment_pp_express_test') == 1) {
			header('Location: https://www.sandbox.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=' . $result['TOKEN']);
		} else {
			header('Location: https://www.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=' . $result['TOKEN']);
		}
	}

	public static function expressReturn() {
		/**
		 * This is the url when PayPal has completed the auth.
		 *
		 * It has no output, instead it sets the data and locates to checkout
		 */
		$data = array(
			'METHOD' => 'GetExpressCheckoutDetails',
			'TOKEN'  => session::$data['paypal']['token']
		);

		$result = extension_payment_pp_express::call($data);
		session::$data['paypal']['payerid']   = $result['PAYERID'];
		session::$data['paypal']['result']    = $result;

		session::$data['comment'] = '';
		if (isset($result['PAYMENTREQUEST_0_NOTETEXT'])) {
			session::$data['comment'] = $result['PAYMENTREQUEST_0_NOTETEXT'];
		}

		if (session::$data['paypal']['guest'] == true) {

			session::$data['guest']['customer_group_id'] = config('customer_group_id');
			session::$data['guest']['firstname'] = trim($result['FIRSTNAME']);
			session::$data['guest']['lastname'] = trim($result['LASTNAME']);
			session::$data['guest']['email'] = trim($result['EMAIL']);

			if (isset($result['PHONENUM'])) {
				session::$data['guest']['telephone'] = $result['PHONENUM'];
			} else {
				session::$data['guest']['telephone'] = '';
			}

			session::$data['guest']['payment']['firstname'] = trim($result['FIRSTNAME']);
			session::$data['guest']['payment']['lastname'] = trim($result['LASTNAME']);

			if (isset($result['BUSINESS'])) {
				session::$data['guest']['payment']['company'] = $result['BUSINESS'];
			} else {
				session::$data['guest']['payment']['company'] = '';
			}

			session::$data['guest']['payment']['company_id'] = '';
			session::$data['guest']['payment']['tax_id'] = '';

			if (cart::hasShipping()) {
				$shipping_name = explode(' ', trim($result['PAYMENTREQUEST_0_SHIPTONAME']));
				$shipping_first_name = $shipping_name[0];
				unset($shipping_name[0]);
				$shipping_last_name = implode(' ', $shipping_name);

				session::$data['guest']['payment']['address_1'] = $result['PAYMENTREQUEST_0_SHIPTOSTREET'];
				if (isset($result['PAYMENTREQUEST_0_SHIPTOSTREET2'])) {
					session::$data['guest']['payment']['address_2'] = $result['PAYMENTREQUEST_0_SHIPTOSTREET2'];
				} else {
					session::$data['guest']['payment']['address_2'] = '';
				}

				session::$data['guest']['payment']['postcode'] = $result['PAYMENTREQUEST_0_SHIPTOZIP'];
				session::$data['guest']['payment']['city'] = $result['PAYMENTREQUEST_0_SHIPTOCITY'];

				session::$data['guest']['shipping']['firstname'] = $shipping_first_name;
				session::$data['guest']['shipping']['lastname'] = $shipping_last_name;
				session::$data['guest']['shipping']['company'] = '';
				session::$data['guest']['shipping']['address_1'] = $result['PAYMENTREQUEST_0_SHIPTOSTREET'];

				if (isset($result['PAYMENTREQUEST_0_SHIPTOSTREET2'])) {
					session::$data['guest']['shipping']['address_2'] = $result['PAYMENTREQUEST_0_SHIPTOSTREET2'];
				} else {
					session::$data['guest']['shipping']['address_2'] = '';
				}

				session::$data['guest']['shipping']['postcode'] = $result['PAYMENTREQUEST_0_SHIPTOZIP'];
				session::$data['guest']['shipping']['city'] = $result['PAYMENTREQUEST_0_SHIPTOCITY'];

				session::$data['shipping_postcode'] = $result['PAYMENTREQUEST_0_SHIPTOZIP'];

				$country_info = db::query("SELECT * FROM `country` WHERE `iso_code_2` = " . esc($result['PAYMENTREQUEST_0_SHIPTOCOUNTRYCODE']) . " AND `status` = '1' LIMIT 1")->row;

				if ($country_info) {
					session::$data['guest']['shipping']['country_id'] = $country_info['country_id'];
					session::$data['guest']['shipping']['country'] = $country_info['name'];
					session::$data['guest']['shipping']['iso_code_2'] = $country_info['iso_code_2'];
					session::$data['guest']['shipping']['iso_code_3'] = $country_info['iso_code_3'];
					session::$data['guest']['shipping']['address_format'] = $country_info['address_format'];
					session::$data['guest']['payment']['country_id'] = $country_info['country_id'];
					session::$data['guest']['payment']['country'] = $country_info['name'];
					session::$data['guest']['payment']['iso_code_2'] = $country_info['iso_code_2'];
					session::$data['guest']['payment']['iso_code_3'] = $country_info['iso_code_3'];
					session::$data['guest']['payment']['address_format'] = $country_info['address_format'];
					session::$data['shipping_country_id'] = $country_info['country_id'];
				} else {
					session::$data['guest']['shipping']['country_id'] = '';
					session::$data['guest']['shipping']['country'] = '';
					session::$data['guest']['shipping']['iso_code_2'] = '';
					session::$data['guest']['shipping']['iso_code_3'] = '';
					session::$data['guest']['shipping']['address_format'] = '';
					session::$data['guest']['payment']['country_id'] = '';
					session::$data['guest']['payment']['country'] = '';
					session::$data['guest']['payment']['iso_code_2'] = '';
					session::$data['guest']['payment']['iso_code_3'] = '';
					session::$data['guest']['payment']['address_format'] = '';
					session::$data['shipping_country_id'] = '';
				}

				if (isset($result['PAYMENTREQUEST_0_SHIPTOSTATE'])) {
					$returned_shipping_zone = $result['PAYMENTREQUEST_0_SHIPTOSTATE'];
				} else {
					$returned_shipping_zone = '';
				}

				$zone_info = db::query("SELECT * FROM `zone` WHERE (`name` = " . esc($returned_shipping_zone) . " OR `code` = " . esc($returned_shipping_zone) . ") AND `status` = '1' AND `country_id` = " . (int)$country_info['country_id'] . " LIMIT 1")->row;

				if ($zone_info) {
					session::$data['guest']['shipping']['zone'] = $zone_info['name'];
					session::$data['guest']['shipping']['zone_code'] = $zone_info['code'];
					session::$data['guest']['shipping']['zone_id'] = $zone_info['zone_id'];
					session::$data['guest']['payment']['zone'] = $zone_info['name'];
					session::$data['guest']['payment']['zone_code'] = $zone_info['code'];
					session::$data['guest']['payment']['zone_id'] = $zone_info['zone_id'];
					session::$data['shipping_zone_id'] = $zone_info['zone_id'];
				} else {
					session::$data['guest']['shipping']['zone'] = '';
					session::$data['guest']['shipping']['zone_code'] = '';
					session::$data['guest']['shipping']['zone_id'] = '';
					session::$data['guest']['payment']['zone'] = '';
					session::$data['guest']['payment']['zone_code'] = '';
					session::$data['guest']['payment']['zone_id'] = '';
					session::$data['shipping_zone_id'] = '';
				}

				session::$data['guest']['shipping_address'] = true;
			} else {
				session::$data['guest']['payment']['address_1'] = '';
				session::$data['guest']['payment']['address_2'] = '';
				session::$data['guest']['payment']['postcode'] = '';
				session::$data['guest']['payment']['city'] = '';
				session::$data['guest']['payment']['country_id'] = '';
				session::$data['guest']['payment']['country'] = '';
				session::$data['guest']['payment']['iso_code_2'] = '';
				session::$data['guest']['payment']['iso_code_3'] = '';
				session::$data['guest']['payment']['address_format'] = '';
				session::$data['guest']['payment']['zone'] = '';
				session::$data['guest']['payment']['zone_code'] = '';
				session::$data['guest']['payment']['zone_id'] = '';
				session::$data['guest']['shipping_address'] = false;
			}

			session::$data['account'] = 'guest';

			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
			unset(session::$data['payment_method']);
			unset(session::$data['payment_methods']);
		} else {
			unset(session::$data['guest']);
			/**
			 * if the user is logged in, add the address to the account and set the ID.
			 */

			if (cart::hasShipping()) {

				$addresses = account_address::getAddresses();

				/**
				 * Compare all of the user addresses and see if there is a match
				 */
				$match = false;
				foreach($addresses as $address) {
					if (trim(strtolower($address['address_1'])) == trim(strtolower($result['PAYMENTREQUEST_0_SHIPTOSTREET'])) && trim(strtolower($address['postcode'])) == trim(strtolower($result['PAYMENTREQUEST_0_SHIPTOZIP']))) {
						$match = true;

						session::$data['payment_address_id'] = $address['address_id'];
						session::$data['payment_country_id'] = $address['country_id'];
						session::$data['payment_zone_id'] = $address['zone_id'];

						session::$data['shipping_address_id'] = $address['address_id'];
						session::$data['shipping_country_id'] = $address['country_id'];
						session::$data['shipping_zone_id'] = $address['zone_id'];
						session::$data['shipping_postcode'] = $address['postcode'];

						break;
					}
				}

				/**
				 * If there is no address match add the address and set the info.
				 */
				if ($match == false) {

					$shipping_name = explode(' ', trim($result['PAYMENTREQUEST_0_SHIPTONAME']));
					$shipping_first_name = $shipping_name[0];
					unset($shipping_name[0]);
					$shipping_last_name = implode(' ', $shipping_name);

					$country_info = db::query("SELECT * FROM `country` WHERE `iso_code_2` = " . esc($result['PAYMENTREQUEST_0_SHIPTOCOUNTRYCODE']) . " AND `status` = '1' LIMIT 1")->row;
					$zone_info = db::query("SELECT * FROM `zone` WHERE (`name` = " . esc($result['PAYMENTREQUEST_0_SHIPTOSTATE']) . " OR `code` = " . esc($result['PAYMENTREQUEST_0_SHIPTOSTATE']) . ") AND `status` = '1' AND `country_id` = " . (int)$country_info['country_id'])->row;

					$address_data = array(
						'firstname'  => $shipping_first_name,
						'lastname'   => $shipping_last_name,
						'company'    => '',
						'company_id' => '',
						'tax_id'     => '',
						'address_1'  => $result['PAYMENTREQUEST_0_SHIPTOSTREET'],
						'address_2'  => (isset($result['PAYMENTREQUEST_0_SHIPTOSTREET2']) ? $result['PAYMENTREQUEST_0_SHIPTOSTREET2'] : ''),
						'postcode'   => $result['PAYMENTREQUEST_0_SHIPTOZIP'],
						'city'       => $result['PAYMENTREQUEST_0_SHIPTOCITY'],
						'zone_id'    => (isset($zone_info['zone_id']) ? $zone_info['zone_id'] : 0),
						'country_id' => (isset($country_info['country_id']) ? $country_info['country_id'] : 0)
					);

					$address_id = account_address::addAddress(customer::getId(), $address_data);

					session::$data['payment_address_id'] = $address_id;
					session::$data['payment_country_id'] = $address_data['country_id'];
					session::$data['payment_zone_id'] = $address_data['zone_id'];

					session::$data['shipping_address_id'] = $address_id;
					session::$data['shipping_country_id'] = $address_data['country_id'];
					session::$data['shipping_zone_id'] = $address_data['zone_id'];
					session::$data['shipping_postcode'] = $address_data['postcode'];
				}
			} else {
				session::$data['payment_address_id'] = '';
				session::$data['payment_country_id'] = '';
				session::$data['payment_zone_id'] = '';
			}
		}

		response::redirect(url::link('extension/payment/pp_express/expressConfirm', ''));
	}

	public static function expressConfirm() {
		load::language('extension/payment/pp_express');
		load::language('checkout/cart');


		// Coupon
		if (isset(request::$post['coupon']) && self::validateCoupon()) {
			session::$data['coupon'] = request::$post['coupon'];

			session::$data['success'] = language::get('text_coupon');

			response::redirect(url::link('extension/payment/pp_express/expressConfirm', ''));
		}

		// Voucher
		if (isset(request::$post['voucher']) && self::validateVoucher()) {
			session::$data['voucher'] = request::$post['voucher'];

			session::$data['success'] = language::get('text_voucher');

			response::redirect(url::link('extension/payment/pp_express/expressConfirm', ''));
		}

		// Reward
		if (isset(request::$post['reward']) && self::validateReward()) {
			session::$data['reward'] = abs(request::$post['reward']);

			session::$data['success'] = language::get('text_reward');

			response::redirect(url::link('extension/payment/pp_express/expressConfirm', ''));
		}

		document::setTitle(language::get('express_text_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'href' => url::link('common/home'),
			'text' => language::get('text_home')
		);

		$data['breadcrumbs'][] = array(
			'href' => url::link('extension/payment/pp_express/express'),
			'text' => language::get('text_title')
		);

		$data['breadcrumbs'][] = array(
			'href' => url::link('extension/payment/pp_express/expressConfirm'),
			'text' => language::get('express_text_title')
		);

		$points = customer::getRewardPoints();

		$points_total = 0;

		foreach (cart::getProducts() as $product) {
			if ($product['points']) {
				$points_total += $product['points'];
			}
		}

		$data['button_shipping'] = language::get('button_express_shipping');
		$data['button_confirm'] = language::get('button_express_confirm');

		if (isset(request::$post['next'])) {
			$data['next'] = request::$post['next'];
		} else {
			$data['next'] = '';
		}

		$data['action'] = url::link('extension/payment/pp_express/expressConfirm', '');


		$products = cart::getProducts();

		foreach ($products as $product) {
			$product_total = 0;

			foreach ($products as $product_2) {
				if ($product_2['product_id'] == $product['product_id']) {
					$product_total += $product_2['quantity'];
				}
			}

			if ($product['minimum'] > $product_total) {
				$data['error_warning'] = sprintf(language::get('error_minimum'), $product['name'], $product['minimum']);
			}

			if ($product['image']) {
				$image = tool_image::resize($product['image'], config::get('theme_' . config('theme') . '_image_cart_width'), config::get('theme_' . config('theme') . '_image_cart_height'));
			} else {
				$image = '';
			}

			$option_data = array();

			foreach ($product['option'] as $option) {
				if ($option['type'] != 'file') {
					$value = $option['value'];
				} else {
					$upload_info = tool_upload::getUploadByCode($option['value']);

					if ($upload_info) {
						$value = $upload_info['name'];
					} else {
						$value = '';
					}
				}

				$option_data[] = array(
					'name'  => $option['name'],
					'value' => (utf8_strlen($value) > 20 ? utf8_substr($value, 0, 20) . '..' : $value)
				);
			}

			// Display prices
			if (customer::isLogged() || !config('customer_price')) {
				$unit_price = tax::calculate($product['price'], $product['tax_class_id'], config('tax'));

				$price = currency::format($unit_price, session::$data['currency']);
				$total = currency::format($unit_price * $product['quantity'], session::$data['currency']);
			} else {
				$price = false;
				$total = false;
			}

			$recurring_description = '';

			if ($product['recurring']) {
				$frequencies = array(
					'day'        => language::get('text_day'),
					'week'       => language::get('text_week'),
					'semi_month' => language::get('text_semi_month'),
					'month'      => language::get('text_month'),
					'year'       => language::get('text_year'),
				);

				if ($product['recurring']['trial']) {
					$recurring_price = currency::format(tax::calculate($product['recurring']['trial_price'] * $product['quantity'], $product['tax_class_id'], config('tax')), session::$data['currency']);
					$recurring_description = sprintf(language::get('text_trial_description'), $recurring_price, $product['recurring']['trial_cycle'], $frequencies[$product['recurring']['trial_frequency']], $product['recurring']['trial_duration']) . ' ';
				}

				$recurring_price = currency::format(tax::calculate($product['recurring']['price'] * $product['quantity'], $product['tax_class_id'], config('tax')), session::$data['currency']);

				if ($product['recurring']['duration']) {
					$recurring_description .= sprintf(language::get('text_payment_description'), $recurring_price, $product['recurring']['cycle'], $frequencies[$product['recurring']['frequency']], $product['recurring']['duration']);
				} else {
					$recurring_description .= sprintf(language::get('text_payment_cancel'), $recurring_price, $product['recurring']['cycle'], $frequencies[$product['recurring']['frequency']], $product['recurring']['duration']);
				}
			}

			$data['products'][] = array(
				'cart_id'               => $product['cart_id'],
				'thumb'                 => $image,
				'name'                  => $product['name'],
				'model'                 => $product['model'],
				'option'                => $option_data,
				'quantity'              => $product['quantity'],
				'stock'                 => $product['stock'] ? true : !(!config('stock_checkout') || config('stock_warning')),
				'reward'                => ($product['reward'] ? sprintf(language::get('text_points'), $product['reward']) : ''),
				'price'                 => $price,
				'total'                 => $total,
				'href'                  => url::link('product/product', 'product_id=' . $product['product_id']),
				'remove'                => url::link('checkout/cart', 'remove=' . $product['cart_id']),
				'recurring'             => $product['recurring'],
				'recurring_name'        => (isset($product['recurring']['recurring_name']) ? $product['recurring']['recurring_name'] : ''),
				'recurring_description' => $recurring_description
			);
		}

		$data['vouchers'] = array();

		if (cart::hasShipping()) {

			$data['has_shipping'] = true;
			/**
			 * Shipping services
			 */
			if (customer::isLogged()) {
				$shipping_address = account_address::getAddress(session::$data['shipping_address_id']);
			} elseif (isset(session::$data['guest'])) {
				$shipping_address = session::$data['guest']['shipping'];
			}

			if (!empty($shipping_address)) {
				// Shipping Methods
				$quote_data = array();


				$results = setting_extension::getExtensions('shipping');

				if (!empty($results)) {
					foreach ($results as $result) {
						if (config::get('shipping_' . $result['code'] . '_status')) {

							$quote = call_model_method('extension_shipping_' . $result['code'], 'getQuote', $shipping_address);

							if ($quote) {
								$quote_data[$result['code']] = array(
									'title'      => $quote['title'],
									'quote'      => $quote['quote'],
									'sort_order' => $quote['sort_order'],
									'error'      => $quote['error']
								);
							}
						}
					}

					if (!empty($quote_data)) {
						$sort_order = array();

						foreach ($quote_data as $key => $value) {
							$sort_order[$key] = $value['sort_order'];
						}

						array_multisort($sort_order, SORT_ASC, $quote_data);

						session::$data['shipping_methods'] = $quote_data;
						$data['shipping_methods'] = $quote_data;

						if (!isset(session::$data['shipping_method'])) {
							//default the shipping to the very first option.
							$key1 = key($quote_data);
							$key2 = key($quote_data[$key1]['quote']);
							session::$data['shipping_method'] = $quote_data[$key1]['quote'][$key2];
						}

						$data['code'] = session::$data['shipping_method']['code'];
						$data['action_shipping'] = url::link('extension/payment/pp_express/shipping', '');
					} else {
						unset(session::$data['shipping_methods']);
						unset(session::$data['shipping_method']);
						$data['error_no_shipping'] = language::get('error_no_shipping');
					}
				} else {
					unset(session::$data['shipping_methods']);
					unset(session::$data['shipping_method']);
					$data['error_no_shipping'] = language::get('error_no_shipping');
				}
			}
		} else {
			$data['has_shipping'] = false;
		}

		// Totals

		$totals = array();
		$taxes = cart::getTaxes();
		$total = 0;

		// Because __call can not keep var references so we put them into an array.
		$total_data = array(
			'totals' => &$totals,
			'taxes'  => &$taxes,
			'total'  => &$total
		);

		// Display prices
		if (customer::isLogged() || !config('customer_price')) {
			$sort_order = array();

			$results = setting_extension::getExtensions('total');

			foreach ($results as $key => $value) {
				$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
			}

			array_multisort($sort_order, SORT_ASC, $results);

			foreach ($results as $result) {
				if (config::get('total_' . $result['code'] . '_status')) {

					// We have to put the totals in an array so that they pass by reference.
					call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
				}
			}

			$sort_order = array();

			foreach ($totals as $key => $value) {
				$sort_order[$key] = $value['sort_order'];
			}

			array_multisort($sort_order, SORT_ASC, $totals);
		}

		$data['totals'] = array();

		foreach ($totals as $total) {
			$data['totals'][] = array(
				'title' => $total['title'],
				'text'  => currency::format($total['value'], session::$data['currency']),
			);
		}

		/**
		 * Payment methods
		 */
		if (customer::isLogged() && isset(session::$data['payment_address_id'])) {
			$payment_address = account_address::getAddress(session::$data['payment_address_id']);
		} elseif (isset(session::$data['guest'])) {
			$payment_address = session::$data['guest']['payment'];
		}

		$method_data = array();


		$results = setting_extension::getExtensions('payment');

		foreach ($results as $result) {
			if (config::get('payment_' . $result['code'] . '_status')) {

				$method = call_model_method('extension_payment_' . $result['code'], 'getMethod', $payment_address, $total);

				if ($method) {
					$method_data[$result['code']] = $method;
				}
			}
		}

		$sort_order = array();

		foreach ($method_data as $key => $value) {
			$sort_order[$key] = $value['sort_order'];
		}

		array_multisort($sort_order, SORT_ASC, $method_data);

		if (!isset($method_data['pp_express'])) {
			session::$data['error_warning'] = language::get('error_unavailable');
			response::redirect(url::link('checkout/checkout', ''));
		}

		session::$data['payment_methods'] = $method_data;
		session::$data['payment_method'] = $method_data['pp_express'];

		$data['action_confirm'] = url::link('extension/payment/pp_express/expressComplete', '');

		if (isset(session::$data['error_warning'])) {
			$data['error_warning'] = session::$data['error_warning'];
			unset(session::$data['error_warning']);
		} else {
			$data['error_warning'] = '';
		}

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];
			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset(session::$data['attention'])) {
			$data['attention'] = session::$data['attention'];
			unset(session::$data['attention']);
		} else {
			$data['attention'] = '';
		}

		$data['coupon'] = load::controller('extension/total/coupon');
		$data['voucher'] = load::controller('extension/total/voucher');
		$data['reward'] = load::controller('extension/total/reward');
		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('extension/payment/pp_express_confirm', $data));
	}

	public static function expressComplete() {
		load::language('extension/payment/pp_express');
		$redirect = '';

		if (cart::hasShipping()) {
			// Validate if shipping address has been set.

			if (customer::isLogged() && isset(session::$data['shipping_address_id'])) {
				$shipping_address = account_address::getAddress(session::$data['shipping_address_id']);
			} elseif (isset(session::$data['guest'])) {
				$shipping_address = session::$data['guest']['shipping'];
			}

			if (empty($shipping_address)) {
				$redirect = url::link('checkout/checkout', '');
			}

			// Validate if shipping method has been set.
			if (!isset(session::$data['shipping_method'])) {
				$redirect = url::link('checkout/checkout', '');
			}
		} else {
			unset(session::$data['shipping_method']);
			unset(session::$data['shipping_methods']);
		}

		// Validate if payment address has been set.

		if (customer::isLogged() && isset(session::$data['payment_address_id'])) {
			$payment_address = account_address::getAddress(session::$data['payment_address_id']);
		} elseif (isset(session::$data['guest'])) {
			$payment_address = session::$data['guest']['payment'];
		}

		// Validate if payment method has been set.
		if (!isset(session::$data['payment_method'])) {
			$redirect = url::link('checkout/checkout', '');
		}

		// Validate cart has products and has stock.
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			$redirect = url::link('checkout/cart');
		}

		// Validate minimum quantity requirements.
		$products = cart::getProducts();

		foreach ($products as $product) {
			$product_total = 0;

			foreach ($products as $product_2) {
				if ($product_2['product_id'] == $product['product_id']) {
					$product_total += $product_2['quantity'];
				}
			}

			if ($product['minimum'] > $product_total) {
				$redirect = url::link('checkout/cart');

				break;
			}
		}

		if ($redirect == '') {
			$totals = array();
			$taxes = cart::getTaxes();
			$total = 0;

			// Because __call can not keep var references so we put them into an array.
			$total_data = array(
				'totals' => &$totals,
				'taxes'  => &$taxes,
				'total'  => &$total
			);


			$sort_order = array();

			$results = setting_extension::getExtensions('total');

			foreach ($results as $key => $value) {
				$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
			}

			array_multisort($sort_order, SORT_ASC, $results);

			foreach ($results as $result) {
				if (config::get('total_' . $result['code'] . '_status')) {

					// We have to put the totals in an array so that they pass by reference.
					call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
				}
			}

			$sort_order = array();

			foreach ($totals as $key => $value) {
				$sort_order[$key] = $value['sort_order'];
			}

			array_multisort($sort_order, SORT_ASC, $totals);

			load::language('checkout/checkout');

			$data = array();

			$data['invoice_prefix'] = config('invoice_prefix');
			$data['store_id'] = config('store_id');
			$data['store_name'] = config('name');

			if ($data['store_id']) {
				$data['store_url'] = config('url');
			} else {
				$data['store_url'] = HTTP_SERVER;
			}

			if (customer::isLogged() && isset(session::$data['payment_address_id'])) {
				$data['customer_id'] = customer::getId();
				$data['customer_group_id'] = config('customer_group_id');
				$data['firstname'] = customer::getFirstName();
				$data['lastname'] = customer::getLastName();
				$data['email'] = customer::getEmail();
				$data['telephone'] = customer::getTelephone();


				$payment_address = account_address::getAddress(session::$data['payment_address_id']);
			} elseif (isset(session::$data['guest'])) {
				$data['customer_id'] = 0;
				$data['customer_group_id'] = session::$data['guest']['customer_group_id'];
				$data['firstname'] = session::$data['guest']['firstname'];
				$data['lastname'] = session::$data['guest']['lastname'];
				$data['email'] = session::$data['guest']['email'];
				$data['telephone'] = session::$data['guest']['telephone'];

				$payment_address = session::$data['guest']['payment'];
			}

			$data['payment_firstname'] = isset($payment_address['firstname']) ? $payment_address['firstname'] : '';
			$data['payment_lastname'] = isset($payment_address['lastname']) ? $payment_address['lastname'] : '';
			$data['payment_company'] = isset($payment_address['company']) ? $payment_address['company'] : '';
			$data['payment_company_id'] = isset($payment_address['company_id']) ? $payment_address['company_id'] : '';
			$data['payment_tax_id'] = isset($payment_address['tax_id']) ? $payment_address['tax_id'] : '';
			$data['payment_address_1'] = isset($payment_address['address_1']) ? $payment_address['address_1'] : '';
			$data['payment_address_2'] = isset($payment_address['address_2']) ? $payment_address['address_2'] : '';
			$data['payment_city'] = isset($payment_address['city']) ? $payment_address['city'] : '';
			$data['payment_postcode'] = isset($payment_address['postcode']) ? $payment_address['postcode'] : '';
			$data['payment_zone'] = isset($payment_address['zone']) ? $payment_address['zone'] : '';
			$data['payment_zone_id'] = isset($payment_address['zone_id']) ? $payment_address['zone_id'] : '';
			$data['payment_country'] = isset($payment_address['country']) ? $payment_address['country'] : '';
			$data['payment_country_id'] = isset($payment_address['country_id']) ? $payment_address['country_id'] : '';
			$data['payment_address_format'] = isset($payment_address['address_format']) ? $payment_address['address_format'] : '';

			$data['payment_method'] = '';
			if (isset(session::$data['payment_method']['title'])) {
				$data['payment_method'] = session::$data['payment_method']['title'];
			}

			$data['payment_code'] = '';
			if (isset(session::$data['payment_method']['code'])) {
				$data['payment_code'] = session::$data['payment_method']['code'];
			}

			if (cart::hasShipping()) {
				if (customer::isLogged()) {

					$shipping_address = account_address::getAddress(session::$data['shipping_address_id']);
				} elseif (isset(session::$data['guest'])) {
					$shipping_address = session::$data['guest']['shipping'];
				}

				$data['shipping_firstname'] = $shipping_address['firstname'];
				$data['shipping_lastname'] = $shipping_address['lastname'];
				$data['shipping_company'] = $shipping_address['company'];
				$data['shipping_address_1'] = $shipping_address['address_1'];
				$data['shipping_address_2'] = $shipping_address['address_2'];
				$data['shipping_city'] = $shipping_address['city'];
				$data['shipping_postcode'] = $shipping_address['postcode'];
				$data['shipping_zone'] = $shipping_address['zone'];
				$data['shipping_zone_id'] = $shipping_address['zone_id'];
				$data['shipping_country'] = $shipping_address['country'];
				$data['shipping_country_id'] = $shipping_address['country_id'];
				$data['shipping_address_format'] = $shipping_address['address_format'];

				$data['shipping_method'] = '';
				if (isset(session::$data['shipping_method']['title'])) {
					$data['shipping_method'] = session::$data['shipping_method']['title'];
				}

				$data['shipping_code'] = '';
				if (isset(session::$data['shipping_method']['code'])) {
					$data['shipping_code'] = session::$data['shipping_method']['code'];
				}
			} else {
				$data['shipping_firstname'] = '';
				$data['shipping_lastname'] = '';
				$data['shipping_company'] = '';
				$data['shipping_address_1'] = '';
				$data['shipping_address_2'] = '';
				$data['shipping_city'] = '';
				$data['shipping_postcode'] = '';
				$data['shipping_zone'] = '';
				$data['shipping_zone_id'] = '';
				$data['shipping_country'] = '';
				$data['shipping_country_id'] = '';
				$data['shipping_address_format'] = '';
				$data['shipping_method'] = '';
				$data['shipping_code'] = '';
			}

			$product_data = array();

			foreach (cart::getProducts() as $product) {
				$option_data = array();

				foreach ($product['option'] as $option) {
					$option_data[] = array(
						'product_option_id'       => $option['product_option_id'],
						'product_option_value_id' => $option['product_option_value_id'],
						'option_id'               => $option['option_id'],
						'option_value_id'         => $option['option_value_id'],
						'name'                    => $option['name'],
						'value'                   => $option['value'],
						'type'                    => $option['type']
					);
				}

				$product_data[] = array(
					'product_id' => $product['product_id'],
					'name'       => $product['name'],
					'model'      => $product['model'],
					'option'     => $option_data,
					'download'   => $product['download'],
					'quantity'   => $product['quantity'],
					'subtract'   => $product['subtract'],
					'price'      => $product['price'],
					'total'      => $product['total'],
					'tax'        => tax::getTax($product['price'], $product['tax_class_id']),
					'reward'     => $product['reward']
				);
			}

			// Gift Voucher
			$voucher_data = array();

			if (!empty(session::$data['vouchers'])) {
				foreach (session::$data['vouchers'] as $voucher) {
					$voucher_data[] = array(
						'description'      => $voucher['description'],
						'code'             => token(10),
						'to_name'          => $voucher['to_name'],
						'to_email'         => $voucher['to_email'],
						'from_name'        => $voucher['from_name'],
						'from_email'       => $voucher['from_email'],
						'voucher_theme_id' => $voucher['voucher_theme_id'],
						'message'          => $voucher['message'],
						'amount'           => $voucher['amount']
					);
				}
			}

			$data['products'] = $product_data;
			$data['vouchers'] = $voucher_data;
			$data['totals'] = $totals;
			$data['comment'] = session::$data['comment'];
			$data['total'] = $total;

			if (isset(request::$cookie['tracking'])) {
				$data['tracking'] = request::$cookie['tracking'];

				$subtotal = cart::getSubTotal();

				// Affiliate

				$affiliate_info = affiliate_affiliate::getAffiliateByCode(request::$cookie['tracking']);

				if ($affiliate_info) {
					$data['affiliate_id'] = $affiliate_info['affiliate_id'];
					$data['commission'] = ($subtotal / 100) * $affiliate_info['commission'];
				} else {
					$data['affiliate_id'] = 0;
					$data['commission'] = 0;
				}

				// Marketing

				$marketing_info = checkout_marketing::getMarketingByCode(request::$cookie['tracking']);

				if ($marketing_info) {
					$data['marketing_id'] = $marketing_info['marketing_id'];
				} else {
					$data['marketing_id'] = 0;
				}
			} else {
				$data['affiliate_id'] = 0;
				$data['commission'] = 0;
				$data['marketing_id'] = 0;
				$data['tracking'] = '';
			}

			$data['language_id'] = config('language_id');
			$data['currency_id'] = currency::getId(session::$data['currency']);
			$data['currency_code'] = session::$data['currency'];
			$data['currency_value'] = currency::getValue(session::$data['currency']);
			$data['ip'] = request::$server['REMOTE_ADDR'];

			if (!empty(request::$server['HTTP_X_FORWARDED_FOR'])) {
				$data['forwarded_ip'] = request::$server['HTTP_X_FORWARDED_FOR'];
			} elseif (!empty(request::$server['HTTP_CLIENT_IP'])) {
				$data['forwarded_ip'] = request::$server['HTTP_CLIENT_IP'];
			} else {
				$data['forwarded_ip'] = '';
			}

			if (isset(request::$server['HTTP_USER_AGENT'])) {
				$data['user_agent'] = request::$server['HTTP_USER_AGENT'];
			} else {
				$data['user_agent'] = '';
			}

			if (isset(request::$server['HTTP_ACCEPT_LANGUAGE'])) {
				$data['accept_language'] = request::$server['HTTP_ACCEPT_LANGUAGE'];
			} else {
				$data['accept_language'] = '';
			}


			$order_id = checkout_order::addOrder($data);
			session::$data['order_id'] = $order_id;


			$paypal_data = array(
				'TOKEN'                      => session::$data['paypal']['token'],
				'PAYERID'                    => session::$data['paypal']['payerid'],
				'METHOD'                     => 'DoExpressCheckoutPayment',
				'PAYMENTREQUEST_0_NOTIFYURL' => url::link('extension/payment/pp_express/ipn', ''),
				'RETURNFMFDETAILS'           => 1
			);

			$paypal_data = array_merge($paypal_data, extension_payment_pp_express::paymentRequestInfo());

			$result = extension_payment_pp_express::call($paypal_data);

			if ($result['ACK'] == 'Success') {
				//handle order status
				switch($result['PAYMENTINFO_0_PAYMENTSTATUS']) {
					case 'Canceled_Reversal':
						$order_status_id = config::get('payment_pp_express_canceled_reversal_status_id');
						break;
					case 'Completed':
						$order_status_id = config::get('payment_pp_express_completed_status_id');
						break;
					case 'Denied':
						$order_status_id = config::get('payment_pp_express_denied_status_id');
						break;
					case 'Expired':
						$order_status_id = config::get('payment_pp_express_expired_status_id');
						break;
					case 'Failed':
						$order_status_id = config::get('payment_pp_express_failed_status_id');
						break;
					case 'Pending':
						$order_status_id = config::get('payment_pp_express_pending_status_id');
						break;
					case 'Processed':
						$order_status_id = config::get('payment_pp_express_processed_status_id');
						break;
					case 'Refunded':
						$order_status_id = config::get('payment_pp_express_refunded_status_id');
						break;
					case 'Reversed':
						$order_status_id = config::get('payment_pp_express_reversed_status_id');
						break;
					case 'Voided':
						$order_status_id = config::get('payment_pp_express_voided_status_id');
						break;
				}

				checkout_order::addOrderHistory($order_id, $order_status_id);

				//add order to paypal table
				$paypal_order_data = array(
					'order_id'         => $order_id,
					'capture_status'   => (config::get('payment_pp_express_transaction') == 'Sale' ? 'Complete' : 'NotComplete'),
					'currency_code'    => $result['PAYMENTINFO_0_CURRENCYCODE'],
					'authorization_id' => $result['PAYMENTINFO_0_TRANSACTIONID'],
					'total'            => $result['PAYMENTINFO_0_AMT']
				);

				$paypal_order_id = extension_payment_pp_express::addOrder($paypal_order_data);

				//add transaction to paypal transaction table
				$paypal_transaction_data = array(
					'paypal_order_id'       => $paypal_order_id,
					'transaction_id'        => $result['PAYMENTINFO_0_TRANSACTIONID'],
					'parent_id' => '',
					'note'                  => '',
					'msgsubid'              => '',
					'receipt_id'            => (isset($result['PAYMENTINFO_0_RECEIPTID']) ? $result['PAYMENTINFO_0_RECEIPTID'] : ''),
					'payment_type'          => $result['PAYMENTINFO_0_PAYMENTTYPE'],
					'payment_status'        => $result['PAYMENTINFO_0_PAYMENTSTATUS'],
					'pending_reason'        => $result['PAYMENTINFO_0_PENDINGREASON'],
					'transaction_entity'    => (config::get('payment_pp_express_transaction') == 'Sale' ? 'payment' : 'auth'),
					'amount'                => $result['PAYMENTINFO_0_AMT'],
					'debug_data'            => json_encode($result)
				);

				extension_payment_pp_express::addTransaction($paypal_transaction_data);

				$recurring_products = cart::getRecurringProducts();

				//loop through any products that are recurring items
				if ($recurring_products) {
					load::language('extension/payment/pp_express');


					$billing_period = array(
						'day'        => 'Day',
						'week'       => 'Week',
						'semi_month' => 'SemiMonth',
						'month'      => 'Month',
						'year'       => 'Year'
					);

					foreach($recurring_products as $item) {
						$data = array(
							'METHOD'             => 'CreateRecurringPaymentsProfile',
							'TOKEN'              => session::$data['paypal']['token'],
							'PROFILESTARTDATE'   => gmdate("Y-m-d\TH:i:s\Z", gmmktime(gmdate("H"), gmdate("i")+5, gmdate("s"), gmdate("m"), gmdate("d"), gmdate("y"))),
							'BILLINGPERIOD'      => $billing_period[$item['recurring']['frequency']],
							'BILLINGFREQUENCY'   => $item['recurring']['cycle'],
							'TOTALBILLINGCYCLES' => $item['recurring']['duration'],
							'AMT'                => currency::format(tax::calculate($item['recurring']['price'], $item['tax_class_id'], config('tax')), session::$data['currency'], false, false) * $item['quantity'],
							'CURRENCYCODE'       => session::$data['currency']
						);

						//trial information
						if ($item['recurring']['trial']) {
							$data_trial = array(
								'TRIALBILLINGPERIOD'      => $billing_period[$item['recurring']['trial_frequency']],
								'TRIALBILLINGFREQUENCY'   => $item['recurring']['trial_cycle'],
								'TRIALTOTALBILLINGCYCLES' => $item['recurring']['trial_duration'],
								'TRIALAMT'                => currency::format(tax::calculate($item['recurring']['trial_price'], $item['tax_class_id'], config('tax')), session::$data['currency'], false, false) * $item['quantity']
							);

							$trial_amt = currency::format(tax::calculate($item['recurring']['trial_price'], $item['tax_class_id'], config('tax')), session::$data['currency'], false, false) * $item['quantity'] . ' ' . session::$data['currency'];
							$trial_text =  sprintf(language::get('text_trial'), $trial_amt, $item['recurring']['trial_cycle'], $item['recurring']['trial_frequency'], $item['recurring']['trial_duration']);

							$data = array_merge($data, $data_trial);
						} else {
							$trial_text = '';
						}

						$recurring_amt = currency::format(tax::calculate($item['recurring']['price'], $item['tax_class_id'], config('tax')), session::$data['currency'], false, false)  * $item['quantity'] . ' ' . session::$data['currency'];
						$recurring_description = $trial_text . sprintf(language::get('text_recurring'), $recurring_amt, $item['recurring']['cycle'], $item['recurring']['frequency']);

						if ($item['recurring']['duration'] > 0) {
							$recurring_description .= sprintf(language::get('text_length'), $item['recurring']['duration']);
						}

						//create new recurring and set to pending status as no payment has been made yet.
						$recurring_id = checkout_recurring::addRecurring($order_id, $recurring_description, $item);

						$data['PROFILEREFERENCE'] = $recurring_id;
						$data['DESC'] = $recurring_description;

						$result = extension_payment_pp_express::call($data);

						if (isset($result['PROFILEID'])) {
							checkout_recurring::editReference($recurring_id, $result['PROFILEID']);
						} else {
							// there was an error creating the recurring, need to log and also alert admin / user

						}
					}
				}

				response::redirect(url::link('checkout/success'));

				if (isset($result['REDIRECTREQUIRED']) && $result['REDIRECTREQUIRED'] == true) {
					//- handle german redirect here
					response::redirect('https://www.paypal.com/cgi-bin/webscr?cmd=_complete-express-checkout&token=' . session::$data['paypal']['token']);
				}
			} else {
				if ($result['L_ERRORCODE0'] == '10486') {
					if (isset(session::$data['paypal_redirect_count'])) {

						if (session::$data['paypal_redirect_count'] == 2) {
							session::$data['paypal_redirect_count'] = 0;
							session::$data['error'] = language::get('error_too_many_failures');
							response::redirect(url::link('checkout/checkout', ''));
						} else {
							session::$data['paypal_redirect_count']++;
						}
					} else {
						session::$data['paypal_redirect_count'] = 1;
					}

					if (config::get('payment_pp_express_test') == 1) {
						response::redirect('https://www.sandbox.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=' . session::$data['paypal']['token']);
					} else {
						response::redirect('https://www.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=' . session::$data['paypal']['token']);
					}
				}

				session::$data['error_warning'] = $result['L_LONGMESSAGE0'];
				response::redirect(url::link('extension/payment/pp_express/expressConfirm', ''));
			}
		} else {
			response::redirect($redirect);
		}
	}

	public static function checkout() {
		if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
			response::redirect(url::link('checkout/cart'));
		}


		if(!isset(session::$data['order_id'])) {
			return false;
		}

		$order_info = checkout_order::getOrder(session::$data['order_id']);

		$max_amount = cart::getTotal() * 1.5;
		$max_amount = currency::format($max_amount, session::$data['currency'], '', false);

		if (cart::hasShipping()) {
			$shipping = 0;

			// PayPal requires some countries to use zone code (not name) to be sent in SHIPTOSTATE
			$ship_to_state_codes = array(
				'30', // Brazil
				'38', // Canada
				'105', // Italy
				'138', // Mexico
				'223', // USA
			);

			if (in_array($order_info['shipping_country_id'], $ship_to_state_codes)) {
				$ship_to_state = $order_info['shipping_zone_code'];
			} else {
				$ship_to_state = $order_info['shipping_zone'];
			}

			$data_shipping = array(
				'PAYMENTREQUEST_0_SHIPTONAME'        => html_entity_decode($order_info['shipping_firstname'] . ' ' . $order_info['shipping_lastname'], ENT_QUOTES, 'UTF-8'),
				'PAYMENTREQUEST_0_SHIPTOSTREET'      => html_entity_decode($order_info['shipping_address_1'], ENT_QUOTES, 'UTF-8'),
				'PAYMENTREQUEST_0_SHIPTOSTREET2'     => html_entity_decode($order_info['shipping_address_2'], ENT_QUOTES, 'UTF-8'),
				'PAYMENTREQUEST_0_SHIPTOCITY'        => html_entity_decode($order_info['shipping_city'], ENT_QUOTES, 'UTF-8'),
				'PAYMENTREQUEST_0_SHIPTOSTATE'       => html_entity_decode($ship_to_state, ENT_QUOTES, 'UTF-8'),
				'PAYMENTREQUEST_0_SHIPTOZIP'         => html_entity_decode($order_info['shipping_postcode'], ENT_QUOTES, 'UTF-8'),
				'PAYMENTREQUEST_0_SHIPTOCOUNTRYCODE' => $order_info['shipping_iso_code_2'],
				'ADDROVERRIDE' 						 => 1,
			);
		} else {
			$shipping = 1;
			$data_shipping = array();
		}

		$data = array(
			'METHOD'             => 'SetExpressCheckout',
			'MAXAMT'             => $max_amount,
			'RETURNURL'          => url::link('extension/payment/pp_express/checkoutReturn', ''),
			'CANCELURL'          => url::link('checkout/checkout', ''),
			'REQCONFIRMSHIPPING' => 0,
			'NOSHIPPING'         => $shipping,
			'LOCALECODE'         => 'EN',
			'LANDINGPAGE'        => 'Login',
			'HDRIMG'             => tool_image::resize(config::get('payment_pp_express_logo'), 750, 90),
			'PAYFLOWCOLOR'       => config::get('payment_pp_express_colour'),
			'CHANNELTYPE'        => 'Merchant',
			'ALLOWNOTE'          => config::get('payment_pp_express_allow_note')
		);

		$data = array_merge($data, $data_shipping);

		if (isset(session::$data['pp_login']['seamless']['access_token']) && (isset(session::$data['pp_login']['seamless']['customer_id']) && session::$data['pp_login']['seamless']['customer_id'] == customer::getId()) && config::get('module_pp_login_seamless')) {
			$data['IDENTITYACCESSTOKEN'] = session::$data['pp_login']['seamless']['access_token'];
		}

		$data = array_merge($data, extension_payment_pp_express::paymentRequestInfo());

		$result = extension_payment_pp_express::call($data);

		/**
		 * If a failed PayPal setup happens, handle it.
		 */
		if (!isset($result['TOKEN'])) {
			session::$data['error'] = $result['L_LONGMESSAGE0'];
			/**
			 * Unable to add error message to user as the session errors/success are not
			 * used on the cart or checkout pages - need to be added?
			 * If PayPal debug log is off then still log error to normal error log.
			 */
			log::write('Unable to create Paypal session' . json_encode($result));

			response::redirect(url::link('checkout/checkout', ''));
		}

		session::$data['paypal']['token'] = $result['TOKEN'];

		if (config::get('payment_pp_express_test') == 1) {
			header('Location: https://www.sandbox.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=' . $result['TOKEN'] . '&useraction=commit');
		} else {
			header('Location: https://www.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=' . $result['TOKEN'] . '&useraction=commit');
		}
	}

	public static function checkoutReturn() {
		load::language('extension/payment/pp_express');


		$data = array(
			'METHOD' => 'GetExpressCheckoutDetails',
			'TOKEN'  => session::$data['paypal']['token']
		);

		$result = extension_payment_pp_express::call($data);

		session::$data['paypal']['payerid'] = $result['PAYERID'];
		session::$data['paypal']['result'] = $result;

		$order_id = session::$data['order_id'];

		$paypal_data = array(
			'TOKEN'                      => session::$data['paypal']['token'],
			'PAYERID'                    => session::$data['paypal']['payerid'],
			'METHOD'                     => 'DoExpressCheckoutPayment',
			'PAYMENTREQUEST_0_NOTIFYURL' => url::link('extension/payment/pp_express/ipn', ''),
			'RETURNFMFDETAILS'           => 1
		);

		$paypal_data = array_merge($paypal_data, extension_payment_pp_express::paymentRequestInfo());

		$result = extension_payment_pp_express::call($paypal_data);

		if ($result['ACK'] == 'Success') {
			//handle order status
			switch($result['PAYMENTINFO_0_PAYMENTSTATUS']) {
				case 'Canceled_Reversal':
					$order_status_id = config::get('payment_pp_express_canceled_reversal_status_id');
					break;
				case 'Completed':
					$order_status_id = config::get('payment_pp_express_completed_status_id');
					break;
				case 'Denied':
					$order_status_id = config::get('payment_pp_express_denied_status_id');
					break;
				case 'Expired':
					$order_status_id = config::get('payment_pp_express_expired_status_id');
					break;
				case 'Failed':
					$order_status_id = config::get('payment_pp_express_failed_status_id');
					break;
				case 'Pending':
					$order_status_id = config::get('payment_pp_express_pending_status_id');
					break;
				case 'Processed':
					$order_status_id = config::get('payment_pp_express_processed_status_id');
					break;
				case 'Refunded':
					$order_status_id = config::get('payment_pp_express_refunded_status_id');
					break;
				case 'Reversed':
					$order_status_id = config::get('payment_pp_express_reversed_status_id');
					break;
				case 'Voided':
					$order_status_id = config::get('payment_pp_express_voided_status_id');
					break;
			}

			checkout_order::addOrderHistory($order_id, $order_status_id);

			//add order to paypal table
			$paypal_order_data = array(
				'order_id'         => $order_id,
				'capture_status'   => (config::get('payment_pp_express_transaction') == 'Sale' ? 'Complete' : 'NotComplete'),
				'currency_code'    => $result['PAYMENTINFO_0_CURRENCYCODE'],
				'authorization_id' => $result['PAYMENTINFO_0_TRANSACTIONID'],
				'total'            => $result['PAYMENTINFO_0_AMT']
			);

			$paypal_order_id = extension_payment_pp_express::addOrder($paypal_order_data);

			//add transaction to paypal transaction table
			$paypal_transaction_data = array(
				'paypal_order_id'       => $paypal_order_id,
				'transaction_id'        => $result['PAYMENTINFO_0_TRANSACTIONID'],
				'parent_id' => '',
				'note'                  => '',
				'msgsubid'              => '',
				'receipt_id'            => (isset($result['PAYMENTINFO_0_RECEIPTID']) ? $result['PAYMENTINFO_0_RECEIPTID'] : ''),
				'payment_type'          => $result['PAYMENTINFO_0_PAYMENTTYPE'],
				'payment_status'        => $result['PAYMENTINFO_0_PAYMENTSTATUS'],
				'pending_reason'        => $result['PAYMENTINFO_0_PENDINGREASON'],
				'transaction_entity'    => (config::get('payment_pp_express_transaction') == 'Sale' ? 'payment' : 'auth'),
				'amount'                => $result['PAYMENTINFO_0_AMT'],
				'debug_data'            => json_encode($result)
			);
			extension_payment_pp_express::addTransaction($paypal_transaction_data);

			$recurring_products = cart::getRecurringProducts();

			//loop through any products that are recurring items
			if ($recurring_products) {

				$billing_period = array(
					'day'        => 'Day',
					'week'       => 'Week',
					'semi_month' => 'SemiMonth',
					'month'      => 'Month',
					'year'       => 'Year'
				);

				foreach ($recurring_products as $item) {
					$data = array(
						'METHOD'             => 'CreateRecurringPaymentsProfile',
						'TOKEN'              => session::$data['paypal']['token'],
						'PROFILESTARTDATE'   => gmdate("Y-m-d\TH:i:s\Z", gmmktime(gmdate('H'), gmdate('i') + 5, gmdate('s'), gmdate('m'), gmdate('d'), gmdate('y'))),
						'BILLINGPERIOD'      => $billing_period[$item['recurring']['frequency']],
						'BILLINGFREQUENCY'   => $item['recurring']['cycle'],
						'TOTALBILLINGCYCLES' => $item['recurring']['duration'],
						'AMT'                => currency::format(tax::calculate($item['recurring']['price'], $item['tax_class_id'], config('tax')), session::$data['currency'], false, false) * $item['quantity'],
						'CURRENCYCODE'       => session::$data['currency']
					);

					//trial information
					if ($item['recurring']['trial'] == 1) {
						$data_trial = array(
							'TRIALBILLINGPERIOD'      => $billing_period[$item['recurring']['trial_frequency']],
							'TRIALBILLINGFREQUENCY'   => $item['recurring']['trial_cycle'],
							'TRIALTOTALBILLINGCYCLES' => $item['recurring']['trial_duration'],
							'TRIALAMT'                => currency::format(tax::calculate($item['recurring']['trial_price'], $item['tax_class_id'], config('tax')), session::$data['currency'], false, false) * $item['quantity']
						);

						$trial_amt = currency::format(tax::calculate($item['recurring']['trial_price'], $item['tax_class_id'], config('tax')), session::$data['currency'], false, false) * $item['quantity'] . ' ' . session::$data['currency'];
						$trial_text =  sprintf(language::get('text_trial'), $trial_amt, $item['recurring']['trial_cycle'], $item['recurring']['trial_frequency'], $item['recurring']['trial_duration']);

						$data = array_merge($data, $data_trial);
					} else {
						$trial_text = '';
					}

					$recurring_amt = currency::format(tax::calculate($item['recurring']['price'], $item['tax_class_id'], config('tax')), session::$data['currency'], false, false)  * $item['quantity'] . ' ' . session::$data['currency'];
					$recurring_description = $trial_text . sprintf(language::get('text_recurring'), $recurring_amt, $item['recurring']['cycle'], $item['recurring']['frequency']);

					if ($item['recurring']['duration'] > 0) {
						$recurring_description .= sprintf(language::get('text_length'), $item['recurring']['duration']);
					}

					//create new recurring and set to pending status as no payment has been made yet.
					$recurring_id = checkout_recurring::addRecurring($order_id, $recurring_description, $item);

					$data['PROFILEREFERENCE'] = $recurring_id;
					$data['DESC'] = $recurring_description;

					$result = extension_payment_pp_express::call($data);

					if (isset($result['PROFILEID'])) {
						checkout_recurring::editReference($recurring_id, $result['PROFILEID']);
					} else {
						// there was an error creating the recurring, need to log and also alert admin / user

					}
				}
			}

			if (isset($result['REDIRECTREQUIRED']) && $result['REDIRECTREQUIRED'] == true) {
				//- handle german redirect here
				response::redirect('https://www.paypal.com/cgi-bin/webscr?cmd=_complete-express-checkout&token=' . session::$data['paypal']['token']);
			} else {
				response::redirect(url::link('checkout/success'));
			}
		} else {
			if ($result['L_ERRORCODE0'] == '10486') {
				if (isset(session::$data['paypal_redirect_count'])) {

					if (session::$data['paypal_redirect_count'] == 2) {
						session::$data['paypal_redirect_count'] = 0;
						session::$data['error'] = language::get('error_too_many_failures');

						response::redirect(url::link('checkout/checkout', ''));
					} else {
						session::$data['paypal_redirect_count']++;
					}
				} else {
					session::$data['paypal_redirect_count'] = 1;
				}

				if (config::get('payment_pp_express_test') == 1) {
					response::redirect('https://www.sandbox.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=' . session::$data['paypal']['token']);
				} else {
					response::redirect('https://www.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token=' . session::$data['paypal']['token']);
				}
			}

			load::language('extension/payment/pp_express');

			$data['breadcrumbs'] = array();

			$data['breadcrumbs'][] = array(
				'href' => url::link('common/home'),
				'text' => language::get('text_home')
			);

			$data['breadcrumbs'][] = array(
				'href' => url::link('checkout/cart'),
				'text' => language::get('text_cart')
			);

			$data['heading_title'] = language::get('error_heading_title');

			$data['text_error'] = '<div class="warning">' . $result['L_ERRORCODE0'] . ' : ' . $result['L_LONGMESSAGE0'] . '</div>';

			$data['button_continue'] = language::get('button_continue');

			$data['continue'] = url::link('checkout/cart');

			unset(session::$data['success']);

			response::addHeader(request::$server['SERVER_PROTOCOL'] . ' 404 Not Found');

			$data['column_left'] = load::controller('common/column_left');
			$data['column_right'] = load::controller('common/column_right');
			$data['content_top'] = load::controller('common/content_top');
			$data['content_bottom'] = load::controller('common/content_bottom');
			$data['footer'] = load::controller('common/footer');
			$data['header'] = load::controller('common/header');

			response::setOutput(load::view('error/not_found', $data));
		}
	}

	public static function ipn() {

		$request = 'cmd=_notify-validate';

		foreach ($_POST as $key => $value) {
			$request .= '&' . $key . '=' . urlencode(html_entity_decode($value, ENT_QUOTES, 'UTF-8'));
		}

		if (config::get('payment_pp_express_test') == 1) {
			$curl = curl_init('https://www.sandbox.paypal.com/cgi-bin/webscr');
		} else {
			$curl = curl_init('https://www.paypal.com/cgi-bin/webscr');
		}

		curl_setopt($curl, CURLOPT_POST, true);
		curl_setopt($curl, CURLOPT_POSTFIELDS, $request);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_HEADER, false);
		curl_setopt($curl, CURLOPT_TIMEOUT, 30);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

		$response = trim(curl_exec($curl));

		if (!$response) {
			extension_payment_pp_express::log(array('error' => curl_error($curl),'error_no' => curl_errno($curl)), 'Curl failed');
		}

		extension_payment_pp_express::log(array('request' => $request,'response' => $response), 'IPN data');

		if ((string)$response == "VERIFIED")  {
			if (isset(request::$post['transaction_entity'])) {
				log::write(request::$post['transaction_entity']);
			}

			if (isset(request::$post['txn_id'])) {
				$transaction = extension_payment_pp_express::getTransactionRow(request::$post['txn_id']);
			} else {
				$transaction = false;
			}

			if (isset(request::$post['parent_txn_id'])) {
				$parent_transaction = extension_payment_pp_express::getTransactionRow(request::$post['parent_txn_id']);
			} else {
				$parent_transaction = false;
			}

			if ($transaction) {
				//transaction exists, check for cleared payment or updates etc
				extension_payment_pp_express::log('Transaction exists', 'IPN data');

				//if the transaction is pending but the new status is completed
				if ($transaction['payment_status'] != request::$post['payment_status']) {
					db::query("UPDATE `paypal_order_transaction` SET `payment_status` = " . esc(request::$post['payment_status']) . " WHERE `transaction_id` = " . esc($transaction['transaction_id']) . " LIMIT 1");
				} elseif ($transaction['payment_status'] == 'Pending' && ($transaction['pending_reason'] != request::$post['pending_reason'])) {
					//payment is still pending but the pending reason has changed, update it.
					db::query("UPDATE `paypal_order_transaction` SET `pending_reason` = " . esc(request::$post['pending_reason']) . " WHERE `transaction_id` = " . esc($transaction['transaction_id']) . " LIMIT 1");
				}
			} else {
				extension_payment_pp_express::log('Transaction does not exist', 'IPN data');

				if ($parent_transaction) {
					//parent transaction exists
					extension_payment_pp_express::log('Parent transaction exists', 'IPN data');

					//add new related transaction
					$transaction = array(
						'paypal_order_id'       => $parent_transaction['paypal_order_id'],
						'transaction_id'        => request::$post['txn_id'],
						'parent_id' => request::$post['parent_txn_id'],
						'note'                  => '',
						'msgsubid'              => '',
						'receipt_id'            => (isset(request::$post['receipt_id']) ? request::$post['receipt_id'] : ''),
						'payment_type'          => (isset(request::$post['payment_type']) ? request::$post['payment_type'] : ''),
						'payment_status'        => (isset(request::$post['payment_status']) ? request::$post['payment_status'] : ''),
						'pending_reason'        => (isset(request::$post['pending_reason']) ? request::$post['pending_reason'] : ''),
						'amount'                => request::$post['mc_gross'],
						'debug_data'            => json_encode(request::$post),
						'transaction_entity'    => (isset(request::$post['transaction_entity']) ? request::$post['transaction_entity'] : '')
					);

					extension_payment_pp_express::addTransaction($transaction);

					/**
					 * If there has been a refund, log this against the parent transaction.
					 */
					if (isset(request::$post['payment_status']) && request::$post['payment_status'] == 'Refunded') {
						if ((request::$post['mc_gross'] * -1) == $parent_transaction['amount']) {
							db::query("UPDATE `paypal_order_transaction` SET `payment_status` = 'Refunded' WHERE `transaction_id` = " . esc($parent_transaction['transaction_id']) . " LIMIT 1");
						} else {
							db::query("UPDATE `paypal_order_transaction` SET `payment_status` = 'Partially-Refunded' WHERE `transaction_id` = " . esc($parent_transaction['transaction_id']) . " LIMIT 1");
						}
					}

					/**
					 * If the capture payment is now complete
					 */
					if (isset(request::$post['auth_status']) && request::$post['auth_status'] == 'Completed' && $parent_transaction['payment_status'] == 'Pending') {
						$captured = currency::format(extension_payment_pp_express::getTotalCaptured($parent_transaction['paypal_order_id']), session::$data['currency'], false, false);
						$refunded = currency::format(extension_payment_pp_express::getRefundedTotal($parent_transaction['paypal_order_id']), session::$data['currency'], false, false);
						$remaining = currency::format($parent_transaction['amount'] - $captured + $refunded, session::$data['currency'], false, false);

						extension_payment_pp_express::log('Captured: ' . $captured, 'IPN data');
						extension_payment_pp_express::log('Refunded: ' . $refunded, 'IPN data');
						extension_payment_pp_express::log('Remaining: ' . $remaining, 'IPN data');

						if ($remaining > 0.00) {
							$transaction = array(
								'paypal_order_id'       => $parent_transaction['paypal_order_id'],
								'transaction_id'        => '',
								'parent_id' 			=> request::$post['parent_txn_id'],
								'note'                  => '',
								'msgsubid'              => '',
								'receipt_id'            => '',
								'payment_type'          => '',
								'payment_status'        => 'Void',
								'pending_reason'        => '',
								'amount'                => '',
								'debug_data'            => 'Voided after capture',
								'transaction_entity'    => 'auth'
							);

							extension_payment_pp_express::addTransaction($transaction);
						}

						extension_payment_pp_express::updateOrder('Complete', $parent_transaction['order_id']);
					}

				} else {
					//parent transaction doesn't exists, need to investigate?
					extension_payment_pp_express::log('Parent transaction not found', 'IPN data');
				}
			}

			/*
			 * Subscription payments
			 *
			 * recurring ID should always exist if its a recurring payment transaction.
			 *
			 * also the reference will match a recurring payment ID
			 */
			if (isset(request::$post['txn_type'])) {
				extension_payment_pp_express::log(request::$post['txn_type'], 'IPN data');

				//payment
				if (request::$post['txn_type'] == 'recurring_payment') {
					$recurring = account_recurring::getOrderRecurringByReference(request::$post['recurring_payment_id']);

					extension_payment_pp_express::log($recurring, 'IPN data');

					if ($recurring != false) {
						db::query("INSERT INTO `order_recurring_transaction` SET `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . ", `date_added` = NOW(), `amount` = '" . (float)request::$post['amount'] . "', `type` = '1'");

						//as there was a payment the recurring is active, ensure it is set to active (may be been suspended before)
						if ($recurring['status'] != 1) {
							db::query("UPDATE `order_recurring` SET `status` = 2 WHERE `order_recurring_id` = " . (int)$recurring['order_recurring_id']);
						}
					}
				}

				//suspend
				if (request::$post['txn_type'] == 'recurring_payment_suspended') {
					$recurring = account_recurring::getOrderRecurringByReference(request::$post['recurring_payment_id']);

					if ($recurring != false) {
						db::query("INSERT INTO `order_recurring_transaction` SET `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . ", `date_added` = NOW(), `type` = '6'");
						db::query("UPDATE `order_recurring` SET `status` = 3 WHERE `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . " LIMIT 1");
					}
				}

				//suspend due to max failed
				if (request::$post['txn_type'] == 'recurring_payment_suspended_due_to_max_failed_payment') {
					$recurring = account_recurring::getOrderRecurringByReference(request::$post['recurring_payment_id']);

					if ($recurring != false) {
						db::query("INSERT INTO `order_recurring_transaction` SET `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . ", `date_added` = NOW(), `type` = '7'");
						db::query("UPDATE `order_recurring` SET `status` = 3 WHERE `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . " LIMIT 1");
					}
				}

				//payment failed
				if (request::$post['txn_type'] == 'recurring_payment_failed') {
					$recurring = account_recurring::getOrderRecurringByReference(request::$post['recurring_payment_id']);

					if ($recurring != false) {
						db::query("INSERT INTO `order_recurring_transaction` SET `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . ", `date_added` = NOW(), `type` = '4'");
					}
				}

				//outstanding payment failed
				if (request::$post['txn_type'] == 'recurring_payment_outstanding_payment_failed') {
					$recurring = account_recurring::getOrderRecurringByReference(request::$post['recurring_payment_id']);

					if ($recurring != false) {
						db::query("INSERT INTO `order_recurring_transaction` SET `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . ", `date_added` = NOW(), `type` = '8'");
					}
				}

				//outstanding payment
				if (request::$post['txn_type'] == 'recurring_payment_outstanding_payment') {
					$recurring = account_recurring::getOrderRecurringByReference(request::$post['recurring_payment_id']);

					if ($recurring != false) {
						db::query("INSERT INTO `order_recurring_transaction` SET `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . ", `date_added` = NOW(), `amount` = '" . (float)request::$post['amount'] . "', `type` = '2'");

						//as there was a payment the recurring is active, ensure it is set to active (may be been suspended before)
						if ($recurring['status'] != 1) {
							db::query("UPDATE `order_recurring` SET `status` = 2 WHERE `order_recurring_id` = " . (int)$recurring['order_recurring_id']);
						}
					}
				}

				//date_added
				if (request::$post['txn_type'] == 'recurring_payment_profile_date_added') {
					$recurring = account_recurring::getOrderRecurringByReference(request::$post['recurring_payment_id']);

					if ($recurring != false) {
						db::query("INSERT INTO `order_recurring_transaction` SET `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . ", `date_added` = NOW(), `type` = '0'");

						if ($recurring['status'] != 1) {
							db::query("UPDATE `order_recurring` SET `status` = 2 WHERE `order_recurring_id` = " . (int)$recurring['order_recurring_id']);
						}
					}
				}

				//cancelled
				if (request::$post['txn_type'] == 'recurring_payment_profile_cancel') {
					$recurring = account_recurring::getOrderRecurringByReference(request::$post['recurring_payment_id']);

					if ($recurring != false && $recurring['status'] != 3) {
						db::query("INSERT INTO `order_recurring_transaction` SET `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . ", `date_added` = NOW(), `type` = '5'");
						db::query("UPDATE `order_recurring` SET `status` = 4 WHERE `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . " LIMIT 1");
					}
				}

				//skipped
				if (request::$post['txn_type'] == 'recurring_payment_skipped') {
					$recurring = account_recurring::getOrderRecurringByReference(request::$post['recurring_payment_id']);

					if ($recurring != false) {
						db::query("INSERT INTO `order_recurring_transaction` SET `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . ", `date_added` = NOW(), `type` = '3'");
					}
				}

				//expired
				if (request::$post['txn_type'] == 'recurring_payment_expired') {
					$recurring = account_recurring::getOrderRecurringByReference(request::$post['recurring_payment_id']);

					if ($recurring != false) {
						db::query("INSERT INTO `order_recurring_transaction` SET `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . ", `date_added` = NOW(), `type` = '9'");
						db::query("UPDATE `order_recurring` SET `status` = 5 WHERE `order_recurring_id` = " . (int)$recurring['order_recurring_id'] . " LIMIT 1");
					}
				}
			}
		} elseif ((string)$response == "INVALID") {
			extension_payment_pp_express::log(array('IPN was invalid'), 'IPN fail');
		} else {
			extension_payment_pp_express::log('Response string unknown: ' . (string)$response, 'IPN data');
		}

		header("HTTP/1.1 200 Ok");
	}

	public static function shipping() {
		self::shippingValidate(request::$post['shipping_method']);

		response::redirect(url::link('extension/payment/pp_express/expressConfirm'));
	}

	protected static function shippingValidate($code) {
		load::language('checkout/cart');
		load::language('extension/payment/pp_express');

		if (empty($code)) {
			session::$data['error_warning'] = language::get('error_shipping');
			return false;
		} else {
			$shipping = explode('.', $code);

			if (!isset($shipping[0]) || !isset($shipping[1]) || !isset(session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]])) {
				session::$data['error_warning'] = language::get('error_shipping');
				return false;
			} else {
				session::$data['shipping_method'] = session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]];
				session::$data['success'] = language::get('text_shipping_updated');
				return true;
			}
		}
	}

	protected static function validateCoupon() {

		$coupon_info = extension_total_coupon::getCoupon(request::$post['coupon']);

		if ($coupon_info) {
			return true;
		} else {
			session::$data['error_warning'] = language::get('error_coupon');
			return false;
		}
	}

	protected static function validateVoucher() {

		$voucher_info = extension_total_voucher::getVoucher(request::$post['voucher']);

		if ($voucher_info) {
			return true;
		} else {
			session::$data['error_warning'] = language::get('error_voucher');
			return false;
		}
	}

	protected static function validateReward() {
		$points = customer::getRewardPoints();

		$points_total = 0;

		foreach (cart::getProducts() as $product) {
			if ($product['points']) {
				$points_total += $product['points'];
			}
		}

		$error = '';

		if (empty(request::$post['reward'])) {
			$error = language::get('error_reward');
		}

		if (request::$post['reward'] > $points) {
			$error = sprintf(language::get('error_points'), request::$post['reward']);
		}

		if (request::$post['reward'] > $points_total) {
			$error = sprintf(language::get('error_maximum'), $points_total);
		}

		if (!$error) {
			return true;
		} else {
			session::$data['error_warning'] = $error;
			return false;
		}
	}
}
