<?php
class ControllerExtensionPaymentPPPayflowIframe {
	public static function index() {

		if(!isset(session::$data['order_id'])) {
			return false;
		}

		$order_info = checkout_order::getOrder(session::$data['order_id']);

		if (config::get('payment_pp_payflow_iframe_test')) {
			$mode = 'TEST';
		} else {
			$mode = 'LIVE';
		}

		$payflow_url = 'https://payflowlink.paypal.com';

		if (config::get('payment_pp_payflow_iframe_transaction_method') == 'sale') {
			$transaction_type = 'S';
		} else {
			$transaction_type = 'A';
		}

		$secure_token_id = md5(session::$data['order_id'] . mt_rand() . microtime());

		extension_payment_pp_payflow_iframe::addOrder($order_info['order_id'], $secure_token_id);

		$shipping_country = localisation_country::getCountry($order_info['shipping_country_id']);
		$shipping_zone = localisation_zone::getZone($order_info['shipping_zone_id']);

		$payment_country = localisation_country::getCountry($order_info['payment_country_id']);
		$payment_zone = localisation_zone::getZone($order_info['payment_zone_id']);

		$url_params = array(
			'TENDER'            => 'C',
			'TRXTYPE'           => $transaction_type,
			'AMT'               => currency::format($order_info['total'], $order_info['currency_code'], false, false),
			'CURRENCY'          => $order_info['currency_code'],
			'CREATESECURETOKEN' => 'Y',
			'SECURETOKENID'     => $secure_token_id,
			'BILLTOFIRSTNAME'   => $order_info['payment_firstname'],
			'BILLTOLASTNAME'    => $order_info['payment_lastname'],
			'BILLTOSTREET'      => trim($order_info['payment_address_1'] . ' ' . $order_info['payment_address_2']),
			'BILLTOCITY'        => $order_info['payment_city'],
			'BILLTOSTATE'       => $payment_zone['code'],
			'BILLTOZIP'         => $order_info['payment_postcode'],
			'BILLTOCOUNTRY'     => $payment_country['iso_code_2'],
		);

		if ($shipping_country) {
			$url_params['SHIPTOFIRSTNAME'] = $order_info['shipping_firstname'];
			$url_params['SHIPTOLASTNAME'] = $order_info['shipping_lastname'];
			$url_params['SHIPTOSTREET'] = trim($order_info['shipping_address_1'] . ' ' . $order_info['shipping_address_2']);
			$url_params['SHIPTOCITY'] = $order_info['shipping_city'];
			$url_params['SHIPTOSTATE'] = $shipping_zone['code'];
			$url_params['SHIPTOZIP'] = $order_info['shipping_postcode'];
			$url_params['SHIPTOCOUNTRY'] = $shipping_country['iso_code_2'];
		}

		$response_params = extension_payment_pp_payflow_iframe::call($url_params);

		if (isset($response_params['SECURETOKEN'])) {
			$secure_token = $response_params['SECURETOKEN'];
		} else {
			$secure_token = '';
		}

		$iframe_params = array(
			'MODE'          => $mode,
			'SECURETOKENID' => $secure_token_id,
			'SECURETOKEN'   => $secure_token,
		);

		$data['iframe_url'] = $payflow_url . '?' . http_build_query($iframe_params, '', "&");
		$data['checkout_method'] = config::get('payment_pp_payflow_iframe_checkout_method');
		$data['button_confirm'] = language::get('button_confirm');
		$data['create'] = HTTPS_SERVER . 'index.php?route=extension/payment/pp_pro_iframe/create';

		return load::view('extension/payment/pp_payflow_iframe', $data);
	}

	public static function paymentReturn() {
		$data['url'] = url::link('checkout/success');

		response::setOutput(load::view('extension/payment/pp_payflow_iframe_return', $data));
	}

	public static function paymentCancel() {
		$data['url'] = url::link('checkout/checkout');

		response::setOutput(load::view('extension/payment/pp_payflow_iframe_return', $data));
	}

	public static function paymentError() {
		$data['url'] = url::link('checkout/checkout');

		response::setOutput(load::view('extension/payment/pp_payflow_iframe_return', $data));
	}

	public static function paymentIpn() {

		if (config::get('payment_pp_pro_iframe_debug')) {
			$log = new Log('pp_pro_iframe.log');
			$log::write('POST: ' . print_r(request::$post, 1));
		}
						
		$order_id = extension_payment_pp_payflow_iframe::getOrderId(request::$post['SECURETOKENID']);

		if ($order_id) {
			$order_info = checkout_order::getOrder($order_id);

			$url_params = array(
				'TENDER'  => 'C',
				'TRXTYPE' => 'I',
				'ORIGID'  => request::$post['PNREF'],
			);

			$response_params = extension_payment_pp_payflow_iframe::call($url_params);

			if ($order_info['order_status_id'] == 0 && $response_params['RESULT'] == '0' && request::$post['RESULT'] == 0) {
				checkout_order::addOrderHistory($order_id, config::get('payment_pp_payflow_iframe_order_status_id'));

				if (request::$post['TYPE'] == 'S') {
					$complete = 1;
				} else {
					$complete = 0;
				}

				$data = array(
					'secure_token_id'       => request::$post['SECURETOKENID'],
					'transaction_reference' => request::$post['PNREF'],
					'transaction_type'      => request::$post['TYPE'],
					'complete'              => $complete,
				);

				extension_payment_pp_payflow_iframe::updateOrder($data);

				$data = array(
					'order_id'              => $order_id,
					'type'                  => request::$post['TYPE'],
					'transaction_reference' => request::$post['PNREF'],
					'amount'                => request::$post['AMT'],
				);

				extension_payment_pp_payflow_iframe::addTransaction($data);
			}
		}

		response::setOutput('Ok');
	}
}