<?php
class ControllerExtensionPaymentRealex {
	public static function index() {
		load::language('extension/payment/realex');

		$data['entry_cc_type'] = language::get('entry_cc_type');

		$data['help_select_card'] = language::get('help_select_card');

		$data['button_confirm'] = language::get('button_confirm');


		if(!isset(session::$data['order_id'])) {
			return false;
		}

		$order_info = checkout_order::getOrder(session::$data['order_id']);

		if (config::get('payment_realex_live_demo') == 1) {
			$data['action'] = config::get('payment_realex_live_url');
		} else {
			$data['action'] = config::get('payment_realex_demo_url');
		}

		if (config::get('payment_realex_card_select') == 1) {
			$card_types = array(
				'visa' => language::get('text_card_visa'),
				'mc' => language::get('text_card_mc'),
				'amex' => language::get('text_card_amex'),
				'switch' => language::get('text_card_switch'),
				'laser' => language::get('text_card_laser'),
				'diners' => language::get('text_card_diners'),
			);

			$data['cards'] = array();

			$accounts = config::get('payment_realex_account');

			foreach ($accounts as $card => $account) {
				if (isset($account['enabled']) && $account['enabled'] == 1) {
					$data['cards'][] = array(
						'type' => $card_types[$card],
						'account' => (isset($account['default']) && $account['default'] == 1 ? config::get('payment_realex_merchant_id') : $account['merchant_id']),
					);
				}
			}

			$data['card_select'] = true;
		} else {
			$data['card_select'] = false;
		}

		if (config::get('payment_realex_auto_settle') == 0) {
			$data['settle'] = 0;
		} elseif (config::get('payment_realex_auto_settle') == 1) {
			$data['settle'] = 1;
		} elseif (config::get('payment_realex_auto_settle') == 2) {
			$data['settle'] = 'MULTI';
		}

		$data['tss'] = (int)config::get('payment_realex_tss_check');
		$data['merchant_id'] = config::get('payment_realex_merchant_id');

		$data['timestamp'] = date("YmdHis");
		$data['order_id'] = session::$data['order_id'] . 'T' . $data['timestamp'] . mt_rand(1, 999);

		$data['amount'] = round(currency::format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false) * 100);
		$data['currency'] = $order_info['currency_code'];

		$tmp = $data['timestamp'] . '.' . $data['merchant_id'] . '.' . $data['order_id'] . '.' . $data['amount'] . '.' . $data['currency'];
		$hash = sha1($tmp);
		$tmp = $hash . '.' . config::get('payment_realex_secret');
		$data['hash'] = sha1($tmp);

		$data['billing_code'] = filter_var(str_replace('-', '', $order_info['payment_postcode']), FILTER_SANITIZE_NUMBER_INT) . '|' . filter_var(str_replace('-', '', $order_info['payment_address_1']), FILTER_SANITIZE_NUMBER_INT);
		$data['payment_country'] = $order_info['payment_iso_code_2'];

		if (cart::hasShipping()) {
			$data['shipping_code'] = filter_var(str_replace('-', '', $order_info['shipping_postcode']), FILTER_SANITIZE_NUMBER_INT) . '|' . filter_var(str_replace('-', '', $order_info['shipping_address_1']), FILTER_SANITIZE_NUMBER_INT);
			$data['shipping_country'] = $order_info['shipping_iso_code_2'];
		} else {
			$data['shipping_code'] = filter_var(str_replace('-', '', $order_info['payment_postcode']), FILTER_SANITIZE_NUMBER_INT) . '|' . filter_var(str_replace('-', '', $order_info['payment_address_1']), FILTER_SANITIZE_NUMBER_INT);
			$data['shipping_country'] = $order_info['payment_iso_code_2'];
		}

		$data['response_url'] = HTTPS_SERVER . 'index.php?route=extension/payment/realex/notify';

		return load::view('extension/payment/realex', $data);
	}

	public static function notify() {

		extension_payment_realex::logger(print_r(request::$post, 1));

		load::language('extension/payment/realex');

		$hash = sha1(request::$post['TIMESTAMP'] . '.' . config::get('payment_realex_merchant_id') . '.' . request::$post['ORDER_ID'] . '.' . request::$post['RESULT'] . '.' . request::$post['MESSAGE'] . '.' . request::$post['PASREF'] . '.' . request::$post['AUTHCODE']);
		$tmp = $hash . '.' . config::get('payment_realex_secret');
		$hash = sha1($tmp);

		//Check to see if hashes match or not
		if ($hash != request::$post['SHA1HASH']) {
			$data['text_response'] = language::get('text_hash_failed');
			$data['text_link'] = sprintf(language::get('text_link'), url::link('checkout/checkout', ''));
		} else {

			$order_id_parts = explode('T', request::$post['ORDER_ID']);
			$order_id = (int)$order_id_parts[0];

			$order_info = checkout_order::getOrder($order_id);

			$auto_settle = (int)config::get('payment_realex_auto_settle');
			$tss = (int)config::get('payment_realex_tss_check');

			$message = '<strong>' . language::get('text_result') . ':</strong> ' . request::$post['RESULT'];
			$message .= '<br /><strong>' . language::get('text_message') . ':</strong> ' . request::$post['MESSAGE'];

			if (isset(request::$post['ORDER_ID'])) {
				$message .= '<br /><strong>' . language::get('text_order_ref') . ':</strong> ' . request::$post['ORDER_ID'];
			}

			if (isset(request::$post['CVNRESULT'])) {
				$message .= '<br /><strong>' . language::get('text_cvn_result') . ':</strong> ' . request::$post['CVNRESULT'];
			}

			if (isset(request::$post['AVSPOSTCODERESULT'])) {
				$message .= '<br /><strong>' . language::get('text_avs_postcode') . ':</strong> ' . request::$post['AVSPOSTCODERESULT'];
			}

			if (isset(request::$post['AVSADDRESSRESULT'])) {
				$message .= '<br /><strong>' . language::get('text_avs_address') . ':</strong> ' . request::$post['AVSADDRESSRESULT'];
			}

			//3D Secure message
			if (isset(request::$post['ECI']) && isset(request::$post['CAVV']) && isset(request::$post['XID'])) {
				$eci = request::$post['ECI'];

				if ((request::$post['ECI'] == 6 || request::$post['ECI'] == 1) && empty(request::$post['CAVV']) && empty(request::$post['XID'])) {
					$scenario_id = 1;
				}

				if ((request::$post['ECI'] == 5 || request::$post['ECI'] == 0) && !empty(request::$post['CAVV']) && !empty(request::$post['XID'])) {
					$scenario_id = 5;
				}

				if ((request::$post['ECI'] == 6 || request::$post['ECI'] == 1) && !empty(request::$post['CAVV']) && !empty(request::$post['XID'])) {
					$scenario_id = 6;
				}

				if (isset($scenario_id)) {
					$scenario_message = language::get('text_3d_s' . $scenario_id);
				} else {
					if (isset(request::$post['CARDTYPE'])) {
						if (request::$post['CARDTYPE'] == 'VISA') {
							$eci = 7;
						} else {
							$eci = 2;
						}
					}

					$scenario_message = language::get('text_3d_liability');
				}

				$message .= '<br /><strong>' . language::get('text_eci') . ':</strong> (' . $eci . ') ' . $scenario_message;
			}

			if ($tss == 1 && isset(request::$post['TSS'])) {
				$message .= '<br /><strong>' . language::get('text_tss') . ':</strong> ' . request::$post['TSS'];
			}

			if (isset(request::$post['TIMESTAMP'])) {
				$message .= '<br /><strong>' . language::get('text_timestamp') . ':</strong> ' . request::$post['TIMESTAMP'];
			}

			if (isset(request::$post['CARDDIGITS'])) {
				$message .= '<br /><strong>' . language::get('text_card_digits') . ':</strong> ' . request::$post['CARDDIGITS'];
			}

			if (isset(request::$post['CARDTYPE'])) {
				$message .= '<br /><strong>' . language::get('text_card_type') . ':</strong> ' . request::$post['CARDTYPE'];
			}

			if (isset(request::$post['EXPDATE'])) {
				$message .= '<br /><strong>' . language::get('text_card_exp') . ':</strong> ' . request::$post['EXPDATE'];
			}

			if (isset(request::$post['CARDNAME'])) {
				$message .= '<br /><strong>' . language::get('text_card_name') . ':</strong> ' . request::$post['CARDNAME'];
			}

			if (isset(request::$post['DCCAUTHCARDHOLDERAMOUNT']) && isset(request::$post['DCCAUTHRATE'])) {
				$message .= '<br /><strong>DCCAUTHCARDHOLDERAMOUNT:</strong> ' . request::$post['DCCAUTHCARDHOLDERAMOUNT'];
				$message .= '<br /><strong>DCCAUTHRATE:</strong> ' . request::$post['DCCAUTHRATE'];
				$message .= '<br /><strong>DCCAUTHCARDHOLDERCURRENCY:</strong> ' . request::$post['DCCAUTHCARDHOLDERCURRENCY'];
				$message .= '<br /><strong>DCCAUTHMERCHANTCURRENCY:</strong> ' . request::$post['DCCAUTHMERCHANTCURRENCY'];
				$message .= '<br /><strong>DCCAUTHMERCHANTAMOUNT:</strong> ' . request::$post['DCCAUTHMERCHANTAMOUNT'];
				$message .= '<br /><strong>DCCCCP:</strong> ' . request::$post['DCCCCP'];
				$message .= '<br /><strong>DCCRATE:</strong> ' . request::$post['DCCRATE'];
				$message .= '<br /><strong>DCCMARGINRATEPERCENTAGE:</strong> ' . request::$post['DCCMARGINRATEPERCENTAGE'];
				$message .= '<br /><strong>DCCEXCHANGERATESOURCENAME:</strong> ' . request::$post['DCCEXCHANGERATESOURCENAME'];
				$message .= '<br /><strong>DCCCOMMISSIONPERCENTAGE:</strong> ' . request::$post['DCCCOMMISSIONPERCENTAGE'];
				$message .= '<br /><strong>DCCEXCHANGERATESOURCETIMESTAMP:</strong> ' . request::$post['DCCEXCHANGERATESOURCETIMESTAMP'];
				$message .= '<br /><strong>DCCCHOICE:</strong> ' . request::$post['DCCCHOICE'];
			}

			if (request::$post['RESULT'] == "00") {
				$realex_order_id = extension_payment_realex::addOrder($order_info, request::$post['PASREF'], request::$post['AUTHCODE'], request::$post['ACCOUNT'], request::$post['ORDER_ID']);

				if ($auto_settle == 1) {
					extension_payment_realex::addTransaction($realex_order_id, 'payment', $order_info);
					checkout_order::addOrderHistory($order_id, config::get('payment_realex_order_status_success_settled_id'), $message, false);
				} else {
					extension_payment_realex::addTransaction($realex_order_id, 'auth', 0.00);
					checkout_order::addOrderHistory($order_id, config::get('payment_realex_order_status_success_unsettled_id'), $message, false);
				}

				$data['text_response'] = language::get('text_success');
				$data['text_link'] = sprintf(language::get('text_link'), url::link('checkout/success', ''));
			} elseif (request::$post['RESULT'] == "101") {
				// Decline
				extension_payment_realex::addHistory($order_id, config::get('payment_realex_order_status_decline_id'), $message);
				$data['text_response'] = language::get('text_decline');
				$data['text_link'] = sprintf(language::get('text_link'), url::link('checkout/checkout', ''));
			} elseif (request::$post['RESULT'] == "102") {
				// Referal B
				extension_payment_realex::addHistory($order_id, config::get('payment_realex_order_status_decline_pending_id'), $message);
				$data['text_response'] = language::get('text_decline');
				$data['text_link'] = sprintf(language::get('text_link'), url::link('checkout/checkout', ''));
			} elseif (request::$post['RESULT'] == "103") {
				// Referal A
				extension_payment_realex::addHistory($order_id, config::get('payment_realex_order_status_decline_stolen_id'), $message);
				$data['text_response'] = language::get('text_decline');
				$data['text_link'] = sprintf(language::get('text_link'), url::link('checkout/checkout', ''));
			} elseif (request::$post['RESULT'] == "200") {
				// Error Connecting to Bank
				extension_payment_realex::addHistory($order_id, config::get('payment_realex_order_status_decline_bank_id'), $message);
				$data['text_response'] = language::get('text_bank_error');
				$data['text_link'] = sprintf(language::get('text_link'), url::link('checkout/checkout', ''));
			} elseif (request::$post['RESULT'] == "204") {
				// Error Connecting to Bank
				extension_payment_realex::addHistory($order_id, config::get('payment_realex_order_status_decline_bank_id'), $message);
				$data['text_response'] = language::get('text_bank_error');
				$data['text_link'] = sprintf(language::get('text_link'), url::link('checkout/checkout', ''));
			} elseif (request::$post['RESULT'] == "205") {
				// Comms Error
				extension_payment_realex::addHistory($order_id, config::get('payment_realex_order_status_decline_bank_id'), $message);
				$data['text_response'] = language::get('text_bank_error');
				$data['text_link'] = sprintf(language::get('text_link'), url::link('checkout/checkout', ''));
			} else {
				// Other error
				extension_payment_realex::addHistory($order_id, config::get('payment_realex_order_status_decline_id'), $message);
				$data['text_response'] = language::get('text_generic_error');
				$data['text_link'] = sprintf(language::get('text_link'), url::link('checkout/checkout', ''));
			}
		}

		response::setOutput(load::view('extension/payment/realex_response', $data));
	}
}