<?php
class ControllerExtensionPaymentSecureTradingPp {
	public static function index() {
		load::language('extension/payment/securetrading_pp');

		if(!isset(session::$data['order_id'])) {
			return false;
		}

		$order_info = checkout_order::getOrder(session::$data['order_id']);

		if ($order_info) {
			$data['order_info'] = $order_info;
			$data['site_reference'] = config::get('payment_securetrading_pp_site_reference');
			$data['version'] = config::get('payment_securetrading_pp_version');
			$data['parent_css'] = config::get('payment_securetrading_pp_parent_css');
			$data['child_css'] = config::get('payment_securetrading_pp_child_css');
			$data['currency'] = $order_info['currency_code'];
			$data['total'] = currency::format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);
			$data['settle_due_date'] = date('Y-m-d', strtotime(date('Y-m-d') . ' +' . config::get('payment_securetrading_pp_settle_due_date') . ' days'));
			$data['settle_status'] = config::get('payment_securetrading_pp_settle_status');

			$payment_country = localisation_country::getCountry($order_info['payment_country_id']);
			$payment_zone = localisation_zone::getZone($order_info['payment_zone_id']);

			$shipping_country = localisation_country::getCountry($order_info['shipping_country_id']);
			$shipping_zone = localisation_zone::getZone($order_info['shipping_zone_id']);

			if ($payment_country['iso_code_3'] == 'USA') {
				$data['billing_county'] = $payment_zone['code'];
			} else {
				$data['billing_county'] = $order_info['payment_zone'];
			}

			if (isset($shipping_country['iso_code_3']) && $shipping_country['iso_code_3'] == 'USA') {
				$data['shipping_county'] = $shipping_zone['code'];
			} else {
				$data['shipping_county'] = $order_info['shipping_zone'];
			}

			if (!isset($shipping_country['iso_code_2'])) {
				$shipping_country['iso_code_2'] = $payment_country['iso_code_2'];
			}

			$data['payment_country'] = $payment_country;
			$data['shipping_country'] = $shipping_country;

			if (config::get('payment_securetrading_pp_site_security_status')) {
				$data['site_security'] = hash('sha256', $order_info['currency_code'] . $data['total'] . $data['site_reference'] . $data['settle_status'] . $data['settle_due_date'] . $order_info['order_id'] . config::get('payment_securetrading_pp_site_security_password'));
			} else {
				$data['site_security'] = false;
			}

			$cards = array(
				'AMEX' => 'American Express',
				'VISA' => 'Visa',
				'DELTA' => 'Visa Debit',
				'ELECTRON' => 'Visa Electron',
				'PURCHASING' => 'Visa Purchasing',
				'VPAY' => 'V Pay',
				'MASTERCARD' => 'MasterCard',
				'MASTERCARDDEBIT' => 'MasterCard Debit',
				'MAESTRO' => 'Maestro',
				'PAYPAL' => 'PayPal',
			);

			$data['cards'] = array();

			foreach ($cards as $key => $value) {
				if (in_array($key, config::get('payment_securetrading_pp_cards_accepted'))) {
					$data['cards'][$key] = $value;
				}
			}

			$data['button_confirm'] = language::get('button_confirm');
			$data['text_payment_details'] = language::get('text_payment_details');
			$data['entry_card_type'] = language::get('entry_card_type');

			return load::view('extension/payment/securetrading_pp', $data);
		}
	}

	public static function ipn() {
		load::language('extension/payment/securetrading_pp');

		$keys = array_keys(request::$post);
		sort($keys);

		$keys_ignore = array('notificationreference', 'responsesitesecurity');

		$string_to_hash = '';

		foreach ($keys as $key) {
			if (!in_array($key, $keys_ignore)) {
				$string_to_hash .= request::$post[$key];
			}
		}

		$string_to_hash .= config::get('payment_securetrading_pp_notification_password');

		if (hash_equals(hash('sha256', $string_to_hash), request::$post['responsesitesecurity']) && request::$post['sitereference'] == config::get('payment_securetrading_pp_site_reference')) {
			$order_info = checkout_order::getOrder(request::$post['orderreference']);

			if ($order_info) {
				$order_total = currency::format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);

				if ($order_total == request::$post['mainamount'] && $order_info['currency_code'] == request::$post['currencyiso3a'] && $order_info['payment_code'] == 'securetrading_pp') {
					$status_code_mapping = array(
						0 => language::get('text_not_given'),
						1 => language::get('text_not_checked'),
						2 => language::get('text_match'),
						4 => language::get('text_not_match'),
					);
					$shipping_country = extension_payment_securetrading_pp::getCountry(request::$post['customercountryiso2a']);
					$payment_country = extension_payment_securetrading_pp::getCountry(request::$post['billingcountryiso2a']);

					$order_info['payment_firstname'] = request::$post['billingfirstname'];
					$order_info['payment_lastname'] = request::$post['billinglastname'];
					$order_info['payment_address_1'] = request::$post['billingpremise'];
					$order_info['payment_address_2'] = request::$post['billingstreet'];
					$order_info['payment_city'] = request::$post['billingtown'];
					$order_info['payment_zone'] = request::$post['billingcounty'];
					$order_info['payment_zone_id'] = 0;
					$order_info['payment_country'] = $payment_country['name'];
					$order_info['payment_country_id'] = $payment_country['country_id'];
					$order_info['payment_postcode'] = request::$post['billingpostcode'];

					$order_info['shipping_firstname'] = request::$post['customerfirstname'];
					$order_info['shipping_lastname'] = request::$post['customerlastname'];
					$order_info['shipping_address_1'] = request::$post['customerpremise'];
					$order_info['shipping_address_2'] = request::$post['customerstreet'];
					$order_info['shipping_city'] = request::$post['customertown'];
					$order_info['shipping_zone'] = request::$post['customercounty'];
					$order_info['shipping_zone_id'] = 0;
					$order_info['shipping_country'] = $shipping_country['name'];
					$order_info['shipping_country_id'] = $shipping_country['country_id'];
					$order_info['shipping_postcode'] = request::$post['customerpostcode'];

					extension_payment_securetrading_pp::editOrder($order_info['order_id'], $order_info);

					$postcode_status = request::$post['securityresponsepostcode'];
					$security_code_status = request::$post['securityresponsesecuritycode'];
					$address_status = request::$post['securityresponseaddress'];

					$message = sprintf(language::get('text_postcode_check'), $status_code_mapping[$postcode_status]) . "\n";
					$message .= sprintf(language::get('text_security_code_check'), $status_code_mapping[$security_code_status]) . "\n";
					$message .= sprintf(language::get('text_address_check'), $status_code_mapping[$address_status]) . "\n";

					if (isset(request::$post['transactionreference'])) {
						$transactionreference = request::$post['transactionreference'];
					} else {
						$transactionreference = '';
					}
					extension_payment_securetrading_pp::addReference($order_info['order_id'], $transactionreference);

					if (request::$post['errorcode'] == '0') {
						$order_status_id = config::get('payment_securetrading_pp_order_status_id');

						extension_payment_securetrading_pp::confirmOrder($order_info['order_id'], $order_status_id);
						extension_payment_securetrading_pp::updateOrder($order_info['order_id'], $order_status_id, $message);
					} elseif (request::$post['errorcode'] == '70000') {
						$order_status_id = config::get('payment_securetrading_pp_declined_order_status_id');

						extension_payment_securetrading_pp::updateOrder($order_info['order_id'], $order_status_id, $message);
					}
				}
			}
		}
	}
}