<?php

class ControllerExtensionPaymentSquareup {
    public static function index() {
        load::language('extension/payment/squareup');

        load::library('squareup');

        $data['action'] = url::link('extension/payment/squareup/checkout', '');
        $data['squareup_js_api'] = Squareup::$PAYMENT_FORM_URL;
        
        if (!empty(session::$data['payment_address']['postcode'])) {
            $data['payment_zip'] = session::$data['payment_address']['postcode'];
        } else {
            $data['payment_zip'] = '';
        }

        if (config::get('payment_squareup_enable_sandbox')) {
            $data['app_id'] = config::get('payment_squareup_sandbox_client_id');
            $data['sandbox_message'] = language::get('warning_test_mode');
        } else {
            $data['app_id'] = config::get('payment_squareup_client_id');
            $data['sandbox_message'] = '';
        }

        $data['cards'] = array();

        if (customer::isLogged()) {
            $data['is_logged'] = true;


            $cards = extension_credit_card_squareup::getCards(customer::getId(), config::get('payment_squareup_enable_sandbox'));

            foreach ($cards as $card) {
                $data['cards'][] = array(
                    'id' => $card['squareup_token_id'],
                    'text' => sprintf(language::get('text_card_ends_in'), $card['brand'], $card['ends_in'])
                );
            }
        } else {
            $data['is_logged'] = false;
        }

        return load::view('extension/payment/squareup', $data);
    }

    public static function checkout() {
        load::language('extension/payment/squareup');


        load::library('squareup');

        if(!isset(session::$data['order_id'])) {
			return false;
		}

        $order_info = checkout_order::getOrder(session::$data['order_id']);

        $shipping_country_info = localisation_country::getCountry($order_info['shipping_country_id']);

        $billing_country_info = localisation_country::getCountry($order_info['payment_country_id']);

        if (!empty($billing_country_info)) {
            $billing_address = array(
                'first_name' => $order_info['payment_firstname'],
                'last_name' => $order_info['payment_lastname'],
                'address_line_1' => $order_info['payment_address_1'],
                'address_line_2' => $order_info['payment_address_2'],
                'locality' => $order_info['payment_city'],
                'sublocality' => $order_info['payment_zone'],
                'postal_code' => $order_info['payment_postcode'],
                'country' => $billing_country_info['iso_code_2'],
                'organization' => $order_info['payment_company']
            );
        } else {
            $billing_address = array();
        }

        if (!empty($shipping_country_info)) {
            $shipping_address = array(
                'first_name' => $order_info['shipping_firstname'],
                'last_name' => $order_info['shipping_lastname'],
                'address_line_1' => $order_info['shipping_address_1'],
                'address_line_2' => $order_info['shipping_address_2'],
                'locality' => $order_info['shipping_city'],
                'sublocality' => $order_info['shipping_zone'],
                'postal_code' => $order_info['shipping_postcode'],
                'country' => $shipping_country_info['iso_code_2'],
                'organization' => $order_info['shipping_company']
            );
        } else {
            $shipping_address = array();
        }

        $json = array();

        try {
            // Ensure we have registered the customer with Square
            $square_customer = extension_credit_card_squareup::getCustomer(customer::getId(), config::get('payment_squareup_enable_sandbox'));

            if (!$square_customer && customer::isLogged()) {
                $square_customer = self::$squareup::addLoggedInCustomer();

                extension_credit_card_squareup::addCustomer($square_customer);
            }

            $use_saved = false;
            $square_card_id = null;

            // check if user is logged in and wanted to save this card
            if (customer::isLogged() && !empty(request::$post['squareup_select_card'])) {
                $card_verified = extension_credit_card_squareup::verifyCardCustomer(request::$post['squareup_select_card'], customer::getId());

                if (!$card_verified) {
                    throw new \Squareup\Exception(language::get('error_card_invalid'));
                }

                $card = extension_credit_card_squareup::getCard(request::$post['squareup_select_card']);

                $use_saved = true;
                $square_card_id = $card['token'];
            } else if (customer::isLogged() && isset(request::$post['squareup_save_card'])) {
                // Save the card
                $card_data = array(
                    'card_nonce' => request::$post['squareup_nonce'],
                    'billing_address' => $billing_address,
                    'cardholder_name' => $order_info['payment_firstname'] . ' ' . $order_info['payment_lastname']
                );

                $square_card = self::$squareup::addCard($square_customer['square_customer_id'], $card_data);

                if (!extension_credit_card_squareup::cardExists(customer::getId(), $square_card)) {
                    extension_credit_card_squareup::addCard(customer::getId(), config::get('payment_squareup_enable_sandbox'), $square_card);
                }
                
                $use_saved = true;
                $square_card_id = $square_card['id'];
            }

            // Prepare Transaction
            $transaction_data = array(
                'idempotency_key' => uniqid(),
                'amount_money' => array(
                    'amount' => self::$squareup::lowestDenomination($order_info['total'], $order_info['currency_code']),
                    'currency' => $order_info['currency_code']
                ),
                'billing_address' => $billing_address,
                'buyer_email_address' => $order_info['email'],
                'delay_capture' => !cart::hasRecurringProducts() && config::get('payment_squareup_delay_capture'),
                'integration_id' => Squareup::$SQUARE_INTEGRATION_ID
            );
            
            if (!empty($shipping_address)) {
                $transaction_data['shipping_address'] = $shipping_address;
            }

            if ($use_saved) {
                $transaction_data['customer_card_id'] = $square_card_id;
                $transaction_data['customer_id'] = $square_customer['square_customer_id'];
            } else {
                $transaction_data['card_nonce'] = request::$post['squareup_nonce'];
            }

            $transaction = self::$squareup::addTransaction($transaction_data);

            if (isset(request::$server['HTTP_USER_AGENT'])) {
                $user_agent = request::$server['HTTP_USER_AGENT'];
            } else {
                $user_agent = '';
            }

            if (isset(request::$server['REMOTE_ADDR'])) {
                $ip = request::$server['REMOTE_ADDR'];
            } else {
                $ip = '';
            }

            extension_payment_squareup::addTransaction($transaction, config::get('payment_squareup_merchant_id'), $billing_address, session::$data['order_id'], $user_agent, $ip);

            if (!empty($transaction['tenders'][0]['card_details']['status'])) {
                $transaction_status = strtolower($transaction['tenders'][0]['card_details']['status']);
            } else {
                $transaction_status = '';
            }

            $order_status_id = config::get('payment_squareup_status_' . $transaction_status);

            if ($order_status_id) {
                if (cart::hasRecurringProducts() && $transaction_status == 'captured') {
                    foreach (cart::getRecurringProducts() as $item) {
                        if ($item['recurring']['trial']) {
                            $trial_price = tax::calculate($item['recurring']['trial_price'] * $item['quantity'], $item['tax_class_id']);
                            $trial_amt = currency::format($trial_price, session::$data['currency']);
                            $trial_text =  sprintf(language::get('text_trial'), $trial_amt, $item['recurring']['trial_cycle'], $item['recurring']['trial_frequency'], $item['recurring']['trial_duration']);

                            $item['recurring']['trial_price'] = $trial_price;
                        } else {
                            $trial_text = '';
                        }

                        $recurring_price = tax::calculate($item['recurring']['price'] * $item['quantity'], $item['tax_class_id']);
                        $recurring_amt = currency::format($recurring_price, session::$data['currency']);
                        $recurring_description = $trial_text . sprintf(language::get('text_recurring'), $recurring_amt, $item['recurring']['cycle'], $item['recurring']['frequency']);

                        $item['recurring']['price'] = $recurring_price;

                        if ($item['recurring']['duration'] > 0) {
                            $recurring_description .= sprintf(language::get('text_length'), $item['recurring']['duration']);
                        }

                        if (!$item['recurring']['trial']) {
                            // We need to override this value for the proper calculation in updateRecurringExpired
                            $item['recurring']['trial_duration'] = 0;
                        }


                        extension_payment_squareup::createRecurring($item, session::$data['order_id'], $recurring_description, $transaction['id']);
                    }
                }

                $order_status_comment = language::get('squareup_status_comment_' . $transaction_status);

                checkout_order::addOrderHistory(session::$data['order_id'], $order_status_id, $order_status_comment, true);
            }

            $json['redirect'] = url::link('checkout/success', '');
        } catch (\Squareup\Exception $e) {
            if ($e::isCurlError()) {
                $json['error'] = language::get('text_token_issue_customer_error');
            } else if ($e::isAccessTokenRevoked()) {
                // Send reminder e-mail to store admin to refresh the token
                extension_payment_squareup::tokenRevokedEmail();

                $json['error'] = language::get('text_token_issue_customer_error');
            } else if ($e::isAccessTokenExpired()) {
                // Send reminder e-mail to store admin to refresh the token
                extension_payment_squareup::tokenExpiredEmail();

                $json['error'] = language::get('text_token_issue_customer_error');
            } else {
                $json['error'] = $e::getMessage();
            }
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    
}
