<?php
/**
 * @package		OpenCart
 * @author		Meng Wenbin
 * @copyright	Copyright (c) 2010 - 2017, Chengdu Guangda Network Technology Co. Ltd. (https://www.opencart.cn/)
 * @license		https://opensource.org/licenses/GPL-3.0
 * @link		https://www.opencart.cn
 */

class ControllerExtensionPaymentWechatPay {
	public static function index() {
		$data['button_confirm'] = language::get('button_confirm');

		$data['redirect'] = url::link('extension/payment/wechat_pay/qrcode');

		return load::view('extension/payment/wechat_pay', $data);
	}

	public static function qrcode() {
		load::language('extension/payment/wechat_pay');

		document::setTitle(language::get('heading_title'));
		document::addScript('catalog/view/javascript/qrcode.js');

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_checkout'),
			'href' => url::link('checkout/checkout', '')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_qrcode'),
			'href' => url::link('extension/payment/wechat_pay/qrcode')
		);


		if(!isset(session::$data['order_id'])) {
			return false;
		}

		$order_info = checkout_order::getOrder(session::$data['order_id']);

		$order_id = trim($order_info['order_id']);
		$data['order_id'] = $order_id;
		$subject = trim(config('name'));
		$currency = config::get('payment_wechat_pay_currency');
		$total_amount = trim(currency::format($order_info['total'], $currency, '', false));
		$notify_url = HTTPS_SERVER . "payment_callback/wechat_pay"; //url::link('wechat_pay/callback');

		$options = array(
			'appid'			 =>  config::get('payment_wechat_pay_app_id'),
			'appsecret'		 =>  config::get('payment_wechat_pay_app_secret'),
			'mch_id'			=>  config::get('payment_wechat_pay_mch_id'),
			'partnerkey'		=>  config::get('payment_wechat_pay_api_secret')
		);

		\Wechat\Loader::config($options);
		$pay = new \Wechat\WechatPay();

		$result = $pay::getPrepayId(NULL, $subject, $order_id, $total_amount * 100, $notify_url, $trade_type = "NATIVE", NULL, $currency);

		$data['error'] = '';
		$data['code_url'] = '';
		if($result === FALSE){
			$data['error_warning'] = $pay::$errMsg;
		} else {
			$data['code_url'] = $result;
		}

		$data['action_success'] = url::link('checkout/success');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('extension/payment/wechat_pay_qrcode', $data));
	}

	public static function isOrderPaid() {
		$json = array();

		$json['result'] = false;

		if (isset(request::$get['order_id'])) {
			$order_id = request::$get['order_id'];

			$order_info = checkout_order::getOrder($order_id);

			if ($order_info['order_status_id'] == config::get('payment_wechat_pay_completed_status_id')) {
				$json['result'] = true;
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function callback() {
		$options = array(
			'appid'			 =>  config::get('payment_wechat_pay_app_id'),
			'appsecret'		 =>  config::get('payment_wechat_pay_app_secret'),
			'mch_id'			=>  config::get('payment_wechat_pay_mch_id'),
			'partnerkey'		=>  config::get('payment_wechat_pay_api_secret')
		);

		\Wechat\Loader::config($options);
		$pay = new \Wechat\WechatPay();
		$notifyInfo = $pay::getNotify();

		if ($notifyInfo === FALSE) {
			log::write('Wechat Pay Error: ' . $pay::$errMsg);
		} else {
			if ($notifyInfo['result_code'] == 'SUCCESS' && $notifyInfo['return_code'] == 'SUCCESS') {
				$order_id = $notifyInfo['out_trade_no'];
				$order_info = checkout_order::getOrder($order_id);
				if ($order_info) {
					$order_status_id = $order_info["order_status_id"];
					if (!$order_status_id) {
						checkout_order::addOrderHistory($order_id, config::get('payment_wechat_pay_completed_status_id'));
					}
				}
				return xml(['return_code' => 'SUCCESS', 'return_msg' => 'DEAL WITH SUCCESS']);
			}
		}
	}
}
