<?php
class ControllerExtensionPaymentWorldpay {
	public static function index() {
		load::language('extension/payment/worldpay');

		$data['worldpay_script'] = 'https://cdn.worldpay.com/v1/worldpay.js';

		$data['worldpay_client_key'] = config::get('payment_worldpay_client_key');

		$data['form_submit'] = url::link('extension/payment/worldpay/send', '');

		if (config::get('payment_worldpay_card') == '1' && customer::isLogged()) {
			$data['payment_worldpay_card'] = true;
		} else {
			$data['payment_worldpay_card'] = false;
		}

		$data['existing_cards'] = array();

		if (customer::isLogged() && $data['payment_worldpay_card']) {
			$data['existing_cards'] = extension_payment_worldpay::getCards(customer::getId());
		}

		$recurring_products = cart::getRecurringProducts();

		if (!empty($recurring_products)) {
			$data['recurring_products'] = true;
		}

		return load::view('extension/payment/worldpay', $data);
	}

	public static function send() {
		load::language('extension/payment/worldpay');

		if(!isset(session::$data['order_id'])) {
			return false;
		}

		$order_info = checkout_order::getOrder(session::$data['order_id']);

		$recurring_products = cart::getRecurringProducts();

		if (empty($recurring_products)) {
			$order_type = 'ECOM';
		} else {
			$order_type = 'RECURRING';
		}

		$country_info = localisation_country::getCountry($order_info['payment_country_id']);

		$billing_address = array(
			"address1" => $order_info['payment_address_1'],
			"address2" => $order_info['payment_address_2'],
			"address3" => '',
			"postalCode" => $order_info['payment_postcode'],
			"city" => $order_info['payment_city'],
			"state" => $order_info['payment_zone'],
			"countryCode" => $country_info['iso_code_2'],
		);

		$order = array(
			"token" => request::$post['token'],
			"orderType" => $order_type,
			"amount" => round(currency::format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false)*100),
			"currencyCode" => $order_info['currency_code'],
			"name" => $order_info['firstname'] . ' ' . $order_info['lastname'],
			"orderDescription" => $order_info['store_name'] . ' - ' . date('Y-m-d H:i:s'),
			"customerOrderCode" => $order_info['order_id'],
			"billingAddress" => $billing_address
		);

		extension_payment_worldpay::logger($order);

		$response_data = extension_payment_worldpay::sendCurl('orders', $order);

		extension_payment_worldpay::logger($response_data);

		if (isset($response_data::$paymentStatus) && $response_data::$paymentStatus == 'SUCCESS') {
			checkout_order::addOrderHistory($order_info['order_id'], config('order_status_id'));

			$worldpay_order_id = extension_payment_worldpay::addOrder($order_info, $response_data::$orderCode);

			extension_payment_worldpay::addTransaction($worldpay_order_id, 'payment', $order_info);

			if (isset(request::$post['save-card'])) {
				$response = extension_payment_worldpay::sendCurl('tokens/' . request::$post['token']);

				extension_payment_worldpay::logger($response);

				$expiry_date = mktime(0, 0, 0, 0, (string)$response::$paymentMethod::$expiryMonth, (string)$response::$paymentMethod::$expiryYear);

				if (isset($response::$paymentMethod)) {
					$card_data = array();
					$card_data['customer_id'] = customer::getId();
					$card_data['Token'] = $response::$token;
					$card_data['Last4Digits'] = (string)$response::$paymentMethod::$maskedCardNumber;
					$card_data['ExpiryDate'] = date("m/y", $expiry_date);
					$card_data['CardType'] = (string)$response::$paymentMethod::$cardType;
					extension_payment_worldpay::addCard(session::$data['order_id'], $card_data);
				}
			}

			//loop through any products that are recurring items
			foreach ($recurring_products as $item) {
				extension_payment_worldpay::recurringPayment($item, session::$data['order_id'] . rand(), request::$post['token']);
			}

			response::redirect(url::link('checkout/success', ''));
		} else {

			session::$data['error'] = language::get('error_process_order');
			response::redirect(url::link('checkout/checkout', ''));
		}
	}

	public static function deleteCard() {
		load::language('extension/payment/worldpay');

		if (isset(request::$post['token'])) {
			if (extension_payment_worldpay::deleteCard(request::$post['token'])) {
				$json['success'] = language::get('text_card_success');
			} else {
				$json['error'] = language::get('text_card_error');
			}

			if (count(extension_payment_worldpay::getCards(customer::getId()))) {
				$json['existing_cards'] = true;
			}
		} else {
			$json['error'] = language::get('text_error');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function webhook() {
		if (isset(request::$get['token']) && hash_equals(config::get('payment_worldpay_secret_token'), request::$get['token'])) {
			$message = json_decode(file_get_contents('php://input'), true);

			if (isset($message['orderCode'])) {
				$order = extension_payment_worldpay::getWorldpayOrder($message['orderCode']);
				extension_payment_worldpay::logger($order);
				switch ($message['paymentStatus']) {
					case 'SUCCESS':
						$order_status_id = config::get('payment_worldpay_success_status_id');
						break;
					case 'FAILED':
						$order_status_id = config::get('payment_worldpay_failed_status_id');
						break;
					case 'SETTLED':
						$order_status_id = config::get('payment_worldpay_settled_status_id');
						break;
					case 'REFUNDED':
						$order_status_id = config::get('payment_worldpay_refunded_status_id');
						break;
					case 'PARTIALLY_REFUNDED':
						$order_status_id = config::get('payment_worldpay_partially_refunded_status_id');
						break;
					case 'CHARGED_BACK':
						$order_status_id = config::get('payment_worldpay_charged_back_status_id');
						break;
					case 'INFORMATION_REQUESTED':
						$order_status_id = config::get('payment_worldpay_information_requested_status_id');
						break;
					case 'INFORMATION_SUPPLIED':
						$order_status_id = config::get('payment_worldpay_information_supplied_status_id');
						break;
					case 'CHARGEBACK_REVERSED':
						$order_status_id = config::get('payment_worldpay_chargeback_reversed_status_id');
						break;
				}

				extension_payment_worldpay::logger($order_status_id);
				if (isset($order['order_id'])) {
					checkout_order::addOrderHistory($order['order_id'], $order_status_id);
				}
			}
		}

		response::addHeader('HTTP/1.1 200 OK');
		response::addHeader('Content-Type: application/json');
	}

	public static function cron() {
		if (request::$get['token'] == config::get('payment_worldpay_secret_token')) {

			$orders = extension_payment_worldpay::cronPayment();

			extension_payment_worldpay::updateCronJobRunTime();

			extension_payment_worldpay::logger($orders);
		}
	}

}
