<?php
class ControllerExtensionRecurringSquareup {
    public static function index() {
        load::language('extension/recurring/squareup');
        

        if (isset(request::$get['order_recurring_id'])) {
            $order_recurring_id = request::$get['order_recurring_id'];
        } else {
            $order_recurring_id = 0;
        }
        
        $recurring_info = account_recurring::getOrderRecurring($order_recurring_id);
        
        if ($recurring_info) {
            $data['cancel_url'] = html_entity_decode(url::link('extension/recurring/squareup/cancel', 'order_recurring_id=' . $order_recurring_id, 'SSL'));

            $data['continue'] = url::link('account/recurring', '');    
            
            if ($recurring_info['status'] == ModelExtensionPaymentSquareup::$RECURRING_ACTIVE) {
                $data['order_recurring_id'] = $order_recurring_id;
            } else {
                $data['order_recurring_id'] = '';
            }

            return load::view('extension/recurring/squareup', $data);
        }
    }
    
    public static function cancel() {
        load::language('extension/recurring/squareup');
        
        
        if (isset(request::$get['order_recurring_id'])) {
            $order_recurring_id = request::$get['order_recurring_id'];
        } else {
            $order_recurring_id = 0;
        }

        $json = array();
        
        $recurring_info = account_recurring::getOrderRecurring($order_recurring_id);

        if ($recurring_info) {
            account_recurring::editOrderRecurringStatus($order_recurring_id, ModelExtensionPaymentSquareup::$RECURRING_CANCELLED);


            $order_info = checkout_order::getOrder($recurring_info['order_id']);

            checkout_order::addOrderHistory($recurring_info['order_id'], $order_info['order_status_id'], language::get('text_order_history_cancel'), true);

            $json['success'] = language::get('text_canceled');
        } else {
            $json['error'] = language::get('error_not_found');
        }

        response::addHeader('Content-Type: application/json');
        response::setOutput(json_encode($json));
    }

    public static function recurring() {
        load::language('extension/payment/squareup');
        

        if (!extension_payment_squareup::validateCRON()) {
            return;
        }

        load::library('squareup');

        $result = array(
            'transaction_success' => array(),
            'transaction_error' => array(),
            'transaction_fail' => array(),
            'token_update_error' => ''
        );

        $result['token_update_error'] = extension_payment_squareup::updateToken();


        foreach (extension_payment_squareup::nextRecurringPayments() as $payment) {
            try {
                if (!$payment['is_free']) {
                    $transaction = self::$squareup::addTransaction($payment['transaction']);

                    $transaction_status = !empty($transaction['tenders'][0]['card_details']['status']) ?
                        strtolower($transaction['tenders'][0]['card_details']['status']) : '';

                    $target_currency = $transaction['tenders'][0]['amount_money']['currency'];

                    $amount = self::$squareup::standardDenomination($transaction['tenders'][0]['amount_money']['amount'], $target_currency);

                    extension_payment_squareup::addTransaction($transaction, config::get('payment_squareup_merchant_id'), $payment['billing_address'], $payment['order_id'], "CRON JOB", "127.0.0.1");

                    $reference = $transaction['id'];
                } else {
                    $amount = 0;
                    $target_currency = config('currency');
                    $reference = '';
                    $transaction_status = 'captured';
                }

                $success = $transaction_status == 'captured';

                extension_payment_squareup::addRecurringTransaction($payment['order_recurring_id'], $reference, $amount, $success);

                $trial_expired = false;
                $recurring_expired = false;
                $profile_suspended = false;

                if ($success) {
                    $trial_expired = extension_payment_squareup::updateRecurringTrial($payment['order_recurring_id']);

                    $recurring_expired = extension_payment_squareup::updateRecurringExpired($payment['order_recurring_id']);

                    $result['transaction_success'][$payment['order_recurring_id']] = currency::format($amount, $target_currency);
                } else {
                    // Transaction was not successful. Suspend the recurring profile.
                    $profile_suspended = extension_payment_squareup::suspendRecurringProfile($payment['order_recurring_id']);

                    $result['transaction_fail'][$payment['order_recurring_id']] = currency::format($amount, $target_currency);
                }


                $order_status_id = config::get('payment_squareup_status_' . $transaction_status);

                if ($order_status_id) {
                    if (!$payment['is_free']) {
                        $order_status_comment = language::get('squareup_status_comment_' . $transaction_status);
                    } else {
                        $order_status_comment = '';
                    }

                    if ($profile_suspended) {
                        $order_status_comment .= language::get('text_squareup_profile_suspended');
                    }

                    if ($trial_expired) {
                        $order_status_comment .= language::get('text_squareup_trial_expired');
                    }

                    if ($recurring_expired) {
                        $order_status_comment .= language::get('text_squareup_recurring_expired');
                    }

                    if ($success) {
                        $notify = (bool)config::get('payment_squareup_notify_recurring_success');
                    } else {
                        $notify = (bool)config::get('payment_squareup_notify_recurring_fail');
                    }

                    checkout_order::addOrderHistory($payment['order_id'], $order_status_id, trim($order_status_comment), $notify);
                }
            } catch (\Squareup\Exception $e) {
                $result['transaction_error'][] = '[ID: ' . $payment['order_recurring_id'] . '] - ' . $e::getMessage();
            }
        };

        if (config::get('payment_squareup_cron_email_status')) {
            extension_payment_squareup::cronEmail($result);
        }
    }
}