<?php
class ControllerExtensionTotalShipping {
	public static function index() {
		if (config::get('total_shipping_status') && config::get('total_shipping_estimator') && cart::hasShipping()) {
			load::language('extension/total/shipping');

			if (isset(session::$data['shipping_address']['country_id'])) {
				$data['country_id'] = session::$data['shipping_address']['country_id'];
			} else {
				$data['country_id'] = config('country_id');
			}


			$data['countries'] = localisation_country::getCountries();

			if (isset(session::$data['shipping_address']['zone_id'])) {
				$data['zone_id'] = session::$data['shipping_address']['zone_id'];
			} else {
				$data['zone_id'] = '';
			}

			if (isset(session::$data['shipping_address']['postcode'])) {
				$data['postcode'] = session::$data['shipping_address']['postcode'];
			} else {
				$data['postcode'] = '';
			}

			if (isset(session::$data['shipping_method'])) {
				$data['shipping_method'] = session::$data['shipping_method']['code'];
			} else {
				$data['shipping_method'] = '';
			}

			return load::view('extension/total/shipping', $data);
		}
	}

	public static function quote() {
		load::language('extension/total/shipping');

		$json = array();

		if (!cart::hasProducts()) {
			$json['error']['warning'] = language::get('error_product');
		}

		if (!cart::hasShipping()) {
			$json['error']['warning'] = sprintf(language::get('error_no_shipping'), url::link('information/contact'));
		}

		if (request::$post['country_id'] == '') {
			$json['error']['country'] = language::get('error_country');
		}

		if (!isset(request::$post['zone_id']) || request::$post['zone_id'] == '' || !is_numeric(request::$post['zone_id'])) {
			$json['error']['zone'] = language::get('error_zone');
		}


		$country_info = localisation_country::getCountry(request::$post['country_id']);

		if ($country_info && $country_info['postcode_required'] && (utf8_strlen(trim(request::$post['postcode'])) < 2 || utf8_strlen(trim(request::$post['postcode'])) > 10)) {
			$json['error']['postcode'] = language::get('error_postcode');
		}

		if (!$json) {
			tax::unsetRates();
			tax::setShippingAddress(request::$post['country_id'], request::$post['zone_id']);
			if (isset(session::$data['payment_address']['country_id']) && isset(session::$data['payment_address']['zone_id'])) {
				tax::setPaymentAddress(session::$data['payment_address']['country_id'], session::$data['payment_address']['zone_id']);
			} elseif (config('tax_default') == 'payment') {
				tax::setPaymentAddress(config('country_id'), config('zone_id'));
			}
			tax::setStoreAddress(config('country_id'), config('zone_id'));
			if ($country_info) {
				$country = $country_info['name'];
				$iso_code_2 = $country_info['iso_code_2'];
				$iso_code_3 = $country_info['iso_code_3'];
				$address_format = $country_info['address_format'];
			} else {
				$country = '';
				$iso_code_2 = '';
				$iso_code_3 = '';
				$address_format = '';
			}


			$zone_info = localisation_zone::getZone(request::$post['zone_id']);

			if ($zone_info) {
				$zone = $zone_info['name'];
				$zone_code = $zone_info['code'];
			} else {
				$zone = '';
				$zone_code = '';
			}

			session::$data['shipping_address'] = array(
				'firstname'      => '',
				'lastname'       => '',
				'company'        => '',
				'address_1'      => '',
				'address_2'      => '',
				'postcode'       => request::$post['postcode'],
				'city'           => '',
				'zone_id'        => request::$post['zone_id'],
				'zone'           => $zone,
				'zone_code'      => $zone_code,
				'country_id'     => request::$post['country_id'],
				'country'        => $country,
				'iso_code_2'     => $iso_code_2,
				'iso_code_3'     => $iso_code_3,
				'address_format' => $address_format
			);

			$quote_data = array();


			$results = setting_extension::getExtensions('shipping');

			foreach ($results as $result) {
				if (config::get('shipping_' . $result['code'] . '_status')) {

					$quote = call_model_method('extension_shipping_' . $result['code'], 'getQuote', session::$data['shipping_address']);

					if ($quote) {
						$quote_data[$result['code']] = array(
							'title'      => $quote['title'],
							'quote'      => $quote['quote'],
							'sort_order' => $quote['sort_order'],
							'error'      => $quote['error']
						);
					}
				}
			}

			$sort_order = array();

			foreach ($quote_data as $key => $value) {
				$sort_order[$key] = $value['sort_order'];
			}

			array_multisort($sort_order, SORT_ASC, $quote_data);

			session::$data['shipping_methods'] = $quote_data;

			if (session::$data['shipping_methods']) {
				$json['shipping_method'] = session::$data['shipping_methods'];
			} else {
				$json['error']['warning'] = sprintf(language::get('error_no_shipping'), url::link('information/contact'));
			}
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function shipping() {
		load::language('extension/total/shipping');

		$json = array();

		if (!empty(request::$post['shipping_method'])) {
			$shipping = explode('.', request::$post['shipping_method']);

			if (!isset($shipping[0]) || !isset($shipping[1]) || !isset(session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]])) {
				$json['warning'] = language::get('error_shipping');
			}
		} else {
			$json['warning'] = language::get('error_shipping');
		}

		if (!$json) {
			$shipping = explode('.', request::$post['shipping_method']);

			session::$data['shipping_method'] = session::$data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]];

			session::$data['success'] = language::get('text_success');

			$json['redirect'] = url::link('checkout/cart');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}

	public static function country() {
		$json = array();


		$country_info = localisation_country::getCountry(request::$get['country_id']);

		if ($country_info) {

			$json = array(
				'country_id'        => $country_info['country_id'],
				'name'              => $country_info['name'],
				'iso_code_2'        => $country_info['iso_code_2'],
				'iso_code_3'        => $country_info['iso_code_3'],
				'address_format'    => $country_info['address_format'],
				'postcode_required' => $country_info['postcode_required'],
				'zone'              => localisation_zone::getZonesByCountryId(request::$get['country_id']),
				'status'            => $country_info['status']
			);
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
