<?php
class ControllerInformationContact {
	private static $error = array();

	public static function index() {
		load::language('information/contact');

		document::setTitle(language::get('heading_title'));

		if ((request::$server['REQUEST_METHOD'] == 'POST') && self::validate()) {
			$mail = new Mail(config('mail_engine'));
			$mail::$parameter = config('mail_parameter');
			$mail::$smtp_hostname = config('mail_smtp_hostname');
			$mail::$smtp_username = config('mail_smtp_username');
			$mail::$smtp_password = html_entity_decode(config('mail_smtp_password'), ENT_QUOTES, 'UTF-8');
			$mail::$smtp_port = config('mail_smtp_port');
			$mail::$smtp_timeout = config('mail_smtp_timeout');

			$mail::setTo(config('email'));
			$mail::setFrom(config('email'));
			$mail::setReplyTo(request::$post['email']);
			$mail::setSender(html_entity_decode(request::$post['name'], ENT_QUOTES, 'UTF-8'));
			$mail::setSubject(html_entity_decode(sprintf(language::get('email_subject'), request::$post['name']), ENT_QUOTES, 'UTF-8'));
			$mail::setText(request::$post['enquiry']);
			$mail::send();

			response::redirect(url::link('information/contact/success'));
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('information/contact')
		);

		if (isset(self::$error['name'])) {
			$data['error_name'] = self::$error['name'];
		} else {
			$data['error_name'] = '';
		}

		if (isset(self::$error['email'])) {
			$data['error_email'] = self::$error['email'];
		} else {
			$data['error_email'] = '';
		}

		if (isset(self::$error['enquiry'])) {
			$data['error_enquiry'] = self::$error['enquiry'];
		} else {
			$data['error_enquiry'] = '';
		}

		$data['button_submit'] = language::get('button_submit');

		$data['action'] = url::link('information/contact', '');


		if (config('image')) {
			$data['image'] = tool_image::resize(config('image'), config::get('theme_' . config('theme') . '_image_location_width'), config::get('theme_' . config('theme') . '_image_location_height'));
		} else {
			$data['image'] = false;
		}

		$data['store'] = config('name');
		$data['address'] = nl2br(config('address'));
		$data['geocode'] = config('geocode');
		$data['geocode_hl'] = config('language');
		$data['telephone'] = config('telephone');
		$data['fax'] = config('fax');
		$data['open'] = nl2br(config('open'));
		$data['comment'] = config('comment');

		$data['locations'] = array();


		foreach((array)config('location') as $location_id) {
			$location_info = localisation_location::getLocation($location_id);

			if ($location_info) {
				if ($location_info['image']) {
					$image = tool_image::resize($location_info['image'], config::get('theme_' . config('theme') . '_image_location_width'), config::get('theme_' . config('theme') . '_image_location_height'));
				} else {
					$image = false;
				}

				$data['locations'][] = array(
					'location_id' => $location_info['location_id'],
					'name'        => $location_info['name'],
					'address'     => nl2br($location_info['address']),
					'geocode'     => $location_info['geocode'],
					'telephone'   => $location_info['telephone'],
					'fax'         => $location_info['fax'],
					'image'       => $image,
					'open'        => nl2br($location_info['open']),
					'comment'     => $location_info['comment']
				);
			}
		}

		if (isset(request::$post['name'])) {
			$data['name'] = request::$post['name'];
		} else {
			$data['name'] = customer::getFirstName();
		}

		if (isset(request::$post['email'])) {
			$data['email'] = request::$post['email'];
		} else {
			$data['email'] = customer::getEmail();
		}

		if (isset(request::$post['enquiry'])) {
			$data['enquiry'] = request::$post['enquiry'];
		} else {
			$data['enquiry'] = '';
		}

		// Captcha
		if (config::get('captcha_' . config('captcha') . '_status') && in_array('contact', (array)config('captcha_page'))) {
			$data['captcha'] = load::controller('extension/captcha/' . config('captcha'), self::$error);
		} else {
			$data['captcha'] = '';
		}

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('information/contact', $data));
	}

	protected static function validate() {
		if ((utf8_strlen(request::$post['name']) < 3) || (utf8_strlen(request::$post['name']) > 32)) {
			self::$error['name'] = language::get('error_name');
		}

		if (!filter_var(request::$post['email'], FILTER_VALIDATE_EMAIL)) {
			self::$error['email'] = language::get('error_email');
		}

		if ((utf8_strlen(request::$post['enquiry']) < 10) || (utf8_strlen(request::$post['enquiry']) > 3000)) {
			self::$error['enquiry'] = language::get('error_enquiry');
		}

		// Captcha
		if (config::get('captcha_' . config('captcha') . '_status') && in_array('contact', (array)config('captcha_page'))) {
			$captcha = load::controller('extension/captcha/' . config('captcha') . '/validate');

			if ($captcha) {
				self::$error['captcha'] = $captcha;
			}
		}

		return !self::$error;
	}

	public static function success() {
		load::language('information/contact');

		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('information/contact')
		);

 		$data['text_message'] = language::get('text_message'); 

		$data['continue'] = url::link('common/home');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('common/success', $data));
	}
}
