<?php
class ControllerMailAffiliate {
	public static function index(&$route, &$args, &$output) {
		load::language('mail/affiliate');
        
		$data['text_welcome'] = sprintf(language::get('text_welcome'), html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8'));
		$data['text_login'] = language::get('text_login');
		$data['text_approval'] = language::get('text_approval');
		$data['text_service'] = language::get('text_service');
		$data['text_thanks'] = language::get('text_thanks');

		
		if (customer::isLogged()) {
			$customer_group_id = customer::getGroupId();
		} else {
			$customer_group_id = $args[1]['customer_group_id'];
		}
		
		$customer_group_info = account_customer_group::getCustomerGroup($customer_group_id);
		
		if ($customer_group_info) {
			$data['approval'] = (config('affiliate_approval') || $customer_group_info['approval']);
		} else {
			$data['approval'] = '';
		}		
		
		$data['login'] = url::link('affiliate/login', '');
		$data['store'] = html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8');

		$mail = new Mail(config('mail_engine'));
		$mail::$parameter = config('mail_parameter');
		$mail::$smtp_hostname = config('mail_smtp_hostname');
		$mail::$smtp_username = config('mail_smtp_username');
		$mail::$smtp_password = html_entity_decode(config('mail_smtp_password'), ENT_QUOTES, 'UTF-8');
		$mail::$smtp_port = config('mail_smtp_port');
		$mail::$smtp_timeout = config('mail_smtp_timeout');

		if (customer::isLogged()) {
			$mail::setTo(customer::getEmail());
		} else {
			$mail::setTo($args[1]['email']);
		}
		
		$mail::setFrom(config('email'));
		$mail::setSender(html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8'));
		$mail::setSubject(sprintf(language::get('text_subject'), html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8')));
		$mail::setText(load::view('mail/affiliate', $data));
		$mail::send();
 	}
	
	public static function alert(&$route, &$args, &$output) {
		// Send to main admin email if new affiliate email is enabled
		if (in_array('affiliate', (array)config('mail_alert'))) {
			load::language('mail/affiliate');
			
			$data['text_signup'] = language::get('text_signup');
			$data['text_website'] = language::get('text_website');
			$data['text_firstname'] = language::get('text_firstname');
			$data['text_lastname'] = language::get('text_lastname');
			$data['text_customer_group'] = language::get('text_customer_group');
			$data['text_email'] = language::get('text_email');
			$data['text_telephone'] = language::get('text_telephone');
			
			if (customer::isLogged()) {
				$customer_group_id = customer::getGroupId();
			
				$data['firstname'] = customer::getFirstName();
				$data['lastname'] = customer::getLastName();
				$data['email'] = customer::getEmail();
				$data['telephone'] = customer::getTelephone();
			} else {	
				$customer_group_id = $args[1]['customer_group_id'];
				
				$data['firstname'] = $args[1]['firstname'];
				$data['lastname'] = $args[1]['lastname'];	
				$data['email'] = $args[1]['email'];
				$data['telephone'] = $args[1]['telephone'];		
			}
			
			$data['website'] = html_entity_decode($args[1]['website'], ENT_QUOTES, 'UTF-8');
			$data['company'] = $args[1]['company'];
							

			$customer_group_info = account_customer_group::getCustomerGroup($customer_group_id);
			
			if ($customer_group_info) {
				$data['customer_group'] = $customer_group_info['name'];
			} else {
				$data['customer_group'] = '';
			}
			
			$mail = new Mail(config('mail_engine'));
			$mail::$parameter = config('mail_parameter');
			$mail::$smtp_hostname = config('mail_smtp_hostname');
			$mail::$smtp_username = config('mail_smtp_username');
			$mail::$smtp_password = html_entity_decode(config('mail_smtp_password'), ENT_QUOTES, 'UTF-8');
			$mail::$smtp_port = config('mail_smtp_port');
			$mail::$smtp_timeout = config('mail_smtp_timeout');

			$mail::setTo(config('email'));
			$mail::setFrom(config('email'));
			$mail::setSender(html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8'));			
			$mail::setSubject(html_entity_decode(language::get('text_new_affiliate'), ENT_QUOTES, 'UTF-8'));
			$mail::setText(load::view('mail/affiliate_alert', $data));
			$mail::send();

			// Send to additional alert emails if new affiliate email is enabled
			$emails = explode(',', config('mail_alert_email'));

			foreach ($emails as $email) {
				if (utf8_strlen($email) > 0 && filter_var($email, FILTER_VALIDATE_EMAIL)) {
					$mail::setTo($email);
					$mail::send();
				}
			}
		}		
	}
}		