<?php
class ControllerMailRegister {
	public static function index(&$route, &$args, &$output) {
		load::language('mail/register');

		$data['text_welcome'] = sprintf(language::get('text_welcome'), html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8'));
		$data['text_login'] = language::get('text_login');
		$data['text_approval'] = language::get('text_approval');
		$data['text_service'] = language::get('text_service');
		$data['text_thanks'] = language::get('text_thanks');

			
		if (isset($args[0]['customer_group_id'])) {
			$customer_group_id = $args[0]['customer_group_id'];
		} else {
			$customer_group_id = config('customer_group_id');
		}
					
		$customer_group_info = account_customer_group::getCustomerGroup($customer_group_id);
		
		if ($customer_group_info) {
			$data['approval'] = $customer_group_info['approval'];
		} else {
			$data['approval'] = '';
		}
			
		$data['login'] = url::link('account/login', '');		
		$data['store'] = html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8');

		$mail = new Mail(config('mail_engine'));
		$mail::$parameter = config('mail_parameter');
		$mail::$smtp_hostname = config('mail_smtp_hostname');
		$mail::$smtp_username = config('mail_smtp_username');
		$mail::$smtp_password = html_entity_decode(config('mail_smtp_password'), ENT_QUOTES, 'UTF-8');
		$mail::$smtp_port = config('mail_smtp_port');
		$mail::$smtp_timeout = config('mail_smtp_timeout');

		$mail::setTo($args[0]['email']);
		$mail::setFrom(config('email'));
		$mail::setSender(html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8'));
		$mail::setSubject(sprintf(language::get('text_subject'), html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8')));
		$mail::setText(load::view('mail/register', $data));
		$mail::send(); 
	}
	
	public static function alert(&$route, &$args, &$output) {
		// Send to main admin email if new account email is enabled
		if (in_array('account', (array)config('mail_alert'))) {
			load::language('mail/register');
			
			$data['text_signup'] = language::get('text_signup');
			$data['text_firstname'] = language::get('text_firstname');
			$data['text_lastname'] = language::get('text_lastname');
			$data['text_customer_group'] = language::get('text_customer_group');
			$data['text_email'] = language::get('text_email');
			$data['text_telephone'] = language::get('text_telephone');
			
			$data['firstname'] = $args[0]['firstname'];
			$data['lastname'] = $args[0]['lastname'];
			
			
			if (isset($args[0]['customer_group_id'])) {
				$customer_group_id = $args[0]['customer_group_id'];
			} else {
				$customer_group_id = config('customer_group_id');
			}
			
			$customer_group_info = account_customer_group::getCustomerGroup($customer_group_id);
			
			if ($customer_group_info) {
				$data['customer_group'] = $customer_group_info['name'];
			} else {
				$data['customer_group'] = '';
			}
			
			$data['email'] = $args[0]['email'];
			$data['telephone'] = $args[0]['telephone'];

			$mail = new Mail(config('mail_engine'));
			$mail::$parameter = config('mail_parameter');
			$mail::$smtp_hostname = config('mail_smtp_hostname');
			$mail::$smtp_username = config('mail_smtp_username');
			$mail::$smtp_password = html_entity_decode(config('mail_smtp_password'), ENT_QUOTES, 'UTF-8');
			$mail::$smtp_port = config('mail_smtp_port');
			$mail::$smtp_timeout = config('mail_smtp_timeout');

			$mail::setTo(config('email'));
			$mail::setFrom(config('email'));
			$mail::setSender(html_entity_decode(config('name'), ENT_QUOTES, 'UTF-8'));
			$mail::setSubject(html_entity_decode(language::get('text_new_customer'), ENT_QUOTES, 'UTF-8'));
			$mail::setText(load::view('mail/register_alert', $data));
			$mail::send();

			// Send to additional alert emails if new account email is enabled
			$emails = explode(',', config('mail_alert_email'));

			foreach ($emails as $email) {
				if (utf8_strlen($email) > 0 && filter_var($email, FILTER_VALIDATE_EMAIL)) {
					$mail::setTo($email);
					$mail::send();
				}
			}
		}	
	}
}		