<?php
class ControllerProductCategory {
	public static function index() {
		load::language('product/category');




		if (isset(request::$get['filter'])) {
			$filter = request::$get['filter'];
		} else {
			$filter = '';
		}

		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'p.sort_order';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		if (isset(request::$get['limit'])) {
			$limit = (int)request::$get['limit'];
		} else {
			$limit = config::get('theme_' . config('theme') . '_product_limit');
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		if (isset(request::$get['path'])) {
			$url = '';

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['limit'])) {
				$url .= '&limit=' . request::$get['limit'];
			}

			$path = '';

			$parts = explode('_', (string)request::$get['path']);

			$category_id = (int)array_pop($parts);

			foreach ($parts as $path_id) {
				if (!$path) {
					$path = (int)$path_id;
				} else {
					$path .= '_' . (int)$path_id;
				}

				$category_info = catalog_category::getCategory($path_id);

				if ($category_info) {
					$data['breadcrumbs'][] = array(
						'text' => $category_info['name'],
						'href' => url::link('product/category', 'path=' . $path . $url)
					);
				}
			}
		} else {
			$category_id = 0;
		}

		$category_info = catalog_category::getCategory($category_id);

		if ($category_info) {
			document::setTitle($category_info['meta_title']);
			document::setDescription($category_info['meta_description']);
			document::setKeywords($category_info['meta_keyword']);

			$data['heading_title'] = $category_info['name'];

			$data['text_compare'] = sprintf(language::get('text_compare'), (isset(session::$data['compare']) ? count(session::$data['compare']) : 0));

			// Set the last category breadcrumb
			$data['breadcrumbs'][] = array(
				'text' => $category_info['name'],
				'href' => url::link('product/category', 'path=' . request::$get['path'])
			);

			if ($category_info['image']) {
				$data['thumb'] = tool_image::resize($category_info['image'], config::get('theme_' . config('theme') . '_image_category_width'), config::get('theme_' . config('theme') . '_image_category_height'));
			} else {
				$data['thumb'] = '';
			}

			$data['description'] = html_entity_decode($category_info['description'], ENT_QUOTES, 'UTF-8');
			$data['compare'] = url::link('product/compare');

			$url = '';

			if (isset(request::$get['filter'])) {
				$url .= '&filter=' . request::$get['filter'];
			}

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['limit'])) {
				$url .= '&limit=' . request::$get['limit'];
			}

			$data['categories'] = array();

			$results = catalog_category::getCategories($category_id);

			foreach ($results as $result) {
				$filter_data = array(
					'filter_category_id'  => $result['category_id'],
					'filter_sub_category' => true
				);

				$data['categories'][] = array(
					'name' => $result['name'] . (config('product_count') ? ' (' . catalog_product::getTotalProducts($filter_data) . ')' : ''),
					'href' => url::link('product/category', 'path=' . request::$get['path'] . '_' . $result['category_id'] . $url)
				);
			}

			$data['products'] = array();

			$filter_data = array(
				'filter_category_id' => $category_id,
				'filter_filter'      => $filter,
				'sort'               => $sort,
				'order'              => $order,
				'start'              => ($page - 1) * $limit,
				'limit'              => $limit
			);

			$product_total = catalog_product::getTotalProducts($filter_data);

			$results = catalog_product::getProducts($filter_data);

			foreach ($results as $result) {
				if ($result['image']) {
					$image = tool_image::resize($result['image'], config::get('theme_' . config('theme') . '_image_product_width'), config::get('theme_' . config('theme') . '_image_product_height'));
				} else {
					$image = tool_image::resize('placeholder.png', config::get('theme_' . config('theme') . '_image_product_width'), config::get('theme_' . config('theme') . '_image_product_height'));
				}

				if (customer::isLogged() || !config('customer_price')) {
					$price = currency::format(tax::calculate($result['price'], $result['tax_class_id'], config('tax')), session::$data['currency']);
				} else {
					$price = false;
				}

				if (!is_null($result['special']) && (float)$result['special'] >= 0) {
					$special = currency::format(tax::calculate($result['special'], $result['tax_class_id'], config('tax')), session::$data['currency']);
					$tax_price = (float)$result['special'];
				} else {
					$special = false;
					$tax_price = (float)$result['price'];
				}
	
				if (config('tax')) {
					$tax = currency::format($tax_price, session::$data['currency']);
				} else {
					$tax = false;
				}

				if (config('review_status')) {
					$rating = (int)$result['rating'];
				} else {
					$rating = false;
				}

				$data['products'][] = array(
					'product_id'  => $result['product_id'],
					'thumb'       => $image,
					'name'        => $result['name'],
					'description' => utf8_substr(trim(strip_tags(html_entity_decode($result['description'], ENT_QUOTES, 'UTF-8'))), 0, config::get('theme_' . config('theme') . '_product_description_length')) . '..',
					'price'       => $price,
					'special'     => $special,
					'tax'         => $tax,
					'minimum'     => $result['minimum'] > 0 ? $result['minimum'] : 1,
					'rating'      => $result['rating'],
					'href'        => url::link('product/product', 'path=' . request::$get['path'] . '&product_id=' . $result['product_id'] . $url)
				);
			}

			$url = '';

			if (isset(request::$get['filter'])) {
				$url .= '&filter=' . request::$get['filter'];
			}

			if (isset(request::$get['limit'])) {
				$url .= '&limit=' . request::$get['limit'];
			}

			$data['sorts'] = array();

			$data['sorts'][] = array(
				'text'  => language::get('text_default'),
				'value' => 'p.sort_order-ASC',
				'href'  => url::link('product/category', 'path=' . request::$get['path'] . '&sort=p.sort_order&order=ASC' . $url)
			);

			$data['sorts'][] = array(
				'text'  => language::get('text_name_asc'),
				'value' => 'pd.name-ASC',
				'href'  => url::link('product/category', 'path=' . request::$get['path'] . '&sort=pd.name&order=ASC' . $url)
			);

			$data['sorts'][] = array(
				'text'  => language::get('text_name_desc'),
				'value' => 'pd.name-DESC',
				'href'  => url::link('product/category', 'path=' . request::$get['path'] . '&sort=pd.name&order=DESC' . $url)
			);

			$data['sorts'][] = array(
				'text'  => language::get('text_price_asc'),
				'value' => 'p.price-ASC',
				'href'  => url::link('product/category', 'path=' . request::$get['path'] . '&sort=p.price&order=ASC' . $url)
			);

			$data['sorts'][] = array(
				'text'  => language::get('text_price_desc'),
				'value' => 'p.price-DESC',
				'href'  => url::link('product/category', 'path=' . request::$get['path'] . '&sort=p.price&order=DESC' . $url)
			);

			if (config('review_status')) {
				$data['sorts'][] = array(
					'text'  => language::get('text_rating_desc'),
					'value' => 'rating-DESC',
					'href'  => url::link('product/category', 'path=' . request::$get['path'] . '&sort=rating&order=DESC' . $url)
				);

				$data['sorts'][] = array(
					'text'  => language::get('text_rating_asc'),
					'value' => 'rating-ASC',
					'href'  => url::link('product/category', 'path=' . request::$get['path'] . '&sort=rating&order=ASC' . $url)
				);
			}

			$data['sorts'][] = array(
				'text'  => language::get('text_model_asc'),
				'value' => 'p.model-ASC',
				'href'  => url::link('product/category', 'path=' . request::$get['path'] . '&sort=p.model&order=ASC' . $url)
			);

			$data['sorts'][] = array(
				'text'  => language::get('text_model_desc'),
				'value' => 'p.model-DESC',
				'href'  => url::link('product/category', 'path=' . request::$get['path'] . '&sort=p.model&order=DESC' . $url)
			);

			$url = '';

			if (isset(request::$get['filter'])) {
				$url .= '&filter=' . request::$get['filter'];
			}

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			$data['limits'] = array();

			$limits = array_unique(array(config::get('theme_' . config('theme') . '_product_limit'), 25, 50, 75, 100));

			sort($limits);

			foreach($limits as $value) {
				$data['limits'][] = array(
					'text'  => $value,
					'value' => $value,
					'href'  => url::link('product/category', 'path=' . request::$get['path'] . $url . '&limit=' . $value)
				);
			}

			$url = '';

			if (isset(request::$get['filter'])) {
				$url .= '&filter=' . request::$get['filter'];
			}

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['limit'])) {
				$url .= '&limit=' . request::$get['limit'];
			}

			$pagination = new Pagination();
			$pagination::$total = $product_total;
			$pagination::$page = $page;
			$pagination::$limit = $limit;
			$pagination::$url = url::link('product/category', 'path=' . request::$get['path'] . $url . '&page={page}');

			$data['pagination'] = $pagination::render();

			$data['results'] = sprintf(language::get('text_pagination'), ($product_total) ? (($page - 1) * $limit) + 1 : 0, ((($page - 1) * $limit) > ($product_total - $limit)) ? $product_total : ((($page - 1) * $limit) + $limit), $product_total, ceil($product_total / $limit));

			// http://googlewebmastercentral.blogspot.com/2011/09/pagination-with-relnext-and-relprev.html
			if ($page == 1) {
			    document::addLink(url::link('product/category', 'path=' . $category_info['category_id']), 'canonical');
			} else {
				document::addLink(url::link('product/category', 'path=' . $category_info['category_id'] . '&page='. $page), 'canonical');
			}
			
			if ($page > 1) {
			    document::addLink(url::link('product/category', 'path=' . $category_info['category_id'] . (($page - 2) ? '&page='. ($page - 1) : '')), 'prev');
			}

			if ($limit && ceil($product_total / $limit) > $page) {
			    document::addLink(url::link('product/category', 'path=' . $category_info['category_id'] . '&page='. ($page + 1)), 'next');
			}

			$data['sort'] = $sort;
			$data['order'] = $order;
			$data['limit'] = $limit;

			$data['continue'] = url::link('common/home');

			$data['column_left'] = load::controller('common/column_left');
			$data['column_right'] = load::controller('common/column_right');
			$data['content_top'] = load::controller('common/content_top');
			$data['content_bottom'] = load::controller('common/content_bottom');
			$data['footer'] = load::controller('common/footer');
			$data['header'] = load::controller('common/header');

			response::setOutput(load::view('product/category', $data));
		} else {
			$url = '';

			if (isset(request::$get['path'])) {
				$url .= '&path=' . request::$get['path'];
			}

			if (isset(request::$get['filter'])) {
				$url .= '&filter=' . request::$get['filter'];
			}

			if (isset(request::$get['sort'])) {
				$url .= '&sort=' . request::$get['sort'];
			}

			if (isset(request::$get['order'])) {
				$url .= '&order=' . request::$get['order'];
			}

			if (isset(request::$get['page'])) {
				$url .= '&page=' . request::$get['page'];
			}

			if (isset(request::$get['limit'])) {
				$url .= '&limit=' . request::$get['limit'];
			}

			$data['breadcrumbs'][] = array(
				'text' => language::get('text_error'),
				'href' => url::link('product/category', $url)
			);

			document::setTitle(language::get('text_error'));

			$data['continue'] = url::link('common/home');

			response::addHeader(request::$server['SERVER_PROTOCOL'] . ' 404 Not Found');

			$data['column_left'] = load::controller('common/column_left');
			$data['column_right'] = load::controller('common/column_right');
			$data['content_top'] = load::controller('common/content_top');
			$data['content_bottom'] = load::controller('common/content_bottom');
			$data['footer'] = load::controller('common/footer');
			$data['header'] = load::controller('common/header');

			response::setOutput(load::view('error/not_found', $data));
		}
	}
}
