<?php
class ControllerProductCompare {
	public static function index() {
		load::language('product/compare');



		if (!isset(session::$data['compare'])) {
			session::$data['compare'] = array();
		}

		if (isset(request::$get['remove'])) {
			$key = array_search(request::$get['remove'], session::$data['compare']);

			if ($key !== false) {
				unset(session::$data['compare'][$key]);

				session::$data['success'] = language::get('text_remove');
			}

			response::redirect(url::link('product/compare'));
		}

		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('product/compare')
		);

		if (isset(session::$data['success'])) {
			$data['success'] = session::$data['success'];

			unset(session::$data['success']);
		} else {
			$data['success'] = '';
		}

		$data['review_status'] = config('review_status');

		$data['products'] = array();

		$data['attribute_groups'] = array();

		foreach (session::$data['compare'] as $key => $product_id) {
			$product_info = catalog_product::getProduct($product_id);

			if ($product_info) {
				if ($product_info['image']) {
					$image = tool_image::resize($product_info['image'], config::get('theme_' . config('theme') . '_image_compare_width'), config::get('theme_' . config('theme') . '_image_compare_height'));
				} else {
					$image = false;
				}

				if (customer::isLogged() || !config('customer_price')) {
					$price = currency::format(tax::calculate($product_info['price'], $product_info['tax_class_id'], config('tax')), session::$data['currency']);
				} else {
					$price = false;
				}

				if (!is_null($product_info['special']) && (float)$product_info['special'] >= 0) {
					$special = currency::format(tax::calculate($product_info['special'], $product_info['tax_class_id'], config('tax')), session::$data['currency']);
				} else {
					$special = false;
				}

				if ($product_info['quantity'] <= 0) {
					$availability = $product_info['stock_status'];
				} elseif (config('stock_display')) {
					$availability = $product_info['quantity'];
				} else {
					$availability = language::get('text_instock');
				}

				$attribute_data = array();

				$attribute_groups = catalog_product::getProductAttributes($product_id);

				foreach ($attribute_groups as $attribute_group) {
					foreach ($attribute_group['attribute'] as $attribute) {
						$attribute_data[$attribute['attribute_id']] = $attribute['text'];
					}
				}

				$data['products'][$product_id] = array(
					'product_id'   => $product_info['product_id'],
					'name'         => $product_info['name'],
					'thumb'        => $image,
					'price'        => $price,
					'special'      => $special,
					'description'  => utf8_substr(strip_tags(html_entity_decode($product_info['description'], ENT_QUOTES, 'UTF-8')), 0, 200) . '..',
					'model'        => $product_info['model'],
					'manufacturer' => $product_info['manufacturer'],
					'availability' => $availability,
					'minimum'      => $product_info['minimum'] > 0 ? $product_info['minimum'] : 1,
					'rating'       => (int)$product_info['rating'],
					'reviews'      => sprintf(language::get('text_reviews'), (int)$product_info['reviews']),
					'weight'       => weight::format($product_info['weight'], $product_info['weight_class_id']),
					'length'       => length::format($product_info['length'], $product_info['length_class_id']),
					'width'        => length::format($product_info['width'], $product_info['length_class_id']),
					'height'       => length::format($product_info['height'], $product_info['length_class_id']),
					'attribute'    => $attribute_data,
					'href'         => url::link('product/product', 'product_id=' . $product_id),
					'remove'       => url::link('product/compare', 'remove=' . $product_id)
				);

				foreach ($attribute_groups as $attribute_group) {
					$data['attribute_groups'][$attribute_group['attribute_group_id']]['name'] = $attribute_group['name'];

					foreach ($attribute_group['attribute'] as $attribute) {
						$data['attribute_groups'][$attribute_group['attribute_group_id']]['attribute'][$attribute['attribute_id']]['name'] = $attribute['name'];
					}
				}
			} else {
				unset(session::$data['compare'][$key]);
			}
		}

		$data['continue'] = url::link('common/home');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('product/compare', $data));
	}

	public static function add() {
		load::language('product/compare');

		$json = array();

		if (!isset(session::$data['compare'])) {
			session::$data['compare'] = array();
		}

		if (isset(request::$post['product_id'])) {
			$product_id = request::$post['product_id'];
		} else {
			$product_id = 0;
		}


		$product_info = catalog_product::getProduct($product_id);

		if ($product_info) {
			if (!in_array(request::$post['product_id'], session::$data['compare'])) {
				if (count(session::$data['compare']) >= 4) {
					array_shift(session::$data['compare']);
				}

				session::$data['compare'][] = request::$post['product_id'];
			}

			$json['success'] = sprintf(language::get('text_success'), url::link('product/product', 'product_id=' . request::$post['product_id']), $product_info['name'], url::link('product/compare'));

			$json['total'] = sprintf(language::get('text_compare'), (isset(session::$data['compare']) ? count(session::$data['compare']) : 0));
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}
