<?php
class ControllerProductSpecial {
	public static function index() {
		load::language('product/special');



		if (isset(request::$get['sort'])) {
			$sort = request::$get['sort'];
		} else {
			$sort = 'p.sort_order';
		}

		if (isset(request::$get['order'])) {
			$order = request::$get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset(request::$get['page'])) {
			$page = (int)request::$get['page'];
		} else {
			$page = 1;
		}

		if (isset(request::$get['limit'])) {
			$limit = (int)request::$get['limit'];
		} else {
			$limit = config::get('theme_' . config('theme') . '_product_limit');
		}

		document::setTitle(language::get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => language::get('text_home'),
			'href' => url::link('common/home')
		);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['page'])) {
			$url .= '&page=' . request::$get['page'];
		}

		if (isset(request::$get['limit'])) {
			$url .= '&limit=' . request::$get['limit'];
		}

		$data['breadcrumbs'][] = array(
			'text' => language::get('heading_title'),
			'href' => url::link('product/special', $url)
		);

		$data['text_compare'] = sprintf(language::get('text_compare'), (isset(session::$data['compare']) ? count(session::$data['compare']) : 0));

		$data['compare'] = url::link('product/compare');

		$data['products'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * $limit,
			'limit' => $limit
		);

		$product_total = catalog_product::getTotalProductSpecials();

		$results = catalog_product::getProductSpecials($filter_data);

		foreach ($results as $result) {
			if ($result['image']) {
				$image = tool_image::resize($result['image'], config::get('theme_' . config('theme') . '_image_product_width'), config::get('theme_' . config('theme') . '_image_product_height'));
			} else {
				$image = tool_image::resize('placeholder.png', config::get('theme_' . config('theme') . '_image_product_width'), config::get('theme_' . config('theme') . '_image_product_height'));
			}

			if (customer::isLogged() || !config('customer_price')) {
				$price = currency::format(tax::calculate($result['price'], $result['tax_class_id'], config('tax')), session::$data['currency']);
			} else {
				$price = false;
			}

			if (!is_null($result['special']) && (float)$result['special'] >= 0) {
				$special = currency::format(tax::calculate($result['special'], $result['tax_class_id'], config('tax')), session::$data['currency']);
				$tax_price = (float)$result['special'];
			} else {
				$special = false;
				$tax_price = (float)$result['price'];
			}

			if (config('tax')) {
				$tax = currency::format($tax_price, session::$data['currency']);
			} else {
				$tax = false;
			}

			if (config('review_status')) {
				$rating = (int)$result['rating'];
			} else {
				$rating = false;
			}

			$data['products'][] = array(
				'product_id'  => $result['product_id'],
				'thumb'       => $image,
				'name'        => $result['name'],
				'description' => utf8_substr(trim(strip_tags(html_entity_decode($result['description'], ENT_QUOTES, 'UTF-8'))), 0, config::get('theme_' . config('theme') . '_product_description_length')) . '..',
				'price'       => $price,
				'special'     => $special,
				'tax'         => $tax,
				'minimum'     => $result['minimum'] > 0 ? $result['minimum'] : 1,
				'rating'      => $result['rating'],
				'href'        => url::link('product/product', 'product_id=' . $result['product_id'] . $url)
			);
		}

		$url = '';

		if (isset(request::$get['limit'])) {
			$url .= '&limit=' . request::$get['limit'];
		}

		$data['sorts'] = array();

		$data['sorts'][] = array(
			'text'  => language::get('text_default'),
			'value' => 'p.sort_order-ASC',
			'href'  => url::link('product/special', 'sort=p.sort_order&order=ASC' . $url)
		);

		$data['sorts'][] = array(
			'text'  => language::get('text_name_asc'),
			'value' => 'pd.name-ASC',
			'href'  => url::link('product/special', 'sort=pd.name&order=ASC' . $url)
		);

		$data['sorts'][] = array(
			'text'  => language::get('text_name_desc'),
			'value' => 'pd.name-DESC',
			'href'  => url::link('product/special', 'sort=pd.name&order=DESC' . $url)
		);

		$data['sorts'][] = array(
			'text'  => language::get('text_price_asc'),
			'value' => 'ps.price-ASC',
			'href'  => url::link('product/special', 'sort=ps.price&order=ASC' . $url)
		);

		$data['sorts'][] = array(
			'text'  => language::get('text_price_desc'),
			'value' => 'ps.price-DESC',
			'href'  => url::link('product/special', 'sort=ps.price&order=DESC' . $url)
		);

		if (config('review_status')) {
			$data['sorts'][] = array(
				'text'  => language::get('text_rating_desc'),
				'value' => 'rating-DESC',
				'href'  => url::link('product/special', 'sort=rating&order=DESC' . $url)
			);

			$data['sorts'][] = array(
				'text'  => language::get('text_rating_asc'),
				'value' => 'rating-ASC',
				'href'  => url::link('product/special', 'sort=rating&order=ASC' . $url)
			);
		}

		$data['sorts'][] = array(
				'text'  => language::get('text_model_asc'),
				'value' => 'p.model-ASC',
				'href'  => url::link('product/special', 'sort=p.model&order=ASC' . $url)
		);

		$data['sorts'][] = array(
			'text'  => language::get('text_model_desc'),
			'value' => 'p.model-DESC',
			'href'  => url::link('product/special', 'sort=p.model&order=DESC' . $url)
		);

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		$data['limits'] = array();

		$limits = array_unique(array(config::get('theme_' . config('theme') . '_product_limit'), 25, 50, 75, 100));

		sort($limits);

		foreach($limits as $value) {
			$data['limits'][] = array(
				'text'  => $value,
				'value' => $value,
				'href'  => url::link('product/special', $url . '&limit=' . $value)
			);
		}

		$url = '';

		if (isset(request::$get['sort'])) {
			$url .= '&sort=' . request::$get['sort'];
		}

		if (isset(request::$get['order'])) {
			$url .= '&order=' . request::$get['order'];
		}

		if (isset(request::$get['limit'])) {
			$url .= '&limit=' . request::$get['limit'];
		}

		$pagination = new Pagination();
		$pagination::$total = $product_total;
		$pagination::$page = $page;
		$pagination::$limit = $limit;
		$pagination::$url = url::link('product/special', $url . '&page={page}');

		$data['pagination'] = $pagination::render();

		$data['results'] = sprintf(language::get('text_pagination'), ($product_total) ? (($page - 1) * $limit) + 1 : 0, ((($page - 1) * $limit) > ($product_total - $limit)) ? $product_total : ((($page - 1) * $limit) + $limit), $product_total, ceil($product_total / $limit));

		// http://googlewebmastercentral.blogspot.com/2011/09/pagination-with-relnext-and-relprev.html
		if ($page == 1) {
		    document::addLink(url::link('product/special', ''), 'canonical');
		} else {
		    document::addLink(url::link('product/special', 'page='. $page ), 'canonical');
		}		
		
		if ($page > 1) {
			document::addLink(url::link('product/special', (($page - 2) ? '&page='. ($page - 1) : '')), 'prev');
		}

		if ($limit && ceil($product_total / $limit) > $page) {
		    document::addLink(url::link('product/special', 'page='. ($page + 1)), 'next');
		}

		$data['sort'] = $sort;
		$data['order'] = $order;
		$data['limit'] = $limit;

		$data['continue'] = url::link('common/home');

		$data['column_left'] = load::controller('common/column_left');
		$data['column_right'] = load::controller('common/column_right');
		$data['content_top'] = load::controller('common/content_top');
		$data['content_bottom'] = load::controller('common/content_bottom');
		$data['footer'] = load::controller('common/footer');
		$data['header'] = load::controller('common/header');

		response::setOutput(load::view('product/special', $data));
	}
}
