<?php
class ControllerToolUpload {
	public static function index() {
		load::language('tool/upload');

		$json = array();

		if (!empty(request::$files['file']['name']) && is_file(request::$files['file']['tmp_name'])) {
			// Sanitize the filename
			$filename = basename(preg_replace('/[^a-zA-Z0-9\.\-\s+]/', '', html_entity_decode(request::$files['file']['name'], ENT_QUOTES, 'UTF-8')));

			// Validate the filename length
			if ((utf8_strlen($filename) < 3) || (utf8_strlen($filename) > 64)) {
				$json['error'] = language::get('error_filename');
			}

			// Allowed file extension types
			$allowed = array();

			$extension_allowed = preg_replace('~\r?\n~', "\n", config('file_ext_allowed'));

			$filetypes = explode("\n", $extension_allowed);

			foreach ($filetypes as $filetype) {
				$allowed[] = trim($filetype);
			}

			if (!in_array(strtolower(substr(strrchr($filename, '.'), 1)), $allowed)) {
				$json['error'] = language::get('error_filetype');
			}

			// Allowed file mime types
			$allowed = array();

			$mime_allowed = preg_replace('~\r?\n~', "\n", config('file_mime_allowed'));

			$filetypes = explode("\n", $mime_allowed);

			foreach ($filetypes as $filetype) {
				$allowed[] = trim($filetype);
			}

			if (!in_array(request::$files['file']['type'], $allowed)) {
				$json['error'] = language::get('error_filetype');
			}

			// Check to see if any PHP files are trying to be uploaded
			$content = file_get_contents(request::$files['file']['tmp_name']);

			if (preg_match('/\<\?php/i', $content)) {
				$json['error'] = language::get('error_filetype');
			}

			// Return any upload error
			if (request::$files['file']['error'] != UPLOAD_ERR_OK) {
				$json['error'] = language::get('error_upload_' . request::$files['file']['error']);
			}
		} else {
			$json['error'] = language::get('error_upload');
		}

		if (!$json) {
			$file = $filename . '.' . token(32);

			move_uploaded_file(request::$files['file']['tmp_name'], DIR_UPLOAD . $file);

			// Hide the uploaded file name so people can not link to it directly.

			$json['code'] = tool_upload::addUpload($filename, $file);

			$json['success'] = language::get('text_upload');
		}

		response::addHeader('Content-Type: application/json');
		response::setOutput(json_encode($json));
	}
}