<?php
class extension_payment_alipay {
	private static $apiMethodName="alipay.trade.page.pay";
	private static $postCharset = "UTF-8";
	private static $alipaySdkVersion = "alipay-sdk-php-20161101";
	private static $apiVersion="1.0";
	private static $logFileName = "alipay.log";
	private static $gateway_url = "https://openapi.alipay.com/gateway.do";
	private static $alipay_public_key;
	private static $private_key;
	private static $appid;
	private static $notifyUrl;
	private static $returnUrl;
	private static $format = "json";
	private static $signtype = "RSA2";

	private static $apiParas = array();

	public static function getMethod($address, $total) {
		load::language('extension/payment/alipay');

		$query = db::query("SELECT * FROM zone_to_geo_zone WHERE geo_zone_id = " . (int)config::get('payment_alipay_geo_zone_id') . " AND country_id = " . (int)$address['country_id'] . " AND (zone_id = " . (int)$address['zone_id'] . " OR zone_id = '0')");

		if (config::get('payment_alipay_total') > 0 && config::get('payment_alipay_total') > $total) {
			$status = false;
		} elseif (!config::get('payment_alipay_geo_zone_id')) {
			$status = true;
		} elseif ($query->num_rows) {
			$status = true;
		} else {
			$status = false;
		}

		$method_data = array();

		if ($status) {
			$method_data = array(
				'code'       => 'alipay',
				'title'      => language::get('text_title'),
				'terms'      => '',
				'sort_order' => config::get('payment_alipay_sort_order')
			);
		}

		return $method_data;
	}

	private static function setParams($alipay_config){
		self::$gateway_url = $alipay_config['gateway_url'];
		self::$appid = $alipay_config['app_id'];
		self::$private_key = $alipay_config['merchant_private_key'];
		self::$alipay_public_key = $alipay_config['alipay_public_key'];
		self::$postCharset = $alipay_config['charset'];
		self::$signtype = $alipay_config['sign_type'];
		self::$notifyUrl = $alipay_config['notify_url'];
		self::$returnUrl = $alipay_config['return_url'];

		if (empty(self::$appid)||trim(self::$appid)=="") {
			throw new Exception("appid should not be NULL!");
		}
		if (empty(self::$private_key)||trim(self::$private_key)=="") {
			throw new Exception("private_key should not be NULL!");
		}
		if (empty(self::$alipay_public_key)||trim(self::$alipay_public_key)=="") {
			throw new Exception("alipay_public_key should not be NULL!");
		}
		if (empty(self::$postCharset)||trim(self::$postCharset)=="") {
			throw new Exception("charset should not be NULL!");
		}
		if (empty(self::$gateway_url)||trim(self::$gateway_url)=="") {
			throw new Exception("gateway_url should not be NULL!");
		}
	}

	static function pagePay($builder,$config) {
		self::setParams($config);
		$biz_content=null;
		if(!empty($builder)){
			$biz_content = json_encode($builder,JSON_UNESCAPED_UNICODE);
		}

		$log = new Log(self::$logFileName);
		$log::write($biz_content);

		self::$apiParas["biz_content"] = $biz_content;

		$response = self::pageExecute($this, "post");
		$log = new Log(self::$logFileName);
		$log::write("response: ".var_export($response,true));

		return $response;
	}

	static function check($arr, $config){
		self::setParams($config);

		$result = self::rsaCheckV1($arr, self::$signtype);

		return $result;
	}

	public static function pageExecute($request, $httpmethod = "POST") {
		$iv=self::$apiVersion;

		$sysParams["app_id"] = self::$appid;
		$sysParams["version"] = $iv;
		$sysParams["format"] = self::$format;
		$sysParams["sign_type"] = self::$signtype;
		$sysParams["method"] = self::$apiMethodName;
		$sysParams["timestamp"] = date("Y-m-d H:i:s");
		$sysParams["alipay_sdk"] = self::$alipaySdkVersion;
		$sysParams["notify_url"] = self::$notifyUrl;
		$sysParams["return_url"] = self::$returnUrl;
		$sysParams["charset"] = self::$postCharset;
		$sysParams["gateway_url"] = self::$gateway_url;

		$apiParams = self::$apiParas;

		$totalParams = array_merge($apiParams, $sysParams);

		$totalParams["sign"] = self::generateSign($totalParams, self::$signtype);

		if ("GET" == strtoupper($httpmethod)) {
			$preString=self::getSignContentUrlencode($totalParams);
			$requestUrl = self::$gateway_url."?".$preString;

			return $requestUrl;
		} else {
			foreach ($totalParams as $key => $value) {
				if (false === self::checkEmpty($value)) {
					$value = str_replace("\"", "&quot;", $value);
					$totalParams[$key] = $value;
				} else {
					unset($totalParams[$key]);
				}
			}
			return $totalParams;
		}
	}

	protected static function checkEmpty($value) {
		if (!isset($value))
			return true;
		if ($value === null)
			return true;
		if (trim($value) === "")
			return true;

		return false;
	}

	public static function rsaCheckV1($params, $signType='RSA') {
		$sign = $params['sign'];
		$params['sign_type'] = null;
		$params['sign'] = null;
		return self::verify(self::getSignContent($params), $sign, $signType);
	}

	static function verify($data, $sign, $signType = 'RSA') {
		$pubKey= self::$alipay_public_key;
		$res = "-----BEGIN PUBLIC KEY-----\n" .
			wordwrap($pubKey, 64, "\n", true) .
			"\n-----END PUBLIC KEY-----";

		(trim($pubKey)) or die('Alipay public key error!');

		if ("RSA2" == $signType) {
			$result = (bool)openssl_verify($data, base64_decode($sign), $res, OPENSSL_ALGO_SHA256);
		} else {
			$result = (bool)openssl_verify($data, base64_decode($sign), $res);
		}

		return $result;
	}

	public static function getSignContent($params) {
		ksort($params);

		$stringToBeSigned = "";
		$i = 0;
		foreach ($params as $k => $v) {
			if (false === self::checkEmpty($v) && "@" != substr($v, 0, 1)) {
				if ($i == 0) {
					$stringToBeSigned .= "$k" . "=" . "$v";
				} else {
					$stringToBeSigned .= "&" . "$k" . "=" . "$v";
				}
				$i++;
			}
		}

		unset ($k, $v);
		return $stringToBeSigned;
	}

	public static function generateSign($params, $signType = "RSA") {
		return self::sign(self::getSignContent($params), $signType);
	}

	protected static function sign($data, $signType = "RSA") {
		$priKey=self::$private_key;
		$res = "-----BEGIN RSA PRIVATE KEY-----\n" .
			wordwrap($priKey, 64, "\n", true) .
			"\n-----END RSA PRIVATE KEY-----";

		if ("RSA2" == $signType) {
			openssl_sign($data, $sign, $res, OPENSSL_ALGO_SHA256);
		} else {
			openssl_sign($data, $sign, $res);
		}

		$sign = base64_encode($sign);
		return $sign;
	}

	static function getPostCharset(){
		return trim(self::$postCharset);
	}
}
