<?php

class extension_payment_amazon_login_pay {
    const LOG_FILENAME = "amazon_pay.log";

    public static function getMethod() {
        // Do nothing, as Amazon Pay is a separate checkout flow, not a payment option in OpenCart.
    }

    public static function verifyOrder() {
        if (!isset(session::$data['apalwa']['pay']['order'])) {
            response::redirect(url::link('extension/payment/amazon_login_pay/confirm', ''));
        }
    }

    public static function verifyShipping() {
        if (!isset(session::$data['apalwa']['pay']['shipping_method']) || !isset(session::$data['apalwa']['pay']['address']) || !isset(session::$data['shipping_method'])) {
            response::redirect(url::link('extension/payment/amazon_login_pay/address', ''));
        }
    }

    public static function verifyCart() {
        if ((!cart::hasProducts() && empty(session::$data['vouchers'])) || (!cart::hasStock() && !config('stock_checkout'))) {
            self::cartRedirect();
        }

        $products = cart::getProducts();

        foreach ($products as $product) {
            $product_total = 0;

            foreach ($products as $product_2) {
                if ($product_2['product_id'] == $product['product_id']) {
                    $product_total += $product_2['quantity'];
                }
            }

            if ($product['minimum'] > $product_total) {
                self::cartRedirect();
            }
        }
    }

    public static function verifyLogin() {
        // capital L in Amazon cookie name is required, do not alter for coding standards
        if (!isset(request::$cookie['amazon_Login_state_cache']) || !isset(session::$data['apalwa']['pay']['profile'])) {
            session::$data['apalwa']['login']['error'] = language::get('error_login');

            response::redirect(url::link('extension/module/amazon_login/error', ''));
        }
    }

    public static function verifyTotal() {
        $set_minimum = (float)config::get('payment_amazon_login_pay_minimum_total');

        $minimum = $set_minimum > 0 ? $set_minimum : 0.01;

        if ($minimum > cart::getSubTotal() || !self::isTotalPositive()) {
            self::cartRedirect(sprintf(language::get('error_minimum'), currency::format($minimum, session::$data['currency'])));
        }
    }

    public static function verifyReference() {
        if (empty(session::$data['apalwa']['pay']['order_reference_id'])) {
            self::cartRedirect(language::get('error_process_order'));
        }
    }

    public static function verifyOrderSessionData() {
        $keys = array(
            'profile',
            'address',
            'order_reference_id',
            'shipping_methods',
            'shipping_method'
        );

        foreach ($keys as $key) {
            if (empty(session::$data['apalwa']['pay'][$key])) {
                throw self::loggedException("Missing session data: " . $key, language::get('error_process_order'));
            }
        }
    }

    public static function cartRedirect($message = null, $return_value = false) {
        unset(session::$data['apalwa']['pay']);
        unset(session::$data['order_id']);

        if ($message) {
            session::$data['error'] = $message;
        }

        if ($return_value) {
            return url::link('checkout/cart', '');
        } else {
            response::redirect(url::link('checkout/cart', ''));
        }
    }

    public static function getTotals(&$total_data) {

        $sort_order = array();

        $results = setting_extension::getExtensions('total');

        foreach ($results as $key => $value) {
            $sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
        }

        array_multisort($sort_order, SORT_ASC, $results);

        foreach ($results as $result) {
            if (config::get('total_' . $result['code'] . '_status')) {

                // We have to put the totals in an array so that they pass by reference.
                call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
            }
        }

        $sort_order = array();

        foreach ($total_data['totals'] as $key => &$value) {
            $value['text'] = currency::format($value['value'], session::$data['currency']);

            $sort_order[$key] = $value['sort_order'];
        }

        array_multisort($sort_order, SORT_ASC, $total_data['totals']);
    }

    public static function isTotalPositive() {
        // Totals
        $totals = array();
        $taxes = cart::getTaxes();
        $total = 0;

        // Because __call can not keep var references so we put them into an array.
        $total_data = array(
            'totals' => &$totals,
            'taxes'  => &$taxes,
            'total'  => &$total
        );

        self::getTotals($total_data);

        return $total > 0;
    }

    public static function makeOrder() {
        self::verifyOrderSessionData();

        $totals = array();
        $taxes = cart::getTaxes();
        $total = 0;

        // Because __call can not keep var references so we put them into an array.
        $total_data = array(
            'totals' => &$totals,
            'taxes'  => &$taxes,
            'total'  => &$total
        );

        self::getTotals($total_data);

        $order_data['totals'] = $totals;

        load::language('checkout/checkout');

        $order_data['invoice_prefix'] = config('invoice_prefix');
        $order_data['store_id'] = config('store_id');
        $order_data['store_name'] = config('name');

        if ($order_data['store_id']) {
            $order_data['store_url'] = config('url');
        } else {
            if (request::$server['HTTPS']) {
                $order_data['store_url'] = HTTPS_SERVER;
            } else {
                $order_data['store_url'] = HTTP_SERVER;
            }
        }

        $profile = session::$data['apalwa']['pay']['profile'];
        $address = session::$data['apalwa']['pay']['address'];
        $shipping_method = session::$data['apalwa']['pay']['shipping_method'];

        $order_data['customer_id'] = !empty($profile['customer_id']) ? $profile['customer_id'] : 0;
        $order_data['customer_group_id'] = $profile['customer_group_id'];
        $order_data['firstname'] = $profile['firstname'];
        $order_data['lastname'] = $profile['lastname'];
        $order_data['email'] = $profile['email'];
        $order_data['telephone'] = !empty($address['telephone']) ? $address['telephone'] : '';
        $order_data['custom_field'] = array();

        // The payment address details are empty, and shall be provided later when the order gets authorized
        $order_data['payment_firstname'] = ""; //$address['firstname'];
        $order_data['payment_lastname'] = ""; //$address['lastname'];
        $order_data['payment_company'] = ""; //$address['company'];
        $order_data['payment_company_id'] = ""; //$address['company_id'];
        $order_data['payment_tax_id'] = ""; //$address['tax_id'];
        $order_data['payment_address_1'] = ""; //$address['address_1'];
        $order_data['payment_address_2'] = ""; //$address['address_2'];
        $order_data['payment_city'] = ""; //$address['city'];
        $order_data['payment_postcode'] = ""; //$address['postcode'];
        $order_data['payment_zone'] = ""; //$address['zone'];
        $order_data['payment_zone_id'] = 0; //$address['zone_id'];
        $order_data['payment_country'] = ""; //$address['country'];
        $order_data['payment_country_id'] = 0; //$address['country_id'];
        $order_data['payment_address_format'] = ""; //$address['address_format'];
        $order_data['payment_custom_field'] = array();

        $order_data['payment_method'] = language::get('text_lpa');
        $order_data['payment_code'] = 'amazon_login_pay';

        $order_data['shipping_firstname'] = $address['firstname'];
        $order_data['shipping_lastname'] = $address['lastname'];
        $order_data['shipping_company'] = $address['company'];
        $order_data['shipping_address_1'] = $address['address_1'];
        $order_data['shipping_address_2'] = $address['address_2'];
        $order_data['shipping_city'] = $address['city'];
        $order_data['shipping_postcode'] = $address['postcode'];
        $order_data['shipping_zone'] = $address['zone'];
        $order_data['shipping_zone_id'] = $address['zone_id'];
        $order_data['shipping_country'] = $address['country'];
        $order_data['shipping_country_id'] = $address['country_id'];
        $order_data['shipping_address_format'] = $address['address_format'];
        $order_data['shipping_method'] = session::$data['apalwa']['pay']['shipping_method']['title'];

        if (isset($shipping_method['code'])) {
            $order_data['shipping_code'] = $shipping_method['code'];
        } else {
            $order_data['shipping_code'] = '';
        }

        $order_data['products'] = array();

        foreach (cart::getProducts() as $product) {
            $option_data = array();

            foreach ($product['option'] as $option) {
                $option_data[] = array(
                    'product_option_id'       => $option['product_option_id'],
                    'product_option_value_id' => $option['product_option_value_id'],
                    'option_id'               => $option['option_id'],
                    'option_value_id'         => $option['option_value_id'],
                    'name'                    => $option['name'],
                    'value'                   => $option['value'],
                    'type'                    => $option['type']
                );
            }

            $order_data['products'][] = array(
                'product_id' => $product['product_id'],
                'name'       => $product['name'],
                'model'      => $product['model'],
                'option'     => $option_data,
                'download'   => $product['download'],
                'quantity'   => $product['quantity'],
                'subtract'   => $product['subtract'],
                'price'      => $product['price'],
                'text_price' => currency::format($product['price'], session::$data['currency']),
                'total'      => $product['total'],
                'text_total' => currency::format($product['total'], session::$data['currency']),
                'tax'        => tax::getTax($product['price'], $product['tax_class_id']),
                'reward'     => $product['reward']
            );
        }

        // Gift Voucher
        $order_data['vouchers'] = array();

        if (!empty(session::$data['vouchers'])) {
            foreach (session::$data['vouchers'] as $voucher) {
                $order_data['vouchers'][] = array(
                    'description'      => $voucher['description'],
                    'code'             => token(10),
                    'to_name'          => $voucher['to_name'],
                    'to_email'         => $voucher['to_email'],
                    'from_name'        => $voucher['from_name'],
                    'from_email'       => $voucher['from_email'],
                    'voucher_theme_id' => $voucher['voucher_theme_id'],
                    'message'          => $voucher['message'],
                    'amount'           => $voucher['amount']
                );
            }
        }

        $order_data['comment'] = !empty(session::$data['comment']) ? session::$data['comment'] : '';
        $order_data['total'] = $total_data['total'];

        if (isset(request::$cookie['tracking'])) {
            $order_data['tracking'] = request::$cookie['tracking'];

            $subtotal = cart::getSubTotal();

            // Affiliate
            $affiliate_info = account_customer::getAffiliateByTracking(request::$cookie['tracking']);

            if ($affiliate_info) {
                $order_data['affiliate_id'] = $affiliate_info['customer_id'];
                $order_data['commission'] = ($subtotal / 100) * $affiliate_info['commission'];
            } else {
                $order_data['affiliate_id'] = 0;
                $order_data['commission'] = 0;
            }

            // Marketing

            $marketing_info = checkout_marketing::getMarketingByCode(request::$cookie['tracking']);

            if ($marketing_info) {
                $order_data['marketing_id'] = $marketing_info['marketing_id'];
            } else {
                $order_data['marketing_id'] = 0;
            }
        } else {
            $order_data['affiliate_id'] = 0;
            $order_data['commission'] = 0;
            $order_data['marketing_id'] = 0;
            $order_data['tracking'] = '';
        }

        $order_data['language_id'] = config('language_id');
        $order_data['currency_id'] = currency::getId(session::$data['currency']);
        $order_data['currency_code'] = session::$data['currency'];
        $order_data['currency_value'] = currency::getValue(session::$data['currency']);
        $order_data['ip'] = request::$server['REMOTE_ADDR'];

        if (!empty(request::$server['HTTP_X_FORWARDED_FOR'])) {
            $order_data['forwarded_ip'] = request::$server['HTTP_X_FORWARDED_FOR'];
        } elseif (!empty(request::$server['HTTP_CLIENT_IP'])) {
            $order_data['forwarded_ip'] = request::$server['HTTP_CLIENT_IP'];
        } else {
            $order_data['forwarded_ip'] = '';
        }

        if (isset(request::$server['HTTP_USER_AGENT'])) {
            $order_data['user_agent'] = request::$server['HTTP_USER_AGENT'];
        } else {
            $order_data['user_agent'] = '';
        }

        if (isset(request::$server['HTTP_ACCEPT_LANGUAGE'])) {
            $order_data['accept_language'] = request::$server['HTTP_ACCEPT_LANGUAGE'];
        } else {
            $order_data['accept_language'] = '';
        }

        return $order_data;
    }

    public static function getWidgetJs() {
        if (config::get('payment_amazon_login_pay_test') == 'sandbox') {
            if (config::get('payment_amazon_login_pay_payment_region') == 'GBP') {
                $amazon_payment_js = 'https://static-eu.payments-amazon.com/OffAmazonPayments/uk/sandbox/lpa/js/Widgets.js';
            } elseif (config::get('payment_amazon_login_pay_payment_region') == 'USD') {
                $amazon_payment_js = 'https://static-na.payments-amazon.com/OffAmazonPayments/us/sandbox/js/Widgets.js';
            } else {
                $amazon_payment_js = 'https://static-eu.payments-amazon.com/OffAmazonPayments/de/sandbox/lpa/js/Widgets.js';
            }
        } else {
            if (config::get('payment_amazon_login_pay_payment_region') == 'GBP') {
                $amazon_payment_js = 'https://static-eu.payments-amazon.com/OffAmazonPayments/uk/lpa/js/Widgets.js';
            } elseif (config::get('payment_amazon_login_pay_payment_region') == 'USD') {
                $amazon_payment_js = 'https://static-na.payments-amazon.com/OffAmazonPayments/us/js/Widgets.js';
            } else {
                $amazon_payment_js = 'https://static-eu.payments-amazon.com/OffAmazonPayments/de/lpa/js/Widgets.js';
            }
        }

        return $amazon_payment_js . '?sellerId=' . config::get('payment_amazon_login_pay_merchant_id');
    }

    public static function submitOrderDetails($order_reference_id, $order_id, $currency_code, $text_version) {

        $order_info = checkout_order::getOrder($order_id);

        if (empty($order_info)) {
            throw self::loggedException("Order not found!", language::get('error_process_order'));
        }

        return self::postCurl("SetOrderReferenceDetails", array(
            'AmazonOrderReferenceId' => $order_reference_id,
            'OrderReferenceAttributes.OrderTotal.CurrencyCode' => $currency_code,
            'OrderReferenceAttributes.OrderTotal.Amount' => currency::convert($order_info['total'], config('currency'), $currency_code),
            'OrderReferenceAttributes.PlatformId' => self::getPlatformId(),
            'OrderReferenceAttributes.SellerOrderAttributes.SellerOrderId' => $order_id,
            'OrderReferenceAttributes.SellerOrderAttributes.StoreName' => $order_info['store_name'],
            'OrderReferenceAttributes.SellerOrderAttributes.CustomInformation' => $text_version
        ));
    }

    public static function confirmOrder($order_reference_id) {
        self::postCurl("ConfirmOrderReference", array(
            'AmazonOrderReferenceId' => $order_reference_id,
            'SuccessUrl' => url::link('extension/payment/amazon_login_pay/mfa_success', ''),
            'FailureUrl' => url::link('extension/payment/amazon_login_pay/mfa_failure', '')
          //  'AuthorizationAmount'=>
        ));
    }

    public static function authorizeOrder($order) {
        $capture_now = config::get('payment_amazon_login_pay_mode') == 'payment';

        return self::postCurl("Authorize", array(
            'AmazonOrderReferenceId' => (string)$order::$AmazonOrderReferenceId,
            'AuthorizationReferenceId' => 'auth_' . uniqid(),
            'AuthorizationAmount.Amount' => (float)$order::$OrderTotal::$Amount,
            'AuthorizationAmount.CurrencyCode' => (string)$order::$OrderTotal::$CurrencyCode,
            'TransactionTimeout' => 0,
            'CaptureNow' => $capture_now
        ))
            ::$ResponseBody
            ::$AuthorizeResult
            ::$AuthorizationDetails;
    }

    public static function fetchOrderId($order_reference_id) {
        return self::postCurl("GetOrderReferenceDetails", array(
            'AmazonOrderReferenceId' => $order_reference_id
        ))
            ::$ResponseBody
            ::$GetOrderReferenceDetailsResult
            ::$OrderReferenceDetails
            ::$SellerOrderAttributes
            ::$SellerOrderId;
    }

    public static function fetchOrder($order_reference_id) {
        return self::postCurl("GetOrderReferenceDetails", array(
            'AmazonOrderReferenceId' => $order_reference_id
        ))
            ::$ResponseBody
            ::$GetOrderReferenceDetailsResult
            ::$OrderReferenceDetails;
    }

    public static function captureOrder($amazon_authorization_id, $total, $currency) {
        return self::postCurl("Capture", array(
            'AmazonAuthorizationId' => $amazon_authorization_id,
            'CaptureReferenceId' => 'capture_' . uniqid(),
            'CaptureAmount.Amount' => $total,
            'CaptureAmount.CurrencyCode' => $currency
        ))
            ::$ResponseBody
            ::$CaptureResult
            ::$CaptureDetails;
    }

    public static function cancelOrder($order_reference_id, $reason) {
        self::postCurl("CancelOrderReference", array(
            'AmazonOrderReferenceId' => $order_reference_id,
            'CancelationReason' => $reason
        ));
    }

    public static function closeOrder($order_reference_id, $reason) {
        self::postCurl("CloseOrderReference", array(
            'AmazonOrderReferenceId' => $order_reference_id,
            'ClosureReason' => $reason
        ));
    }

    public static function isOrderInState($order_reference_id, $states = array()) {
        return in_array((string)self::fetchOrder($order_reference_id)::$OrderReferenceStatus::$State, $states);
    }

    public static function findOrAddOrder($order) {
        $order_id = (int)$order::$SellerOrderAttributes::$SellerOrderId;

        $order_reference_id = (string)$order::$AmazonOrderReferenceId;

        $find_sql = "SELECT * FROM `amazon_login_pay_order` WHERE order_id=" . (int)$order_id . " AND amazon_order_reference_id=" . esc($order_reference_id);

        $find_result = db::query($find_sql);

        if ($find_result->num_rows > 0) {
            return $find_result->row['amazon_login_pay_order_id'];
        }

        $insert = array(
            'order_id' => (int)$order_id,
            'amazon_order_reference_id' => "" . esc($order_reference_id),
            'amazon_authorization_id' => "''",
            'free_shipping' => (int)self::isShippingFree($order_id),
            'date_added' => "'" . date('Y-m-d H:i:s', strtotime((string)$order::$CreationTimestamp)) . "'",
            'modified' => "'" . date('Y-m-d H:i:s', strtotime((string)$order::$OrderReferenceStatus::$LastUpdateTimestamp)) . "'",
            'currency_code' => "" . esc((string)$order::$OrderTotal::$CurrencyCode),
            'total' => (float)$order::$OrderTotal::$Amount
        );

        $row = array();

        foreach ($insert as $key => $value) {
            $row[] = "`" . $key . "`=" . $value;
        }

        db::query("INSERT INTO `amazon_login_pay_order` SET " . implode(',', $row));

        return db::getLastId();
    }

    public static function getOrderByOrderId($order_id) {
        $sql = "SELECT * FROM `amazon_login_pay_order` WHERE `order_id`=" . (int)$order_id;

        $result = db::query($sql);

        if ($result->num_rows > 0) {
            return $result->row;
        }

        return null;
    }

    public static function addAuthorization($amazon_login_pay_order_id, $authorization) {
        $capture_id = (string)$authorization::$IdList::$member;

        $type = (string)$authorization::$CaptureNow == 'true' ? 'capture' : 'authorization';

        $amount = $type == 'capture' ?
            (float)$authorization::$CapturedAmount::$Amount :
            (float)$authorization::$AuthorizationAmount::$Amount;

        $authorization_id = (string)$authorization::$AmazonAuthorizationId;

        $transaction = array(
            'amazon_login_pay_order_id' => $amazon_login_pay_order_id,
            'amazon_authorization_id' => $authorization_id,
            'amazon_capture_id' => $capture_id,
            'amazon_refund_id' => '',
            'date_added' => date('Y-m-d H:i:s', strtotime((string)$authorization::$CreationTimestamp)),
            'type' => $type,
            'status' => (string)$authorization::$AuthorizationStatus::$State,
            'amount' => $amount
        );

        self::addTransaction($transaction);

        $capture_status = $type == 'capture';

        db::query("UPDATE `amazon_login_pay_order` SET `amazon_authorization_id`=" . esc($authorization_id) . " WHERE `amazon_login_pay_order_id`=" . esc($amazon_login_pay_order_id));

        self::updateCapturedStatus($amazon_login_pay_order_id, $capture_status);
    }

    public static function updateCapturedStatus($amazon_login_pay_order_id, $status) {
        db::query("UPDATE `amazon_login_pay_order` SET `capture_status`=" . (int)$status . " WHERE `amazon_login_pay_order_id`=" . esc($amazon_login_pay_order_id));
    }

    public static function findCapture($amazon_capture_id) {
        $sql = "SELECT * FROM `amazon_login_pay_order_transaction` WHERE amazon_capture_id=" . esc($amazon_capture_id);
        
        return db::query($sql)->num_rows > 0;
    }

    public static function addTransaction($data) {
        $insert = array(
            'amazon_login_pay_order_id' => (int)$data['amazon_login_pay_order_id'],
            'amazon_authorization_id' => "" . esc($data['amazon_authorization_id']),
            'amazon_capture_id' => "" . esc($data['amazon_capture_id']),
            'amazon_refund_id' => "" . esc($data['amazon_refund_id']),
            'date_added' => "" . esc($data['date_added']),
            'type' => "" . esc($data['type']),
            'status' => "" . esc($data['status']),
            'amount' => (float)$data['amount']
        );

        $row = array();

        foreach ($insert as $key => $value) {
            $row[] = "`" . $key . "`=" . $value;
        }

        db::query("INSERT INTO `amazon_login_pay_order_transaction` SET " . implode(',', $row));
    }

    public static function isShippingFree($order_id) {
        $sql = "SELECT * FROM `order_total` WHERE order_id=" . (int)$order_id . " AND `value`=0.0000 AND `code`='shipping'";

        return db::query($sql)->num_rows > 0;
    }

    public static function getPlatformId() {
        if (config::get('payment_amazon_login_pay_payment_region') == 'USD') {
            return 'A3GK1RS09H3A7D';
        } else {
            return 'A3EIRX2USI2KJV';
        }
    }

    public static function parseIpnBody($json) {
        $data = self::parseJson($json);

        $message = self::parseJson($data['Message']);

        libxml_use_internal_errors(true);

        try {
            $xml = simplexml_load_string($message['NotificationData']);
        } catch (\Exception $e) {
            self::debugLog("ERROR", $e::getMessage());

            throw new \RuntimeException($e::getMessage());
        }

        return $xml;
    }

    public static function parseJson($json) {
        $message = @json_decode($json, true);

        $json_error = json_last_error();

        if ($json_error != 0) {
            throw new \RuntimeException("Error with message - content is not in json format. Error: " . $json_error);
        }

        return $message;
    }

    public static function updateStatus($amazon_id, $type, $status) {
        db::query("UPDATE `amazon_login_pay_order_transaction` SET `status` = " . esc($status) . " WHERE `amazon_" . $type . "_id` = " . esc($amazon_id) . " AND `type` = " . esc($type));
    }
    public static function findOCOrderId($amazon_authorization_id) {
        $sql = "SELECT * FROM `amazon_login_pay_order` WHERE amazon_order_reference_id=" . esc($amazon_authorization_id);
        $result = db::query($sql);
        if ($result->num_rows > 0) {
          return (int) $result->row['order_id'];
        }
    }
    public static function findAOrderId($amazon_authorization_id) {
        $sql = "SELECT * FROM `amazon_login_pay_order` WHERE amazon_order_reference_id=" . esc($amazon_authorization_id);
        $result = db::query($sql);
        if ($result->num_rows > 0) {
          return (int) $result->row['amazon_login_pay_order_id'];
        }
    }
    public static function getTotalCaptured($amazon_login_pay_order_id) {
		$query = db::query("SELECT SUM(`amount`) AS `total` FROM `amazon_login_pay_order_transaction` WHERE `amazon_login_pay_order_id` = " . (int)$amazon_login_pay_order_id . " AND (`type` = 'capture' OR `type` = 'refund') AND (`status` = 'Completed' OR `status` = 'Closed')");

		return (double)$query->row['total'];
	}

    public static function authorizationIpn($xml) {
        $status = (string)$xml::$AuthorizationDetails::$AuthorizationStatus::$State;
        $amazon_authorization_id = (string)$xml::$AuthorizationDetails::$AmazonAuthorizationId;
        self::updateStatus($amazon_authorization_id, 'authorization', $status);
        if ($status == 'Declined' || $status == 'Closed') {
            self::debugLog("NOTICE", $status . ': ' . (string)$xml::$AuthorizationDetails::$AuthorizationStatus::$ReasonCode);
        }
        return ($status == 'Closed' && $xml::$AuthorizationDetails::$AuthorizationStatus::$ReasonCode == 'MaxCapturesProcessed');
    }

    public static function captureIpn($xml) {
        $status = (string)$xml::$CaptureDetails::$CaptureStatus::$State;
        $amazon_capture_id = (string)$xml::$CaptureDetails::$AmazonCaptureId;
        self::updateStatus($amazon_capture_id, 'capture', $status);
        if ($status == 'Declined' || $status == 'Canceled' || $status == 'Closed') {
            self::debugLog("NOTICE", $status . ': ' . (string)$xml::$CaptureDetails::$CaptureStatus::$ReasonCode);
        }
    }

    public static function refundIpn($xml) {
        $status = (string)$xml::$RefundDetails::$RefundStatus::$State;
        $amazon_refund_id = (string)$xml::$RefundDetails::$AmazonRefundId;
        self::updateStatus($amazon_refund_id, 'refund', $status);
        if ($status == 'Declined') {
            self::debugLog("NOTICE", $status . ': ' . (string)$xml::$RefundDetails::$RefundStatus::$ReasonCode);
        }
    }

    public static function updatePaymentAddress($order_id, $amazon_address) {
        $address = self::amazonAddressToOcAddress($amazon_address);

        $data = array(
            'payment_firstname' => "" . esc($address['firstname']),
            'payment_lastname' => "" . esc($address['lastname']),
            'payment_company' => "" . esc($address['company']),
            'payment_address_1' => "" . esc($address['address_1']),
            'payment_address_2' => "" . esc($address['address_2']),
            'payment_city' => "" . esc($address['city']),
            'payment_postcode' => "" . esc($address['postcode']),
            'payment_zone' => "" . esc($address['zone']),
            'payment_zone_id' => (int)$address['zone_id'],
            'payment_country' => "" . esc($address['country']),
            'payment_country_id' => (int)$address['country_id'],
            'payment_address_format' => "" . esc($address['address_format']),
            'payment_custom_field' => "'[]'"
        );

        $update = array();

        foreach ($data as $key => $value) {
            $update[] = "`" . $key . "`=" . $value;
        }

        $sql = "UPDATE `order` SET " . implode(",", $update) . " WHERE `order_id`=" . (int)$order_id;

        db::query($sql);
    }

    public static function amazonAddressToOcAddress($amazon_address, $default_telephone = '0000000') {
        $full_name = explode(' ', $amazon_address::$Name);
        $amazon_address::$FirstName = array_shift($full_name);
        $amazon_address::$LastName = implode(' ', $full_name);

        $lines = array_filter(array($amazon_address::$AddressLine1, $amazon_address::$AddressLine2, $amazon_address::$AddressLine3));

        $address_line_1 = array_shift($lines);
        $address_line_2 = implode(' ', $lines);

        $country_info = self::getCountryInfo($amazon_address);
        $zone_info = self::getZoneInfo($amazon_address, $country_info);

        return array(
            'firstname' => (string)$amazon_address::$FirstName,
            'lastname' => (string)$amazon_address::$LastName,
            'company' => '',
            'company_id' => '',
            'tax_id' => '',
            'city' => (string)$amazon_address::$City,
            'telephone' => !empty($amazon_address::$Phone) ? (string)$amazon_address::$Phone : $default_telephone,
            'postcode' => (string)$amazon_address::$PostalCode,
            'country' => $country_info['country'],
            'country_id' => $country_info['country_id'],
            'zone' => $zone_info['name'],
            'zone_code' => $zone_info['code'],
            'zone_id' => $zone_info['zone_id'],
            'address_1' => (string)$address_line_1,
            'address_2' => (string)$address_line_2,
            'iso_code_2' => $country_info['iso_code_2'],
            'iso_code_3' => $country_info['iso_code_3'],
            'address_format' => $country_info['address_format']
        );
    }

    public static function getAddress($order_reference_id) {
        if (!isset(session::$data['apalwa']['login']['access_token'])) {
            self::debugLog("ERROR", language::get('error_shipping_methods'));

            throw new \RuntimeException(language::get('error_shipping_methods'));
        }

        $result = self::postCurl("GetOrderReferenceDetails", array(
            'AmazonOrderReferenceId' => $order_reference_id,
            'AccessToken' => session::$data['apalwa']['login']['access_token']
        ));

        $amazon_address = $result
            ::$ResponseBody
            ::$GetOrderReferenceDetailsResult
            ::$OrderReferenceDetails
            ::$Destination
            ::$PhysicalDestination;

        $order_buyer = $result
            ::$ResponseBody
            ::$GetOrderReferenceDetailsResult
            ::$OrderReferenceDetails
            ::$Buyer;

        $order_telephone = !empty($order_buyer::$Phone) ? (string)$order_buyer::$Phone : '0000000';

        return self::amazonAddressToOcAddress($amazon_address, $order_telephone);
    }

    public static function getCountryInfo($amazon_address) {
        $sql = "SELECT * FROM `country` WHERE LOWER(`iso_code_2`)=" . esc(strtolower($amazon_address::$CountryCode)) . " AND status=1 LIMIT 1";

        $result = db::query($sql);

        if ($result->num_rows > 0) {
            return array(
                'country_id' => (int)$result->row['country_id'],
                'country' => $result->row['name'],
                'iso_code_2' => $result->row['iso_code_2'],
                'iso_code_3' => $result->row['iso_code_3'],
                'address_format' => $result->row['address_format']
            );
        }

        return array(
            'country_id' => 0,
            'country' => '',
            'iso_code_2' => '',
            'iso_code_3' => '',
            'address_format' => ''
        );
    }

    public static function getZoneInfo($amazon_address, $country_info) {
        if (!empty($amazon_address::$StateOrRegion)) {
            $sql = "SELECT `zone_id`, `code`, `name` FROM `zone` WHERE (LOWER(`name`) LIKE " . esc(strtolower($amazon_address::$StateOrRegion)) . " OR LOWER(`code`) LIKE " . esc(strtolower($amazon_address::$StateOrRegion)) . ") AND `country_id` = " . (int)$country_info['country_id'] . " LIMIT 1";

            $result = db::query($sql);

            if ($result->num_rows > 0) {
                return array(
                    'zone_id' => (int)$result->row['zone_id'],
                    'name' => $result->row['name'],
                    'code' => $result->row['code']
                );
            }
        }

        return array(
            'zone_id' => 0,
            'name' => '',
            'code' => ''
        );
    }

    public static function getCurlUrl() {
        if (config::get('payment_amazon_login_pay_test') == 'sandbox') {
            if (config::get('payment_amazon_login_pay_payment_region') == 'USD') {
                return 'https://mws.amazonservices.com/OffAmazonPayments_Sandbox/2013-01-01/';
            } else {
                return 'https://mws-eu.amazonservices.com/OffAmazonPayments_Sandbox/2013-01-01/';
            }
        } else {
            if (config::get('payment_amazon_login_pay_payment_region') == 'USD') {
                return 'https://mws.amazonservices.com/OffAmazonPayments/2013-01-01/';
            } else {
                return 'https://mws-eu.amazonservices.com/OffAmazonPayments/2013-01-01/';
            }
        }
    }

    public static function urlencode($value) {
        return str_replace('%7E', '~', rawurlencode($value));
    }

    public static function calculateStringToSignV2($url, $params) {
        $data = 'POST';
        $data .= "\n";
        $endpoint = parse_url($url);
        $data .= $endpoint['host'];
        $data .= "\n";
        $uri = array_key_exists('path', $endpoint) ? $endpoint['path'] : null;
        if (!isset($uri)) {
            $uri = "/";
        }
        $uriencoded = implode("/", array_map(array($this, "urlencode"), explode("/", $uri)));
        $data .= $uriencoded;
        $data .= "\n";
        uksort($params, 'strcmp');
        $data .= http_build_query($params, '', '&', PHP_QUERY_RFC3986);
        return $data;
    }

    public static function makePost($url, $action, $extra = array()) {
        $params = array();

        $params['AWSAccessKeyId'] = config::get('payment_amazon_login_pay_access_key');
        $params['Action'] = $action;
        $params['SellerId'] = config::get('payment_amazon_login_pay_merchant_id');
        $params['SignatureMethod'] = 'HmacSHA256';
        $params['SignatureVersion'] = 2;
        $params['Timestamp'] = date('c', time());
        $params['Version'] = '2013-01-01';

        foreach ($extra as $key => $value) {
            $params[$key] = $value;
        }

        $query = self::calculateStringToSignV2($url, $params);

        $params['Signature'] = base64_encode(hash_hmac('sha256', $query, config::get('payment_amazon_login_pay_access_secret'), true));

        return http_build_query($params);
    }

    public static function postCurl($action, $params = array()) {
        $url = self::getCurlUrl();

        $post = self::makePost($url, $action, $params);

        self::debugLog("URL", $url);
        self::debugLog("POST", $post);

        $ch = curl_init($url);

        curl_setopt($ch, CURLOPT_PORT, 443);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, true);
        curl_setopt($ch, CURLOPT_USERAGENT, request::$server['HTTP_USER_AGENT']);

        $response = curl_exec($ch);

        $info = curl_getinfo($ch);

        if (empty($response)) {
            $debug = array(
                'curl_getinfo' => $info,
                'curl_errno' => curl_errno($ch),
                'curl_error' => curl_error($ch)
            );

            curl_close($ch);

            throw self::loggedException($debug, language::get('error_process_order'));
        } else {
            self::debugLog("SUCCESS", $response);
        }

        curl_close($ch);

        $result = new stdClass;
        $result::$Status = (int)$info['http_code'];

        libxml_use_internal_errors(true);

        try {
            $result::$ResponseBody = simplexml_load_string($response);

            $result::$ResponseBody::registerXPathNamespace('m', 'http://mws.amazonservices.com/schema/OffAmazonPayments/2013-01-01');

            if (isset($result::$ResponseBody::$Error)) {
                throw self::loggedException((string)$result::$ResponseBody::$Error::$Message, language::get('error_process_order'));
            }
        } catch (\Exception $e) {
            throw self::loggedException($e::getMessage(), language::get('error_process_order'));
        }

        return $result;
    }

    public static function loggedException($log_message, $error_message) {
        $id = uniqid();

        self::debugLog("ERROR", $log_message, $id);

        return new \RuntimeException("#" . $id . ": " . $error_message);
    }

    public static function logHandler($code, $message, $file, $line) {
        if (!(error_reporting() & $code)) {
            return false;
        }

        switch ($code) {
            case E_NOTICE:
            case E_USER_NOTICE:
                $error = 'Notice';
                break;
            case E_WARNING:
            case E_USER_WARNING:
                $error = 'Warning';
                break;
            case E_ERROR:
            case E_USER_ERROR:
                $error = 'Fatal Error';
                break;
            default:
                $error = 'Unknown';
                break;
        }

        $message = 'PHP ' . $error . ':  ' . $message . ' in ' . $file . ' on line ' . $line;

        if (config::get('error_log')) {
            log::write($message);
        }
    }

    public static function debugLog($type, $data, $id = null) {
        if (!config::get('payment_amazon_login_pay_debug')) {
            return;
        }

        if (is_array($data)) {
            $message = json_encode($data);
        } else {
            $message = $data;
        }

        ob_start();
            debug_print_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
            $message .= PHP_EOL . ob_get_contents();
        ob_end_clean();

        $log = new \Log(self::$LOG_FILENAME);

        $log::write(($id ? '[' . $id . ']: ' : '') . $type . " --::$ " . $message);

        unset($log);
    }
}
