<?php
class extension_payment_cardconnect {
	public static function getMethod($address, $total) {
		load::language('extension/payment/cardconnect');

		$query = db::query("SELECT * FROM `zone_to_geo_zone` WHERE `geo_zone_id` = " . (int)config::get('cardconnect_geo_zone') . " AND `country_id` = " . (int)$address['country_id'] . " AND (`zone_id` = " . (int)$address['zone_id'] . " OR `zone_id` = '0')");

		if (config::get('cardconnect_total') > 0 && config::get('cardconnect_total') > $total) {
			$status = false;
		} elseif (!config::get('cardconnect_geo_zone')) {
			$status = true;
		} elseif ($query->num_rows) {
			$status = true;
		} else {
			$status = false;
		}

		$method_data = array();

		if ($status) {
			$method_data = array(
				'code'			=> 'cardconnect',
				'title'			=> language::get('text_title'),
				'terms'			=> '',
				'sort_order'	=> config::get('cardconnect_sort_order')
			);
		}

		return $method_data;
	}

	public static function getCardTypes() {
		$cards = array();

		$cards[] = array(
			'text'  => 'Visa',
			'value' => 'VISA'
		);

		$cards[] = array(
			'text'  => 'MasterCard',
			'value' => 'MASTERCARD'
		);

		$cards[] = array(
			'text'  => 'Discover Card',
			'value' => 'DISCOVER'
		);

		$cards[] = array(
			'text'  => 'American Express',
			'value' => 'AMEX'
		);

		return $cards;
	}

	public static function getMonths() {
		$months = array();

		for ($i = 1; $i <= 12; $i++) {
			$months[] = array(
				'text'  => sprintf('%02d', $i),
				'value' => sprintf('%02d', $i)
			);
		}

		return $months;
	}

	public static function getYears() {
		$years = array();

		$today = getdate();

		for ($i = $today['year']; $i < $today['year'] + 11; $i++) {
			$years[] = array(
				'text'  => sprintf('%02d', $i % 100),
				'value' => sprintf('%02d', $i % 100)
			);
		}

		return $years;
	}

	public static function getCard($token, $customer_id) {
		$query = db::query("SELECT * FROM `cardconnect_card` WHERE `token` = " . esc($token) . " AND `customer_id` = " . (int)$customer_id);

		if ($query->num_rows) {
			return $query->row;
		} else {
			return false;
		}
	}

	public static function getCards($customer_id) {
		$query = db::query("SELECT * FROM `cardconnect_card` WHERE `customer_id` = " . (int)$customer_id);

		return $query->rows;
	}

	public static function addCard($cardconnect_order_id, $customer_id, $profileid, $token, $type, $account, $expiry) {
		db::query("INSERT INTO `cardconnect_card` SET `cardconnect_order_id` = " . (int)$cardconnect_order_id . ", `customer_id` = " . (int)$customer_id . ", `profileid` = " . esc($profileid) . ", `token` = " . esc($token) . ", `type` = " . esc($type) . ", `account` = " . esc($account) . ", `expiry` = " . esc($expiry) . ", `date_added` = NOW()");
	}

	public static function deleteCard($token, $customer_id) {
		db::query("DELETE FROM `cardconnect_card` WHERE `token` = " . esc($token) . " AND `customer_id` = " . (int)$customer_id);
	}

	public static function addOrder($order_info, $payment_method) {
		db::query("INSERT INTO `cardconnect_order` SET `order_id` = " . (int)$order_info['order_id'] . ", `customer_id` = " . (int)customer::getId() . ", `payment_method` = " . esc($payment_method) . ", `retref` = " . esc($order_info['retref']) . ", `authcode` = " . esc($order_info['authcode']) . ", `currency_code` = " . esc($order_info['currency_code']) . ", `total` = '" . currency::format($order_info['total'], $order_info['currency_code'], false, false) . "', `date_added` = NOW()");

		return db::getLastId();
	}

	public static function addTransaction($cardconnect_order_id, $type, $status, $order_info) {
		db::query("INSERT INTO `cardconnect_order_transaction` SET `cardconnect_order_id` = " . (int)$cardconnect_order_id . ", `type` = " . esc($type) . ", `retref` = " . esc($order_info['retref']) . ", `amount` = '" . (float)currency::format($order_info['total'], $order_info['currency_code'], false, false) . "', `status` = " . esc($status) . ", `date_modified` = NOW(), `date_added` = NOW()");
	}

	public static function getSettlementStatuses($merchant_id, $date) {
		self::log('Getting settlement statuses from CardConnect');

		$url = 'https://' . config::get('cardconnect_site') . '.cardconnect.com:' . ((config::get('cardconnect_environment') == 'live') ? 8443 : 6443) . '/cardconnect/rest/settlestat?merchid=' . $merchant_id . '&date=' . $date;

		$header = array();

		$header[] = 'Content-type: application/json';
		$header[] = 'Authorization: Basic ' . base64_encode(config::get('cardconnect_api_username') . ':' . config::get('cardconnect_api_password'));

		extension_payment_cardconnect::log('Header: ' . print_r($header, true));

		extension_payment_cardconnect::log('URL: ' . $url);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		$response_data = curl_exec($ch);
		if (curl_errno($ch)) {
			extension_payment_cardconnect::log('cURL error: ' . curl_errno($ch));
		}
		curl_close($ch);

		$response_data = json_decode($response_data, true);

		self::log('Response: ' . print_r($response_data, true));

		return $response_data;
	}

	public static function updateTransactionStatusByRetref($retref, $status) {
		db::query("UPDATE `cardconnect_order_transaction` SET `status` = " . esc($status) . ", `date_modified` = NOW() WHERE `retref` = " . esc($retref));
	}

	public static function updateCronRunTime() {
		db::query("DELETE FROM `setting` WHERE `key` = 'cardconnect_cron_time'");

		db::query("INSERT INTO `setting` SET `store_id` = '0', `code` = 'cardconnect', `key` = 'cardconnect_cron_time', `value` = NOW(), `serialized` = '0'");
	}

	public static function log($data) {
		if (config::get('cardconnect_logging')) {
			$log = new Log('cardconnect.log');

			$log::write($data);
		}
	}
}