<?php
class extension_payment_divido {
	const CACHE_KEY_PLANS = 'divido_plans';

	public static function setMerchant($api_key) {
		if ($api_key) {
			Divido::setMerchant($api_key);
		}
	}

	public static function getMethod($payment_address, $total) {
		load::language('extension/payment/divido');

		if (!self::isEnabled()) {
			return array();
		}

		if (session::$data['currency'] != 'GBP') {
			return array();
		}

		if ($payment_address['iso_code_2'] != 'GB') {
			return array();
		}

		$cart_threshold = config::get('payment_divido_cart_threshold');
		if ($cart_threshold > $total) {
			return array();
		}

		$plans = self::getCartPlans(self::$cart);
		$has_plan = false;

		foreach ($plans as $plan) {
			$planMinTotal = $total - ($total * ($plan::$min_deposit / 100));
			if ($plan::$min_amount <= $planMinTotal) {
				$has_plan = true;
				break;
			}
		}

		if (!$has_plan) {
			return array();
		}

		$title = language::get('text_checkout_title');
		if ($title_override = config::get('payment_divido_title')) {
			$title = $title_override;
		}

		$method_data = array(
			'code' => 'divido',
			'title' => $title,
			'terms' => '',
			'sort_order' => config::get('payment_divido_sort_order')
		);

		return $method_data;
	}

	public static function getProductSettings($product_id) {
		return db::query("SELECT `display`, `plans` FROM `divido_product` WHERE `product_id` = " . (int)$product_id)->row;
	}

	public static function isEnabled() {
		$api_key = config::get('payment_divido_api_key');
		$enabled = config::get('payment_divido_status');

		return !empty($api_key) && $enabled == 1;
	}

	public static function hashOrderId($order_id, $salt) {
		return hash('sha256', $order_id . $salt);
	}

	public static function saveLookup($order_id, $salt, $proposal_id = null, $application_id = null, $deposit_amount = null) {
		$order_id = (int)$order_id;
		$salt = db::escape($salt);
		$proposal_id = db::escape($proposal_id);
		$application_id = db::escape($application_id);
		$deposit_amount = db::escape($deposit_amount);

		$query_get_lookup = "SELECT `application_id` from `divido_lookup` WHERE order_id = " . $order_id;
		$result_get_lookup = db::query($query_get_lookup);

		if ($result_get_lookup->num_rows == 0) {
			$proposal_id = ($proposal_id) ? "'" . $proposal_id . "'" : 'NULL';
			$application_id = ($application_id) ? "'" . $application_id . "'" : 'NULL';
			$deposit_amount = ($deposit_amount) ? $deposit_amount : 'NULL';

			$query_upsert = "INSERT INTO `divido_lookup` (`order_id`, `salt`, `proposal_id`, `application_id`, `deposit_amount`) VALUES (" . $order_id . ", '" . $salt . "', " . $proposal_id . ", " . $application_id . ", " . $deposit_amount . ")";
		} else {
			$query_upsert = "UPDATE `divido_lookup` SET `salt` = '" . $salt . "'";

			if ($proposal_id) {
				$query_upsert .= ", `proposal_id` = '" . $proposal_id . "'";
			}

			if ($application_id) {
				$query_upsert .= ", `application_id` = '" . $application_id . "'";
			}

			if ($deposit_amount) {
				$query_upsert .= ", `deposit_amount` = " . $deposit_amount;
			}

			$query_upsert .= " WHERE `order_id` = " . $order_id;
		}

		db::query($query_upsert);
	}

	public static function getLookupByOrderId($order_id) {
		return db::query("SELECT * FROM `divido_lookup` WHERE `order_id` = " . $order_id);
	}
	public static function getGlobalSelectedPlans() {
		$all_plans     = self::getAllPlans();
		$display_plans = config::get('payment_divido_planselection');

		if ($display_plans == 'all' || empty($display_plans)) {
			return $all_plans;
		}

		$selected_plans = config::get('payment_divido_plans_selected');
		if (!$selected_plans) {
			return array();
		}

		$plans = array();
		foreach ($all_plans as $plan) {
			if (in_array($plan::$id, $selected_plans)) {
				$plans[] = $plan;
			}
		}

		return $plans;
	}

	public static function getAllPlans() {
		if ($plans = cache::get(self::$CACHE_KEY_PLANS)) {
			// OpenCart 2.1 decodes json objects to associative arrays so we
			// need to make sure we're getting a list of simple objects back.
			$plans = array_map(static function ($plan) {
				return (object)$plan;
			}, $plans);

			return $plans;
		}

		$api_key = config::get('payment_divido_api_key');
		if (!$api_key) {
			throw new Exception("No Divido api-key defined");
		}

		Divido::setMerchant($api_key);

		$response = Divido_Finances::all();
		if ($response::$status != 'ok') {
			throw new Exception("Can't get list of finance plans from Divido!");
		}

		$plans = $response::$finances;

		// OpenCart 2.1 switched to json for their file storage cache, so
		// we need to convert to a simple object.
		$plans_plain = array();
		foreach ($plans as $plan) {
			$plan_copy = new stdClass();
			$plan_copy::$id                 = $plan::$id;
			$plan_copy::$text               = $plan::$text;
			$plan_copy::$country            = $plan::$country;
			$plan_copy::$min_amount         = $plan::$min_amount;
			$plan_copy::$min_deposit        = $plan::$min_deposit;
			$plan_copy::$max_deposit        = $plan::$max_deposit;
			$plan_copy::$interest_rate      = $plan::$interest_rate;
			$plan_copy::$deferral_period    = $plan::$deferral_period;
			$plan_copy::$agreement_duration = $plan::$agreement_duration;

			$plans_plain[] = $plan_copy;
		}

		cache::set(self::$CACHE_KEY_PLANS, $plans_plain);

		return $plans_plain;
	}

	public static function getCartPlans($cart)	{
		$plans = array();
		$products = $cart::getProducts();
		foreach ($products as $product) {
			$product_plans = self::getProductPlans($product['product_id']);
			if ($product_plans) {
				$plans = array_merge($plans, $product_plans);
			}
		}

		return $plans;
	}

	public static function getPlans($default_plans) {
		if ($default_plans) {
			$plans = self::getGlobalSelectedPlans();
		} else {
			$plans = self::getAllPlans();
		}

		return $plans;
	}

	public static function getOrderTotals() {
		$totals = array();
		$taxes = cart::getTaxes();
		$total = 0;

		// Because __call can not keep var references so we put them into an array.
		$total_data = array(
			'totals' => &$totals,
			'taxes'  => &$taxes,
			'total'  => &$total
		);


		$sort_order = array();

		$results = setting_extension::getExtensions('total');

		foreach ($results as $key => $value) {
			$sort_order[$key] = config::get('total_' . $value['code'] . '_sort_order');
		}

		array_multisort($sort_order, SORT_ASC, $results);

		foreach ($results as $result) {
			if (config::get('total_' . $result['code'] . '_status')) {

				// We have to put the totals in an array so that they pass by reference.
				call_model_method('extension_total_' . $result['code'], 'getTotal', $total_data);
			}
		}

		$sort_order = array();

		foreach ($totals as $key => $value) {
			$sort_order[$key] = $value['sort_order'];
		}

		array_multisort($sort_order, SORT_ASC, $totals);

		return array($total, $totals);
	}

	public static function getProductPlans($product_id) {

		$product_info       = catalog_product::getProduct($product_id);
		$settings           = self::getProductSettings($product_id);
		$product_selection  = config::get('payment_divido_productselection');
		$divido_categories  = config::get('payment_divido_categories');
		$price_threshold    = config::get('payment_divido_price_threshold');

		if ($divido_categories) {
			$product_categories = catalog_product::getCategories($product_id);

			$all_categories = array();
			foreach ($product_categories as $product_category) {
				$all_categories[] = $product_category['category_id'];
			}
			$category_matches = array_intersect($all_categories, $divido_categories);

			if (!$category_matches) {
				return null;
			}
		}

		if (empty($settings)) {
			$settings = array(
				'display' => 'default',
				'plans'   => '',
			);
		}

		if ($product_selection == 'selected' && $settings['display'] == 'custom' && empty($settings['plans'])) {
			return null;
		}

		$price = 0;
		if ((config('customer_price') && customer::isLogged()) || !config('customer_price')) {
			$base_price = !empty($product_info['special']) ? $product_info['special'] : $product_info['price'];
			$price = tax::calculate($base_price, $product_info['tax_class_id'], config('tax'));
		}

		if ($product_selection == 'threshold' && !empty($price_threshold) && $price < $price_threshold) {
			return null;
		}

		if ($settings['display'] == 'default') {
			$plans = self::getPlans(true);
			return $plans;
		}

		// If the product has non-default plans, fetch all of them.
		$available_plans = self::getPlans(false);
		$selected_plans  = explode(',', $settings['plans']);

		$plans = array();
		foreach ($available_plans as $plan) {
			if (in_array($plan::$id, $selected_plans)) {
				$plans[] = $plan;
			}
		}

		if (empty($plans)) {
			return null;
		}

		return $plans;
	}

}
