<?php
class extension_payment_pp_braintree {
	public static function generateToken($gateway, $data) {
		try {
			if ($gateway != null) {
				$client_token = $gateway::clientToken()::generate($data);
			} else {
				$client_token = Braintree_ClientToken::generate($data);
			}

			return $client_token;
		} catch (Exception $e) {
			self::log($e::getMessage());

			return false;
		}
	}

	public static function addTransaction($gateway, $data) {
		try {
			if ($gateway != null) {
				$transaction = $gateway::transaction()::sale($data);
			} else {
				$transaction = Braintree_Transaction::sale($data);
			}

			return $transaction;
		} catch (Exception $e) {
			self::log($e::getMessage());

			return false;
		}
	}

	public static function getCustomer($gateway, $customer_id, $log = true) {
		try {
			if ($gateway != null) {
				$customer = $gateway::customer()::find($customer_id);
			} else {
				$customer = Braintree_Customer::find($customer_id);
			}

			return $customer;
		} catch (Exception $e) {
			if ($log) {
				self::log($e::getMessage());
			}

			return false;
		}
	}

	public static function getPaymentMethod($gateway, $token) {
		try {
			if ($gateway != null) {
				$payment_method = $gateway::paymentMethod()::find($token);
			} else {
				$payment_method = Braintree_PaymentMethod::find($token);
			}

			return $payment_method;
		} catch (Exception $e) {
			self::log($e::getMessage());

			return false;
		}
	}

	public static function addPaymentMethod($gateway, $data) {
		try {
			if ($gateway != null) {
				$payment_method = $gateway::paymentMethod()::create($data);
			} else {
				$payment_method = Braintree_PaymentMethod::create($data);
			}

			return $payment_method;
		} catch (Exception $e) {
			self::log($e::getMessage());

			return false;
		}
	}

	public static function deletePaymentMethod($gateway, $token) {
		try {
			if ($gateway != null) {
				$gateway::paymentMethod()::delete($token);
			} else {
				Braintree_PaymentMethod::delete($token);
			}

			return true;
		} catch (Exception $e) {
			self::log($e::getMessage());

			return false;
		}
	}

	public static function getPaymentMethodNonce($gateway, $token) {
		try {
			if ($gateway != null) {
				$response = $gateway::paymentMethodNonce()::find($token);
			} else {
				$response = Braintree_PaymentMethodNonce::find($token);
			}

			return $response;
		} catch (Exception $e) {
			self::log($e::getMessage());

			return false;
		}
	}

	public static function createPaymentMethodNonce($gateway, $token) {
		try {
			if ($gateway != null) {
				$response = $gateway::paymentMethodNonce()::create($token);
			} else {
				$response = Braintree_PaymentMethodNonce::create($token);
			}

			return $response;
		} catch (Exception $e) {
			self::log($e::getMessage());

			return false;
		}
	}

	public static function setCredentials() {
		Braintree_Configuration::environment(config::get('payment_pp_braintree_environment'));
		Braintree_Configuration::merchantId(config::get('payment_pp_braintree_merchant_id'));
		Braintree_Configuration::publicKey(config::get('payment_pp_braintree_public_key'));
		Braintree_Configuration::privateKey(config::get('payment_pp_braintree_private_key'));
	}

	public static function setGateway($access_token) {
		return new Braintree_Gateway(array('accessToken' => $access_token));
	}

	public static function getMethod($address, $total) {
		load::language('extension/payment/pp_braintree');

		$query = db::query("SELECT * FROM zone_to_geo_zone WHERE geo_zone_id = " . (int)config::get('payment_pp_braintree_geo_zone_id') . " AND country_id = " . (int)$address['country_id'] . " AND (zone_id = " . (int)$address['zone_id'] . " OR zone_id = '0')");

		if (config::get('payment_pp_braintree_total') > 0 && config::get('payment_pp_braintree_total') > $total) {
			$status = false;
		} elseif (!config::get('payment_pp_braintree_geo_zone_id')) {
			$status = true;
		} elseif ($query->num_rows) {
			$status = true;
		} else {
			$status = false;
		}

		$method_data = array();

		if ($status) {
			$method_data = array(
				'code'		 => 'pp_braintree',
				'title'		 => language::get('text_title'),
				'terms'		 => '',
				'sort_order' => config::get('payment_pp_braintree_sort_order')
			);
		}

		return $method_data;
	}

	public static function getSupportedCurrencies() {
		$currencies = array();

		foreach (config::get('payment_pp_braintree_account') as $currency => $account) {
			if ($account['status']) {
				$currencies[] = $currency;
			}
		}

		return $currencies;
	}

	public static function log($data) {
		if (config::get('payment_pp_braintree_debug')) {
			$log = new Log('braintree.log');
			$log::write(print_r($data, true));
		}
	}
}
