<?php
class extension_payment_securetrading_ws {
	public static function getMethod($address, $total) {
		load::language('extension/payment/securetrading_ws');

		$query = db::query("SELECT * FROM zone_to_geo_zone WHERE geo_zone_id = " . (int)config::get('payment_securetrading_ws_geo_zone_id') . " AND country_id = " . (int)$address['country_id'] . " AND (zone_id = " . (int)$address['zone_id'] . " OR zone_id = '0')");

		if (config::get('payment_securetrading_ws_total') > $total) {
			$status = false;
		} elseif (!config::get('payment_securetrading_ws_geo_zone_id')) {
			$status = true;
		} elseif ($query->num_rows) {
			$status = true;
		} else {
			$status = false;
		}

		$method_data = array();

		if ($status) {
			$method_data = array(
				'code'       => 'securetrading_ws',
				'title'      => language::get('text_title'),
				'terms'      => '',
				'sort_order' => config::get('payment_securetrading_ws_sort_order')
			);
		}

		return $method_data;
	}

	public static function call($data) {
		$ch = curl_init();

		$defaults = array(
			CURLOPT_POST => 1,
			CURLOPT_HEADER => 0,
			CURLOPT_SSL_VERIFYPEER => 0,
			CURLOPT_URL => 'https://webservices.securetrading.net/xml/',
			CURLOPT_FRESH_CONNECT => 1,
			CURLOPT_RETURNTRANSFER => 1,
			CURLOPT_FORBID_REUSE => 1,
			CURLOPT_TIMEOUT => 15,
			CURLOPT_HTTPHEADER => array(
				'User-Agent: OpenCart - Secure Trading WS',
				'Content-Length: ' . strlen($data),
				'Authorization: Basic ' . base64_encode(config::get('payment_securetrading_ws_username') . ':' . config::get('payment_securetrading_ws_password')),
			),
			CURLOPT_POSTFIELDS => $data,
		);

		curl_setopt_array($ch, $defaults);

		$response = curl_exec($ch);

		if ($response === false) {
			log::write('Secure Trading WS CURL Error: (' . curl_errno($ch) . ') ' . curl_error($ch));
		}

		curl_close($ch);

		return $response;
	}

	public static function format($number, $currency, $value = '', $format = false) {

		$decimal_place = currency::getDecimalPlace($currency);

		if (!$value) {
			$value = currency::getValue($currency);
		}

		$amount = $value ? (float)$number * $value : (float)$number;

		$amount = number_format($amount, (int)$decimal_place);

		if (!$format) {
			return $amount;
		}
	}

	public static function getOrder($order_id) {
		$qry = db::query("SELECT * FROM `securetrading_ws_order` WHERE `order_id` = " . (int)$order_id . " LIMIT 1");

		return $qry->row;
	}

	public static function addMd($order_id, $md) {
		db::query("INSERT INTO securetrading_ws_order SET order_id = " . (int)$order_id . ", md = " . esc($md) . ", `created` = now(), `modified` = now()");
	}

	public static function removeMd($md) {
		db::query("DELETE FROM securetrading_ws_order WHERE md = " . esc($md));
	}

	public static function updateReference($order_id, $transaction_reference) {
		db::query("UPDATE securetrading_ws_order SET transaction_reference = " . esc($transaction_reference) . " WHERE order_id = " . (int)$order_id);

		if (db::countAffected() == 0) {
			db::query("INSERT INTO securetrading_ws_order SET order_id = " . (int)$order_id . ", transaction_reference = " . esc($transaction_reference) . ", `created` = now(), `modified` = now()");
		}
	}

	public static function getOrderId($md) {
		$row = db::query("SELECT order_id FROM securetrading_ws_order WHERE md = " . esc($md) . " LIMIT 1")->row;

		if (isset($row['order_id']) && !empty($row['order_id'])) {
			return $row['order_id'];
		} else {
			return false;
		}
	}

	public static function confirmOrder($order_id, $order_status_id, $comment = '', $notify = false) {

		db::query("UPDATE `order` SET order_status_id = 0 WHERE order_id = " . (int)$order_id);

		checkout_order::addOrderHistory($order_id, $order_status_id, $comment, $notify);

		$order_info = checkout_order::getOrder($order_id);

		$securetrading_ws_order = self::getOrder($order_info['order_id']);

		$amount = currency::format($order_info['total'], $order_info['currency_code'], false, false);

		switch(config::get('payment_securetrading_ws_settle_status')){
			case 0:
				$trans_type = 'auth';
				break;
			case 1:
				$trans_type = 'auth';
				break;
			case 2:
				$trans_type = 'suspended';
				break;
			case 100:
				$trans_type = 'payment';
				break;
			default :
				$trans_type = '';
		}

		db::query("UPDATE `securetrading_ws_order` SET `settle_type` = '" . config::get('payment_securetrading_ws_settle_status') . "', `modified` = now(), `currency_code` = " . esc($order_info['currency_code']) . ", `total` = '" . $amount . "' WHERE order_id = " . (int)$order_info['order_id']);

		db::query("INSERT INTO `securetrading_ws_order_transaction` SET `securetrading_ws_order_id` = " . (int)$securetrading_ws_order['securetrading_ws_order_id'] . ", `amount` = '" . $amount . "', type = '" . $trans_type . "',  `created` = now()");
	}

	public static function updateOrder($order_id, $order_status_id, $comment = '', $notify = false) {

		db::query("UPDATE `order` SET order_status_id = " . (int)$order_status_id . " WHERE order_id = "  . (int)$order_id);

		checkout_order::addOrderHistory($order_id, $order_status_id, $comment, $notify);
	}

	public static function logger($message) {
		$log = new Log('secure.log');
		$log::write($message);
	}
}
