<?php

class extension_payment_worldpay {

	public static function getMethod($address, $total) {
		load::language('extension/payment/worldpay');

		$query = db::query("SELECT * FROM zone_to_geo_zone WHERE geo_zone_id = " . (int)config::get('payment_worldpay_geo_zone_id') . " AND country_id = " . (int)$address['country_id'] . " AND (zone_id = " . (int)$address['zone_id'] . " OR zone_id = '0')");

		if (config::get('payment_worldpay_total') > 0 && config::get('payment_worldpay_total') > $total) {
			$status = false;
		} elseif (!config::get('payment_worldpay_geo_zone_id')) {
			$status = true;
		} elseif ($query->num_rows) {
			$status = true;
		} else {
			$status = false;
		}

		$method_data = array();

		if ($status) {
			$method_data = array(
				'code' => 'worldpay',
				'title' => language::get('text_title'),
				'terms' => '',
				'sort_order' => config::get('payment_worldpay_sort_order')
			);
		}

		return $method_data;
	}

	public static function getCards($customer_id) {

		$query = db::query("SELECT * FROM worldpay_card WHERE customer_id = " . (int)$customer_id);

		$card_data = array();


		foreach ($query->rows as $row) {

			$card_data[] = array(
				'card_id' => $row['card_id'],
				'customer_id' => $row['customer_id'],
				'token' => $row['token'],
				'digits' => $row['digits'],
				'expiry' => $row['expiry'],
				'type' => $row['type'],
			);
		}
		return $card_data;
	}

	public static function addCard($order_id, $card_data) {
		db::query("INSERT into worldpay_card SET customer_id = " . esc($card_data['customer_id']) . ", order_id = " . esc($order_id) . ", digits = " . esc($card_data['Last4Digits']) . ", expiry = " . esc($card_data['ExpiryDate']) . ", type = " . esc($card_data['CardType']) . ", token = " . esc($card_data['Token']));
	}

	public static function deleteCard($token) {
		db::query("DELETE FROM worldpay_card WHERE customer_id = '" . customer::isLogged() . "' AND `token` = " . esc($token));

		if (db::countAffected() > 0) {
			return true;
		} else {
			return false;
		}
	}

	public static function addOrder($order_info, $order_code) {
		db::query("INSERT INTO `worldpay_order` SET `order_id` = " . (int)$order_info['order_id'] . ", `order_code` = " . esc($order_code) . ", `date_added` = now(), `date_modified` = now(), `currency_code` = " . esc($order_info['currency_code']) . ", `total` = '" . currency::format($order_info['total'], $order_info['currency_code'], false, false) . "'");

		return db::getLastId();
	}

	public static function getOrder($order_id) {
		$qry = db::query("SELECT * FROM `worldpay_order` WHERE `order_id` = " . (int)$order_id . " LIMIT 1");

		if ($qry->num_rows) {
			$order = $qry->row;
			$order['transactions'] = self::getTransactions($order['worldpay_order_id']);

			return $order;
		} else {
			return false;
		}
	}

	public static function addTransaction($worldpay_order_id, $type, $order_info) {
		db::query("INSERT INTO `worldpay_order_transaction` SET `worldpay_order_id` = " . (int)$worldpay_order_id . ", `date_added` = now(), `type` = " . esc($type) . ", `amount` = '" . currency::format($order_info['total'], $order_info['currency_code'], false, false) . "'");
	}

	public static function getTransactions($worldpay_order_id) {
		$qry = db::query("SELECT * FROM `worldpay_order_transaction` WHERE `worldpay_order_id` = " . (int)$worldpay_order_id);

		if ($qry->num_rows) {
			return $qry->rows;
		} else {
			return false;
		}
	}

	public static function recurringPayment($item, $order_id_rand, $token) {

		//trial information
		if ($item['recurring']['trial'] == 1) {
			$price = $item['recurring']['trial_price'];
			$trial_amt = currency::format(tax::calculate($item['recurring']['trial_price'], $item['tax_class_id'], config('tax')), session::$data['currency'], false, false) * $item['quantity'] . ' ' . session::$data['currency'];
			$trial_text = sprintf(language::get('text_trial'), $trial_amt, $item['recurring']['trial_cycle'], $item['recurring']['trial_frequency'], $item['recurring']['trial_duration']);
		} else {
			$price = $item['recurring']['price'];
			$trial_text = '';
		}

		$recurring_amt = currency::format(tax::calculate($item['recurring']['price'], $item['tax_class_id'], config('tax')), session::$data['currency'], false, false) * $item['quantity'] . ' ' . session::$data['currency'];
		$recurring_description = $trial_text . sprintf(language::get('text_recurring'), $recurring_amt, $item['recurring']['cycle'], $item['recurring']['frequency']);

		if ($item['recurring']['duration'] > 0) {
			$recurring_description .= sprintf(language::get('text_length'), $item['recurring']['duration']);
		}

		$order_recurring_id = checkout_recurring::addRecurring(session::$data['order_id'], $recurring_description, $item['recurring']);
		
		checkout_recurring::editReference($order_recurring_id, $order_id_rand);

		$order_info = checkout_order::getOrder(session::$data['order_id']);

		$order = array(
			"token" => $token,
			"orderType" => 'RECURRING',
			"amount" => (int)($price * 100),
			"currencyCode" => $order_info['currency_code'],
			"name" => $order_info['firstname'] . ' ' . $order_info['lastname'],
			"orderDescription" => $order_info['store_name'] . ' - ' . date('Y-m-d H:i:s'),
			"customerOrderCode" => 'orderRecurring-' . $order_recurring_id
		);

		extension_payment_worldpay::logger($order);

		$response_data = extension_payment_worldpay::sendCurl('orders', $order);

		extension_payment_worldpay::logger($response_data);

		$next_payment = new DateTime('now');
		$trial_end = new DateTime('now');
		$subscription_end = new DateTime('now');

		if ($item['recurring']['trial'] == 1 && $item['recurring']['trial_duration'] != 0) {
			$next_payment = self::calculateSchedule($item['recurring']['trial_frequency'], $next_payment, $item['recurring']['trial_cycle']);
			$trial_end = self::calculateSchedule($item['recurring']['trial_frequency'], $trial_end, $item['recurring']['trial_cycle'] * $item['recurring']['trial_duration']);
		} elseif ($item['recurring']['trial'] == 1) {
			$next_payment = self::calculateSchedule($item['recurring']['trial_frequency'], $next_payment, $item['recurring']['trial_cycle']);
			$trial_end = new DateTime('0000-00-00');
		}

		if ($trial_end > $subscription_end && $item['recurring']['duration'] != 0) {
			$subscription_end = new DateTime(date_format($trial_end, 'Y-m-d H:i:s'));
			$subscription_end = self::calculateSchedule($item['recurring']['frequency'], $subscription_end, $item['recurring']['cycle'] * $item['recurring']['duration']);
		} elseif ($trial_end == $subscription_end && $item['recurring']['duration'] != 0) {
			$next_payment = self::calculateSchedule($item['recurring']['frequency'], $next_payment, $item['recurring']['cycle']);
			$subscription_end = self::calculateSchedule($item['recurring']['frequency'], $subscription_end, $item['recurring']['cycle'] * $item['recurring']['duration']);
		} elseif ($trial_end > $subscription_end && $item['recurring']['duration'] == 0) {
			$subscription_end = new DateTime('0000-00-00');
		} elseif ($trial_end == $subscription_end && $item['recurring']['duration'] == 0) {
			$next_payment = self::calculateSchedule($item['recurring']['frequency'], $next_payment, $item['recurring']['cycle']);
			$subscription_end = new DateTime('0000-00-00');
		}

		if (isset($response_data::$paymentStatus) && $response_data::$paymentStatus == 'SUCCESS') {
			self::addRecurringOrder($order_info, $response_data::$orderCode, $token, $price, $order_recurring_id, date_format($trial_end, 'Y-m-d H:i:s'), date_format($subscription_end, 'Y-m-d H:i:s'));

			self::updateRecurringOrder($order_recurring_id, date_format($next_payment, 'Y-m-d H:i:s'));

			self::addProfileTransaction($order_recurring_id, $response_data::$orderCode, $price, 1);
		} else {
			self::addProfileTransaction($order_recurring_id, '', $price, 4);
		}
	}

	public static function cronPayment() {

		$profiles = self::getProfiles();
		$cron_data = array();
		$i = 1;
		foreach ($profiles as $profile) {
			$recurring_order = self::getRecurringOrder($profile['order_recurring_id']);

			$today = new DateTime('now');
			$unlimited = new DateTime('0000-00-00');
			$next_payment = new DateTime($recurring_order['next_payment']);
			$trial_end = new DateTime($recurring_order['trial_end']);
			$subscription_end = new DateTime($recurring_order['subscription_end']);

			$order_info = checkout_order::getOrder($profile['order_id']);

			if (($today > $next_payment) && ($trial_end > $today || $trial_end == $unlimited)) {
				$price = currency::format($profile['trial_price'], $order_info['currency_code'], false, false);
				$frequency = $profile['trial_frequency'];
				$cycle = $profile['trial_cycle'];
			} elseif (($today > $next_payment) && ($subscription_end > $today || $subscription_end == $unlimited)) {
				$price = currency::format($profile['recurring_price'], $order_info['currency_code'], false, false);
				$frequency = $profile['recurring_frequency'];
				$cycle = $profile['recurring_cycle'];
			} else {
				continue;
			}

			$order = array(
				"token" => $recurring_order['token'],
				"orderType" => 'RECURRING',
				"amount" => (int)($price * 100),
				"currencyCode" => $order_info['currency_code'],
				"name" => $order_info['firstname'] . ' ' . $order_info['lastname'],
				"orderDescription" => $order_info['store_name'] . ' - ' . date('Y-m-d H:i:s'),
				"customerOrderCode" => 'orderRecurring-' . $profile['order_recurring_id'] . '-repeat-' . $i++
			);

			extension_payment_worldpay::logger($order);

			$response_data = extension_payment_worldpay::sendCurl('orders', $order);

			extension_payment_worldpay::logger($response_data);

			$cron_data[] = $response_data;

			if (isset($response_data::$paymentStatus) && $response_data::$paymentStatus == 'SUCCESS') {
				self::addProfileTransaction($profile['order_recurring_id'], $response_data::$orderCode, $price, 1);
				$next_payment = self::calculateSchedule($frequency, $next_payment, $cycle);
				$next_payment = date_format($next_payment, 'Y-m-d H:i:s');
				self::updateRecurringOrder($profile['order_recurring_id'], $next_payment);
			} else {
				self::addProfileTransaction($profile['order_recurring_id'], '', $price, 4);
			}
		}
		$log = new Log('worldpay_recurring_orders.log');
		$log::write(print_r($cron_data, 1));
		return $cron_data;
	}

	private static function calculateSchedule($frequency, $next_payment, $cycle) {
		if ($frequency == 'semi_month') {
			$day = date_format($next_payment, 'd');
			$value = 15 - $day;
			$isEven = false;
			if ($cycle % 2 == 0) {
				$isEven = true;
			}

			$odd = ($cycle + 1) / 2;
			$plus_even = ($cycle / 2) + 1;
			$minus_even = $cycle / 2;

			if ($day == 1) {
				$odd = $odd - 1;
				$plus_even = $plus_even - 1;
				$day = 16;
			}

			if ($day <= 15 && $isEven) {
				$next_payment::modify('+' . $value . ' day');
				$next_payment::modify('+' . $minus_even . ' month');
			} elseif ($day <= 15) {
				$next_payment::modify('first day of this month');
				$next_payment::modify('+' . $odd . ' month');
			} elseif ($day > 15 && $isEven) {
				$next_payment::modify('first day of this month');
				$next_payment::modify('+' . $plus_even . ' month');
			} elseif ($day > 15) {
				$next_payment::modify('+' . $value . ' day');
				$next_payment::modify('+' . $odd . ' month');
			}
		} else {
			$next_payment::modify('+' . $cycle . ' ' . $frequency);
		}
		return $next_payment;
	}

	private static function addRecurringOrder($order_info, $order_code, $token, $price, $order_recurring_id, $trial_end, $subscription_end) {
		db::query("INSERT INTO `worldpay_order_recurring` SET `order_id` = " . (int)$order_info['order_id'] . ", `order_recurring_id` = " . (int)$order_recurring_id . ", `order_code` = " . esc($order_code) . ", `token` = " . esc($token) . ", `date_added` = now(), `date_modified` = now(), `next_payment` = now(), `trial_end` = '" . $trial_end . "', `subscription_end` = '" . $subscription_end . "', `currency_code` = " . esc($order_info['currency_code']) . ", `total` = '" . currency::format($price, $order_info['currency_code'], false, false) . "'");
	}

	private static function updateRecurringOrder($order_recurring_id, $next_payment) {
		db::query("UPDATE `worldpay_order_recurring` SET `next_payment` = '" . $next_payment . "', `date_modified` = now() WHERE `order_recurring_id` = " . (int)$order_recurring_id);
	}

	private static function getRecurringOrder($order_recurring_id) {
		$qry = db::query("SELECT * FROM worldpay_order_recurring WHERE order_recurring_id = " . (int)$order_recurring_id);
		return $qry->row;
	}

	private static function addProfileTransaction($order_recurring_id, $order_code, $price, $type) {
		db::query("INSERT INTO `order_recurring_transaction` SET `order_recurring_id` = " . (int)$order_recurring_id . ", `date_added` = NOW(), `amount` = '" . (float)$price . "', `type` = " . (int)$type . ", `reference` = " . esc($order_code));
	}

	private static function getProfiles() {
		$sql = "
			SELECT `or`.order_recurring_id
			FROM `order_recurring` `or`
			JOIN `order` `o` USING(`order_id`)
			WHERE o.payment_code = 'worldpay'";

		$qry = db::query($sql);

		$order_recurring = array();

		foreach ($qry->rows as $profile) {
			$order_recurring[] = self::getProfile($profile['order_recurring_id']);
		}
		return $order_recurring;
	}

	private static function getProfile($order_recurring_id) {
		$qry = db::query("SELECT * FROM order_recurring WHERE order_recurring_id = " . (int)$order_recurring_id);
		return $qry->row;
	}

	public static function getWorldpayOrder($worldpay_order_id) {
		$qry = db::query("SELECT * FROM worldpay_order WHERE order_code = " . (int)$worldpay_order_id);
		return $qry->row;
	}

	public static function updateCronJobRunTime() {
		db::query("DELETE FROM `setting` WHERE `code` = 'payment_worldpay' AND `key` = 'payment_worldpay_last_cron_job_run'");
		db::query("INSERT INTO `setting` (`store_id`, `code`, `key`, `value`, `serialized`) VALUES (0, 'payment_worldpay', 'payment_worldpay_last_cron_job_run', NOW(), 0)");
	}

	public static function sendCurl($url, $order = null) {
		$curl = curl_init();
		curl_setopt($curl, CURLOPT_URL, 'https://api.worldpay.com/v1/' . $url);
		$content_length = 0;
		if ($order) {
			$json = json_encode($order);
			$content_length = strlen($json);
			curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
			curl_setopt($curl, CURLOPT_POSTFIELDS, $json);
		}
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 0);
		curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
		curl_setopt($curl, CURLOPT_TIMEOUT, 10);
		curl_setopt(
				$curl, CURLOPT_HTTPHEADER, array(
			"Authorization: " . config::get('payment_worldpay_service_key'),
			"Content-Type: application/json",
			"Content-Length: " . $content_length
				)
		);

		$result = json_decode(curl_exec($curl));
		curl_close($curl);
		return $result;
	}

	public static function logger($data) {
		if (config::get('payment_worldpay_debug')) {
			$log = new Log('worldpay_debug.log');
			$backtrace = debug_backtrace();
			$log::write($backtrace[6]['class'] . '::$' . $backtrace[6]['function'] . ' Data:  ' . print_r($data, 1));
		}
	}

	public static function recurringPayments() {
		/*
		 * Used by the checkout to state the module
		 * supports recurring profiles.
		 */
		return true;
	}

}
