<?php
/**
 * @package		OpenCart
 * @author		Daniel Kerr
 * @copyright	Copyright (c) 2005 - 2017, OpenCart, Ltd. (https://www.opencart.com/)
 * @license		https://opensource.org/licenses/GPL-3.0
 * @link		https://www.opencart.com
*/

/**
* Loader class
*/
final class Load {

	/**
	 * Constructor
	 *
 	*/
	static function init() {
	}

	/**
	 * 
	 *
	 * @param	string	$route
	 * @param	array	$data
	 *
	 * @return	mixed
 	*/	
	static function controller($route, $data = array()) {
		// Sanitize the call
		$route = preg_replace('/[^a-zA-Z0-9_\/]/', '', (string)$route);
		
		$action = $route;
		
		$backup = language::$data;

		try {
			$output = action::execute($action, array(&$data));
		} catch (Exception $e) {
			language::setAll($backup);
			return false;
		}
		
		language::setAll($backup);
		
		return $output;
	}

	/**
	 * 
	 *
	 * @param	string	$route
 	*/	
	static function model($route) {
		// Sanitize the call
		$route = preg_replace('/[^a-zA-Z0-9_\/]/', '', (string)$route);

		$name = str_replace('/', '_', $route);
		if (OPENCART_ADMIN) $name = 'admin_' . $name;
		
		if (!class_exists($name)) {
			$class = 'Model' . preg_replace('/[^a-zA-Z0-9]/', '', $route);
			if (class_exists($class)) {
				global $registry;
				$registry->set('model_' . str_replace('/', '_', $route), new $class);
			}
		}
	}

	/**
	 * 
	 *
	 * @param	string	$route
	 * @param	array	$data
	 *
	 * @return	string
 	*/
	static function view($route, $data = array()) {
		// Sanitize the call
		$route = preg_replace('/[^a-zA-Z0-9_\/]/', '', (string)$route);
		
		$template = new Template(config::get('template_engine'));

		if (!OPENCART_ADMIN) {
			if (!config::get('theme_' . config('theme') . '_status')) {
				exit('Error: A theme has not been assigned to this store!');
			}

			// If the default theme is selected we need to know which directory its pointing to
			if (config('theme') == 'default') {
				$directory = config::get('theme_default_directory');
			} else {
				$directory = config('theme');
			}

			if ($directory != 'default' && is_file(DIR_TEMPLATE . $directory . '/template/' . $route . '.twig')) {
				config::set('template_directory', $directory . '/template/');
			} elseif (is_file(DIR_TEMPLATE . 'default/template/' . $route . '.twig')) {
				config::set('template_directory', 'default/template/');
			}

			// If there is a theme override we should get it

			$theme_info = design_theme::getTheme($route, $directory);

			if ($theme_info) {
				$code = html_entity_decode($theme_info['code'], ENT_QUOTES, 'UTF-8');

				return $code;
			}
		}

		$template::$data = array_merge(language::$data, $data);	

		$output = $template::render(config::get('template_directory') . $route);
		
		return $output;
	}

	/**
	 * 
	 *
	 * @param	string	$route
 	*/
	static function library($route) {
		// Sanitize the call
		$route = preg_replace('/[^a-zA-Z0-9_\/]/', '', (string)$route);
			
		$file = 'system/library/' . $route . '.php';
		$class = str_replace('/', '\\', $route);

		if (is_file($file)) {
			include_once($file);

		} else {
			throw new \Exception('Error: Could not load library ' . $route . '!');
		}
	}

	/**
	 * 
	 *
	 * @param	string	$route
 	*/	
	static function helper($route) {
		$file = 'system/helper/' . preg_replace('/[^a-zA-Z0-9_\/]/', '', (string)$route) . '.php';

		if (is_file($file)) {
			include_once($file);
		} else {
			throw new \Exception('Error: Could not load helper ' . $route . '!');
		}
	}

	/**
	 * 
	 *
	 * @param	string	$route
 	*/	
	static function config($route) {
		event::trigger('config/' . $route . '/before', array(&$route));
		
		config::load($route);
		
		event::trigger('config/' . $route . '/after', array(&$route));
	}

	/**
	 * 
	 *
	 * @param	string	$route
	 * @param	string	$key
	 *
	 * @return	array
 	*/
	static function language($route, $key = '') {
		// Sanitize the call
		$route = preg_replace('/[^a-zA-Z0-9_\/]/', '', (string)$route);
		
		$output = language::load($route, $key);

		if (db::isConnected()) {
			$results = design_translation::getTranslations($route);

			foreach ($results as $result) {
				if (!$key) {
					language::set($result['key'], html_entity_decode($result['value'], ENT_QUOTES, 'UTF-8'));
				} else {
					language::get($key)::set($result['key'], html_entity_decode($result['value'], ENT_QUOTES, 'UTF-8'));
				}
			}
		}
		
		return $output;
	}
}