<?php
/**
 * @package		OpenCart
 * @author		Daniel Kerr
 * @copyright	Copyright (c) 2005 - 2017, OpenCart, Ltd. (https://www.opencart.com/)
 * @license		https://opensource.org/licenses/GPL-3.0
 * @link		https://www.opencart.com
 */

/**
 * Registry class
 */
final class Registry {
	private $data = array();

	/**
	 *
	 *
	 * @param	string	$key
	 *
	 * @return	mixed
	 */
	public function get($class) {
		if (!empty($this->data[$class])) return $this->data[$class];

		$real_class = $class;

		if (str_starts_with($real_class, 'model_')) {
			$real_class = substr($real_class, 6);

			if (OPENCART_ADMIN) $real_class = 'admin_' . $real_class;
		}

		if (!class_exists($real_class)) {
			$old_class = str_replace('_', '', $class);

			if (class_exists($old_class)) {
				$this->data[$class] = new $old_class();

				return $this->data[$class];
			}
			die("Class $class not found!");
		}

		$wrapper = new StaticWrapper($real_class);

		$this->data[$class] = $wrapper;

		return $this->data[$class];

	}

	/**
	 *
	 *
	 * @param	string	$key
	 * @param	string	$value
	 */
	public function set($key, $value) {
		$this->data[$key] = $value;
	}

	/**
	 *
	 *
	 * @param	string	$key
	 *
	 * @return	bool
	 */
	public function has($key) {
		return !empty($this->get($key));
	}
}

class StaticWrapper extends \stdClass {

	private string $class;

	public function __construct($class) {
		$this->class = $class;
	}

	public function &__get($key) {
		return $this->class::$$key;
	}

	public function __set($key, $value) {
		return $this->class::$$key = $value;
	}

	public function __call($func, $args) {
		return call_user_func_array("$this->class::$func", $args);
	}
}

function call_model_method($model, $method, ...$args) {
	if (function_exists("$model::$method"))
		return $model::$method(...$args);

	global $registry;
	return $registry->get("model_$model")->$method(...$args);
}

function model_method_exists($model, $method) {
	if (function_exists("$model::$method"))
		return true;

	global $registry;
	return property_exists($registry->get("model_$model"), $method);
}