<?php

global $registry;
$registry = new Registry();

// Config
config::load('default');
config::load($application_config);

request::init();

// Log
log::init(config::get('error_filename'));

date_default_timezone_set(config::get('date_timezone'));

set_error_handler(function($code, $message, $file, $line) {
	// error suppressed with @
	if (!(error_reporting() & $code)) {
		return false;
	}

	switch ($code) {
		case E_NOTICE:
		case E_USER_NOTICE:
			$error = 'Notice';
			break;
		case E_WARNING:
		case E_USER_WARNING:
			$error = 'Warning';
			break;
		case E_ERROR:
		case E_USER_ERROR:
			$error = 'Fatal Error';
			break;
		default:
			$error = 'Unknown';
			break;
	}

	$file = str_replace(substr(DIR_SYSTEM, 0, -7), '', $file);

	if (config::get('error_display')) {
		echo "<b>$error</b>: $message in <a href='".SITE_URL."source.php?file=$file:$line' target='_blank'>$file</a> on line <b>$line</b><br>";
	}

	if (config::get('error_log')) {
		log::write('PHP ' . $error . ':  ' . $message . ' in ' . $file . ' on line ' . $line);
	}

	return true;
});

response::addHeader('Content-Type: text/html; charset=utf-8');
response::setCompression(config('compression'));

if (config::get('db_autostart')) {
	db::init('mysqli', DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);
	db::query("SET time_zone = " . esc(date('P')));
}

// Session
session::init(config::get('session_engine'));

if (config::get('session_autostart')) {
	/*
	We are adding the session cookie outside of the session class as I believe
	PHP messed up in a big way handling sessions. Why in the hell is it so hard to
	have more than one concurrent session using cookies!

	Is it not better to have multiple cookies when accessing parts of the system
	that requires different cookie sessions for security reasons.

	Also cookies can be accessed via the URL parameters. So why force only one cookie
	for all sessions!
	*/

	if (isset($_COOKIE[config::get('session_name')])) {
		$session_id = $_COOKIE[config::get('session_name')];
	} else {
		$session_id = '';
	}

	session::start($session_id);

	setcookie(config::get('session_name'), session::getId(), ini_get('session.cookie_lifetime'), ini_get('session.cookie_path'), ini_get('session.cookie_domain'));
}

// Cache
cache::init(config::get('cache_engine'), config::get('cache_expire'));

// Url
if (config::get('url_autostart')) {
	url::init(config::get('site_url'), config::get('site_ssl'));
}

// Language
language::init(config::get('language_directory'));

// Config Autoload
if (config::has('config_autoload')) {
	foreach (config::get('config_autoload') as $value) {
		load::config($value);
	}
}

// Language Autoload
if (config::has('language_autoload')) {
	foreach (config::get('language_autoload') as $value) {
		load::language($value);
	}
}

// Library Autoload
if (config::has('library_autoload')) {
	foreach (config::get('library_autoload') as $value) {
		load::library($value);
	}
}

// Model Autoload
if (config::has('model_autoload')) {
	foreach (config::get('model_autoload') as $value) {
	}
}

// Pre Actions
if (config::has('action_pre_action')) {
	foreach (config::get('action_pre_action') as $value) {
		route::addPreAction($value);
	}
}

// Dispatch
route::dispatch(config::get('action_router'), config::get('action_error'));

// Output
response::output();