<?php
class Currency {
	private static $language;
	private static $currencies = array();

	public static function init() {

		$query = db::query("SELECT * FROM currency");

		foreach ($query->rows as $result) {
			self::$currencies[$result['code']] = array(
				'currency_id'   => $result['currency_id'],
				'title'         => $result['title'],
				'symbol_left'   => $result['symbol_left'],
				'symbol_right'  => $result['symbol_right'],
				'decimal_place' => $result['decimal_place'],
				'value'         => $result['value']
			);
		}
	}

	public static function format($number, $currency, $value = '', $format = true) {
		$symbol_left = self::$currencies[$currency]['symbol_left'];
		$symbol_right = self::$currencies[$currency]['symbol_right'];
		$decimal_place = self::$currencies[$currency]['decimal_place'];

		if (!$value) {
			$value = self::$currencies[$currency]['value'];
		}

		$amount = $value ? (float)$number * $value : (float)$number;
		
		$amount = round($amount, (int)$decimal_place);
		
		if (!$format) {
			return $amount;
		}

		$string = '';

		if ($symbol_left) {
			$string .= $symbol_left;
		}

		$string .= number_format($amount, (int)$decimal_place, language::get('decimal_point'), language::get('thousand_point'));

		if ($symbol_right) {
			$string .= $symbol_right;
		}

		return $string;
	}

	public static function convert($value, $from, $to) {
		if (isset(self::$currencies[$from])) {
			$from = self::$currencies[$from]['value'];
		} else {
			$from = 1;
		}

		if (isset(self::$currencies[$to])) {
			$to = self::$currencies[$to]['value'];
		} else {
			$to = 1;
		}

		return $value * ($to / $from);
	}
	
	public static function getId($currency) {
		if (isset(self::$currencies[$currency])) {
			return self::$currencies[$currency]['currency_id'];
		} else {
			return 0;
		}
	}

	public static function getSymbolLeft($currency) {
		if (isset(self::$currencies[$currency])) {
			return self::$currencies[$currency]['symbol_left'];
		} else {
			return '';
		}
	}

	public static function getSymbolRight($currency) {
		if (isset(self::$currencies[$currency])) {
			return self::$currencies[$currency]['symbol_right'];
		} else {
			return '';
		}
	}

	public static function getDecimalPlace($currency) {
		if (isset(self::$currencies[$currency])) {
			return self::$currencies[$currency]['decimal_place'];
		} else {
			return 0;
		}
	}

	public static function getValue($currency) {
		if (isset(self::$currencies[$currency])) {
			return self::$currencies[$currency]['value'];
		} else {
			return 0;
		}
	}

	public static function has($currency) {
		return isset(self::$currencies[$currency]);
	}
}
