<?php
/**
 * @package		OpenCart
 * @author		Daniel Kerr
 * @copyright	Copyright (c) 2005 - 2017, OpenCart, Ltd. (https://www.opencart.com/)
 * @license		https://opensource.org/licenses/GPL-3.0
 * @link		https://www.opencart.com
*/

/**
* DB class
*/

class DB {
	public static $connection;

	// Engine left here for compatability with Opencart 3 extensions, it is not used, engine is always MySQLi )
	public static function init($engine, $hostname, $username, $password, $database, $port = '3306') {
		try {
			$mysqli = @new \MySQLi($hostname, $username, $password, $database, $port);
		} catch (mysqli_sql_exception $e) {
			throw new \Exception('Error: Could not make a database link using ' . $username . '@' . $hostname . '!');
		}

		if (!$mysqli->connect_errno) {
			mysqli_report(MYSQLI_REPORT_ERROR);
			db::$connection = $mysqli;
//			db::$connection->report_mode = MYSQLI_REPORT_ERROR;
			db::$connection->set_charset('utf8');
			db::$connection->query("SET SESSION sql_mode = 'NO_ZERO_IN_DATE,NO_ENGINE_SUBSTITUTION'");
		} else {
			throw new \Exception('Error: Could not make a database link using ' . $username . '@' . $hostname . '!');
		}
		//register_shutdown_function("db::finish");
	}

	/**
	 * @throws Exception
	 */
	public static function query($sql) {
		$query = db::$connection->query($sql);

		if (!db::$connection->errno) {
			if ($query instanceof \mysqli_result) {

				$data = $query->fetch_all(MYSQLI_ASSOC);

				$result = new \stdClass();
				$result->num_rows = $query->num_rows;
				$result->row = isset($data[0]) ? $data[0] : [];
				$result->rows = $data;

				$query->close();

				unset($data);

				return $result;
			} else {
				return true;
			}
		} else {
			throw new \Exception('Error: ' . db::$connection->error  . '<br />Error No: ' . db::$connection->errno . '<br />' . $sql);
		}
	}

	public static function escape($value) {
		return db::$connection->real_escape_string(($value===null) ? '' : $value);
	}

	public static function countAffected() {
		return db::$connection->affected_rows;
	}

	public static function getLastId() {
		return db::$connection->insert_id;
	}

	public static function isConnected() {
		if (db::$connection) {
			return db::$connection->ping();
		} else {
			return false;
		}
	}

	/**
	 * __destruct
	 *
	 * Closes the DB connection when this object is destroyed.
	 *
	 */
	public static function finish() {
		if (db::$connection) {
			db::$connection->close();

			db::$connection = '';
		}
	}
}

function esc(?string $text):string {
	return "'".db::escape($text)."'";
}