<?php
/**
 * @package		OpenCart
 * @author		Daniel Kerr
 * @copyright	Copyright (c) 2005 - 2017, OpenCart, Ltd. (https://www.opencart.com/)
 * @license		https://opensource.org/licenses/GPL-3.0
 * @link		https://www.opencart.com
*/

/**
* Mail class
*/
class Mail extends \stdClass {
	protected static $to;
	protected static $from;
	protected static $sender;
	protected static $reply_to;
	protected static $subject;
	protected static $text;
	protected static $html;
	protected static $attachments = array();
	public static $parameter;

	/**
	 * Constructor
	 *
	 * @param	string	$adaptor
	 *
 	*/
	public static function init($adaptor = 'mail') {
		$class = 'Mail\\' . $adaptor;
		
		if (class_exists($class)) {
			self::$adaptor = new $class();
		} else {
			trigger_error('Error: Could not load mail adaptor ' . $adaptor . '!');
			exit();
		}	
	}
	
	/**
     * 
     *
     * @param	mixed	$to
     */
	public static function setTo($to) {
		self::$to = $to;
	}
	
	/**
     * 
     *
     * @param	string	$from
     */
	public static function setFrom($from) {
		self::$from = $from;
	}
	
	/**
     * 
     *
     * @param	string	$sender
     */
	public static function setSender($sender) {
		self::$sender = $sender;
	}
	
	/**
     * 
     *
     * @param	string	$reply_to
     */
	public static function setReplyTo($reply_to) {
		self::$reply_to = $reply_to;
	}
	
	/**
     * 
     *
     * @param	string	$subject
     */
	public static function setSubject($subject) {
		self::$subject = $subject;
	}
	
	/**
     * 
     *
     * @param	string	$text
     */
	public static function setText($text) {
		self::$text = $text;
	}
	
	/**
     * 
     *
     * @param	string	$html
     */
	public static function setHtml($html) {
		self::$html = $html;
	}
	
	/**
     * 
     *
     * @param	string	$filename
     */
	public static function addAttachment($filename) {
		self::$attachments[] = $filename;
	}
	
	/**
     * 
     *
     */
	public static function send() {
		if (!self::$to) {
			throw new \Exception('Error: E-Mail to required!');
		}

		if (!self::$from) {
			throw new \Exception('Error: E-Mail from required!');
		}

		if (!self::$sender) {
			throw new \Exception('Error: E-Mail sender required!');
		}

		if (!self::$subject) {
			throw new \Exception('Error: E-Mail subject required!');
		}

		if ((!self::$text) && (!self::$html)) {
			throw new \Exception('Error: E-Mail message required!');
		}
		
		foreach (get_object_vars($this) as $key => $value) {
			self::$adaptor->$$key = $value;
		}
		
		self::$adaptor->send();
	}
}