<?php
namespace Mail;
class Smtp extends \stdClass {
	public static $smtp_hostname;
	public static $smtp_username;
	public static $smtp_password;
	public static $smtp_port = 25;
	public static $smtp_timeout = 5;
	public static $max_attempts = 3;
	public static $verp = false;

	public static function send() {
		if (is_array(self::$to)) {
			$to = implode(',', self::$to);
		} else {
			$to = self::$to;
		}

		$boundary = '----=_NextPart_' . md5(time());

		$header = 'MIME-Version: 1.0' . PHP_EOL;
		$header .= 'To: <' . $to . '>' . PHP_EOL;
		$header .= 'Subject: =?UTF-8?B?' . base64_encode(self::$subject) . '?=' . PHP_EOL;
		$header .= 'Date: ' . date('D, d M Y H:i:s O') . PHP_EOL;
		$header .= 'From: =?UTF-8?B?' . base64_encode(self::$sender) . '?= <' . self::$from . '>' . PHP_EOL;

		if (!self::$reply_to) {
			$header .= 'Reply-To: =?UTF-8?B?' . base64_encode(self::$sender) . '?= <' . self::$from . '>' . PHP_EOL;
		} else {
			$header .= 'Reply-To: =?UTF-8?B?' . base64_encode(self::$reply_to) . '?= <' . self::$reply_to . '>' . PHP_EOL;
		}

		$header .= 'Return-Path: ' . self::$from . PHP_EOL;
		$header .= 'X-Mailer: PHP/' . phpversion() . PHP_EOL;
		$header .= 'Content-Type: multipart/mixed; boundary="' . $boundary . '"' . PHP_EOL . PHP_EOL;

		if (!self::$html) {
			$message = '--' . $boundary . PHP_EOL;
			$message .= 'Content-Type: text/plain; charset="utf-8"' . PHP_EOL;
			$message .= 'Content-Transfer-Encoding: base64' . PHP_EOL . PHP_EOL;
			$message .= chunk_split(base64_encode(self::$text)) . PHP_EOL;
		} else {
			$message = '--' . $boundary . PHP_EOL;
			$message .= 'Content-Type: multipart/alternative; boundary="' . $boundary . '_alt"' . PHP_EOL . PHP_EOL;
			$message .= '--' . $boundary . '_alt' . PHP_EOL;
			$message .= 'Content-Type: text/plain; charset="utf-8"' . PHP_EOL;
			$message .= 'Content-Transfer-Encoding: base64' . PHP_EOL . PHP_EOL;

			if (self::$text) {
				$message .= chunk_split(base64_encode(self::$text)) . PHP_EOL;
			} else {
				$message .= chunk_split(base64_encode('This is a HTML email and your email client software does not support HTML email!')) . PHP_EOL;
			}

			$message .= '--' . $boundary . '_alt' . PHP_EOL;
			$message .= 'Content-Type: text/html; charset="utf-8"' . PHP_EOL;
			$message .= 'Content-Transfer-Encoding: base64' . PHP_EOL . PHP_EOL;
			$message .= chunk_split(base64_encode(self::$html)) . PHP_EOL;
			$message .= '--' . $boundary . '_alt--' . PHP_EOL;
		}

		foreach (self::$attachments as $attachment) {
			if (file_exists($attachment)) {
				$handle = fopen($attachment, 'r');

				$content = fread($handle, filesize($attachment));

				fclose($handle);

				$message .= '--' . $boundary . PHP_EOL;
				$message .= 'Content-Type: application/octet-stream; name="' . basename($attachment) . '"' . PHP_EOL;
				$message .= 'Content-Transfer-Encoding: base64' . PHP_EOL;
				$message .= 'Content-Disposition: attachment; filename="' . basename($attachment) . '"' . PHP_EOL;
				$message .= 'Content-ID: <' . urlencode(basename($attachment)) . '>' . PHP_EOL;
				$message .= 'X-Attachment-Id: ' . urlencode(basename($attachment)) . PHP_EOL . PHP_EOL;
				$message .= chunk_split(base64_encode($content));
			}
		}

		$message .= '--' . $boundary . '--' . PHP_EOL;

		if (substr(self::$smtp_hostname, 0, 3) == 'tls') {
			$hostname = substr(self::$smtp_hostname, 6);
		} else {
			$hostname = self::$smtp_hostname;
		}

		$handle = fsockopen($hostname, self::$smtp_port, $errno, $errstr, self::$smtp_timeout);

		if (!$handle) {
			throw new \Exception('Error: ' . $errstr . ' (' . $errno . ')');
		} else {
			if (substr(PHP_OS, 0, 3) != 'WIN') {
				socket_set_timeout($handle, self::$smtp_timeout, 0);
			}

			while ($line = fgets($handle, 515)) {
				if (substr($line, 3, 1) == ' ') {
					break;
				}
			}

			fputs($handle, 'EHLO ' . getenv('SERVER_NAME') . "\r\n");

			$reply = '';

			while ($line = fgets($handle, 515)) {
				$reply .= $line;

				//some SMTP servers respond with 220 code before responding with 250. hence, we need to ignore 220 response string
				if (substr($reply, 0, 3) == 220 && substr($line, 3, 1) == ' ') {
					$reply = '';

					continue;
				} else if (substr($line, 3, 1) == ' ') {
					break;
				}
			}

			if (substr($reply, 0, 3) != 250) {
				throw new \Exception('Error: EHLO not accepted from server!');
			}

			if (substr(self::$smtp_hostname, 0, 3) == 'tls') {
				fputs($handle, 'STARTTLS' . "\r\n");

				self::handleReply($handle, 220, 'Error: STARTTLS not accepted from server!');

				if (stream_socket_enable_crypto($handle, true, STREAM_CRYPTO_METHOD_TLS_CLIENT) !== true) {
					throw new \Exception('Error: TLS could not be established!');
				}

				fputs($handle, 'EHLO ' . getenv('SERVER_NAME') . "\r\n");

				self::handleReply($handle, 250, 'Error: EHLO not accepted from server!');
			}

			if (!empty(self::$smtp_username) && !empty(self::$smtp_password)) {
				fputs($handle, 'AUTH LOGIN' . "\r\n");

				self::handleReply($handle, 334, 'Error: AUTH LOGIN not accepted from server!');

				fputs($handle, base64_encode(self::$smtp_username) . "\r\n");

				self::handleReply($handle, 334, 'Error: Username not accepted from server!');

				fputs($handle, base64_encode(self::$smtp_password) . "\r\n");

				self::handleReply($handle, 235, 'Error: Password not accepted from server!');

			} else {
				fputs($handle, 'HELO ' . getenv('SERVER_NAME') . "\r\n");

				self::handleReply($handle, 250, 'Error: HELO not accepted from server!');
			}

			if (self::$verp) {
				fputs($handle, 'MAIL FROM: <' . self::$from . '>XVERP' . "\r\n");
			} else {
				fputs($handle, 'MAIL FROM: <' . self::$from . '>' . "\r\n");
			}

			self::handleReply($handle, 250, 'Error: MAIL FROM not accepted from server!');

			if (!is_array(self::$to)) {
				fputs($handle, 'RCPT TO: <' . self::$to . '>' . "\r\n");

				$reply = self::handleReply($handle, false, 'RCPT TO [!array]');

				if ((substr($reply, 0, 3) != 250) && (substr($reply, 0, 3) != 251)) {
					throw new \Exception('Error: RCPT TO not accepted from server!');
				}
			} else {
				foreach (self::$to as $recipient) {
					fputs($handle, 'RCPT TO: <' . $recipient . '>' . "\r\n");

					$reply = self::handleReply($handle, false, 'RCPT TO [array]');

					if ((substr($reply, 0, 3) != 250) && (substr($reply, 0, 3) != 251)) {
						throw new \Exception('Error: RCPT TO not accepted from server!');
					}
				}
			}

			fputs($handle, 'DATA' . "\r\n");

			self::handleReply($handle, 354, 'Error: DATA not accepted from server!');

			// According to rfc 821 we should not send more than 1000 including the CRLF
			$message = str_replace("\r\n", "\n", $header . $message);
			$message = str_replace("\r", "\n", $message);

			$lines = explode("\n", $message);

			foreach ($lines as $line) {
				// $results = str_split($line, $length);
				// see https://php.watch/versions/8.2/str_split-empty-string-empty-array
				$results = ($line === '') ? [''] : str_split($line, 998);

				foreach ($results as $result) {
					fputs($handle, $result . "\r\n");
				}
			}

			fputs($handle, '.' . "\r\n");

			self::handleReply($handle, 250, 'Error: DATA not accepted from server!');

			fputs($handle, 'QUIT' . "\r\n");

			self::handleReply($handle, 221, 'Error: QUIT not accepted from server!');

			fclose($handle);
		}
	}

	private static function handleReply($handle, $status_code = false, $error_text = false, $counter = 0) {
		$reply = '';

		while (($line = fgets($handle, 515)) !== false) {
			$reply .= $line;

			if (substr($line, 3, 1) == ' ') {
				break;
			}
		}

		// Handle slowish server responses (generally due to policy servers)
		if (!$line && empty($reply) && $counter < self::$max_attempts) {
			sleep(1);

			$counter++;

			return self::handleReply($handle, $status_code, $error_text, $counter);
		}

		if ($status_code) {
			if (substr($reply, 0, 3) != $status_code) {
				throw new \Exception($error_text);
			}
		}

		return $reply;
	}
}
