<?php
/**
 * @package		OpenCart
 * @author		Daniel Kerr
 * @copyright	Copyright (c) 2005 - 2017, OpenCart, Ltd. (https://www.opencart.com/)
 * @license		https://opensource.org/licenses/GPL-3.0
 * @link		https://www.opencart.com
*/

/**
* Session class
*/
class Session {
	protected static $adaptor;
	protected static $session_id;
	public static $data = array();

	/**
	 * Constructor
	 *
	 * @param	string	$adaptor
 	*/
	public static function init($adaptor) {
		$class = 'Session\\' . $adaptor;
		
		if (class_exists($class)) {
			self::$adaptor = new $class();
			
			register_shutdown_function(get_called_class().'::close');
		} else {
			trigger_error('Error: Could not load cache adaptor ' . $adaptor . ' session!');
			exit();
		}	
	}
	
	/**
	 * 
	 *
	 * @return	string
 	*/	
	public static function getId() {
		return self::$session_id;
	}

	/**
	 *
	 *
	 * @param	string	$session_id
	 *
	 * @return	string
 	*/	
	public static function start($session_id = '') {
		if (!$session_id) {
			if (function_exists('random_bytes')) {
				$session_id = substr(bin2hex(random_bytes(26)), 0, 26);
			} else {
				$session_id = substr(bin2hex(openssl_random_pseudo_bytes(26)), 0, 26);
			}
		}

		if (preg_match('/^[a-zA-Z0-9,\-]{22,52}$/', $session_id)) {
			self::$session_id = $session_id;
		} else {
			exit('Error: Invalid session ID!');
		}
		
		self::$data = self::$adaptor->read($session_id);
		
		return $session_id;
	}
	
	/**
	 * 
 	*/
	public static function close() {
		self::$adaptor->write(self::$session_id, self::$data);
	}
	
	/**
	 * 
 	*/	
	public static function destroy() {
		self::$adaptor->destroy(self::$session_id);
	}
}
