<?php

namespace Squareup;

class Exception extends \Exception {
    const ERR_CODE_ACCESS_TOKEN_REVOKED = 'ACCESS_TOKEN_REVOKED';
    const ERR_CODE_ACCESS_TOKEN_EXPIRED = 'ACCESS_TOKEN_EXPIRED';

    private static $log;
    private static $language;
    private static $errors;
    private static $isCurlError;

    private static $overrideFields = array(
        'billing_address.country',
        'shipping_address.country',
        'email_address',
        'phone_number'
    );

    public static function init($errors, $is_curl_error = false) {
        self::$errors = $errors;
        self::$isCurlError = $is_curl_error;
        self::$log = log;

        $message = self::concatErrors();

        if (config('error_log')) {
            log::write($message);
        }

        parent::__construct($message);
    }

    public static function isCurlError() {
        return self::$isCurlError;
    }

    public static function isAccessTokenRevoked() {
        return self::errorCodeExists(self::$ERR_CODE_ACCESS_TOKEN_REVOKED);
    }

    public static function isAccessTokenExpired() {
        return self::errorCodeExists(self::$ERR_CODE_ACCESS_TOKEN_EXPIRED);
    }

    protected static function errorCodeExists($code) {
        if (is_array(self::$errors)) {
            foreach (self::$errors as $error) {
                if ($error['code'] == $code) {
                    return true;
                }
            }
        }

        return false;
    }

    protected static function overrideError($field) {
        return language::get('squareup_override_error_' . $field);
    }

    protected static function parseError($error) {
        if (!empty($error['field']) && in_array($error['field'], self::$overrideFields)) {
            return self::overrideError($error['field']);
        }

        $message = $error['detail'];

        if (!empty($error['field'])) {
            $message .= sprintf(language::get('squareup_error_field'), $error['field']);
        }

        return $message;
    }

    protected static function concatErrors() {
        $messages = array();

        if (is_array(self::$errors)) {
            foreach (self::$errors as $error) {
                $messages[] = self::parseError($error);
            }
        } else {
            $messages[] = self::$errors;
        }

        return implode(' ', $messages);
    }
}