<?php
// Error Reporting
error_reporting(E_ALL);

// Check Version
if (version_compare(phpversion(), '7.3', '<') == true) {
	exit('PHP7.3+ Required');
}

if (!ini_get('date.timezone')) {
	date_default_timezone_set('UTC');
}

// Windows IIS Compatibility
if (!isset($_SERVER['DOCUMENT_ROOT'])) {
	if (isset($_SERVER['SCRIPT_FILENAME'])) {
		$_SERVER['DOCUMENT_ROOT'] = str_replace('\\', '/', substr($_SERVER['SCRIPT_FILENAME'], 0, 0 - strlen($_SERVER['PHP_SELF'])));
	}
}

if (!isset($_SERVER['DOCUMENT_ROOT'])) {
	if (isset($_SERVER['PATH_TRANSLATED'])) {
		$_SERVER['DOCUMENT_ROOT'] = str_replace('\\', '/', substr(str_replace('\\\\', '\\', $_SERVER['PATH_TRANSLATED']), 0, 0 - strlen($_SERVER['PHP_SELF'])));
	}
}

if (!isset($_SERVER['REQUEST_URI'])) {
	$_SERVER['REQUEST_URI'] = substr($_SERVER['PHP_SELF'], 1);

	if (isset($_SERVER['QUERY_STRING'])) {
		$_SERVER['REQUEST_URI'] .= '?' . $_SERVER['QUERY_STRING'];
	}
}

if (!isset($_SERVER['HTTP_HOST'])) {
	$_SERVER['HTTP_HOST'] = getenv('HTTP_HOST');
}

// Check if SSL
if ((isset($_SERVER['HTTPS']) && (($_SERVER['HTTPS'] == 'on') || ($_SERVER['HTTPS'] == '1'))) || (isset($_SERVER['HTTPS']) && (isset($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT'] == 443))) {
	$_SERVER['HTTPS'] = true;
} elseif (!empty($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https' || !empty($_SERVER['HTTP_X_FORWARDED_SSL']) && $_SERVER['HTTP_X_FORWARDED_SSL'] == 'on') {
	$_SERVER['HTTPS'] = true;
} else {
	$_SERVER['HTTPS'] = false;
}

// Modification Override
function modification($filename) {
	if (defined('DIR_CATALOG')) {
		$file = DIR_MODIFICATION . 'admin/' .  substr($filename, strlen(DIR_APPLICATION));
	} elseif (defined('DIR_OPENCART')) {
		$file = DIR_MODIFICATION . 'install/' .  substr($filename, strlen(DIR_APPLICATION));
	} else {
		$file = DIR_MODIFICATION . 'catalog/' . substr($filename, strlen(DIR_APPLICATION));
	}

	if (substr($filename, 0, strlen(DIR_SYSTEM)) == DIR_SYSTEM) {
		$file = DIR_MODIFICATION . 'system/' . substr($filename, strlen(DIR_SYSTEM));
	}

	if (is_file($file)) {
		return $file;
	}

	return $filename;
}

function x_replace_once($search, $replace, $content) {
	$p = strpos($content, $search);
	if ($p === false) return $content;
	return substr($content, 0, $p) . $replace . substr($content, $p + strlen($search));
}

function opencart_autoload($class) {
	$file = str_replace('\\', '/', strtolower($class)) . '.php';
	if (str_contains($class, '_')) {
		if (str_starts_with($class, 'admin_')) {
			$file = substr($file, 6);
			$path = DIR_APPLICATION . 'model/' . x_replace_once('_', '/', $file);
		} else
			$path = DIR_OPENCART . 'catalog/model/'. x_replace_once('_', '/', $file);

		if (!is_file($path)) {
			$file = x_replace_once('_', '/', $file);
			$file = x_replace_once('_', '/', $file);
			$path = substr($path, 0, -strlen($file)) . $file;
		}
	} else $path = 'system/library/' . $file;

	if (!is_file($path)) $path = 'system/engine/' . $file;
	if (!is_file($path)) $path = 'system/library/cart/' . $file;

	if (is_file($path)) {
		include_once(modification($path));
		if (method_exists($class, 'init')) {
			$method = new ReflectionMethod($class, 'init');
			if (!$method->getParameters()) call_user_func_array("$class::init", array());
		}
		return true;
	} else {
		return false;
	}
}

function start($application_config) {

	// config.php values for Opencart 3 extensions

	$cwd = getcwd();

	define('OPENCART_SERVER', 'https://www.opencart.com/');

	if ($application_config == 'install') {
		chdir(DIR_OPENCART);
		define('OPENCART_ADMIN', true);

	} else {
		if ($application_config == 'admin') {
			$admin = substr($cwd, strrpos($cwd, '/') + 1); // Admin folder name
			if (!defined('SITE_URL')) require_once '../config.php';

			define('OPENCART_ADMIN', true);
			define('HTTP_SERVER', SITE_URL . $admin . '/');
			define('HTTP_CATALOG', SITE_URL);
			define('HTTPS_CATALOG', HTTP_CATALOG);
			define('DIR_APPLICATION', $cwd . '/');
			if (!defined('DIR_OPENCART')) define('DIR_OPENCART', substr($cwd, 0, strrpos($cwd, '/') + 1));
			chdir(DIR_OPENCART);
			define('DIR_CATALOG', DIR_OPENCART . 'catalog/');
			define('DIR_TEMPLATE', DIR_APPLICATION . 'view/template/');
		} else {
			define('OPENCART_ADMIN', false);
			define('HTTP_SERVER', SITE_URL);
			define('DIR_OPENCART', $cwd . '/');
			define('DIR_APPLICATION', DIR_OPENCART . 'catalog/');
			define('DIR_TEMPLATE', DIR_APPLICATION . 'view/theme/');
		}

		define('HTTPS_SERVER', HTTP_SERVER);
		define('DIR_SYSTEM', DIR_OPENCART . 'system/');
		define('DIR_IMAGE', DIR_OPENCART . 'image/');
		if (!defined('DIR_STORAGE')) define('DIR_STORAGE', 'system/storage/');
		define('DIR_LANGUAGE', DIR_APPLICATION . 'language/');
		define('DIR_CONFIG', 'system/config/');
		define('DIR_CACHE', DIR_STORAGE . 'cache/');
		define('DIR_DOWNLOAD', DIR_STORAGE . 'download/');
		define('DIR_LOGS', DIR_STORAGE . 'logs/');
		define('DIR_MODIFICATION', DIR_STORAGE . 'modification/');
		define('DIR_SESSION', DIR_STORAGE . 'session/');
		define('DIR_UPLOAD', DIR_STORAGE . 'upload/');

		define('DB_DRIVER', 'mysqli');
		if (!defined('DB_PORT')) define('DB_PORT', '3306');
		define('DB_PREFIX', '');
	}

	// TODO: Autoload order, why the vendor autoload works first?

	spl_autoload_register('opencart_autoload');
	spl_autoload_extensions('.php');

	// Autoloader
	if (defined('DIR_STORAGE') && is_file(DIR_STORAGE . 'vendor/autoload.php')) {
		require_once(DIR_STORAGE . 'vendor/autoload.php');
	}

	// Helper
	require_once('system/helper/general.php');
	require_once('system/helper/utf8.php');

	require_once('system/framework.php');	
}